create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists common_tables_delete;^
	create procedure common_tables_delete(
		in `$session` text
	) begin 
		
		-- Level 3.
		drop table if exists _Users;
		drop table if exists _Ips;
		drop table if exists _SettingsTypes;
		drop table if exists _Settings;
		drop table if exists _SessionsIps;
		drop table if exists _SessionsUsersAgents;
		
		-- Level 2.
		drop table if exists UsersData;
		drop table if exists IpsData;
		drop table if exists SettingsTypesData;
		drop table if exists SettingsData;
		drop table if exists Logs;
		drop table if exists Exceptions;
		drop table if exists Traces;
		drop table if exists SessionsIpsData;
		drop table if exists SessionsUsersAgentsData;
		drop table if exists LoginsTries;
		drop table if exists Terminal;
	
		-- Level 1.
		drop table if exists Procedures;
		drop table if exists Sessions;
		drop table if exists ErrorsMessages;
	
		-- Level 0.
		drop table if exists `Databases`;
		drop table if exists Users;
		drop table if exists Ips;
		drop table if exists SettingsTypes;
		drop table if exists Settings;
		drop table if exists Errors;
		drop table if exists Parameters;
		drop table if exists Messages;
		drop table if exists `Lines`;
		drop table if exists Projects;
		drop table if exists TracesFiles;
		drop table if exists UsersAgents;
		drop table if exists SessionsIps;
		drop table if exists SessionsUsersAgents;
		drop table if exists ErrorsMessagesGroups;
		drop table if exists ErrorsTemporaryGroups;
		drop table if exists ErrorsTemporaryMessages;
		drop table if exists TerminalData;
		
	end;^

	drop procedure if exists common_tables_create;^
	create procedure common_tables_create(
		in `$session` text
	) begin 
		
		-- Level 0.
		create table if not exists Users(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_id primary key(id)
		);
	
		create table if not exists Ips(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint ips_id primary key(id)
		);
	
		create table if not exists `Databases`(
			id integer not null auto_increment,
			name varchar(64) not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint databases_id primary key(id)
		);
		
		create table if not exists SettingsTypes(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_types_id primary key(id)
		);
		
		create table if not exists Settings(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_id primary key(id)
		);
	
		create table if not exists Errors(
			id integer not null auto_increment,
			code varchar(512) not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint errors_id primary key(id)
		);
		
		create table if not exists Parameters(
			id integer not null auto_increment,
			json text not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint parameters_id primary key(id)
		);
		
		create table if not exists Messages(
			id integer not null auto_increment,
			`text` text not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint messages_id primary key(id)
		);
		
		create table if not exists TracesFiles(
			id integer not null auto_increment,
			`path` varchar(512) not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint traces_files_id primary key(id)
		);
		
		create table if not exists Projects(
			id integer not null auto_increment,
			name varchar(64) not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint projects_id primary key(id)
		);
		
		create table if not exists `Lines`(
			id integer not null auto_increment,
			trace text not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint lines_id primary key(id)
		);
	
		create table if not exists UsersAgents(
			id integer not null auto_increment,
			`data` text not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_agents_id primary key(id)
		);
		
		create table if not exists SessionsIps(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint sessions_ips_id primary key(id)
		);
		
		create table if not exists SessionsUsersAgents(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint sessions_users_agents_id primary key(id)
		);
	
		create table if not exists ErrorsMessagesGroups(
			id integer not null auto_increment,
			name varchar(64) not null,
			description varchar(512),
			date_in datetime not null default now(),
			date_out datetime,
			constraint errors_messages_groups_id primary key(id)
		);
	
		create table if not exists ErrorsTemporaryGroups(
			id integer not null auto_increment,
			i integer not null,
			name varchar(64) not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint errors_temporary_groups_id primary key(id)
		);
	
		create table if not exists ErrorsTemporaryMessages(
			id integer not null auto_increment,
			`group` integer not null,
			i smallint,
			message varchar(64),
			date_in datetime not null default now(),
			date_out datetime,
			constraint errors_temporary_messages_id primary key(id)
		);
	
		create table if not exists TerminalData(
			id integer not null auto_increment,
			`data` text,
			date_in datetime not null default now(),
			date_out datetime,
			constraint terminal_data_id primary key(id)
		);
	
		-- Level 1.
		create table if not exists Procedures(
			id integer not null auto_increment,
			`database` integer not null,
			name varchar(64) not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint procedures_id primary key(id),
			constraint procedures_database foreign key(`database`) references `Databases`(id)
		);
	
		create table if not exists Sessions(
			id integer not null auto_increment,
			`user` integer not null,
			date_last datetime not null default now(),
			date_in datetime not null default now(),
			date_out datetime,
			constraint sessions_id primary key(id),
			constraint sessions_user foreign key(`user`) references Users(id)
		);
	
		create table if not exists ErrorsMessages(
			id integer not null auto_increment,
			`group` integer not null,
			i integer not null,
			name varchar(64),
			date_in datetime not null default now(),
			date_out datetime,
			constraint errors_messages_id primary key(id),
			constraint errors_messages_group foreign key(`group`) references ErrorsMessagesGroups(id)
		);
		
		-- Level 2.
		create table if not exists UsersData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			nick varchar(32) not null,
			`password` varchar(256) not null,
			`accessible` boolean not null default false,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_data_id primary key(id),
			constraint users_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint users_data_session foreign key(`session`) references Sessions(id),
			constraint users_data_entity foreign key(entity) references Users(id)
		);
	
		create table if not exists IpsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			address varchar(39) not null,
			`accessible` boolean not null default true,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint ips_data_id primary key(id),
			constraint ips_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint ips_data_session foreign key(`session`) references Sessions(id),
			constraint ips_data_entity foreign key(entity) references Ips(id)
		);
	
		create table if not exists SettingsTypesData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			name varchar(64) not null,
			description text,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_types_data_id primary key(id),
			constraint settings_types_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint settings_types_data_session foreign key(`session`) references Sessions(id),
			constraint settings_types_data_entity foreign key(entity) references SettingsTypes(id)
		);
	
		create table if not exists SettingsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			`type` integer not null,
			name varchar(64) not null,
			`range` integer not null default 1,
			value text,
			description text,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_data_id primary key(id),
			constraint settings_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint settings_data_session foreign key(`session`) references Sessions(id),
			constraint settings_data_entity foreign key(entity) references Settings(id),
			constraint settings_data_type foreign key(`type`) references SettingsTypes(id)
		);
		
		create table if not exists Logs(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			ip integer,
			user_agent integer,
			own_error integer not null,
			own_parameters integer not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint logs_id primary key(id),
			constraint logs_procedure foreign key(`procedure`) references Procedures(id),
			constraint logs_session foreign key(`session`) references Sessions(id),
			constraint logs_ip foreign key(ip) references Ips(id),
			constraint logs_user_agent foreign key(user_agent) references UsersAgents(id),
			constraint logs_own_error foreign key(own_error) references Errors(id),
			constraint logs_own_parameters foreign key(own_parameters) references Parameters(id)
		);
		
		create table if not exists Exceptions(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			ip integer,
			user_agent integer,
			own_error integer not null,
			own_parameters integer not null,
			message integer not null,
			status varchar(16),
			code integer,
			date_in datetime not null default now(),
			date_out datetime,
			constraint exceptions_id primary key(id),
			constraint exceptions_procedure foreign key(`procedure`) references Procedures(id),
			constraint exceptions_session foreign key(`session`) references Sessions(id),
			constraint exceptions_ip foreign key(ip) references Ips(id),
			constraint exceptions_user_agent foreign key(user_agent) references UsersAgents(id),
			constraint exceptions_own_error foreign key(own_error) references Errors(id),
			constraint exceptions_own_parameters foreign key(own_parameters) references Parameters(id),
			constraint exceptions_message foreign key(message) references Messages(id)
		);
	
		create table if not exists Traces(
			id integer not null auto_increment,
			`session` integer not null,
			ip integer, 
			user_agent integer,
			project integer not null,
			file integer not null,
			own_error integer not null,
			own_parameters integer not null,
			trace integer not null,
			message integer not null,
			line integer not null,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint traces_id primary key(id),
			constraint traces_session foreign key(`session`) references Sessions(id),
			constraint traces_ip foreign key(ip) references Ips(id),
			constraint traces_user_agent foreign key(user_agent) references UsersAgents(id),
			constraint traces_project foreign key(project) references Projects(id),
			constraint traces_file foreign key(file) references TracesFiles(id),
			constraint traces_own_error foreign key(own_error) references AnP.Errors(id),
			constraint traces_own_parameters foreign key(own_parameters) references AnP.Parameters(id),
			constraint traces_trace foreign key(trace) references `Lines`(id),
			constraint traces_message foreign key(message) references AnP.Messages(id)
		);
	
		create table if not exists SessionsIpsData(
			id integer not null auto_increment,
			entity integer not null,
			`session` integer not null,
			ip integer not null,
			ok bool not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint sessions_ips_data_id primary key(id),
			constraint sessions_ips_data_entity foreign key(entity) references SessionsIps(id),
			constraint sessions_ips_data_session foreign key(`session`) references Sessions(id),
			constraint sessions_ips_data_ip foreign key(ip) references Ips(id)
		);
	
		create table if not exists SessionsUsersAgentsData(
			id integer not null auto_increment,
			entity integer not null,
			`session` integer not null,
			user_agent integer not null,
			ok bool not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint sessions_users_agents_data_id primary key(id),
			constraint sessions_users_agents_data_entity foreign key(entity) references SessionsUsersAgents(id),
			constraint sessions_users_agents_data_session foreign key(`session`) references Sessions(id),
			constraint sessions_users_agents_data_user_agent foreign key(user_agent) references UsersAgents(id)
		);
	
		create table if not exists LoginsTries(
			id integer not null auto_increment,
			`session` integer not null,
			`user` integer not null,
			ip integer not null,
			i integer not null default 0,
			date_in datetime not null default now(),
			date_out datetime,
			constraint logins_tries_id primary key(id),
			constraint logins_tries_session foreign key(`session`) references Sessions(id),
			constraint logins_tries_user foreign key(`user`) references Users(id),
			constraint logins_tries_ip foreign key(ip) references Ips(id)
		);
	
		create table if not exists Terminal(
			id integer not null auto_increment,
			`procedure` integer,
			`session` integer,
			`data` integer not null, 
			date_in datetime not null default now(),
			date_out datetime,
			constraint terminal_id primary key(id),
			constraint terminal_procedure foreign key(`procedure`) references Procedures(id),
			constraint terminal_session foreign key(`session`) references Sessions(id),
			constraint terminal_data foreign key(`data`) references TerminalData(id)
		);
	
		-- Level 3.
		create table if not exists _Users(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			nick varchar(32) not null,
			`password` varchar(256) not null,
			`accessible` boolean not null default false,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _users_id primary key(id),
			constraint _users_procedure foreign key(`procedure`) references Procedures(id),
			constraint _users_session foreign key(`session`) references Sessions(id),
			constraint _users_entity foreign key(entity) references Users(id)
		);
	
		create table if not exists _Ips(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			address varchar(39) not null,
			`accessible` boolean not null default true,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _ips_id primary key(id),
			constraint _ips_procedure foreign key(`procedure`) references Procedures(id),
			constraint _ips_session foreign key(`session`) references Sessions(id),
			constraint _ips_entity foreign key(entity) references Ips(id)
		);
	
		create table if not exists _SettingsTypes(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			name varchar(64) not null,
			description text,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _settings_types_id primary key(id),
			constraint _settings_types_procedure foreign key(`procedure`) references Procedures(id),
			constraint _settings_types_session foreign key(`session`) references Sessions(id),
			constraint _settings_types_entity foreign key(entity) references SettingsTypes(id)
		);
	
		create table if not exists _Settings(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			`type` integer not null,
			name varchar(64) not null,
			`range` integer not null default 1,
			value text,
			description text,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _settings_id primary key(id),
			constraint _settings_procedure foreign key(`procedure`) references Procedures(id),
			constraint _settings_session foreign key(`session`) references Sessions(id),
			constraint _settings_entity foreign key(entity) references Settings(id),
			constraint _settings_type foreign key(`type`) references SettingsTypes(id)
		);
	
		create table if not exists _SessionsIps(
			id integer not null auto_increment,
			entity integer not null, 
			`data` integer not null,
			`session` integer not null,
			ip integer not null,
			ok bool not null,
			date_in datetime not null,
			date_created datetime not null,
			constraint _sessions_ips_id primary key(id),
			constraint _sessions_ips_entity foreign key(entity) references SessionsIps(id),
			constraint _sessions_ips_session foreign key(`session`) references Sessions(id),
			constraint _sessions_ips_ip foreign key(ip) references Ips(id)
		);
	
		create table if not exists _SessionsUsersAgents(
			id integer not null auto_increment,
			entity integer not null, 
			`data` integer not null,
			`session` integer not null,
			user_agent integer not null,
			ok bool not null,
			date_in datetime not null,
			date_created datetime not null,
			constraint _sessions_users_agents_id primary key(id),
			constraint _sessions_users_agents_entity foreign key(entity) references SessionsUsersAgents(id),
			constraint _sessions_users_agents_session foreign key(`session`) references Sessions(id),
			constraint _sessions_users_agents_user_agent foreign key(user_agent) references UsersAgents(id)
		);
		
	end;^

	drop procedure if exists common_tables_update;^
	create procedure common_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default 'AnP';
		
		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Databases' && column_name = 'name' limit 1) is null then 
			alter table `Databases` add column name varchar(64) not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsMessagesGroups' && column_name = 'name' limit 1) is null then 
			alter table ErrorsMessagesGroups add column name varchar(64) not null after `session`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsMessagesGroups' && column_name = 'description' limit 1) is null then 
			alter table ErrorsMessagesGroups add column description varchar(512) after name;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsTemporaryGroups' && column_name = 'i' limit 1) is null then 
			alter table ErrorsMessagesGroups add column i integer after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsTemporaryGroups' && column_name = 'name' limit 1) is null then 
			alter table ErrorsMessagesGroups add column name varchar(64) after i;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsTemporaryMessages' && column_name = 'group' limit 1) is null then 
			alter table ErrorsMessagesGroups add column `group` integer not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsTemporaryMessages' && column_name = 'i' limit 1) is null then 
			alter table ErrorsMessagesGroups add column i smallint not null after `group`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsTemporaryMessages' && column_name = 'message' limit 1) is null then 
			alter table ErrorsMessagesGroups add column message varchar(64) after i;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'TerminalData' && column_name = 'data' limit 1) is null then 
			alter table TerminalData add column `data` text after `session`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersData' && column_name = 'nick' limit 1) is null then begin 
			alter table UsersData add column nick varchar(32) not null after entity;
			alter table _Users add column nick varchar(32) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersData' && column_name = 'password' limit 1) is null then begin 
			alter table UsersData add column `password` varchar(256) not null after nick;
			alter table _Users add column `password` varchar(256) not null after nick;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersData' && column_name = 'accessible' limit 1) is null then begin 
			alter table UsersData add column `accessible` boolean not null default false after `password`;
			alter table _Users add column `accessible` boolean not null default false after `password`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsData' && column_name = 'address' limit 1) is null then begin 
			alter table IpsData add column address varchar(39) not null after entity;
			alter table _Ips add column address varchar(39) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsData' && column_name = 'accessible' limit 1) is null then begin 
			alter table IpsData add column `accessible` boolean not null default true after address;
			alter table _Ips add column `accessible` boolean not null default true after address;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Procedures' && column_name = 'database' limit 1) is null then begin 
			alter table Procedures add column `database` integer not null after id;
			alter table Procedures add constraint procedures_database foreign key(`database`) references `Databases`(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Procedures' && column_name = 'name' limit 1) is null then 
			alter table Procedures add column name varchar(64) not null after `database`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Sessions' && column_name = 'user' limit 1) is null then begin 
			alter table Sessions add column `user` integer not null after id;
			alter table Sessions add constraint sessions_user foreign key(`user`) references Users(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Sessions' && column_name = 'date_last' limit 1) is null then 
			alter table Sessions add column date_last datetime not null default now() after ip;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsMessages' && column_name = 'group' limit 1) is null then begin 
			alter table ErrorsMessages add column `group` integer not null after `session`;
			alter table ErrorsMessages add constraint errors_messages_group foreign key(`group`) references ErrorsMessagesGroups(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsMessages' && column_name = 'i' limit 1) is null then 
			alter table ErrorsMessages add column i integer not null after `group`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'ErrorsMessages' && column_name = 'name' limit 1) is null then 
			alter table ErrorsMessages add column name varchar(64) after i;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsTypesData' && column_name = 'name' limit 1) is null then begin 
			alter table SettingsTypesData add column name varchar(64) not null after entity;
			alter table _SettingsTypes add column name varchar(64) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsTypesData' && column_name = 'description' limit 1) is null then begin 
			alter table SettingsTypesData add column description text after name;
			alter table _SettingsTypes add column description text after name;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'type' limit 1) is null then begin 
			alter table SettingsData add column `type` integer not null after entity;
			alter table SettingsData add constraint settings_data_type foreign key(`type`) references SettingsTypes(id);
			alter table _Settings add column `type` integer not null after `data`;
			alter table _Settings add constraint _settings_type foreign key(`type`) references SettingsTypes(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'name' limit 1) is null then begin 
			alter table SettingsData add column name varchar(64) not null after `type`;
			alter table _Settings add column name varchar(64) not null after `type`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'range' limit 1) is null then begin 
			alter table SettingsData add column `range` integer not null default 1 after name;
			alter table _Settings add column `range` integer not null default 1 after name;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'value' limit 1) is null then begin 
			alter table SettingsData add column value text after `range`;
			alter table _Settings add column value text after `range`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'description' limit 1) is null then begin 
			alter table SettingsData add column description text after value;
			alter table _Settings add column description text after value;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Errors' && column_name = 'code' limit 1) is null then 
			alter table Errors add column code varchar(512) not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Errors' && column_name = 'description' limit 1) is null then 
			alter table Errors add column description varchar(512) after code;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Parameters' && column_name = 'json' limit 1) is null then 
			alter table Parameters add column json text not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Parameters' && column_name = 'description' limit 1) is null then 
			alter table Parameters add column description varchar(512) after json;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Messages' && column_name = 'text' limit 1) is null then 
			alter table Messages add column `text` text not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Messages' && column_name = 'description' limit 1) is null then 
			alter table Messages add column description varchar(512) after `text`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'TracesFiles' && column_name = 'path' limit 1) is null then 
			alter table TracesFiles add column `path` varchar(512) not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'TracesFiles' && column_name = 'description' limit 1) is null then 
			alter table TracesFiles add column description varchar(512) after `path`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Projects' && column_name = 'name' limit 1) is null then 
			alter table Projects add column name varchar(64) not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Projects' && column_name = 'description' limit 1) is null then 
			alter table Projects add column description varchar(512) after name;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Lines' && column_name = 'trace' limit 1) is null then 
			alter table `Lines` add column trace text not null after id;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Lines' && column_name = 'description' limit 1) is null then 
			alter table `Lines` add column description varchar(512) after trace;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersAgents' && column_name = 'data' limit 1) is null then 
			alter table UsersAgents add column `data` text not null after `session`;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Logs' && column_name = 'ip' limit 1) is null then begin 
			alter table Logs add column ip integer not null after `session`;
			alter table Logs add constraint logs_ip foreign key(ip) references Ips(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Logs' && column_name = 'user_agent' limit 1) is null then begin 
			alter table Logs add column user_agent integer not null after ip;
			alter table Logs add constraint logs_user_agent foreign key(user_agent) references UsersAgents(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Logs' && column_name = 'own_error' limit 1) is null then begin 
			alter table Logs add column own_error integer not null after user_agent;
			alter table Logs add constraint logs_own_error foreign key(own_error) references Errors(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Logs' && column_name = 'own_parameters' limit 1) is null then begin 
			alter table Logs add column own_parameters integer not null after error;
			alter table Logs add constraint logs_own_parameters foreign key(own_parameters) references Parameters(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'ip' limit 1) is null then begin 
			alter table Exceptions add column ip integer not null after `session`;
			alter table Exceptions add constraint exceptions_ip foreign key(ip) references Ips(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'user_agent' limit 1) is null then begin 
			alter table Exceptions add column user_agent integer not null after ip;
			alter table Exceptions add constraint exceptions_user_agent foreign key(user_agent) references UsersAgents(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'own_error' limit 1) is null then begin 
			alter table Exceptions add column own_error integer not null after user_agent;
			alter table Exceptions add constraint exceptions_own_error foreign key(own_error) references Errors(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'own_parameters' limit 1) is null then begin 
			alter table Exceptions add column pown_arameters integer not null after error;
			alter table Exceptions add constraint exceptions_own_parameters foreign key(own_parameters) references Parameters(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'message' limit 1) is null then begin 
			alter table Exceptions add column message integer not null after parameters;
			alter table Exceptions add constraint exceptions_message foreign key(message) references Messages(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'status' limit 1) is null then 
			alter table Exceptions add column status varchar(16) after message;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Exceptions' && column_name = 'code' limit 1) is null then 
			alter table Exceptions add column code integer after status;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'ip' limit 1) is null then begin 
			alter table Traces add column ip integer not null after `session`;
			alter table Traces add constraint traces_ip foreign key(ip) references Ips(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'user_agent' limit 1) is null then begin 
			alter table Traces add column user_agent integer not null after ip;
			alter table Traces add constraint traces_user_agent foreign key(user_agent) references UsersAgents(id);
		end;end if;
	
		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'project' limit 1) is null then begin 
			alter table Traces add column project integer not null after user_agent;
			alter table Traces add constraint traces_project foreign key(project) references Projects(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'file' limit 1) is null then begin 
			alter table Traces add column file integer not null after project;
			alter table Traces add constraint traces_file foreign key(file) references TracesFiles(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'own_error' limit 1) is null then begin 
			alter table Traces add column own_error integer not null after file;
			alter table Traces add constraint traces_own_error foreign key(own_error) references Errors(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'own_parameters' limit 1) is null then begin 
			alter table Traces add column own_parameters integer not null after own_error;
			alter table Traces add constraint traces_own_parameters foreign key(own_parameters) references Parameters(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'trace' limit 1) is null then begin 
			alter table Traces add column trace integer not null after own_parameters;
			alter table Traces add constraint traces_trace foreign key(trace) references `Lines`(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'message' limit 1) is null then begin 
			alter table Traces add column message integer not null after trace;
			alter table Traces add constraint traces_message foreign key(message) references Messages(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Traces' && column_name = 'line' limit 1) is null then begin 
			alter table Traces add column line integer not null after message;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SessionsIpsData' && column_name = 'session' limit 1) is null then begin 
			alter table SessionsIpsData add column `session` integer not null after entity;
			alter table SessionsIpsData add constraint sessions_ips_data_session foreign key(`session`) references Sessions(id);
			alter table _SessionsIps add column `session` integer not null after `data`;
			alter table _SessionsIps add constraint _sessions_ips_session foreign key(`session`) references Sessions(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SessionsIpsData' && column_name = 'ip' limit 1) is null then begin 
			alter table SessionsIpsData add column ip integer not null after `session`;
			alter table SessionsIpsData add constraint sessions_ips_data_ip foreign key(ip) references Ips(id);
			alter table _SessionsIps add column ip integer not null after `session`;
			alter table _SessionsIps add constraint _sessions_ips_ip foreign key(ip) references Ips(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SessionsIpsData' && column_name = 'ok' limit 1) is null then begin 
			alter table SessionsIpsData add column ok bool not null after ip;
			alter table _SessionsIps add column ok bool not null after ip;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SessionsUsersAgentsData' && column_name = 'session' limit 1) is null then begin 
			alter table SessionsUsersAgentsData add column `session` integer not null after entity;
			alter table SessionsUsersAgentsData add constraint sessions_users_agents_data_session foreign key(`session`) references Sessions(id);
			alter table _SessionsUsersAgents add column `session` integer not null after `data`;
			alter table _SessionsUsersAgents add constraint _sessions_users_agents_session foreign key(`session`) references Sessions(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SessionsUsersAgentsData' && column_name = 'user_agent' limit 1) is null then begin 
			alter table SessionsUsersAgentsData add column user_agent integer not null after `session`;
			alter table SessionsUsersAgentsData add constraint sessions_users_agents_data_user_agent foreign key(user_agent) references UsersAgents(id);
			alter table _SessionsUsersAgents add column user_agent integer not null after `session`;
			alter table _SessionsUsersAgents add constraint _sessions_users_agents_user_agent foreign key(user_agent) references UsersAgents(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SessionsUsersAgentsData' && column_name = 'ok' limit 1) is null then begin 
			alter table SessionsUsersAgentsData add column ok bool not null after user_agent;
			alter table _SessionsUsersAgents add column ok bool not null after user_agent;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LoginsTries' && column_name = 'session' limit 1) is null then begin 
			alter table LoginsTries add column `session` integer not null after `session`;
			alter table LoginsTries add constraint logins_tries_session foreign key(`session`) references Sessions(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LoginsTries' && column_name = 'user' limit 1) is null then begin 
			alter table LoginsTries add column `user` integer not null after `session`;
			alter table LoginsTries add constraint logins_tries_user foreign key(`user`) references Users(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LoginsTries' && column_name = 'ip' limit 1) is null then begin 
			alter table LoginsTries add column ip integer not null after `user`;
			alter table LoginsTries add constraint logins_tries_ip foreign key(ip) references Ips(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LoginsTries' && column_name = 'i' limit 1) is null then 
			alter table LoginsTries add column i integer not null default 0 after ip;
		end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'Terminal' && column_name = 'data' limit 1) is null then 
			alter table Terminal add column `data` integer not null after `session`;
		end if;
		
	end;^
	
	-- call common_tables_delete(null);^
	call common_tables_create(null);^
	call common_tables_update(null);^
	
	drop procedure if exists users_data_update;^
	create procedure users_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from Users where id = $entity && date_out is null limit 1) is null || 
			(select 1 from UsersData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _Users where entity = $entity;
		else begin
			if (select 1 from _Users where entity = $entity limit 1) is null then 
				insert into _Users(`procedure`, `session`, entity, `data`, nick, `password`, `accessible`, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, nick, `password`, `accessible`, deleted, date_in, (select date_in from Users where id = $entity) from UsersData where id = (
						select max(id) from UsersData where entity = $entity && date_out is null 
					);
			else 
				update _Users plain 
				join (select `procedure`, `session`, entity, id, nick, `password`, `accessible`, deleted, date_in, (select date_in from Users where id = $entity) from UsersData where id = (
					select max(id) from UsersData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.nick := `data`.nick,
					plain.`password` := `data`.`password`,
					plain.`accessible` := `data`.`accessible`,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists users_full_update;^
	create procedure users_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in $nick varchar(32),
		in `$password` varchar(256),
		in `$accessible` boolean,
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from Users where id = $entity && date_out is null limit 1) is null then 
			delete from _Users where entity = $entity;
		else begin 
			if (select 1 from _Users where entity = $entity limit 1) then 
				update _Users set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _Users(`procedure`, `session`, entity, `data`, nick, `password`, `accessible`, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, $nick, `$password`, `$accessible`, $deleted, $date_in, (select date_in from Users where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists users_trigger_update_entity;^
	create trigger users_trigger_update_entity after update on Users for each row begin 
		call users_data_update(new.id);
	end;^
	
	drop trigger if exists users_trigger_insert;^
	create trigger users_trigger_insert after insert on UsersData for each row begin 
		if new.date_out is null then 
			call users_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.nick, new.`password`, new.`accessible`, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists users_trigger_update;^
	create trigger users_trigger_update after update on UsersData for each row begin 
		if new.entity != old.entity then 
			call users_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call users_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.nick, new.`password`, new.`accessible`, new.deleted, new.date_in);
		else 
			call users_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists users_trigger_delete;^
	create trigger users_trigger_delete after delete on UsersData for each row begin 
		call users_data_update(old.entity);
	end;^
	
	drop procedure if exists ips_data_update;^
	create procedure ips_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from Ips where id = $entity && date_out is null limit 1) is null || 
			(select 1 from IpsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _Ips where entity = $entity;
		else begin
			if (select 1 from _Ips where entity = $entity limit 1) is null then 
				insert into _Ips(`procedure`, `session`, entity, `data`, address, `accessible`, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, address, `accessible`, deleted, date_in, (select date_in from Ips where id = $entity) from IpsData where id = (
						select max(id) from IpsData where entity = $entity && date_out is null 
					);
			else 
				update _Ips plain 
				join (select `procedure`, `session`, entity, id, address, `accessible`, deleted, date_in, (select date_in from Ips where id = $entity) from IpsData where id = (
					select max(id) from IpsData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.address := `data`.address,
					plain.`accessible` := `data`.`accessible`,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists ips_full_update;^
	create procedure ips_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in $address varchar(39),
		in `$accessible` boolean,
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from Ips where id = $entity && date_out is null limit 1) is null then 
			delete from _Ips where entity = $entity;
		else begin 
			if (select 1 from _Ips where entity = $entity limit 1) then 
				update _Ips set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _Ips(`procedure`, `session`, entity, `data`, address, `accessible`, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, $address, `$accessible`, $deleted, $date_in, (select date_in from Ips where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists ips_trigger_update_entity;^
	create trigger ips_trigger_update_entity after update on Ips for each row begin 
		call ips_data_update(new.id);
	end;^
	
	drop trigger if exists ips_trigger_insert;^
	create trigger ips_trigger_insert after insert on IpsData for each row begin 
		if new.date_out is null then 
			call ips_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.address, new.`accessible`, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists ips_trigger_update;^
	create trigger ips_trigger_update after update on IpsData for each row begin 
		if new.entity != old.entity then 
			call ips_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call ips_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.address, new.`accessible`, new.deleted, new.date_in);
		else 
			call ips_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists ips_trigger_delete;^
	create trigger ips_trigger_delete after delete on IpsData for each row begin 
		call ips_data_update(old.entity);
	end;^
	
	drop procedure if exists settings_types_data_update;^
	create procedure settings_types_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from SettingsTypes where id = $entity && date_out is null limit 1) is null || 
			(select 1 from SettingsTypesData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _SettingsTypes where entity = $entity;
		else begin
			if (select 1 from _SettingsTypes where entity = $entity limit 1) is null then 
				insert into _SettingsTypes(`procedure`, `session`, entity, `data`, name, description, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, name, description, deleted, date_in, (select date_in from SettingsTypes where id = $entity) from SettingsTypesData where id = (
						select max(id) from SettingsTypesData where entity = $entity && date_out is null 
					);
			else 
				update _SettingsTypes plain 
				join (select `procedure`, `session`, entity, id, name, description, deleted, date_in, (select date_in from SettingsTypes where id = $entity) from SettingsTypesData where id = (
					select max(id) from SettingsTypesData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.name := `data`.name,
					plain.description := `data`.description,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists settings_types_full_update;^
	create procedure settings_types_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in $name varchar(64),
		in $description text,
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from SettingsTypes where id = $entity && date_out is null limit 1) is null then 
			delete from _SettingsTypes where entity = $entity;
		else begin 
			if (select 1 from _SettingsTypes where entity = $entity limit 1) then 
				update _SettingsTypes set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _SettingsTypes(`procedure`, `session`, entity, `data`, name, description, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, $name, $description, $deleted, $date_in, (select date_in from SettingsTypes where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists settings_types_trigger_update_entity;^
	create trigger settings_types_trigger_update_entity after update on SettingsTypes for each row begin 
		call settings_types_data_update(new.id);
	end;^
	
	drop trigger if exists settings_types_trigger_insert;^
	create trigger settings_types_trigger_insert after insert on SettingsTypesData for each row begin 
		if new.date_out is null then 
			call settings_types_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.name, new.description, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists settings_types_trigger_update;^
	create trigger settings_types_trigger_update after update on SettingsTypesData for each row begin 
		if new.entity != old.entity then 
			call settings_types_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call settings_types_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.name, new.description, new.deleted, new.date_in);
		else 
			call settings_types_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists settings_types_trigger_delete;^
	create trigger settings_types_trigger_delete after delete on SettingsTypesData for each row begin 
		call settings_types_data_update(old.entity);
	end;^
	
	drop procedure if exists settings_data_update;^
	create procedure settings_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from Settings where id = $entity && date_out is null limit 1) is null || 
			(select 1 from SettingsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _Settings where entity = $entity;
		else begin
			if (select 1 from _Settings where entity = $entity limit 1) is null then 
				insert into _Settings(`procedure`, `session`, entity, `data`, `type`, name, `range`, value, description, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, `type`, name, `range`, value, description, deleted, date_in, (select date_in from Settings where id = $entity) from SettingsData where id = (
						select max(id) from SettingsData where entity = $entity && date_out is null 
					);
			else 
				update _Settings plain 
				join (select `procedure`, `session`, entity, id, `type`, name, `range`, value, description, deleted, date_in, (select date_in from Settings where id = $entity) from SettingsData where id = (
					select max(id) from SettingsData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.`type` := `data`.`type`,
					plain.name := `data`.name,
					plain.`range` := `data`.`range`,
					plain.value := `data`.value,
					plain.description := `data`.description,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists settings_full_update;^
	create procedure settings_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in `$type` integer,
		in $name varchar(64),
		in `$range` integer,
		in $value text,
		in $description text,
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from Settings where id = $entity && date_out is null limit 1) is null then 
			delete from _Settings where entity = $entity;
		else begin 
			if (select 1 from _Settings where entity = $entity limit 1) then 
				update _Settings set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _Settings(`procedure`, `session`, entity, `data`, `type`, name, `range`, value, description, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, `$type`, $name, `$range`, $value, $description, $deleted, $date_in, (select date_in from Settings where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists settings_trigger_update_entity;^
	create trigger settings_trigger_update_entity after update on Settings for each row begin 
		call settings_data_update(new.id);
	end;^
	
	drop trigger if exists settings_trigger_insert;^
	create trigger settings_trigger_insert after insert on SettingsData for each row begin 
		if new.date_out is null then 
			call settings_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.`type`, new.name, new.`range`, new.value, new.description, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists settings_trigger_update;^
	create trigger settings_trigger_update after update on SettingsData for each row begin 
		if new.entity != old.entity then 
			call settings_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call settings_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.`type`, new.name, new.`range`, new.value, new.description, new.deleted, new.date_in);
		else 
			call settings_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists settings_trigger_delete;^
	create trigger settings_trigger_delete after delete on SettingsData for each row begin 
		call settings_data_update(old.entity);
	end;^
	
	drop procedure if exists sessions_ips_data_update;^
	create procedure sessions_ips_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from SessionsIps where id = $entity && date_out is null limit 1) is null || 
			(select 1 from SessionsIpsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _SessionsIps where entity = $entity;
		else begin
			if (select 1 from _SessionsIps where entity = $entity limit 1) is null then 
				insert into _SessionsIps(entity, `data`, `session`, ip, ok, date_in, date_created)
					select entity, id, `session`, ip, ok, date_in, (select date_in from SessionsIps where id = $entity) from SessionsIpsData where id = (
						select max(id) from SessionsIpsData where entity = $entity && date_out is null 
					) && ok;
			else 
				update _SessionsIps plain 
				join (select entity, id, `session`, ip, ok, date_in, (select date_in from SessionsIps where id = $entity) from SessionsIpsData where id = (
					select max(id) from SessionsIpsData where entity = $entity && date_out is null 
				) && ok) `data` on plain.entity = `data`.entity
				set 
					plain.`data` := `data`.id,
					plain.`session` := `data`.`session`,
					plain.ip := `data`.ip,
					plain.ok := `data`.ok,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists sessions_ips_full_update;^
	create procedure sessions_ips_full_update(
		in $entity integer,
		in `$data` integer,
		in `$session` integer,
		in $ip integer,
		in $ok bool,
		in $date_in datetime
	) begin
		
		if (select 1 from SessionsIps where id = $entity && date_out is null limit 1) is null then 
			delete from _SessionsIps where entity = $entity;
		elseif $ok then begin 
			if (select 1 from _SessionsIps where entity = $entity limit 1) then 
				update _SessionsIps set 
					`data` := `$data`,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _SessionsIps(entity, `data`, `session`, ip, ok, date_in, date_created) values 
					($entity, `$data`, `$session`, $ip, $ok, $date_in, (select date_in from SessionsIps where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists sessions_ips_trigger_update_entity;^
	create trigger sessions_ips_trigger_update_entity after update on SessionsIps for each row begin 
		call sessions_ips_data_update(new.id);
	end;^
	
	drop trigger if exists sessions_ips_trigger_insert;^
	create trigger sessions_ips_trigger_insert after insert on SessionsIpsData for each row begin 
		if new.date_out is null then 
			call sessions_ips_full_update( new.entity, new.id, new.`session`, new.ip, new.ok, new.date_in);
		end if;
	end;^
	
	drop trigger if exists sessions_ips_trigger_update;^
	create trigger sessions_ips_trigger_update after update on SessionsIpsData for each row begin 
		if new.entity != old.entity then 
			call sessions_ips_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call sessions_ips_full_update(new.entity, new.id, new.`session`, new.ip, new.ok, new.date_in);
		else 
			call sessions_ips_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists sessions_ips_trigger_delete;^
	create trigger sessions_ips_trigger_delete after delete on SessionsIpsData for each row begin 
		call sessions_ips_data_update(old.entity);
	end;^
	
	drop procedure if exists sessions_users_agents_data_update;^
	create procedure sessions_users_agents_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from SessionsUsersAgents where id = $entity && date_out is null limit 1) is null || 
			(select 1 from SessionsUsersAgentsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _SessionsUsersAgents where entity = $entity;
		else begin
			if (select 1 from _SessionsUsersAgents where entity = $entity limit 1) is null then 
				insert into _SessionsUsersAgents(entity, `data`, `session`, user_agent, ok, date_in, date_created)
					select entity, id, `session`, user_agent, ok, date_in, (select date_in from SessionsUsersAgents where id = $entity) from SessionsUsersAgentsData where id = (
						select max(id) from SessionsUsersAgentsData where entity = $entity && date_out is null 
					) && ok;
			else 
				update _SessionsUsersAgents plain 
				join (select entity, id, `session`, user_agent, ok, date_in, (select date_in from SessionsUsersAgents where id = $entity) from SessionsUsersAgentsData where id = (
					select max(id) from SessionsUsersAgentsData where entity = $entity && date_out is null 
				) && ok) `data` on plain.entity = `data`.entity
				set 
					plain.`data` := `data`.id,
					plain.`session` := `data`.`session`,
					plain.user_agent := `data`.user_agent,
					plain.ok := `data`.ok,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists sessions_users_agents_full_update;^
	create procedure sessions_users_agents_full_update(
		in $entity integer,
		in `$data` integer,
		in `$session` integer,
		in $user_agent integer,
		in $ok bool,
		in $date_in datetime
	) begin
		
		if (select 1 from SessionsUsersAgents where id = $entity && date_out is null limit 1) is null then 
			delete from _SessionsUsersAgents where entity = $entity;
		elseif $ok then begin 
			if (select 1 from _SessionsUsersAgents where entity = $entity limit 1) then 
				update _SessionsUsersAgents set 
					`data` := `$data`,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _SessionsUsersAgents(entity, `data`, `session`, user_agent, ok, date_in, date_created) values 
					($entity, `$data`, `$session`, $user_agent, $ok, $date_in, (select date_in from SessionsUsersAgents where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists sessions_users_agents_trigger_update_entity;^
	create trigger sessions_users_agents_trigger_update_entity after update on SessionsUsersAgents for each row begin 
		call sessions_users_agents_data_update(new.id);
	end;^
	
	drop trigger if exists sessions_users_agents_trigger_insert;^
	create trigger sessions_users_agents_trigger_insert after insert on SessionsUsersAgentsData for each row begin 
		if new.date_out is null then 
			call sessions_users_agents_full_update(new.entity, new.id, new.`session`, new.user_agent, new.ok, new.date_in);
		end if;
	end;^
	
	drop trigger if exists sessions_users_agents_trigger_update;^
	create trigger sessions_users_agents_trigger_update after update on SessionsUsersAgentsData for each row begin 
		if new.entity != old.entity then 
			call sessions_users_agents_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call sessions_users_agents_full_update(new.entity, new.id, new.`session`, new.user_agent, new.ok, new.date_in);
		else 
			call sessions_users_agents_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists sessions_users_agents_trigger_delete;^
	create trigger sessions_users_agents_trigger_delete after delete on SessionsUsersAgentsData for each row begin 
		call sessions_users_agents_data_update(old.entity);
	end;^
	
delimiter ;