create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists procedures_get;^
	create procedure procedures_get(
		in `$database` varchar(64),
		in `$procedure` varchar(64),
		out $id integer
	) begin 
		
		declare $database_id integer default (select id from `Databases` where date_out is null && name = `$database` limit 1);
	
		if $database_id is null then begin 
			insert into `Databases`(name) values(`$database`);
			set $database_id := last_insert_id(); 
		end;end if;
		
		set $id := (select id from Procedures where date_out is null && `database` = $database_id && name = `$procedure` limit 1);
	
		if $id is null then begin 
			insert into Procedures(`database`, name) values($database_id, `$procedure`);
			set $id := last_insert_id();  
		end;end if;
		
	end;^
	
	drop procedure if exists errors_get;^
	create procedure errors_get(
		in $error varchar(512),
		out $id integer
	) begin 
		
		set $id := (select id from Errors where date_out is null && code = ifnull($error, '') limit 1);
	
		if $id is null then begin 
			insert into Errors(code) values(ifnull($error, ''));
			set $id := last_insert_id(); 
		end;end if;
		
	end;^
	
	drop procedure if exists parameters_get;^
	create procedure parameters_get(
		in $parameters text,
		out $id integer
	) begin 
		
		set $id := (select id from Parameters where date_out is null && json = $parameters limit 1);
	
		if $id is null then begin 
			insert into Parameters(json) values($parameters);
			set $id := last_insert_id(); 
		end;end if;
		
	end;^
	
	drop procedure if exists messages_get;^
	create procedure messages_get(
		in $message text,
		out $id integer
	) begin 
		
		set $id := (select id from Messages where date_out is null && `text` = $message limit 1);
	
		if $id is null then begin 
			insert into Messages(`text`) values($message);
			set $id := last_insert_id(); 
		end;end if;
		
	end;^
	
	drop procedure if exists projects_get;^
	create procedure projects_get(
		in $name varchar(64),
		out $id integer
	) begin 
		
		set $id := (select id from Projects where date_out is null && name = $name limit 1);
	
		if $id is null then begin 
			insert into Projects(name) values($name);
			set $id := last_insert_id(); 
		end;end if;
		
	end;^
	
	drop procedure if exists traces_files_get;^
	create procedure traces_files_get(
		in `$path` varchar(512),
		out $id integer
	) begin 
		
		set $id := (select id from TracesFiles where date_out is null && `path` = `$path` limit 1);
	
		if $id is null then begin 
			insert into TracesFiles(`path`) values(`$path`);
			set $id := last_insert_id(); 
		end;end if;
		
	end;^
	
	drop procedure if exists lines_get;^
	create procedure lines_get(
		in $trace text,
		out $id integer
	) begin 
		
		set $id := (select id from `Lines` where date_out is null && trace = $trace limit 1);
	
		if $id is null then begin 
			insert into `Lines`(trace) values($trace);
			set $id := last_insert_id(); 
		end;end if;
		
	end;^

	drop function if exists bitwise;^
	create function bitwise(
		$code bigint,
		$bits smallint
	) returns bigint begin 
		return floor(case 
			when $bits > 0 then $code * power(2, $bits)
			when $bits < 0 then $code / power(2, -$bits)
		else $code end);
	end;^

	drop function if exists bw;^
	create function bw(
		$code bigint,
		$bits smallint
	) returns bigint begin 
		return bitwise($code, $bits);
	end;^
	
	drop function if exists settings_get;^
	create function settings_get(
		$name varchar(64),
		`$nulls` bool
	) returns text begin 
		
		declare $value text default (select value from _Settings where deleted is null && name = $name limit 1);
	
		if $value is null && !`$nulls` then 
			set $value := (select value from _Settings where deleted is null && name = 'default_value' limit 1);
		end if;
	
		return $value;
	end;^
	
	drop function if exists string_to_integer;^
	create function string_to_integer(
		$string text
	) returns integer begin 
		return if($string regexp '^[0-9]+$', convert($string, integer), null);
	end;^
	
	drop function if exists settings_get_int;^
	create function settings_get_int(
		$name varchar(64),
		`$nulls` bool
	) returns integer begin 
		return string_to_integer(settings_get($name, `$nulls`));
	end;^
	
	drop function if exists string_to_bool;^
	create function string_to_bool(
		$string text
	) returns integer begin 
		
		set $string := lower($string);
	
		return (case 
			when $string in ('1', 'true', 'yes') then true
			when $string in ('0', 'false', 'no') then false
		else null end);
	end;^
	
	drop function if exists settings_get_bool;^
	create function settings_get_bool(
		$name varchar(64),
		`$nulls` bool
	) returns bool begin 
		return string_to_bool(settings_get($name, `$nulls`));
	end;^
	
	drop function if exists sessions_get_machine;^
	create function sessions_get_machine() returns integer begin 
		return (select id from Sessions where `user` = (
			select entity from _Users where deleted is null && nick = settings_get('machine_user', false) limit 1
		) limit 1);
	end;^
	
	drop function if exists sessions_get_unknown;^
	create function sessions_get_unknown() returns integer begin 
		return (select id from Sessions where `user` = (
			select entity from _Users where deleted is null && nick = settings_get('unknown_user', false) limit 1
		) limit 1);
	end;^
	
	drop procedure if exists logs_set;^
	create procedure logs_set(
		in `$session` integer,
		in `$database` varchar(64),
		in `$procedure` varchar(64),
		in $ip integer,
		in $user_agent integer,
		in $error varchar(512),
		in $parameters text,
		in `$level` tinyint
	) begin 
		if `$level` is null || ifnull(settings_get_int('logs_level', false), ~-(1 << 3)) & (1 << `$level`) then begin 
		
			declare $procedure_id integer;
			declare $error_id integer;
			declare $parameters_id integer;
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
			call errors_get($error, $error_id);
			call parameters_get($parameters, $parameters_id);
		
			insert into Logs(`procedure`, `session`, ip, user_agent, own_error, own_parameters) values
				($procedure_id, ifnull((
					select id from Sessions where id = `$session` limit 1
				), sessions_get_unknown()), $ip, $user_agent, $error_id, $parameters_id);
			
		end;end if;
	end;^
	
	drop procedure if exists exceptions_set;^
	create procedure exceptions_set(
		in `$session` integer,
		in `$database` varchar(64),
		in `$procedure` varchar(64),
		in $ip integer,
		in $user_agent integer,
		in $error varchar(512),
		in $parameters text,
		in $message text,
		in $status varchar(16),
		in $code integer
	) begin 
		
		declare $procedure_id integer;
		declare $error_id integer;
		declare $parameters_id integer;
		declare $message_id integer;
		
		call procedures_get(`$database`, `$procedure`, $procedure_id);
		call errors_get($error, $error_id);
		call parameters_get($parameters, $parameters_id);
		call messages_get($message, $message_id);
	
		insert into Exceptions(`procedure`, `session`, ip, user_agent, own_error, own_parameters, message, status, code) values
			($procedure_id, ifnull((
				select id from Sessions where id = `$session` limit 1
			), sessions_get_unknown()), $ip, $user_agent, $error_id, $parameters_id, $message_id, $status, $code);
		
	end;^

	drop procedure if exists common_tables_fill;^
	create procedure common_tables_fill(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default ifnull(settings_get('database_anp', false), 'AnP');
	
		declare `$procedure` varchar(64) default 'common_tables_fill';
		declare $procedure_id integer;
		declare $machine varchar(32) default ifnull(settings_get('machine_user', false), 'machine');
		declare `$unknown` varchar(32) default ifnull(settings_get('unknown_user', false), 'unknown');
		declare $local_ip varchar(39) default ifnull(settings_get('local_ip', false), '::1');
		declare $guest varchar(32) default ifnull(settings_get('guest_user', false), 'guest');
		declare $user_agent text default ifnull(settings_get('user_agent', false), 'AnP Machine DB Server');
		declare $machine_id integer default (select entity from _Users where deleted is null && nick = $machine limit 1);
		declare $unknown_id integer default (select entity from _Users where deleted is null && nick = `$unknown` limit 1);
		declare $guest_id integer default (select entity from _Users where deleted is null && nick = $guest limit 1);
		declare $local_ip_id integer default (select entity from _Ips where deleted is null && address = $local_ip limit 1);
		declare $machine_session integer default (select id from Sessions where `user` = $machine_id order by id asc limit 1);
		declare $unknown_session integer default (select id from Sessions where `user` = $unknown_id order by id asc limit 1);
		declare $user_agent_id integer default (select id from UsersAgents where `data` = $user_agent order by id asc limit 1);
		declare $machine_add bool default $machine_id is null;
		declare $local_ip_add bool default $local_ip_id is null;
	
		drop temporary table if exists default_settings_temporary_anp;
		create temporary table default_settings_temporary_anp(
			id integer not null auto_increment,
			`type` varchar(32) not null,
			name varchar(64) not null,
			`range` integer not null default (1 << 0),
			value text,
			description text,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_settings_temporary_anp_id primary key(id)
		);
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
	
		insert into default_settings_temporary_anp(`type`, name, `range`, value, description) values
			('varchar', 'database_anp', 1 << 0, `$database`, 'Default AnP database name.'),
			('varchar', 'machine_user', 1 << 0, $machine, 'Default AnP machine Bot user.'),
			('varchar', 'unknown_user', 1 << 0, `$unknown`, 'Default AnP unknown user.'),
			('varchar', 'guest_user', 1 << 0, $guest, 'Default AnP guest name user.'),
			('varchar', 'local_ip', 1 << 0, $local_ip, 'Default local IP address.'),
			('varchar', 'base64_alphabet', 1 << 0, 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/', 'Base64 alphabet without "=" fill character.'),
			('varchar', 'default_value', 1 << 0, null, 'Default value for unknown settings items.'),
			('integer', 'logs_level', 1 << 0, ~-(1 << 3), 'Default Logs level.'),
			('integer', 'default_integer_value', 1 << 0, null, 'Default integer value for unknown settings items.'),
			('json', 'users_excluded', 1 << 0, concat('["', $machine, '","', `$unknown`, '","', $guest, '"]'), 'Default excluded users for sessions validations.'),
			('json', 'ips_excluded', 1 << 0, '[]', 'Default excluded IPs for sessions validations.'),
			('integer', 'session_timeout', 1 << 1, 300, 'The session timeout for inactivity time in seconds.'),
			('boolean', 'allow_any_new_ip', 1 << 0, true, 'Verifies if any new IP is allowed to access to application.'),
			('boolean', 'allow_any_new_user', 1 << 0, true, 'Verifies if any new user is allowed to access to application.'),
			('integer', 'password_minimum_length', 1 << 0, 4, 'The minimum password length.'),
			('boolean', 'password_needs_lower', 1 << 0, true, 'Verifies if password needs at least 1 lower case.'),
			('boolean', 'password_needs_upper', 1 << 0, true, 'Verifies if password needs at least 1 upper case.'),
			('boolean', 'password_needs_digit', 1 << 0, true, 'Verifies if password needs at least 1 digit.'),
			('boolean', 'password_needs_special', 1 << 0, true, 'Verifies if password needs at least 1 special character.'),
			('boolean', 'mail_allow_reuse', 1 << 0, false, 'Verifies if allow reuse the E-Mails.'),
			('varchar', 'password_hash_function', 1 << 0, 'sha2', 'Hash method for hashing passwords.'),
			('integer', 'hash_sha2_length', 1 << 0, 512, 'Length for SHA2 function Hash.'),
			('integer', 'ips_maximum_sessions', 1 << 1, 10, 'The maximum number of simultaneous actived sessions per IP.'),
			('integer', 'users_maximum_sessions', 1 << 1, 1, 'The maximum number of simultaneous actived sessions per user.'),
			('boolean', 'session_check_ip', 1 << 1, true, 'Verifies if session checks its IP address.'),
			('boolean', 'session_check_user_agent', 1 << 1, true, 'Verifies if session checks its client User Agent.'),
			('varchar', 'default_user_agent', 1 << 0, $user_agent, 'Default User Agent for internal Bot sessions.'),
			('integer', 'user_login_bad_times', 1 << 1, 5, 'Default user Login tries before lock account temporary for lock brute force attack.'),
			('integer', 'user_lock_time', 1 << 1, 900, 'Default user Login lock time for lock brute force attack before tries.'),
			('integer', 'ip_login_bad_times', 1 << 1, 20, 'Default IP Login tries before lock account temporary for lock brute force attack.'),
			('integer', 'ip_lock_time', 1 << 1, 3600, 'Default IP Login lock time for lock brute force attack before tries.'),
			('integer', 'users_passwords_differents', 1 << 0, 3, 'Last differents passwords for user password changing.'), 
			('varchar', 'random_chain_alphabet', 1 << 0, '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz', 'Basic alphabet for random String chains.'), 
			('integer', 'random_chain_length', 1 << 0, 23, 'Default length for random String chains.');
	
		if $machine_add then begin
			insert into Users values();
			set $machine_id := last_insert_id();
		end;end if;
	
		if $user_agent_id is null then begin 
			insert into UsersAgents(`data`) values($user_agent);
			set $user_agent_id := last_insert_id();
		end;end if;
	
		if $local_ip_add then begin 
			insert into Ips values();
			set $local_ip_id := last_insert_id();
		end;end if;
	
		if $machine_session is null then begin 
			insert into Sessions(`user`) values($machine_id);
			set $machine_session := last_insert_id();
			insert into SessionsIps values();
			insert into SessionsIpsData(entity, `session`, ip, ok) values 
				(last_insert_id(), $machine_session, $local_ip_id, true);
			insert into SessionsUsersAgents values();
			insert into SessionsUsersAgentsData(entity, `session`, user_agent, ok) values 
				(last_insert_id(), $machine_session, $user_agent_id, true);
		end;end if;
	
		if $machine_add then 
			insert into UsersData(`session`, `procedure`, entity, nick, password, `accessible`) values
				($machine_session, $procedure_id, $machine_id, $machine, '', false);
		end if;
	
		if $local_ip_add then 
			insert into IpsData(`session`, `procedure`, entity, address) values
				($machine_session, $procedure_id, $local_ip_id, $local_ip);
		end if;
	
		if $guest_id is null then begin 
			insert into Users values();
			insert into UsersData(`session`, `procedure`, entity, nick, password, `accessible`) values
				($machine_session, $procedure_id, last_insert_id(), $guest, '', false);
		end;end if;
	
		if $unknown_id is null then begin 
			insert into Users values();
			set $unknown_id := last_insert_id();
			insert into UsersData(`session`, `procedure`, entity, nick, password, `accessible`) values
				($machine_session, $procedure_id, $unknown_id, `$unknown`, '', false);
		end;end if;
		if $unknown_session is null then begin 
			insert into Sessions(`user`) values($unknown_id);
			set $unknown_session := last_insert_id();
			insert into SessionsIps values();
			insert into SessionsIpsData(entity, `session`, ip, ok) values 
				(last_insert_id(), $unknown_session, $local_ip_id, true);
			insert into SessionsUsersAgents values();
			insert into SessionsUsersAgentsData(entity, `session`, user_agent, ok) values 
				(last_insert_id(), $unknown_session, $user_agent_id, true);
		end;end if;
	
		insert into default_settings_temporary_anp(`type`, name, `range`, value, description) values 
			('integer', 'machine_session', 1 << 0, $machine_session, 'Default machine session ID.'),
			('integer', 'unknown_session', 1 << 0, $unknown_session, 'Default unknown user session ID.');
	
		default_settings:begin
			
			declare $done bool default false;
			declare `$type` varchar(32);
			declare $name varchar(64);
			declare `$range` integer;
			declare $value text;
			declare $description text;
			declare $type_id integer;
			declare default_settings_cursor cursor for select `type`, name, `range`, value, description from default_settings_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_settings_cursor;
			default_settings_reader:loop 
				
				fetch default_settings_cursor into `$type`, $name, `$range`, $value, $description;
				if $done then 
					leave default_settings_reader;
				end if;
			
				set $type_id := (select id from _SettingsTypes where deleted is null && name = `$type` limit 1);
			
				if $type_id is null then begin 
					insert into SettingsTypes values();
					set $type_id := last_insert_id();
					insert into SettingsTypesData(`session`, `procedure`, entity, name) values 
						($machine_session, $procedure_id, $type_id, `$type`);
				end;end if;
			
				if (select 1 from _Settings where deleted is null && name = $name limit 1) is null then begin 
					insert into Settings values();
					insert into SettingsData(`procedure`, `session`, entity, `type`, name, `range`, value, description) values 
						($procedure_id, $machine_session, last_insert_id(), $type_id, $name, `$range`, $value, $description);
				end;end if;
			
			end loop default_settings_reader;
			close default_settings_cursor;
		
		end default_settings;
	
		drop temporary table default_settings_temporary_anp;
		
	end;^
	
	call common_tables_fill(null);^
	
	drop view if exists SessionsFullView;^
	create view SessionsFullView as select 
		sessions.id as id,
		users.entity as user_id,
		users.nick as nick,
		users.`accessible` as user_accessible,
		users.deleted as user_deleted, 
		ips.entity as ip_id,
		ips.address as ip,
		ips.`accessible` as ip_accessible, 
		ips.deleted as ip_deleted, 
		users_agents.id as user_agent_id,
		users_agents.`data` as user_agent, 
		sessions.date_in as date_in, 
		sessions.date_out as date_out,
		sessions.date_last as date_last
	from Sessions sessions 
	left join _Users users on sessions.`user` = users.entity 
	left join _SessionsIps sessions_ips on sessions_ips.`session` = sessions.id 
	left join _Ips ips on sessions_ips.ip = ips.entity 
	left join _SessionsUsersAgents sessions_users_agents on sessions_users_agents.`session` = sessions.id 
	left join UsersAgents users_agents on sessions_users_agents.user_agent = users_agents.id;^
	
	drop view if exists ProceduresFullView;^
	create view ProceduresFullView as select 
		procedures.id as id,
		`databases`.id as database_id,
		`databases`.name as `database`,
		procedures.name as `procedure`,
		`databases`.date_in as database_date_in,
		procedures.date_in as procedure_date_in,
		`databases`.date_out as database_date_out,
		procedures.date_out as procedure_date_out
	from Procedures procedures 
	join `Databases` `databases` on procedures.`database` = `databases`.id;^

	drop function if exists where_set;^
	create function where_set(
		$new_where text,
		$old_where text
	) returns text begin 
		return if(ifnull($new_where, '') = '', ifnull($old_where, ''), if(ifnull($old_where, '') = '', $new_where, concat('(', $new_where, ') && (', $old_where, ')')));
	end;^
	
	drop procedure if exists tables_list;^
	create procedure tables_list(
		in `$view` text,
		in $page integer,
		in $items_per_page integer,
		in `$where` text,
		in $order_by text,
		in $error bool,
		out $pages integer,
		out $items integer,
		out $results integer
	) begin 
		
		declare $item_from integer;
		-- declare exit handler 
		
		drop temporary table if exists AnPTTTableList;
		drop temporary table if exists AnPTTTableListResults;
	
		if $error then 
			set `$where` := where_set('false', `$where`);
		end if;
	
		-- set @i := 0;
		-- prepare list_statement from concat('create temporary table AnPTTTableList as select *, @i := @i + 1 as `row` from (select * from ', `$view`, ' list_view', if(ifnull(`$where`, '') = '', '', concat(' where ', `$where`)), if(ifnull($order_by, '') = '', '', concat(' order by ', $order_by)), ') `table`;');
		prepare list_statement from concat('create temporary table AnPTTTableList as select *, rownum() as `row` from (select * from ', `$view`, ' list_view', if(ifnull(`$where`, '') = '', '', concat(' where ', `$where`)), if(ifnull($order_by, '') = '', '', concat(' order by ', $order_by)), ') subtable;');
		execute list_statement;
		deallocate prepare list_statement;
	
		set $items_per_page := (case
			when $items_per_page is null then ifnull(settings_get_int('items_per_page', false), 30)
			when $items_per_page < 1 then 1
		else $items_per_page end);
		set $items := (select count(1) from AnPTTTableList limit 1);
		set $pages := ceil($items / $items_per_page);
		set $page := (case
			when ifnull($page, 0) < 1 then 1
			when $pages > 0 and $page > $pages then $pages
		else $page end);
		set $item_from := ($page - 1) * $items_per_page;
	
		prepare list_statement_results from concat('create temporary table AnPTTTableListResults as select * from AnPTTTableList limit ', $item_from, ',', $items_per_page, ';');
		execute list_statement_results;
		deallocate prepare list_statement_results;
	
		set $results := (select count(1) from AnPTTTableListResults limit 1);
		
		select * from AnPTTTableListResults;
	
		drop temporary table if exists AnPTTTableList;
		drop temporary table if exists AnPTTTableListResults;
	
	end;^
	
	drop function if exists machine_get_session_id;^
	create function machine_get_session_id() returns integer begin 
		return settings_get_int('machine_session', false);
	end;^
	
	drop view if exists ErrorsView;^
	create view ErrorsView as select * from Errors where date_out is null;^
	
	drop view if exists ParametersView;^
	create view ParametersView as select * from Parameters where date_out is null;^
	
	drop view if exists MessagesView;^
	create view MessagesView as select * from Messages where date_out is null;^
	
	-- Las dos siguientes vistas est�n puestas por culpa de las banderas de la I18N.
	drop view if exists FilesSingleView;^
	create view FilesSingleView as select 
		null as id,
		null as `type`,
		null as name,
		null as weight,
		null as date_in;^
		
	drop view if exists LanguagesMultiView;^
	create view LanguagesMultiView as select 
		null as id,
		null as country_id,
		null as country,
		null as country_key,
		null as country_code, 
		null as flag_id,
		null as flag_type,
		null as flag_name,
		null as flag_weight, 
		null as name,
		null as `key`,
		null as code,
		null as date_in;^
		
	drop view if exists TerminalView;^
	create view TerminalView as select 
		terminal.id as id,
		sessions.id as session_id,
		sessions.user_id as user_id,
		sessions.nick as nick,
		sessions.user_accessible as user_accessible,
		sessions.user_deleted as user_deleted, 
		sessions.ip_id as ip_id,
		sessions.ip as ip,
		sessions.ip_accessible as ip_accessible, 
		sessions.ip_deleted as ip_deleted, 
		sessions.user_agent_id as user_agent_id,
		sessions.user_agent as user_agent, 
		sessions.date_in as session_date_in, 
		sessions.date_out as session_date_out,
		sessions.date_last as session_date_last,
		procedures.id as procedure_id,
		procedures.id as database_id,
		procedures.`database` as `database`,
		procedures.`procedure` as `procedure`,
		procedures.database_date_in as database_date_in,
		procedures.procedure_date_in as procedure_date_in,
		`data`.id as data_id,
		`data`.`data` as `data`,
		`data`.date_in as data_date_in,
		terminal.date_in as date_in
	from Terminal terminal 
	left join SessionsFullView sessions on terminal.`session` = sessions.id 
	left join ProceduresFullView procedures on terminal.`procedure` = procedures.id 
	join TerminalData `data` on terminal.`data` = `data`.id 
	where terminal.date_out is null && `data`.date_out is null;^
	
	drop procedure if exists terminal_set;^
	create procedure terminal_set(
		in `$session` integer,
		in `$database` varchar(64),
		in `$procedure` varchar(64), 
		in `$data` text
	) begin 
		
		declare $data_id integer default (select id from TerminalData where `data` = `$data` limit 1);
		declare $procedure_id integer default null;
	
		if ifnull(`$database`, '') != '' && ifnull(`$procedure`, '') != '' then 
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		end if;
	
		if ifnull(`$session`, 0) < 1 || (select 1 from Sessions where id = `$session` limit 1) is null then 
			set `$session` := null;
		end if;
	
		if $data_id is null then begin 
			insert into TerminalData(`data`) values(`$data`);
			set $data_id := last_insert_id();
		end;end if;
	
		insert into Terminal(`session`, `procedure`, `data`) values(`$session`, $procedure_id, $data_id);
		
	end;^
	
	drop procedure if exists terminal_clean;^
	create procedure terminal_clean() begin 
		
		delete from Terminal where true;
		delete from TerminalData where true;
	
		alter table Terminal auto_increment = 1;
		alter table TerminalData auto_increment = 1;
	
	end;^
	
delimiter ;