create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists string_to_characters_array;^
	create function string_to_characters_array(
		$string varchar(16383)
	) returns varchar(16383) begin 
		
		declare `$array` varchar(16383) default '';
		declare $i integer default 1;
		declare $l integer default length($string);
		declare `$character` char(1);
		
		while $i <= $l do
			set `$character` := substring($string, $i, 1);
			set `$array` := concat(`$array`, ',"', if(`$character` in ('"', '\\'), concat('\\', `$character`), `$character`), '"');
			set $i := $i + 1;
		end while;
	
		return concat('[', substring(`$array`, 2, 16383), ']');
	end;^
	
	drop function if exists errors_to_integer;^
	create function errors_to_integer(
		$code varchar(2048)
	) returns bigint begin
		
		declare $alphabet char(64) default settings_get('base64_alphabet', false);
		declare $results bigint default 0;
	
		if substring($code, 1, 1) = '[' then 
			set $results := (select sum(bw(hexa, (i - 1) * 6)) from json_table($code, '$[*]' columns(
				i for ordinality,
				hexa tinyint path '$'
			)) as b where i < 11);
		else begin 
			
			declare $i tinyint default 0;
			declare $l smallint default length($code);
			declare `$position` integer;
		
			while $i < $l && $i < 10 do 
				set `$position` := locate(substring($code, $i + 1, 1), binary $alphabet);
				set $results := $results | if(`$position`, bw(`$position` - 1, $i * 6), 0);
				set $i := $i + 1;
			end while;
		
		end;end if;
	
		return $results;
	end;^
	
	drop function if exists errors_to_string;^
	create function errors_to_string(
		$code varchar(2048)
	) returns varchar(512) begin
		
		declare $alphabet char(64) default settings_get('base64_alphabet', false);
		declare $results varchar(512) default '';
	
		if substring($code, 1, 1) = '[' then 
			set $results := (select group_concat(substring($alphabet, hexa + 1, 1) separator '') from json_table($code, '$[*]' columns(
				hexa tinyint path '$'
			)) as b);
		else begin 
			
			declare `$integer` bigint default $code + 0;
		
			while $code do
				set $results := concat($results, substring($alphabet, ($code & 0x3F) + 1, 1));
				set $code := bw($code, -6);
			end while;
		
		end;end if;
	
		return if($results = '', substring($alphabet, 1, 1), $results);
	end;^
	
	drop function if exists errors_to_array;^
	create function errors_to_array(
		$code varchar(512)
	) returns varchar(2048) begin 
		
		declare $alphabet varchar(64) default settings_get('base64_alphabet', false);
		declare $l integer default length($code) + 1;
		declare $i integer default 1;
		declare $results varchar(2048) default '';
	
		while $i < $l do
			set $results := concat($results, ',', (locate(substring($code, $i, 1), binary $alphabet) - 1));
			set $i := $i + 1;
		end while;
	
		return concat('[', substring($results, 2, 2048), ']');
	end;^
	
	drop function if exists errors_to_array_back;^
	create function errors_to_array_back(
		$code varchar(2048)
	) returns varchar(512) begin
		
		declare $alphabet varchar(64) default settings_get('base64_alphabet', false);
	
		return (select group_concat(substring($alphabet, hexa + 1, 1) separator '') from json_table($code, '$[*]' columns(
			hexa tinyint path '$'
		)) as b);
	end;^
	
	drop function if exists convert_to_base;^
	create function convert_to_base(
		`$integer` bigint,
		$base tinyint
	) returns varchar(2048) begin 
		
		declare $results varchar(2048) default '';
		declare $alphabet varchar(64) default concat('0123456789', substring(settings_get('base64_alphabet', false), 1, 54));
	
		while `$integer` do
			set $results := concat(substring($alphabet, (`$integer` % $base) + 1, 1), $results);
			set `$integer` := floor(`$integer` / $base);
		end while;
		
		return $results;
	end;^
	
	drop function if exists errors_to_array_binary;^
	create function errors_to_array_binary(
		$code varchar(2048)
	) returns varchar(3584) begin 
		return concat('[', (select group_concat(concat('"', right(concat('000000', convert_to_base(hexa, 2)), 6), '"') separator ',') from json_table(if(substring($code, 1, 1) = '[', $code, errors_to_array($code)), '$[*]' columns(
			hexa tinyint path '$'
		)) as b), ']');
	end;^
	
	drop function if exists errors_has;^
	create function errors_has(
		$code varchar(512)
	) returns boolean begin 
		return if(ifnull($code, '') = '', false, binary $code regexp concat('[^', substring(settings_get('base64_alphabet', null), 1, 1), ']'));
	end;^
	
	drop function if exists errors_get_hexa;^
	create function errors_get_hexa(
		$alphabet char(64),
		$code varchar(512),
		`$position` smallint
	) returns tinyint begin 
		
		declare $i tinyint default locate(binary substring($code, `$position` + 1, 1), $alphabet) - 1;
	
		return if($i < 0, null, $i);
	end;^
	
	drop function if exists errors_get_bit;^
	create function errors_get_bit(
		$code varchar(512),
		`$position` smallint
	) returns bool begin
		return (errors_get_hexa(settings_get('base64_alphabet', 0), $code, floor(`$position` / 6)) & (1 << (`$position` % 6))) != 0;
	end;^
	
	drop function if exists errors_set_in;^
	create function errors_set_in(
		$alphabet char(64),
		$code varchar(512),
		$hexa tinyint,
		`$position` smallint
	) returns varchar(512) begin 
		
		declare $l smallint default length($code);
		declare $zero char(1) default substring($alphabet, 1, 1);
	
		while length($code) <= `$position` do
			set $code := concat($code, $zero);
		end while;
	
		return if(
			`$position` >= 0 && `$position` < $l && $hexa >= 0 && $hexa < 0x40, 
			concat(left($code, `$position`), substring($alphabet, $hexa + 1, 1), right($code, $l - `$position` - 1)), 
			$code
		);
	end;^
	
	drop function if exists errors_bitwise;^
	create function errors_bitwise(
		$code varchar(512),
		$bits smallint
	) returns varchar(512) begin 
		
		declare $alphabet varchar(64) default settings_get('base64_alphabet', false);
		declare `$first` char(1) default substring($alphabet, 1, 1);
		declare $results varchar(512);
	
		if ifnull($code, '') = '' then 
			set $results := `$first`;
		elseif $bits = 0 then 
			set $results := $code;
		else begin 
			
			declare $rest tinyint default abs($bits) % 6;
			declare $r tinyint default 6 - $rest;
			declare `$start` integer default floor(abs($bits) / 6);
			
			if $bits > 0 then begin 
				
				declare $fill varchar(512) default '';
				declare $l smallint default length($code);
				declare $i smallint default $l;
				declare `$last` tinyint default locate(substring($code, $l, 1), binary $alphabet) - 1 >> $r;
			
				while length($fill) < `$start` do
					set $fill := concat(`$first`, $fill);
				end while;
			
				if $bits % 6 then begin 
					
					declare $l smallint default length($code);
					declare $i smallint default $l;
					declare `$last` tinyint default locate(substring($code, $l, 1), binary $alphabet) - 1 >> $r;
				
					if `$last` then 
						set $code := concat($code, substring($alphabet, `$last` + 1, 1));
					end if;
				
					while $i > 1 do
						set $code := concat(substring($code, 1, $i - 1), substring($alphabet, (
							((locate(substring($code, $i, 1), binary $alphabet) - 1 << $rest) & 0x3F) |
							(locate(substring($code, $i - 1, 1), binary $alphabet) - 1 >> $r)
						) + 1, 1), substring($code, $i + 1, 512));
						set $i := $i - 1;
					end while;
					set $code := concat(substring($alphabet, ((locate(substring($code, 1, 1), binary $alphabet) - 1 << $rest) & 0x3F) + 1, 1), substring($code, 2, 512));
				
				end;end if;
			
				set $results := concat($fill, $code);
			
			end;else begin 
				if `$start` > 0 then 
					set $code := substring($code, `$start` + 1, 512);
				end if;
			
				if abs($bits) % 6 then begin 
					
					declare $l smallint default length($code);
					declare $i smallint default 1;
				
					while $i < $l do
						set $code := concat(substring($code, 1, $i - 1), substring($alphabet, (
							(locate(substring($code, $i, 1), binary $alphabet) - 1 >> $rest) |
							((locate(substring($code, $i + 1, 1), binary $alphabet) - 1 << $r) & 0x3F)
						) + 1, 1), substring($code, $i + 1, 512));
						set $i := $i + 1;
					end while;
					set $code := concat(substring($code, 1, $l - 1), substring($alphabet, (locate(substring($code, $l, 1), binary $alphabet) - 1 >> $rest) + 1, 1));
				
				end;end if;
			
				set $results := $code;
			
			end;end if;
			
		end;end if;
	
		return if($results = '', `$first`, $results);
	end;^
	
	drop function if exists errors_compact;^
	create function errors_compact(
		$code varchar(512),
		$alphabet char(64)
	) returns varchar(512) begin
		
		declare $l smallint default length($code);
		declare $zero char(1) default substring(ifnull($alphabet, settings_get('base64_alphabet', 0)), 1, 1);
		
		while $l > 1 && substring($code, $l, 1) = $zero do
			set $l := $l - 1;
		end while;
	
		if $l then
			set $code := substring($code, 1, $l);
		end if;
		
		return $code;
	end;^
	
	drop function if exists errors_length_code;^
	create function errors_length_code(
		$alphabet char(64),
		`$bit` smallint,
		`$length` smallint
	) returns varchar(512) begin
		
		declare $length_code varchar(512) default substring($alphabet, 1, 1);
		declare $l smallint default floor(`$length` / 6);
		
		if `$length` is not null && `$length` then begin 
			
			declare $k tinyint default `$length` % 6;
		
			set $length_code := '';
			while length($length_code) < $l do
				set $length_code := concat(substring($alphabet, 64, 1), $length_code);
			end while;
			if $k then 
				set $length_code := concat(substring($alphabet, ~-(1 << $k) + 1, 1), $length_code);
			end if;
			if `$bit` then 
				set $length_code := errors_bitwise($length_code, `$bit`);
			end if;
		
		end;end if;
	
		return $length_code;	
	end;^
	
	drop function if exists errors_reset;^
	create function errors_reset(
		$code varchar(512),
		`$bit` smallint,
		`$length` smallint,
		$alphabet char(64)
	) returns varchar(512) begin
		
		declare $zero char(1);
		declare $l smallint;
	
		if ifnull($alphabet, '') in ('', '0') then 
			set $alphabet := settings_get('base64_alphabet', false);
		end if;
		set $zero := substring($alphabet, 1, 1);
		if $code is null then
			set $code := '';
		end if;
		
		if `$length` > 0 then begin 
		
			declare $i smallint default floor(`$bit` / 6);
			declare $rest tinyint default `$bit` % 6;
			declare $subtotal smallint default `$length` - (6 - $rest);
			declare `$last` tinyint default $subtotal % 6;
			declare $start_fragment tinyint default if(`$length` + $rest < 6, (`$length` + $rest) % 6, 0);
			declare $m smallint default length($code);
			declare $total smallint default floor((`$bit` + `$length`) / 6);
		
			set $l := $i + 1 + floor($subtotal / 6);
		
			if `$length` || $rest then begin 
				set $code := errors_set_in($alphabet, $code, errors_get_hexa($alphabet, $code, $i) & (~-(1 << $rest) | if($start_fragment, ~-(1 << 6 - $start_fragment) << $start_fragment, 0)), $i);
				set $i := $i + 1;
			end;end if;
			if $m < $total then begin 
				set $code := substring($code, 1, $i);
				while length($code) < $total do
					set $code := concat($code, $zero);
				end while;
			end;else begin
				set $l := if($l > $m, $m, $l - 1);
				while $i <= $l do 
					set $code := errors_set_in($alphabet, $code, 0, $i);
					set $i := $i + 1;
				end while;
				if `$last` > 0 && $i < $m then 
					set $code = errors_set_in($alphabet, $code, errors_get_hexa($alphabet, $code, $i) & (~-(1 << 6 - `$last`) << `$last`), $i);
				end if;
			end;end if;
		
		end;else begin
			
			set $l := floor(`$bit` / 6);
		
			while length($code) < $l do
				set $code := concat($code, $zero);
			end while;
		
		end;end if;
		
		return $code;
	end;^
	
	drop function if exists errors_join;^
	create function errors_join(
		$code varchar(512),
		$error varchar(512),
		`$bit` smallint,
		`$length` smallint 
	) returns varchar(512) begin 
		
		declare $alphabet char(64) default settings_get('base64_alphabet', 0);
	
		set $code := errors_reset($code, `$bit`, `$length`, $alphabet);
	
		if errors_has($error) then begin
			
			declare $l smallint;
			declare $i smallint default 0;
			declare $zero char(1) default substring($alphabet, 1, 1);
		
			if ifnull(`$bit`, 0) then begin 
				set $error := errors_bitwise($error, `$bit`);
				set $i := floor(`$bit` / 6);
			end;end if;
			set $l := length($error);
		
			while length($code) <= $l do
				set $code := concat($code, $zero);
			end while;
		
			while $i < $l do
				set $code := errors_set_in($alphabet, $code, ifnull(errors_get_hexa($alphabet, $code, $i), 0) | errors_get_hexa($alphabet, $error, $i), $i);
				set $i := $i + 1;
			end while;
		
		end;end if;
		
		return errors_compact($code, $alphabet);
	end;^
	
	drop function if exists errors_set;^
	create function errors_set(
		$code varchar(512),
		$error bigint,
		`$bit` integer,
		`$length` integer
	) returns varchar(512) begin
		return errors_join($code, errors_to_string($error), `$bit`, `$length`);
	end;^
	
	drop function if exists errors_messages_get_group;^
	create function errors_messages_get_group(
		$name varchar(64)
	) returns integer begin 
		return (select id from ErrorsMessagesGroups where date_out is null && name = $name limit 1);
	end;^
	
	drop procedure if exists errors_messages_clean;^
	create procedure errors_messages_clean(
		in $name varchar(64)
	) begin 
		
		declare $group_id integer default errors_messages_get_group($name);
		
		delete from ErrorsMessages where `group` = $group_id;
		delete from ErrorsMessagesGroups where id = $group_id;
		
	end;^
	
	drop procedure if exists errors_messages_add;^
	create procedure errors_messages_add(
		in $name varchar(64),
		in $i smallint,
		in $message varchar(64)
	) begin 
		
		declare $group_id integer default errors_messages_get_group($name);
	
		if $group_id is null then begin 
			insert into ErrorsMessagesGroups(name) values($name);
			set $group_id := last_insert_id();
		end;end if;
	
		if (select 1 from ErrorsMessages where date_out is null && `group` = $group_id && i = $i && name = $message limit 1) is null then begin 
			if (select 1 from ErrorsMessages where date_out is null && `group` = $group_id && i = $i limit 1) is null then 
				insert into ErrorsMessages(`group`, i, name) values($group_id, $i, $message);
			else 
				update ErrorsMessages set name := $message where date_out is null && `group` = $group_id && i = $i;
			end if;
		end;end if;
		
	end;^
	
	drop procedure if exists errors_get_data;^
	create procedure errors_get_data(
		in $name varchar(64),
		in $code varchar(512),
		in `$session` bool
	) begin 
		
		declare `$length` smallint default length($code);
		declare $bits smallint default `$length` * 6;
		declare $group_id integer default (select id from ErrorsMessagesGroups where date_out is null && name = $name limit 1);
		declare $session_id integer default if(`$session`, (select id from ErrorsMessagesGroups where date_out is null && name = 'sessions_validate_full' limit 1), null);
		
		drop temporary table if exists default_errors_messages_temporary_anp;
		create temporary table default_errors_messages_temporary_anp(
			id integer not null auto_increment,
			i smallint not null,
			message varchar(64) not null,
			constraint default_errors_messages_temporary_anp_id primary key(id)
		);
	
		if $group_id is not null then 
			insert into default_errors_messages_temporary_anp(i, message)
				select i, name from ErrorsMessages where 
					date_out is null && 
					`group` = $group_id && 
					i >= 0 && 
					name is not null;
		end if;
	
		if $session_id is not null then 
			insert into default_errors_messages_temporary_anp(i, message)
				select i, name from ErrorsMessages where 
					date_out is null && 
					`group` = $session_id && 
					i >= 0 && 
					name is not null && 
					i not in (select i from default_errors_messages_temporary_anp);
		end if;
	
		set @i := -1;

		insert into default_errors_messages_temporary_anp(i, message) 
			select unknown_messages.i, concat('message_', unknown_messages.i) 
			from (
				select  @i := @i + 1 as i from information_schema.columns
			) unknown_messages 
			where unknown_messages.i not in (select i from default_errors_messages_temporary_anp)
			limit $bits;
		
		set @i := -1;
		
		select 
			errors_messages.i as i,
			`characters`.hexa as hexa,
			errors_messages.message as message 
		from (
			select 
				@i := @i + 1 as i, 
				errors_to_integer(substring($code, @i + 1, 1)) as hexa 
			from information_schema.columns a 
			join information_schema.columns b on true 
			limit `$length` 
		) `characters` 
		join default_errors_messages_temporary_anp errors_messages on 
			`characters`.hexa && 
			floor(errors_messages.i / 6) = `characters`.i && 
			(1 << (errors_messages.i % 6)) & `characters`.hexa;
	
		drop temporary table default_errors_messages_temporary_anp;
		
	end;^
	
	drop procedure if exists errors_messages_set;^
	create procedure errors_messages_set(
		in `$session` text
	) begin 
			
		declare $group_done bool default false;
		declare $group_i integer;
		declare $group_name varchar(64);
		declare $counter integer;
		declare group_cursor cursor for select i, name from ErrorsTemporaryGroups where date_out is null;
		declare continue handler for not found set $group_done := true;
	
		open group_cursor;
		group_reader:loop 
			
			fetch group_cursor into $group_i, $group_name;
			if $group_done then 
				leave group_reader;
			end if;
		
			call errors_messages_clean($group_name);
			set $counter := 30;
		
			messages_block:begin 
				
				declare $messages_done bool default false;
				declare $i smallint default 30;
				declare $message varchar(64);
				declare messages_cursor cursor for select i, message from ErrorsTemporaryMessages where date_out is null && `group` = $group_i;
				declare continue handler for not found set $messages_done := true;
	
				open messages_cursor;
				messages_reader:loop 
					
					fetch messages_cursor into $i, $message;
					if $messages_done then 
						leave messages_reader;
					end if;
				
					call errors_messages_add($group_name, ifnull($i, $counter), $message);
					set $counter := $counter + 1;
		
				end loop messages_reader;
				close messages_cursor;
				
			end messages_block;
		
		end loop group_reader;
		close group_cursor;
		
		delete from ErrorsTemporaryGroups where true;
		delete from ErrorsTemporaryMessages where true;
		
	end;^
	
	
delimiter ;