use AnP;

delimiter ;^

	drop view if exists SessionsIpsValidateView;^
	create view SessionsIpsValidateView as select 
		ips.entity as entity, 
		ips.address as address, 
		ips.`accessible` as `accessible`, 
		ips.deleted as deleted, 
		sessions_ips.`session` as `session` 
	from _SessionsIps sessions_ips 
	join _Ips ips on sessions_ips.ip = ips.entity;^
	
	drop view if exists SessionsUsersAgentsValidateView;^
	create view SessionsUsersAgentsValidateView as select 
		users_agents.id as id, 
		users_agents.`data` as `data`, 
		sessions_users_agents.`session` as `session` 
	from _SessionsUsersAgents sessions_users_agents 
	join UsersAgents users_agents on sessions_users_agents.user_agent = users_agents.id;^
	
	drop view if exists SessionsValidateView;^
	create view SessionsValidateView as select 
		sessions.id as id,
		users.entity as user_id,
		users.nick as nick,
		users.`accessible` as user_accessible,
		users.deleted as user_deleted, 
		ips.entity as ip_id,
		ips.address as ip,
		ips.`accessible` as ip_accessible, 
		ips.deleted as ip_deleted, 
		users_agents.id as user_agent_id,
		users_agents.`data` as user_agent,
		sessions.date_out as date_out,
		sessions.date_last as date_last
	from Sessions sessions 
	left join _Users users on sessions.`user` = users.entity 
	left join _SessionsIps sessions_ips on sessions.id = sessions_ips.`session`
	left join _Ips ips on sessions_ips.ip = ips.entity
	left join _SessionsUsersAgents sessions_users_agents on sessions.id = sessions_users_agents.`session`
	left join UsersAgents users_agents on sessions_users_agents.user_agent = users_agents.id;^
	
	-- drop function if exists sessions_has_permissions;^
	create function if not exists sessions_has_permissions(
		`$session` integer,
		$permissions text
	) returns bool begin
		return true;
	end;^
	
	-- drop function if exists sessions_is_banned;^
	create function if not exists sessions_is_banned(
		`$session` integer
	) returns bool begin 
		return false;
	end;^
	
	-- drop function if exists sessions_timeout;^
	create function if not exists sessions_timeout(
		`$session` integer
	) returns bool begin
		return false;
	end;^
	
	-- drop function if exists sessions_is_guest;^
	create function if not exists sessions_is_guest(
		`$session` integer
	) returns bool begin 
		return true;
	end;^
	
	-- drop function if exists users_settings_get;^
	create function if not exists users_settings_get(
		`$user` integer,
		$name varchar(64)
	) returns text begin 
		return (select value from _Settings where deleted is null && name = $name limit 1);
	end;^
	
	-- drop function if exists users_settings_get_int;^
	create function if not exists users_settings_get_int(
		`$user` integer,
		$name varchar(64)
	) returns bool begin 
		return string_to_integer(users_settings_get(`$user`, $name));
	end;^
	
	-- drop function if exists users_settings_get_bool;^
	create function if not exists users_settings_get_bool(
		`$user` integer,
		$name varchar(64)
	) returns bool begin 
		return string_to_bool(users_settings_get(`$user`, $name));
	end;^

	drop procedure if exists sessions_validate_full;^
	create procedure sessions_validate_full(
		in `$session` text, 
		in $permissions text, 
		out $error varchar(512),
		out $session_id integer,
		out $ip_id integer,
		out $user_agent_id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_validate_full';
		declare $procedure_id integer;
		declare $error_integer integer;
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
	
		set $error_integer := (case 
			when `$session` is null then 1 << 0 
			when `$session` = '' then 1 << 1 
			when `$session` not like '%,%,%' then 1 << 2
		else 0 end);
	
		if !$error_integer then begin 
			
			declare $separator1 integer default locate(',', `$session`);
			declare $separator2 integer default locate(',', `$session`, $separator1 + 1);
			declare $session_text text default if($separator1, trim(substring(`$session`, 1, $separator1 - 1)), null);
			declare $ip varchar(39) default if($separator2, trim(substring(`$session`, $separator1 + 1, $separator2 - $separator1 - 1)), null);
			declare $user_agent text default if($separator2, trim(substring(`$session`, $separator2 + 1, 9999999)), null);
	
			if $ip = 'null' then 
				set $ip := null;
			end if;
		
			if $user_agent = 'null' then 
				set $user_agent := null;
			end if;
		
			set $session_id := if($session_text regexp '^[0-9]+$', cast($session_text as integer), null);
		
			set $error_integer := (
				((case 
					when $session_id is null then 1 << 0
					when $session_id < 1 then 1 << 1 
				else 0 end) << 0) | 
				((case 
					when $ip is null then 1 << 0
					when $ip = '' then 1 << 1 
					-- when $ip not regexp '^[0-9]{1,3}?(\.[0-9]{1,3}){1,3}$' then 1 << 2
				else 0 end) << 2) |
				((case 
					when $user_agent is null then 0 -- 1 << 0
					when $user_agent = '' then 0 -- 1 << 1
				else 0 end) << 5) |
			0) << 3;
		
			set $ip_id := (select entity from _Ips where deleted is null && if($ip is null, address is null, address = $ip) limit 1);
			set $user_agent_id := (select id from UsersAgents where date_out is null && if($user_agent is null, `data` is null, `data` = $user_agent) limit 1);
			
			if $ip_id is null then begin 
				insert into Ips values();
				set $ip_id := last_insert_id();
				insert into IpsData(`procedure`, `session`, entity, address, `accessible`) values 
					($procedure_id, ifnull((
						select id from Sessions where id = $session_id
					), settings_get_int('unknown_session', false)), $ip_id, $ip, settings_get_bool('allow_any_new_ip', false));
			end;end if;
		
			if $user_agent_id is null then begin 
				insert into UsersAgents(`data`) values($user_agent);
				set $user_agent_id := last_insert_id();
			end;end if;
		
			if !$error_integer then begin 
				
				declare $user_id integer default (select `user` from Sessions where id = $session_id limit 1);
				declare $check_ip bool default users_settings_get_bool($user_id, 'session_check_ip');
				declare $check_user_agent bool default users_settings_get_bool($user_id, 'session_check_user_agent');
			
				set $error_integer := ifnull((
					select (
						(if(user_id is null, 1 << 0, (
							if(user_accessible || sessions_is_guest($session_id), 0, 1 << 1) |
							if(user_deleted is null, 0, 1 << 2) |
							ifnull((select 1 << 3 from _Users where entity = user_id && concat('%"', nick, '"%') like settings_get('users_excluded', false) limit 1), 0) | 
						0)) << 1) |
						(if(
							$check_ip, 
							if($ip_id = ip_id, if(ip_id is null, 1 << 1, (
								if(ip_accessible, 0, 1 << 3) |
								if(ip_deleted is null, 0, 1 << 4) |
								ifnull((select 1 << 5 from _Ips where entity = ip_id && concat('%"', ip, '"%') like settings_get('ips_excluded', false) limit 1), 0) | 
							0)), 1 << 0), 
							if($ip_id is null, 1 << 1, ifnull((
								select (
									if(`accessible`, 0, 1 << 3) |
									if(deleted is null, 0, 1 << 4) |
									if(concat('%"', address, '"%') like settings_get('ips_excluded', false), 1 << 5, 0) | 
								0) from _Ips where entity = $ip_id limit 1
							), 1 << 2))
						) << 5) |
						(if(
							$check_user_agent, 
							if($user_agent_id = user_agent_id, if(user_agent_id is null, 1 << 1, 0), 1 << 0),
							if($user_agent_id is null, 1 << 1, ifnull((select 0 from UsersAgents where date_out is null && id = $user_agent_id limit 1), 1 << 2))
						) << 11) |
						((case 
							when date_out is not null then 1 << 0
							when sessions_timeout(id) then 1 << 1
						else 0 end) << 14) |
						if(sessions_is_banned(id), 1 << 16, 0) |
					0) from SessionsValidateView where id = $session_id limit 1
				), 1 << 0) << 10;
					
				if $check_ip && (select 1 from _SessionsIps where 
					`session` = $session_id && 
					ip = $ip_id 
				limit 1) is null then begin 
					
					declare $session_ip_id integer default (select entity from _SessionsIps where `session` = $session_id limit 1);
				
					if $session_ip_id is null then begin 
						insert into SessionsIps values();
						insert into SessionsIpsData(entity, `session`, ip, ok) values 
							(last_insert_id(), $session_id, $ip_id, !$check_ip);
					end;else 
						insert into SessionsIpsData(entity, `session`, ip, ok) values 
							($session_ip_id, $session_id, $ip_id, !$check_user_agent);
					end if;
					
				end;end if;
					
				if $check_user_agent && (select 1 from _SessionsUsersAgents where 
					`session` = $session_id && 
					user_agent = $user_agent_id 
				limit 1) is null then begin 
					
					declare $session_user_agent_id integer default (select entity from _SessionsUsersAgents where `session` = $session_id limit 1);
				
					if $session_user_agent_id is null then begin 
						insert into SessionsUsersAgents values();
						insert into SessionsUsersAgentsData(entity, `session`, user_agent, ok) values 
							(last_insert_id(), $session_id, $user_agent_id, !$check_user_agent);
					end;else 
						insert into SessionsUsersAgentsData(entity, `session`, user_agent, ok) values 
							($session_user_agent_id, $session_id, $user_agent_id, !$check_user_agent);
					end if;
				
				end;end if;
			
			end;end if;
		
			set $session_id := (select id from Sessions where id = $session_id limit 1);
			
		end;end if;
	
		if !$error_integer then 
			set $error_integer := if(sessions_has_permissions($session_id, $permissions), 0, 1 << 27);
		end if;
	
		set $error := errors_set('', $error_integer, 1, 0);
	
	end;^
	
	drop function if exists sessions_get_id;^
	create function sessions_get_id(
		`$session` text
	) returns integer begin 
		
		declare $session_text text default if(ifnull(`$session`, '') = '', null, substring(`$session`, 1, locate(',', `$session`) - 1));
	
		return if($session_text regexp '^[0-9]+$', cast($session_text as integer), null);
	end;^
	
	drop procedure if exists anp_initial_sessions_set_errors_messages;^
	create procedure anp_initial_sessions_set_errors_messages(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'anp_initial_sessions_set_errors_messages';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $error varchar(512) default '';
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;
	
		insert into ErrorsTemporaryGroups(i, name) values 
			(1, 'sessions_validate_full');
	
		insert into ErrorsTemporaryMessages(`group`, i, message) values 
			(1, 0, 'exception'),
			(1, 1, 'session_null'),
			(1, 2, 'session_empty'),
			(1, 3, 'session_bad_format'),
			(1, 4, 'session_id_null'),
			(1, 5, 'session_id_bad'),
			(1, 6, 'ip_null'),
			(1, 7, 'ip_empty'),
			(1, 8, 'ip_bad_format'),
			(1, 9, 'user_agent_null'),
			(1, 10, 'user_agent_empty'),
			(1, 11, 'session_not_exists'),
			(1, 12, 'user_id_null'),
			(1, 13, 'user_unaccessible'),
			(1, 14, 'user_deleted'),
			(1, 15, 'user_excluded'),
			(1, 16, 'ip_not_same'),
			(1, 17, 'ip_id_null'),
			(1, 18, 'ip_not_exists'),
			(1, 19, 'ip_unaccessible'),
			(1, 20, 'ip_deleted'),
			(1, 21, 'ip_excluded'),
			(1, 22, 'user_agent_not_same'),
			(1, 23, 'user_agent_id_null'),
			(1, 24, 'user_agent_not_exists'),
			(1, 25, 'session_date_out'),
			(1, 26, 'session_timeout'),
			(1, 27, 'session_banned'),
			(1, 28, 'session_not_perimssions');
		
		call errors_messages_set(`$session`);
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, '{}', 1 << 1);
		
	end;^
	
	drop procedure if exists traces_set;^
	create procedure traces_set(
		in `$session` text, 
		in $project varchar(64), 
		in `$path` varchar(512), 
		in $own_error varchar(512),
		in $own_parameters text, 
		in $trace text, 
		in $message text, 
		in $line integer, 
		out $error varchar(512), 
		out $id integer 
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'traces_set';
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $project_id integer;
		declare $path_id integer;
		declare $error_id integer;
		declare $parameters_id integer;
		declare $trace_id integer;
		declare $message_id integer;
		declare $parameters text default json_set('{}', 
			'$.project', $project, 
			'$.path', `$path`,
			'$.own_error', $own_error,
			-- '$.own_parameters', $own_parameters,
			-- '$.trace', $trace, 
			-- '$.message', $message,
			'$.line', $line
		);
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text; 
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		set $id := 0;
		call sessions_validate_full(`$session`, null, $error, $session_id, $ip_id, $user_agent_id);
	
		call projects_get($project, $project_id);
		call traces_files_get(`$path`, $path_id);
		call errors_get($own_error, $error_id);
		call parameters_get($own_parameters, $parameters_id);
		call lines_get($trace, $trace_id);
		call messages_get($message, $message_id);
	
		insert into Traces(`session`, ip, user_agent, project, file, own_error, own_parameters, trace, message, line) values 
			($session_id, $ip_id, $user_agent_id, $project_id, $path_id, $error_id, $parameters_id, $trace_id, $message_id, $line);
		set $id := last_insert_id();
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
	
		set $error := errors_set('', 0, 0, 0);
		
	end;^
	
	call anp_initial_sessions_set_errors_messages(null);^

	drop function if exists password_validate;^
	create function password_validate(
		`$password` varchar(256)
	) returns integer begin 
		return (case 
			when `$password` is null then 1 << 0
			when `$password` = '' then 1 << 1
		else (
			if(length($password) < ifnull(settings_get_int('password_minimum_length', false), 1), 1 << 2, 0) |
			if(ifnull(settings_get_bool('password_needs_lower', false), false) && binary `$password` not regexp binary '[a-z]', 1 << 3, 0) |
			if(ifnull(settings_get_bool('password_needs_upper', false), false) && binary `$password` not regexp binary '[A-Z]', 1 << 4, 0) |
			if(ifnull(settings_get_bool('password_needs_digit', false), false) && `$password` not regexp '[0-9]', 1 << 5, 0) |
			if(ifnull(settings_get_bool('password_needs_special', false), false) && `$password` regexp '^[a-zA-Z0-9]+$', 1 << 6, 0) |
		0) end);
	end;^
	
	drop function if exists sessions_password_hash;^
	create function sessions_password_hash(
		$password varchar(256)
	) returns varchar(256) begin 
		return substring((case settings_get('password_hash_function', false) 
			when 'md5' then md5($password)
			when 'sha' then sha($password)
			when 'sha1' then sha1($password)
			when 'sha2' then sha2($password, ifnull(settings_get_int('hash_sha2_length', false), 512))
			when 'password' then password($password)
			when 'old_password' then old_password($password)
			when 'des' then des_encrypt($password)
			when 'aes' then aes_encrypt($password, sha2($password, ifnull(settings_get_int('hash_sha2_length', false), 512)))
			when 'encrypt' then encrypt($password)
		else $password end), 1, 256);
	end;^

	drop procedure if exists users_groups_create;^
	create procedure users_groups_create(
		in $procedure_id integer, 
		in $session_id integer, 
		in $nick varchar(32), 
		in $id integer 
	) begin 

		declare $group_id integer;

		insert into Groups values();
		set $group_id := last_insert_id();
		insert into GroupsData(`procedure`, `session`, entity, name, `default`, is_entity, description) values 
			($procedure_id, $session_id, $group_id, $nick, false, true, concat('Group for entity user "', $id, '".'));
			
		insert into UsersGroups values();
		insert into UsersGroupsData(`procedure`, `session`, entity, `user`, `group`, belongs) values 
			($procedure_id, $session_id, last_insert_id(), $id, $group_id, true);

		default_groups:begin
			
			declare $done bool default false;
			declare `$group` integer;
			declare default_groups_cursor cursor for select entity from _Groups where deleted is null && `default`;
			declare continue handler for not found set $done := true;
		
			open default_groups_cursor;
			default_groups_reader:loop 
				
				fetch default_groups_cursor into `$group`;
				if $done then 
					leave default_groups_reader;
				end if;
			
				insert into UsersGroups values();
				insert into UsersGroupsData(`procedure`, `session`, entity, `user`, `group`, belongs) values 
					($procedure_id, $session_id, last_insert_id(), $id, `$group`, true);
			
			end loop default_groups_reader;
			close default_groups_cursor;
		
		end default_groups;

	end;^

	drop function if exists users_get_entity_group;^
	create function users_get_entity_group(
		$entity integer 
	) returns integer begin 
		return (
			select groups.entity 
			from _UsersGroups users_groups 
			join _Groups groups on users_groups.`group` = groups.entity 
			join _Users users on users_groups.`user` = users.entity 
			where 
				users.entity = $entity && 
				users_groups.belongs && 
				groups.is_entity && 
				users.nick = groups.name 
			order by users_groups.entity asc 
			limit 1
		);
	end;^

	drop procedure if exists users_update_group_data;^
	create procedure users_update_group_data(
		in $procedure_id integer, 
		in $session_id integer, 
		in $group_id integer, 
		in $nick varchar(32)
	) begin 
		
		declare $group_name varchar(64);
		declare $group_default bool;
		declare $group_is_entity bool;
		declare $group_description varchar(512);

		select name, `default`, is_entity, description 
		into $group_name, $group_default, $group_is_entity, $group_description
		from _Groups where entity = $group_id limit 1;

		if $group_name != $nick then 
			insert into GroupsData(`procedure`, `session`, entity, name, `default`, is_entity, description) values 
				($procedure_id, $session_id, $group_id, $nick, $group_default, $group_is_entity, $group_description);
		end if;

	end;^
	
	drop function if exists create_random_chain;^
	create function create_random_chain(
		`$length` smallint, 
		$alphabet varchar(256)
	) returns varchar(256) begin 
		
		declare `$chain` varchar(256) default '';
		declare $l integer;
	
		if `$length` is null then 
			set `$length` := settings_get_int('random_chain_length', false);
		end if;
		if $alphabet is null then 
			set $alphabet := settings_get('random_chain_alphabet', false);
		end if;
	
		set $l := length($alphabet);
	
		while length(`$chain`) < `$length` do
			set `$chain` := concat(`$chain`, substring($alphabet, floor(rand() * $l) + 1, 1));
		end while;
	
		return `$chain`;
	end;^
	
	drop function if exists sessions_single_validate;^
	create function sessions_single_validate(
		$id integer
	) returns integer begin 
		return (case 
			when $id is null then 1 << 0 
			when $id < 1 then 1 << 1 
		else ifnull((
			select if(date_out is null, 0, 1 << 3) from Sessions where id = $id limit 1
		), 1 << 2) end);
	end;^
	
delimiter ;