create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists users_tables_delete;^
	create procedure users_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _Users;
		drop table if exists UsersData;
		drop table if exists Users;
		
	end;^

	drop procedure if exists users_tables_create;^
	create procedure users_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists Users(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_id primary key(id)
		);
	
		create table if not exists UsersData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			nick varchar(32) not null,
			`password` varchar(256) not null,
			`accessible` boolean not null default false,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_data_id primary key(id),
			constraint users_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint users_data_session foreign key(`session`) references Sessions(id),
			constraint users_data_entity foreign key(entity) references Users(id)
		);
	
		create table if not exists _Users(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			nick varchar(32) not null,
			`password` varchar(256) not null,
			`accessible` boolean not null default false,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _users_id primary key(id),
			constraint _users_procedure foreign key(`procedure`) references Procedures(id),
			constraint _users_session foreign key(`session`) references Sessions(id),
			constraint _users_entity foreign key(entity) references Users(id)
		);
		
	end;^

	drop procedure if exists users_tables_update;^
	create procedure users_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'users_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersData' && column_name = 'nick' limit 1) is null then begin 
			alter table UsersData add column nick varchar(32) not null after entity;
			alter table _Users add column nick varchar(32) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersData' && column_name = 'password' limit 1) is null then begin 
			alter table UsersData add column `password` varchar(256) not null after nick;
			alter table _Users add column `password` varchar(256) not null after nick;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersData' && column_name = 'accessible' limit 1) is null then begin 
			alter table UsersData add column `accessible` boolean not null default false after `password`;
			alter table _Users add column `accessible` boolean not null default false after `password`;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call users_tables_delete(null);^
	call users_tables_create(null);^
	call users_tables_update(null);^
	
delimiter ;