create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists users_insert_validate;^
	create function users_insert_validate(
		$nick varchar(32),
		`$password` varchar(256),
		`$accessible` boolean
	) returns varchar(512) begin 
		
		declare $error varchar(512) default '';

		set $error := errors_set($error, (case 
			when $nick is null then 1 << 0
			when $nick = '' then 1 << 1 
		else (
			ifnull((select 1 << 2 from _Users where deleted is null && nick = $nick limit 1), 0) | 
			ifnull((select 1 << 3 from _Groups where deleted is null && name = $nick limit 1), 0) | 
		0) end), 0, 0);

		set $error := errors_set($error, (password_validate(`$password`) >> 2) << 2, 4, 0);

		set $error := errors_set($error, (case 
			when `$accessible` is null then 1 << 0
		else 0 end), 11, 0);
	
		return $error;
	end;^
	
	drop procedure if exists users_add;^
	create procedure users_add(
		in `$session` text,
		in $nick varchar(32),
		in `$password` varchar(256),
		in `$accessible` boolean,
		out $error varchar(512),
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'users_add';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}',
			'$.nick', $nick,
			-- '$.password', `$password`,
			'$.accessible', `$accessible`,
			'$.entity', 0, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		set $id := 0;
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_join($error, users_insert_validate($nick, `$password`, `$accessible`), 30, 0);
	
		if !errors_has($error) then begin 

			declare $group_id integer;
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		
			insert into Users values();
			set $id := last_insert_id();
			insert into UsersData(`procedure`, `session`, entity, nick, `password`, `accessible`) values
				($procedure_id, $session_id, $id, $nick, if(ifnull(`$password`, '') = '', '', sessions_password_hash(`$password`)), `$accessible`);

			call users_groups_create($procedure_id, $session_id, $nick, $id);
			
			set $parameters := json_set($parameters, 
				'$.entity', $id,
				'$.id', last_insert_id()
			);
			
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^
	
delimiter ;