create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists users_edit_validate;^
	create function users_edit_validate(
		$entity integer,
		$nick varchar(32),
		`$password` varchar(256),
		`$accessible` boolean
	) returns varchar(512) begin 
		
		declare $error varchar(512) default errors_set('', users_validate($entity, true), 0, 0);

		set $error := errors_set($error, (case 
			when $nick is null then 1 << 0
			when $nick = '' then 1 << 1 
		else 0 end), 4, 0);

		set $error := errors_set($error, (password_validate(`$password`) >> 2) << 2, 6, 0);

		set $error := errors_set($error, (case 
			when `$accessible` is null then 1 << 0
		else 0 end), 13, 0);

		if !errors_has($error) then 
			set $error := errors_set($error, (
				if((select 1 from _Users where entity = $entity && nick != $nick limit 1) is null, 0, (
					ifnull((select 1 << 0 from _Users where deleted is null && entity != $entity && nick = $nick limit 1), 0) | 
					ifnull((select 1 << 1 from _Groups where deleted is null && name = $nick limit 1), 0) | 
				0)) |
				ifnull((
					select 1 << 2 from _Users where 
						entity = $entity && 
						nick = $nick && 
						`password` = if(ifnull(`$password`, '') = '', '', sessions_password_hash(`$password`)) && 
						`accessible` = `$accessible` 
					limit 1
				), 0) | 
			0), 14, 0);
		end if;
	
		return $error;
	end;^
	
	drop procedure if exists users_edit;^
	create procedure users_edit(
		in `$session` text,
		in $entity integer,
		in $nick varchar(32),
		in `$password` varchar(256),
		in `$accessible` boolean,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'users_edit';
		declare $procedure_id integer;
		declare $id integer;
		declare $parameters text default json_set('{}',
			'$.nick', $nick,
			-- '$.password', `$password`,
			'$.accessible', `$accessible`,
			'$.entity', $entity, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_join($error, users_edit_validate($entity, $nick, `$password`, `$accessible`), 30, 0);
	
		if !errors_has($error) then begin 

			declare $group_id integer default users_get_entity_group($entity);
		
			call procedures_get(`$database`, `$procedure`, $procedure_id);

			if ifnull(`$password`, '') = '' then 
				set `$password` := (select `password` from _Users where entity = $entity limit 1);
			end if;
		
			insert into UsersData(`procedure`, `session`, entity, nick, `password`, `accessible`) values 
				($procedure_id, $session_id, $entity, $nick, `$password`, `$accessible`);
			set $id := last_insert_id();

			if $group_id is not null then 
				call users_update_group_data($procedure_id, $session_id, $group_id, $nick);
			end if;
			
			set $parameters := json_set($parameters, 
				'$.id', $id
			);
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

delimiter ;