create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists users_state_validate;^
	create function users_state_validate(
		$id integer
	) returns integer begin 
		return (case 
			when $id is null then 1 << 0
			when $id < 1 then 1 << 1
		else ifnull((
			select ifnull((
				select 1 << 2 from _Users where `data` = $id limit 1
			), 0) from UsersData where date_out is null && id = $id limit 1
		), 1 << 3) end);
	end;^
	
	drop procedure if exists users_restore;^
	create procedure users_restore(
		in `$session` text,
		in $id integer,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'users_restore';
		declare $procedure_id integer;
		declare $old_id integer;
		declare $parameters text default json_set('{}', 
			'$.old', 0,
			'$.new', $id,
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_set($error, users_state_validate($id), 30, 0);
	
		if !errors_has($error) then begin 

			declare $entity integer;
			declare $nick varchar(32);
			declare $group_id varchar(64);

			select entity, nick into $entity, $nick from UsersData where id = $id limit 1;
			set $group_id := users_get_entity_group($entity);

			set $error := errors_set($error, ifnull((select 1 << 0 from _Groups where 
				deleted is null && 
				($group_id is null || entity != $group_id) && 
				name = $nick 
			limit 1), 0), 34, 0);

			if !errors_has($error) then begin 
				
				call procedures_get(`$database`, `$procedure`, $procedure_id);
				set $old_id := (select `data` from _Users where entity = (
					select entity from UsersData where id = $id limit 1
				));
			
				insert into UsersData(`procedure`, `session`, entity, nick, `password`, `accessible`, deleted) 
					select $procedure_id, $session_id, entity, nick, `password`, `accessible`, deleted from UsersData where id = $id limit 1;
				
				if $group_id is not null then 
					call users_update_group_data($procedure_id, $session_id, $group_id, $nick);
				end if;
				
				set $parameters := json_set($parameters, 
					'$.old', $old_id, 
					'$.id', last_insert_id() 
				);

			end;end if;
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

delimiter ;