create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists groups_tables_delete;^
	create procedure groups_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _Groups;
		drop table if exists GroupsData;
		drop table if exists Groups;
		
	end;^

	drop procedure if exists groups_tables_create;^
	create procedure groups_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists Groups(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint groups_id primary key(id)
		);
	
		create table if not exists GroupsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			name varchar(64) not null,
			`default` boolean not null default false,
			is_entity boolean not null default false,
			description varchar(512),
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint groups_data_id primary key(id),
			constraint groups_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint groups_data_session foreign key(`session`) references Sessions(id),
			constraint groups_data_entity foreign key(entity) references Groups(id)
		);
	
		create table if not exists _Groups(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			name varchar(64) not null,
			`default` boolean not null default false,
			is_entity boolean not null default false,
			description varchar(512),
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _groups_id primary key(id),
			constraint _groups_procedure foreign key(`procedure`) references Procedures(id),
			constraint _groups_session foreign key(`session`) references Sessions(id),
			constraint _groups_entity foreign key(entity) references Groups(id)
		);
		
	end;^

	drop procedure if exists groups_tables_update;^
	create procedure groups_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'groups_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'GroupsData' && column_name = 'name' limit 1) is null then begin 
			alter table GroupsData add column name varchar(64) not null after entity;
			alter table _Groups add column name varchar(64) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'GroupsData' && column_name = 'default' limit 1) is null then begin 
			alter table GroupsData add column `default` boolean not null default false after name;
			alter table _Groups add column `default` boolean not null default false after name;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'GroupsData' && column_name = 'is_entity' limit 1) is null then begin 
			alter table GroupsData add column is_entity boolean not null default false after `default`;
			alter table _Groups add column is_entity boolean not null default false after `default`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'GroupsData' && column_name = 'description' limit 1) is null then begin 
			alter table GroupsData add column description varchar(512) after is_entity;
			alter table _Groups add column description varchar(512) after is_entity;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call groups_tables_delete(null);^
	call groups_tables_create(null);^
	call groups_tables_update(null);^
	
delimiter ;