create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists groups_data_update;^
	create procedure groups_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from Groups where id = $entity && date_out is null limit 1) is null || 
			(select 1 from GroupsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _Groups where entity = $entity;
		else begin
			if (select 1 from _Groups where entity = $entity limit 1) is null then 
				insert into _Groups(`procedure`, `session`, entity, `data`, name, `default`, is_entity, description, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, name, `default`, is_entity, description, deleted, date_in, (select date_in from Groups where id = $entity) from GroupsData where id = (
						select max(id) from GroupsData where entity = $entity && date_out is null 
					);
			else 
				update _Groups plain 
				join (select `procedure`, `session`, entity, id, name, `default`, is_entity, description, deleted, date_in, (select date_in from Groups where id = $entity) from GroupsData where id = (
					select max(id) from GroupsData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.name := `data`.name,
					plain.`default` := `data`.`default`,
					plain.is_entity := `data`.is_entity,
					plain.description := `data`.description,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists groups_full_update;^
	create procedure groups_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in $name varchar(64),
		in `$default` boolean,
		in $is_entity boolean,
		in $description varchar(512),
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from Groups where id = $entity && date_out is null limit 1) is null then 
			delete from _Groups where entity = $entity;
		else begin 
			if (select 1 from _Groups where entity = $entity limit 1) then 
				update _Groups set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					name := $name,
					`default` := `$default`,
					is_entity := $is_entity,
					description := $description,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _Groups(`procedure`, `session`, entity, `data`, name, `default`, is_entity, description, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, $name, `$default`, $is_entity, $description, $deleted, $date_in, (select date_in from Groups where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists groups_trigger_update_entity;^
	create trigger groups_trigger_update_entity after update on Groups for each row begin 
		call groups_data_update(new.id);
	end;^
	
	drop trigger if exists groups_trigger_insert;^
	create trigger groups_trigger_insert after insert on GroupsData for each row begin 
		if new.date_out is null then 
			call groups_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.name, new.`default`, new.is_entity, new.description, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists groups_trigger_update;^
	create trigger groups_trigger_update after update on GroupsData for each row begin 
		if new.entity != old.entity then 
			call groups_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call groups_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.name, new.`default`, new.is_entity, new.description, new.deleted, new.date_in);
		else 
			call groups_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists groups_trigger_delete;^
	create trigger groups_trigger_delete after delete on GroupsData for each row begin 
		call groups_data_update(old.entity);
	end;^
	
delimiter ;