create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists groups_insert_validate;^
	create function groups_insert_validate(
		$name varchar(64),
		`$default` boolean,
		$is_entity boolean,
		$description varchar(512)
	) returns varchar(512) begin 
		
		declare $error varchar(512) default '';

		set $error := errors_set($error, (case 
			when $name is null then 1 << 0
		else 0 end), 0, 0);
		set $error := errors_set($error, (case 
			when `$default` is null then 1 << 0
		else 0 end), 1, 0);
		set $error := errors_set($error, (case 
			when $is_entity is null then 1 << 0
		else 0 end), 2, 0);
	
		return $error;
	end;^
	
	drop procedure if exists groups_add;^
	create procedure groups_add(
		in `$session` text,
		in $name varchar(64),
		in `$default` boolean,
		in $is_entity boolean,
		in $description varchar(512),
		out $error varchar(512),
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'groups_add';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}',
			'$.name', $name,
			'$.default', `$default`,
			'$.is_entity', $is_entity,
			'$.description', $description,
			'$.entity', 0, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		set $id := 0;
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_join($error, groups_insert_validate($name, `$default`, $is_entity, $description), 30, 0);
	
		if !errors_has($error) then begin
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		
			insert into Groups values();
			set $id := last_insert_id();
			insert into GroupsData(`procedure`, `session`, entity, name, `default`, is_entity, description) values
				($procedure_id, $session_id, $id, $name, `$default`, $is_entity, $description);
			
			set $parameters := json_set($parameters, 
				'$.entity', $id,
				'$.id', last_insert_id()
			);
			
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^
	
delimiter ;