create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists ips_tables_delete;^
	create procedure ips_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _Ips;
		drop table if exists IpsData;
		drop table if exists Ips;
		
	end;^

	drop procedure if exists ips_tables_create;^
	create procedure ips_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists Ips(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint ips_id primary key(id)
		);
	
		create table if not exists IpsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			address varchar(39) not null,
			`accessible` boolean not null default true,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint ips_data_id primary key(id),
			constraint ips_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint ips_data_session foreign key(`session`) references Sessions(id),
			constraint ips_data_entity foreign key(entity) references Ips(id)
		);
	
		create table if not exists _Ips(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			address varchar(39) not null,
			`accessible` boolean not null default true,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _ips_id primary key(id),
			constraint _ips_procedure foreign key(`procedure`) references Procedures(id),
			constraint _ips_session foreign key(`session`) references Sessions(id),
			constraint _ips_entity foreign key(entity) references Ips(id)
		);
		
	end;^

	drop procedure if exists ips_tables_update;^
	create procedure ips_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'ips_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsData' && column_name = 'address' limit 1) is null then begin 
			alter table IpsData add column address varchar(39) not null after entity;
			alter table _Ips add column address varchar(39) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsData' && column_name = 'accessible' limit 1) is null then begin 
			alter table IpsData add column `accessible` boolean not null default true after address;
			alter table _Ips add column `accessible` boolean not null default true after address;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call ips_tables_delete(null);^
	call ips_tables_create(null);^
	call ips_tables_update(null);^
	
delimiter ;