create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop view if exists IpsDataView;^
	create view IpsDataView as select 
		`data`.*,
		entities.date_in as date_created
	from (select * from IpsData where id in (
		select max(id) from IpsData where date_out is null group by entity
	)) `data`
	join Ips entities on `data`.entity = entities.id 
	where entities.date_out is null;^
	
	drop procedure if exists ips_plain_reset;^
	create procedure ips_plain_reset(
		in `$session` text
	) begin 

		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'ips_plain_reset';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
	
		if $session_id is null then 
			set $session_id := sessions_get_machine();
		end if; 

		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		start transaction;

		delete from _Ips where true;
		alter table _Ips auto_increment = 1;
		insert into _Ips(`procedure`, `session`, entity, `data`, address, `accessible`, deleted, date_in, date_created) 
			select `procedure`, `session`, entity, id, address, `accessible`, deleted, date_in, date_created from IpsDataView;

		commit;

		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 1 << 1);

	end;^
	
	drop procedure if exists ips_plain_update;^
	create procedure ips_plain_update(
		in `$session` text
	) begin 

		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'ips_plain_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
	
		if $session_id is null then 
			set $session_id := sessions_get_machine();
		end if; 

		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		start transaction;

		delete from _Ips where entity not in (select entity from IpsDataView);
		update _Ips plain 
		join IpsDataView `data` on plain.entity = `data`.entity
		set 
			plain.`procedure` := `data`.`procedure`,
			plain.`session` := `data`.`session`,
			plain.`data` := `data`.id,
			plain.address := `data`.address,
			plain.`accessible` := `data`.`accessible`,
			plain.deleted := `data`.deleted,
			plain.date_in := `data`.date_in, 
			plain.date_created := `data`.date_created;
		insert into _Ips(`procedure`, `session`, entity, `data`, address, `accessible`, deleted, date_in, date_created) 
			select `procedure`, `session`, entity, id, address, `accessible`, deleted, date_in, date_created from IpsDataView where entity not in (
				select entity from _Ips
			);

		commit;

		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 1 << 1);

	end;^
	
delimiter ;