create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists permissions_history;^
	create procedure permissions_history(
		in `$session` text,
		in $entity integer,
		in $page integer,
		in $items_per_page integer,
		in `$where` text,
		in $order_by text,
		out $items integer,
		out $pages integer,
		out $results integer,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'permissions_history';
		declare $has_errors boolean;
		declare $parameters text default json_set('{}', 
			'$.entity', $entity,
			'$.page', $page,
			'$.items_per_page', $items_per_page,
			'$.where', `$where`,
			'$.order_by', $order_by
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
		
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_set($error, permissions_validate($entity, true), 30, 0);
		set $has_errors := errors_has($error);
	
		set `$where` := where_set(concat('id=', $entity), `$where`);
		call tables_list(concat(`$database`, '.PermissionsHistoryView'), $page, $items_per_page, `$where`, $order_by, $has_errors, $pages, $items, $results);

		select * from PermissionsView where !$has_errors && id = $entity limit 1;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 1);
		
	end;^

delimiter ;