create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists settings_types_tables_delete;^
	create procedure settings_types_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _SettingsTypes;
		drop table if exists SettingsTypesData;
		drop table if exists SettingsTypes;
		
	end;^

	drop procedure if exists settings_types_tables_create;^
	create procedure settings_types_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists SettingsTypes(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_types_id primary key(id)
		);
	
		create table if not exists SettingsTypesData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			name varchar(64) not null,
			description varchar(512),
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_types_data_id primary key(id),
			constraint settings_types_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint settings_types_data_session foreign key(`session`) references Sessions(id),
			constraint settings_types_data_entity foreign key(entity) references SettingsTypes(id)
		);
	
		create table if not exists _SettingsTypes(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			name varchar(64) not null,
			description varchar(512),
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _settings_types_id primary key(id),
			constraint _settings_types_procedure foreign key(`procedure`) references Procedures(id),
			constraint _settings_types_session foreign key(`session`) references Sessions(id),
			constraint _settings_types_entity foreign key(entity) references SettingsTypes(id)
		);
		
	end;^

	drop procedure if exists settings_types_tables_update;^
	create procedure settings_types_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_types_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsTypesData' && column_name = 'name' limit 1) is null then begin 
			alter table SettingsTypesData add column name varchar(64) not null after entity;
			alter table _SettingsTypes add column name varchar(64) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsTypesData' && column_name = 'description' limit 1) is null then begin 
			alter table SettingsTypesData add column description varchar(512) after name;
			alter table _SettingsTypes add column description varchar(512) after name;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call settings_types_tables_delete(null);^
	call settings_types_tables_create(null);^
	call settings_types_tables_update(null);^
	
delimiter ;