create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists settings_types_data_update;^
	create procedure settings_types_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from SettingsTypes where id = $entity && date_out is null limit 1) is null || 
			(select 1 from SettingsTypesData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _SettingsTypes where entity = $entity;
		else begin
			if (select 1 from _SettingsTypes where entity = $entity limit 1) is null then 
				insert into _SettingsTypes(`procedure`, `session`, entity, `data`, name, description, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, name, description, deleted, date_in, (select date_in from SettingsTypes where id = $entity) from SettingsTypesData where id = (
						select max(id) from SettingsTypesData where entity = $entity && date_out is null 
					);
			else 
				update _SettingsTypes plain 
				join (select `procedure`, `session`, entity, id, name, description, deleted, date_in, (select date_in from SettingsTypes where id = $entity) from SettingsTypesData where id = (
					select max(id) from SettingsTypesData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.name := `data`.name,
					plain.description := `data`.description,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists settings_types_full_update;^
	create procedure settings_types_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in $name varchar(64),
		in $description varchar(512),
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from SettingsTypes where id = $entity && date_out is null limit 1) is null then 
			delete from _SettingsTypes where entity = $entity;
		else begin 
			if (select 1 from _SettingsTypes where entity = $entity limit 1) then 
				update _SettingsTypes set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					name := $name,
					description := $description,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _SettingsTypes(`procedure`, `session`, entity, `data`, name, description, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, $name, $description, $deleted, $date_in, (select date_in from SettingsTypes where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists settings_types_trigger_update_entity;^
	create trigger settings_types_trigger_update_entity after update on SettingsTypes for each row begin 
		call settings_types_data_update(new.id);
	end;^
	
	drop trigger if exists settings_types_trigger_insert;^
	create trigger settings_types_trigger_insert after insert on SettingsTypesData for each row begin 
		if new.date_out is null then 
			call settings_types_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.name, new.description, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists settings_types_trigger_update;^
	create trigger settings_types_trigger_update after update on SettingsTypesData for each row begin 
		if new.entity != old.entity then 
			call settings_types_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call settings_types_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.name, new.description, new.deleted, new.date_in);
		else 
			call settings_types_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists settings_types_trigger_delete;^
	create trigger settings_types_trigger_delete after delete on SettingsTypesData for each row begin 
		call settings_types_data_update(old.entity);
	end;^
	
delimiter ;