create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists users_groups_tables_delete;^
	create procedure users_groups_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _UsersGroups;
		drop table if exists UsersGroupsData;
		drop table if exists UsersGroups;
		
	end;^

	drop procedure if exists users_groups_tables_create;^
	create procedure users_groups_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists UsersGroups(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_groups_id primary key(id)
		);
	
		create table if not exists UsersGroupsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			`group` integer not null,
			`user` integer not null,
			belongs boolean not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint users_groups_data_id primary key(id),
			constraint users_groups_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint users_groups_data_session foreign key(`session`) references Sessions(id),
			constraint users_groups_data_entity foreign key(entity) references UsersGroups(id),
			constraint users_groups_data_group foreign key(`group`) references Groups(id),
			constraint users_groups_data_user foreign key(`user`) references Users(id)
		);
	
		create table if not exists _UsersGroups(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			`group` integer not null,
			`user` integer not null,
			belongs boolean not null,
			date_in datetime not null,
			date_created datetime not null,
			constraint _users_groups_id primary key(id),
			constraint _users_groups_procedure foreign key(`procedure`) references Procedures(id),
			constraint _users_groups_session foreign key(`session`) references Sessions(id),
			constraint _users_groups_entity foreign key(entity) references UsersGroups(id),
			constraint _users_groups_group foreign key(`group`) references Groups(id),
			constraint _users_groups_user foreign key(`user`) references Users(id)
		);
		
	end;^

	drop procedure if exists users_groups_tables_update;^
	create procedure users_groups_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'users_groups_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersGroupsData' && column_name = 'group' limit 1) is null then begin 
			alter table UsersGroupsData add column `group` integer not null after entity;
			alter table UsersGroupsData add constraint users_groups_data_group foreign key(`group`) references Groups(id);
			alter table _UsersGroups add column `group` integer not null after `data`;
			alter table _UsersGroups add constraint _users_groups_group foreign key(`group`) references Groups(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersGroupsData' && column_name = 'user' limit 1) is null then begin 
			alter table UsersGroupsData add column `user` integer not null after `group`;
			alter table UsersGroupsData add constraint users_groups_data_user foreign key(`user`) references Users(id);
			alter table _UsersGroups add column `user` integer not null after `group`;
			alter table _UsersGroups add constraint _users_groups_user foreign key(`user`) references Users(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'UsersGroupsData' && column_name = 'belongs' limit 1) is null then begin 
			alter table UsersGroupsData add column belongs boolean not null after `user`;
			alter table _UsersGroups add column belongs boolean not null after `user`;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call users_groups_tables_delete(null);^
	call users_groups_tables_create(null);^
	call users_groups_tables_update(null);^
	
delimiter ;