create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists ips_groups_tables_delete;^
	create procedure ips_groups_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _IpsGroups;
		drop table if exists IpsGroupsData;
		drop table if exists IpsGroups;
		
	end;^

	drop procedure if exists ips_groups_tables_create;^
	create procedure ips_groups_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists IpsGroups(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint ips_groups_id primary key(id)
		);
	
		create table if not exists IpsGroupsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			`group` integer not null,
			ip integer not null,
			belongs boolean not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint ips_groups_data_id primary key(id),
			constraint ips_groups_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint ips_groups_data_session foreign key(`session`) references Sessions(id),
			constraint ips_groups_data_entity foreign key(entity) references IpsGroups(id),
			constraint ips_groups_data_group foreign key(`group`) references Groups(id),
			constraint ips_groups_data_ip foreign key(ip) references Ips(id)
		);
	
		create table if not exists _IpsGroups(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			`group` integer not null,
			ip integer not null,
			belongs boolean not null,
			date_in datetime not null,
			date_created datetime not null,
			constraint _ips_groups_id primary key(id),
			constraint _ips_groups_procedure foreign key(`procedure`) references Procedures(id),
			constraint _ips_groups_session foreign key(`session`) references Sessions(id),
			constraint _ips_groups_entity foreign key(entity) references IpsGroups(id),
			constraint _ips_groups_group foreign key(`group`) references Groups(id),
			constraint _ips_groups_ip foreign key(ip) references Ips(id)
		);
		
	end;^

	drop procedure if exists ips_groups_tables_update;^
	create procedure ips_groups_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'ips_groups_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsGroupsData' && column_name = 'group' limit 1) is null then begin 
			alter table IpsGroupsData add column `group` integer not null after entity;
			alter table IpsGroupsData add constraint ips_groups_data_group foreign key(`group`) references Groups(id);
			alter table _IpsGroups add column `group` integer not null after `data`;
			alter table _IpsGroups add constraint _ips_groups_group foreign key(`group`) references Groups(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsGroupsData' && column_name = 'ip' limit 1) is null then begin 
			alter table IpsGroupsData add column ip integer not null after `group`;
			alter table IpsGroupsData add constraint ips_groups_data_ip foreign key(ip) references Ips(id);
			alter table _IpsGroups add column ip integer not null after `group`;
			alter table _IpsGroups add constraint _ips_groups_ip foreign key(ip) references Ips(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'IpsGroupsData' && column_name = 'belongs' limit 1) is null then begin 
			alter table IpsGroupsData add column belongs boolean not null after ip;
			alter table _IpsGroups add column belongs boolean not null after ip;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call ips_groups_tables_delete(null);^
	call ips_groups_tables_create(null);^
	call ips_groups_tables_update(null);^
	
delimiter ;