create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists permissions_groups_tables_delete;^
	create procedure permissions_groups_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _PermissionsGroups;
		drop table if exists PermissionsGroupsData;
		drop table if exists PermissionsGroups;
		
	end;^

	drop procedure if exists permissions_groups_tables_create;^
	create procedure permissions_groups_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists PermissionsGroups(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint permissions_groups_id primary key(id)
		);
	
		create table if not exists PermissionsGroupsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			`group` integer not null,
			permission integer not null,
			belongs boolean not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint permissions_groups_data_id primary key(id),
			constraint permissions_groups_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint permissions_groups_data_session foreign key(`session`) references Sessions(id),
			constraint permissions_groups_data_entity foreign key(entity) references PermissionsGroups(id),
			constraint permissions_groups_data_group foreign key(`group`) references Groups(id),
			constraint permissions_groups_data_permission foreign key(permission) references Permissions(id)
		);
	
		create table if not exists _PermissionsGroups(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			`group` integer not null,
			permission integer not null,
			belongs boolean not null,
			date_in datetime not null,
			date_created datetime not null,
			constraint _permissions_groups_id primary key(id),
			constraint _permissions_groups_procedure foreign key(`procedure`) references Procedures(id),
			constraint _permissions_groups_session foreign key(`session`) references Sessions(id),
			constraint _permissions_groups_entity foreign key(entity) references PermissionsGroups(id),
			constraint _permissions_groups_group foreign key(`group`) references Groups(id),
			constraint _permissions_groups_permission foreign key(permission) references Permissions(id)
		);
		
	end;^

	drop procedure if exists permissions_groups_tables_update;^
	create procedure permissions_groups_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'permissions_groups_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'PermissionsGroupsData' && column_name = 'group' limit 1) is null then begin 
			alter table PermissionsGroupsData add column `group` integer not null after entity;
			alter table PermissionsGroupsData add constraint permissions_groups_data_group foreign key(`group`) references Groups(id);
			alter table _PermissionsGroups add column `group` integer not null after `data`;
			alter table _PermissionsGroups add constraint _permissions_groups_group foreign key(`group`) references Groups(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'PermissionsGroupsData' && column_name = 'permission' limit 1) is null then begin 
			alter table PermissionsGroupsData add column permission integer not null after `group`;
			alter table PermissionsGroupsData add constraint permissions_groups_data_permission foreign key(permission) references Permissions(id);
			alter table _PermissionsGroups add column permission integer not null after `group`;
			alter table _PermissionsGroups add constraint _permissions_groups_permission foreign key(permission) references Permissions(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'PermissionsGroupsData' && column_name = 'belongs' limit 1) is null then begin 
			alter table PermissionsGroupsData add column belongs boolean not null after permission;
			alter table _PermissionsGroups add column belongs boolean not null after permission;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call permissions_groups_tables_delete(null);^
	call permissions_groups_tables_create(null);^
	call permissions_groups_tables_update(null);^
	
delimiter ;