create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists permissions_groups_data_update;^
	create procedure permissions_groups_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from PermissionsGroups where id = $entity && date_out is null limit 1) is null || 
			(select 1 from PermissionsGroupsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _PermissionsGroups where entity = $entity;
		else begin
			if (select 1 from _PermissionsGroups where entity = $entity limit 1) is null then 
				insert into _PermissionsGroups(`procedure`, `session`, entity, `data`, `group`, permission, belongs, date_in, date_created)
					select `procedure`, `session`, entity, id, `group`, permission, belongs, date_in, (select date_in from PermissionsGroups where id = $entity) from PermissionsGroupsData where id = (
						select max(id) from PermissionsGroupsData where entity = $entity && date_out is null 
					);
			else 
				update _PermissionsGroups plain 
				join (select `procedure`, `session`, entity, id, `group`, permission, belongs, date_in, (select date_in from PermissionsGroups where id = $entity) from PermissionsGroupsData where id = (
					select max(id) from PermissionsGroupsData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.`group` := `data`.`group`,
					plain.permission := `data`.permission,
					plain.belongs := `data`.belongs,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists permissions_groups_full_update;^
	create procedure permissions_groups_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in `$group` integer,
		in $permission integer,
		in $belongs boolean,
		in $date_in datetime
	) begin
		
		if (select 1 from PermissionsGroups where id = $entity && date_out is null limit 1) is null then 
			delete from _PermissionsGroups where entity = $entity;
		else begin 
			if (select 1 from _PermissionsGroups where entity = $entity limit 1) then 
				update _PermissionsGroups set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					`group` := `$group`,
					permission := $permission,
					belongs := $belongs,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _PermissionsGroups(`procedure`, `session`, entity, `data`, `group`, permission, belongs, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, `$group`, $permission, $belongs, $date_in, (select date_in from PermissionsGroups where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists permissions_groups_trigger_update_entity;^
	create trigger permissions_groups_trigger_update_entity after update on PermissionsGroups for each row begin 
		call permissions_groups_data_update(new.id);
	end;^
	
	drop trigger if exists permissions_groups_trigger_insert;^
	create trigger permissions_groups_trigger_insert after insert on PermissionsGroupsData for each row begin 
		if new.date_out is null then 
			call permissions_groups_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.`group`, new.permission, new.belongs, new.date_in);
		end if;
	end;^
	
	drop trigger if exists permissions_groups_trigger_update;^
	create trigger permissions_groups_trigger_update after update on PermissionsGroupsData for each row begin 
		if new.entity != old.entity then 
			call permissions_groups_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call permissions_groups_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.`group`, new.permission, new.belongs, new.date_in);
		else 
			call permissions_groups_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists permissions_groups_trigger_delete;^
	create trigger permissions_groups_trigger_delete after delete on PermissionsGroupsData for each row begin 
		call permissions_groups_data_update(old.entity);
	end;^
	
delimiter ;