create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists settings_tables_delete;^
	create procedure settings_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _Settings;
		drop table if exists SettingsData;
		drop table if exists Settings;
		
	end;^

	drop procedure if exists settings_tables_create;^
	create procedure settings_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists Settings(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_id primary key(id)
		);
	
		create table if not exists SettingsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			`type` integer not null,
			name varchar(64) not null,
			`range` integer not null default 1,
			value text,
			description text,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_data_id primary key(id),
			constraint settings_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint settings_data_session foreign key(`session`) references Sessions(id),
			constraint settings_data_entity foreign key(entity) references Settings(id),
			constraint settings_data_type foreign key(`type`) references SettingsTypes(id)
		);
	
		create table if not exists _Settings(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			`type` integer not null,
			name varchar(64) not null,
			`range` integer not null default 1,
			value text,
			description text,
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _settings_id primary key(id),
			constraint _settings_procedure foreign key(`procedure`) references Procedures(id),
			constraint _settings_session foreign key(`session`) references Sessions(id),
			constraint _settings_entity foreign key(entity) references Settings(id),
			constraint _settings_type foreign key(`type`) references SettingsTypes(id)
		);
		
	end;^

	drop procedure if exists settings_tables_update;^
	create procedure settings_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'type' limit 1) is null then begin 
			alter table SettingsData add column `type` integer not null after entity;
			alter table SettingsData add constraint settings_data_type foreign key(`type`) references SettingsTypes(id);
			alter table _Settings add column `type` integer not null after `data`;
			alter table _Settings add constraint _settings_type foreign key(`type`) references SettingsTypes(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'name' limit 1) is null then begin 
			alter table SettingsData add column name varchar(64) not null after `type`;
			alter table _Settings add column name varchar(64) not null after `type`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'range' limit 1) is null then begin 
			alter table SettingsData add column `range` integer not null default 1 after name;
			alter table _Settings add column `range` integer not null default 1 after name;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'value' limit 1) is null then begin 
			alter table SettingsData add column value text after `range`;
			alter table _Settings add column value text after `range`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsData' && column_name = 'description' limit 1) is null then begin 
			alter table SettingsData add column description text after value;
			alter table _Settings add column description text after value;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call settings_tables_delete(null);^
	call settings_tables_create(null);^
	call settings_tables_update(null);^
	
delimiter ;