create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists settings_data_update;^
	create procedure settings_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from Settings where id = $entity && date_out is null limit 1) is null || 
			(select 1 from SettingsData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _Settings where entity = $entity;
		else begin
			if (select 1 from _Settings where entity = $entity limit 1) is null then 
				insert into _Settings(`procedure`, `session`, entity, `data`, `type`, name, `range`, value, description, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, `type`, name, `range`, value, description, deleted, date_in, (select date_in from Settings where id = $entity) from SettingsData where id = (
						select max(id) from SettingsData where entity = $entity && date_out is null 
					);
			else 
				update _Settings plain 
				join (select `procedure`, `session`, entity, id, `type`, name, `range`, value, description, deleted, date_in, (select date_in from Settings where id = $entity) from SettingsData where id = (
					select max(id) from SettingsData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.`type` := `data`.`type`,
					plain.name := `data`.name,
					plain.`range` := `data`.`range`,
					plain.value := `data`.value,
					plain.description := `data`.description,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists settings_full_update;^
	create procedure settings_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in `$type` integer,
		in $name varchar(64),
		in `$range` integer,
		in $value text,
		in $description text,
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from Settings where id = $entity && date_out is null limit 1) is null then 
			delete from _Settings where entity = $entity;
		else begin 
			if (select 1 from _Settings where entity = $entity limit 1) then 
				update _Settings set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					`type` := `$type`,
					name := $name,
					`range` := `$range`,
					value := $value,
					description := $description,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _Settings(`procedure`, `session`, entity, `data`, `type`, name, `range`, value, description, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, `$type`, $name, `$range`, $value, $description, $deleted, $date_in, (select date_in from Settings where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists settings_trigger_update_entity;^
	create trigger settings_trigger_update_entity after update on Settings for each row begin 
		call settings_data_update(new.id);
	end;^
	
	drop trigger if exists settings_trigger_insert;^
	create trigger settings_trigger_insert after insert on SettingsData for each row begin 
		if new.date_out is null then 
			call settings_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.`type`, new.name, new.`range`, new.value, new.description, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists settings_trigger_update;^
	create trigger settings_trigger_update after update on SettingsData for each row begin 
		if new.entity != old.entity then 
			call settings_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call settings_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.`type`, new.name, new.`range`, new.value, new.description, new.deleted, new.date_in);
		else 
			call settings_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists settings_trigger_delete;^
	create trigger settings_trigger_delete after delete on SettingsData for each row begin 
		call settings_data_update(old.entity);
	end;^
	
delimiter ;