create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists settings_insert_validate;^
	create function settings_insert_validate(
		`$type` integer,
		$name varchar(64),
		`$range` integer,
		$value text,
		$description text
	) returns varchar(512) begin 
		
		declare $error varchar(512) default '';

		set $error := errors_set($error, settings_types_validate(`$type`, true), 0, 0);
		set $error := errors_set($error, (case 
			when $name is null then 1 << 0
		else 0 end), 4, 0);
		set $error := errors_set($error, (case 
			when `$range` is null then 1 << 0
		else 0 end), 5, 0);
	
		return $error;
	end;^
	
	drop procedure if exists settings_add;^
	create procedure settings_add(
		in `$session` text,
		in `$type` integer,
		in $name varchar(64),
		in `$range` integer,
		in $value text,
		in $description text,
		out $error varchar(512),
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_add';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}',
			'$.type', `$type`,
			'$.name', $name,
			'$.range', `$range`,
			'$.value', $value,
			'$.description', $description,
			'$.entity', 0, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		set $id := 0;
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_join($error, settings_insert_validate(`$type`, $name, `$range`, $value, $description), 30, 0);
	
		if !errors_has($error) then begin
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		
			insert into Settings values();
			set $id := last_insert_id();
			insert into SettingsData(`procedure`, `session`, entity, `type`, name, `range`, value, description) values
				($procedure_id, $session_id, $id, `$type`, $name, `$range`, $value, $description);
			
			set $parameters := json_set($parameters, 
				'$.entity', $id,
				'$.id', last_insert_id()
			);
			
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^
	
delimiter ;