create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists settings_edit_validate;^
	create function settings_edit_validate(
		$entity integer,
		`$type` integer,
		$name varchar(64),
		`$range` integer,
		$value text,
		$description text
	) returns varchar(512) begin 
		
		declare $error varchar(512) default errors_set('', settings_validate($entity, true), 0, 0);

		set $error := errors_set($error, settings_types_validate(`$type`, true), 4, 0);
		set $error := errors_set($error, (case 
			when $name is null then 1 << 0
		else 0 end), 8, 0);
		set $error := errors_set($error, (case 
			when `$range` is null then 1 << 0
		else 0 end), 9, 0);

		if !errors_has($error) then 
			set $error := errors_set($error, (
				ifnull((
					select 1 << 0 from _Settings where deleted is null && entity = $entity && `type` = `$type` && name = $name && `range` = `$range` && value = $value && description = $description limit 1
				), 0) | 
			0), 10, 0);
		end if;
	
		return $error;
	end;^
	
	drop procedure if exists settings_edit;^
	create procedure settings_edit(
		in `$session` text,
		in $entity integer,
		in `$type` integer,
		in $name varchar(64),
		in `$range` integer,
		in $value text,
		in $description text,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_edit';
		declare $procedure_id integer;
		declare $id integer;
		declare $parameters text default json_set('{}',
			'$.type', `$type`,
			'$.name', $name,
			'$.range', `$range`,
			'$.value', $value,
			'$.description', $description,
			'$.entity', $entity, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_join($error, settings_edit_validate($entity, `$type`, $name, `$range`, $value, $description), 30, 0);
	
		if !errors_has($error) then begin 
		
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		
			insert into SettingsData(`procedure`, `session`, entity, `type`, name, `range`, value, description) values
				($procedure_id, $session_id, $entity, `$type`, $name, `$range`, $value, $description);
			set $id := last_insert_id();
			
			set $parameters := json_set($parameters, 
				'$.id', $id
			);
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

delimiter ;