create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists settings_delete;^
	create procedure settings_delete(
		in `$session` text,
		in $entity integer,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_delete';
		declare $procedure_id integer;
		declare $data_id integer;
		declare $parameters text default json_set('{}', 
			'$.entity', $entity, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		call sessions_validate_full(`$session`, '["administrator"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_set($error, settings_validate($entity, true), 30, 0);
	
		if !errors_has($error) then begin 
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
			set $data_id := (select `data` from _Settings where entity = $entity limit 1);
		
			insert into SettingsData(`procedure`, `session`, entity, `type`, name, `range`, value, description, deleted) 
				select $procedure_id, $session_id, entity, `type`, name, `range`, value, description, now() from SettingsData where id = $data_id limit 1;
			
			set $parameters := json_set($parameters, 
				'$.id', last_insert_id() 
			);
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

delimiter ;