create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop view if exists FilesListView;^
	create view FilesListView as select * from FilesView where deleted is null;^
	
	drop procedure if exists files_list;^
	create procedure files_list(
		in `$session` text,
		in $page integer,
		in $items_per_page integer,
		in `$where` text,
		in $order_by text,
		out $items integer,
		out $pages integer,
		out $results integer,
		out $error varchar(512)
	) begin
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'files_list';
		declare $parameters text default json_set('{}',
			'$.page', $page,
			'$.items_per_page', $items_per_page, 
			'$.where', `$where`, 
			'$.order_by', $order_by
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $has_errors boolean;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
		
		call sessions_validate_full(`$session`, '["administrador"]', $error, $session_id, $ip_id, $user_agent_id);
		set $has_errors := errors_has($error);
		call tables_list(concat(`$database`, '.FilesListView'), $page, $items_per_page, `$where`, $order_by, $has_errors, $pages, $items, $results);
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 1);
		
	end;^

delimiter ;