create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists files_hashes_tables_delete;^
	create procedure files_hashes_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists FilesHashes;
		
	end;^

	drop procedure if exists files_hashes_tables_create;^
	create procedure files_hashes_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists FilesHashes(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			file integer not null,
			`algorithm` integer not null,
			`hash` varchar(256) not null,
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint files_hashes_id primary key(id),
			constraint files_hashes_procedure foreign key(`procedure`) references Procedures(id),
			constraint files_hashes_session foreign key(`session`) references Sessions(id),
			constraint files_hashes_file foreign key(file) references Files(id),
			constraint files_hashes_algorithm foreign key(`algorithm`) references Hashes(id)
		);
		
	end;^

	drop procedure if exists files_hashes_tables_update;^
	create procedure files_hashes_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'files_hashes_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'FilesHashes' && column_name = 'file' limit 1) is null then begin 
			alter table FilesHashes add column file integer not null after id;
			alter table FilesHashes add constraint files_hashes_file foreign key(file) references Files(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'FilesHashes' && column_name = 'algorithm' limit 1) is null then begin 
			alter table FilesHashes add column `algorithm` integer not null after file;
			alter table FilesHashes add constraint files_hashes_algorithm foreign key(`algorithm`) references Hashes(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'FilesHashes' && column_name = 'hash' limit 1) is null then begin 
			alter table FilesHashes add column `hash` varchar(256) not null after `algorithm`;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, 0, '{}', 1 << 1);
	
	end;^
	
	-- call files_hashes_tables_delete(null);^
	call files_hashes_tables_create(null);^
	call files_hashes_tables_update(null);^
	
delimiter ;