create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists countries_tables_delete;^
	create procedure countries_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _Countries;
		drop table if exists CountriesData;
		drop table if exists Countries;
		
	end;^

	drop procedure if exists countries_tables_create;^
	create procedure countries_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists Countries(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint countries_id primary key(id)
		);
	
		create table if not exists CountriesData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			name varchar(64) not null,
			`key` varchar(64) not null,
			code varchar(4) not null,
			flag integer,
			description varchar(512),
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint countries_data_id primary key(id),
			constraint countries_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint countries_data_session foreign key(`session`) references Sessions(id),
			constraint countries_data_entity foreign key(entity) references Countries(id),
			constraint countries_data_flag foreign key(flag) references Files(id)
		);
	
		create table if not exists _Countries(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			name varchar(64) not null,
			`key` varchar(64) not null,
			code varchar(4) not null,
			flag integer,
			description varchar(512),
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _countries_id primary key(id),
			constraint _countries_procedure foreign key(`procedure`) references Procedures(id),
			constraint _countries_session foreign key(`session`) references Sessions(id),
			constraint _countries_entity foreign key(entity) references Countries(id),
			constraint _countries_flag foreign key(flag) references Files(id)
		);
		
	end;^

	drop procedure if exists countries_tables_update;^
	create procedure countries_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'countries_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'CountriesData' && column_name = 'name' limit 1) is null then begin 
			alter table CountriesData add column name varchar(64) not null after entity;
			alter table _Countries add column name varchar(64) not null after `data`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'CountriesData' && column_name = 'key' limit 1) is null then begin 
			alter table CountriesData add column `key` varchar(64) not null after name;
			alter table _Countries add column `key` varchar(64) not null after name;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'CountriesData' && column_name = 'code' limit 1) is null then begin 
			alter table CountriesData add column code varchar(4) not null after `key`;
			alter table _Countries add column code varchar(4) not null after `key`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'CountriesData' && column_name = 'flag' limit 1) is null then begin 
			alter table CountriesData add column flag integer after code;
			alter table CountriesData add constraint countries_data_flag foreign key(flag) references Files(id);
			alter table _Countries add column flag integer after code;
			alter table _Countries add constraint _countries_flag foreign key(flag) references Files(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'CountriesData' && column_name = 'description' limit 1) is null then begin 
			alter table CountriesData add column description varchar(512) after flag;
			alter table _Countries add column description varchar(512) after flag;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call countries_tables_delete(null);^
	call countries_tables_create(null);^
	call countries_tables_update(null);^
	
delimiter ;