create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists countries_state_validate;^
	create function countries_state_validate(
		$id integer
	) returns integer begin 
		return (case 
			when $id is null then 1 << 0
			when $id < 1 then 1 << 1
			when $id in (select `data` from _Countries) then 1 << 2
		else ifnull((
			select 0 from CountriesData where date_out is null && id = $id limit 1
		), 1 << 3) end);
	end;^
	
	drop procedure if exists countries_restore;^
	create procedure countries_restore(
		in `$session` text,
		in $id integer,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'countries_restore';
		declare $procedure_id integer;
		declare $old_id integer;
		declare $parameters text default json_set('{}', 
			'$.old', 0,
			'$.new', $id,
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		call sessions_validate_full(`$session`, '["administrador"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_set($error, countries_state_validate($id), 30, 0);
	
		if !errors_has($error) then begin 
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
			set $old_id := (select `data` from _Countries where entity = (
				select entity from CountriesData where id = $id limit 1
			));
		
			insert into CountriesData(`procedure`, `session`, entity, name, `key`, code, flag, description, deleted) 
				select $procedure_id, $session_id, entity, name, `key`, code, flag, description, deleted from CountriesData where id = $id limit 1;
			
			set $parameters := json_set($parameters, 
				'$.old', $old_id, 
				'$.id', last_insert_id() 
			);
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

	drop function if exists countries_state_i18n_validate;^
	create function countries_state_i18n_validate(
		$id integer
	) returns integer begin 
		return (case 
			when $id is null then 1 << 0
			when $id < 1 then 1 << 1
			when $id in (select i18n from _CountriesI18N) then 1 << 2
		else ifnull((
			select 0 from CountriesI18N where date_out is null && id = $id limit 1
		), 1 << 3) end);
	end;^
	
	drop procedure if exists countries_i18n_restore;^
	create procedure countries_i18n_restore(
		in `$session` text,
		in $id integer,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'countries_i18n_restore';
		declare $procedure_id integer;
		declare $old_id integer;
		declare $parameters text default json_set('{}', 
			'$.old', 0,
			'$.new', $id,
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		call sessions_validate_full(`$session`, '["administrador"]', $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_set($error, countries_state_i18n_validate($id), 30, 0);
	
		if !errors_has($error) then begin 
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
			set $old_id := (select i18n from _CountriesI18N where entity = (
				select entity from CountriesI18N where id = $id limit 1
			));
		
			insert into CountriesI18N(`procedure`, `session`, entity) 
				select $procedure_id, $session_id, entity from CountriesI18N where id = $id limit 1;
			
			set $parameters := json_set($parameters, 
				'$.old', $old_id, 
				'$.id', last_insert_id() 
			);
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

delimiter ;