create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists settings_groups_tables_delete;^
	create procedure settings_groups_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _SettingsGroups;
		drop table if exists SettingsGroupsData;
		drop table if exists SettingsGroups;
		
	end;^

	drop procedure if exists settings_groups_tables_create;^
	create procedure settings_groups_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists SettingsGroups(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_groups_id primary key(id)
		);
	
		create table if not exists SettingsGroupsData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			`group` integer not null,
			settings integer not null,
			value text,
			belongs boolean not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint settings_groups_data_id primary key(id),
			constraint settings_groups_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint settings_groups_data_session foreign key(`session`) references Sessions(id),
			constraint settings_groups_data_entity foreign key(entity) references SettingsGroups(id),
			constraint settings_groups_data_group foreign key(`group`) references Groups(id),
			constraint settings_groups_data_settings foreign key(settings) references Settings(id)
		);
	
		create table if not exists _SettingsGroups(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			`group` integer not null,
			settings integer not null,
			value text,
			belongs boolean not null,
			date_in datetime not null,
			date_created datetime not null,
			constraint _settings_groups_id primary key(id),
			constraint _settings_groups_procedure foreign key(`procedure`) references Procedures(id),
			constraint _settings_groups_session foreign key(`session`) references Sessions(id),
			constraint _settings_groups_entity foreign key(entity) references SettingsGroups(id),
			constraint _settings_groups_group foreign key(`group`) references Groups(id),
			constraint _settings_groups_settings foreign key(settings) references Settings(id)
		);
		
	end;^

	drop procedure if exists settings_groups_tables_update;^
	create procedure settings_groups_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_groups_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsGroupsData' && column_name = 'group' limit 1) is null then begin 
			alter table SettingsGroupsData add column `group` integer not null after entity;
			alter table SettingsGroupsData add constraint settings_groups_data_group foreign key(`group`) references Groups(id);
			alter table _SettingsGroups add column `group` integer not null after `data`;
			alter table _SettingsGroups add constraint _settings_groups_group foreign key(`group`) references Groups(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsGroupsData' && column_name = 'settings' limit 1) is null then begin 
			alter table SettingsGroupsData add column settings integer not null after `group`;
			alter table SettingsGroupsData add constraint settings_groups_data_settings foreign key(settings) references Settings(id);
			alter table _SettingsGroups add column settings integer not null after `group`;
			alter table _SettingsGroups add constraint _settings_groups_settings foreign key(settings) references Settings(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsGroupsData' && column_name = 'value' limit 1) is null then begin 
			alter table SettingsGroupsData add column value text after settings;
			alter table _SettingsGroups add column value text after settings;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'SettingsGroupsData' && column_name = 'belongs' limit 1) is null then begin 
			alter table SettingsGroupsData add column belongs boolean not null after value;
			alter table _SettingsGroups add column belongs boolean not null after value;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call settings_groups_tables_delete(null);^
	call settings_groups_tables_create(null);^
	call settings_groups_tables_update(null);^
	
delimiter ;