create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop procedure if exists languages_tables_delete;^
	create procedure languages_tables_delete(
		in `$session` text
	) begin
		
		drop table if exists _Languages;
		drop table if exists LanguagesData;
		drop table if exists Languages;
		
	end;^

	drop procedure if exists languages_tables_create;^
	create procedure languages_tables_create(
		in `$session` text
	) begin 
		
		create table if not exists Languages(
			id integer not null auto_increment,
			date_in datetime not null default now(),
			date_out datetime,
			constraint languages_id primary key(id)
		);
	
		create table if not exists LanguagesData(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null,
			country integer not null,
			name varchar(64) not null,
			`key` varchar(64) not null,
			code varchar(4) not null,
			flag integer,
			description varchar(512),
			deleted datetime,
			date_in datetime not null default now(),
			date_out datetime,
			constraint languages_data_id primary key(id),
			constraint languages_data_procedure foreign key(`procedure`) references Procedures(id),
			constraint languages_data_session foreign key(`session`) references Sessions(id),
			constraint languages_data_entity foreign key(entity) references Languages(id),
			constraint languages_data_country foreign key(country) references Countries(id),
			constraint languages_data_flag foreign key(flag) references Files(id)
		);
	
		create table if not exists _Languages(
			id integer not null auto_increment,
			`procedure` integer not null,
			`session` integer not null,
			entity integer not null, 
			`data` integer not null,
			country integer not null,
			name varchar(64) not null,
			`key` varchar(64) not null,
			code varchar(4) not null,
			flag integer,
			description varchar(512),
			deleted datetime,
			date_in datetime not null,
			date_created datetime not null,
			constraint _languages_id primary key(id),
			constraint _languages_procedure foreign key(`procedure`) references Procedures(id),
			constraint _languages_session foreign key(`session`) references Sessions(id),
			constraint _languages_entity foreign key(entity) references Languages(id),
			constraint _languages_country foreign key(country) references Countries(id),
			constraint _languages_flag foreign key(flag) references Files(id)
		);
		
	end;^

	drop procedure if exists languages_tables_update;^
	create procedure languages_tables_update(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'languages_tables_update';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LanguagesData' && column_name = 'country' limit 1) is null then begin 
			alter table LanguagesData add column country integer not null after entity;
			alter table LanguagesData add constraint languages_data_country foreign key(country) references Countries(id);
			alter table _Languages add column country integer not null after `data`;
			alter table _Languages add constraint _languages_country foreign key(country) references Countries(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LanguagesData' && column_name = 'name' limit 1) is null then begin 
			alter table LanguagesData add column name varchar(64) not null after country;
			alter table _Languages add column name varchar(64) not null after country;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LanguagesData' && column_name = 'key' limit 1) is null then begin 
			alter table LanguagesData add column `key` varchar(64) not null after name;
			alter table _Languages add column `key` varchar(64) not null after name;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LanguagesData' && column_name = 'code' limit 1) is null then begin 
			alter table LanguagesData add column code varchar(4) not null after `key`;
			alter table _Languages add column code varchar(4) not null after `key`;
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LanguagesData' && column_name = 'flag' limit 1) is null then begin 
			alter table LanguagesData add column flag integer after code;
			alter table LanguagesData add constraint languages_data_flag foreign key(flag) references Files(id);
			alter table _Languages add column flag integer after code;
			alter table _Languages add constraint _languages_flag foreign key(flag) references Files(id);
		end;end if;

		if (select 1 from information_schema.columns where table_schema = `$database` && table_name = 'LanguagesData' && column_name = 'description' limit 1) is null then begin 
			alter table LanguagesData add column description varchar(512) after flag;
			alter table _Languages add column description varchar(512) after flag;
		end;end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, '', '{}', 2);
	
	end;^
	
	-- call languages_tables_delete(null);^
	call languages_tables_create(null);^
	call languages_tables_update(null);^
	
delimiter ;