create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop procedure if exists languages_data_update;^
	create procedure languages_data_update(
		in $entity integer
	) begin 
		
		if (
			(select 1 from Languages where id = $entity && date_out is null limit 1) is null || 
			(select 1 from LanguagesData where entity = $entity && date_out is null limit 1) is null 
		) then 
			delete from _Languages where entity = $entity;
		else begin
			if (select 1 from _Languages where entity = $entity limit 1) is null then 
				insert into _Languages(`procedure`, `session`, entity, `data`, country, name, `key`, code, flag, description, deleted, date_in, date_created)
					select `procedure`, `session`, entity, id, country, name, `key`, code, flag, description, deleted, date_in, (select date_in from Languages where id = $entity) from LanguagesData where id = (
						select max(id) from LanguagesData where entity = $entity && date_out is null 
					);
			else 
				update _Languages plain 
				join (select `procedure`, `session`, entity, id, country, name, `key`, code, flag, description, deleted, date_in, (select date_in from Languages where id = $entity) from LanguagesData where id = (
					select max(id) from LanguagesData where entity = $entity && date_out is null 
				)) `data` on plain.entity = `data`.entity
				set 
					plain.`procedure` := `data`.`procedure`,
					plain.`session` := `data`.`session`,
					plain.`data` := `data`.id,
					plain.country := `data`.country,
					plain.name := `data`.name,
					plain.`key` := `data`.`key`,
					plain.code := `data`.code,
					plain.flag := `data`.flag,
					plain.description := `data`.description,
					plain.deleted := `data`.deleted,
					plain.date_in := `data`.date_in
				where plain.entity = $entity;
			end if;
		end;end if;
	
	end;^
	
	drop procedure if exists languages_full_update;^
	create procedure languages_full_update(
		in `$procedure` integer,
		in `$session` integer,
		in $entity integer,
		in `$data` integer,
		in $country integer,
		in $name varchar(64),
		in `$key` varchar(64),
		in $code varchar(4),
		in $flag integer,
		in $description varchar(512),
		in $deleted datetime,
		in $date_in datetime
	) begin
		
		if (select 1 from Languages where id = $entity && date_out is null limit 1) is null then 
			delete from _Languages where entity = $entity;
		else begin 
			if (select 1 from _Languages where entity = $entity limit 1) then 
				update _Languages set 
					`procedure` := `$procedure`,
					`session` := `$session`,
					`data` := `$data`,
					country := $country,
					name := $name,
					`key` := `$key`,
					code := $code,
					flag := $flag,
					description := $description,
					deleted := $deleted,
					date_in := $date_in
				where entity = $entity;
			else 
				insert into _Languages(`procedure`, `session`, entity, `data`, country, name, `key`, code, flag, description, deleted, date_in, date_created) values 
					(`$procedure`, `$session`, $entity, `$data`, $country, $name, `$key`, $code, $flag, $description, $deleted, $date_in, (select date_in from Languages where id = $entity limit 1));
			end if;
		end;end if;
		
	end;^
	
	drop trigger if exists languages_trigger_update_entity;^
	create trigger languages_trigger_update_entity after update on Languages for each row begin 
		call languages_data_update(new.id);
	end;^
	
	drop trigger if exists languages_trigger_insert;^
	create trigger languages_trigger_insert after insert on LanguagesData for each row begin 
		if new.date_out is null then 
			call languages_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.country, new.name, new.`key`, new.code, new.flag, new.description, new.deleted, new.date_in);
		end if;
	end;^
	
	drop trigger if exists languages_trigger_update;^
	create trigger languages_trigger_update after update on LanguagesData for each row begin 
		if new.entity != old.entity then 
			call languages_data_update(old.entity);
		end if;
		if new.date_out is null then 
			call languages_full_update(new.`procedure`, new.`session`, new.entity, new.id, new.country, new.name, new.`key`, new.code, new.flag, new.description, new.deleted, new.date_in);
		else 
			call languages_data_update(new.entity);
		end if;
	end;^
	
	drop trigger if exists languages_trigger_delete;^
	create trigger languages_trigger_delete after delete on LanguagesData for each row begin 
		call languages_data_update(old.entity);
	end;^
	
delimiter ;