create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop function if exists languages_insert_validate;^
	create function languages_insert_validate(
		$country integer,
		$name varchar(64),
		`$key` varchar(64),
		$code varchar(4),
		$flag_id integer,
		$description varchar(512)
	) returns varchar(512) begin 
		
		declare $error varchar(512) default '';

		set $error := errors_set($error, countries_validate($country, true), 0, 0);
		set $error := errors_set($error, (case 
			when $name is null then 1 << 0
		else 0 end), 4, 0);
		set $error := errors_set($error, (case 
			when `$key` is null then 1 << 0
		else 0 end), 5, 0);
		set $error := errors_set($error, (case 
			when $code is null then 1 << 0
		else 0 end), 6, 0);
	
		return $error;
	end;^
	
	drop procedure if exists languages_add;^
	create procedure languages_add(
		in `$session` text,
		in $country integer,
		in $name varchar(64),
		in `$key` varchar(64),
		in $code varchar(4),
		in $flag_id integer,
		in $flag_name varchar(256),
		in $flag_mime varchar(64),
		in $flag_size integer,
		in $flag_date_modified bigint,
		in $flag_path varchar(512),
		in $flag_hashes text,
		in $description varchar(512),
		out $error varchar(512),
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'languages_add';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}',
			'$.country', $country,
			'$.name', $name,
			'$.key', `$key`,
			'$.code', $code,
			'$.flag_id', $flag_id,
			'$.flag_name', $flag_name,
			'$.flag_mime', $flag_mime,
			'$.flag_size', $flag_size,
			'$.flag_date_modified', $flag_date_modified,
			'$.flag_path', $flag_path,
			'$.flag_hashes', $flag_hashes,
			'$.description', $description,
			'$.entity', 0, 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $suberror integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		set $id := 0;
		call sessions_validate_full(`$session`, '["administrador"]', $error, $session_id, $ip_id, $user_agent_id);
		if $flag_id is null && ifnull($flag_name, '') != '' then begin 
			call files_set($session_id, $ip_id, $user_agent_id, 'AnP.Languages.flag', $flag_name, $flag_mime, $flag_size, $flag_date_modified, $flag_path, $flag_hashes, true, true, $suberror, $flag_id);
			set $error := errors_set($error, $suberror >> 13, 38, 0);
		end;end if;
		set $parameters := json_set($parameters, '$.flag_id', $flag_id);

		set $error := errors_join($error, languages_insert_validate($country, $name, `$key`, $code, $flag_id, $description), 30, 0);
	
		if !errors_has($error) then begin
			
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		
			insert into Languages values();
			set $id := last_insert_id();
			insert into LanguagesData(`procedure`, `session`, entity, country, name, `key`, code, flag, description) values
				($procedure_id, $session_id, $id, $country, $name, `$key`, $code, $flag_id, $description);
			
			set $parameters := json_set($parameters, 
				'$.entity', $id,
				'$.id', last_insert_id()
			);
			
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^
	
delimiter ;