create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^
	
	drop view if exists FilesSingleView;^
	create view FilesSingleView as select 
		files.id as id,
		types.mime as `type`,
		files.name as name,
		files.weight as weight
	from Files files 
	join FilesTypes types on files.`type` = types.id 
	where 
		types.date_out is null && 
		files.date_out is null;^
		
	drop view if exists FilesMultiView;^
	create view FilesMultiView as select 
		files.id as id,
		types.id as type_id,
		types.mime as `type`,
		files.name as name,
		files.weight as weight,
		files.date_in as date_in
	from Files files 
	join FilesTypes types on files.`type` = types.id 
	where 
		files.date_out is null && 
		types.date_out is null;^
		
	drop view if exists FilesFullView;^
	create view FilesFullView as select 
		files.id as id,
		procedures.database_id as database_id,
		procedures.`database` as `database`,
		procedures.id as procedure_id,
		procedures.`procedure` as `procedure`,
		sessions.id as `session`,
		sessions.user_id as user_id,
		sessions.nick as nick,
		sessions.ip_id as ip_id,
		sessions.ip as ip,
		types.id as type_id,
		types.mime as `type`,
		files.name as name,
		files.weight as weight,
		files.date_in as date_in
	from Files files 
	join FilesTypes types on files.`type` = types.id 
	join SessionsFullView sessions on files.`session` = sessions.id 
	join ProceduresFullView procedures on files.`procedure` = procedures.id 
	where 
		files.date_out is null && 
		types.date_out is null;^
		
	drop view if exists LanguagesMultiView;^
	create view LanguagesMultiView as select 
		languages.entity as id,
		countries.entity as country_id,
		countries.name as country,
		countries.`key` as country_key,
		countries.code as country_code, 
		flags.id as flag_id,
		flags.`type` as flag_type,
		flags.name as flag_name,
		flags.weight as flag_weight, 
		languages.name as name,
		languages.`key` as `key`,
		languages.code as code,
		languages.date_in as date_in
	from _Languages languages 
	join _Countries countries on languages.country = countries.entity 
	join FilesSingleView flags on countries.flag = flags.id 
	where 
		languages.deleted is null && 
		countries.deleted is null;^

	drop view if exists SessionsGroupsView;^
	create view SessionsGroupsView as select distinct * from (
		select 
			sessions.id as `session`, 
			groups.entity as group_id,
			groups.name as `group`, 
			groups.is_entity as is_entity, 
			'ip' as `type` 
		from Sessions sessions 
		join _SessionsIps sessions_ips on sessions.id = sessions_ips.`session`
		join _IpsGroups ips_groups on sessions_ips.ip = ips_groups.ip 
		join _Groups groups on groups.entity = ips_groups.`group` 
		where 
			ips_groups.belongs && 
			groups.deleted is null
		union all 
		select 
			sessions.id as `session`, 
			groups.entity as group_id, 
			groups.name as `group`, 
			groups.is_entity as is_entity, 
			'user' as `type` 
		from Sessions sessions 
		join _UsersGroups users_groups on sessions.`user` = users_groups.`user` 
		join _Groups groups on groups.entity = users_groups.`group` 
		where 
			users_groups.belongs && 
			groups.deleted is null
	) permissions;^
	
	drop view if exists SessionsPermissionsView;^
	create view SessionsPermissionsView as select distinct 
		groups.`session` as `session`,
		permissions.id as permission_id,
		permissions.name as permission
	from SessionsGroupsView groups 
	join _PermissionsGroups permissions_groups on groups.group_id = permissions_groups.`group` 
	join _Permissions permissions on permissions.entity = permissions_groups.permission 
	where 
		permissions.deleted is null && 
		permissions_groups.belongs;^
	
	drop function if exists sessions_has_permissions;^
	create function sessions_has_permissions(
		`$session` integer,
		$permissions text
	) returns bool begin 
		
		set $permissions := trim($permissions);
	
		return (case 
			when ifnull($permissions, '') in ('', '[]') then true 
			when substring($permissions, 1, 1) = '[' then ifnull((
				select true from SessionsPermissionsView where `session` = `$session` && locate(concat('"', permission, '"'), $permissions) limit 1
			), false)
		else ifnull((
			select true from SessionsPermissionsView where 
				`session` = `$session` && 
				permission = $permissions 
			limit 1
		), false) end);
	end;^
	
	drop function if exists sessions_is_banned;^
	create function sessions_is_banned(
		`$session` integer
	) returns bool begin 
		return ifnull((
			select true from SessionsPermissionsView where 
				`session` = `$session` && 
				permission = 'banned' 
			limit 1
		), false);
	end;^
	
	drop view if exists SessionsSettingsView;^
	create view SessionsSettingsView as select distinct 
		settings.entity as entity,
		sessions.id as `session`, 
		settings.name as name, 
		settings_types.name as `type`,
		if(groups.entity is null, settings.value, groups.value) as value,
		ifnull(groups.date_in, settings.date_in) as date_in
	from _Settings settings 
	join Sessions sessions on true 
	join _SettingsTypes settings_types on settings.`type` = settings_types.entity 
	left join (
		select 
			settings_groups.entity as entity,
			settings_groups.settings as settings,
			settings_groups.value as value,
			settings_groups.date_in as date_in,
			settings_groups.`session` as `session`
		from _SettingsGroups settings_groups
		join (
			select * from SessionsGroupsView where group_id in (
				select min(group_id) from SessionsGroupsView where `type` = 'user' && is_entity group by `session`
			)
		) sessions_groups on settings_groups.`group` = sessions_groups.group_id 
		where settings_groups.belongs
	) groups on 
		sessions.id = groups.`session` && 
		groups.settings = settings.entity  
	where 
		sessions.date_out is null && 
		settings.deleted is null && 
		settings.`range` & (1 << 1);^

	drop view if exists IpsSettingsView;^
	create view IpsSettingsView as select distinct 
		settings.entity as entity,
		ips.entity as ip, 
		settings.name as name, 
		settings_types.name as `type`,
		if(settings_ips_groups.`group` is null, settings.value, settings_ips_groups.value) as value
	from _Settings settings 
	join _SettingsTypes settings_types on settings.`type` = settings_types.entity 
	join _Ips ips on true 
	left join (
		select 
			settings_groups.`group` as `group`,
			settings_groups.settings as settings,
			ips_groups.ip as ip,
			settings_groups.value as value 
		from _SettingsGroups settings_groups 
		join _Groups groups on groups.is_entity 
		join _IpsGroups ips_groups on 
			settings_groups.`group` = ips_groups.`group` && 
			ips_groups.`group` = groups.entity 
		where 
			settings_groups.belongs && 
			ips_groups.belongs 
	) settings_ips_groups on 
		settings_ips_groups.ip = ips.entity && 
		settings_ips_groups.settings = settings.entity 
	where settings.`range` & (1 << 1);^

	drop function if exists ips_settings_get;^
	create function ips_settings_get(
		$ip integer,
		$name varchar(64)
	) returns text begin 
		return (select value from IpsSettingsView where ip = $ip && name = $name limit 1);
	end;^

	drop function if exists ips_settings_get_int;^
	create function ips_settings_get_int(
		$ip integer,
		$name varchar(64)
	) returns integer begin 
		return string_to_integer(ips_settings_get($ip, $name));
	end;^

	drop function if exists ips_settings_get_bool;^
	create function ips_settings_get_bool(
		$ip integer,
		$name varchar(64)
	) returns integer begin 
		return string_to_bool(ips_settings_get($ip, $name));
	end;^

	drop view if exists UsersSettingsView;^
	create view UsersSettingsView as select distinct 
		settings.entity as entity,
		users.entity as `user`, 
		settings.name as name, 
		settings_types.name as `type`,
		if(settings_users_groups.`group` is null, settings.value, settings_users_groups.value) as value
	from _Settings settings 
	join _SettingsTypes settings_types on settings.`type` = settings_types.entity 
	join _Users users on true 
	left join (
		select 
			settings_groups.`group` as `group`,
			settings_groups.settings as settings,
			users_groups.`user` as `user`,
			settings_groups.value as value 
		from _SettingsGroups settings_groups 
		join _Groups groups on groups.is_entity 
		join _UsersGroups users_groups on 
			settings_groups.`group` = users_groups.`group` && 
			users_groups.`group` = groups.entity 
		where 
			settings_groups.belongs && 
			users_groups.belongs 
	) settings_users_groups on 
		settings_users_groups.`user` = users.entity && 
		settings_users_groups.settings = settings.entity 
	where settings.`range` & (1 << 1);^

	drop function if exists users_settings_get;^
	create function users_settings_get(
		`$user` integer,
		$name varchar(64)
	) returns text begin 
		return (select value from UsersSettingsView where `user` = `$user` && name = $name limit 1);
	end;^

	drop function if exists users_settings_get_int;^
	create function users_settings_get_int(
		`$user` integer,
		$name varchar(64)
	) returns integer begin 
		return string_to_integer(users_settings_get(`$user`, $name));
	end;^

	drop function if exists users_settings_get_bool;^
	create function users_settings_get_bool(
		`$user` integer,
		$name varchar(64)
	) returns bool begin 
		return string_to_bool(users_settings_get(`$user`, $name));
	end;^
	
	drop function if exists sessions_settings_get;^
	create function sessions_settings_get(
		`$session` integer,
		$name varchar(64)
	) returns text begin 
		return (select value from SessionsSettingsView where `session` = `$session` && name = $name limit 1);
	end;^
	
	drop function if exists sessions_settings_get_int;^
	create function sessions_settings_get_int(
		`$session` integer,
		$name varchar(64)
	) returns integer begin 
		return string_to_integer(sessions_settings_get(`$session`, $name));
	end;^
	
	drop function if exists sessions_settings_get_bool;^
	create function sessions_settings_get_bool(
		`$session` integer,
		$name varchar(64)
	) returns bool begin 
		return string_to_bool(sessions_settings_get(`$session`, $name));
	end;^
	
	drop function if exists sessions_timeout;^
	create function sessions_timeout(
		`$session` integer
	) returns bool begin
		return ifnull((
			select 
				date_out is not null || 
				timestampdiff(second, date_last, now()) > users_settings_get_int(`user`, 'session_timeout') 
			from Sessions where id = `$session` limit 1
		), false);
	end;^
	
	drop procedure if exists full_tables_fill;^
	create procedure full_tables_fill(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'full_tables_fill';
		declare $procedure_id integer;
		declare $session_id integer default sessions_get_id(`$session`);
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $machine varchar(32) default settings_get('machine_user', false);
		declare `$unknown` varchar(32) default settings_get('unknown_user', false);
		declare $guest varchar(32) default settings_get('guest_user', false);
		
		drop temporary table if exists default_groups_temporary_anp;
		drop temporary table if exists default_permissions_temporary_anp;
		drop temporary table if exists default_permissions_groups_temporary_anp;
		drop temporary table if exists default_users_groups_temporary_anp;
		drop temporary table if exists default_ips_groups_temporary_anp;
		drop temporary table if exists default_users_settings_temporary_anp;
	
		if `$session` is null then 
			set `$session` := machine_get_session_id();
		end if;
	
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;
	
		create temporary table default_groups_temporary_anp(
			id integer not null auto_increment,
			name varchar(64) not null,
			`default` boolean not null,
			is_entity boolean not null,
			description text,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_groups_temporary_anp_id primary key(id)
		);
	
		create temporary table default_permissions_temporary_anp(
			id integer not null auto_increment,
			name varchar(64) not null,
			description text,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_permissions_temporary_anp_id primary key(id)
		);
	
		create temporary table default_permissions_groups_temporary_anp(
			id integer not null auto_increment,
			permission varchar(64) not null,
			`group` varchar(64) not null,
			belongs bool not null default true,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_permissions_groups_temporary_anp_id primary key(id)
		);
	
		create temporary table default_users_groups_temporary_anp(
			id integer not null auto_increment,
			`user` varchar(32) not null,
			`group` varchar(64) not null,
			belongs bool not null default true,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_users_groups_temporary_anp_id primary key(id)
		);
	
		create temporary table default_ips_groups_temporary_anp(
			id integer not null auto_increment,
			ip varchar(39) not null,
			`group` varchar(64) not null,
			belongs bool not null default true,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_ips_groups_temporary_anp_id primary key(id)
		);
	
		create temporary table default_users_settings_temporary_anp(
			id integer not null auto_increment,
			nick varchar(32) not null,
			name varchar(64) not null,
			value text not null,
			date_in datetime not null default now(),
			date_out datetime,
			constraint default_users_settings_temporary_anp_id primary key(id)
		);
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
	
		insert into default_groups_temporary_anp(name, `default`, is_entity, description) 
		select nick, false, true, concat('The entity group for ', nick, '.') from _Users union all
		select address, false, true, concat('The entity group for ', address, '.') from _Ips;
		
		insert into default_users_groups_temporary_anp(`user`, `group`) 
		select nick, nick from _Users;
		
		insert into default_ips_groups_temporary_anp(ip, `group`) 
		select address, address from _Ips;
	
		insert into default_groups_temporary_anp(name, `default`, is_entity, description) values 
			('bots', false, false, 'The item type is a bot.'),
			('users', true, false, 'The item type is a simple registered user.'),
			('guests', false, false, 'The item type is a Guest user.'),
			('banned', false, false, 'The item is banned.'),
			('administrators', false, false, 'The item type is a global administrator.');
	
		insert into default_permissions_temporary_anp(name, description) values 
			('user', 'The item has only the simple registered user permissions.'),
			('guest', 'The item has only the Guest user perimssions.'),
			('banned', 'The item is banned.'),
			('administrator', 'The item has the global administrator.');
	
		insert into default_permissions_groups_temporary_anp(permission, `group`) values 
			('user', 'users'),
			('guest', 'guests'),
			('banned', 'banned'),
			('administrator', 'administrators');
	
		insert into default_users_groups_temporary_anp(`user`, `group`) values 
			('guest', 'guests'),
			('machine', 'bots'),
			('machine', 'administrators');
	
		insert into default_users_settings_temporary_anp(nick, name, value) values 
			($machine, 'session_timeout', 0),
			($machine, 'users_maximum_sessions', 0),
			(`$unknown`, 'session_timeout', 0),
			(`$unknown`, 'users_maximum_sessions', 0),
			($guest, 'session_timeout', 3600),
			($guest, 'users_maximum_sessions', 2147483647);
		
		default_groups:begin
			
			declare $done bool default false;
			declare $name varchar(64);
			declare `$default` bool;
			declare $is_entity bool;
			declare $description varchar(512);
			declare default_groups_cursor cursor for select name, `default`, is_entity, description from default_groups_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_groups_cursor;
			default_groups_reader:loop 
				
				fetch default_groups_cursor into $name, `$default`, $is_entity, $description;
				if $done then 
					leave default_groups_reader;
				end if;
			
				if (select 1 from _Groups where name = $name limit 1) is null then begin 
					insert into Groups values();
					insert into GroupsData(`procedure`, `session`, entity, name, `default`, is_entity, description) values 
						($procedure_id, `$session`, last_insert_id(), $name, `$default`, $is_entity, $description);
				end;end if;
			
			end loop default_groups_reader;
			close default_groups_cursor;
			
		end default_groups;
		
		default_permissions:begin
			
			declare $done bool default false;
			declare $name varchar(64);
			declare $description varchar(512);
			declare default_permissions_cursor cursor for select name, description from default_permissions_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_permissions_cursor;
			default_permissions_reader:loop 
				
				fetch default_permissions_cursor into $name, $description;
				if $done then 
					leave default_permissions_reader;
				end if;
			
				if (select 1 from _Permissions where name = $name limit 1) is null then begin 
					insert into Permissions values();
					insert into PermissionsData(`procedure`, `session`, entity, name, description) values 
						($procedure_id, `$session`, last_insert_id(), $name, $description);
				end;end if;
			
			end loop default_permissions_reader;
			close default_permissions_cursor;
			
		end default_permissions;
		
		default_permissions_groups:begin
			
			declare $done bool default false;
			declare $permission varchar(64);
			declare `$group` varchar(64);
			declare $belongs bool;
			declare $description varchar(512);
			declare default_permissions_groups_cursor cursor for select permission, `group`, belongs from default_permissions_groups_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_permissions_groups_cursor;
			default_permissions_groups_reader:loop 
				
				fetch default_permissions_groups_cursor into $permission, `$group`, $belongs;
				if $done then 
					leave default_permissions_groups_reader;
				end if;
			
				if (
					select 1 
					from _PermissionsGroups permissions_groups 
					join _Permissions permissions on permissions_groups.permission = permissions.entity 
					join _Groups groups on permissions_groups.`group` = groups.entity 
					where 
						permissions.name = $permission && 
						groups.name = `$group` 
					limit 1
				) is null then begin 
					insert into PermissionsGroups values();
					insert into PermissionsGroupsData(`procedure`, `session`, entity, permission, `group`, belongs) values 
						($procedure_id, `$session`, last_insert_id(), (
							select entity from _Permissions where name = $permission limit 1
						), (
							select entity from _Groups where name = `$group` limit 1
						), $belongs);
				end;end if;
			
			end loop default_permissions_groups_reader;
			close default_permissions_groups_cursor;
			
		end default_permissions_groups;
		
		default_users_groups:begin
			
			declare $done bool default false;
			declare `$user` varchar(32);
			declare `$group` varchar(64);
			declare $belongs bool;
			declare $description varchar(512);
			declare default_users_groups_cursor cursor for select `user`, `group`, belongs from default_users_groups_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_users_groups_cursor;
			default_users_groups_reader:loop 
				
				fetch default_users_groups_cursor into `$user`, `$group`, $belongs;
				if $done then 
					leave default_users_groups_reader;
				end if;
			
				if (
					select 1 
					from _UsersGroups users_groups 
					join _Users users on users_groups.`user` = users.entity 
					join _Groups groups on users_groups.`group` = groups.entity 
					where 
						users.nick = `$user` && 
						groups.name = `$group` 
					limit 1
				) is null then begin 
					insert into UsersGroups values();
					insert into UsersGroupsData(`procedure`, `session`, entity, `user`, `group`, belongs) values 
						($procedure_id, `$session`, last_insert_id(), (
							select entity from _Users where nick = `$user` limit 1
						), (
							select entity from _Groups where name = `$group` limit 1
						), $belongs);
				end;end if;
			
			end loop default_users_groups_reader;
			close default_users_groups_cursor;
			
		end default_users_groups;
		
		default_ips_groups:begin
			
			declare $done bool default false;
			declare $ip varchar(64);
			declare `$group` varchar(64);
			declare $belongs bool;
			declare $description varchar(512);
			declare default_ips_groups_cursor cursor for select ip, `group`, belongs from default_ips_groups_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_ips_groups_cursor;
			default_ips_groups_reader:loop 
				
				fetch default_ips_groups_cursor into $ip, `$group`, $belongs;
				if $done then 
					leave default_ips_groups_reader;
				end if;
			
				if (
					select 1 
					from _IpsGroups ips_groups 
					join _Ips ips on ips_groups.ip = ips.entity 
					join _Groups groups on ips_groups.`group` = groups.entity 
					where 
						ips.address = $ip && 
						groups.name = `$group` 
					limit 1
				) is null then begin 
					insert into IpsGroups values();
					insert into IpsGroupsData(`procedure`, `session`, entity, ip, `group`, belongs) values 
						($procedure_id, `$session`, last_insert_id(), (
							select entity from _Ips where address = $ip limit 1
						), (
							select entity from _Groups where name = `$group` limit 1
						), $belongs);
				end;end if;
			
			end loop default_ips_groups_reader;
			close default_ips_groups_cursor;
			
		end default_ips_groups;
		
		default_users_settings:begin
			
			declare $done bool default false;
			declare $nick varchar(32);
			declare $name varchar(64);
			declare $value text;
			declare $user_id integer;
			declare default_users_settings_cursor cursor for select nick, name, value from default_users_settings_temporary_anp;
			declare continue handler for not found set $done := true;
		
			open default_users_settings_cursor;
			default_users_settings_reader:loop 
				
				fetch default_users_settings_cursor into $nick, $name, $value;
				if $done then 
					leave default_users_settings;
				end if;
			
				set $user_id = (select entity from _Users where deleted is null && nick = $nick limit 1);

				if users_settings_get($user_id, $name) != $value then begin 
					insert into SettingsGroups values();
					insert into SettingsGroupsData(`procedure`, `session`, entity, `group`, settings, value, belongs) values 
						($procedure_id, `$session`, last_insert_id(), (
							select entity from _Groups where deleted is null && name = $nick limit 1
						), (
							select entity from _Settings where deleted is null && name = $name limit 1
						), $value, true);
				end;end if;
			
			end loop default_users_settings_reader;
			close default_users_settings_cursor;
			
		end default_users_settings;
		
		drop temporary table if exists default_groups_temporary_anp;
		drop temporary table if exists default_permissions_temporary_anp;
		drop temporary table if exists default_permissions_groups_temporary_anp;
		drop temporary table if exists default_users_groups_temporary_anp;
		drop temporary table if exists default_ips_groups_temporary_anp;
		drop temporary table if exists default_users_settings_temporary_anp;
	
	end;^
	
	call full_tables_fill(null);^
	
	drop view if exists LogsFullView;^
	create view LogsFullView as select 
		logs.id as id,
		procedures.database_id as database_id,
		procedures.id as procedure_id,
		procedures.`database` as `database`,
		procedures.`procedure` as `procedure`,
		sessions.id as session_id,
		sessions.user_id as user_id,
		sessions.nick as nick,
		sessions.ip_id as ip_id,
		sessions.ip as ip,
		sessions.user_agent_id as user_agent_id,
		sessions.user_agent as user_agent,
		ips.entity as log_ip_id,
		ips.address as log_ip,
		users_agents.id as log_user_agent_id,
		users_agents.`data` as log_user_agent,
		errors.id as error_id,
		errors.code as error,
		parameters.id as parameters_id,
		parameters.json as parameters,
		logs.date_in as date_in
	from Logs logs
	join ProceduresFullView procedures on logs.`procedure` = procedures.id
	join SessionsFullView sessions on logs.`session` = sessions.id
	left join _Ips ips on logs.ip = ips.entity
	left join UsersAgents users_agents on logs.user_agent = users_agents.id 
	join Errors errors on logs.own_error = errors.id 
	join Parameters parameters on logs.own_parameters = parameters.id 
	where logs.date_out is null 
	order by logs.id asc;^
	
	drop view if exists ExceptionsFullView;^
	create view ExceptionsFullView as select 
		exceptions.id as id,
		procedures.database_id as database_id,
		procedures.id as procedure_id,
		procedures.`database` as `database`,
		procedures.`procedure` as `procedure`,
		sessions.id as session_id,
		sessions.user_id as user_id,
		sessions.nick as nick,
		sessions.ip_id as ip_id,
		sessions.ip as ip,
		sessions.user_agent_id as user_agent_id,
		sessions.user_agent as user_agent,
		ips.entity as log_ip_id,
		ips.address as log_ip,
		users_agents.id as log_user_agent_id,
		users_agents.`data` as log_user_agent,
		errors.id as error_id,
		errors.code as error,
		parameters.id as parameters_id,
		parameters.json as parameters,
		messages.id as message_id,
		messages.`text` as message,
		exceptions.status as status,
		exceptions.code as code,
		exceptions.date_in as date_in
	from Exceptions exceptions
	join ProceduresFullView procedures on exceptions.`procedure` = procedures.id
	join SessionsFullView sessions on exceptions.`session` = sessions.id
	left join _Ips ips on exceptions.ip = ips.entity
	left join UsersAgents users_agents on exceptions.user_agent = users_agents.id 
	join Errors errors on exceptions.own_error = errors.id 
	join Parameters parameters on exceptions.own_parameters = parameters.id
	join Messages messages on exceptions.message = messages.id
	where exceptions.date_out is null 
	order by exceptions.id asc;^
	
	drop view if exists TracesFullView;^
	create view TracesFullView as select 
		traces.id as id, 
		sessions.id as session_id, 
		sessions.user_id as user_id,
		sessions.nick as nick,
		sessions.ip_id as ip_id,
		sessions.ip as ip,
		sessions.user_agent_id as user_agent_id,
		sessions.user_agent as user_agent, 
		sessions.date_in as session_date_in, 
		sessions.date_out as session_date_out,
		sessions.date_last as session_date_last, 
		projects.id as project_id, 
		projects.name as project, 
		files.id as file_id, 
		files.`path` as `path`,
		errors.id as error_id, 
		errors.code as error, 
		parameters.id as parameters_id, 
		parameters.json as parameters, 
		`lines`.id as traces_lines_id, 
		`lines`.trace as trace, 
		messages.id as message_id, 
		messages.`text` as message, 
		traces.line as line, 
		traces.date_in as date_in 
	from Traces traces 
	join SessionsFullView sessions on traces.`session` = sessions.id 
	join Projects projects on traces.project = projects.id 
	join TracesFiles files on traces.file = files.id 
	join Errors errors on traces.own_error = errors.id 
	join Parameters parameters on traces.own_parameters = parameters.id
	join `Lines` `lines` on traces.trace = `lines`.id 
	join Messages messages on traces.message = messages.id
	where traces.date_out is null 
	order by traces.id asc;^
	
	drop view if exists LanguagesIdSelectView;^
	create view LanguagesIdSelectView as select entity as id, `key` from _Languages where deleted is null;^
	
	drop procedure if exists languages_get_id_select;^
	create procedure languages_get_id_select() begin 
		
		select * from LanguagesIdSelectView;
		
	end;^
	
	drop function if exists languages_validate;^
	create function languages_validate(
		$language_id integer,
		$undeleted bool
	) returns integer begin 
		return (case 
			when $language_id is null then 1 << 0
			when $language_id < 1 then 1 << 1
		else ifnull((
			select if($undeleted, if(deleted is null, 0, 1 << 3), if(deleted is null, 1 << 3, 0)) from _Languages where id = $language_id limit 1
		), 1 << 2) end);
	end;^
	
	drop procedure if exists locates_set_raw;^
	create procedure locates_set_raw(
		in $session_id integer,
		in $ip_id integer,
		in $user_agent_id integer,
		in `$group` varchar(64),
		in $coordenates varchar(32),
		in $owner bool,
		out $error integer,
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'locates_set_raw';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}', 
			'$.group', `$group`, 
			'$.coordenates', $coordenates,
			'$.owner', $owner
		);
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text; 
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		set $error := 0;
		set $id := 0;
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
	
		set $error := (
			/*((case 
				when $session_id is null then 1 << 0
				when $session_id < 1 then 1 << 1
			else ifnull((
				select if(date_out is null, 0, 1 << 3 from Sessions where id = $session_id limit 1
			), 1 << 2) end) << 0) |
			((case 
				when $ip_id is null then 1 << 0
				when $ip_id < 1 then 1 << 1
			else ifnull((
				select (case 
					when deleted is null then 1 << 3 
					when `accessible` = 0 then 1 << 4 
				else 0 end) from _Ips where entity = $ip_id limit 1
			), 1 << 2) end) << 4) |
			((case 
				when $user_agent_id is null then 1 << 0
				when $user_agent_id < 1 then 1 << 1
			else ifnull((
				select if(date_out is null, 1 << 3, 0 from UsersAgents where id = $user_agent_id limit 1
			), 1 << 2) end) << 9) |*/
			((case 
				when $coordenates is null then 1 << 0
				when $coordenates = '' then 1 << 1
				when $coordenates not like '%,%,%' then 1 << 2
			else 0 end) << 13) |
			(if(@owner is null, 1 << 0, 0) << 16) |
			((case 
				when `$group`  is null then 1 << 0
				when `$group` = '' then 1 << 1
			else 0 end) << 17) |
		0) << 1;

		if !$error then begin 
			
			declare $longitude_i integer default locate(',', $coordenates);
			declare $longitude float default convert(substring($coordenates, 1, $longitude_i), float);
			declare $latitude_i integer default locate(',', $coordenates, $longitude_i + 1);
			declare $latitude float default convert(substring($coordenates, $longitude_i + 1, $latitude_i), float);
			declare $altitude float default convert(substring($coordenates, $latitude_i + 1, 32), float);

			set $error := (
				((case 
					when $longitude is null then 1 << 0
					when $longitude < -90 then 1 << 1
					when $longitude > 90 then 1 << 2
				else 0 end) << 0) |
				((case 
					when $latitude is null then 1 << 0
					when $latitude < -180 then 1 << 1
					when $latitude > 180 then 1 << 2
				else 0 end) << 3) |
				((case 
					when $altitude is null then 1 << 0
					when $altitude < -50000 then 1 << 1
					when $altitude > 100000 then 1 << 2
				else 0 end) << 6) |
			0) << 19;

			if !$error then begin 

				declare $group_id integer default (select id from LocatesGroups where deleted is null && name = `$group` limit 1);

				if $group_id is null then begin 
					insert into LocatesGroups(`procedure`, `session`, name) values
						($procedure_id, $session_id, `$group`);
					set $group_id := last_insert_id();
				end;end if;
				
				if $owner then  
					set $id := ifnull((
						select id from Locates where 
							deleted is null and 
							`group` = $group_id &&  
							longitude = $longitude && 
							latitude = $latitude && 
							altitude = $altitude
					), 0);
				end if;

				if !$id then begin  
					insert into Locates(`procedure`, `session`, `group`, longitude, latitude, altitude) values 
						($procedure_id, $session_id, $group_id, $longitude, $latitude, $altitude);
					set $id := last_insert_id();
				end;end if;

			end;end if;

		end;end if;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
	
	end;^
	
	drop procedure if exists locates_set;^
	create procedure locates_set(
		in $session_id integer,
		in $ip_id integer,
		in $user_agent_id integer,
		in `$group` varchar(64),
		in $longitude float,
		in $latitude float,
		in $altitude float,
		in $owner bool,
		in $allow_null bool, 
		out $error integer,
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'locates_set';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}', 
			'$.group', `$group`, 
			'$.longitude', $longitude,
			'$.latitude', $latitude,
			'$.altitude', $altitude,
			'$.owner', $owner, 
			'$.allow_null', $allow_null
		);
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text; 
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		set $error := 0;
		set $id := null;
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
	
		set $error := (
			/*((case 
				when $session_id is null then 1 << 0
				when $session_id < 1 then 1 << 1
			else ifnull((
				select if(date_out is null, 0, 1 << 3 from Sessions where id = $session_id limit 1
			), 1 << 2) end) << 0) |
			((case 
				when $ip_id is null then 1 << 0
				when $ip_id < 1 then 1 << 1
			else ifnull((
				select (case 
					when deleted is null then 1 << 3 
					when `accessible` = 0 then 1 << 4 
				else 0 end) from _Ips where entity = $ip_id limit 1
			), 1 << 2) end) << 4) |
			((case 
				when $user_agent_id is null then 1 << 0
				when $user_agent_id < 1 then 1 << 1
			else ifnull((
				select if(date_out is null, 1 << 3, 0 from UsersAgents where id = $user_agent_id limit 1
			), 1 << 2) end) << 9) |*/
			((case 
				when `$group` is null then 1 << 0
				when `$group` = '' then 1 << 1
			else 0 end) << 13) |
			((case 
				when $longitude is null then if($allow_null, 0, 1 << 0)
				when $longitude < -90 then 1 << 1
				when $longitude > 90 then 1 << 2
			else 0 end) << 15) |
			((case 
				when $latitude is null then if($allow_null, 0, 1 << 0)
				when $latitude < -180 then 1 << 1
				when $latitude > 180 then 1 << 2
			else 0 end) << 18) |
			((case 
				when $altitude is null then if($allow_null, 0, 1 << 0)
				when $altitude < -50000 then 1 << 1
				when $altitude > 100000 then 1 << 2
			else 0 end) << 21) |
			(if($owner is null, 1 << 0, 0) << 24) |
		0) << 1;

		if !$error && (
			$longitude is not null || 
			$latitude is not null || 
			$altitude is not null 
		) then begin 

			declare $group_id integer default (select id from LocatesGroups where deleted is null && name = `$group` limit 1);

			if $group_id is null then begin 
				insert into LocatesGroups(`procedure`, `session`, name) values
					($procedure_id, $session_id, `$group`);
				set $group_id := last_insert_id();
			end;end if;
			
			if $owner then  
				set $id := ifnull((
					select id from Locates where 
						deleted is null and 
						`group` = $group_id &&  
						longitude = $longitude && 
						latitude = $latitude && 
						altitude = $altitude 
					limit 1
				), 0);
			end if;

			if !$id then begin 
				insert into Locates(`procedure`, `session`, `group`, longitude, latitude, altitude) values 
					($procedure_id, $session_id, $group_id, $longitude, $latitude, $altitude);
				set $id := last_insert_id();
			end;end if;

		end;end if;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 2);
	
	end;^
	
	drop procedure if exists files_set;^
	create procedure files_set(
		in $session_id integer,
		in $ip_id integer,
		in $user_agent_id integer,
		in `$group` varchar(256),
		in $name varchar(256),
		in `$type` varchar(64),
		in $weight integer, 
		in $date_modified bigint, 
		in `$path` varchar(512),
		in $hashes text, 
		in $owner bool,
		in $allow_null bool,
		out $error integer,
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'files_set';
		declare $procedure_id integer;
		declare $parameters text default json_set('{}', 
			'$.group', `$group`, 
			'$.name', $name,
			'$.weight', $weight,
			'$.type', `$type`,
			'$.date_modified', $date_modified, 
			'$.path', `$path`,
			'$.owner', $owner, 
			'$.allow_null', $allow_null
		);
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text; 
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		set $error := 0;
		set $id := 0;
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
	
		set $error := (
			/*((case 
				when $session_id is null then 1 << 0
				when $session_id < 1 then 1 << 1
			else ifnull((
				select if(date_out is null, 0, 1 << 3 from Sessions where id = $session_id limit 1
			), 1 << 2) end) << 0) |
			((case 
				when $ip_id is null then 1 << 0
				when $ip_id < 1 then 1 << 1
			else ifnull((
				select (case 
					when deleted is null then 1 << 3 
					when `accessible` = 0 then 1 << 4 
				else 0 end) from _Ips where entity = $ip_id limit 1
			), 1 << 2) end) << 4) |
			((case 
				when $user_agent_id is null then 1 << 0
				when $user_agent_id < 1 then 1 << 1
			else ifnull((
				select if(date_out is null, 1 << 3, 0 from UsersAgents where id = $user_agent_id limit 1
			), 1 << 2) end) << 9) |*/
			((case 
				when `$group` is null then 1 << 0
				when `$group` = '' then 1 << 1
			else 0 end) << 13) |
			((case 
				when $name is null then if($allow_null, 0, 1 << 0)
				when $name = '' then 1 << 1
			else 0 end) << 15) |
			((case 
				when `$type` is null then if($allow_null, 0, 1 << 0)
				when `$type` = '' then 1 << 1
			else 0 end) << 17) |
			((case 
				when $weight is null then if($allow_null, 0, 1 << 0)
				when $weight < 0 then 1 << 1
				when $weight < 1 then 1 << 2
			else 0 end) << 19) |
			(if($date_modified is null && !$allow_null, 1 << 0, 0) << 22) |
			((case 
				when `$path` is null then if($allow_null, 0, 1 << 0)
				when `$path` = '' then 1 << 1
			else 0 end) << 23) |
			((case 
				when $hashes is null then if($allow_null, 0, 1 << 0)
				when $hashes = '' then 1 << 1
			else 0 end) << 25) |
			(if($owner is null, 1 << 0, 0) << 27) |
		0) << 1;

		if !$error && (
			`$type` is not null && 
			$name is not null && 
			`$path` is not null && 
			$weight is not null && 
			$hashes is not null 
		) then begin 

			declare $group_id integer default (select id from FilesGroups where date_out is null && deleted is null && name = `$group` limit 1);
			declare $type_id integer default (select id from FilesTypes where date_out is null && deleted is null && mime = `$type` limit 1);

			if $group_id is null then begin 
				insert into FilesGroups(`procedure`, `session`, name) values
					($procedure_id, $session_id, `$group`);
				set $group_id := last_insert_id();
			end;end if;

			if $type_id is null then begin 
				insert into FilesTypes(`procedure`, `session`, mime) values
					($procedure_id, $session_id, `$type`);
				set $type_id := last_insert_id();
			end;end if;
			
			if $owner then  
				set $id := ifnull((
					select id from Files where 
						date_out is null && 
						deleted is null && 
						`group` = $group_id && 
						`type` = $type_id && 
						name = $name && 
						`path` = `$path` && 
						weight = $weight && 
						date_modified = $date_modified 
					limit 1
				), 0);
			end if;

			if !$id then begin 
				
				declare $hash_name varchar(16);
				declare $hash_value varchar(256);
				declare $hash_colon integer;
				declare $algorithm_id integer;
				
				insert into Files(`procedure`, `session`, `group`, `type`, name, `path`, weight, date_modified) values 
					($procedure_id, $session_id, $group_id, $type_id, $name, `$path`, $weight, $date_modified);
				set $id := last_insert_id();
			
				while length($hashes) do 
					
					set $hash_colon := locate(':', $hashes);
				
					if $hash_colon then begin 
						
						set $hash_name := substring($hashes, 1, $hash_colon - 1);
						set $hashes := substring($hashes, $hash_colon + 1, 999999999);
						set $hash_colon := locate(',', $hashes);
					
						if !$hash_colon then 
							set $hash_colon := 99999999;
						end if;
					
						set $hash_value := substring($hashes, 1, $hash_colon - 1);
						set $hashes := substring($hashes, $hash_colon + 1, 999999999);
						set $algorithm_id := (select id from Hashes where date_out is null && deleted is null && name = $hash_name limit 1);
					
						if $algorithm_id is null then begin 
							insert into Hashes(`procedure`, `session`, name) values 
								($procedure_id, $session_id, $hash_name);
							set $algorithm_id := last_insert_id();
						end;end if;
					
						insert into FilesHashes(`procedure`, `session`, file, `algorithm`, hash) values 
							($procedure_id, $session_id, $id, $algorithm_id, $hash_value);
						
					end;
						set $hashes := '';
					end if;
				
				end while;
				
			end;end if;

		end;end if;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 2);
		
	end;^
	
	drop view if exists FilesLoadItemView;^
	create view FilesLoadItemView as select 
		files.id as id, 
		groups.id as group_id, 
		groups.name as `group`, 
		types.id as type_id, 
		types.mime as mime, 
		files.name as name,
		files.`path` as `path`, 
		files.weight as `size`, 
		files.date_modified as last_modified, 
		files.description as description 
	from Files files 
	join FilesGroups groups on files.`group` = groups.id 
	join FilesTypes types on files.`type` = types.id 
	where 
		files.date_out is null && files.deleted is null && 
		groups.date_out is null && groups.deleted is null && 
		types.date_out is null && types.deleted is null;^
	
	drop view if exists FilesLoadItemView;^
	create view FilesLoadItemView as select 
		files.id as id, 
		groups.id as group_id, 
		groups.name as `group`, 
		types.id as type_id, 
		types.mime as mime, 
		files.name as name,
		files.`path` as `path`, 
		files.weight as `size`, 
		files.date_modified as last_modified, 
		files.description as description 
	from Files files 
	join FilesGroups groups on files.`group` = groups.id 
	join FilesTypes types on files.`type` = types.id 
	where 
		files.date_out is null && files.deleted is null && 
		groups.date_out is null && groups.deleted is null && 
		types.date_out is null && types.deleted is null;^
		
	drop procedure if exists files_get_path;^
	create procedure files_get_path(
		in `$session` text,
		in $file_id integer, 
		in `$group` varchar(256), 
		in $file_name varchar(256), 
		in $extra_permissions text,
		out $error varchar(512),
		out `$path` varchar(512), 
		out $file_type varchar(64) 
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'files_get_path';
		declare $parameters text default json_set('{}', 
			'$.file_name', $file_name, 
			'$.extra_permissions', $extra_permissions
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $has_errors bool;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text; 
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		call sessions_validate_full(`$session`, $extra_permissions, $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_set($error, (
			(files_validate($file_id, true) << 0) | 
			((case 
				when `$group` is null then 1 << 0 
				when `$group` = '' then 1 << 1 
			else 0 end) << 4) |
			((case 
				when $file_name is null then 0 
				when $file_name = '' then 0 
			else ifnull((
				select (
					if(name != $file_name, 1 << 3, 0) | 
					if(deleted is null, 0, 1 << 4) | 
					if(date_out is null, 0, 1 << 5) | 
				0) from Files where id = $file_id limit 1
			), 1 << 2) end) << 6) | 
		0), 30, 0);
		set $has_errors := errors_has($error);
	
		if !$has_errors then begin 
			
			declare $group_id integer default (select id from FilesGroups where date_out is null && deleted is null && name = `$group` limit 1);
		
			set $error := errors_set($error, (case 
				when $group_id is null then 1 << 0 
				when (select 1 from Files where id = $file_id && `group` = $group_id limit 1) is null then 1 << 1 
			else 0 end), 42, 0);
			set $has_errors := errors_has($error);
		
		end;end if;
	
		select `path`, mime into `$path`, $file_type from FilesLoadItemView where !$has_errors && id = $file_id limit 1;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 2);
		
	end;^

	drop function if exists locates_errors;^
	create function locates_errors(
		$error integer
	) returns integer begin 
		return (($error >> 10) << 1) | ($error & 1);
	end;^

	drop function if exists sessions_get_user;^
	create function sessions_get_user(
		`$session` integer
	) returns integer begin 
		return (select `user` from Sessions where id = `$session` limit 1);
	end;^
	
delimiter ;
