create database if not exists AnP character set utf8mb4 collate utf8mb4_general_ci;
use AnP;

delimiter ;^

	drop function if exists ips_validate_get;^
	create function ips_validate_get(
		$ip varchar(39)
	) returns varchar(512) begin 
		return (case 
			when $ip is null then 1 << 0
			when $ip = '' then 1 << 1 
			-- when $ip not regexp '' then 1 << 2
		else ifnull((
			select (
				if(ips.`accessible`, 0, 1 << 3) |
				ifnull((
					select 1 << 4 
					from _IpsGroups ips_groups
					join _Groups groups on ips_groups.`group` = groups.entity 
					where 
						ips_groups.belongs && 
						groups.deleted is null && 
						ips_groups.ip = ips.entity && 
						groups.name = 'banned' 
					limit 1
				), 0) |
				if((
					select count(1) from SessionsFullView where
						ip_id = ips.entity && 
						timestampdiff(second, date_last, now()) < ips_settings_get_int(ips.entity, 'session_timeout') 
					limit 1
				) < ips_settings_get_int(ips.entity, 'ips_maximum_sessions'), 0, 1 << 5) | 
			0) from _Ips ips where ips.deleted is null && ips.address = $ip limit 1
		), 0) end);
	end;^
	
	drop procedure if exists sessions_get_guest;^
	create procedure sessions_get_guest(
		in `$session` text,
		out $error varchar(512),
		out $id integer,
		out `$user` varchar(32),
		out $timeout integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_get_guest';
		declare $machine_session integer default sessions_get_machine();
		declare $parameters text default json_set('{}', 
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $procedure_id integer;
		declare $has_error bool;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set(ifnull($session_id, $machine_session), `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		call sessions_validate_full(`$session`, null, $error, $session_id, $ip_id, $user_agent_id);
	
		if errors_get_bit($error, 1) || errors_get_bit($error, 4) then begin
			set $session_id := sessions_get_machine();
			set $error := '';
			if $ip_id is null then 
				set $ip_id := (select entity from _Ips where deleted is null && address = settings_get('local_ip', false) limit 1);
			end if;
			if $user_agent_id is null then 
				set $user_agent_id := (select id from UsersAgents where date_out is null && `data` = settings_get('default_user_agent', false) limit 1);
			end if;
		end;end if;
	
		set $has_error := errors_has($error);
		set $id := 0;
		
		if !$has_error then begin 
			
			declare $user_id integer;
			
			set `$user` := settings_get('guest_user', false);
			set $user_id := (select entity from _Users where nick = `$user` limit 1);
		
			insert into Sessions(`user`) values($user_id);
			set $id := last_insert_id();
			insert into SessionsIps values();
			insert into SessionsIpsData(entity, `session`, ip, ok) values 
				(last_insert_id(), $id, $ip_id, true);
			insert into SessionsUsersAgents values();
			insert into SessionsUsersAgentsData(entity, `session`, user_agent, ok) values 
				(last_insert_id(), $id, $user_agent_id, true);
			
			set $parameters := json_set($parameters, '$.id', $id);
		
			set $timeout := users_settings_get_int($user_id, 'session_timeout');
			
		end;end if;
	
		select permission from SessionsPermissionsView where !$has_error && `session` = $id;
		
		call logs_set(ifnull($session_id, $machine_session), `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 0);
		
	end;^
	
	drop procedure if exists sessions_update;^
	create procedure sessions_update(
		in `$session` text,
		out $error varchar(512),
		out `$user` varchar(32),
		out $timeout integer, 
		out $reset_password bool 
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_update';
		declare $parameters text default '{}';
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $has_error bool;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		call sessions_validate_full(`$session`, null, $error, $session_id, $ip_id, $user_agent_id);
		set $has_error := errors_has($error);
	
		if !$has_error then begin 

			update Sessions set date_last := now() where id = $session_id;
		
			select users.nick, ifnull(users.password, '') = '' 
			into `$user`, $reset_password 
			from _Users users
			join Sessions sessions on users.entity = sessions.`user`
			where sessions.id = $session_id
			limit 1;

			set $timeout := users_settings_get_int((select `user` from Sessions where id = $session_id limit 1), 'session_timeout');

		end;elseif errors_get_bit($error, 26) then 
			update Sessions set date_out = now() where id = $session_id;
		end if;
	
		select permission from SessionsPermissionsView where !$has_error && `session` = $session_id;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 2);
	
	end;^
	
	drop procedure if exists sessions_close;^
	create procedure sessions_close(
		in `$session` text,
		out $error varchar(512)
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_close';
		declare $parameters text default '{}';
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		call sessions_validate_full(`$session`, null, $error, $session_id, $ip_id, $user_agent_id);
	
		if !errors_has($error) then 
			update Sessions set date_out := now() where id = $session_id;
		end if;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 0);
	
	end;^
	
	drop function if exists sessions_register_validate;^
	create function sessions_register_validate(
		$nick varchar(32),
		$email varchar(256),
		$password varchar(256) 
	) returns varchar(512) begin 
		
		declare $error varchar(512) default '';
	
		set $error := errors_set($error, (case 
			when $nick is null then 1 << 0
			when $nick = '' then 1 << 1
		else ifnull((
			select 1 << 2 from _Users where nick = $nick limit 1
		), ifnull((
			select 1 << 3 from _Groups where name = $nick limit 1
		), 0)) end), 0, 0);
	
		set $error := errors_set($error, (case 
			when $email is null then 1 << 0
			when $email = '' then 1 << 1
			when $email not like '%@%.%' then 1 << 2
		else if(ifnull(settings_get_bool('mail_allow_reuse', false), false), 0, ifnull((
			select 1 << 3 from Emails where deleted is null && address = $email limit 1
		), 0)) end), 4, 0);
	
		set $error := errors_set($error, password_validate($password), 8, 0);
	
		return $error;
	end;^
	
	drop procedure if exists sessions_register;^
	create procedure sessions_register(
		in `$session` text,
		in $nick varchar(32),
		in $email varchar(256),
		in $password varchar(256),
		out $error varchar(512),
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_register';
		declare $parameters text default json_set('{}', 
			'$.nick', $nick,
			'$.email', $email,
			-- '$.password', $password,
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		set $id := 0;
		call sessions_validate_full(`$session`, null, $error, $session_id, $ip_id, $user_agent_id);
		set $error := errors_join($error, sessions_register_validate($nick, $email, $password), 30, 0);
	
		if !errors_has($error) then begin 
			
			declare $email_id integer default (select id from Emails where deleted is null && address = $email limit 1);
			declare $procedure_id integer;
			declare $group_id integer;
		
			call procedures_get(`$procedure`, `$database`, $procedure_id);
		
			insert into Users values();
			set $id := last_insert_id();
			insert into UsersData(`procedure`, `session`, entity, nick, password, `accessible`) values
				($procedure_id, $session_id, $id, $nick, sessions_password_hash($password), settings_get_bool('allow_any_new_user', false));
			
			if $email_id is null then begin 
				insert into Emails(`procedure`, `session`, address) values
					($procedure_id, $session_id, $email);
				set $email_id := last_insert_id();
			end;end if;
		
			if (select 1 from _UsersEmails where belongs && `user` = $id && email = $email_id limit 1) is null then begin 
				insert into UsersEmails values();
				insert into UsersEmailsData(`procedure`, `session`, entity, email, `user`, belongs) values
					($procedure_id, $session_id, last_insert_id(), $email_id, $id, true);
			end;end if;
		
			call users_groups_create($procedure_id, $session_id, $nick, $id);
		
			if (
				select 1 
				from _UsersGroups users_groups
				join _Groups groups on groups.entity = users_groups.`group` 
				join _Users users on users.entity = users_groups.`user`
				where 
					users_groups.belongs && 
					users.`accessible` && 
					groups.deleted is null && 
					users.deleted is null && 
					groups.name = 'administrators'
				limit 1
			) is null then begin 
				insert into UsersGroups values();
				insert into UsersGroupsData(`procedure`, `session`, entity, `user`, `group`, belongs) values 
					($procedure_id, $session_id, last_insert_id(), $id, (
						select entity from _Groups where deleted is null && name = 'administrators' limit 1
					), true);
			end;end if;
		
		end;end if;
	
		commit;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 0);
		
	end;^
	
	drop function if exists sessions_login_validate;^
	create function sessions_login_validate(
		`$user` varchar(256),
		$password varchar(256),
		$ip_id integer
	) returns varchar(512) begin 
		
		declare $error varchar(512);
	
		set $error := errors_set('', (
			((case 
				when `$user` is null then 1 << 0
				when `$user` = '' then 1 << 1
			else 0 end) << 0) |
			((case 
				when $password is null then 1 << 0
				when $password = '' then 1 << 1
			else 0 end) << 2) |
		0), 0, 0);
	
		if !errors_has($error) then begin 
			
			declare $user_id integer;
			declare $ips_times integer default ips_settings_get_int($ip_id, 'ip_login_bad_times');
			declare $users_times integer;
		
			drop temporary table if exists ips_tries_temporary_anp;
			drop temporary table if exists users_tries_temporary_anp;
		
			create temporary table ips_tries_temporary_anp(
				id integer not null auto_increment,
				i integer not null,
				date_in datetime not null,
				constraint ips_tries_temporary_anp_id primary key(id)
			);
		
			create temporary table users_tries_temporary_anp(
				id integer not null auto_increment,
				i integer not null,
				date_in datetime not null,
				constraint ips_tries_temporary_anp_id primary key(id)
			);
	
			if `$user` like '%@%.%' then 
				set $user_id := (
					select users_emails.`user`
					from Emails emails 
					join _UsersEmails users_emails on emails.id = users_emails.email 
					where 
						users_emails.belongs && 
						emails.deleted is null && 
						emails.address = `$user` 
					limit 1
				);
			else 
				set $user_id := (select entity from _Users where deleted is null && nick = `$user` limit 1);
			end if;
		
			set $users_times := ifnull(users_settings_get_int($user_id, 'user_login_bad_times'), 0);
		
			insert into ips_tries_temporary_anp(i, date_in) 
				select i, date_in from LoginsTries where date_out is null && ip = $ip_id limit $ips_times;
			insert into users_tries_temporary_anp(i, date_in) 
				select i, date_in from LoginsTries where date_out is null && `user` = $user_id limit $users_times;
		
			set $error := errors_set($error, (
				((case 
					when $user_id is null then 1 << 0
					when $user_id < 1 then 1 << 1 
				else ifnull((
					select (
						if(deleted is null, 0, 1 << 3) |
						if((
							0 not in (select i from users_tries_temporary_anp) && 
							(select count(1) = $users_times from users_tries_temporary_anp limit 1) && 
							timestampdiff(second, (select date_in from users_tries_temporary_anp order by i desc limit 1), now()) <= users_settings_get_int($user_id, 'user_lock_time')
						), 1 << 4, (
							if(`accessible`, 0, 1 << 0) | 
							ifnull((
								select 1 << 1 
								from _UsersGroups users_groups
								join _Groups groups on users_groups.`group` = groups.entity 
								where 
									users_groups.belongs && 
									groups.deleted is null && 
									users_groups.`user` = $user_id && 
									groups.name = 'banned' 
								limit 1
							), 0) |
							if((
								select count(1) from Sessions where 
									`user` = $user_id && 
									date_out is null && 
									timestampdiff(second, date_last, now()) < users_settings_get_int($user_id, 'session_timeout')
								limit 1
							) < users_settings_get_int($user_id, 'users_maximum_sessions'), 0, 1 << 2) | 
						0) << 5) |
					0) from _Users where entity = $user_id limit 1
				), 1 << 2) end) << 0) |
				((case 
					when $ip_id is null then 1 << 0 
					when $ip_id < 1 then 1 << 1
				else ifnull((
					select (
						if(deleted is null, 0, 1 << 3) |
						if((
							0 not in (select i from users_tries_temporary_anp) && 
							(select count(1) = $users_times from users_tries_temporary_anp limit 1) && 
							timestampdiff(second, (select date_in from users_tries_temporary_anp order by i desc limit 1), now()) <= users_settings_get_int($user_id, 'user_lock_time')
						), 1 << 4, (
							if(`accessible`, 0, 1 << 0) | 
							ifnull((
								select 1 << 1 
								from _IpsGroups ips_groups
								join _Groups groups on ips_groups.`group` = groups.entity 
								where 
									ips_groups.belongs && 
									groups.deleted is null && 
									ips_groups.ip = $ip_id && 
									groups.name = 'banned' 
								limit 1
							), 0) |
							if((
								select count(1) from SessionsFullView where 
									ip_id = $ip_id && 
									date_out is null && 
									timestampdiff(second, date_last, now()) < ips_settings_get_int($ip_id, 'session_timeout')
								limit 1
							) < ips_settings_get_int($ip_id, 'ips_maximum_sessions'), 0, 1 << 2) | 
						0) << 5) |
					0) from _Ips where entity = $ip_id limit 1
				), 1 << 2) end) << 8) |
			0), 4, 0);
		
			if !errors_has($error) then 
				set $error := errors_set($error, (
					select if(`accessible` && (ifnull(password, '') = '' || binary password = binary sessions_password_hash($password)), 0, 1 << 0) from _Users where entity = $user_id limit 1
				), 20, 0);
			end if;
		
			drop temporary table ips_tries_temporary_anp;
			drop temporary table users_tries_temporary_anp;
		
		end;end if;

		return $error;
	end;^
	
	drop procedure if exists sessions_login;^
	create procedure sessions_login(
		in `$session` text, 
		in `$user` varchar(256),
		in $password varchar(256),
		out $error varchar(512),
		out $id integer,
		out $user_nick varchar(32),
		out $timeout integer, 
		out $reset_password bool
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_login';
		declare $parameters text default json_set('{}', 
			'$.user', `$user`,
			-- '$.password', $password,
			'$.id', 0
		);
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $has_errors bit;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text; 
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
		
		set $id := 0;
		set $reset_password := false;
	
		call sessions_validate_full(`$session`, 'guest', $error, $session_id, $ip_id, $user_agent_id);
		set $has_errors := errors_has($error);
	
		if !$has_errors then begin 
			
			set $error := errors_join($error, sessions_login_validate(`$user`, $password, $ip_id), 30, 0);
			set $has_errors := errors_has($error);
		
			if !$has_errors then begin 
				
				declare $user_id integer;
			
				if `$user` like '%@%.%' then 
					select users_emails.`user`, users.nick 
					into $user_id, $user_nick 
					from Emails emails
					join _UsersEmails users_emails on emails.id = users_emails.email 
					join _Users users on users.entity = users_emails.`user` 
					where 
						users_emails.belongs && 
						emails.deleted is null && 
						emails.address = `$user` 
					limit 1;
				else 
					select entity, nick into $user_id, $user_nick from _Users where deleted is null && nick = `$user` limit 1;
				end if;
					
				insert into Sessions(`user`) values($user_id);
				set $id := last_insert_id();
				insert into SessionsIps values();
				insert into SessionsIpsData(entity, `session`, ip, ok) values 
					(last_insert_id(), $id, $ip_id, true);
				insert into SessionsUsersAgents values();
				insert into SessionsUsersAgentsData(entity, `session`, user_agent, ok) values 
					(last_insert_id(), $id, $user_agent_id, true);
			
				update Sessions set date_out := now() where id = $session_id;

				set $reset_password := (select `accessible` && ifnull(password, '') = '' from _Users where entity = $user_id limit 1);
				set $timeout := users_settings_get_int($user_id, 'session_timeout');
			
			end;end if;
		
		end;else 
			set $error := errors_join('', $error, 1 << 0, 0);
		end if;
	
		select permission from SessionsPermissionsView where !$has_errors && `session` = $id;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 0);
		
	end;^

	drop procedure if exists sessions_autocloser;^
	create procedure sessions_autocloser(
		in `$session` text,
		out $error varchar(512)
	) begin 
	
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_autocloser';
		declare $machine_session integer default settings_get_int('machine_session', false);
		declare $unknown_session integer default settings_get_int('unknown_session', false);
		declare $session_id integer default ifnull(sessions_get_id(`$session`), $unknown_session);
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, '{}', @message, @_status, @_code);
		end;
	
		set $error := '';
	
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;

		if errors_has($error) = 0 then 
			update Sessions set date_out := now() where 
				date_out is null and 
				id not in ($machine_session, $unknown_session) and 
				timestampdiff(second, date_last, now()) > users_settings_get_int(`user`, 'session_timeout');
		end if;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, '{}', 1 << 0);
	
	end;^
	
	drop procedure if exists sessions_errors_messages_set;^
	create procedure sessions_errors_messages_set(
		in `$session` text
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_errors_messages_set';
		declare $session_id integer default ifnull(sessions_get_id(`$session`), sessions_get_machine());
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $error varchar(512) default '';
		
		select ip_id, user_agent_id into $ip_id, $user_agent_id from SessionsFullView where id = $session_id limit 1;
	
		insert into ErrorsTemporaryGroups(i, name) values 
			(6, 'sessions_get_guest'),
			(5, 'sessions_update'),
			(4, 'sessions_close'),
			(3, 'sessions_register'),
			(2, 'sessions_login'), 
			(1, 'sessions_autocloser');
		insert into ErrorsTemporaryMessages(`group`, message) values 
			(2, 'user_null'),
			(2, 'user_empty'),
			(2, 'password_null'),
			(2, 'password_empty'),
			(2, 'user_id_null'),
			(2, 'user_id_bad'),
			(2, 'user_not_exists'),
			(2, 'user_deleted'),
			(2, 'user_antinuke'),
			(2, 'user_unaccessible'),
			(2, 'user_banned'),
			(2, 'user_limit_sessions'),
			(2, 'ip_id_null'),
			(2, 'ip_id_bad'),
			(2, 'ip_not_exists'),
			(2, 'ip_deleted'),
			(2, 'ip_antinuke'),
			(2, 'ip_unaccessible'),
			(2, 'ip_banned'),
			(2, 'ip_limit_sessions'),
			(2, 'password_wrong'),
			(3, 'nick_null'),
			(3, 'nick_empty'),
			(3, 'nick_exists'),
			(3, 'nick_group_exists'),
			(3, 'email_null'),
			(3, 'email_empty'),
			(3, 'email_bad_format'),
			(3, 'email_exists'),
			(3, 'password_null'),
			(3, 'password_empty'),
			(3, 'password_minimum_length'),
			(3, 'password_needs_lower'),
			(3, 'password_needs_upper'),
			(3, 'password_needs_digit'),
			(3, 'password_needs_special');
		
		call errors_messages_set(`$session`);
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, '{}', 1 << 1);
		
	end;^
	
	call sessions_errors_messages_set(null);^

	drop function if exists sessions_reset_password_validate;^
	create function sessions_reset_password_validate(
		$session_id integer, 
		$password varchar(512)
	) returns integer begin 

		declare $error integer default password_validate($password);

		if !$error then begin 

			declare $user_id integer default (select `user` from Sessions where id = $session_id limit 1);
			declare $differents integer default ifnull(settings_get_int('users_passwords_differents', false), 3);

			set $error := if((
				select 1 from (
					select distinct password from UsersData where entity = $user_id order by id desc limit $differents
				) passwords where passwords.password = sessions_password_hash($password) limit 1
			) is null , 0, 1 << 7);

		end;end if;

		return $error;
	end;^

	drop procedure if exists sessions_reset_password;^
	create procedure sessions_reset_password(
		in `$session` text,
		in $password varchar(256),
		out $error varchar(512)
	) begin 
	
		declare `$database` varchar(64) default settings_get('database_anp', false);
		declare `$procedure` varchar(64) default 'sessions_reset_password';
		declare $procedure_id integer;
		declare $session_id integer;
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $parameters text default '{}';
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, '{}', @message, @_status, @_code);
		end;

		call sessions_validate_full(`$session`, 'user', $error, $session_id, $ip_id, $user_agent_id);

		set $parameters := json_set($parameters, '$.password', $password);

		if !errors_has($error) then begin 

			set $error := errors_set($error, sessions_reset_password_validate($session_id, $password), 30, 0);

			if !errors_has($error) then begin 

				declare $data_id integer default (select `data` from _Users where entity = (select `user` from Sessions where id = $session_id limit 1) limit 1);

				call procedures_get(`$database`, `$procedure`, $procedure_id);

				insert into UsersData(`procedure`, `session`, entity, nick, password, `accessible`) 
				select $procedure_id, $session_id, entity, nick, sessions_password_hash($password), `accessible` from UsersData where id = $data_id limit 1;

			end;end if;

		end;end if;
		
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 1 << 0);

	end;^
	
delimiter ;