use AnP;

delimiter ;^

	drop function if exists settings_single_add_validate;^
	create function settings_single_add_validate(
		`$type` varchar(64),
		$name varchar(64),
		`$range` integer,
		$value text,
		$description text
	) returns integer begin 
		return (
			((case 
				when `$type` is null then 1 << 0 
				when `$type` = '' then 1 << 1 
			else 0 end) << 0) | 
			((case 
				when $name is null then 1 << 0 
				when $name = '' then 1 << 1 
			else ifnull((
				select 1 << 2 from _Settings where deleted is null && name = $name limit 1
			), 0) end) << 2) | 
			((case 
				when `$range` is null then 1 << 0 
				when `$range` < 0 then 1 << 1 
			else 0 end) << 5) | 
			(if($value is null, 1 << 0, 0) << 7) | 
		0);
	end;^

	drop procedure if exists settings_single_add;^
	create procedure settings_single_add(
		in $session_id integer, 
		in `$type` varchar(64),
		in $name varchar(64),
		in `$range` integer,
		in $value text,
		in $description text,
		out $error varchar(512),
		out $id integer
	) begin 
		
		declare `$database` varchar(64) default settings_get('database_anp2', false);
		declare `$procedure` varchar(64) default 'settings_single_add';
		declare $ip_id integer;
		declare $user_agent_id integer;
		declare $parameters text default json_set('{}',
			'$.type', `$type`,
			'$.name', $name,
			'$.range', `$range`,
			'$.value', $value,
			'$.description', $description,
			'$.entity', 0, 
			'$.id', 0
		);
		declare exit handler for sqlexception begin
			get diagnostics condition 1
				@_status = returned_sqlstate,
				@_code = mysql_errno,
				@message = message_text;
			rollback;
			set $error := errors_set($error, 1 << 0, 0, 0);
			call exceptions_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, @message, @_status, @_code);
		end;
	
		start transaction;
	
		select ip_id, user_agent_id 
		into $ip_id, $user_agent_id 
		from SessionsFullView 
		where id = $session_id;
	
		set $id := 0;
		set $error := errors_set('', (
			(if($session_id = AnP.sessions_get_machine(), 0, sessions_single_validate($session_id)) << 0) | 
			(settings_single_add_validate(`$type`, $name, `$range`, $value, $description) << 4) | 
		0), 0, 0);
	
		if !errors_has($error) then begin 
			
			declare $type_id integer default (select entity from _SettingsTypes where deleted is null && name = `$type` limit 1);
			declare $procedure_id integer;
		
			call procedures_get(`$database`, `$procedure`, $procedure_id);
		
			if $type_id is null then begin 
				insert into SettingsTypes values();
				set $type_id := last_insert_id();
				insert into SettingsTypesData(`$session`, `$procedure`, entity, name) values 
					($session_id, $procedure_id, $type_id, `$type`);
			end;end if;
		
			insert into Settings values();
			set $id := last_insert_id();
			insert into SettingsData(`session`, `procedure`, entity, `type`, name, `range`, value, description) values 
				($session_id, $procedure_id, $id, $type_id, $name, `$range`, $value, $description);
		
		end;end if;
	
		commit;
	
		call logs_set($session_id, `$database`, `$procedure`, $ip_id, $user_agent_id, $error, $parameters, 0);
		
	end;^

delimiter ; 