<?php

    namespace AnP;

    abstract class Controller{

        protected $anp;
        protected $data_connector;
        private $parent;
        private $entity;
        private static $default_key = null;
        protected $key;

        public function __construct($anp, $parent = null){

            $this->anp = $anp;
            $this->parent = $parent;
            $this->data_connector = $anp->get_data_connector();

        }

        protected function request($names, $default = null){
            return $this->anp->routes->get($names, $default);
        }

        protected function default_key(){
            return self::$default_key === null ? (self::$default_key = $this->anp->settings->get(["default_connection_key", "default_connection", "connection_key"])) : self::$default_key;
        }

        private function process($data, $code, $message){

            if(is_callable($data)){
                try{
                    $data = $data();
                    ($code === null || $code < 200) && ($code = $this->anp->settings->get(["default_http_code", "http_code"]));
                    !$message && ($message = $this->anp->settings->get(["default_http_message", "http_message"]));
                }catch(\Exception $exception){
                    $data = null;
                    try{
                        $this->anp->connections->get($this->default_key())->query();
                        $code = $this->anp->settings->get(["default_http_code_exception", "http_code_exception"]);
                    }catch(\Exception $subexception){
                        $code = $this->anp->settings->get(["default_http_code_no_tracer", "http_code_no_tracer"]);
                    };
                };
            }else
                $code === null && ($code = $this->anp->settings->get(["default_http_code", "http_code"]));

            return [$data, $code, $message];
        }

        protected function json($data = null, $code = null, $message = null){

            list($data, $code, $message) = $this->process($data, $code, $message);

            $json = [
                "code" => $code,
                "ok" => ($code >= 200 && $code < 300) || in_array($code, [301, 302, 304]),
                "i18n" => "error_" . $code . "_message",
                "message" => $message === null ? $this->anp->i18n->get("error_" . $code . "_message") : $message
            ];
            $data && ($json["content"] = $data);

            header("Content-Type: application/json; charset=utf-8");
            echo $this->anp->json_encode($json);
            exit(0);

        }

        protected function json_response($data){

            $code = $this->anp->settings->get(["default_http_code", "http_code"]);
            $message = isset($data["message"]) ? $data["message"] : "error_" . $code . "_message";

            foreach([
                "null" => fn() => $data === null,
                "not" => fn() => !\AnP::is_dictionary($data),
                "not_error" => fn() => !isset($data["error"]),
                "error_null" => fn() => $data["error"] === null,
                "invalid_error" => fn() => !is_string($data["error"]),
                "sql_exception" => fn() => strlen($data["error"]) && $this->anp->errors->to_integer($data["error"][0]) & (1 << 3)
            ] as $key => $check)
                if($check()){
                    list($code, $message) = [$this->anp->settings->get(["default_http_code_" . $key, "http_code_json_" . $key]), "error_500_json_" . $key];
                    break;
                };
            // $data["message"] = $message;

            $this->json($data, $code);

        }

        protected function execute(){

            $results = null;

            // try{
            
                $connectors = $this->anp->get_data_connectors_names();

                if(isset($connectors[$this->data_connector])){

                    $connector = $connectors[$this->data_connector];

                    if(!isset($this->$connector)){
                        
                        $current_class = get_class($this->parent);
                        $original = "PHP/Controllers/" . $this->data_connector . "/" . preg_replace('/\\\\/', ".", $current_class) . "." . $this->data_connector . ".php";
                        $path = $this->anp->get_absolute_path($original);

                        if($path){

                            $class = '\\' . $current_class . '\\' . $this->data_connector;

                            include $path;

                            $this->$connector = new $class($this->anp);

                        };

                    };

                    if(isset($this->$connector))
                        $results = $this->$connector;
                };

            // }catch(\Exception $exception){
            //     $this->set_traces($exception, 0, false);
            // };

            return $results;
        }

        protected function list_query($query, $variables = null){

            $connection = $this->anp->connections->get($this->key);

            if(!$connection)
                return ["error" => 1 << 0];

            $response = $connection->query($query, array_merge([
                "page" => $this->request("page"),
                "items_per_page" => $this->request("items_per_page"),
                "where" => $this->request("where"),
                "order_by" => $this->request("order_by")
            ], is_array($variables) ? $variables : []));

            if(count($response["tables"]) > 1){
                $response["entity_data"] = empty($response["tables"][1]) ? [] : $response["tables"][1][0];
                array_pop($response["tables"]);
            };
            
            return $response;
        }

        protected function file_save($id, $name, $data, $path, $hashes_required = null){

            $ok = true;
            $hashes = null;
            $full_path = null;

            try{

                if(!$id){

                    $full_path = $this->anp->path_format($path . "/" . $name);
                    $directories = preg_split('/[\\\\\/]+/', $full_path);
                    $slash = $this->anp->get_slash();
                    $subpath = "";

                    array_pop($directories);
                    foreach($directories as $i => $directory){
                        if(!$directory)
                            continue;
                        if(!$i && $directory[1] == ":")
                            $subpath = $directory;
                        else 
                            if(!file_exists($subpath .= $slash . $directory))
                                mkdir($subpath);
                    };

                    if(file_exists($full_path)){

                        $date = date("YmdHis");
                        $subname = preg_replace('/^((.+)(\.[^\\\\\/\.]+)|(.+))$/', "$2$4." . $date . "$3", $full_path);

                        if(file_exists($subname)){

                            $i = 0;

                            while(file_exists($subname_i = preg_replace('/^((.+)(\.[^\\\\\/\.]+)|(.+))$/', "$2$4." . (++ $i) . "$3", $subname)));
                            $subname = $subname_i;

                        };

                        $full_path = $subname;

                    };

                    $opened = fopen($full_path, "wb");
                    fwrite($opened, base64_decode($data));
                    fclose($opened);

                    $allow_hashes = \hash_algos();
                    $hashes = "";

                    if(!is_array(is_string($hashes_required) ? $hashes_required = [$hashes_required] : $hashes_required))
                        $hashes_required = $this->anp->settings->get("files_hashes");
                        
                    if($hashes_required)
                        foreach((is_array($hashes_required) ? $hashes_required : [$hashes_required]) as $hash)
                            if(in_array($hash = strtolower($hash), $allow_hashes))
                                $hashes .= ($hashes ? "," : "") . $hash . ":" . hash($hash, $data);

                };

            }catch(\Exception $exception){
                $ok = false;
            };

            return [$ok, $full_path, $hashes];
        }

        protected function set_traces($exception, $error = 0, $print = true){

            $query = null;
            $variables = $this->anp->routes->get_variables();
            $for_delete = [];
            $key_connection = null;

            foreach($variables as $key => $value)
                if(
                    preg_match('/^(.+)_data$/', $key, $matches, \PREG_OFFSET_CAPTURE) && 
                    isset($variables[$matches[1][0] . "_name"]) && 
                    isset($variables[$matches[1][0] . "_mime"]) && 
                    isset($variables[$matches[1][0] . "_last_modified"]) 
                )
                    $for_delete[] = $key;

            foreach($for_delete as $key)
                unset($variables[$key]);

            switch($this->data_connector){
                case "MariaDB":
                    $key_connection = "AnP_MariaDB";
                    $query = "call traces_set({session}, {project}, {path}, {error}, {parameters}, {trace}, {message}, {line}, @error, @id);";
                break;
                case "SQLServer":
                    $key_connection = "AnP_SQLServer";
                    $query = "execute dbo.traces_set {session}, {project}, {path}, {error}, {parameters}, {trace}, {message}, {line}, @error output, @id output";
                break;
                default:
                break;
            };

            $this->anp->connections->get($key_connection)->query($query, [
                "project" => $this->anp->settings->get("project_name"),
                "path" => $exception->getFile(),
                "error" => $this->anp->errors->to_string($error), 
                "parameters" => \json_encode($variables),
                "trace" => $exception->getTraceAsString(),
                "message" => $exception->getMessage(),
                "line" => $exception->getLine()
            ]);

            if($print)
                $this->json_response([
                    "error" => $this->anp->errors->set("", 1, 0, 0), 
                    "message" => "exception_php"
                ]);

        }

    };
