<?php

    namespace AnP;

    class Errors{

        private $anp;
        private $alphabet;

        public function __construct($anp, $input = []){

            $this->anp = $anp;

        }

        private function get_alphabet(){
            return $this->alphabet ? $this->alphabet : ($this->alphabet = $this->anp->settings->get("base64_alphabet"));
        }

        public function to_array($code, $length = 0){

            $array = [];

            if(is_string($code)){

                $alphabet = $this->get_alphabet();
                $l = $c = strlen($code);

                $length && ($length = ceil($length / 6)) < $l && ($l = $length);

                for($i = 0; $i < $l; $i ++)
                    $array[] = $i < $c ? strpos($alphabet, $code[$i]) : 0;

            }elseif(is_integer($code)){

                // $l = ($l = ceil(log($code, 2))) < $length ? $length : $l;
                while($code){
                    $array[] = $code & 0x3F;
                    $code = ($code >> 6) & PHP_INT_MAX;
                };

                // for($i = 0; $i < $l; $i += 6)
                //     $array[] = ($code >>> $i) & 0x3F;

            }elseif(is_array($code)){

                $array = $code;
                $length /= 6;

                while(count($code) < $length)
                    $code[] = $alphabet[0];

            };

            return $array;
        }

        public function to_array_binary($code){

            foreach(($code = $this->to_array($code)) as $i => $value)
                $code[$i] = substr("000000" . base_convert($value, 10, 2), -6);

            return $code;
        }

        public function to_integer($code, $length = 0){

            $integer = 0;

            if(is_string($code)){

                $alphabet = $this->get_alphabet();
                $l = strlen($code);

                for($i = 0; $i < $l; $i ++){
                    try{
                        $integer |= strpos($alphabet, $code[$i]) << $i * 6;
                    }catch(\Exception $exception){
                        break;
                    };
                };

            }elseif(is_integer($code))
                $integer = $code;
            elseif(is_array($code))
                foreach($code as $i => $hexa){
                    try{
                        $integer |= $hexa << $i * 6;
                    }catch(\Exception $exception){
                        break;
                    };
                };

            return $integer;
        }

        public function to_string($code, $length = 0){

            $string = "";
            $alphabet = $this->get_alphabet();

            if(is_string($code)){

                $string = $code;
                $length /= 6;

                while(strlen($string) < $length)
                    $string .= $alphabet[0];

            }elseif(is_integer($code)){
                while($code){
                    $string .= $alphabet[$code & 0x3F];
                    $code = ($code >> 6) & PHP_INT_MAX;
                };
                // for($i = 0; $code >> $i || $i < $length; $i += 6)
                //     $string .= $alphabet[$code >> $i & 0x3F];
            }elseif(is_array($code)){
                foreach($code as $i => $hexa)
                    $string .= $alphabet[$hexa];
                while(strlen($string) < $length)
                    $string .= $alphabet[0];
            };

            return $string ? $string : substr($alphabet, 0, 1);
        }

        public function has($code){
            if(is_string($code))
                return $code && !preg_match('/^' . $this->get_alphabet()[0] . '+$/', $code);
            if(is_integer($code))
                return !!$code;
            if(is_array($code)){
                foreach($code as $hexa)
                    if($hexa)
                        return true;
                return false;
            };
            return null;
        }

        public function type($code){
            foreach(["string", "integer", "array"] as $method)
                if(("is_" . $method)($code))
                    return $method;
            return "unknown";
        }

        public function to_unknown($code){
            return $code;
        }

        public function compact($code){
            if(is_string($code)){

                $alphabet = $this->get_alphabet();

                while(strlen($code) && $code[strlen($code) - 1] == $alphabet[0])
                    $code = substr($code, 0, strlen($code) - 1);

                return $code ? $code : $alphabet[0];
            };
            if(is_array($code)){
                while(!$code[count($code) - 1])
                    array_pop($code);
                return count($code) ? $code : [0];
            };
            return $code ? $code : 0;
        }

        public function bitwise($code, $bits){

            if(!$bits || !$this->has($code))
                return $code;

            ($reverse = $bits < 0) && ($bits *= -1);
            $start = $bits / 6 >> 0;
            $rest = $bits % 6;
            $type = "to_" . $this->type($code);

            if($reverse){

                $code = substr($this->to_string($code), $start);

                if($rest){
                    if($code){

                        $results = [];
                        $l = count($code = $this->to_array($code)) - 1;
                        $r = 6 - $rest;
                        $block = ~-(1 << $rest);

                        for($i = 0; $i < $l; $i ++)
                            $code[$i] = ($code[$i] >> $rest) | (($code[$i + 1] & $block) << $r);
                        $code[$l] >>= $rest;

                    }else
                        $code = [0];

                };

            }else{

                $code = $this->to_array($code);

                if($rest){

                    $block = [~-(1 << ($r = 6 - $rest)), ~-(1 << $rest) << $r];
                    $l = count($code) - 1;

                    ($last = ($code[$l] & $block[1]) >> $r) && ($code[$l + 1] = $last);
                    for($i = $l; $i > 0; $i --)
                        $code[$i] = (($code[$i] & $block[0]) << $rest) | ($code[$i - 1] >> $r);
                    $code[0] = ($code[0] & $block[0]) << $rest;

                };

                for($i = 0; $i < $start; $i ++)
                    array_unshift($code, 0);

            };

            return $this->$type($code);
        }

        public function set($code, $error, $bit = 0, $length = 0){

            $error = $this->to_array($error);
            $bit && ($error = $this->bitwise($error, $bit));
            $i = $bit / 6 >> 0;
            $code = $this->to_array($code);

            if($length){

                $ampersand = [];
                $start = $bit % 6;
                $m = ($length + $start) / 6 >> 0;
                $end = ($start + $length) % 6;

                for($j = 0; $j < $m || $j < 1; $j ++)
                    $ampersand[] = 0;
                // print_r([$ampersand, $length, $start, $m]);
                $ampersand[0] |= ~-(1 << $start);
                $end && ($ampersand[] = ~-(1 << (6 - $end)) << $end);

                foreach($ampersand as $j => $hexa){
                    if(!isset($code[$j + $i]))
                        break;
                    $code[$j + $i] &= $hexa;
                };

            };

            if($this->has($error)){

                $l = count($error);

                while(count($code) <= $i)
                    $code[] = 0;
                // print_r([$this->to_string($code)]);

                for(; $i < $l; $i ++)
                    $code[$i] = (isset($code[$i]) ? $code[$i] : 0) | $error[$i];

            };

            return $this->compact($this->to_string($code));
        }

        public function get($code, $messages = [], $variables = []){

            $results = [];
            $array = $this->to_array($code);
            $l = count($messages);

            foreach($array as $i => $hexa)
                if($hexa)
                    for($j = 0; $j < 6; $j ++)
                        $hexa >> $j & 1 && ($results[] = [
                            "i18n" => $i18n = ($k = $i * 6 + $j) < $l ? $messages[$k] : "error_" . $k,
                            "text" => $this->anp->i18n->get($i18n, $variables),
                            "i" => $k
                        ]);

            return [
                "code" => $this->to_string($code),
                "messages" => $results,
                "variables" => $variables
            ];
        }

    };
