<?php

    namespace AnP;

    class I18N{

        private $anp;
        private $started = false;
        private $i18n = [];
        private $language = null;
        private $default_language = null;

        public function __construct($anp, $input = []){

            $this->anp = $anp;

        }

        public function add($inputs, $overwrite = null){
            if(!$inputs)
                return;

            !is_bool($overwrite) && ($overwrite = $this->get(["i18n_overwrite", "overwrite"]));

            foreach(!is_array($inputs) || \AnP::is_dictionary($inputs) ? [$inputs] : $inputs as $input){
                if(!$input)
                    continue;
                if(is_string($input)){
                    if(!($json = $this->anp->json_decode($input, false)) && !($json = $this->anp->load_json($input, false)))
                        continue;
                    $input = $json;
                };
                if(!is_array($input))
                    continue;
                if(\AnP::is_dictionary($input)){
                    foreach($input as $language => $sentences){
                        !isset($this->i18n[$language]) && ($this->i18n[$language] = []);
                        foreach($sentences as $key => $sentence)
                            ($overwrite || !isset($this->i18n[$language][$key])) && ($this->i18n[$language][$key] = $sentence);
                    };
                }else
                    $this->add($input, $overwrite);
            };

        }

        public function get_language(){
            return "" + $this->language;
        }

        public function change_language($language){

            $language && $this->language != $language && isset($this->i18n[$language]) && ($this->language = $language);

        }

        public function start(){

            if($this->started)
                return;
            $this->started = true;

            foreach(["default_i18n_files", "i18n_files"] as $key)
                $this->add($this->anp->settings->get($key), true);
            $this->change_language($this->default_language = $this->get(["default_language", "language"]));

        }

        public function default_text($default = null){
            return $default ?? $this->anp->settings->get("default_text");
        }

        private function get_sentence($names, $default){
            if(!$names)
                return $this->default_text($default);

            !is_array($names) && ($names = [$names]);

            foreach(array_merge([$this->language, $this->default_language], array_keys($this->i18n)) as $language)
                if($language && isset($this->i18n[$language]))
                    foreach($names as $name)
                        if($name && isset($this->i18n[$language][$name]))
                            return $this->i18n[$language][$name];
            foreach($names as $name)
                if($name)
                    return $name;
            return $this->default_text($default);
        }

        public function get($names = null, $variables = null, $default = null){
            return \AnP::string_variables($this->get_sentence($names, $default), $variables);
        }

    };
