<?php

    namespace AnP;

    class Routes{

        private $anp = null;
        private $started = false;
        private $routes = [""];
        private $method_name = null;
        private $anp_variables = [];
        private $variables = [];
        private $routes_root = [];

        public function __construct($anp, $input = []){

            $this->anp = $anp;

        }

        public function add($inputs, $overwrite = null){
            if(!$inputs)
                return;

            !is_bool($overwrite) && ($overwrite = $this->anp->settings->get(["routes_overwrite", "overwrite"]));

            foreach(is_array($inputs) ? $inputs : [$inputs] as $input){
                is_string($input) && ($json = $this->anp->json_decode($input)) || ($json = $this->anp->load_json($input)) && ($input = $json);
                // $this->anp->print([$json, $input, $this->anp->load_json($input)]);
                if(is_array($input))
                    $this->add($input, $overwrite);
                elseif(is_string($input) && preg_match('/^' .
                    '(([^\:]+)?\:)?' . // Method.
                    '(\/[^\s]*)\s+' . // Route.
                    '(' .
                        '([a-zA-Z0-9_]+)@([a-zA-Z0-9_]+)|' . // Controller.
                        '([^\s\'"]+)|"(([^\\\\"]+|\\\\.)*)"|\'(([^\\\\\']+|\\\\.)*)\'' . // File Relative Path.
                    ')' .
                    '(\s+([^\s]+))?' . // Permissions.
                '$/m', $input, $matches, \PREG_OFFSET_CAPTURE)){
                    while(count($matches) < 14)
                        $matches[] = ["", -1];
                    $variables = [null];
                    !isset($this->routes[$method = strtolower($matches[2][0] ? $matches[2][0] : $this->anp->settings->get(["default_route_method", "default_method", "route_method", "method"]))]) && ($this->routes[$method] = []);
                    $path = $matches[10][0] ? $matches[10][0] : ($matches[8][0] ? $matches[8][0] : ($matches[7][0] ? $matches[7][0] : null));
                    $this->routes[$method][] = [
                        "pattern" => '/^' . preg_replace_callback('/\\\\{(([^\{\}\\\\]+|\\\\[^\{\}])+)\\\\\}/', function($values) use(&$variables){
                            $variables[] = preg_replace('/\\\\(.)/', "$1", $values[1]);
                            return '([^\/]+)';
                        }, \AnP::string_to_re(preg_replace('/^(.*)\/?$/', "$1/", $matches[3][0]))) . ($path ? '(.+)' : '?') . '$/m',
                        "action" => $matches[5][0],
                        "controller" => $matches[6][0],
                        "path" => $matches[5][0] ? null : preg_replace('/^(.+)\/?$/', "$1/", $path) . "{path}",
                        "permissions" => $matches[13][0] ? preg_split('/[\s\.,\|\/\\\\;\:\-]+/', $matches[13][0]) : [],
                        "variables" => array_merge($variables, $path ? ["path"] : [])
                    ];
                };
            };

        }

        public function add_routes_root($routes_root){
            foreach($routes_root ? (is_array($routes_root) ? $routes_root : [$routes_root]) : [] as $route)
                is_string($route) && !in_array($route, $this->routes_root) && ($this->routes_root[] = $route);
        }

        public function start(){

            if($this->started)
                return;
            $this->started = true;

            $key = $this->anp->settings->get(["ajax_variable_name", "variable_name"]);
            $values = null;

            foreach([$_POST, $_GET] as $set)
                if(isset($set[$key])){
                    $values = $set[$key];
                    break;
                };

            switch($this->anp->settings->get(["ajax_send_mode", "send_mode", "mode"])){
                case "base64":
                    try{
                        $this->anp_variables = json_decode(\AnP::url_decode(base64_decode($values)), true);
                    }catch(\Exception $exception){};
                break;
                case "json":
                    try{
                        $this->anp_variables = @json_decode(\AnP::url_decode($values), true);
                    }catch(\Exception $exception){};
                break;
                case "normal":
                default:
                break;
            };

            foreach(["default_routes_files", "routes_files"] as $key){
                // $this->anp->print([$key => $this->anp->settings->get($key)]);
                $this->add($this->anp->settings->get($key), true);
            };
            foreach(["default_routes_root", "routes_root"] as $key)
                $this->add_routes_root($this->anp->settings->get($key));
            $this->method_name = $this->anp->settings->get("method_name");

            // $this->anp->print($this->routes["get"]);

        }

        public function response_json($content = null, $code = null, $message = null){

            header("content-type: application/json");
            echo $this->anp->json_encode([
                "ok" => ($code >= 200 && $code < 300) || in_array($code, [301, 302, 304]),
                "code" => $code,
                "message" => $message,
                "content" => $content
            ]);
            exit(0);

        }

        public function go($url = null, $method = null){

            !$url && ($url = preg_replace('/^([^;?]+)(\?.*)?$/', "$1", $_SERVER["REQUEST_URI"]));

            foreach($this->routes_root as $root)
                if(is_string($root) && substr($url, 0, $l = strlen($root)) == $root){
                    $url = substr($url, $l);
                    break;
                };

            !$method && ($method = strtolower($_SERVER["REQUEST_METHOD"])) == "post" && $this->method_name && isset($_POST[$this->method_name]) && ($method = $_POST[$this->method_name]);

            $test = [];
            foreach($this->routes as $submethod => $subroutes)
                if(is_array($subroutes))
                    foreach($subroutes as $route)
                        $test[] = [$submethod, $route["pattern"]];

            if(isset($this->routes[$method])){

                $ok = false;

                foreach($this->routes[$method] as $route){
                    if($ok = preg_match($route["pattern"], $url, $matches, \PREG_OFFSET_CAPTURE)){

                        $variables = [];
                        $l = count($route["variables"]);

                        for($i = 1; $i < $l; $i ++)
                            $variables[$route["variables"][$i]] = $matches[$i][0];

                        if($route["path"]){
                            $this->anp->print([\AnP::string_variables($route["path"], $variables), $route]);
                        }else{
                            if(!($controller = $this->anp->controllers->get($route["controller"])))
                                $this->response_json(["url" => $url], $this->anp->settings->get("route_error_controller"), "route_error_controller");
                            if(!method_exists($controller, $action = $route["action"]))
                                $this->response_json(["url" => $url], $this->anp->settings->get("route_error_method"), "route_error_method");
                            try{
                                $controller->$action($this->variables = $variables);
                            }catch(\Exception $exception){
                                $this->response_json([
                                    "url" => $url,
                                    "exception" => $exception
                                ], $this->anp->settings->get("route_error_exception"), "route_error_exception");
                            };
                        };

                        break;
                    };
                };

                $this->response_json(["url" => $url], $this->anp->settings->get("route_error_not_found"), "route_error_not_found");

            };

            $this->response_json(["url" => $url], $this->anp->settings->get("route_error_not_method"), "route_error_not_method");

        }

        public function get($names, $default = null){

            !is_array($names) && ($names = [$names]);

            foreach([
                "variables" => $this->variables, 
                "uri_variables" => $this->anp_variables, 
                "session" => $_SESSION, 
                "cookie" => $_COOKIE, 
                "post" => $_POST, 
                "get" => $_GET
            ] as $key => $set)
                if(is_array($set))
                    foreach($names as $name)
                        if($name && isset($set[$name]))
                            return is_string($set[$name]) && in_array($key, ["cookie", "post", "get"]) ? urldecode($set[$name]) : $set[$name];
            return $default;
        }

        public function get_variables(){

            $variables = [];
            $anp_key = $this->anp->settings->get(["ajax_variable_name", "variable_name"]);

            foreach(array_filter([
                $this->variables, $this->anp_variables, $_POST, $_GET
            ], fn($item) => \AnP::is_dictionary($item)) as $set)
                foreach($set as $key => $value)
                    if(!in_array($key, [$anp_key, "q", "__method"]) && !isset($variables[$key]))
                        $variables[$key] = $this->get($key);

            return $variables;
        }

    };
