<?php

    namespace AnP;

    class Sessions{

        private $anp;
        private $key = null;
        private $started = false;
        private $exists = true;

        public function __construct($anp, $input = []){

            $this->anp = $anp;

        }

        private static function permissions($permissions){

            $results = [];

            foreach($permissions as $permission)
                $results[] = $permission["permission"];

            return $results;
        }

        public static function get_data($response){
            return [
                "ip" => \AnP::get_ip(),
                "user" => $response["error"] ? null : $response["variables"]["user"],
                "connection_error" => $response["error"],
                "login_error" => $response["error"] ? null : $response["variables"]["error"],
                "error" => 0,
                "id" => $response["error"] ? null : $response["variables"]["id"],
                "permissions" => $response["error"] ? [] : self::permissions($response["tables"][0]),
                "timeout" => $response["error"] ? null : $response["variables"]["timeout"],
                "reset_password" => $response["variables"]["reset_password"]
            ];
        }

        private function get_guest(){

            $date = date("YmdHis");
            $response = $this->anp->controllers->get("sessions")->guest();

            $response["variables"]["reset_password"] = false;

            $_SESSION[$this->key] = array_merge(self::get_data($response), [
                "from" => $date,
                "last" => $date,
                "update_error" => null
            ]);
            // print_r($_SESSION[$this->key]);

        }

        public function start(){

            // session_destroy();
            // exit();

            if(
                !($this->exists = !empty($_SESSION[$this->key ? $this->key : ($this->key = $this->anp->settings->get("session_key"))])) || 
                !$_SESSION[$this->key]["id"]
            )
                $this->get_guest();

        }

        public function get(){return isset($_SESSION[$this->key]) ? $_SESSION[$this->key]["id"] : null;}

        public function get_permissions(){return $_SESSION[$this->key]["permissions"];}

        public function update(){

            $response = $this->anp->controllers->get("sessions")->update();

            if(!$response["error"] && $this->anp->errors->has($response["variables"]["error"])){
                $this->close();
                $this->get_guest();
                return;
            };

            foreach([
                "user" => $response["error"] ? null : $response["variables"]["user"],
                "last" => date("YmdHis"),
                "connection_error" => $response["error"],
                "update_error" => $response["error"] ? null : $response["variables"]["error"],
                "permissions" => $response["error"] ? [] : self::permissions($response["tables"][0]),
                "timeout" => $response["error"] ? null : $response["variables"]["timeout"],
                "reset_password" => $response["variables"]["reset_password"]
            ] as $key => $value)
                $_SESSION[$this->key][$key] = $value;

        }

        public function get_full(){

            $this->exists && $this->update();
            
            return isset($_SESSION[$this->key]) ? $_SESSION[$this->key] : null;
        }

        public function close(){
            
            unset($_SESSION[$this->key]);

        }

        public function set_login($response){

            if(!$response["error"] && !$this->anp->errors->has($response["variables"]["error"])){    

                $date = date("YmdHis");
                
                $_SESSION[$this->key] = array_merge(self::get_data($response), [
                    "from" => $date,
                    "last" => $date,
                    "update_error" => null
                ]);

            };

        }

    };
