<?php

    namespace AnP;

    class Settings{

        private $anp;
        private static $default_settings = [
            "nulls" => false,
            "default_value" => null,
            "show_exceptions" => true,
            "autostart" => true,
            "root_path" => __DIR__ . "/..",
            "default_settings_files" => "/JSON/AnP.php.settings.json",
            "show_prints" => false
        ];
        private $started = false;
        private $settings = [];
        private $input = null;
        private $secrets = [];

        public function default($default = null, $nulls = null){
            return $default !== null || ($nulls ?? $this->get("nulls", null, false, false)) ? $default : $this->get(["default_value", "default"], null, null, true);
        }

        public function get($names = null, $inputs = null, $default = null, $nulls = null){
            if(!$names)
                return $this->default($default, $nulls);

            !is_array($names) && ($names = [$names]);
            $inputs = is_array($inputs) ? (\AnP::is_dictionary($inputs) ? [$inputs] : $inputs) : [];
            !is_bool($nulls) && ($nulls = $this->get("nulls", null, false, false));

            foreach(array_merge(
                is_array($inputs) ? (\AnP::is_dictionary($inputs) ? [$inputs] : $inputs) : [],
                [$this->input, $this->secrets, $this->settings, self::$default_settings]
            ) as $input)
                if(\AnP::is_dictionary($input))
                    foreach($names as $name)
                        if($name && isset($input[$name]) && ($nulls || $input[$name] !== null))
                            return $input[$name];
            return $this->default($default, $nulls);
        }

        private function add_secrets($secrets){
            if(!$secrets)
                return;

            foreach(is_array($secrets) && !\AnP::is_dictionary($secrets) ? $secrets : [$secrets] as $secret){
                if(!$secret)
                    continue;
                if(is_string($secret)){
                    if(!($json = $this->anp->json_decode($secret, false)) && !($json = $this->anp->load_json($secret, false)))
                        continue;
                    $secret = $json;
                };
                if(!is_array($secret))
                    continue;
                if(\AnP::is_dictionary($secret))
                    foreach($secret as $key => $value)
                        $this->secrets[$key] = $value;
                else
                    $this->add_secrets($secret);
            };

        }

        public function add($inputs, $overwrite = null){
            if(!$inputs)
                return;

            !is_bool($overwrite) && ($overwrite = $this->get(["settings_overwrite", "overwrite"]));

            foreach(!is_array($inputs) || \AnP::is_dictionary($inputs) ? [$inputs] : $inputs as $input){
                if(!$input)
                    continue;
                if(is_string($input)){
                    if(!($json = $this->anp->json_decode($input, false)) && !($json = $this->anp->load_json($input, false)))
                        continue;
                    $input = $json;
                };
                if(!is_array($input))
                    continue;
                if(\AnP::is_dictionary($input))
                    foreach($input as $key => $value)
                        ($overwrite || !isset($this->settings[$key])) && ($this->settings[$key] = $value);
                else
                    $this->add($input, $overwrite);
            };

        }

        public function start(){

            if($this->started)
                return;
            $this->started = true;

            foreach(["default_settings_files", "settings_files"] as $key){
                // $this->anp->print([$key, $this->get($key)]);
                // print_r([$key, $this->get($key)]);
                $this->add($this->get($key), true);
            };
            foreach(["default_secrets_files", "secrets_files"] as $key)
                $this->add_secrets($this->get($key));
            $this->anp->add_root_paths($this->get(["root_paths", "root_path"]));

            // print_r(["custom" => $this->anp->load_json("/JSON/VehiculosPoliciaLocal.php.settings.custom.json")]);

            // $this->anp->print($this->input);
            // $this->anp->print($this->settings);
            // $this->anp->print($this->secrets);

            // $this->anp->print(["secrets", $this->get("secrets_files"), $this->anp->get_absolute_path($this->get("secrets_files"))]);

        }

        public function __construct($anp, $input = null){

            $this->anp = $anp;
            $this->input = $input;
            
            $this->anp->add_root_paths($this->get(["root_paths", "root_path"]));

        }

    };
