<?php !isset($_SESSION) && session_start();

    class AnP{

        private $started = false;
        private static $string_to_re_special = [
            " " => "s",
            "\t" => "t",
            "\r" => "r",
            "\n" => "n"
        ];
        private $slash = "";
        private $root_paths = ["", __DIR__];
        private $data_connector = null;
        private $data_connectors_names = null;

        public $settings;
        public $i18n;
        public $traces;
        public $errors;
        public $controllers;
        public $routes;
        public $sessions;

        public static function is_dictionary($value){
            return is_array($value) && array_values($value) != $value;
        }

        public function json_encode($data, $default = null){
            try{
                if($results = json_encode($data))
                    return $results;
            }catch(\Exception $exception){
                if($this->settings->get(["show_exceptions_json_encode", "show_exceptions_json", "show_exceptions"]))
                    echo $exception->getMessage();
                $this->settings->get(["save_exceptions_json_encode", "save_exceptions_json", "save_exceptions"]) && $this->traces && $this->traces->save('\AnP->json_encode', $exception);
            };
            return $this->settings->default($default);
        }

        public function json_decode($data, $default = null){
            try{
                if(is_array($results = @json_decode($data, true)))
                    return $results;
            }catch(\Exception $exception){
                if($this->settings->get(["show_exceptions_json_decode", "show_exceptions_json", "show_exceptions"]))
                    echo $exception->getMessage();
                $this->settings->get(["save_exceptions_json_decode", "save_exceptions_json", "save_exceptions"]) && $this->traces && $this->traces->save('\AnP->json_decode', $exception);
            };
            return $this->settings->default($default);
        }

        public function path_format($path){
            return preg_replace('/[\\\\\/]+/', $this->slash, $path);
        }

        public function add_root_paths($new_roots){
            foreach($new_roots ? (is_array($new_roots) ? $new_roots : [$new_roots]) : [] as $root){
                if(is_array($root))
                    $this->add_root_paths($root);
                elseif(is_string($root) && !in_array($root = $this->path_format($root), $this->root_paths))
                    $this->root_paths[] = $root;
            };
        }

        public function get_absolute_path($path){
            // $this->print($this->root_paths);
            foreach($this->root_paths as $root){
                // $this->print([
                //     "absolute" => $this->path_format($root . "/" . $path),
                //     "exist" => file_exists($this->path_format($root . "/" . $path))
                // ]);
                if(file_exists($file = $this->path_format($root . "/" . $path)))
                    return $file;
            };
            return null;
        }

        public function load_file($path, $default = null){
            // $this->print([$path, $this->root_paths]);
            foreach($this->root_paths as $root)
                if(file_exists($file = $root . "/" . $path))
                    return file_get_contents($file);
            return $this->path_format($this->settings->default($default));
        }

        public function load_json($path, $default = null){
            return $this->json_decode($this->load_file($path, $default), $default);
        }

        public static function string_variables($string, $variables = null, $default = null){
            if(!is_array($variables))
                return $string;

            self::is_dictionary($variables) ? $variables = [$variables] : $variables;

            return preg_replace_callback('/\{([^\{\}]+)\}/', function($values) use($variables, $default){
                foreach($variables as $set)
                    if(isset($set[$values[1]]))
                        return strval($set[$values[1]]);
                return strval($default ?? $values[0]);
            }, strval($string));
        }

        public function print($data = null){
            if(!$this->settings->get("show_prints"))
                return;

            $trace = debug_backtrace();
            $root = count($trace) == 1;

            print_r([
                "class" => "\\" . ($root ? "" : $trace[1]["class"]),
                "method" => $root ? null : $trace[1]["function"],
                "line" => $trace[0]["line"],
                "file" => $trace[0]["file"],
                "data" => $data
            ]);
            echo "<br />\n";

        }

        public function start(){

            if($this->started)
                return;
            $this->started = true;

            $this->add_root_paths($this->settings->get(["root_paths", "root_path"]));

            foreach(["settings", "i18n", "traces", "connections", "controllers", "routes", "sessions"] as $name)
                $this->$name && method_exists($this->$name, "start") && $this->$name->start();

            $this->routes && $this->routes->go();

        }

        public function __construct($input = []){

            $this->slash = strpos(__DIR__, "\\") ? "\\" : "/";

            foreach([
                "Settings" => "settings",
                "I18N" => "i18n",
                "Traces" => "traces",
                "Errors" => "errors",
                "Connections" => "connections",
                "Controllers" => "controllers",
                "Routes" => "routes",
                "sessions" => "sessions"
            ] as $class => $name)
                class_exists($class = '\AnP\\' . $class) && ($this->$name = new $class($this, $input));

            $this->settings && $this->settings->get("autostart") && $this->start();

        }

        public static function string_to_re($string){
            return preg_replace_callback('/[\r\n\t\.\-\,\:\?\!\+\*\^\$\\\\\/\(\)\{\}\[\]\&\|\@\#\~]/', function($values){
                return "\\" . (isset(\Anp::$string_to_re_special[$values[0]]) ? \Anp::$string_to_re_special[$values[0]] : $values[0]);
            }, $string);
        }

        public static function utf8_encode($string){
            try{
                return utf8_encode($string);
            }catch(Exception $exception){};
            return null;
        }

        public static function get_ip(){
            foreach(["HTTP_X_REAL_IP", "HTTP_CLIENT_IP", "HTTP_X_FORWARDED_FOR", "REMOTE_ADDR"] as $key){
                if(!empty($_SERVER[$key]))
                    return explode(",", $_SERVER[$key])[0];
                if($ips = getenv($key))
                    return explode(",", $ips)[0];
            };
            return null;
        }

        public function session_get(){return $this->sessions ? $this->sessions->get() : null;}

        public static function is_utf8($string){
            return is_string($string) && preg_match('//u', $string);
        }

        public static function url_encode($string){
            return urlencode(preg_replace_callback('/[\\\\\+ ]/', function($values){
                switch($values[0]){
                    case "+":
                        return "\\p";
                    case "\\":
                        return "\\\\";
                    case " ":
                        return "\\s";
                };
            }, $string));
        }

        public static function url_decode($string){
            return preg_replace_callback('/\\\\([\\\\ps])/', function($values){
                switch($values[1]){
                    case "p":
                        return "+";
                    case "\\":
                        return "\\";
                    case "s":
                        return " ";
                };
            }, urldecode($string));
        }

        public function get_data_connector(){
            return "" . ($this->data_connector ? $this->data_connector : $this->data_connector = $this->settings->get("data_connector"));
        }

        public function get_data_connectors_names(){
            return $this->data_connectors_names ? $this->data_connectors_names : $this->data_connectors_names = $this->settings->get("data_connectors_names");
        }

        public function get_slash(){
            return $this->slash;
        }

    };
