<?php

    class AnPBuilder{

        const default_settings = [
            "nulls" => false,
            "default_value" => null,
            "root" => "/^\\/app(\\/(index\\.php)?)?\\/?$/",
            "language" => "en",
            "language_code" => "en_US",
            "direction" => "ltr",
            "key" => "anp",
            "url" => "https://anp.k3y.pw",
            "name" => "AnP",
            "license_text" => "© 2020-2021 CopyLeft. GPLv3",
            "license_link" => "https://www.gnu.org/licenses/gpl-3.0.txt",
            "license_icon" => "https://www.gnu.org/graphics/gplv3-88x31.png",
            "description" => "A project made with AnP Framework.",
            "keywords" => "anp,attach,play,attach&play,framework,web,application",
            "html_file" => __DIR__ . "/../HTML/app.html",
            "authors" => "KyMAN",
            "script" => '<script data-type="text/javascript" data-language="ECMAScript 2015" charset="utf-8"></script>',
            "extension" => "png",
            "domain" => "k3y.pw"
        ];
        private $input = null;

        public static function is_dictionary($value){
            return is_array($value) && array_values($value) != $value;
        }

        public static function is_array($value){
            return is_array($value) && array_values($value) == $value;
        }

        public function default_value($default = null, $nulls = null){
            return $default !== null || (is_bool($nulls) ? $nulls : $this->settings("nulls", null, false, false)) ? $default : $this->settings("default_value", null, null, true);
        }

        public function settings($names = null, $inputs = null, $default = null, $nulls = null){
            if($names){

                !is_array($names) && ($names = [$names]);
                !is_bool($nulls) && ($nulls = $this->settings("nulls", null, false, false));

                foreach(array_merge($inputs ? (self::is_array($inputs) ? $inputs : [$inputs]) : [], [$this->input, self::default_settings]) as $input)
                    if(self::is_dictionary($input))
                        foreach($names as $name)
                            if(is_string($name) && isset($input[$name]) && ($nulls || $input[$name] !== null))
                                return $input[$name];
            };
            return $this->default_value($default, $nulls);
        }

        public static function string_variables($string, $inputs = null, $default = null){

            $inputs && !self::is_array($inputs) && ($inputs = [$inputs]);

            return preg_replace_callback('/\{([^\{\}]+)\}/', function($values) use($inputs, $default){
                if($inputs)
                    foreach($inputs as $input)
                        if(self::is_dictionary($input) && isset($input[$values[1]]))
                            return $input[$values[1]];
                return $default !== null ? $default : $values[0];
            }, $string);
        }

        public function print_index_html(){

            ($kstats = $this->settings("kstats")) && ($kstats = ("\n\n" . 
                "        " . '<script data-type="text/javascript" data-language="ECMAScript 2015" src="https://kstats.k3y.pw/ecma/KStats.ecma.js" data-crossorigin="anonymous" charset="utf-8"></script>' . "\n\n" . 
                "        " . '<script data-type="text/javascript" data-language="ECMAScript 2015" charset="utf-8">' . "\n\n" . 
                "            " . 'kstats = new KStats({url : "https://kstats.k3y.pw/api/' . $kstats . '/{session}/json/set"});' . "\n\n" . 
                "        " . '</script>'
            ));

            header("content-type: text/html;charset=utf-8");
            header("charset: utf-8");
            echo self::string_variables(file_get_contents($this->settings("html_file")), [
                "language" => $this->settings("language"),
                "language_code" => $this->settings("language_code"),
                "direction" => $this->settings("direction"),
                "key" => $this->settings("key"),
                "url" => $this->settings("url"),
                "name" => $this->settings("name"),
                "license_text" => $this->settings("license_text"),
                "license_link" => $this->settings("license_link"),
                "license_icon" => $this->settings("license_icon"),
                "authors" => $this->settings("authors"),
                "since" => $this->settings("since"),
                "version" => $this->settings("version"),
                "description" => $this->settings("description"),
                "keywords" => $this->settings("keywords"),
                "script" => $this->settings("script"),
                "kstats" => $kstats ?? "",
                "extension" => $this->settings("extension"),
                "domain" => $this->settings("domain")
            ]);
            exit(0);

        }

        public function __construct($input = null){

            $this->input = $input;

            foreach(["since", "version"] as $key)
                !isset($this->input[$key]) && ($this->input[$key] = date("Ymd"));

            if($_SERVER["REQUEST_METHOD"] == "POST" || !preg_match($this->settings("root"), preg_replace('/^((\/[^\?]*)+)?\/(\?.*)$/', "$1", $_SERVER["REQUEST_URI"]))){
                include __DIR__ . "/include.php";
                is_callable($callback = $this->settings("callback")) && $callback();
            }else
                $this->print_index_html();

        }

        public static function get_host(){
            return "http" . (isset($_SERVER["HTTPS"]) && strtolower($_SERVER["HTTPS"]) == "on" ? "s" : "") . "://" . $_SERVER["HTTP_HOST"];
        }

        public static function get_cache($callback){

            ob_start();

            is_callable($callback) && $callback();

            return ob_get_clean();
        }

    };
