<?php

    namespace AnP\Controllers;

    class Sessions extends \AnP\Controller{

        public function __construct($anp){
            parent::__construct($anp, $this);

            $this->entity = "Sessions";
            $this->class = get_class($this);

        }

        public function guest(){
            return $this->execute()->guest();
        }

        public function update(){
            return $this->execute()->update();
        }

        private function session_share_data($data = null){
            $share = ["ip" => \AnP::get_ip()];

            if(!$data)
                $data = $this->anp->sessions->get_full();

            foreach(["error", "id", "permissions", "user", "timeout"] as $key)
                $share[$key] = $data[$key];

            return $share;
        }

        public function get(){

            try{
                $this->json($this->session_share_data());
            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function close(){

            try{
               
                $response = $this->execute()->close();

                $this->anp->sessions->close();

                $this->json($response);

            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function register(){

            try{

                $response = $this->execute()->register();

                $this->json([
                    "connection_error" => $response["error"],
                    "variables" => [
                        "error" => $response["error"] ? null : $response["variables"]["error"]
                    ]
                ]);

            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function login(){

            try{

                $this->anp->sessions->set_login($response = $this->execute()->login());

                $this->json(\AnP\Sessions::get_data($response));

            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function user_agent(){

            try{

                $this->json($_SERVER["HTTP_USER_AGENT"]);

            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function get_languages_id(){

            $this->execute()->get_languages_id();
            
        }

        public function reset_password(){

            try{
                $this->execute()->reset_password();
            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function logs(){

            try{
                $this->execute()->logs();
            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function exceptions(){

            try{
                $this->execute()->exceptions();
            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

        public function traces(){

            try{
                $this->execute()->traces();
            }catch(\Exception $exception){
                $this->set_traces($exception);
            };

        }

    };
