<?php 

    namespace AnP\Controllers\Languages;

    class MariaDB extends \AnP\Controller{

        protected $key = "AnP_MariaDB";

        public function list(){
            
            $this->json_response($this->list_query((
                "call languages_list({session}, {page}, {items_per_page}, {where}, {order_by}, @items, @pages, @results, @error);"
            )));

        }

        public function select(){
            
            $this->json_response($this->list_query((
                "call languages_select_list({session}, {page}, {items_per_page}, {where}, {order_by}, @items, @pages, @results, @error);"
            )));

        }

        public function basics(){
            
            $this->json_response($this->list_query((
                "call languages_get_basics({session}, @error);"
            )));

        }

        public function add(){ 

            list($flag_ok, $flag_path, $flag_hashes) = $this->file_save($this->request("flag_id"), $this->request("flag_name"), $this->request("flag_data"), $this->anp->settings->get(["languages_flag_path", "uploaded_files_path"]));

            if(!$flag_ok){
                $this->json_response(["error" => $this->anp->errors->set("", 1 << 0, 0, 0), "error_message" => "verifiers_failed"]);
                return;
            };

            $this->json_response($this->anp->connections->get($this->key)->query((
                "call languages_add({session}, {country}, {name}, {key}, {code}, {flag_id}, {flag_name}, {flag_mime}, {flag_size}, {flag_last_modified}, {flag_path}, {flag_hashes}, {description}, @error, @id);"
            ), [
                "country" => $this->request("country"),
                "name" => $this->request("name"),
                "key" => $this->request("key"),
                "code" => $this->request("code"),
                "flag_id" => $this->request("flag_id"),
                "flag_name" => $this->request("flag_name"),
                "flag_mime" => $this->request("flag_mime"),
                "flag_size" => $this->request("flag_size"),
                "flag_last_modified" => $this->request("flag_last_modified"),
                "flag_path" => $flag_path,
                "flag_hashes" => $flag_hashes,
                "description" => $this->request("description")
            ])); 

        }

        public function load(){

            $results = $this->anp->connections->get($this->key)->query((
                "call languages_load({session}, {id}, @error);"
            ), ["id" => $this->request("id")]);

            if(!$results["error"] && !$results["driver_error"] && $this->anp->errors->has($results["variables"]["error"]))
                $results["data"] = null;
            else{
                $results["data"] = $results["tables"][0][0]; 
            };
            
            if(isset($results["tables"]))
                unset($results["tables"]);
            
            $this->json_response($results);

        }

        public function edit(){ 

            list($flag_ok, $flag_path, $flag_hashes) = $this->file_save($this->request("flag_id"), $this->request("flag_name"), $this->request("flag_data"), $this->anp->settings->get(["languages_flag_path", "uploaded_files_path"]));

            if(!$flag_ok){
                $this->json_response(["error" => $this->anp->errors->set("", 1 << 0, 0, 0), "error_message" => "verifiers_failed"]);
                return;
            };

            $this->json_response($this->anp->connections->get($this->key)->query((
                "call languages_edit({session}, {id}, {country}, {name}, {key}, {code}, {flag_id}, {flag_name}, {flag_mime}, {flag_size}, {flag_last_modified}, {flag_path}, {flag_hashes}, {description}, @error);"
            ), [
                "id" => $this->request("id"),
                "country" => $this->request("country"),
                "name" => $this->request("name"),
                "key" => $this->request("key"),
                "code" => $this->request("code"),
                "flag_id" => $this->request("flag_id"),
                "flag_name" => $this->request("flag_name"),
                "flag_mime" => $this->request("flag_mime"),
                "flag_size" => $this->request("flag_size"),
                "flag_last_modified" => $this->request("flag_last_modified"),
                "flag_path" => $flag_path,
                "flag_hashes" => $flag_hashes,
                "description" => $this->request("description")
            ])); 

        }

        public function delete(){
            
            $this->json_response($this->anp->connections->get($this->key)->query((
                "call languages_delete({session}, {id}, @error);"
            ), ["id" => $this->request("id")]));

        }

        public function deleted(){
            
            $this->json_response($this->list_query((
                "call languages_deleted_list({session}, {page}, {items_per_page}, {where}, {order_by}, @items, @pages, @results, @error);"
            )));

        }

        public function undelete(){
            
            $this->json_response($this->anp->connections->get($this->key)->query((
                "call languages_undelete({session}, {id}, @error);"
            ), ["id" => $this->request("id")]));

        }

        public function history(){
            
            $this->json_response($this->list_query((
                "call languages_history({session}, {id}, {page}, {items_per_page}, {where}, {order_by}, @items, @pages, @results, @error);"
            ), ["id" => $this->request("id")]));

        }

        public function restore(){
            
            $this->json_response($this->anp->connections->get($this->key)->query((
                "call languages_restore({session}, {id}, @error);"
            ), ["id" => $this->request("id")]));

        }

        public function get_flag_file(){
            
            $response = $this->anp->connections->get($this->key)->query((
                "call files_get_path({session}, {file_id}, 'AnP.Languages.flag', {file_name}, '[\"administrador\"]', @error, @path, @type)"
            ), [
                "file_id" => $this->request("file_id"),
                "file_name" => $this->request("file_name")
            ]);

            return isset($response["variables"]) ? $response["variables"] : null;
        }

    };
