<?php

    namespace AnP\PDO;

    class MySQL{

        private $anp;
        private $connection_string = null;
        private $connection = null;
        private $error = 0;
        private $input = null;

        private function close(){

            if($this->connection){
                try{
                    $this->connection->rollback();
                }catch(\Exception $exception){
                    $this->error |= 1 << 1;
                };
            }else
                $this->error |= 1 << 2;

            $this->connection = null;

        }

        private function connect(){
            if($this->connection)
                return;

            try{

                $this->connection = new \PDO(\AnP::string_variables($this->anp->settings->get(["mysql_pdo_connection_string", "mysql_pdo_string_connection", "mariadb_pdo_connection_string", "mariadb_pdo_string_connection", "mysql_connection_string", "mysql_string_connection", "mariadb_connection_string", "mariadb_string_connection", "connection_string", "string_connection"], $this->input), [
                    "engine" => $this->anp->settings->get(["mysql_pdo_engine", "mariadb_pdo_engine", "mysql_engine", "mariadb_engine", "engine"], $this->input),
                    "database" => $this->anp->settings->get(["mysql_pdo_database", "mariadb_pdo_database", "mysql_database", "mariadb_database", "database"], $this->input),
                    "host" => $this->anp->settings->get(["mysql_pdo_host", "mariadb_pdo_host", "mysql_host", "mariadb_host", "host"], $this->input),
                    "port" => $this->anp->settings->get(["mysql_pdo_port", "mariadb_pdo_port", "mysql_port", "mariadb_port", "port"], $this->input),
                    "charset" => $this->anp->settings->get(["mysql_pdo_charset", "mariadb_pdo_charset", "mysql_charset", "mariadb_charset", "charset"], $this->input)
                ]), $this->anp->settings->get(["mysql_pdo_user", "mariadb_pdo_user", "mysql_user", "mariadb_user", "user"], $this->input), $this->anp->settings->get(["mysql_pdo_password", "mariadb_pdo_password", "mysql_password", "mariadb_password", "password"], $this->input));

                $this->connection->setAttribute(\PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION);

            }catch(\Exception $exception){
                $this->error |= 1 << 3;
                $this->anp->print(["message" => $exception->getMessage()]);
                $this->close();
            };

        }

        public function __construct($anp, $input = null){

            $this->anp = $anp;
            $this->input = $input;

            $this->anp->settings->get(["mysql_autoconnect", "mariadb_autoconnect", "autoconnect"], $this->input) && $this->connect();

        }

        public function __destruct(){

            $this->close();

        }

        public static function variable_value($variable, $fix_semicolons = true){
            return (
                $variable === null ? "null" : (
                is_string($variable) ? "'" . ($fix_semicolons ? preg_replace('/([\\\'])/', "\\\\$1", $variable) : $variable) . "'" : (
                is_bool($variable) ? $variable ? "true" : "false" : (
                is_array($variable) ? "null" : 
                $variable)))
            );
        }

        public static function to_datetime($date){

            $milliseconds = "" . ($date % 1000);

            return date("Y-m-d H:i:s", $date / 1000) . "." . substr("000" . $milliseconds, 3 - strlen($milliseconds)) . "000";
        }

        public static function to_varchar($value){
            return preg_replace_callback('/[\'\\\\]/', function($values){
                return "\\\\" . $values[0];
            }, $value);
        }

        public function query_variables($query, $variables = []){

            $outs = "";
            $names = [];

            !is_array($variables) && ($variables = []);

            $keys = array_keys($variables);

            if(in_array("page", $keys) && in_array("items_per_page", $keys) && in_array("where", $keys) && in_array("order_by", $keys)){

                if(!empty($variables["where"])){

                    $where = "";
                    $options = $this->anp->routes->get("search_options");
                    $case_sensitive = $options && isset($options["case_sensitive"]) && $options["case_sensitive"] ? "binary " : "";

                    if(is_string($variables["where"]))
                        $variables["where"] = strlen($variables["where"]) && $variables["where"][0] == "[" ? json_decode($variables["where"], true) : null;

                    if($variables["where"])
                        foreach($variables["where"] as $i => $condition){

                            $fragment = null;

                            if($i){

                                list($key, $value, $type) = $condition;

                                switch($value){
                                    case "NULL":
                                        $fragment = "`" . $key . "` is null";
                                    break;
                                    case "NOTNULL":
                                    case "NOT NULL":
                                    case "NOT_NULL":
                                        $fragment = "`" . $key . "` is not null";
                                    break;
                                    default:
                                        switch($type){
                                            case "string":
                                            case "text":
                                            case "varchar":
                                                $fragment = $case_sensitive . "`" . $key . "` like " . $case_sensitive . "'%" . self::to_varchar($value) . "%'";
                                            break;
                                            case "bool":
                                            case "boolean":
                                                $fragment = ($value ? "" : "!") . "`" . $key . "`";
                                            break;
                                            case "number":
                                            case "integer":
                                            case "session":
                                            case "float":
                                            case "size":
                                                $fragment = $value[0] == $value[1] ? "`" . $key . "`=" . self::to_varchar($value[0]) : (
                                                    ($value[0] === null ? "" : "`" . $key . "`>=" . self::to_varchar($value[0]) . (
                                                        $value[1] === null ? "" : "&&"
                                                    )) . 
                                                    ($value[1] === null ? "" : "`" . $key . "`<=" . self::to_varchar($value[1]))
                                                );
                                            break;
                                            case "datetime":
                                            case "calendar":
                                                if($value[0] == $value[1])
                                                    $fragment = "`" . $key . "`='" . self::to_datetime($value[0]) . "'";
                                                elseif($value[0] !== null && $value[1] !== null)
                                                    $fragment = "`" . $key . "` between '" . self::to_datetime($value[0]) . "' and '" . self::to_datetime($value[1]) . "'";
                                                elseif($value[0] !== null)
                                                    $fragment = "`" . $key . "`>='" . self::to_datetime($value[0]) . "'";
                                                elseif($value[1] !== null)
                                                    $fragment = "`" . $key . "`<='" . self::to_datetime($value[1]) . "'";
                                            break;
                                        };
                                    break;
                                };

                            }elseif($condition){

                                list($string, $fields) = $condition;
                                $fragment = "";

                                foreach($fields as $data){
                                    list($key, $type) = $data;
                                    $key_formatted;
                                    switch($type){
                                        case "number":
                                            $key_formatted = "concat('',`" . $key . "`)";
                                        break;
                                        case "string":
                                            $key_formatted = $case_sensitive . "`" . $key . "`";
                                        break;
                                    };
                                    $fragment .= ($fragment ? "||" : "") . $key_formatted . " like " . ($type == "string" ? $case_sensitive : "") . "'%" . self::to_varchar($string) . "%'";
                                };

                                $fragment = "(" . $fragment . ")";

                            };

                            if($fragment)
                                $where .= ($where ? "&&" : "") . $fragment;

                        };

                    $variables["where"] = $where;

                }else 
                    $variables["where"] = null;

                if(!empty($variables["order_by"])){

                    $order_by = "";

                    foreach($variables["order_by"] as $column){
                        list($key, $order) = $column;
                        $order_by .= ($order_by ? "," : "") . "`" . $key . "` " . ($order ? "desc" : "asc");
                    };

                    $variables["order_by"] = $order_by ? $order_by : null;

                }else 
                    $variables["order_by"] = null;

            };

            preg_replace_callback('/@([a-z_\d]+)/i', function($values) use(&$outs, &$names){
                if(in_array($values[1], $names))
                    return;
                $outs .= ($outs ? ", " : "") . "@" . $values[1] . " as '" . $values[1] . "'";
                $names[] = $values[1];
            }, $query);
            $outs && ($outs = "select " . $outs . ";");

            // print_r([$variables]);
            return preg_replace_callback('/\{([^\{\}]+)\}/', function($values) use($variables){
                return self::variable_value(!$variables || !isset($variables[$values[1]]) ? $this->anp->routes->get($values[1]) : $variables[$values[1]], in_array($values[1], ["where", "order_by"]));
            }, $query) . ($query[strlen($query) - 1] == ";" ? "" : ";") . $outs;
        }

        private function variable_type($value){
            if($value === null || ($lower = strtolower($value)) == "null")
                return null;
            if(($trimmed = trim($value)) && in_array($trimmed[0], ["[", "{"])){
                $json = null;
                try{
                    $json = $this->anp->json_decode($value);
                }catch(\Exception $exception){};
                if($json)
                    return $json;
            };
            if(preg_match('/^([0-9]+(\.([0-9]+)?)?|\.[0-9]+)$/', $value))
                return (float)$value;
            if(in_array($lower, ["true", "false"]))
                return $lower == "true";
            // return $this->utf8_encode($value);
            return $value;
        }

        public function query($query, $input = null){

            $this->connect();

            if($this->anp->errors->has(($results = [
                "tables" => [],
                "variables" => [],
                "error" => 0,
                "driver_error" => (
                    $this->error |
                    ($query ? 0 : 1 << 4) |
                    ($this->connection ? 0 : 1 << 5)
                )
            ])["driver_error"]))
                return $results;

            try{
                $this->connection->rollback();
            }catch(\Exception $exception){};
            $this->connection->beginTransaction();

            // try{

                $query_processed = $this->query_variables($query, array_merge([
                    "session" => (($session = $this->anp->sessions->get()) ? $session : "null") . "," . (\AnP::get_ip() ?? "null") . "," . ($_SERVER["HTTP_USER_AGENT"] ?? "null")
                    // "session" => $this->anp->session_get()
                ], is_array($input) ? $input : []));
                // print_r([$this->anp->sessions->get(), $query_processed]);
                $results["query"] = $query_processed;
                ($statement = $this->connection->prepare($query_processed))->execute();

                do{
                    try{
                        $table = [];
                        $fetch_all = $statement->fetchAll(\PDO::FETCH_ASSOC);
                        // $this->anp->print($fetch_all);
                        foreach($statement->rowCount() == 0 ? [] : $fetch_all as $new_row){
                            $row = [];
                            foreach($new_row as $key => $value){
                                $row[$key] = $this->variable_type($value);
                            };
                            $table[] = $row;
                        };
                        $results["tables"][] = $table;
                    }catch(\Exception $exception){};
                }while($statement->nextRowset());
                
                try{
                    $this->connection->commit();
                }catch(\Exception $exception){};

            // }catch(\Exception $exception){
            //     try{
            //         $this->connection->rollback();
            //     }catch(\exception $exception){};
            // };

            if(preg_match('/(,\s+?|\()\@/', $query)){
                $l = count($results["tables"]) - 1;
                foreach($results["tables"][$l][0] as $key => $value)
                    $results["variables"][$key] = $this->variable_type($value);
                unset($results["tables"][$l]);
            };

            return $results;
        }

    };
