AnPApp.ErrorsTests = function(anp_app, input){

    const self = this,
          form_values = {};
    let started = false;

    const checkbox_option = name => {
        return {
            i18n : name,
            name : name,
            type : "checkbox",
            checked : form_values[name] !== undefined ? form_values[name] : (form_values[name] = anp_app.settings_get(["errors_" + name, name])),
            onchange : "{object_name}.errors_tests.checkbox_changed(this, event);"
        };
    };

    const create_random_error_code = () => {

        let code = "";
        const new_hexa_probability = anp_app.settings_get("new_hexa_probability"),
              minimum_length = anp_app.settings_get("random_error_code_minimum_length"),
              alphabet = anp_app.settings_get("base64_alphabet").substr(0, 64),
              length = alphabet.length,
              maximum_length = anp_app.settings_get("random_error_code_maximum_length");

        for(let i = 0; i < maximum_length && (code.length < minimum_length || Math.random() < new_hexa_probability); i ++)
            code += alphabet[Math.random() * length >> 0];

        return code;
    };

    const create_random_code = () => form_values.code_type == "integer" ? Math.random() * 10 ** 8 >> 0 : create_random_error_code();

    const create_random_code_button = name => {return {i18n : name, type : "group", items : [
        {i18n : name, name : name, type : "text", value : form_values[name] || (form_values[name] = anp_app.settings_get(["code_error", "error", "code"]) || create_random_code()), onchange : "{object_name}.errors_tests.field_changed(this, event);"},
        {type : "button", i18n : "create_random", icon : "create_random", onclick : "{object_name}.errors_tests.create_random_code(this, event);"}
    ]}};

    const common_options = (action, custom_options) => {

        !form_values.code_type && (form_values.code_type = anp_app.settings_get(["code_error_type", "error_type"]));

        return anp_app.comp.forms.create({
            class : "errors-options",
            i18n : "errors_options",
            structure : [
                create_random_code_button("code"),
                {i18n : "code_type", name : "code_type", type : "selector", options : [
                    {i18n : "integer", value : "integer"},
                    {i18n : "string", value : "string"}
                ], value : form_values.code_type, onchange : "{object_name}.errors_tests.selector_changed(this, event);"},
                checkbox_option("js"),
                checkbox_option("ecma"),
                checkbox_option("php"),
                checkbox_option("python"),
                checkbox_option("sqlite"),
                checkbox_option("mysql"),
                checkbox_option("transact_sql")
            ].concat(custom_options || []),
            buttons : [
                {icon : "process", i18n : "process", onclick : "{object_name}.errors_tests." + action + "(this, event);"}
            ]
        });
    };

    const create_positions = () => [{
        type : "number",
        min : 1,
        max : 512,
        i18n : "positions",
        name : "positions",
        value : form_values.positions || (form_values.positions = anp_app.settings_get(["bitwise_positions", "positions"])),
        onchange : "{object_name}.errors_tests.field_changed(this, event);"
    }];

    const build_view = (name, fields) => (`
        <fieldset` + anp_app.attr.set({class : name.replace(/[^a-z0-9]+/gi, "-")}) + `>
            ` + anp_app.comp.i18n(name, "legend") + `
            ` + anp_app.comp.i18n(name + "_text", "p") + `
            ` + common_options(name, fields) + `
            <div` + anp_app.attr.set({class : "results"}) + `></div>
        </fieldset>
    `);

    const format_build = input => build_view("error_format");

    const bitwise_right_build = input => build_view("bitwise_right", create_positions());

    const bitwise_left_build = input => build_view("bitwise_left", create_positions());

    const binaries_build = input => build_view("error_binary");

    const random_position = () => Math.random() ** 3 * anp_app.settings_get("random_error_code_maximum_length") >> 0;

    this.create_random_position = (item, event) => item.parentNode.querySelector("[type=number]").value = random_position();

    const join_build = input => build_view("error_join", [
        create_random_code_button("join_code"),
        {type : "group", i18n : "position", items : [
            {type : "number", min : 0, max : 511, i18n : "position", name : "position", value : random_position()},
            {type : "button", i18n : "create_random", icon : "create_random", onclick : "{object_name}.errors_tests.create_random_position(this, event);"}
        ]}
    ]);

    const set_build = input => build_view("error_set", [
        create_random_code_button("join_code"),
        {type : "group", i18n : "position", items : [
            {type : "number", min : 0, max : 511, i18n : "position", name : "position", value : random_position()},
            {type : "button", i18n : "create_random", icon : "create_random", onclick : "{object_name}.errors_tests.create_random_position(this, event);"}
        ]},
        {type : "group", i18n : "length", items : [
            {type : "number", min : 0, max : 511, i18n : "length", name : "length", value : random_position()},
            {type : "button", i18n : "create_random", icon : "create_random", onclick : "{object_name}.errors_tests.create_random_position(this, event);"}
        ]}
    ]);

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp_app.comp.error_format = format_build;
        anp_app.comp.bitwise_right = bitwise_right_build;
        anp_app.comp.bitwise_left = bitwise_left_build;
        anp_app.comp.binaries = binaries_build;
        anp_app.comp.error_join = join_build;
        anp_app.comp.error_set = set_build;

        anp_app.is_function(callback) && callback();

    };

    const construct = () => {};

    this.selector_changed = (item, event) => {

        const name = item.querySelector("[type=text]").getAttribute("name"),
              value = anp_app.comp.selectors.get_value(item);

        form_values[name] = value;

        if(name == "code_type"){

            const code_field = anp_app.item_self.querySelector("form [name=code]");

            code_field.value = form_values.code = anp_app.errors["to_" + value](value == "integer" ? code_field.value : Number(code_field.value));

        };

    };

    this.checkbox_changed = (item, event) => form_values[item.getAttribute("name")] = item.checked;

    this.field_changed = (item, event) => form_values[item.getAttribute("name")] = item.value;

    this.create_random_code = (item, event) => form_values.code = item.parentNode.querySelector("[type=text]").value = create_random_code();

    const get_code = values => {

        const code = values.code || create_random_error_code();

        return values.code_type == "integer" ? anp_app.errors.to_string(Number(code)) : code;
    };

    this.row_unselected = (item, event) => anp_app.item_self.querySelectorAll("fieldset.bitwise-right .results tr[data-selected=true]").forEach(row => row.setAttribute("data-selected", false));

    this.row_selected = (item, event) => {

        self.row_unselected(item, event);
        anp_app.item_self.querySelectorAll("fieldset.bitwise-right .results tr[data-i='" + item.getAttribute("data-i") + "']").forEach(row => row.setAttribute("data-selected", true));

    };

    const draw_bitwise = results => {

        let html = ``,
            header = `<tr>`;

        ["error", "i", "string", "array", "integer"].forEach(key => header += anp_app.comp.i18n(key, "th"));
        header += `</tr>`;

        for(const language in results){

            let table = ``;

            results[language].forEach((result, i) => table += (`<tr` + anp_app.attr.set({
                data_i : i,
                data_selected : false,
                onmouseover : anp_app.object_name + ".errors_tests.row_selected(this, event);",
                onmouseout : anp_app.object_name + ".errors_tests.row_unselected(this, event);"
            }) + `>
                <td` + anp_app.attr.set({data_key : "error"}) + `>` + result.error + `</td>
                <td` + anp_app.attr.set({data_key : "i"}) + `>` + result.i + `</td>
                <td` + anp_app.attr.set({data_key : "string"}) + `>` + result.string + `</td>
                <td` + anp_app.attr.set({data_key : "array"}) + `>` + JSON.stringify(result.array) + `</td>
                <td` + anp_app.attr.set({data_key : "integer"}) + `>` + result.integer + `</td>
            </tr>`));

            html += (`
                <fieldset` + anp_app.attr.set({data_language : language}) + `>
                    ` + anp_app.comp.i18n(language, "legend") + `
                    <table` + anp_app.attr.set({class : "data-codes"}) + `>
                        <thead>` + header + `</thead>
                        <tbody>` + table + `</tbody>
                        <tfoot>` + header + `</tfoot>
                    </table>
                </fieldset>
            `);

        };

        anp_app.item_self.querySelector("fieldset.bitwise-right .results,fieldset.bitwise-left .results").innerHTML = html;

    };

    const bitwise = (values, direction) => {

        let loaded = 0;
        const error = get_code(values),
              results = {},
              end = () => ++ loaded == 5 && draw_bitwise(results);

        ["js", "ecma"].forEach(key => {
            if(values[key]){
                results[key] = [];
                for(let i = 0; i < values.positions; i ++){
                    results[key].push({
                        error : error,
                        i : i,
                        string : anp_app.errors.bitwise(error, i * direction)
                    });
                    results[key][i].array = anp_app.errors.to_array(results[key][i].string);
                    results[key][i].integer = anp_app.errors.to_integer(results[key][i].string);
                };
            };
        });

        ["php", "python", "sqlite", "mysql", "transact_sql"].forEach(key => {
            if(values[key])
                anp_app.ajax.send("post", "/app/errors/tests/" + key + "/bitwise/" + (direction > 0 ? "left" : "right"), {
                    code : error,
                    positions : values.positions
                }, (data, ok) => {
                    ok && data && (results[key] = data);
                    end();
                });
            else
                end();
        });

    };

    this.bitwise_right = (item, event) => {

        const [values, form] = anp_app.comp.forms.get_values(item);

        bitwise(values, -1);

    };

    this.bitwise_left = (item, event) => {

        const [values, form] = anp_app.comp.forms.get_values(item);

        bitwise(values, 1);

    };

    const draw_format = results => {

        let html = ``,
            header = (`
                <tr>
                    <th></th>
            `);

        ["integer", "string", "array"].forEach(key => header += anp_app.comp.i18n(key, "th"));
        header += (`
                <th></th>
            </tr>
        `);

        for(const key in results){
            if(!results[key])
                continue;

            html += (`
                <fieldset` + anp_app.attr.set({data_language : key}) + `>
                    ` + anp_app.comp.i18n(key, "legend") + `
                    <table>
                        <thead>` + header + `</thead>
                        <tbody>
            `);
            for(const type  in results[key]){
                html += (`
                    <tr` + anp_app.attr.set({data_type : type}) + `>
                        ` + anp_app.comp.i18n(type, "th") + `
                `);
                for(const subtype in results[key][type])
                    html += `<td` + anp_app.attr.set({data_subtype : subtype}) + `>` + (
                        results[key][type][subtype] === null ? "null" :
                        anp_app.is_array(results[key][type][subtype]) ? JSON.stringify(results[key][type][subtype]) :
                        results[key][type][subtype]
                    ) + `</td>`;
                html += (`
                        ` + anp_app.comp.i18n(type, "th") + `
                    </tr>
                `);
            };
            html += (`
                        </tbody>
                        <tfoot>` + header + `</tfoot>
                    </table>
                </fieldset>
            `);

        };

        anp_app.item_self.querySelector("fieldset.error-format .results").innerHTML = html;

    };

    this.error_format = (item, event) => {

        let loaded = 0;
        const results = {},
              [values, form] = anp_app.comp.forms.get_values(item),
              code = values.code ? values.code_type == "string" ? values.code : anp_app.errors.to_string(Number(values.code)) : anp_app.errors.to_string(Math.random() * 10 ** 9 >> 0),
              end = () => ++ loaded == 5 && draw_format(results);

        ["js", "ecma"].forEach(key => {
            if(!values[key])
                return;

            let string = code,
                integer = anp_app.errors.to_integer(string),
                array = anp_app.errors.to_array(string);

            results[key] = {
                integer : {
                    integer : anp_app.errors.to_integer(integer),
                    string : anp_app.errors.to_string(integer),
                    array : anp_app.errors.to_array(integer)
                },
                string : {
                    integer : anp_app.errors.to_integer(string),
                    string : anp_app.errors.to_string(string),
                    array : array
                },
                array : {
                    integer : anp_app.errors.to_integer(array),
                    string : anp_app.errors.to_string(array),
                    array : anp_app.errors.to_array(array)
                }
            };

        });

        ["php", "python", "sqlite", "mysql", "transact_sql"].forEach(key => {
            if(values[key])
                anp_app.ajax.send("post", "/app/errors/tests/" + key + "/format", {code : code}, (data, ok) => {
                    ok && data && (results[key] = data);
                    end();
                });
            else
                end();
        });

    };

    const draw_binary = results => {

        let html = ``;

        for(const key in results){

            let header = `<tr>`;

            results[key].string.forEach((item, i) => header += `<th` + anp_app.attr.set({
                data_i18n : "sub_i",
                data_1i8n_variables : anp_app.base64_encode(anp_app.json_encode({i : i}))
            }) + `>` + anp_app.i18n.get("sub_i", {i : i}) + `</th>`);
            header += `</tr>`;

            html += (`
                <fieldset` + anp_app.attr.set({data_language : key}) + `>
                    ` + anp_app.comp.i18n(key, "legend") + `
                    <table>
                        <thead>` + header + `</thead>
                        <tbody>
            `);
            for(const name in results[key])
                html += (`
                    <tr` + anp_app.attr.set({data_name : name}) + `>` + results[key][name].reduce((total, item, j) => (
                        (j > 1 ? total : `<td` + anp_app.attr.set({data_j : j - 1}) + `>` + total + `</td>`) +
                        `<td` + anp_app.attr.set({data_j : j}) + `>` + item + `</td>`
                    )) + `</tr>
                `);
            html += (`
                        <tbody>
                        <tfoot>` + header + `</tfoot>
                    </table>
                </fieldset>
            `);

        };

        anp_app.item_self.querySelector("fieldset.error-binary .results").innerHTML = html;

    };

    this.error_binary = (item, event) => {

        let loaded = 0;
        const results = {},
              [values, form] = anp_app.comp.forms.get_values(item),
              code = values.code ? values.code_type == "string" ? values.code : anp_app.errors.to_string(Number(values.code)) : anp_app.errors.to_string(Math.random() * 10 ** 9 >> 0),
              end = () => ++ loaded == 5 && draw_binary(results);

        ["js", "ecma"].forEach(key => values[key] && (results[key] = {
            string : code.split(""),
            array : anp_app.errors.to_array(code),
            binary : anp_app.errors.to_array_binary(code)
        }));

        ["php", "python", "sqlite", "mysql", "transact_sql"].forEach(key => {
            if(values[key])
                anp_app.ajax.send("post", "/app/errors/tests/" + key + "/binary", {code : code}, (data, ok) => {
                    ok && data && (results[key] = data);
                    end();
                });
            else
                end();
        });

    };

    const draw_joiner = results => {

        let html = ``,
            headers = ["string", "string_array", "array", "binary", "integer"];

        console.error(results);

        ["js", "ecma", "php", "python", "sqlite", "mysql", "transact_sql"].forEach(key => {
            if(!results[key])
                return;

            html += (`
                <fieldset` + anp_app.attr.set({data_language : key}) + `>
                    ` + anp_app.comp.i18n(key, "legend") + `
            `);
            Object.keys(results[key][0]).forEach((name, column) => {
                html += (`
                    <table>
                        <thead>
                            <tr>` + anp_app.comp.i18n(name, "th") + `</tr>
                        </thead>
                        <tbody>
                `);
                results[key].forEach((result, j) => html += (`
                    <tr` + anp_app.attr.set({data_field : headers[column]}) + `>
                        <td` + anp_app.attr.set({class : name}) + `>` + (result[name] instanceof Array ? anp_app.json_encode(result[name]) : result[name]) + `</td>
                    </tr>
                `));
                html += (`
                        </tbody>
                        <thead>
                            <tr>` + anp_app.comp.i18n(name, "th") + `</tr>
                        </thead>
                    </table>
                `);
            });
            html += `</fieldset>`;

        });
        /*for(const key in results){
            if(!results[key])
                continue;
            html += (`
                <fieldset` + anp_app.attr.set({data_language : key}) + `>
                    ` + anp_app.comp.i18n(key, "legend") + `
            `);
            Object.keys(results[key][0]).forEach((name, column) => {
                html += (`
                    <table>
                        <thead>
                            <tr>` + anp_app.comp.i18n(name, "th") + `</tr>
                        </thead>
                        <tbody>
                `);
                results[key].forEach((result, j) => html += (`
                    <tr` + anp_app.attr.set({data_field : headers[column]}) + `>
                        <td` + anp_app.attr.set({class : name}) + `>` + (result[name] instanceof Array ? anp_app.json_encode(result[name]) : result[name]) + `</td>
                    </tr>
                `));
                html += (`
                        </tbody>
                        <thead>
                            <tr>` + anp_app.comp.i18n(name, "th") + `</tr>
                        </thead>
                    </table>
                `);
            });
            html += `</fieldset>`;
        };*/

        anp_app.item_self.querySelector("fieldset.error-join .results,fieldset.error-set .results").innerHTML = html;

    };

    this.error_join = (item, event) => {

        let loaded = 0;
        const results = {},
              [values, form] = anp_app.comp.forms.get_values(item),
              code = values.code ? values.code_type == "string" ? values.code : anp_app.errors.to_string(Number(values.code)) : anp_app.errors.to_string(Math.random() * 10 ** 9 >> 0),
              end = () => ++ loaded == 5 && draw_joiner(results);

        ["js", "ecma"].forEach(key => {
            if(!values[key])
                return;

            const joiner = values.join_code,
                  bitwise = anp_app.errors.bitwise(joiner, values.position),
                  result = anp_app.errors.set(code, joiner, values.position);

            results[key] = [{
                code : code,
                joiner : joiner,
                bitwise : bitwise,
                results : result
            }, {
                code : code.split(""),
                joiner : joiner.split(""),
                bitwise : bitwise.split(""),
                results : result.split("")
            }, {
                code : anp_app.errors.to_array(code),
                joiner : anp_app.errors.to_array(joiner),
                bitwise : anp_app.errors.to_array(bitwise),
                results : anp_app.errors.to_array(result)
            }, {
                code : anp_app.errors.to_array_binary(code),
                joiner : anp_app.errors.to_array_binary(joiner),
                bitwise : anp_app.errors.to_array_binary(bitwise),
                results : anp_app.errors.to_array_binary(result)
            }, {
                code : anp_app.errors.to_integer(code),
                joiner : anp_app.errors.to_integer(joiner),
                bitwise : anp_app.errors.to_integer(bitwise),
                results : anp_app.errors.to_integer(result)
            }];

        });

        ["php", "python", "sqlite", "mysql", "transact_sql"].forEach(key => {
            if(values[key])
                anp_app.ajax.send("post", "/app/errors/tests/" + key + "/join", {
                    code : code,
                    joiner : values.join_code,
                    position : values.position
                }, (data, ok) => {
                    ok && data && (results[key] = data);
                    end();
                });
            else
                end();
        });

    };

    this.error_set = (item, event) => {

        let loaded = 0;
        const results = {},
              [values, form] = anp_app.comp.forms.get_values(item),
              code = values.code ? values.code_type == "string" ? values.code : anp_app.errors.to_string(Number(values.code)) : anp_app.errors.to_string(Math.random() * 10 ** 9 >> 0),
              end = () => ++ loaded == 5 && draw_joiner(results);

        console.log(values);

        ["js", "ecma"].forEach(key => {
            if(!values[key])
                return;

            const joiner = values.join_code,
                  bitwise = anp_app.errors.bitwise(joiner, values.position),
                  reset = anp_app.errors.set(code, 0, values.position, values.length),
                  result = anp_app.errors.set(code, joiner, values.position, values.length);

            results[key] = [{
                code : code,
                joiner : joiner,
                bitwise : bitwise,
                reset : reset,
                results : result
            }, {
                code : code.split(""),
                joiner : joiner.split(""),
                bitwise : bitwise.split(""),
                reset : reset.split(""),
                results : result.split("")
            }, {
                code : anp_app.errors.to_array(code),
                joiner : anp_app.errors.to_array(joiner),
                bitwise : anp_app.errors.to_array(bitwise),
                reset : anp_app.errors.to_array(reset),
                results : anp_app.errors.to_array(result)
            }, {
                code : anp_app.errors.to_array_binary(code),
                joiner : anp_app.errors.to_array_binary(joiner),
                bitwise : anp_app.errors.to_array_binary(bitwise),
                reset : anp_app.errors.to_array_binary(reset),
                results : anp_app.errors.to_array_binary(result)
            }, {
                code : anp_app.errors.to_integer(code),
                joiner : anp_app.errors.to_integer(joiner),
                bitwise : anp_app.errors.to_integer(bitwise),
                reset : anp_app.errors.to_integer(reset),
                results : anp_app.errors.to_integer(result)
            }];

        });

        ["php", "python", "sqlite", "mysql", "transact_sql"].forEach(key => {
            if(values[key])
                anp_app.ajax.send("post", "/app/errors/tests/" + key + "/set", {
                    code : code,
                    joiner : values.join_code,
                    position : values.position,
                    length : values.length
                }, (data, ok) => {
                    ok && data && (results[key] = data);
                    end();
                });
            else
                end();
        });

    };

    construct();

};
