AnPApp.SQL = function(anp_app, input){

    const self = this,
          transact_reserved = [
              "permissions", "name", "ip", "message", "parameters", "procedures", "value",
              "sessions", "description", "password", "session", "data", "group", "user",
              "procedure", "bit", "values", "database", "databases", "type", "range", "types",
              "traces", "description", "value", "address"
          ];
    let started = false;

    this.build = input => (`
        <fieldset` + anp_app.attr.set({data_module : "SQL"}) + `>
            ` + anp_app.comp.i18n("sql", "legend") + `
            ` + anp_app.comp.i18n("sql_text", "p") + `
            ` + anp_app.comp.buttons({buttons : [
                {i18n : "my_to_transact", icon : "my_to_transact", onclick : "{object_name}.sql.call_my_to_transact(this, event);"}
            ]}) + `
        </fieldset>
    `);

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp_app.comp.sql = self.build;

        anp_app.is_function(callback) && callback();

    };

    const construct = () => {};

    const set = (sql, start, matches, results) => [sql.substr(0, start) + results, sql.substr(start + matches[0].length)];

    const set_transact_key = key => transact_reserved.includes((key = key.replace(/^`(.+)`$/, "$1")).toLowerCase()) ? "[" + key + "]" : key;

    const set_transact_no_key = key => key.replace(/^(`\$?([^`]+)`|\$?([a-z0-9_]+)|\$?([^\s]+))$/, "$2$3$4");

    const set_transact_type = type => (
        /^bool(ean)?$/.test(type) ? "bit" :
        /^text$/.test(type) ? "varchar(max)" :
        type
    );

    const to_snake = text => text.replace(/[^a-z0-9]+/gi, "_").replace(/[A-Z]+/g, characters => "_" + characters.toLowerCase()).replace(/^_+/, "");

    const transact_blocks = [

        // Strings and blocks.
        [/'(([^\\']+|\\.)*)'|"(([^\\"]+|\\.)*)"/, matches => matches[0][0] + (matches[1] || matches[3] || matches[5] || "").replace(new RegExp("\\\\([\\\\" + matches[0][0] + "])", "g"), "$1$1") + matches[0][0]],

        // Database.
        [/create\s+database(\s+if\s+not\s+exists)?\s+(`[^`]+`|[^\s;]+)\s*([^;]+)?;/i, matches => (
            (matches[1] ? "if (select top 1 1 from sys.databases where name = '" + matches[2] + "') is null " : "") +
            "create database " + set_transact_key(matches[2]) + "\ngo"
        )],
        [/drop\s+database(\s+if\s+exists)?\s+(`[^`]+`|[^\s;]+);/i, matches => (
            (matches[1] ? "if (select top 1 1 from sys.databases where name = '" + matches[2] + "') is not null " : "") +
            "drop database " + set_transact_key(matches[2]) + "\ngo"
        )],

        // Procedures.
        [/create\s+procedure(\s+if\s+not\s+exists)?\s+(`[^`]+`|[^\s\(]+)\s*(\((([^\(\)]+|\([^\)]+\)|[\r\n]+)*)\))\s*begin/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'P') is null " : "") +
            "create procedure dbo." + set_transact_key(matches[2]) + (matches[4] ? matches[4].replace(/(\-{2}[^\r\n]+|\/\*(([^\*]+|\*[^\/]|[\r\n]+)*)\*\/)|((in|inout|out|output)\s+)?(`[^`]+`|[^\s\r\n,]+)\s+([^,\(\s]+(\s*\([^\(\)]+\))?)/gi, (...arguments) => {
                return arguments[1] ? arguments[1] : "@" + set_transact_no_key(arguments[6]) + " " + set_transact_type(arguments[7]) + (["out", "output", "inout"].includes((arguments[5] || "").toLowerCase()) ? " output" : "");
            }) : "") + " as begin"
        )],
        [/drop\s+procedure(\s+if\s+exists)?\s+(`[^`]+`|[^\s;]+)(;\^?|\$+)/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'P') is not null " : "") +
            "drop procedure dbo." + set_transact_key(matches[2]) + "\ngo"
        )],

        // Functions.
        [/create\s+function(\s+if\s+not\s+exists)?\s+(`[^`]+`|[^\s\(]+)\s*(\((([^\(\)]+|\([^\)]+\)|[\r\n]+)*)\))\s*returns\s+([^\s\(]+(\s*(\([^\(\)]+\)))?)\s+begin/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'FN') is null " : "") +
            "create function dbo." + set_transact_key(matches[2]) + "(" + (matches[4] ? matches[4].replace(/(\-{2}[^\r\n]+|\/\*(([^\*]+|\*[^\/]|[\r\n]+)*)\*\/)|(in\s+)?(`[^`]+`|[^\s\r\n,]+)\s+([^,\(\s]+(\s*\([^\(\)]+\))?)/gi, (...arguments) => {
                return arguments[1] ? arguments[1] : "@" + set_transact_no_key(arguments[5]) + " " + set_transact_type(arguments[6]);
            }) : "") + ") returns " + (matches[6] == "text" ? "varchar(max)" : set_transact_type(matches[6])) + " begin"
        )],
        [/drop\s+function(\s+if\s+exists)?\s+(`[^`]+`|[^\s;]+)(;\^?|\$+)/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'FN') is not null " : "") +
            "drop function dbo." + set_transact_key(matches[2]) + "\ngo"
        )],

        // Tables.
        [/create\s+table(\s+if\s+not\s+exists)?\s+(`[^`]+`|[^\s\(]+)\s*\((([^\(\)]+|\([^\(\)]*\)|[\n\r]+)*)\);?/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'U') is null " : "") +
            "create table dbo." + set_transact_key(matches[2]) + "(" + (matches[3] ? matches[3].replace(/(constraint\s+[^\s]+\s+)?(foreign|primary)\s+key\s*\((`[^`]+`|[^\(\)]+)\)(\s+references\s+(`[^`]+`|[^\(\s]+)\s*\((`[^`]+`|[^\(\)]+)\))?(\s*,)?|(\-{2}[^\r\n]+|\/\*(([^\*]+|\*[^\/]|[\r\n]+)*)\*\/)|(`[^`]+`|[^\s]+)\s+([^\s,\(]+(\s*\([^\)]+\))?)(\s+not\s+null)?(\s+(auto_?increment|default\s+([^,\(\s]+)(\s*\([^\(\)]*\))?))?(\s*,)?/gi, (...arguments) => (
                arguments[8] ? arguments[8] :
                arguments[2] ? (arguments[1] || "constraint " + (matches[2][0] == "_" ? "_" : "") + to_snake(matches[2]) + "_" + arguments[3] + " ") + arguments[2] + " key(" + set_transact_key(arguments[3]) + ")" + (arguments[4] ? " references " + set_transact_key(arguments[5]) + "(" + set_transact_key(arguments[6]) + ")" : "") :
                set_transact_key(arguments[11]) + " " + my_to_transact(arguments[12]) + (arguments[14] ? " not null" : "") + (arguments[15] ? (
                    /^auto_?increment$/.test(arguments[16]) ? " identity(1, 1)" : " constraint " + (matches[2][0] == "_" ? "_" : "") + to_snake(matches[2]) + "_" + set_transact_no_key(arguments[11]) + " default " + my_to_transact(arguments[17] + (arguments[18] || ""))
                ) : "")
            ) + (arguments[7] || arguments[19] ? "," : "")) : "") + ")"
        )],
        [/drop\s+table(\s+if\s+exists)?\s+(`[^`]+`|[^\s;]+)(;\^?|\$+)/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'U') is not null " : "") +
            "drop table dbo." + set_transact_key(matches[2])
        )],
        [/(insert into|from|update)\s+((`([^`]+)`|[a-z0-9_]+|\.)+)/i, matches => matches[1] + " " + my_to_transact(matches[2].includes(".") ? matches[2] : "dbo." + set_transact_key(matches[2]))],

        // Triggers.
        [/create\s+trigger(\s+if\s+not\s+exists)?\s+(`[^`]+`|[^\s]+)\s*(after|before)\s+(((insert|delete|update)(,\s*)?)+)\s+on\s+([^\s]+|`[^`]+`)\s+(for\s+each\s+row\s+)?begin/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'TR') is null " : "") +
            "create trigger dbo." + set_transact_key(matches[2]) + " on dbo." + set_transact_key(matches[8]) + " " + matches[3] + " " + matches[4] + " as begin"
        )],
        [/drop\s+trigger(\s+if\s+exists)?\s+(`[^`]+`|[^\s;]+)(;\^?|\$+)/i, matches => (
            (matches[1] ? "if object_id(N'dbo." + set_transact_key(matches[2]) + "', N'TR') is not null " : "") +
            "drop trigger dbo." + set_transact_key(matches[2]) + "\ngo"
        )],

        // Endings.
        [/end(;[;\^]|\${2})/i, matches => "end\ngo"],
        [/(end;\s*)?end(\s+[^\s]+);/i, matches => "end"],
        [/;[;\^]|\${2}/, matches => "\ngo"],

        // Special methods and words.
        [/\bauto_?increment\b/i, matches => "identity(1, 1)"],
        [/\bnow\s*\(\)/i, matches => "getdate()"],
        [/\blast_insert_id\s*\(\)/i, matches => "scope_identity()"],
        [/\bifnull\s*\(/i, matches => "isnull("],
        [/\bbool(ean)?\b/i, matches => "bit"],
        [/\btext\b/i, matches => "varchar(max)"],
        [/\blocate\b/i, matches => "charindex"],
        [/call\s+([^\(\s]+)\s*\(([^\(\)]*)\)(;[;\^]?|\$+)/i, matches => "execute dbo." + set_transact_key(matches[1]) + " " + my_to_transact(matches[2])],
        [/\:\=/, matches => "="],
        [/\(?(\d+|\$?[0-9a-z_]+|`\$?[^`]+`)\s*(\<{2}|\>{2})\s*(\$?[0-9a-z_]+|`\$([^`]+)`)\)?/i, matches => "dbo.bw(" + my_to_transact(matches[1]) + ", " + (matches[2] == ">>" ? "-" : "") + my_to_transact(matches[3]) + ")"],
        [/\bthen\s*begin\b/i, matches => "begin"],
        [/`\$([^`]+)`|\$([a-z0-9_]+)/i, matches => "@" + (matches[1] || matches[2])],
        [/(;)?else(\s*if\b)?/i, matches => (matches[1] ? "\n" : " ") + "else" + (matches[2] ? " if" : "")],
        [/\s*\&{2}\s*/, matches => " and "],
        [/\s*\|{2}\s*/, matches => " or "],
        [/(?!(\!\=))!/, matches => "not "],
        [/\b(false|true)\b/i, matches => matches[1] == "true" ? 1 : 0],
        [/\bdo\b/i, matches => "begin"],
        [/\bregexp\b/i, matches => "like"],
        [/\blength\s*\(/i, matches => "len("],

        // Variables.
        [/declare\s+(`\$?([^`]+)`|\$?([a-z0-9_]+))\s+([^\s\(;]+(\s*\([^\(\)]+\))?)(\s+default\s+(('(([^'\\]+|\\[^']|[\r\n]+)*)'|[^;']+|[\r\n]+)+))?(\s*;)?/i, matches => {
            return "declare @" + (matches[2] || matches[3]) + " " + matches[4] + (matches[6] ? " = " + my_to_transact(matches[7]) : "");
        }],
        [/set\s+(`\$?([^`]+)`|\$?([a-z0-9_]+))\s*\:?\=\s*(('(([^'\\]+|\\[^']|[\r\n]+)*)'|[^;']+|[\r\n]+)+)(\s*;)?/i, matches => {
            return "set @" + (matches[2] || matches[3]) + " = " + my_to_transact(matches[4]);
        }],

        // Reserved words used.
        [/`([^`]+)`/, matches => set_transact_key(matches[1])],

        // For removing.
        [/\bdelimiter\b[^\r\n]*/i, matches => ""],
        [/;([\r\n]|\s*\-{2}[^\r\n]+)/, matches => matches[1]],
        [/[^\s:]+\:begin/i, matches => "begin"],

        // Comments.
        [/\-{2}[^\r\n]+|\/\*(([^\*]+|\*[^\/]|[\r\n]+)*)\*\//, matches => matches[0]],

        // JSON methods.
        [/\bjson_(compact)\(/i, matches => "dbo." + matches[0]],

    ];

    const my_to_transact = this.my_to_transact = sql => {

        const positions = [];
        let matches,
            last_position = 0,
            processed = "",
            fragment;

        let x = 0;
        const l = 1000;

        while(true && ++ x < l){

            let next = null,
                next_position = sql.length;

            transact_blocks.forEach((block, i) => {
                if(positions[i] === null)
                    return;
                if(positions[i] === undefined || (positions[i].position -= last_position) < 0)
                    positions[i] = (matches = sql.match(block[0])) ? {
                        matches : matches,
                        position : matches.index
                    } : null;
                if(positions[i] && positions[i].position < next_position){
                    next = i;
                    next_position = positions[i].position;
                };
            });

            if(next === null)
                break;

            [fragment, sql] = set(sql, next_position, positions[next].matches, transact_blocks[next][1](positions[next].matches));
            last_position = next_position + positions[next].matches[0].length;
            processed += fragment;

        };
        processed += sql;

        return processed;
    };

    this.call_my_to_transact = (item, event) => anp_app.open_local_files({
        accept : ".sql",
        multiple : true
    }, files => {

        let i = 0;
        const zip = new JSZip(),
              end = () => ++ i == files.length && anp_app.jszip_download(zip);

        files.forEach((file, i) => {
            zip.file(file.name, my_to_transact(anp_app.base64_decode(file.data.split(",")[1])))
            end();
        });

    });

    construct();

};
