use AnP;

delimiter ;^
	
	drop procedure if exists errors_tests_bitwise;^
	create procedure errors_tests_bitwise(
		in $code varchar(512),
		in $positions smallint,
		in $direction smallint
	) begin 
		
		set @i := -1;
		
		select 
			$code as error, 
			@i := @i + 1 as i, 
			@string := errors_bitwise($code, @i * $direction) as string,
			errors_to_array(@string) as 'array',
			errors_to_integer(@string) as 'integer'
		from information_schema.columns limit $positions;
		
	end;^
	
	drop procedure if exists errors_tests_format;^
	create procedure errors_tests_format(
		in $code varchar(2048)
	) begin 
		
		declare `$integer` bigint;
		declare $string varchar(512);
		declare `$array` varchar(2048);
	
		if substring($code, 1, 1) = '[' then begin 
			set `$array` := $code;
			set $string := errors_to_array_back(`$array`);
		end;else begin 
			set $string := $code;
			set `$array` := errors_to_array($string);
		end;end if;
		set `$integer` := errors_to_integer($code);
		
		select concat('{'
			'"integer":', `$integer`, ',',
			'"string":"', errors_to_string(`$integer`), '",', 
			'"array":', errors_to_array(errors_to_string(`$integer`)),
		'}') as json_values union all
		select concat('{'
			'"integer":', errors_to_integer($string), ',',
			'"string":"', $string, '",', 
			'"array":', errors_to_array($string),
		'}') as json_values union all
		select concat('{'
			'"integer":', errors_to_integer(`$array`), ',',
			'"string":"', errors_to_string(`$array`), '",', 
			'"array":', `$array`,
		'}') as json_values;
		
	end;^
	
	drop procedure if exists errors_tests_binary;^
	create procedure errors_tests_binary(
		in $code varchar(2048)
	) begin 
		
		declare $string varchar(2048) default '';
		declare `$array` varchar(2048);
		declare $i integer default 1;
		declare $l integer;
	
		if substring($code, 1, 1) = '[' then begin 
			set `$array` := $code;
			set $code := errors_to_string($code);
		end;else 
			set `$array` := errors_to_array($code);
		end if;
		set $l := length($code);
	
		while $i <= $l do
			set $string := concat($string, ',"', substring($code, $i, 1), '"');
			set $i := $i + 1;
		end while;
		
		select 
			concat('[', substring($string, 2, 2048), ']') as string,
			`$array` as `array`,
			errors_to_array_binary($code) as `binary`;
		
	end;^
	
	drop procedure if exists errors_tests_join;^
	create procedure errors_tests_join(
		in $code varchar(512),
		in $joiner varchar(512),
		in `$position` smallint
	) begin 
		
		drop temporary table if exists AnPTTErrorsTestsJoin;
		create temporary table AnPTTErrorsTestsJoin(
			id integer not null auto_increment,
			code varchar(512) not null,
			joiner varchar(512) not null,
			bitwise varchar(512) not null,
			results varchar(512) not null,
			date_in datetime not null default now(),
			date_out datetime,
			primary key(id)
		);
	
		insert into AnPTTErrorsTestsJoin(code, joiner, bitwise, results) select 
			$code, 
			$joiner,
			errors_bitwise($joiner, `$position`),
			errors_join($code, $joiner, `$position`, null);
	
		select code, joiner, bitwise, results from AnPTTErrorsTestsJoin union all
		select 
			string_to_characters_array(code),
			string_to_characters_array(joiner),
			string_to_characters_array(bitwise),
			string_to_characters_array(results)
		from AnPTTErrorsTestsJoin union all
		select 
			errors_to_array(code),
			errors_to_array(joiner),
			errors_to_array(bitwise),
			errors_to_array(results)
		from AnPTTErrorsTestsJoin union all
		select 
			errors_to_array_binary(code),
			errors_to_array_binary(joiner),
			errors_to_array_binary(bitwise),
			errors_to_array_binary(results)
		from AnPTTErrorsTestsJoin union all
		select 
			errors_to_integer(code),
			errors_to_integer(joiner),
			errors_to_integer(bitwise),
			errors_to_integer(results)
		from AnPTTErrorsTestsJoin;
	
		drop temporary table AnPTTErrorsTestsJoin;
		
	end;^
	
	drop procedure if exists errors_tests_set;^
	create procedure errors_tests_set(
		in $code varchar(512),
		in $joiner varchar(512),
		in `$position` smallint,
		in `$length` smallint
	) begin 
		
		drop temporary table if exists AnPTTErrorsTestsSet;
		create temporary table AnPTTErrorsTestsSet(
			id integer not null auto_increment,
			code varchar(512) not null,
			joiner varchar(512) not null,
			bitwise varchar(512) not null,
			reset varchar(512) not null,
			results varchar(512) not null,
			date_in datetime not null default now(),
			date_out datetime,
			primary key(id)
		);
	
		insert into AnPTTErrorsTestsSet(code, joiner, bitwise, reset, results) select 
			$code, 
			$joiner,
			errors_bitwise($joiner, `$position`),
			errors_join($code, errors_to_string(0), `$position`, `$length`),
			errors_join($code, $joiner, `$position`, `$length`);
	
		select code, joiner, bitwise, reset, results from AnPTTErrorsTestsSet union all
		select 
			string_to_characters_array(code),
			string_to_characters_array(joiner),
			string_to_characters_array(bitwise),
			string_to_characters_array(reset),
			string_to_characters_array(results)
		from AnPTTErrorsTestsSet union all
		select 
			errors_to_array(code),
			errors_to_array(joiner),
			errors_to_array(bitwise),
			errors_to_array(reset),
			errors_to_array(results)
		from AnPTTErrorsTestsSet union all
		select 
			errors_to_array_binary(code),
			errors_to_array_binary(joiner),
			errors_to_array_binary(bitwise),
			errors_to_array_binary(reset),
			errors_to_array_binary(results)
		from AnPTTErrorsTestsSet union all
		select 
			errors_to_integer(code),
			errors_to_integer(joiner),
			errors_to_integer(bitwise),
			errors_to_integer(reset),
			errors_to_integer(results)
		from AnPTTErrorsTestsSet;
	
		drop temporary table AnPTTErrorsTestsSet;
		
	end;^
	

delimiter ;