#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os.path import dirname as directory_name
from os.path import abspath as path_absolute
from os.path import exists as path_exists

anp_root = directory_name(path_absolute(__file__)) + "/../../.."
root = anp_root + "/Python/"
print_status = False

for file in [".Common", "", ".Callbacks", ".Settings", ".Errors"]:
    path = root + "AnP" + file + ".py"
    print_status and print("TRY: " + path)
    if path_exists(path):
        with open(path) as data:
            exec(compile(data.read(), path, "exec"), globals())
        print_status and print("OK: " + path)
    else:
        print_status and print("ERROR: " + path)

anp = AnP({
	"roots" : [anp_root],
	"default_settings_files" : anp_root + "/JSON/AnP.py.settings.json",
	"settings_files" : anp_root + "/JSON/AnP.py.settings.json",
	"secrets_files" : anp_root + "/JSON/AnP.py.secrets.json",
    "allow_print_logs" : False
})

l = len(sys.argv)
action = sys.argv[1] if l > 1 else None
results = []

# print(anp.errors.bitwise("c7E+/as", 9))

def test_bitwise(code, positions, direction):

    results = []

    for i in range(positions):
        string = anp.errors.bitwise(code, i * direction)
        results.append({
            "error" : code,
            "i" : i,
            "string" : string,
            "array" : anp.errors.to_array(string),
            "integer" : anp.errors.to_integer(string)
        })

    return results

if action == "bitwise_left":
    results = test_bitwise(anp.json_decode(sys.argv[2]) if sys.argv[2][0] == "[" else sys.argv[2], int(sys.argv[3]), 1)
elif action == "bitwise_right":
    results = test_bitwise(anp.json_decode(sys.argv[2]) if sys.argv[2][0] == "[" else sys.argv[2], int(sys.argv[3]), -1)
elif action == "format":
    string = sys.argv[2]
    integer = anp.errors.to_integer(string)
    array = anp.errors.to_array(string)
    results = {
        "integer" : {
            "integer" : anp.errors.to_integer(integer),
            "string" : string,
            "array" : array
        },
        "string" : {
            "integer" : anp.errors.to_integer(string),
            "string" : anp.errors.to_string(string),
            "array" : anp.errors.to_array(string)
        },
        "array" : {
            "integer" : anp.errors.to_integer(array),
            "string" : anp.errors.to_string(array),
            "array" : anp.errors.to_array(array)
        }
    }
elif action == "binary":
    code = sys.argv[2]
    results = {
        "string" : [character for character in code],
        "array" : anp.errors.to_array(code),
        "binary" : anp.errors.to_array_binary(code)
    }
elif action == "join":
    code = sys.argv[2]
    joiner = sys.argv[3]
    position = int(sys.argv[4])
    bitwise = anp.errors.bitwise(joiner, position)
    results = anp.errors.set(code, joiner, position)
    results = [{
        "code" : code,
        "joiner" : joiner,
        "bitwise" : bitwise,
        "results" : results
    }, {
        "code" : list(code),
        "joiner" : list(joiner),
        "bitwise" : list(bitwise),
        "results" : list(results)
    }, {
        "code" : anp.errors.to_array(code),
        "joiner" : anp.errors.to_array(joiner),
        "bitwise" : anp.errors.to_array(bitwise),
        "results" : anp.errors.to_array(results)
    }, {
        "code" : anp.errors.to_array_binary(code),
        "joiner" : anp.errors.to_array_binary(joiner),
        "bitwise" : anp.errors.to_array_binary(bitwise),
        "results" : anp.errors.to_array_binary(results)
    }, {
        "code" : anp.errors.to_integer(code),
        "joiner" : anp.errors.to_integer(joiner),
        "bitwise" : anp.errors.to_integer(bitwise),
        "results" : anp.errors.to_integer(results)
    }]
elif action == "set":
    code = sys.argv[2]
    joiner = sys.argv[3]
    position = int(sys.argv[4])
    length = int(sys.argv[5])
    bitwise = anp.errors.bitwise(joiner, position)
    reset = anp.errors.set(code, 0, position, length)
    results = anp.errors.set(code, joiner, position, length)
    results = [{
        "code" : code,
        "joiner" : joiner,
        "bitwise" : bitwise,
        "reset" : reset,
        "results" : results
    }, {
        "code" : list(code),
        "joiner" : list(joiner),
        "bitwise" : list(bitwise),
        "reset" : list(reset),
        "results" : list(results)
    }, {
        "code" : anp.errors.to_array(code),
        "joiner" : anp.errors.to_array(joiner),
        "bitwise" : anp.errors.to_array(bitwise),
        "reset" : anp.errors.to_array(reset),
        "results" : anp.errors.to_array(results)
    }, {
        "code" : anp.errors.to_array_binary(code),
        "joiner" : anp.errors.to_array_binary(joiner),
        "bitwise" : anp.errors.to_array_binary(bitwise),
        "reset" : anp.errors.to_array_binary(reset),
        "results" : anp.errors.to_array_binary(results)
    }, {
        "code" : anp.errors.to_integer(code),
        "joiner" : anp.errors.to_integer(joiner),
        "bitwise" : anp.errors.to_integer(bitwise),
        "reset" : anp.errors.to_integer(reset),
        "results" : anp.errors.to_integer(results)
    }]

print(anp.json_encode(results))
