use AnP

if object_id(N'dbo.errors_tests_bitwise', N'P') is not null drop procedure dbo.errors_tests_bitwise
go
create procedure dbo.errors_tests_bitwise 
	@code varchar(512),
	@positions integer,
	@direction smallint
as begin 

	set nocount on
	select 
		error, i, string, 
		dbo.errors_to_array(string) as array,
		dbo.errors_to_integer(string) as 'integer'
	from (select 
		@code as error,
		subtable_view.i as i,
		dbo.errors_bitwise(@code, subtable_view.i * @direction) as string
	from (select top (@positions) (row_number() over(order by table_catalog asc)) - 1 as i from information_schema.columns) subtable_view) table_view

end
go

if object_id(N'dbo.errors_tests_format', N'P') is not null drop procedure dbo.errors_tests_format
go
create procedure dbo.errors_tests_format 
	@code varchar(2048)
as begin 

	declare @integer bigint
	declare @string varchar(512)
	declare @array varchar(2048)

	set nocount on

	if substring(@code, 1, 1) = '[' begin 
		set @array = @code
		set @string = dbo.errors_to_array_back(@array)
	end else begin 
		set @string = @code
		set @array = dbo.errors_to_array(@string)
	end
	set @integer = dbo.errors_to_integer(@code)

	select '{' + 
		'"integer":' + convert(varchar(32), @integer) + ',' + 
		'"string":"' + dbo.errors_to_string(@integer) + '",' + 
		'"array":' + dbo.errors_to_array(dbo.errors_to_string(@integer)) + 
	'}' as json_values union all
	select '{' + 
		'"integer":' + convert(varchar(32), dbo.errors_to_integer(@string)) + ',' + 
		'"string":"' + @string + '",' + 
		'"array":' + dbo.errors_to_array(@string) + 
	'}' as json_values union all
	select '{' + 
		'"integer":' + convert(varchar(32), dbo.errors_to_integer(@array)) + ',' + 
		'"string":"' + dbo.errors_to_string(@array) + '",' + 
		'"array":' + @array + 
	'}' as json_values

end
go

if object_id(N'dbo.errors_tests_binary', N'P') is not null drop procedure dbo.errors_tests_binary
go
create procedure dbo.errors_tests_binary 
	@code varchar(2048)
as begin

	declare @array varchar(2048)

	set nocount on

	if substring(@code, 1, 1) = '[' begin 
		set @array = @code
		set @code = dbo.errors_to_string(@code)
	end else 
		set @array = dbo.errors_to_array(@code)

	select 
		dbo.string_to_characters_array(@code) as string,
		@array as array,
		dbo.errors_to_array_binary(@code) as [binary]

end
go

if object_id(N'dbo.errors_tests_join', N'P') is not null drop procedure dbo.errors_tests_join
go
create procedure dbo.errors_tests_join 
	@code varchar(512),
	@joiner varchar(512),
	@position smallint
as begin 
	
	set nocount on

	if object_id('Ntempdb..AnPTTErrorsTestsJoin', N'U') is not null drop table #AnPTTErrorsTestsJoin
	create table #AnPTTErrorsTestsJoin(
		id integer not null identity(1, 1),
		code varchar(512) not null,
		joiner varchar(512) not null,
		bitwise varchar(512) not null,
		results varchar(512) not null,
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)

	insert into #AnPTTErrorsTestsJoin(code, joiner, bitwise, results) select 
		@code,
		@joiner,
		dbo.errors_bitwise(@joiner, @position),
		dbo.errors_join(@code, @joiner, @position, null)

	select code, joiner, bitwise, results from #AnPTTErrorsTestsJoin union all
	select 
		dbo.string_to_characters_array(code),
		dbo.string_to_characters_array(joiner),
		dbo.string_to_characters_array(bitwise),
		dbo.string_to_characters_array(results)
	from #AnPTTErrorsTestsJoin union all
	select 
		dbo.errors_to_array(code),
		dbo.errors_to_array(joiner),
		dbo.errors_to_array(bitwise),
		dbo.errors_to_array(results)
	from #AnPTTErrorsTestsJoin union all
	select 
		dbo.errors_to_array_binary(code),
		dbo.errors_to_array_binary(joiner),
		dbo.errors_to_array_binary(bitwise),
		dbo.errors_to_array_binary(results)
	from #AnPTTErrorsTestsJoin union all
	select 
		convert(varchar(32), dbo.errors_to_integer(code)),
		convert(varchar(32), dbo.errors_to_integer(joiner)),
		convert(varchar(32), dbo.errors_to_integer(bitwise)),
		convert(varchar(32), dbo.errors_to_integer(results))
	from #AnPTTErrorsTestsJoin

	drop table #AnPTTErrorsTestsJoin

end
go

if object_id(N'dbo.errors_tests_set', N'P') is not null drop procedure dbo.errors_tests_set
go
create procedure dbo.errors_tests_set 
	@code varchar(512),
	@joiner varchar(512),
	@position smallint,
	@length smallint
as begin 
	
	set nocount on

	if object_id('Ntempdb..AnPTTErrorsTestsSet', N'U') is not null drop table #AnPTTErrorsTestsSet
	create table #AnPTTErrorsTestsSet(
		id integer not null identity(1, 1),
		code varchar(512) not null,
		joiner varchar(512) not null,
		bitwise varchar(512) not null,
		[reset] varchar(512) not null,
		results varchar(512) not null,
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)

	insert into #AnPTTErrorsTestsSet(code, joiner, bitwise, [reset], results) select 
		@code,
		@joiner,
		dbo.errors_bitwise(@joiner, @position),
		dbo.errors_join(@code, dbo.errors_to_string(0), @position, @length),
		dbo.errors_join(@code, @joiner, @position, @length)

	select code, joiner, bitwise, [reset], results from #AnPTTErrorsTestsSet union all
	select 
		dbo.string_to_characters_array(code),
		dbo.string_to_characters_array(joiner),
		dbo.string_to_characters_array(bitwise),
		dbo.string_to_characters_array([reset]),
		dbo.string_to_characters_array(results)
	from #AnPTTErrorsTestsSet union all
	select 
		dbo.errors_to_array(code),
		dbo.errors_to_array(joiner),
		dbo.errors_to_array(bitwise),
		dbo.errors_to_array([reset]),
		dbo.errors_to_array(results)
	from #AnPTTErrorsTestsSet union all
	select 
		dbo.errors_to_array_binary(code),
		dbo.errors_to_array_binary(joiner),
		dbo.errors_to_array_binary(bitwise),
		dbo.errors_to_array_binary([reset]),
		dbo.errors_to_array_binary(results)
	from #AnPTTErrorsTestsSet union all
	select 
		convert(varchar(32), dbo.errors_to_integer(code)),
		convert(varchar(32), dbo.errors_to_integer(joiner)),
		convert(varchar(32), dbo.errors_to_integer(bitwise)),
		convert(varchar(32), dbo.errors_to_integer([reset])),
		convert(varchar(32), dbo.errors_to_integer(results))
	from #AnPTTErrorsTestsSet

	drop table #AnPTTErrorsTestsSet

end
go
