AnP.Ajax = function(anp, input){

    const self = this;
    let started = false,
        loaders = 0,
        loader_started = false,
        anp_root = null, 
        app_root = null;

    const construct = () => {};

    this.start = callback => {

        if(started)
            return;
        started = true;

        AnP.Preload && new AnP.Preload(anp, ".anp-loader", loader => {
            if(!loader)
                return;
            loader_started = true;
            anp.item_self.querySelector(".anp-loader").setAttribute("data-executing", !!loaders);
        });

        anp.is_function(callback) && callback();

    };

    this.date_format = date => (
        ("0000" + date.getFullYear()).slice(-4) + "-" + 
        ("00" + (date.getMonth() + 1)).slice(-2) + "-" + 
        ("00" + date.getDate()).slice(-2) + "T" + 
        ("00" + date.getHours()).slice(-2) + ":" + 
        ("00" + date.getMinutes()).slice(-2) + ":" + 
        ("00" + date.getSeconds()).slice(-2) + "." + 
        "000Z"
    );

    this.load = (input, variables, callback) => {

        (anp.is_function(variables) || anp.is_object(callback)) && ([variables, callback] = [callback, variables]);

        if(!input)
            input = {};
        else if(!anp.is_object(input))
            input = anp.is_string(input) ? {url : input} : {};

        const url = anp.string_variables(anp.settings_get(["ajax_url", "url", "file", "link", "path"], input), {
            anp_root : anp_root === null ? anp_root = anp.settings_get("anp_root") : anp_root, 
            app_root : app_root === null ? app_root = anp.settings_get("app_root") : app_root
        });

        // console.log([url, anp_root]);

        !anp.is_function(callback) && (callback = anp.settings_get("callback", input));
        !anp.is_bool(input.loader) && (input.loader = true);
        input.loader && ++ loaders == 1 && loader_started && anp.item_self.querySelector(".anp-loader").setAttribute("data-executing", true);

        if(!url){
            anp.is_function(callback) && callback(null, 0, 0, false, "NO_URL");
            return;
        };

        let uri_variables = null,
            ended = false,
            method = anp.settings_get(["ajax_method", "method"], input, "get", false).toLowerCase();
        const timeout = anp.settings_get(["ajax_timeout", "timeout"], input, 2000, false),
              headers = [].concat(anp.settings_get(["ajax_headers", "headers"], input, [], false)),
              date = Date.now(),
              ajax = new XMLHttpRequest(),
              end = error => {
                  if(ended)
                      return;
                  ended = true;
                  if(
                      !((ajax.status >= 200 && ajax.status < 300) || [301, 302, 304].includes(ajax.status)) && 
                      /^\/[^\/].+\.[^\.\/]+$/.test(url) && 
                      (anp_root === null ? anp_root = anp.settings_get("anp_root") : anp_root) + url != url
                  )
                      self.load({...input, url : anp_root + url}, variables, callback);
                  else{
                      anp.is_function(callback) && callback(ajax.responseText, ajax.status, ajax.readyState, error == "OK", error);
                      //   console.log([url, ajax.responseText]);
                  };
                  input.loader && 
                  !--loaders && 
                  loader_started && 
                  anp.item_self.querySelector(".anp-loader").setAttribute("data-executing", false);
              },
              mime = anp.settings_get("mime", input);

        if(method != "get"){
            !variables && (variables = {});
            variables[anp.settings_get(["custom_method_variable_name", "custom_method_variable"], input)] = method;
            method = "post";
        };
        if(method == "post"){

            const post_headers = anp.settings_get(["ajax_post_headers", "post_headers"]);

            post_headers && anp.is_array(post_headers) && (anp.is_array(post_headers[0]) ? post_headers : [post_headers]).forEach(post_header => headers.unshift(post_header));

        };

        const l = headers.length;

        if(variables){

            const set = {};

            Object.keys(variables).forEach(key => set[key] = (
                variables[key] instanceof Date ? self.date_format(variables[key]) :
                // anp.is_object(variables[key]) ? anp.json_encode(variables[key]) :
                variables[key]
            ))

            switch(anp.settings_get(["ajax_send_mode", "send_mode", "mode"], input)){
                case "base64":
                    uri_variables = encodeURIComponent(anp.settings_get(["ajax_variable_name", "variable_name"], input, "anp", false)) + "=" + encodeURIComponent(btoa(anp.url_encode(anp.json_encode(set))));
                break;
                case "json":
                    uri_variables = encodeURIComponent(anp.settings_get(["ajax_variable_name", "variable_name"], input, "anp", false)) + "=" + anp.url_encode(anp.json_encode(set));
                break;
                case "normal":
                default:
                    uri_variables = "";
                    for(let name in variables)
                        uri_variables += (uri_variables ? "&" : "") + encodeURIComponent(name) + "=" + encodeURIComponent(
                            // variables[name] instanceof Date ? self.date_format(variables[name]) :
                            anp.is_object(set[name]) ? anp.json_encode(set[name]) :
                            set[name]
                        );
                break;
            };
        };

        // console.log(uri_variables ? anp.url_decode(atob(decodeURIComponent(uri_variables.substr(4)))) : null);

        // console.log(uri_variables ? anp.json_decode(anp.base64_decode(uri_variables.replace(/^[^=]+=(.+)$/, "$1"))) : null);
        // console.log(uri_variables ? anp.json_decode(decodeURIComponent(anp.base64_decode(decodeURIComponent(uri_variables.replace(/^[^=]+=(.+)$/, "$1"))))) : null);

        ajax.open(method, url, anp.settings_get(["ajax_asynchronous", "asynchronous"], input, true, false));
        ajax.timeout = timeout;
        headers.forEach(header => anp.is_array(header) && header.length == 2 && ajax.setRequestHeader(header[0], header[1]));
        mime && ajax.overrideMimeType(mime);
        ajax.onreadystatechange = () => {
            if(ended)
                return;
            if(ajax.readyState == 4)
                end("OK");
            else if(Date.now() - date > timeout)
                end("FORCED_TIMEOUT");
        };
        ajax.send(uri_variables);

        ajax.ontimeout = () => end("TIMEOUT");
        ajax.onabort = () => end("ABORTED");
        ajax.onerror = () => end("HTTP_ERROR");

        return ajax;
    };

    this.send = (method, url, variables, callback, timeout, loader) => {
        // console.log(url);

        if(variables && variables.__subdata__)
            delete variables.__subdata__;

        // console.log(variables);

        return self.load({
            url : url,
            method : method,
            timeout : timeout || anp.settings_get(["ajax_timeout", "timeout"]),
            loader : loader,
            mime : "text/plain; charset=utf-8"
        }, {
            language_id : anp.i18n.get_id(),
            ...(variables || {})
        }, (response, http_code, state, ok, message) => {
            // console.log([response, http_code, state, ok, message]);
            if(state != 4){
                anp.toast("warn", "ajax_ready_state_" + state, {
                    url : url,
                    code : http_code,
                    state : state
                });
                callback(null, false);
                return;
            };

            const data = anp.json_decode(anp.utf8_decode(response));

            if(!data){
                anp.toast("warn", "ajax_no_response", {
                    url : url,
                    code : http_code,
                    state : state
                });
                callback(null, false);
                return;
            };

            if(!variables.__no_content && anp.null_or_undefined(data.content)){
                anp.toast("warn", "ajax_content_null", {
                    url : url,
                    code : http_code,
                    state : state
                });
                callback(null, false);
                return;
            };

            if(anp.errors.has(data.content.error)){
                anp.errors.validate(
                    data.content.error, 
                    [
                        "no_driver_key"
                    ], 
                    {
                        url : url,
                        http_code : http_code,
                        state : state
                    }, 
                    "anp_controller_error"
                );
                callback(null, false);
                return;
            }
            
            if(anp.errors.has(data.content.driver_error)){
                anp.errors.validate(
                    data.content.driver_error, 
                    [
                        "commit_exception",
                        "rollback_posterror_exception",
                        "no_connection",
                        "connection_exception",
                        "no_query",
                        "no_query_connection"
                    ], 
                    {
                        url : url,
                        http_code : http_code,
                        state : state
                    }, 
                    "anp_server_error"
                );
                callback(null, false);
                return;
            };

            const sub_ok = anp.is_bool(data.ok) ? data.ok : true;

            !sub_ok && anp.toast("warn", "ajax_route_error", {
                url : url,
                code : data.code,
                state : 4,
                message : anp.i18n_get(data.message)
            });

            callback(data.content, sub_ok);

        });
    };

    this.get = (url, callback) => self.load(url, callback);

    this.post = (url, variables, callback) => self.load({
        url : url,
        method : "post"
    }, variables, callback);

    construct();

};
