AnP.Base = function(anp, input, callback){

    const self = this,
          screen = {x : 0, y : 0, zoom : 0},
          events = {};
    let built = false,
        thread = null,
        toast_timeout = null;

    let item_self = this.item_self;
    let hash_self = this.hash_self;

    const construct = () => {

        if(anp.settings_get(["base_autobuild", "autobuild"], input))
            self.build(callback);
        else
            anp.is_function(callback) && callback();

    };

    const set_autosize = () => {

        const minimum_font_size = Number(item_self.getAttribute("data-minimum-font-size")),
              font_size = screen.zoom * screen[screen.x < screen.y ? "x" : "y"] / Number(item_self.getAttribute("data-cells"));

        item_self.setAttribute("data-direction", screen.x == screen.y ? "square" : screen.x > screen.y ? "horizontal" : "vertical");
        item_self.style.fontSize = (font_size < minimum_font_size ? minimum_font_size : font_size) + "px";

    };

    const thread_function = () => {
        if(!item_self)
            return;

        const zoom = Number(item_self.getAttribute("data-zoom"));

        anp.with_toasts() && item_self.querySelectorAll(".toasts>.toast").forEach(toast => Date.now() - Number(toast.getAttribute("data-date")) > toast_timeout && toast.remove());

        if(screen.x == item_self.offsetWidth && screen.y == item_self.offsetHeight && screen.zoom == zoom)
            return;

        screen.x = item_self.offsetWidth;
        screen.y = item_self.offsetHeight;
        screen.zoom = zoom;

        set_autosize();

    };

    const get_current_settings = () => {

        const cookie_settings_raw = JSON.parse(anp.cookie_get(anp.settings_get(["settings_cookie_key", "settings_key"], input)) || "[]"),
              cookie_settings = {},
              settings = {
                  cells : anp.settings_get("cells", input),
                  zoom : anp.settings_get("zoom_size", input),
                  minimum_font_size : anp.settings_get("minimum_font_size", input),
                  allow_breadcrumb : anp.settings_get("allow_breadcrumb", input),
                  allow_settings : anp.settings_get("allow_settings", input)
              };

        Object.keys(settings).forEach((key, i) => cookie_settings[key] = anp.null_or_undefined(cookie_settings_raw[i]) ? settings[key] : cookie_settings_raw[i]);
        ["allow_breadcrumb", "allow_settings"].forEach(key => cookie_settings[key] = !!cookie_settings[key]);

        return cookie_settings;
    };

    this.build = callback => {

        if(built)
            return;
        built = true;

        const selector = anp.settings_get("position", input);

        hash_self = self.hash_self = anp.hash();
        toast_timeout = anp.settings_get("toasts_timeout");

        new AnP.Preload(anp, selector, (position, status) => {

            if(!position){
                console.error(anp.i18n_get("start_position_bad", {
                    code : status,
                    selector : anp.is_string(selector) ? selector : selector ? "object" : "null"
                }));
                return;
            };

            const git = anp.settings_get("application_git", input),
                  link = anp.settings_get("application_url", input),
                  logo = anp.settings_get("application_logo", input),
                  application = anp.settings_get("application_name", input),
                  preloader = anp.hash(),
                  cookie_settings = get_current_settings(),
                  settings = {
                      cells : anp.settings_get("cells", input),
                      zoom : anp.settings_get("zoom_size", input),
                      minimum_font_size : anp.settings_get("minimum_font_size", input),
                      allow_breadcrumb : anp.settings_get("allow_breadcrumb", input),
                      allow_settings : anp.settings_get("allow_settings", input)
                  };
            let class_attribute = "",
                main_menu = ``,
                main_menu_items = anp.settings_get("main_menu_items", input);

            main_menu_items && (anp.is_array(main_menu_items) ? main_menu_items : [main_menu_items]).forEach(item => {

                const i18n_field = anp.settings_get(["default_i18n", "i18n", "text"], [item, input]);

                main_menu += `<li` + anp.attr.set({
                    data_hidden : !!anp.settings_get(["data_hidden", "hidden"], [item, input])
                }) + `><a` + anp.attr.set([{
                    href : anp.settings_get(["default_url", "url"], [item, input]),
                    data_i18n : i18n_field,
                    title : anp.i18n_get(i18n_field),
                    target : anp.settings_get(["default_target", "target"], [item, input]),
                    data_i18n_without : true
                }, item.permissions ? {permissions : item.permissions, data_visible : false} : null]) + `>` +
                    anp.comp.icon(anp.settings_get(["default_icon", "icon"], [item, input])) +
                    anp.comp.i18n(i18n_field, "span") +
                `</a></li>`;

            });

            ["default_class", "class"].forEach(key => {

                const _class = anp.settings_get(key, input);

                _class && (class_attribute += _class + " ");

            });

            anp.attr.set(item_self = self.item_self = position.appendChild(document.createElement("div")), {
                id : hash_self = self.hash_self = anp.hash(),
                class : class_attribute + hash_self,
                data_hash : hash_self,
                data_application : application,
                data_git : git,
                data_url : link,
                data_cells : cookie_settings.cells,
                data_zoom : cookie_settings.zoom,
                data_direction : "unknown",
                data_allow_users : anp.settings_get("allow_users", input),
                data_minimum_font_size : cookie_settings.minimum_font_size,
                data_allow_breadcrumb : cookie_settings.allow_breadcrumb,
                data_allow_settings : cookie_settings.allow_settings, 
                data_gui_mode : anp.is_dark_mode() ? "dark" : "light", 
                data_mobile : anp.is_mobile(), 
                data_gui_mode_forced : anp.get_gui_mode_forced(), 
                data_mobile_forced : anp.get_device_mode_forced()
            });

            item_self.innerHTML = (`
                <div` + anp.attr.set({class : "html-cache-container"}) + `></div>
                <header>
                    <h1` + anp.attr.set({
                        class : "logo",
                        title : application
                    }) + `><a` + anp.attr.set({
                        href : link,
                        target : "_self"
                    }) + `>
                        ` + anp.comp.images.create({src : logo, alt : application}) + `
                        <span` + anp.attr.set({class : "text"}) + `>` + application + `</span>
                    </a></h1>
                    <nav` + anp.attr.set({class : "main-menu"}) + `>
                        <ul>` + main_menu + `</ul>
                    </nav>
                    ` + (anp.sessions ? anp.sessions.create() : ``) + `
                    <nav` + anp.attr.set({class : "breadcrumb"}) + `></nav>
                </header>
                <main` + anp.attr.set({class : "body"}) + `></main>
                <footer>
                    <a` + anp.attr.set({
                        class : "license",
                        href : anp.settings_get("license_link", input),
                        data_i18n : "license_text_full",
                        data_i18n_without : true,
                        title : anp.i18n_get("license_text_full", input),
                        target : "_blank"
                    }) + `>
                        <span` + anp.attr.set({data_i18n : "license_text"}) + `>` + anp.i18n_get("license_text", input) + `</span>
                        ` + anp.comp.images.create({src : anp.settings_get("license_icon", input), alt : anp.settings_get("license_type", input)}) + `
                    </a>
                    ` + (anp.i18n ? anp.i18n.create() : ``) + `
                    ` + anp.comp.group({
                        class : "settings-basics",
                        items : [
                            {type : "button", i18n : "less_zoom", icon : "less_zoom", onmousedown : "{object_name}.base.less_zoom_start(this, event);", onmouseup : "{object_name}.base.less_zoom_end(this, event);"},
                            {type : "number", i18n : "zoom", step : 1, min : 1, max : 500, value : cookie_settings.zoom * 100 >> 0, onchange : "{object_name}.base.set_new_zoom(this, event);"},
                            {type : "button", i18n : "reset_zoom", icon : "reset_zoom", onclick : "{object_name}.base.reset_zoom(this, event);"},
                            {type : "button", i18n : "settings", icon : "settings", onclick : "{object_name}.base.show_settings(this, event);", visible : !!anp.comp.forms, allowed : cookie_settings.allow_settings},
                            {type : "button", i18n : "more_zoom", icon : "more_zoom", onmousedown : "{object_name}.base.more_zoom_start(this, event);", onmouseup : "{object_name}.base.more_zoom_end(this, event);"}
                        ]
                    }) + `
                </footer>
                ` + (anp.comp.forms ? anp.comp.forms.create({
                    class : "settings-basics-form",
                    i18n : "settings_basics",
                    data_show : false,
                    structure : [
                        {i18n : "cells", name : "cells", type : "number", min : 1, max : 200, value : settings.cells, onchange : "{object_name}.base.set_new_cells(this, event);"},
                        {i18n : "zoom", name : "zoom", type : "group", items : [
                            {type : "number", name : "zoom", min : 1, max : 500, value : settings.zoom * 100 >> 0, onchange : "{object_name}.base.set_new_zoom(this, event);"},
                            {type : "icon", icon : "percentage"}
                        ]},
                        {i18n : "minimum_font_size", name : "minimum_font_size", type : "group", items : [
                            {type : "number", name : "minimum_font_size", min : 1, max : 64, value : settings.minimum_font_size, onchange : "{object_name}.base.set_new_minimum_font_size(this, event);"},
                            {type : "icon", icon : "pixels"}
                        ]},
                        {i18n : "allow_breadcrumb", name : "allow_breadcrumb", type : "checkbox", checked : settings.allow_breadcrumb, onchange : "{object_name}.base.set_allow_breadcrumb(this, event);"},
                        {i18n : "allow_settings", name : "allow_settings", type : "checkbox", checked : settings.allow_settings, onchange : "{object_name}.base.set_allow_settings(this, event);"}, 
                        {i18n : "force_gui_mode", name : "gui_mode", type : "selector", options : [
                            {i18n : "default", value : "default"}, 
                            {i18n : "light", value : "light"}, 
                            {i18n : "dark", value : "dark"}
                        ]}, 
                        {i18n : "force_mobile", name : "mobile", type : "selector", options : [
                            {i18n : "default", name : "default"}, 
                            {i18n : "desktop", value : "desktop"}, 
                            {i18n : "mobile", value : "mobile"}
                        ]}
                    ],
                    buttons : [
                        {i18n : "reset", icon : "reset", onclick : "{object_name}.base.reset_settings(this, event);"},
                        {i18n : "back", icon : "back", onclick : "{object_name}.base.close_settings(this, event);"}
                    ]
                }) : ``) + `
                ` + (anp.with_toasts() ? `<ul` + anp.attr.set({class : "toasts"}) + `></ul>` : ``) + `
                <div` + anp.attr.set({class : "anp-loader", data_executing : false}) + `></div>
                <div` + anp.attr.set({data_preloader : preloader}) + `></div>
            `);

            new AnP.Preload(anp, "[data-preloader=" + preloader + "]", (preloader_item, status, asynchronous) => {

                if(!preloader_item){
                    console.error(anp.i18n_get("base_preloader_error", {
                        code : status,
                        hash : preloader,
                        asynchronous : asynchronous
                    }));
                    return;
                };

                preloader_item.remove();

                thread = anp.threads.add(thread_function);
                setTimeout(() => anp.routes.go(true), 50);

                if(anp.sessions)
                    anp.sessions.start(callback);
                else
                    anp.is_function(callback) && callback();

            });

        });

    };

    this.update_breadcrumb = route => {

        let html = `<a href="#/" target="_self" title="/">/</a>`,
            current = "";

        route.replace(/^\//, "").split("/").forEach(fragment => html += `<a href="#` + (current += "/" + fragment) + `" target="_self" title="` + fragment + `">` + fragment + `</a>`);
        anp.item_self.querySelector("nav.breadcrumb").innerHTML = html;

    };

    this.is_built = () => !!built;

    const save_settings = () => anp.cookie_set(anp.settings_get(["settings_cookie_key", "settings_key"]), JSON.stringify([
        Number(item_self.getAttribute("data-cells")) >> 0,
        Number(item_self.getAttribute("data-zoom")),
        Number(item_self.getAttribute("data-minimum-font-size")) >> 0,
        item_self.getAttribute("data-allow-breadcrumb") == "true" ? 1 : 0,
        item_self.getAttribute("data-allow-settings") == "true" ? 1 : 0
    ]), anp.settings_get("settings_expire"));

    this.set_new_zoom = (item, event) => {
        item_self.setAttribute("data-zoom", Number(item.value) / 100);
        item_self.querySelector(".settings-basics [type=number]").value = item.value;
        save_settings();
        set_autosize();
    };

    const set_zoom = value => {

        const field = item_self.querySelector(".settings-basics [type=number]");

        field.value = Number(field.value) + value;
        self.set_new_zoom(field);

    };

    this.less_zoom_start = (item, event) => {

        const field = item_self.querySelector(".settings-basics [type=number]");

        set_zoom(-1);
        events.less_zoom_timeout = setTimeout(() => events.less_zoom_thread = anp.threads.add(() => Number(field.value) > Number(field.getAttribute("min")) && set_zoom(-1)), anp.settings_get(["zoom_timeout", "timeout"]));

    };

    this.less_zoom_end = (item, event) => {

        clearTimeout(events.less_zoom_timeout);
        anp.threads.remove(events.less_zoom_thread);

    };

    this.more_zoom_start = (item, event) => {

        const field = item_self.querySelector(".settings-basics [type=number]");

        set_zoom(1);
        events.more_zoom_timeout = setTimeout(() => events.more_zoom_thread = anp.threads.add(() => Number(field.value) < Number(field.getAttribute("max")) && set_zoom(1)), anp.settings_get(["zoom_timeout", "timeout"]));

    };

    this.more_zoom_end = (item, event) => {

        clearTimeout(events.more_zoom_timeout);
        anp.threads.remove(events.more_zoom_thread);

    };

    this.reset_zoom = (item, event) => {

        const field = item_self.querySelector(".settings-basics [type=number]");

        field.value = 100;
        self.set_new_zoom(field);

    };

    this.show_settings = (item, event) => {

        const form = item_self.querySelector(".settings-basics-form"),
              settings = get_current_settings();

        settings.zoom = settings.zoom * 100 >> 0;
        anp.comp.forms.set(form, settings);
        form.setAttribute("data-show", true);

    };

    this.close_settings = (item, event) => {

        item_self.querySelector(".settings-basics-form").setAttribute("data-show", false);
        item_self.querySelector(".settings-basics [type=number]").value = Number(item_self.getAttribute("data-zoom")) * 100;

    };

    this.set_new_cells = (item, event) => {

        item_self.setAttribute("data-cells", item.value);
        save_settings();
        set_autosize();

    };

    this.set_new_minimum_font_size = (item, event) => {

        item_self.setAttribute("data-minimum-font-size", item.value);
        save_settings();
        set_autosize();

    };

    this.set_allow_breadcrumb = (item, event) => {

        item_self.setAttribute("data-allow-breadcrumb", item.checked);
        save_settings();

    };

    this.set_allow_settings = (item, event) => {

        item_self.setAttribute("data-allow-settings", item.checked);
        save_settings();

    };

    // Crear guardar y cargar configuración.
    const set_settings = settings => {

        item_self.setAttribute("data-cells", settings.cells);
        item_self.setAttribute("data-zoom", settings.zoom);
        item_self.setAttribute("data-minimum-font-size", settings.minimum_font_size);
        item_self.setAttribute("data-allow-breadcrumb", !!settings.allow_breadcrumb);
        item_self.setAttribute("data-allow-settings", !!settings.allow_settings);

        save_settings();
        self.show_settings(item, event);

    };

    this.reset_settings = (item, event) => anp.confirm("confirm_reset_basic_settings", () => set_settings({
        cells : anp.settings_get("cells", input),
        zoom : anp.settings_get("zoom_size", input),
        minimum_font_size : anp.settings_get("minimum_font_size", input),
        allow_breadcrumb : anp.settings_get("allow_breadcrumb", input),
        allow_settings : anp.settings_get("allow_settings", input)
    }));

    construct();

};
