AnP.Components = function(anp, input){

    const self = this,
          cache = {};
    let started = false,
         thread = null,
         float_box_click = null,
         float_box_scroll = null,
         float_box_z = 0;

    let images = this.images;
    let forms = this.forms;
    let files = this.files;
    let tables = this.tables;
    let static_tables = this.static_tables;
    let calendars = this.calendars;
    let selectors = this.selectors;
    let marks_module = this.marks_module;
    let maps = this.maps;
    let colors = this.colors;
    let selects_lists = this.selects_lists;
    let syntax_highlighter = this.syntax_highlighter;

    const construct = () => {

        AnP.Components.Images && (images = self.images = new AnP.Components.Images(anp, input));
        AnP.Components.Files && (files = self.files = new AnP.Components.Files(anp, input));
        AnP.Components.Selectors && (selectors = self.selectors = new AnP.Components.Selectors(anp, input));
        AnP.Components.Forms && (forms = self.forms = new AnP.Components.Forms(anp, input));
        AnP.Components.Tables && (tables = self.tables = new AnP.Components.Tables(anp, input));
        AnP.Components.StaticTables && (static_tables = self.static_tables = new AnP.Components.StaticTables(anp, input));
        AnP.Components.Calendars && (calendars = self.calendars = new AnP.Components.Calendars(anp, input));
        AnP.Components.Marks && (marks_module = self.marks_module = new AnP.Components.Marks(anp, input));
        AnP.Components.Maps && (maps = self.maps = new AnP.Components.Maps(anp, input));
        AnP.Components.Colors && (colors = self.colors = new AnP.Components.Colors(anp, input));
        AnP.Components.SelectsLists && (selects_lists = self.selects_lists = new AnP.Components.SelectsLists(anp, input));
        AnP.Components.SyntaxHighlighter && (syntax_highlighter = self.syntax_highlighter = new AnP.Components.SyntaxHighlighter(anp, input));

    };

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp.launch(self, [
            "images", "files", "selectors", "forms", "tables", "calendars", 
            "marks_module", "maps", "colors", "selects_lists", 
            "syntax_highlighter"
        ], () => {

            float_box_z = anp.settings_get("float_box_z_start");
            thread = anp.threads.add(cache_function);
            float_box_click = anp.item_self.addEventListener("click", float_box_method);
            float_box_scroll = anp.item_self.addEventListener("scroll", float_box_method);

            anp.is_function(callback) && callback();

        });

    };

    const cache_function = () => {

        const hashes = [];

        anp.item_self.querySelectorAll(".input[data-type=text]").forEach(field => {

            const hash = field.getAttribute("data-hash"),
                  input = field.querySelector("input,textarea");

            hashes.push(hash);

            if(cache[hash] !== input.value){

                const setter = cache[hash] === undefined;

                field.querySelector(".length").innerText = (cache[hash] = input.value).length;
                
                !setter && input.hasAttribute("data-on-change") && 
                eval(input.getAttribute("data-on-change").replace(/\bthis\b/, "input"));

            };

        });

        for(const key in cache)
            if(!hashes.includes(key))
                delete cache[key];

    };

    const get_float_box = item => {

        if(item)
            while(item.hasAttribute && (!item.hasAttribute("data-module-box") || item.getAttribute("data-module-box") != "float_box") && (item = item.parentNode));

        return item && item.hasAttribute ? item : null;
    };

    const get_label = item => {

        if(item)
            while(item.tagName && item.tagName.toLowerCase() != "label" && (item = item.parentNode));

        return item && item.hasAttribute ? item : null;
    };

    const float_box_method = event => {

        let hash = null,
            box = get_float_box(event.target);

        if(box && box.hasAttribute && box.hasAttribute("data-for"))
            hash = box.getAttribute("data-for");
        else{

            const label = get_label(event.target);

            label && label.hasAttribute && label.hasAttribute("for") && (hash = label.getAttribute("for"));

        };

        anp.item_self.querySelectorAll("[data-module-box=float_box]").forEach(box => box.getAttribute("data-for") != hash && box.remove());

    };

    this.i18n = (i18n, tag, variables, _default, dotted) => `<` + (tag = (tag || anp.settings_get("default_tag")).toLowerCase()) + anp.attr.set({
        i18n : i18n,
        data_dotted : dotted || false, 
        ...(anp.is_object(variables) ? {data_i18n_variables : anp.base64_encode(anp.json_encode(variables))} : {})
    }) + `>` + anp.i18n_get(i18n, variables, _default) + (dotted ? "..." : "") + `</` + tag + `>`;

    this.icon = icon => `<span` + anp.attr.set({icon : typeof icon == "object" ? icon.icon : icon}) + `></span>`;

    this.full_item = (name, tag) => (`
        <` + (tag = (tag || anp.settings_get("default_tag")).toLowerCase()) + anp.attr.set({i18n : name, i18n_without : true, title : anp.i18n_get(name)}) + `>
            ` + self.icon(name) + `
            ` + self.i18n(name, "span") + `
        </` + tag + `>
    `);

    this.error = code => (`
        <section` + anp.attr.set({
            class : "error-" + code,
            data_i18n : "error_" + code,
            data_i18n_without : true,
            title : anp.i18n_get("error_" + code),
            data_code : code
        }) + `>
            ` + self.i18n("error_" + code, "h1") + `
            ` + self.i18n("error_" + code + "_text", "h3") + `
            ` + self.i18n("error_" + code + "_description", "p") + `
        </section>
    `);

    this.toggle = (item, event) => !item.hasAttribute("disabled") && item.setAttribute("data-toggle", !item.hasAttribute("data-toggle") || item.getAttribute("data-toggle") != "true");

    this.button = input => {

        input.toggle !== undefined && (input.onclick = anp.object_name + ".comp.toggle(this, event);" + (input.onclick || ""));
        if(input.go){
            input.onclick = anp.object_name + ".go('" + input.go + "', " + (["new", "blank"].some(key => input[key] || input["_" + key]) ? "true" : "false") + ")"
            delete input.go;
        }else if(input.go_save){
            input.onclick = anp.object_name + ".comp.forms.go_save(" + (input.view_key ? "'" + input.view_key + "'" : "null") + ", '" + input.go_save + "', " + (["new", "blank"].some(key => input[key] || input["_" + key]) ? "true" : "false") + ")"
            delete input.go_save;
        }else if(input.go_remove){
            input.onclick = anp.object_name + ".comp.forms.go_remove(" + (input.view_key ? "'" + input.view_key + "'" : "null") + ", '" + input.go_remove + "', " + (["new", "blank"].some(key => input[key] || input["_" + key]) ? "true" : "false") + ")"
            delete input.go_remove;
        };
        input.url_pointer && typeof input.url_pointer_here != "boolean" && (input.url_pointer_here = anp.get_url_pointer(input.url_pointer));

        return (`
            <button` + anp.attr.set(Object.assign({
                type : "button"
            }, input, {
                class : "button" + (input.class ? " " + input.class : "")
            }, input.i18n ? {data_i18n_without : true, title : anp.i18n_get(input.i18n)} : null, input.go ? {onclick : (input.onclick ? input.onclick + ";" : "") + anp.object_name + ".go('" + input.go + "');"} : null), ["icon", "go"]) + `>
                ` + (input.icon ? self.icon(input.icon) : ``) + `
                ` + (input.i18n ? self.i18n(input.i18n, "span") : ``) + `
            </button>
        `);
    };

    this.buttons = input => {

        if(!input)
            input = {};
        else if(anp.is_array(input))
            input = {buttons : input};
        else if(!anp.is_dictionary(input))
            input = {};
        input.class && (input.class += " buttons");

        let html = `<div` + anp.attr.set(Object.assign({class : "buttons"}, input), "buttons") + `>`;
        
        input.buttons && (anp.is_array(input.buttons) ? input.buttons : [input.buttons]).forEach(button => button && (html += self.button(button)));

        return html + `</div>`;
    };

    const i18n_field = input => {

        let i18n = null;

        if(input){
            !input.i18n && input.name && (input.i18n = input.name);
            if(input.i18n){
                i18n = anp.i18n_get(input.i18n);
                i18n = {data_i18n_without : true, title : i18n, placeholder : i18n + "..."};
            };
        };

        return i18n;
    };

    const create_input_text = (html, input) => (`
        <span` + anp.attr.set({
            class : "input",
            data_type : "text",
            data_maximum : input.maxlength || 0,
            data_hash : anp.hash(),
            data_focus : false,
            data_disabled : input.disabled || false
        }) + `>
            ` + html + `
            <span` + anp.attr.set({class : "data"}) + `>
                <span` + anp.attr.set({
                    class : "maximum",
                    data_has : !isNaN(input.maxlength)
                }) + `>` + (input.maxlength || 0) + `</span>` +
                `<span` + anp.attr.set({class : "length"}) + `>0</span>
            </span>
        </span>
    `);

    const create_input_number = (html, input) => (`
        <span` + anp.attr.set({
            class : "input",
            data_type : "number",
            data_maximum : input.maxlength || 0,
            data_hash : anp.hash(),
            data_focus : false,
            data_disabled : !!input.disabled
        }) + `>
            ` + html + `
            <span` + anp.attr.set({class : "data"}) + `>
                <span` + anp.attr.set({
                    class : "minimum",
                    data_has : !isNaN(input.min)
                }) + `>` + (input.min || 0) + `</span>` +
                `<span` + anp.attr.set({
                    class : "maximum",
                    data_has : !isNaN(input.max)
                }) + `>` + (input.max || 0) + `</span>
            </span>
        </span>
    `);

    const focus_input = input => {return {
        onfocusin : anp.object_name + ".comp.on_focus_in(this, event);" + (input.focus_in || input.onfocusin || ""),
        onfocusout : anp.object_name + ".comp.on_focus_out(this, event);" + (input.focus_out || input.onfocusout || "")
    }};

    this.on_focus_in = (item, event) => item.parentNode.setAttribute("data-focus", true);
    this.on_focus_out = (item, event) => item.parentNode.setAttribute("data-focus", false);

    this.text = input => create_input_text(input && input.multiline ? `<textarea` + anp.attr.set([input, i18n_field(input), focus_input(input)], "value") + `>` + (input.value || ``) + `</textarea>` : `<input` + anp.attr.set(Object.assign({
        type : "text"
    }, input, i18n_field(input), focus_input(input))) + ` />`, input);

    this.password = input => create_input_text(`<input` + anp.attr.set(Object.assign({
        type : "password"
    }, input, i18n_field(input), focus_input(input))) + ` />`, input);

    this.mail = input => create_input_text(`<input` + anp.attr.set(Object.assign({pattern : "^[\\@]+\\@[^\\.]+(\\.[^\\.]+)+$"}, input, {
        type : "email"
    }, i18n_field(input), focus_input(input))) + ` />`, input);

    this.number = input => create_input_number(`<input` + anp.attr.set(Object.assign({
        type : "number"
    }, input, i18n_field(input), focus_input(input))) + ` />`, input);

    this.checkbox = input => {
        !input && (input = {});

        !input.id && (input.id = anp.hash());
        !input.type && (input.type = "checkbox");

        return (`
            <label` + anp.attr.set([{
                class : "checkbox",
                for : input.id
            }, input.i18n ? {
                data_i18n : input.i18n,
                data_i18n_without : true,
                title : anp.i18n_get(input.i18n)
            } : null]) + `>
                <input` + anp.attr.set(input) + ` />
                ` + self.icon("checkbox") + `
                ` + (!input.no_text && input.i18n ? self.i18n(input.i18n, "span") : ``) + `
            </label>
        `);
    };

    this.radio = this.radiobutton = input => self.checkbox({...(anp.is_dictionary(input) ? input : {}), type : "radio"});

    this.group = inputs => {

        let html = `<span` + anp.attr.set({
            class : "group comp-group" + (inputs && inputs.class ? " " + inputs.class : ""),
            data_visible : anp.is_bool(inputs.visible) ? inputs.visible : true
        }) + `>`;

        inputs && (anp.is_array(inputs.items ? inputs = inputs.items : inputs) ? inputs : [inputs]).forEach(input => anp.is_string(input) ? (html += input) : anp.is_dictionary(input) && (html += input.type && self[input.type] ? self[input.type || "text"](input) : ``));

        return html + `</span>`;
    };

    this.html = html => html ? html.html || html : ``;

    this.multiview = input => {
        // console.log(input);

        let html = `<div` + anp.attr.set({class : "multiview" + (input.class ? " " + input.class : "")}) + `>`;

        input.views && (anp.is_array(input.views) ? input.views : [input.views]).forEach(view => view && (html += anp.views.get(view)));

        return html + `</div>`;
    };

    this.menu = input => {

        !anp.is_dictionary(input) && (input = {});

        let html = `<nav` + anp.attr.set([input, {
            class : "menu group" + (input.class ? " " + input.class : ""),
            data_direction : input.direction || anp.settings_get("default_menu_direction"),
            data_module : "menu"
        }], ["buttons", "options", "direction"]) + `>`;
        const options = input.options || input.buttons;

        options && (anp.is_array(options) ? options : [options]).forEach(option => option && (html += self.button(option)));

        return html + `</nav>`;
    };

    this.create_float_box = (item, event, input) => {

        const label = get_label(item),
               hash = label.hasAttribute && label.hasAttribute("for") ? label.getAttribute("for") : null;

        if(!hash || anp.item_self.querySelector("[data-module-box=float_box][data-for=" + hash + "]"))
            return;

        !anp.is_dictionary(input) && (input = {});

        const body = document.querySelector("body"),
              box = anp.attr.set(anp.item_self.appendChild(document.createElement("form")), [{
                  method : "get",
                  action : "#",
                  onsubmit : "return false;",
                  data_module_box : "float_box",
                  data_for : hash,
                  style : (
                      "z-index:" + (float_box_z ++) + ";" +
                      (input.left ? "left : " + input.left : input.right ? "right : " + input.right : body.offsetWidth / 2 > event.clientX ? "left : " + event.clientX : "right : " + (body.offsetWidth - event.clientX)) + "px;" +
                      (input.top ? "top : " + input.top : input.bottom ? "bottom : " + input.bottom : body.offsetHeight / 2 > event.clientY ? "top : " + event.clientY : "bottom : " + (body.offsetHeight - event.clientY)) + "px;" +
                      (input.width ? "width : " + input.width + "px;" : "") +
                      (input.height ? "height : " + input.height + "px;" : "")
                  )
              }, input.name ? {data_module : input.name} : null]);

        box.innerHTML = input.html || ``;

    };

    this.range = input => `<input` + anp.attr.set({type : "range", ...input, ...(input.i18n ? {
        data_i18n_without : true,
        title : anp.i18n.get(input.i18n)
    } : {})}) + ` />`;

    this.money = input => {

        const id = input.id || input.hash;
              
        return self.group({
            class : "money-field",
            data_visible : false,
            items : [
                {
                    type : "number", 
                    i18n : input.i18n || input.name, 
                    name : input.name || input.i18n, 
                    ...(id ? {id : id} : {}), 
                    ...(input.range ? {
                        min : range[0],
                        max : range[1]
                    } : {}), 
                    ...(isNaN(input.value) ? {} : {value : input.value}), 
                    ...(input.maximum ? {max : input.maximum} : {}),
                    step : input.step || .01,
                    disabled : input.disabled,
                    readonly : input.readonly, 
                    min : input.min, 
                    max : input.max
                },
                `<label` + anp.attr.set({data_symbol : input.symbol || input.name || input.i18n}) + `></label>`
            ]
        });
    };

    this.null = () => '<span' + anp.attr.set({class : "null-value"}) + '></span>';

    this.html_field = input => `<span` + anp.attr.set({
        ...input,
        data_type : "html",
        data_name : input.name,
        data_has_content : !!input.html,
        data_subdata : input.subdata ? anp.base64_encode(anp.json_encode(input.subdata)) : null
    }, ["type", "html", "name", "subdata"]) + `>` + (
        self.i18n(input.name, "span", null, null, true) + 
        `<span` + anp.attr.set({class : "content"}) + `>` + (input.html || "") + `</span>`
    ) + `</span>`;

    this.select_list = input => selects_lists ? selects_lists.build(input) : '';

    this.url = input => {
        if(anp.null_or_undefined(input))
            return "-";

        if(!((anp.is_string(input) ? input = {url : input} : input).url))
            return self.text({
                maxlength : 256,
                ...input,
                type : "text"
            });

        let text;

        return `<a` + anp.attr.set([{
            href : input.url,
            target : input.target || anp.settings_get("url_target")
        }, input.i18n ? {
            data_i18n : input.i18n,
            title : text = anp.i18n.get(input.i18n, input.text)
        } : {title : text = anp.is_string(input.text) ? input.text : input.url}]) + `>` + text + `</a>`;
    };

    construct();

};
