AnP.Errors = function(anp, input){

    const self = this;
    let alphabet = null;

    const construct = () => {};

    this.null_or_undefined = value => value === undefined ? 1 << 0 : value === null ? 1 << 1 : 0;
    this.null_or_undefined_messages = name => [
        (name || (name = "value")) + "_undefined",
        name + "_null"
    ];

    this.empty = value => self.null_or_undefined(value) || ((anp.is_array(value) ? value.length : anp.is_object(value) ? Object.keys(value).length : value) ? 0 : 1 << 2);
    this.empty_messages = name => self.null_or_undefined_messages(name).concat([(name || "value") + "_empty"]);

    this.number = value => self.null_or_undefined(value) || (isNaN(value) ? 1 << 2 : 0);
    this.number_messages = name => self.null_or_undefined_messages(name).concat([(name || "value") + "_nan"]);

    this.integer = value => self.number(value) || (value === value >> 0 ? 0 : 1 << 3);
    this.integer_messages = name => self.number_messages(name).concat([(name || "value") + "_not_integer"]);

    this.id = value => self.integer(value) || (value < 1 ? 1 << 4 : 0);
    this.id_messages = name => self.integer_messages(name).concat([(name || "value") + "_not_id"]);

    this.bool = value => self.null_or_undefined(value) || (anp.is_bool(value) ? 0 : 1 << 2);
    this.bool_messages = name => self.null_or_undefined_messages(name).concat([(name || "value") + "_not_bool"]);

    const get_alphabet = this.get_alphabet = () => alphabet || (alphabet = anp.settings_get("base64_alphabet"));

    const to_array = this.to_array = (code, length) => {

        const alphabet = get_alphabet();
        let array = [];

        if(anp.is_string(code))
            array = code.split("").map(hexa => alphabet.indexOf(hexa));
        else if(anp.is_integer(code)){
            while(code){
                array.push(code & 0x3F);
                code >>>= 6;
            };
        }else if(anp.is_array(code))
            array = code;

        while(array.length < length)
            array.push(alphabet[0]);

        return array && array.length ? array : [0];
    };

    const to_array_binary = this.to_array_binary = code => (code = to_array(code)).map(hexa => ("000000" + hexa.toString(2)).slice(-6));

    const to_integer = this.to_integer = (code, length) => {
        if(anp.is_integer(code))
            return code;
        if(anp.is_array(code))
            return code.length ? code.length > 1 ? code.reduce((total, hexa, i) => total | (hexa << i * 6)) : code[0] : 0;
        if(!anp.is_string(code))
            return 0;

        const alphabet = get_alphabet();

        return code ? code.length > 1 ? code.split("").reduce((total, hexa, i) => (i > 1 ? total : alphabet.indexOf(total)) | (alphabet.indexOf(hexa) << i * 6)) : alphabet.indexOf(code) : 0;
    };

    const to_string = this.to_string = (code, length) => {

        let string = "";

        if(anp.is_string(code))
            string = code;
        else{

            const alphabet = get_alphabet();

            if(anp.is_integer(code)){
                while(code){
                    string += alphabet[code & 0x3F];
                    code >>>= 6;
                };
            }else if(anp.is_array(code))
                string = code.length ? code.length > 1 ? code.reduce((total, hexa, i) => (i > 1 ? total : alphabet[total]) + alphabet[hexa]) : alphabet[code[0]] : alphabet[0];

        };

        while(string.length < length)
            string += alphabet[0];

        return string || alphabet[0];
    };

    const has = this.has = code => (
        anp.is_string(code) ? code.split("").some(hexa => hexa != get_alphabet()[0]) :
        anp.is_integer(code) ? !!code :
        anp.is_array(code) ? code.some(hexa => !!hexa) :
        null
    );

    const type = this.type = code => ["string", "integer", "array"].filter(key => anp["is_" + key](code))[0] || "unknown";

    const to_unknown = this.to_unknown = code => code;

    const compact = this.compact = code => {
        if(anp.is_string(code)){

            const alphabet = get_alphabet();

            while(code[code.length - 1] == alphabet[0])
                code = code.substr(0, code.length - 1);

            return code || alphabet[0];
        };
        if(anp.is_array(code)){
            while(!code[code.length - 1])
                code.pop();
            return code.length ? code : [0];
        };
        return code || 0;
    };

    const bitwise = this.bitwise = (code, bits) => {

        if(!bits || !has(code))
            return code;


        const reverse = bits < 0,
              start = (reverse ? bits *= -1 : bits) / 6 >> 0,
              rest = bits % 6,
              type_method = "to_" + type(code);

        if(reverse){

            code = to_string(code).substr(start);

            if(rest){
                if(code){

                    const results = [],
                          l = (code = to_array(code)).length - 1,
                          r = 6 - rest,
                          block = ~-(1 << rest);

                    for(let i = 0; i < l; i ++)
                        code[i] = (code[i] >> rest) | ((code[i + 1] & block) << r);
                    code[l] >>= rest;

                }else
                    code = [0];
            };

        }else{

            code = to_array(code);

            if(rest){

                const block = [~-(1 << (r = 6 - rest)), ~-(1 << rest) << r],
                      l = code.length - 1,
                      last = (code[l] & block[1]) >> r

                last && (code[l + 1] = last);
                for(let i = l; i > 0; i --)
                    code[i] = ((code[i] & block[0]) << rest) | (code[i - 1] >> r);
                code[0] = (code[0] & block[0]) << rest;

            };

            for(let i = 0; i < start; i ++)
                code.unshift(0);

        };

        return self[type_method](code);
    };

    const set = this.set = (code, error, bit, length) => {

        code = to_array(code);
        error = to_array(error);
        bit && (error = bitwise(error, bit));

        let i = bit / 6 >> 0;

        if(length){

            const ampersand = [],
                  start = bit % 6,
                  m = (length + start) / 6 >> 0,
                  end = (start + length) % 6;

            for(let j = 0; j < m; j ++)
                ampersand.push(0);
            ampersand[0] |= ~-(1 << start);
            end && ampersand.push(~-(1 << (6 - end)) << end);

            ampersand.forEach((hexa, j) => code[j + i] &= hexa);

        };

        if(has(error)){

            const l = error.length;

            while(code.length <= i)
                code.push(0);

            for(; i < l; i ++)
                code[i] = (code[i] || 0) | error[i];

        };

        return compact(to_string(code));
    };

    this.validate = (form, error, messages, variables, error_message, ok_message) => {

        if(!anp.null_or_undefined(form) && !anp.is_html_object(form)){
            [error, messages, variables, error_message, ok_message] = [form, error, messages, variables, error_message, ok_message];
            form = null;
        };

        anp.is_dictionary(ok_message) && ([ok_message, variables] = [variables, ok_message]);
        // form && (form = anp.comp.forms.get(form)).querySelectorAll(".error").forEach(error_field => error_field.innerHTML = ``);
        form && form.querySelectorAll(".error").forEach(error_field => error_field.innerHTML = ``);
        
        if(!self.has(error)){
            ok_message && anp.toast("ok", ok_message, variables, null, 1);
            return true;
        };

        const list = [],
              code = self.to_string(error);
        let html_list = ``;

        self.to_array(error).forEach((hexa, j) => {
            if(hexa)
                for(let i = 0; !i || (hexa >>= 1); i ++)
                    if(hexa & 1){

                        const k = i + (j * 6),
                              message = messages[k] || "error_" + k,
                              text = anp.i18n_get(message, [{code : code}, variables]);

                        list.push(message);
                        html_list += `<li` + anp.attr.set({
                            class : "error",
                            data_i18n : message,
                            title : text,
                            data_i18n_without : true, 
                            data_i : k
                        }) + `><span` + anp.attr.set({class : "i"}) + `>` + k + `</span><span` + anp.attr.set({
                            class : "message", 
                            data_i18n : message,
                            title : text,
                            data_i : k
                        }) + `>` + text + `</span></li>`;
                        
                    };
        });

        anp.toast("warn", error_message, [{
            code : code,
            list : `<ul` + anp.attr.set({class : "error-list"}) + `>` + html_list + `</ul>`
        }].concat(anp.is_dictionary(variables) ? [variables] : anp.is_array(variables) ? variables : []), null, 1);

        if(form){

            const error_all = form.querySelector(".structure+.error-all");

            error_all && (error_all.innerHTML = html_list);

        };

        return false;
    };

    construct();

};
