AnP.Preload = function(anp, selector, callback, autostart){

    const self = this;
    let status = 0;

    const OK = this.OK = 0;
    const EXCEPTION = this.EXCEPTION = 1 << 0;
    const SUCCESS = this.SUCCESS = 1 << 1;
    const NO_SELECTOR = this.NO_SELECTOR = 1 << 2;
    const BAD_SELECTOR = this.BAD_SELECTOR = 1 << 3;
    const TIMEOUT = this.TIMEOUT = 1 << 4;
    const ABORTED = this.ABORTED = 1 << 5;
    const LOADING = this.LOADING = 1 << 6;
    const STOPPED = this.STOPPED = 1 << 7;
    const NO_CALLBACK = this.NO_CALLBACK = 1 << 8;
    const ASYNCHRONOUS_SUCCESS = this.ASYNCHRONOUS_SUCCESS = 1 << 9;
    const NO_STRING_SELECTOR = this.NO_STRING_SELECTOR = 1 << 10;

    const construct = () => {

        (anp.is_bool(autostart) ? autostart : anp.settings_get(["preload_autostart", "autostart"])) && self.start();

    };

    this.status = () => 0 | status;

    this.start = () => {
        if(!anp.is_function(callback)){
            status |= NO_CALLBACK;
            return;
        };
        if(!selector){
            callback(null, status |= NO_SELECTOR, false);
            return;
        };
        if(anp.is_html_object(selector)){
            callback(selector, status |= SUCCESS, false);
            return;
        };
        if(!anp.is_string(selector)){
            callback(null, status |= NO_STRING_SELECTOR, false);
            return;
        };

        let item;

        try{
            if(item = anp.item_self.querySelector(selector)){
                callback(item, status |= SUCCESS, false);
                return;
            }
        }catch(exception){
            console.error(exception);
            callback(null, status |= BAD_SELECTOR, false);
            return;
        };

        const date = Date.now(),
              timeout = anp.settings_get(["preload_timeout", "timeout"]);
        let preload = anp.threads.add(() => {
            if(status & ABORTED){
                anp.threads.remove(preload);
            }else if(item = anp.item_self.querySelector(selector)){
                callback(item, status |= ASYNCHRONOUS_SUCCESS, true);
                anp.threads.remove(preload);
            }else if(Date.now() - date > timeout){
                callback(null, status |= TIMEOUT, true);
                anp.threads.remove(preload);
            }
        });

    };

    this.abort = () => status |= ABORTED;

    construct();

};
