AnP.Routes = function(anp, input){

    const self = this,
          routes = {},
          route_pattern = /^(\/([^\s]+)?)\s+([^\s]+)(\s+(.+?))?$/;
    let started = false,
        case_sensitive = false,
        last_url = null,
        url_gone = null,
        thread = null,
        variables = {},
        history = [],
        was_back = false,
        current_pattern = null,
        go_back_pattern = /\/(select|add|[0-9]+(\/(texts\/[0-9]+(\/edit)?|edit|history))?)$/;

    const construct = () => {};

    const add_item = (data, overwrite) => {

        const variables = [null],
              route = new RegExp("^" + anp.string_to_re(data.route).replace(/\\\{([a-z\d_]+)\\\}/gi, (...arguments) => {
                  variables.push(arguments[1]);
                  return "([^\\/]+)";
              }).replace(/^\\\/$|^(\\\/)?(.+)(\\\/)?$/, (...arguments) => {
                  if(arguments[0] == "\\/")
                      return "\\/?";
                  return "\\/" + arguments[2] + "\\/?";
              }) + "$", case_sensitive ? "" : "i");

        (overwrite || !routes[route]) && (routes[route] = {
            variables : variables,
            view : data.view,
            permissions : !data.permissions ? null : anp.is_array(data.permissions) ? data.permissions : ("" + data.permissions).split(","),
            pattern : route
        });

    };

    const add_object = (object, overwrite, callback) => {
        if(anp.is_dictionary(object)){
            object.route && add_item(object, overwrite);
            callback();
        }else if(anp.is_array(object))
            add(object, overwrite, callback, 0);
        else
            callback();
    };

    const add = (items, overwrite, callback, i) => {

        if(i >= items.length){
            anp.is_function(callback) && callback();
            return;
        };

        const end = () => add(items, overwrite, callback, i + 1);

        if(anp.is_string(items[i])){

            const matches = items[i].match(route_pattern);

            if(matches){
                add_item({
                    route : matches[1],
                    view : matches[3],
                    permissions : matches[5]
                }, overwrite);
                end();
            }else{

                const data = anp.json_decode(items[i]);

                if(data)
                    add_object(data, overwrite, end);
                else
                    anp.ajax.load(items[i], data => {
                        if(data && (data = anp.json_decode(data)))
                            add_object(data, overwrite, end);
                        else
                            end();
                    });

            };

        }else if(anp.is_object(items[i]))
            add_object(items[i], overwrite, end);
        else
            end();

    };

    this.add = (items, overwrite, callback) => add(items ? anp.is_array(items) ? items : [items] : [], anp.is_bool(overwrite) ? overwrite : anp.settings_get(["routes_overwrite", "overwrite"]), callback, 0);

    this.launch = () => thread === null && (thread = anp.threads.add(change_function));

    this.start = callback => {

        if(started)
            return;
        started = true;

        let back_pattern = anp.settings_get("go_back_pattern");

        case_sensitive = anp.settings_get(["routes_case_sensitive", "case_sensitive"]);

        if(back_pattern){
            try{
                (back_pattern = new RegExp(back_pattern)) && 
                (go_back_pattern = back_pattern);
            }catch(exception){};
        };

        self.add(anp.settings_get("default_routes_files"), true, () => self.add(anp.settings_get("routes_files"), true, callback));

    };

    this.autostart_go = () => anp.settings_get("autostart_routes") && self.launch();

    this.set_was_back = () => was_back = true;

    this.go = forced => {

        if(last_url === null || (!forced && url_gone === last_url))
            return;
        url_gone != last_url && history.push(url_gone = last_url);
        anp.base.update_breadcrumb(url_gone);

        if(was_back)
            was_back = false;
        else if(anp.was_back())
            anp.remove_was_back();

        anp.update_url_pointers();

        Object.keys(variables).forEach(key => anp.remove_globals(key));

        for(const pattern in routes){

            const matches = last_url.match(routes[pattern].pattern);

            if(!matches)
                continue;
            if(!anp.sessions.has_permissions(routes[pattern].permissions)){
                anp.item_self.querySelector(".body").innerHTML = anp.comp.error(403);
                return;
            };

            const l = matches.length;

            variables = {};
            for(let i = 1; i < l; i ++)
                anp.set_global(routes[pattern].variables[i], variables[routes[pattern].variables[i]] = matches[i]);

            anp.hash_self && (anp.item_self.querySelector(".body").innerHTML = anp.views.get(routes[pattern].view));

            current_pattern = pattern;

            return;
        };

        anp.hash_self && (anp.item_self.querySelector(".body").innerHTML = anp.comp.error(404));
        current_pattern = null;

    };

    const change_function = () => {
        if(!anp.hash_self)
            return;

        const url = window.location.hash.substr(1);

        if(last_url == url)
            return;
        last_url = url;

        self.go();

    };

    this.get_variables = () => Object.assign({}, variables);

    this.get_variable = name => variables[name] !== undefined ? variables[name] : null;

    this.get_last = () => history.length > 1 ? history[history.length - 2] : null;

    this.get_history = i => isNaN(i) ? history : history.length > i + 1 ? history[history.length - i - 1] : null;

    this.get_object = () => {
        if(anp.settings_get("allow_get_routes"))
            return routes;

        anp.toast("warning", "get_routes_not_allowed");

        return null;
    };

    this.get_permissions = () => current_pattern ? routes[current_pattern].permissions : null;

    this.get_last_url = () => history.length - 1 ? history[history.length - 2] : null;

    this.get_view_name = url => {
        for(const pattern in routes)
            if(routes[pattern].pattern.test(url))
                return routes[pattern].view;
        return null;
    };

    this.back = () => {
        
        const l = history.length - 1,
              root = history[l].replace(go_back_pattern, "");
        let j;

        // console.log([root, history]);

        for(let i = l - 1; i >= 0; i --)
            if((j = history[i].indexOf(root)) == -1 || (!j && root == history[i])){

                const last = history[i];

                history = history.slice(0, i);
                window.location = "#" + last;

                break;
            };

    };
    
    construct();

};
