AnP.Sessions = function(anp, input){

    const self = this,
          errors_messages = [
              "query_exception",
              "session_null",
              "session_empty",
              "session_bad_format",
              "session_id_null",
              "session_id_bad",
              "ip_null",
              "ip_empty",
              "ip_bad_format",
              "user_agent_null",
              "user_agent_empty",
              "session_not_exists",
              "user_id_null",
              "user_unaccessible",
              "user_deleted",
              "user_excluded",
              "ip_not_same",
              "ip_id_null",
              "ip_not_exists",
              "ip_unaccessible",
              "ip_deleted",
              "ip_excluded",
              "user_agent_not_same",
              "user_agent_id_null",
              "user_agent_not_exists",
              "session_date_out",
              "session_timeout",
              "session_banned",
              "session_not_perimssions",
              null
          ];
    let started = false,
        session = null,
        permissions = [],
        route = null,
        refresh_timeout = null,
        thread = null,
        last_communicate = 0,
        anp_root = null,
        there_was_action = false,
        timeout = null,
        last_action_time = 0,
        user_nick = null,
        last_user_nick,
        is_in_login = false,
        allow_users,
        reset_password = false,
        last_guest = 0,
        is_closing = false;

    const construct = () => {};

    const set_permissions_views = this.set_permissions_views = () => anp.item_self.querySelectorAll("[data-permissions]").forEach(item => {

        const subpermissions = anp.json_decode(anp.base64_decode(item.getAttribute("data-permissions")));

        return item.setAttribute("data-visible", !subpermissions || subpermissions.some(permission => permissions.includes(permission)));
    });

    const refresh = callback => {

        if(is_closing)
            return;

        anp.ajax.send("post", anp.anp_route() + "/sessions/update", {
            session : session,
            __no_content : true
        }, (response, ok) => {
            if(ok && anp.errors.validate(response.error, errors_messages, response, "update_session_error_message")){
                !last_communicate && (last_communicate = Date.now());
                anp.cookie_set(anp.settings_get("cookie_session_name"), session);
                permissions = response.permissions || [];
                timeout = response.timeout * 1000;
                set_permissions_views();
                anp.item_self.querySelector("[data-field=ip] .value").innerHTML = response.ip;
                anp.item_self.querySelector("[data-field=nick] .value").innerHTML = (user_nick = response.user) == "guest" ? anp.settings_get("default_nick", input) : response.user;
                anp.is_function(callback) && callback();
            }else{
                session = null;
                permissions = ["guest"];
                anp.item_self.querySelector("[data-field=nick] .value").innerHTML = anp.settings_get("unknown_nick", input);
                anp.go(window.location.hash == "#" ? "#/" : "#");
                get_guest(callback);
            };
        }, null, false);

    };

    const thread_method = () => {

        const sessions_view = anp.item_self.querySelector("[data-view=sessions_login]");

        if(sessions_view){
            if(!is_in_login){
                is_in_login = true;
                if(last_user_nick){
                    sessions_view.querySelector("[name=user]").value = last_user_nick;
                    setTimeout(() => sessions_view.querySelector("[name=password]").focus(), 100);
                };
            };
        }else 
            is_in_login && (is_in_login = false);

        if(!refresh_timeout || !last_communicate || Date.now() - last_communicate < refresh_timeout || !there_was_action){
            if(timeout !== null && user_nick != "guest" && Date.now() - last_action_time > timeout + refresh_timeout){
                last_action_time = Date.now();
                close(() => refresh(() => {
                    anp.toast("warning", "session_timeout");
                    if(!self.has_permissions(anp.routes.get_permissions()))
                        window.location = "#/users/login"
                }));
            };
            return;
        };

        last_communicate = Date.now();
        there_was_action = false;

        refresh();

    };

    const get_guest = callback => {

        const current_date = Date.now();

        if(current_date - last_guest < 2000)
            return;

        last_guest = Date.now();
        anp.ajax.send("post", anp.anp_route() + "/sessions/get", {}, (response, ok) => {
            // console.log(response);
            if(ok && anp.errors.validate(response.error, errors_messages.concat([
                "ip_exception",
                "ip_null",
                "ip_empty",
                "ip_not_accessible"
            ], {}, "get_guest_error_message"))){
                last_action_time = last_communicate = Date.now();
                anp.item_self.querySelector("[data-field=ip] .value").innerHTML = response.ip;
                anp.cookie_set(anp.settings_get("cookie_session_name"), session = response.id);
                permissions = response.permissions || [];
                set_permissions_views();
            };
            anp.is_function(callback) && callback();
        });

    };

    const get_anp_root = () => anp_root || (anp_root = (
        anp.settings_get(["anp_root", "anp_domain", "domain"]) || 
        window.location.href.replace(/^(https?\:\/{2}[^\/]+)(\/.*)?$/, "$1")
    ));

    this.start = callback => {

        if(started)
            return;
        started = true;

        allow_users = anp.settings_get("allow_users");

        refresh_timeout = anp.settings_get(["sessions_refresh_timeout", "refresh_timeout"]);
        get_anp_root();

        const saved_session = anp.cookie_get(anp.settings_get("cookie_session_name"));

        if(allow_users){
            if(saved_session){
                session = saved_session;
                refresh(callback);
            }else
                get_guest(callback);
        }else 
            anp.is_function(callback) && callback();

        allow_users && (thread = anp.threads.add(thread_method));
        last_action_time = Date.now();

        if(anp.item_self){

            const events = anp.settings_get("session_events");

            (events ? anp.is_array(events) ? events : [events] : []).forEach(event => anp.is_string(event) && anp.item_self.addEventListener(event, () => {
                there_was_action = true;
                last_action_time = Date.now();
            }));

        };

    };

    const field = (key, value) => {

        const text = anp.i18n.get(key);

        return (`
            <li` + anp.attr.set({
                data_field : key,
                data_i18n : key,
                data_i18n_without : true,
                title : text
            }) + `>
                <span` + anp.attr.set({data_i18n : key}) + `>` + text + `</span>
                <span` + anp.attr.set({class : "value"}) + `>` + value + `</span>
            </li>
        `);
    };

    this.create = input => {

        const user = anp.settings_get("default_nick", input),
              ip = anp.settings_get("default_ip", input);
              texts = {user : null, ip : null};

        for(const key in texts)
            texts[key] = anp.i18n.get(key);

        return (`
            <div` + anp.attr.set({
                class : "user-box",
                data_mode : "guest"
            }) + `>
                <ul` + anp.attr.set({class : "info"}) + `>
                    ` + field("nick", user) + `
                    ` + field("ip", ip) + `
                </ul>
                ` + anp.comp.buttons({buttons : [
                    {i18n : "login", icon : "login", go : "#/users/login", permissions : "guest", url_pointer : "^\\/users\\/login"},
                    {i18n : "profile", icon : "profile", go : "#/users", permissions : "user", url_pointer : "^\\/users"},
                    {i18n : "logout", icon : "logout", onclick : anp.object_name + ".sessions.logout(this, event);", permissions : "user"},
                    {i18n : "administration", icon : "administration", go : "#/administration", permissions : "administrator", url_pointer : "^\\/administration"}
                ]}) + `
                ` + anp.comp.images.create({class : "avatar", src : anp.string_variables(anp.settings_get("default_avatar", input), {anp_root : get_anp_root()}), alt : user}) + `
            </div>
        `);
    };

    this.get = () => session || null;

    this.register = (item, event) => {

        const [values, form] = anp.comp.forms.get_values(item);

        if(!anp.errors.validate(form, (
            (anp.errors.empty(values.nick) << 1) |
            (anp.errors.empty(values.password) << 4) |
            (anp.errors.empty(values.confirm_password) << 7 || (values.password == values.confirm_password ? 0 : 1 << 10)) |
            (anp.errors.empty(values.email) << 11 || (/^[^\@]+\@[^\.]+(\.[^\.]+)+$/.test(values.email) ? 0 : 1 << 14)) |
            0
        ), ["exception_js"].concat(
            anp.errors.empty_messages("nick"),
            anp.errors.empty_messages("password"),
            anp.errors.empty_messages("confirm_password"),
            ["confirm_no_password"],
            anp.errors.empty_messages("email"),
            ["email_bad_format"],
            []
        ), values, "register_local_error_message"))
            return;

        values.session = session;

        anp.ajax.send("post", anp.anp_route() + "/sessions/register", values, (response, ok) => {
            if(ok && anp.errors.validate(form, response.variables.error, errors_messages.concat([
                "nick_null",
                "nick_empty",
                "nick_exists",
                "nick_group_exists",
                "email_null",
                "email_empty",
                "email_bad_format",
                "email_exists",
                "password_null",
                "password_empty",
                "password_minimum_length",
                "password_needs_lower",
                "password_needs_upper",
                "password_needs_digit",
                "password_needs_special"
            ]), values, "register_error_message", "user_created_ok")){
                anp.go("#/users/login");
                setTimeout(() => {
                    anp.item_self.querySelector("[name=user]").value = values.nick;
                    anp.item_self.querySelector("[name=password]").focus();
                }, 100);
            };
        });

    };

    const close = callback => anp.ajax.send("post", anp.anp_route() + "/sessions/close", {session : session}, (response, ok) => {
        if(ok && anp.errors.validate(null, response.error, errors_messages, {}, "logout_error_message")){

            const nick_field = anp.item_self.querySelector("[data-field=nick] .value");

            anp.settings_get("session_timeout_drop_cache") && anp.comp.forms.drop_cache();
            nick_field.innerText != "Guest" && (last_user_nick = nick_field.innerText);
            nick_field.innerHTML = anp.settings_get("default_nick", input);
            anp.is_function(callback) && callback();

        };
    });

    this.login = (item, event) => {

        const [values, form] = anp.comp.forms.get_values(item);

        if(!anp.errors.validate(form, (
            (anp.errors.empty(values.user) << 1) |
            (anp.errors.empty(values.password) << 4) |
            0
        ), ["exception_js"].concat(
            anp.errors.empty_messages("user"),
            anp.errors.empty_messages("password"),
            []
        ), values, "login_local_error_message"))
            return;

        values.session = session;

        anp.ajax.send("post", anp.anp_route() + "/sessions/login", values, (response, ok) => {
            // console.log(response);
            if(ok && anp.errors.validate(form, response.login_error, errors_messages.concat([
                "user_null",
                "user_empty",
                "password_null",
                "password_empty",
                "bad_password_user", // user_id_null -> user_not_exists
                "user_id_bad",
                "bad_password_user", // user_not_exists
                "user_deleted",
                "user_antinuke",
                "user_unaccessible",
                "user_banned",
                "user_limit_sessions",
                "ip_not_exists", // ip_id_null -> ip_not_exists
                "ip_id_bad",
                "ip_not_exists", // ip_not_exists
                "ip_deleted",
                "ip_antinuke",
                "ip_unaccessible",
                "ip_banned",
                "ip_limit_sessions",
                "bad_password_user" // password_wrong
            ]), values, "login_error_message")){

                const last_route = anp.routes.get_last_url();

                session = response.id;
                last_action_time = last_communicate = Date.now();
                anp.item_self.querySelector("[data-field=ip] .value").innerHTML = response.ip;
                anp.item_self.querySelector("[data-field=nick] .value").innerHTML = values.user;
                anp.cookie_set(anp.settings_get("cookie_session_name"), session = response.id);
                anp.toast("ok", "user_login_ok", values);
                permissions = response.permissions || [];
                timeout = response.timeout * 1000;
                user_nick = response.user;
                reset_password = response.reset_password;
                anp.go(
                    reset_password ? "#/users/reset_password" : 
                    last_route && last_route != "users/login" ? "#" + last_route : 
                    route || "#" + anp.settings_get("default_route", "/")
                );
                set_permissions_views();

            };
        });

    };

    const execute_result = element => {
        if(!element)
            return;

        if(anp.is_function(element))
            element();
        else if(anp.is_string(element))
            anp.go(element);

    };

    this.has_permissions = (check, reject, ok) => {
        if(!check)
            return true;

        let has = false;

        !anp.is_array(check) && (check = [check]);

        permissions && check.forEach(permission => {
            if(permissions.includes(permission))
                return has = true;
        });

        execute_result(has ? ok : reject);

        if(!has && check.includes("registered")){
            route = window.location.hash;
            anp.toast("warn", "user_registered_need")
            anp.go("#/users/login");
        };

        return has;
    };

    this.logout = (item, event) => {

        is_closing = true;

        anp.errors.validate(
            null, 
            (
                (permissions === null ? 1 << 2 : permissions[0] == "guest" ? 1 << 1 : 0) |
                (session === null ? 1 << 3 : 0) |
                0
            ), 
            [
                "exception_js",
                "close_guest",
                "close_permissions_null",
                "session_null"
            ], 
            {
                user : anp.item_self.querySelector("header>.user-box [data-i18n=nick]+.value").innerText, 
                ip : anp.item_self.querySelector("header>.user-box [data-i18n=ip]+.value").innerText
            }, 
            "logout_error_message", "logout_success"
        ) && close(() => get_guest(() => {
            anp.go("#/users/login");
            is_closing = false;
        }));

    };

    this.get_errors_list = () => [].concat(errors_messages);

    this.reset_password = (item, event) => {

        const [values, form] = anp.comp.forms.get_values(item);

        if(!anp.errors.validate(form, (
            ((
                values.password === undefined ? 1 << 0 : 
                values.password === null ? 1 << 1 : 
                !anp.is_string(values.password) ? 1 << 2 : 
                !values.password ? 1 << 3 : 
            0) << 0) | 
            ((
                values.confirm_password === undefined ? 1 << 0 : 
                values.confirm_password === null ? 1 << 1 : 
                !anp.is_string(values.confirm_password) ? 1 << 2 : 
                !values.confirm_password ? 1 << 3 : 
            0) << 4) | 
        0) || (
            values.password != values.confirm_password ? 1 << 8 : 0
        ), [
            "password_undefined",
            "password_null",
            "password_not_string",
            "password_empty",
            "confirm_password_undefined",
            "confirm_password_null",
            "confirm_password_not_string",
            "confirm_password_empty",
            "password_not_confirm"
        ], values, "reset_password_local_error_message"))
            return;

        anp.ajax.send("post", anp.anp_route() + "/sessions/reset_password", values, (response, ok) => {
            anp.errors.validate(form, response.variables.error, errors_messages.concat([
                "password_null",
                "password_empty",
                "password_minimum_length",
                "password_needs_lower",
                "password_needs_upper",
                "password_needs_digit",
                "password_needs_special",
                "password_already_used"
            ], values, "reset_password_error_message"), "reset_password_ok_message") && 
            anp.go("#" + anp.settings_get("default_route", "/"));
        });

    };

    construct();

};
