AnP.Components.Calendars = function(anp, input){

    const self = this;
    let started = false,
        click_event = null,
        thread = null,
        cache = null;

    const construct = () => {};

    const get_box = item => {

        let box = item;

        if(box)
            while((!box.hasAttribute || !box.hasAttribute("data-module") || box.getAttribute("data-module") != "calendar_box") && (box = box.parentNode));

        return box || null;
    };

    const get = item => {

        let input = item;

        if(input)
            while((!input.hasAttribute || !input.hasAttribute("data-module") || input.getAttribute("data-module") != "calendar") && (input = input.parentNode));

        return input || null;
    };

    const click_out = event => {

        const box = get_box(event.target),
              input = get(event.target),
              hash = input && input.hasAttribute("for") ? input.getAttribute("for") : null;

        anp.item_self.querySelectorAll("[data-module=calendar_box]").forEach(item => {if(item != box && item.getAttribute("data-for") != hash)item.remove();});

    };

    const set_input = (input, date) => {
        if(!date)
            return;

        const entry = input.querySelector("[data-module-name=calendar]"),
              field = input.querySelector("[name]");

        entry != document.activeElement && (entry.value = anp.date_format(date, input.getAttribute("data-format")));
        entry.value && input.setAttribute("data-date", date.getTime());
        field && (field.value = (
            ("0000" + date.getUTCFullYear()).slice(-4) + 
            ("00" + (date.getUTCMonth() + 1)).slice(-2) + 
            ("00" + date.getUTCDate()).slice(-2) + 
            ("00" + date.getUTCHours()).slice(-2) + 
            ("00" + date.getUTCMinutes()).slice(-2) + 
            ("00" + date.getUTCSeconds()).slice(-2)
        ));

    };

    const set_day = (box, day) => {

        box.querySelector("[name=day]").value = day;
        box.querySelectorAll("[data-day][data-selected=true]").forEach(selected => selected.setAttribute("data-selected", false));
        box.querySelectorAll("[data-real-day='" + day + "']").forEach(day => day.setAttribute("data-selected", true));

    };

    const get_month_days = (year, month) => month == 1 ? 28 + (!(year % 4) && (year % 100 || !(year % 400)) ? 1 : 0) : 30 + (1 - ((month + (month / 7 >> 0)) % 2));

    const update = item => {

        const year = Number((item = get_box(item)).querySelector("[name=year]").value),
              month = Number(item.querySelector("[name=month]").value),
              month_days = get_month_days(year, month),
              last_month_days = month ? get_month_days(year, month) : get_month_days(year - 1, 11),
              first_week = item.querySelectorAll("[data-week='0']>span");

        for(let i = 0; i < 6; i ++)
            first_week[i].innerText = Number(first_week[i].getAttribute("data-day")) + last_month_days;
        item.querySelectorAll("[data-week='4']>span,[data-week='5']>span").forEach(box => {

            const day = Number(box.getAttribute("data-day"));

            if(day > month_days){
                box.setAttribute("data-real-day", box.innerText = day - month_days);
                box.setAttribute("data-month", 1);
            }else if(day >= 28){
                box.setAttribute("data-real-day", box.innerText = day);
                box.setAttribute("data-month", 0);
            };

        });
        item.querySelector(".day").setAttribute("data-start", (new Date(year, month, 1).getDay() + 6) % 7);

    };

    const set_box = (box, date) => {

        box.querySelector("[name=year]").value = date.getFullYear();
        box.querySelector("[name=month]").value = date.getMonth();
        box.querySelector("[name=hour]").value = date.getHours();
        set_day(box, date.getDate());
        box.querySelector("[name=minute]").value = date.getMinutes();
        box.querySelector("[name=second]").value = date.getSeconds();

        update(box);

    };

    const thread_method = () => {

        const input = anp.item_self.querySelector("[data-module=calendar] input:focus");

        if(!input){
            cache = null;
            return;
        };

        const date_text = input.value;

        if(cache === date_text)
            return;
        cache = date_text;

        let date = new Date();
        const item = get(input),
              box = anp.item_self.querySelector("[data-module=calendar_box][data-for=" + item.getAttribute("for") + "]"),
              focus = input == document.activeElement ? [input.selectionStart, input.selectionEnd] : null;

        if(date_text){

            const format = item.getAttribute("data-format"),
                  variables = [null],
                  matches = date_text.match(new RegExp("^" + anp.string_to_re(format).replace(/\\\{([^\\\{\}]+)\\\}/g, (...arguments) => {
                      variables.push(arguments[1]);
                      return "(\\d+)";
                  }) + "$"));

            if(matches){

                const l = variables.length,
                      values = {};

                for(let i = 1; i < l; i ++)
                    values[variables[i]] = Number(matches[i]);

                date = new Date(
                    Number(values.yyyy || values.y || (isNaN(values.yy) ? 2000 + values.yy : 0)),
                    Number(values.mm || values.m || 0) - 1,
                    Number(values.dd || values.d || 0),
                    Number(values.hh || values.h || 0),
                    Number(values.ii || values.i || 0),
                    Number(values.ss || values.s || 0)
                );

            };

        };

        set_input(item, date);
        box && set_box(box, date);

        focus && input.setSelectionRange(focus[0], focus[1]);

    };

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp.comp.datetime = anp.comp.calendar = self.create;

        click_event = addEventListener("click", click_out);
        thread = anp.threads.add(thread_method);

        anp.is_function(callback) && callback();

    };

    const selector = (name, options) => `<select` + anp.attr.set({name : name, onchange : anp.object_name + ".comp.calendars." + name + "_change(this, event);"}) + `>` + options + `</select>`;

    const date_item = (name, options) => (`
        <section` + anp.attr.set({class : name + " group"}) + `>
            ` + anp.comp.button({i18n : "previous", icon : "date_previous", onclick : anp.object_name + ".comp.calendars." + name + "_previous(this, event);"}) + `
            ` + selector(name, options) + `
            ` + anp.comp.button({i18n : "next", icon : "date_next", onclick : anp.object_name + ".comp.calendars." + name + "_next(this, event);"}) + `
        </section>
    `);

    this.show = (item, event) => {

        const input = get(item),
              data_for = input.getAttribute("for");

        if(anp.item_self.querySelector("[data-module=calendar_box][data-for=" + data_for + "]"))
            return;

        const years_range = input.getAttribute("data-years-range").split(","),
              preloader = anp.hash();
        let years = ``,
            months = ``,
            days = ``,
            hours = ``,
            minutes_and_seconds = ``;

        for(let i = years_range[0]; i <= years_range[1]; i ++)
            years += `<option` + anp.attr.set({value : i, title : i}) + `>` + i + `</option>`;
        for(let i = 0; i < 12; i ++){

            const text = anp.i18n_get("month_" + i);

            months += `<option` + anp.attr.set({value : i, data_i18n : "month_" + i, title : text}) + `>` + text + `</option>`;

        };
        for(let j = 0; j < 6; j ++){
            days += `<span` + anp.attr.set({class : "week", data_week : j}) + `>`;
            for(let i = 0; i < 13; i ++){

                const day = i + (j * 7) - 5;

                days += `<span` + anp.attr.set({data_day : day, data_real_day : day, data_selected : false, data_month : day < 1 ? -1 : 0, onclick : anp.object_name + ".comp.calendars.day_change(this, event);"}) + `>` + day + `</span>`;

            };
            days += `</span>`;
        };
        for(let i = 0; i < 24; i ++)
            hours += `<option` + anp.attr.set({value : i, title : i}) + `>` + ((i < 10 ? "0" : "") + i) + `</option>`;
        for(let i = 0; i < 60; i ++)
            minutes_and_seconds += `<option` + anp.attr.set({value : i, title : i}) + `>` + ((i < 10 ? "0" : "") + i) + `</option>`;

        new AnP.Preload(anp, "[data-preloader=" + preloader + "]", preloader => {

            const box = get_box(preloader);

            try{
                set_box(box, input.querySelector("input").value ? new Date(Number(input.getAttribute("data-date"))) : new Date());
            }catch(exception){};
            ["time"].forEach(attribute => box.setAttribute("data-" + attribute, input.getAttribute("data-" + attribute)));

            preloader.remove();
        });

        anp.comp.create_float_box(item, event, {
            name : "calendar_box",
            html : (`
                <fieldset>
                    ` + anp.comp.i18n("calendar", "legend") + `
                    ` + date_item("year", years) + `
                    ` + date_item("month", months) + `
                    <section` + anp.attr.set({class : "day", data_start : 0}) + `>
                        <input` + anp.attr.set({name : "day", type : "hidden"}) + ` />
                        <div>` + days + `</div>
                    </section>
                    <section` + anp.attr.set({class : "time group"}) + `>
                        ` + selector("hour", hours) + `
                        ` + selector("minute", minutes_and_seconds) + `
                        ` + selector("second", minutes_and_seconds) + `
                    </section>
                    <div` + anp.attr.set({data_preloader : preloader}) + `></div>
                </fieldset>
            `)
        });

    };

    this.clean = (item, event) => {
        with(item = get(item)){
            querySelector("input").value = getAttribute("value") || "";
            setAttribute("data-date", 0);
        };
    };

    this.get = item => {
        
        const date = item.parentNode.querySelector("input").value ? new Date(Number(get(item).getAttribute("data-date"))) : null;

        // console.log(date ? Number(new String(date).match(/GMT([\+\-][0-9]{1,2})/)[1]) : null);

        return date;
    };

    this.create = (input, callback) => {

        !anp.is_dictionary(input) && (input = {});

        const format = anp.settings_get(["calendars_format", "format"], input),
              date = input.value ? anp.to_date(input.value).getTime() || 0 : 0,
              text_attributes = {
                  onclick : anp.object_name + ".comp.calendars.show(this, event);", 
                  data_module_name : "calendar"
              };

        !input.id && (input.id = input.hash || anp.hash());

        for(const key in input)
            !["name"].includes(key) && (text_attributes[key] = input[key]);
        input.value !== undefined && (text_attributes.value = anp.date_format(date, format));

        let html = (`
            <label` + anp.attr.set([{class : "calendar-input group" + (input.class ? " " + input.class : "")}, input.i18n ? {
                data_i18n : input.i18n,
                data_i18n_without : true,
                title : anp.i18n_get(input.i18n),
                for : input.id,
                data_format : format,
                data_module : "calendar",
                data_years_range : anp.settings_get(["calendars_years_range", "years_range"], input).join(","),
                data_date : date,
                data_time : anp.settings_get(["calendar_time", "time"], input)
            } : null]) + `>
                ` + anp.comp.text(text_attributes, "class") + `
                <input` + anp.attr.set({type : "hidden", ...(input.name ? {name : input.name} : {}), data_module_name : "calendar"}) + ` />
                ` + anp.comp.button({i18n : "calendar", icon : "calendar", onclick : anp.object_name + ".comp.calendars.show(this, event);"}) + `
                ` + anp.comp.button({i18n : "clean", icon : "clean", onclick : anp.object_name + ".comp.calendars.clean(this, event);"}) + `
            </label>
        `);

        return html;
    };

    const update_date = item => set_input(
        anp.item_self.querySelector("label[for=" + (item = get_box(item)).getAttribute("data-for") + "][data-module]"),
        new Date(
            Number(item.querySelector("[name=year]").value),
            Number(item.querySelector("[name=month]").value),
            Number(item.querySelector("[name=day]").value),
            Number(item.querySelector("[name=hour]").value),
            Number(item.querySelector("[name=minute]").value),
            Number(item.querySelector("[name=second]").value)
        )
    );

    this.year_change = (item, event) => {

        update(item);
        update_date(item);

    };

    this.month_change = (item, event) => {

        update(item);
        update_date(item);

    };

    this.day_change = (item, event) => {

        const box = get_box(item),
              month = Number(item.getAttribute("data-month"));

        set_day(box, Number(item.innerText));

        if(month == -1)
            self.month_previous(item, event);
        else if(month == 1)
            self.month_next(item, event);
        else
            update_date(item);

    };

    this.hour_change = (item, event) => update_date(item);

    this.minute_change = (item, event) => update_date(item);

    this.second_change = (item, event) => update_date(item);

    this.year_previous = (item, event) => {

        const field = get_box(item).querySelector("[name=year]");
        let = year = Number(field.value) - 1;

        field.value = year < Number(field.querySelector("option:first-child").value) ? field.querySelector("option:last-child").value : year;

        self.year_change(item, event);

    };

    this.year_next = (item, event) => {

        const field = get_box(item).querySelector("[name=year]");
        let = year = Number(field.value) + 1;

        field.value = year > Number(field.querySelector("option:last-child").value) ? field.querySelector("option:first-child").value : year;

        self.year_change(item, event);

    };

    this.month_previous = (item, event) => {

        const field = get_box(item).querySelector("[name=month]");

        if(field.value == "0"){
            field.value = 11;
            self.year_previous(item);
        }else{
            field.value = Number(field.value) - 1;
            self.month_change(item, event);
        };

    };

    this.month_next = (item, event) => {

        const field = get_box(item).querySelector("[name=month]");

        if(field.value == "11"){
            field.value = 0;
            self.year_next(item);
        }else{
            field.value = Number(field.value) + 1;
            self.month_change(item, event);
        };

    };

    this.set = (item, value) => {
        if(!value || !item)
            return;

        const field = get(item);

        field && set_input(field, anp.to_date(value));

    };

    construct();

};
