AnP.Components.Colors = function(anp, input){

    const self = this,
          colors = {
              names : {},
              codes : {}
          };
    let started = false,
        division = 3,
        changing = false;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        division = anp.settings_get("colors_tones_division");
        self.set_colors(anp.settings_get("default_colors"), true, () => self.set_colors(anp.settings_get("colors"), true, () => {
            end(true);
        }));

        anp.comp.color = self.build_input;

        return true;
    };

    const set_colors = (data, overwrite, callback, i) => {

        if(data.length <= i){
            anp.is_function(callback) && callback();
            return;
        };

        const end = () => set_colors(data, overwrite, callback, i + 1);

        if(anp.is_dictionary(data[i])){
            !overwrite && (overwrite = anp.settings_get(["colors_overwrite", "overwrite"]));
            for(const key in data[i]){
                data[i][key].length == 4 && (data[i][key] = data[i][key].replace(/([0-9a-f])/ig, "$1$1"));
                (overwrite || !colors.codes[key]) && (colors.codes[key] = data[i][key]);
                (overwrite || !colors.names[data[i][key]]) && (colors.names[data[i][key]] = key);
            };
            end();
        }else if(anp.is_array(data[i]))
            set_colors(data[i], overwrite, end, 0);
        else if(anp.is_string(data[i])){

            let json;

            try{
                json = anp.json_decode(data[i]);
            }catch(exception){};

            if(json){
                set_colors(anp.is_array(json) ? json : [json], overwrite, end, 0);
                end();
            }else 
                anp.ajax.get(data[i], response => {
                    try{
                        json = anp.json_decode(response);
                    }catch(exception){};
                    if(json)
                        set_colors(anp.is_array(json) ? json : [json], overwrite, end, 0);
                    else 
                        end();
                });

        }else 
            end();
    }

    this.set_colors = (data, overwrite, callback) => set_colors(data ? anp.is_array(data) ? data : [data] : [], overwrite, callback, 0);

    const get = item => {

        if(item)
            while(item.classList && !item.classList.contains("colors-module") && (item = item.parentNode));

        return item ? item.querySelector("[type=color]") : null;
    };

    this.change_simplified = (item, event) => {

        if(changing){
            changing = false;
            return;
        };
        
        const code = anp.comp.selectors.get_value(item),
              module = get(item);

        self.change(module, code);
        module.value = code;

    };

    this.build_input = input => {

        const id = input.id || input.hash,
              i18n = input.i18n || input.name, 
              name = input.name || input.i18n;

        return (`
            <span` + anp.attr.set({
                class : "colors-module",
                ...(i18n ? {
                    data_i18n : i18n,
                    data_i18n_without : true,
                    title : anp.i18n_get(i18n)
                } : {})
            }) + `>
                <div` + anp.attr.set({class : "combobox"}) + `>` + anp.comp.selector({
                    data_onchange : "{object_name}.comp.colors.change_simplified(this, event);",
                    options : Object.entries(colors.names).map(([code, name]) => ({name : (
                        `<span` + anp.attr.set({class : "color-sample", style : "background-color : " + code + ";"}) + `></span>` + 
                        anp.comp.i18n(name, "span")
                    ), value : code}))
                 }) + `</div>
                <div` + anp.attr.set({
                    class : "info-box",
                    onclick : "this.querySelector('input').click();"
                }) + `>
                    <span` + anp.attr.set({class : "sample"}) + `><span></span></span>
                    <span` + anp.attr.set({class : "code"}) + `></span>
                    <input` + anp.attr.set({
                        type : "color", 
                        ...(id ? {id : id} : {}), 
                        ...(name ? {name : name} : {}),
                        ...(input.value ? {value : input.value} : {}),
                        onchange : "{object_name}.comp.colors.change(this, event);" + (input.onchange || "")
                    }) + ` />
                    <span` + anp.attr.set({class : "name"}) + `></span>
                    <span` + anp.attr.set({class : "name-i18n"}) + `></span>
                    <span` + anp.attr.set({class : "sample-simplified"}) + `><span></span></span>
                    <span` + anp.attr.set({class : "code-simplified"}) + `></span>
                    <span` + anp.attr.set({class : "code-basic"}) + `></span>
                    <span` + anp.attr.set({class : "color-by-name"}) + `><span></span></span>
                </div>
            </span>
        `);
    };

    const get_formats = value => {

        const rgb = [],
              fragment = 0xF / (division - 1);
        let code_simplified,
            code_basic;

        value.replace(/[0-9a-f]{2}/g, tone => rgb.push(Math.round((parseInt(tone, 16) / (0x10 * fragment)) * 1)));
        code_basic = "#" + rgb.map(tone => Math.round(tone * fragment).toString(16)).join("").toUpperCase();
        code_simplified = code_basic.replace(/([0-9A-F])/g, "$1$1");

        return [code_simplified, code_basic, colors.names[code_simplified] || "unknown"];
    };

    this.change = (item, event) => {

        const code = anp.is_string(event) ? event : item.value,
              [simplified, basic, name] = get_formats(code);

        item.parentNode.querySelector(".sample>span").style.backgroundColor = code;
        item.parentNode.querySelector(".code").innerText = code.toUpperCase();
        item.parentNode.querySelector(".name").innerText = name;
        with(item.parentNode.querySelector(".name-i18n")){
            setAttribute("data-i18n", name)
            innerText = anp.i18n_get(name);
        };
        item.parentNode.querySelector(".sample-simplified>span").style.backgroundColor = simplified;
        item.parentNode.querySelector(".code-simplified").innerText = simplified;
        item.parentNode.querySelector(".code-basic").innerText = basic;
        item.parentNode.querySelector(".color-by-name>span").setAttribute("data-color", name);

        changing = true;
        !anp.is_string(event) && anp.comp.selectors.set_value(item.parentNode.parentNode.querySelector("label"), simplified);

    };

    this.create_field = value => {

        const [simplified, basic, name] = get_formats(value),
              text = anp.i18n_get(name);

        return (`<span` + anp.attr.set({
            class : "color-field",
            data_value : value,
            data_simplified : simplified,
            data_basic : basic,
            data_name : name,
            data_i18n : name,
            data_i18n_without : true,
            title : text
        }) + `>
            <span` + anp.attr.set({class : "sample"}) + `><span` + anp.attr.set({style : "background-color : " + value + ";"}) + `></span></span>
            <span` + anp.attr.set({class : "code"}) + `>` + value + `</span>
            <span` + anp.attr.set({class : "name"}) + `>` + name + `</span>
            <span` + anp.attr.set({class : "name-i18n", data_i18n : name}) + `>` + text + `</span>
            <span` + anp.attr.set({class : "sample-simplified"}) + `><span` + anp.attr.set({style : "background-color : " + simplified + ";"}) + `></span></span>
            <span` + anp.attr.set({class : "code-simplified"}) + `>` + simplified + `</span>
            <span` + anp.attr.set({class : "code-basic"}) + `>` + basic + `</span>
            <span` + anp.attr.set({class : "color-by-name"}) + `><span` + anp.attr.set({data_color : name}) + `></span></span>
        </span>`);
    };

    construct();

};