AnP.Components.Forms = function(anp, input){

    const self = this,
          type_synonyms = {
              i18n : "text", 
              varchar : "text",
              string : "text",
              integer : "number",
              float : "number",
              bool : "checkbox", 
              boolean : "checkbox",
              decimal : "number"
          },
          cache = {};
    let started = false;

    const construct = () => {};

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp.comp.form = self.create;

        anp.is_function(callback) && callback();

    };

    const structure = structure => {

        let html = `<div` + anp.attr.set({class : "structure"}) + `>`;

        if(anp.is_array(structure))
            structure.forEach(field => {
                if(anp.is_dictionary(field)){

                    const type = type_synonyms[field.type] || field.type;

                    html += (`
                        <div` + anp.attr.set({
                            class : "field", 
                            data_i18n_without : true, 
                            title : anp.i18n_get(field.i18n || (field.i18n = field.name)), 
                            data_i18n : field.i18n, 
                            data_visible : !field.hidden
                        }) + `>
                            <label` + anp.attr.set({for : (field.id || (field.id = anp.hash()))}) + `>
                                ` + anp.comp.i18n(field.i18n, "span") + `
                                ` + anp.comp.i18n(field.i18n + "_description", "span") + `
                            </label>
                            <div` + anp.attr.set({class : "content"}) + `>` + (anp.is_function(anp.comp[type]) ? anp.comp[type](Object.assign({}, field, {type : type})) : ``) + `</div>
                            <ul` + anp.attr.set({class : "error"}) + `></ul>
                        </div>
                    `);
                };
            });

        return html + `</div>`;
    };

    this.create = input =>  {

        const hash = anp.hash();

        new AnP.Preload(anp, "[data-preloader=" + hash + "]", preloader => {
            if(!preloader)
                return;

            const form = preloader.parentNode,
                  name_field = form.querySelector("[name]"),
                  view = form.parentNode;
            let load_callback = null;

            preloader.remove();
            name_field && name_field.focus();

            input.onload && (anp.is_array(input.onload) ? input.onload : [input.onload]).forEach(callback => {
                if(anp.is_function(callback))
                    load_callback = callback(form);
                else if(anp.is_string(callback)){

                    const method = anp.set_globals(callback);

                    try{

                        const method_callable = eval(method);

                        anp.is_function(method_callable) && (load_callback = method_callable(form));

                    }catch(exception){
                        console.error([method, exception]);
                    };

                };
            });

            if(view && view.hasAttribute && view.hasAttribute("data-view")){
                
                const key = view.getAttribute("data-view");
                
                cache[key] && self.set(form, cache[key][0]);

            };

            anp.is_function(load_callback) && load_callback();

        });

        return (`
            <form` + anp.attr.set(Object.assign({}, input, {
                class : "form" + ((anp.is_dictionary(input) ? input : input = {}).class ? " " + input.class : ""),
                method : "get",
                action : "#",
                onsubmit : (input.submit ? "try{" + input.submit + "}catch(exception){console.error(exception);};" : "") + "return false;",
                data_comp : "form"
            }, anp.attr.get_values(input, ["view_name", "view"])), ["structure", "buttons", "i18n", "type", "variables", "onload"]) + `>
                <fieldset>
                    ` + anp.comp.i18n(input.i18n, "legend", input.variables) + `
                    ` + anp.comp.i18n(input.i18n + "_text", "p", input.variables) + `
                    ` + structure(input.structure) + `
                    <ul` + anp.attr.set({class : "error error-all"}) + `></ul>
                    ` + anp.comp.buttons({buttons : [
                        {i18n : "clean", icon : "clean", type : "reset"}
                    ].concat(input.buttons ? anp.is_array(input.buttons) ? input.buttons : [input.buttons] : [], [
                        input.submit ? {i18n : "submit", icon : "submit", type : "submit"} : null
                    ])}) + `
                </fieldset>
                <div data-preloader="` + hash + `"></div>
            </form>
        `);
    };

    const get = this.get = form => {

        if(form)
            while(form.tagName.toLowerCase() != "form" && (form = form.parentNode));

        return form || null;
    };

    this.get_values = (form, origin) => {

        const values = {__subdata__ : {}},
              done = [];

        (origin ? form : form = get(form)).querySelectorAll("[name]").forEach(field => {

            let value,
                name = field.getAttribute("name");
            const matches = name.match(/^(.+)\[\]$/);

            matches && (values[name = matches[1]] = []);

            if(done.includes(name))
                return;

            if(field.hasAttribute("data-module-name")){
                switch(field.getAttribute("data-module-name")){
                    case "files":
                        value = anp.comp.files.get(field);
                        if(!field.hasAttribute("multiple")){
                            if(value && value.length){

                                const i = value[0] ? 0 : 1;

                                for(const item_key in value[i])
                                    values[name + "_" + item_key] = value[i][item_key];

                            };
                            value = undefined;
                        };
                    break;
                    case "calendar":
                        value = anp.comp.calendars.get(field);
                    break;
                    case "selector":
                        (value = anp.comp.selectors.get_value(field)) && (value = isNaN(Number(value)) ? value : Number(value));
                    break;
                    case "forms":
                        value = anp.comp.forms.get(field);
                    break;
                    case "map":

                        const main_name = field.getAttribute("data-main-name");

                        if(main_name){
                            if(!done.includes(main_name)){
                                [
                                    values[main_name + "_longitude"],
                                    values[main_name + "_latitude"],
                                    values[main_name + "_altitude"]
                                ] = anp.comp.maps.get_values(field);
                                done.push(main_name);
                                value = undefined;
                            };
                        }else 
                            value = anp.comp.maps.get(field);
                        
                    break;
                    case "select_list":
                        value = anp.comp.selects_lists.get_value(field);
                        values.__subdata__[name] = anp.comp.selects_lists.get_subdata(field);
                    break;
                };
            }else 
                switch(field.tagName.toLowerCase()){
                    case "input":
                        switch(field.getAttribute("type")){
                            case "text":
                            case "password":
                            case "email":
                            case "hidden":
                                value = field.value;
                            break;
                            case "number":
                                value = Number(field.value);
                                field.parentNode.classList.contains("submodule-foreign-list") && 
                                (values.__subdata__[name] = anp.json_decode(anp.base64_decode(field.parentNode.querySelector("[data-type=html]").getAttribute("data-subdata"))));
                            break;
                            case "checkbox":
                                value = field.checked;
                            break;
                            case "radio":
                                field.checked && (value = field.value);
                            break;
                            case "color":
                                value = field.value;
                            break;
                        };
                    break;
                    case "textarea":
                    case "select":
                        value = field.value;
                    break;
                };

            if(value !== undefined){
                if(anp.is_array(values[name]))
                    values[name].push(value);
                else
                    values[name] = value;
            };

        });

        return [values, form];
    };

    this.submit = (form, event) => get(form).onsubmit(form, event);

    this.set = (form, data) => {
        if(!anp.is_dictionary(data))
            return;

        const done = [];

        form = get(form);

        for(const key in data){

            let field = form.querySelector("[name=" + key + "]"),
                subkey;

            if(
                (!field || !field.hasAttribute) &&  
                (!/_id$/.test(key) || !(field = form.querySelector("[name=" + (subkey = key.replace(/^(.+)_id$/, "$1")) + "]")))
            )
                continue;
                
            if(field.hasAttribute("data-module-name")){
                switch(field.getAttribute("data-module-name")){
                    case "selector":
                        anp.comp.selectors.set_value(field, data[key]);
                    break;
                    case "calendar":
                        anp.comp.calendars.set(field, data[key]);
                    break;
                    case "map":

                        const main_name = field.getAttribute("data-main-name");

                        if(main_name){
                            if(!done.includes(main_name)){
                                anp.comp.maps.set(
                                    field, 
                                    data[main_name + "_longitude"], 
                                    data[main_name + "_latitude"], 
                                    data[main_name + "_altitude"]
                                );
                                done.push(main_name);
                            };
                        }else 
                            anp.comp.maps.set_raw(field, data[key]);
                    break;
                    case "select_list":
                        anp.comp.selects_lists.set(field, data[key], data.__subdata__[key]);
                    break;
                    case "files":
                        anp.comp.files.set(field, (
                            anp.is_array(data[subkey]) ? data[subkey] : 
                            data[subkey + "_id"] ? [{
                                id : data[subkey + "_id"],
                                name : data[subkey + "_name"],
                                mime : data[subkey + "_mime"],
                                size : data[subkey + "_size"],
                                last_modified : data[subkey + "_last_modified"],
                                url : (
                                    data[subkey + "_url"] || 
                                    (/\/{2}[^\/]+\/app\//.test(window.location.href) ? "/app" : "") + "/file/" + data[subkey + "_id"] + "/" + data[subkey + "_name"]
                                )
                            }] : 
                            []
                        ));
                    break;
                };
            }else{
                switch(field.getAttribute("type")){
                    case "number":
                        field.value = Number(data[key]);
                    break;
                    case "checkbox":
                    case "radio":
                        field.checked = data[key];
                    break;
                    default:
                        field.value = data[key];
                    break;
                };
                field.getAttribute("onchange") && field.onchange();
            };

        };

    };

    this.fill_components = (form, data) => {

        form = get(form);

        for(const name in data){

            const field = form.querySelector("[name=" + name + "]");

            if(field)
                switch(field.tagName.toLowerCase()){
                    case "select":

                        let html = '<option value="" disabled selected hidden data-i18n="' + name + '_choose">' + anp.i18n_get(name + "_option") + '</option>';

                        anp.is_array(data[name]) && data[name].forEach(item => {

                            const i18n = item.i18n || item.name;

                            html += '<option value="' + item.value + '" data-i18n="' + i18n + '">' + anp.i18n_get(i18n, item.name) + '</option>';

                        });
                        field.innerHTML = html;

                    break;
                    default:
                        if(field.hasAttribute("data-module-name") && field.getAttribute("data-module-name") == "selector")
                            anp.comp.selectors.set(field, data[name]);
                    break;
                };

        };

    };

    this.go_save = (key, url, _new) => {

        document.querySelectorAll((key ? "[data-view=" + key + "] " : "") + "form.form").forEach(form => {

            const view = form.parentNode;

            if(!view || !view.hasAttribute || !view.hasAttribute("data-view"))
                return;

            cache[view.getAttribute("data-view")] = self.get_values(form);

        });

        anp.go(url, _new);

    };

    this.go_remove = (key, url, _new) => {

        if(key){
            if(cache[key])
                delete cache[key];
        }else 
            document.querySelectorAll("form.form").forEach(form => {

                const view = form.parentNode;

                if(!view || !view.hasAttribute || !view.hasAttribute("data-view"))
                    return;

                const key = view.getAttribute("data-view");

                if(cache[key])
                    delete cache[key];

            });

        anp.go(url, _new);

    };

    this.drop_cache = () => {
        for(const key in cache)
            delete cache[key];
    };

    this.set_texts = (form, key, variables) => {

        const legend = (form = self.get(form)).querySelector("legend"),
              paragraph = form.querySelector("legend+p");

        legend.setAttribute("data-i18n", key);
        legend.innerText = anp.i18n_get(key, variables);

        paragraph.setAttribute("data-i18n", key + "_text");
        paragraph.innerText = anp.i18n_get(key + "_text", variables);

        if(variables){
            variables = anp.encode_set(variables);
            legend.setAttribute("data-i18n-variables", variables);
            paragraph.setAttribute("data-i18n-variables", variables);
        };

    };

    this.get_variables = form => {
        try{
            return anp.decode_set(self.get(form).querySelector("legend").getAttribute("data-i18n-variables"));
        }catch(exception){};
        return null;
    };

    construct();

};
