AnP.Components.Maps = function(anp, input){

    // http://leaflet-extras.github.io/leaflet-providers/preview/index.html
    // https://leafletjs.com/reference.html

    const self = this,
          minimums = {
              longitude : -180,
              latitude : -90,
              altitude : 3600000
          },
          maximums = {
              longitude : 180,
              latitude : 90,
              altitude : 100000
          },
          steps = {
              longitude : .000001,
              latitude : .000001,
              altitude : 1
          },
          maps = {},
          geolocation = {
              longitude : 0, 
              latitude : 0, 
              altitude : 0
          },
          providers = {};
    let started = false,
        thread = null,
        check_last = 0,
        check_time = 2000,
        precission = 6,
        custom_options = {};

    const construct = () => {};

    const update_geolocation = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        navigator.geolocation.getCurrentPosition(location => {
            geolocation.longitude = location.coords.longitude;
            geolocation.latitude = location.coords.latitude;
            geolocation.altitude = location.coords.accuracy;
            end(true);
        }, () => end(false), {
            enableHighAccuracy: true,
            timeout: check_time,
            maximumAge: 0
        });

    };

    const set = value => Math.round(value * 10 ** precission) / 10 ** precission;

    const create_map = (id, longitude, latitude, altitude) => {

        const box = anp.item_self.querySelector("#" + id),
              provider = box.getAttribute("data-provider"),
              options = {
                  ...((provider ? providers[provider.replace(/[^a-zA-Z0-9_]+/g, "_").toLowerCase()] : null) || {
                      url : "https://tile.openstreetmap.org/{z}/{x}/{y}.png",
                      attribution : '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                  }),
                  ...custom_options
              },
              url = options.url;

        if(options.url)
            delete options.url;

        maps[id] = {
            map : L.map(id).setView([latitude, longitude], 13),
            width : 0,
            height : 0,
            loaded : false,
            box : box,
            markers : {}
        };

        L.tileLayer(url, options).addTo(maps[id].map);

    };

    const thread_method = () => {

        const now = Date.now();

        anp.item_self.querySelectorAll("[data-map-loaded=false]").forEach(box => {

            const id = box.querySelector(".map-box>div").getAttribute("id");
            let longitude = box.querySelector("input[data-i18n=longitude]").value,
                latitude = box.querySelector("input[data-i18n=latitude]").value;

            longitude = longitude ? Number(longitude) : geolocation.longitude;
            latitude = latitude ? Number(latitude) : geolocation.latitude;

            box.setAttribute("data-map-loaded", true);

            create_map(id, longitude, latitude, null);

            maps[id].markers.main = L.marker([latitude, longitude]).addTo(maps[id].map);

            maps[id].map.on("click", event => {

                const longitude = event.latlng.lng,
                      latitude = event.latlng.lat;
                let box = event.originalEvent.target;

                maps[id].markers.main.setLatLng(event.latlng);

                while(box && box.hasAttribute && !box.hasAttribute("data-map-loaded") && (box = box.parentNode));

                if(box){

                    const field = box.querySelector("[type=text]"),
                          coordenates = [set(longitude), set(latitude), set(0)];

                    box.querySelector("input[data-i18n=longitude]").value = coordenates[0];
                    box.querySelector("input[data-i18n=latitude]").value = coordenates[1];
                    box.querySelector("input[data-i18n=altitude]").value = coordenates[2];
                    get(box).setAttribute("data-has-value", true);
                    field && (field.value = coordenates.join(","));

                };

            });

        });

        anp.item_self.querySelectorAll("[data-map-sample-loaded=false]").forEach(box => {

            const id = box.getAttribute("id"),
                  longitude = Number(box.getAttribute("data-longitude")),
                  latitude = Number(box.getAttribute("data-latitude")),
                  provider = box.
                  options = {
                      attribution : '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                  };

            box.setAttribute("data-map-sample-loaded", true);

            create_map(id, longitude, latitude, null);

            maps[id].markers.main = L.marker([latitude, longitude]).addTo(maps[id].map);

        });

        if(now - check_last > check_time){
            check_last = now;

            update_geolocation(status => {
                for(const key in maps)
                    if(!anp.item_self.querySelector("#" + key))
                        delete maps[key];
            });

        };

        for(const key in maps)
            if(maps[key].width != maps[key].map.offsetWidth || maps[key].height != maps[key].map.offsetHeight){

                maps[key].width = maps[key].map.offsetWidth;
                maps[key].height = maps[key].map.offsetHeight;

                if(maps[key].loaded)
                    maps[key].map.invalidateObserve(maps[key].box);
                else 
                    maps[key].loaded = true;

            };

    };

    const add_providers = (inputs, overwrite, callback, i) => {

        if(i >= inputs.length){
            anp.is_function(callback) && callback();
            return;
        };

        const end = () => add_providers(inputs, overwrite, callback, i + 1);

        if(anp.is_array(inputs[i]))
            add_providers(inputs[i], overwrite, end, 0);
        else if(anp.is_object(inputs[i])){
            for(const key in inputs[i])
                (overwrite || !providers[key]) && (providers[key] = inputs[i][key]);
            end();
        }else if(anp.is_string(inputs[i])){

            let json;

            try{
                json = anp.json_decode(inputs[i]);
            }catch(exception){};

            if(json)
                add_providers(anp.is_array(json) ? json : [json], overwrite, end, 0);
            else 
                anp.ajax.get(inputs[i], response => {
                    try{
                        json = anp.json_decode(response);
                    }catch(exception){};
                    if(json)
                        add_providers(anp.is_array(json) ? json : [json], overwrite, end, 0);
                    else 
                        end();
                });

        }else 
            end();

    };

    this.add_providers = (inputs, overwrite, callback) => add_providers(anp.is_array(inputs) ? inputs : [inputs], anp.is_bool(overwrite) ? overwrite : anp.settings_get(["maps_providers_overwrite", "overwrite"]), callback, 0);

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        anp.comp.place = self.build_place;
        anp.comp.place_sample = self.sample;

        if(anp.settings_get("allow_geolocation")){
            thread = anp.threads.add(thread_method);
            self.add_providers(anp.settings_get("default_maps_providers_files"), true, () => {
                self.add_providers(anp.settings_get("maps_providers_files"), true, () => {
                    // console.log(providers);
                    end(true);
                });
            });
        }else 
            end(true);

        return true;
    };

    const place_form_field = (name, key, input) => {

        const text = anp.i18n.get(key),
              hash = anp.hash();

        !input && (input = {});

        return (`
            <div` + anp.attr.set({
                i18n : key,
                class : key,
                title : text,
                i18n_without : true,
                data_module_name : "maps"
            }) + `>
                <label` + anp.attr.set({for : hash}) + `>
                    ` + anp.comp.icon(key, "span") + `
                    ` + anp.comp.i18n(key, "span") + `
                </label>
                <span` + anp.attr.set({class : "component"}) + `>` + anp.comp.number({
                    id : hash,
                    name : name + "_" + key,
                    i18n : key,
                    i18n_without : true,
                    placeholder : text + "...",
                    readonly : input.readonly || input.only_click,
                    data_module_name : "map",
                    data_main_name : name, 
                    disabled : input.disabled,
                    min : isNaN(input.minimums) ? input.minimums && !isNaN(input.minimums[key]) ? input.minimums[key] : minimums[key] : input.minimums,
                    max : isNaN(input.maximums) ? input.maximums && !isNaN(input.maximums[key]) ? input.maximums[key] : maximums[key] : input.maximums,
                    step : isNaN(input.steps) ? input.steps && !isNaN(input.steps[key]) ? input.steps[key] : steps[key] : input.steps,
                    ...(input && !isNaN(input[key]) ? {value : input[key]} : {})
                }) + `</span>
            </div>
        `);
    };

    this.build_place = input => {

        const hashes = {
                  longitude : anp.hash(),
                  latitude : anp.hash(),
                  altitude : anp.hash()
              },
              name = input.name || input.i18n;

        return (`
            <div` + anp.attr.set({
                class : "maps-place", 
                data_field : name, 
                data_map_loaded : false,
                data_has_value : anp.is_number(input.longitude) || anp.is_number(input.latitude) || anp.is_number(input.altitude)
            }) + `>
                <div` + anp.attr.set({class : "subform"}) + `>
                    ` + place_form_field(name, "longitude", input) + `
                    ` + place_form_field(name, "latitude", input) + `
                    ` + place_form_field(name, "altitude", input) + `
                </div>
                <input` + anp.attr.set({type : "text", name : input.name, data_module_name : "map"}) + ` />
                <div` + anp.attr.set({class : "map-box"}) + `>
                    <div` + anp.attr.set({
                        id : anp.hash(),
                        data_provider : anp.settings_get(["maps_provider", "provider"], input)
                    }) + `></div>
                </div>
            </div>
        `);
    };

    this.get_current_geolocation = () => ({...geolocation});

    const get = map => {

        if(map)
            while(map.classList && !map.classList.contains("maps-place") && (map = map.parentNode));

        return map || null;
    };

    this.set_raw = (map, location) => {

        const value = location.split(","),
              id = (map = get(map)).querySelector(".map-box>div").getAttribute("id"),
              geolocation = new L.LatLng(Number(value[1]), Number(value[0]));

        map.querySelectorAll(".subform input").forEach((field, i) => field.value = set(value[i]));
        maps[id].markers.main.setLatLng(geolocation);
        maps[id].map.panTo(geolocation);
        map.setAttribute("data-has-value", true);

    };

    this.set = (map, longitude, latitude, altitude) => {

        const id = (map = get(map)).querySelector(".map-box>div").getAttribute("id"),
              geolocation = new L.LatLng(latitude, longitude),
              has_value = anp.is_number(longitude) || anp.is_number(latitude) || anp.is_number(altitude);

        if(has_value){
            map.querySelector("[type=text]").value = (
                (map.querySelector(".longitude input").value = set(longitude)) + "," + 
                (map.querySelector(".latitude input").value = set(latitude)) + "," + 
                (map.querySelector(".altitude input").value = set(altitude))
            );
            maps[id].markers.main.setLatLng(geolocation);
            maps[id].map.panTo(geolocation);
        };
        map.setAttribute("data-has-value", has_value);

    };

    this.get_raw = map => {

        map = get(map);

        return [
            Number(map.querySelector("input[data-i18n=longitude]").value),
            Number(map.querySelector("input[data-i18n=latitude]").value),
            Number(map.querySelector("input[data-i18n=altitude]").value)
        ].join(",");
    };

    this.sample = (key, data, input) => anp.null_or_undefined(data[key + "_longitude"]) || isNaN(data[key + "_longitude"]) ? `-` : `<div` + anp.attr.set({
        id : anp.hash(),
        class : "maps-place-sample",
        data_longitude : data[key + "_longitude"],
        data_latitude : data[key + "_latitude"],
        data_map_sample_loaded : false,
        data_provider : anp.settings_get(["maps_provider", "provider"], input)
    }) + `></div>`;

    this.get_values = map => {

        const is_null = (map = get(map)).getAttribute("data-has-value") == "false";

        return ["longitude", "latitude", "altitude"].map(key => is_null ? null : Number(map.querySelector("." + key + " input").value));
    };

    this.get = map => self.get_values(map).join(",");

    construct();

};