AnP.Components.Selectors = function(anp, input){

    const self = this,
          cache = {};
    let started = false,
        thread = null;

    const construct = () => {};

    const get = item => {

        if(item){
            anp.is_string(item) && (item = anp.item_self.querySelector(item));
            while(item.hasAttribute && (!item.hasAttribute("data-module") || item.getAttribute("data-module") != "selector") && (item = item.parentNode));
        };

        return item && item.hasAttribute ? item : null;
    };

    const get_box = item => {

        if(item)
            while(item.hasAttribute && (!item.hasAttribute("data-module") || item.getAttribute("data-module") != "selector_box") && (item = item.parentNode));

        return item && item.hasAttribute ? item : null;
    };

    const get_field = this.get_field = item => anp.item_self.querySelector("[data-module=selector][for=" + get_box(item).getAttribute("data-for") + "]");

    const search = () => {

        const actived = [];

        anp.item_self.querySelectorAll("[data-module=selector_box]").forEach(box => {

            const hash = box.getAttribute("data-for"),
                  text = box.querySelector("[type=text]").value.toLowerCase();

            actived.push(hash);
            if(cache[hash] !== undefined && cache[hash] == text)
                return;

            cache[hash] = text;
            box.querySelectorAll(".list>label").forEach(item => item.setAttribute("data-visible", !text || item.querySelector("span[data-i18n]").innerText.toLowerCase().indexOf(text) != -1));

        });

        for(const key in cache)
            if(!actived.includes(key))
                delete cache[key];

    };

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp.comp.selector = self.create;
        thread = anp.threads.add(search);

        anp.is_function(callback) && callback();

    };

    const options_html = (options, selected, onchange, format) => {

        let html = ``,
            was_selected = false;
        const name = anp.hash(),
              no_selected = anp.null_or_undefined(selected);

        options && (anp.is_array(options) ? options : [options]).forEach(option => {
            if(!anp.is_dictionary(option))
                return;

            // console.log([selected, format, option]);

            const hash = anp.hash(),
                  is_selected = !was_selected && (no_selected ? !!option.selected : selected == option.value),
                  i18n = format || option.name || option.i18n,
                  variables = anp.json_decode(option.name || option.i18n),
                  text = anp.i18n_get(i18n, variables);

            if(is_selected){
                was_selected = true;
                selected = {
                    i18n : i18n,
                    name : text,
                    value : option.value,
                    data_i18n_variables : variables ? anp.json_encode(variables) : "{}"
                };
            };

            html += (`
                <label` + anp.attr.set({class : "option", for : hash, data_value : option.value, data_i18n : i18n, data_i18n_without : true, title : text, onclick : "{object_name}.comp.selectors.select(this, event);" + (onchange ? onchange.replace(/\(this,/, "(" + anp.object_name + ".comp.selectors.get_field(this),") : ""), data_selected : is_selected, data_visible : true}) + `>
                    ` + anp.comp.radio({name : name, value : option.value, id : hash, checked : is_selected}) + `
                    ` + anp.comp.i18n(i18n, "span", variables) + `
                </label>
            `);

        });

        return [html, selected || null];
    };

    this.deploy = (item, event) => {

        const selector = get(item);

        if(selector.getAttribute("data-disabled") == "true")
            return;

        if(anp.item_self.querySelector("[data-hash=" + selector.getAttribute("data-hash") + "].selector-box"))
            return;

        const search_text = anp.i18n_get("search"),
              preloader = anp.hash(),
              body = document.querySelector("body"),
              bounds = selector.getBoundingClientRect(),
              position = {},
              selected = self.get_value(selector),
              box = selector.querySelector("[type=text],span");

        new AnP.Preload(anp, "[data-preloader=" + preloader + "]", preload_item => {
            if(!preload_item)
                return;
            preload_item.parentNode.querySelector("[type=text]").focus();
            preload_item.remove();
        });

        if(body.offsetHeight / 2 > bounds.top)
            position.top = bounds.top + selector.offsetHeight;
        else
            position.bottom = body.offsetHeight - bounds.top;

        anp.comp.create_float_box(item, event, Object.assign(position, {
            name : "selector_box",
            left : bounds.left,
            width : selector.offsetWidth,
            html : (`
                <fieldset>
                    ` + anp.comp.i18n("options", "legend") + `
                    ` + anp.comp.text({data_i18n : "search", data_i18n_without : true, title : search_text, placeholder : search_text + "...", onkeydown : anp.object_name + ".comp.selectors.key_down(this, event);", onkeyup : anp.object_name + ".comp.selectors.key_up(this, event);"}) + `
                    <div` + anp.attr.set({class : "list"}) + `>` + options_html(
                        anp.json_decode(anp.base64_decode(selector.getAttribute("data-options"))), 
                        anp.null_or_undefined(selected) ? selector.getAttribute("data-default-value") : selected, 
                        box.getAttribute("data-onchange"), 
                        box.getAttribute("data-format")
                    )[0] + `</div>
                    <div` + anp.attr.set({data_preloader : preloader}) + `></div>
                </fieldset>
            `)
        }));

    };

    this.key_down = (item, event) => {
        switch(event.keyCode){
            case 9:

                const box = get_box(item);
                let from = anp.item_self.querySelector("[data-module=selector][for=" + box.getAttribute("data-for") + "]");

                if(from)
                    while(from.classList && !from.classList.contains("field") && (from = from.parentNode));
                if(from){

                    const fields = from.parentNode.querySelectorAll(".field"),
                          l = fields.length;
                    let i = 0,
                        field;

                    for(; i < l; i ++)
                        if(fields[i] == from){
                            i = (i + 1) % l;
                            break;
                        };

                    console.log([i, fields[i]]);

                    if(field = fields[i].querySelector("[name]"))
                        field.focus();
                    else 
                        with(fields[i].querySelector("[data-module] button")){
                            focus();
                            click();
                        };

                };

                event.preventDefault();
                box.remove();

            break;
            case 27:
                get_box(item).remove();
            break;
        };
    };

    this.create = input => {

        !anp.is_dictionary(input) && (input = {});
        !input.i18n && (input.i18n = input.name);

        const text = input.i18n ? anp.i18n_get(input.i18n) : "",
              hash = input.id || input.hash ||anp.hash(),
              input_attributes = Object.assign({}, input),
              has_selected = !anp.null_or_undefined(input.value);
        let options = input.options || [];

        if(input_attributes.class)
            delete input_attributes.class;
        if(input_attributes.onchange){
            input_attributes.data_onchange = input_attributes.onchange;
            delete input_attributes.onchange;
        };

        Object.assign(input_attributes, {
            onclick : anp.object_name + ".comp.selectors.deploy(this, event);",
            readonly : true,
            data_module_name : "selector"
        });

        !anp.is_array(options) && (options = [options]);

        (input_attributes.options || []).some(option => {
            if(has_selected ? option.value == input.value : option.selected){
                input_attributes.i18n = option.name || option.i18n;
                input.value = input_attributes.value = anp.i18n.get(option.name || option.i18n);
                input_attributes.data_value = option.value;
                return true;
            };
            return false;
        });
        ["options", "name", "type"].forEach(key => {
            if(input_attributes[key])
                delete input_attributes[key];
        });

        return (`
            <label` + anp.attr.set([
                {
                    class : "selector group" + (input.class ? " " + input.class : ""), 
                    data_module : "selector", 
                    for : hash, data_options : anp.base64_encode(anp.json_encode(options)), 
                    data_disabled : input.disabled || false
                },
                input.i18n ? {data_i18n : input.i18n, data_i18n_without : true, title : text} : null,
                input.value ? {data_default_value : input.value} : null
            ]) + `>
                ` + (
                    // anp.comp.text({...input_attributes, type : "text"})
                    `<span` + anp.attr.set({...input_attributes, ...(input.value ? {data_value : input.value} : {})}, "value") + `>` + anp.i18n_get(input.i18n || input.name) + `...</span>`
                ) + `
                <input` + anp.attr.set({type : "hidden", ...(input.name ? {name : input.name} : {}), data_module_name : "selector"}) + ` />
                ` + anp.comp.button({i18n : "deploy", icon : "deploy", onclick : input_attributes.onclick, disabled : input.disabled || false}) + `
            </label>
        `);
    };

    this.set = (selector, options, value) => {
        if(!selector)
            return;

        if(anp.is_string(selector))
            try{selector = anp.item_self.querySelector(selector);}catch(exception){};
        if(!anp.is_html_object(selector)|| !(selector = get(selector)))
            return;

        const box = selector.querySelector("[type=text],span"),
              [html, selected] = options_html(
                  options, 
                  anp.null_or_undefined(value) ? selector.getAttribute("data-default-value") : value, 
                  box.getAttribute("data-onchange"), 
                  box.getAttribute("data-format")
              );

        selector.setAttribute("data-options", anp.base64_encode(anp.json_encode(options)));
        anp.item_self.querySelectorAll("[data-for=" + selector.getAttribute("for") + "] .list").forEach(item => item.innerHTML = html);

        if(selected){

            const field = selector.querySelector("[type=hidden]"),
                  text_field = selector.querySelector("[type=text],span");

            anp.attr.set(text_field, {
                data_i18n : selected.i18n,
                data_value : selected.value
            })[text_field.tagName.toLowerCase() == "input" ? "value" : "innerHTML"] = selected.name;
            field && (field.value = selected.value);

        };

    };

    const set_value = (item, name, value, i18n) => {

        const field = item.querySelector("[type=hidden]"),
              sample = item.querySelector("[type=text],span"),
              format = sample.getAttribute("data-format");

        format && (name = anp.string_variables(format, anp.json_decode(name)));

        anp.attr.set(sample, {
            data_i18n : name,
            data_value : value
        })[sample.tagName.toLowerCase() == "input" ? "value" : "innerHTML"] = i18n || anp.i18n_get(name);
        field && (field.value = value);

    };

    this.select = (item, event) => {

        const box = get_box(item);

        set_value(anp.item_self.querySelector("[data-module=selector][for=" + box.getAttribute("data-for") + "]"), item.getAttribute("data-i18n"), item.getAttribute("data-value"), item.getAttribute("title"));
        box.remove();

    };

    this.get_value = item => get(item).querySelector("[type=text],span").getAttribute("data-value");

    this.set_value = (item, value) => {
        if(!item)
            return;

        const field = get(item);

        if(!field)
            return;

        const options = anp.json_decode(anp.base64_decode(field.getAttribute("data-options")));

        anp.json_decode(anp.base64_decode(field.getAttribute("data-options"))).some(item => {
            if(item.value == value){
                set_value(field, item.name, item.value);
                return true;
            };
        });

    };

    this.key_up = (item, event) => {

        if(event.keyCode == 13){
            item.parentNode.parentNode.querySelector(".list>label>.checkbox [type=radio]:checked").parentNode.parentNode.click();
            return;
        };

        const position = (
            event.keyCode == 38 ? -1 : 
            event.keyCode == 40 ? 1 : 
        0);
        
        if(!position)
            return;

        const options = item.parentNode.parentNode.querySelectorAll(".list>label>.checkbox [type=radio]"),
              l = options.length;
        let i = 0;

        for(; i < l; i ++)
            if(options[i].checked){
                options[i].checked = false;
                i = i + position;
                break;
            };

        options[i = (i + l) % l] && (options[i].checked = true);

    };

    this.disable = (selector, disabled) => {
        if(selector){
            selector.setAttribute("data-disabled", disabled);
            if(disabled)
                selector.querySelector("button[data-i18n=deploy]").setAttribute("disabled", true);
            else 
                with(selector.querySelector("button[data-i18n=deploy]"))hasAttribute("disabled") && removeAttribute("disabled");
        };
    };

    construct();

};
