AnP.Components.SyntaxHighlighter = function(anp, input){

    const self = this,
          cache = {};
    let started = false,
        thread = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        anp.comp.syntax_highlighter_module = self.create;
        thread = anp.threads.add(thread_function);

        document.addEventListener("selectionchange", selection_change);

        end(true);

        return true;
    };

    const field = (key, value, extra) => (`<span` + anp.attr.set({
        class : "field " + key.replace(/[^a-z0-9]+/g, "-"),
        data_i18n : key,
        data_i18n_without : true,
        title : anp.i18n.get(key), 
        ...(extra || {})
    }) + `>
        ` + anp.comp.icon(key, "span") + `
        ` + anp.comp.i18n(key, "span") + `
        <span` + anp.attr.set({class : "value"}) + `>` + value + `</span>
    </span>`);

    this.create = input => {

        const key = anp.settings_get(["key", "syntax_highlighter_key"], input = (
                  anp.is_array(input) || anp.is_dictionary(input) ? input : 
                  anp.is_string(input) ? {name : input} : 
              {})),
              maximum_length = input.maxlength || null;

        ["down", "up"].forEach(subkey => {
            !input["onkey" + subkey] && (input["onkey" + subkey] = "");
            input["onkey" + subkey] && !/;$/.test(input["onkey" + subkey]) && (input["onkey" + subkey] += ";");
            input["onkey" + subkey] += "{object_name}.comp.syntax_highlighter.change(this, event, \"" + subkey + "\");";
        });

        return (`<div` + anp.attr.set({
            class : "syntax-highlighter-component",
            data_component : "SyntaxHighlighter",
            data_hash : anp.hash(), 
            data_key : key
        }) + `>
            ` + field("language", key) + `
            <div` + anp.attr.set({class : "back"}) + `>
                <div` + anp.attr.set({class : "background"}) + `></div>
                <ol` + anp.attr.set({class : "lines", start : 1, type : 1}) + `></ol>
                <div` + anp.attr.set({class : "syntax-highligher"}) + `></div>
            </div>
            <div` + anp.attr.set({class : "text-box"}) + `>
                <textarea` + anp.attr.set({
                    data_i18n : key,
                    data_i18n_without : true,
                    placeholder : anp.i18n.get(key) + "...",
                    data_component_name : "SyntaxHighlighter", 
                    ...input
                }, null, [
                    "name", "maxlength", "onkeydown", "onkeyup", "data_i18n", 
                    "data_i18n_without", "placeholder", "data_component_name"
                ]) + `>` + (input.value || "") + `</textarea>
            </div>
            <div` + anp.attr.set({class : "info"}) + `>
                ` + field("line", 0) + `
                ` + field("column", 0) + `
                ` + field("selected", 0) + `
                ` + field("lines", 0) + `
                ` + field("characters", 0) + `
                ` + field("maximum_characters", maximum_length, {data_maximum_length : maximum_length}) + `
            </div>
            ` + anp.comp.buttons({buttons : self["test_" + key] ? [{i18n : "test", icon : "test", onclick : "{object_name}.comp.syntax_highlighter.test_" + key + "(this, event);"}] : []}) + `
        </div>`);
    };

    const get = item => {

        if(item)
            while(
                item.hasAttribute && 
                (!item.hasAttribute("data-component") || item.getAttribute("data-component") != "SyntaxHighlighter") && 
                (item = item.parentNode)
            );

        return item || null;
    };

    this.change = (item, event, direction) => {
        // console.log([event.keyCode, direction, event.key]);

        const t = {
                  start_fragment : item.value.substring(0, item.selectionStart),
                  start : item.selectionStart + 1,
                  fragment : item.value.substring(item.selectionStart, item.selectionEnd),
                  end_fragment : item.value.substring(item.selectionEnd),
                  end : item.selectionEnd
              },
              box = get(item);

        box.querySelector(".characters>.value").innerText = item.value.length;
        box.querySelector(".lines>.value").innerText = item.value.split(/\r\n|[\r\n]/).length;

        switch(event.keyCode){
            case 9: // Tabulation.
                if(direction == "down"){
                    event.preventDefault();

                    item.value = t.start_fragment + "\t" + t.end_fragment;
                    item.setSelectionRange(t.start, t.start);

                };
            break;
            case 13: // Line break.
                if(direction == "down"){
                    event.preventDefault();

                    t.tabulations = (t.tabulations = t.start_fragment.split(/\r\n|[\r\n]/))[t.tabulations.length - 1].match(/^(\s*)/)[0] || "";
                    t.last_character = item.value.substring(item.selectionStart - 1, item.selectionStart);
                    t.is_block = ["{", "[", "("].includes(t.last_character);
                    t.start += t.tabulations.length + (t.is_block ? 1 : 0);

                    item.value = t.start_fragment + "\n" + t.tabulations + (
                        t.is_block ? "\t\n" + t.tabulations : ""
                    ) + t.end_fragment;
                    item.setSelectionRange(t.start, t.start);

                };
            break;
            default:
                
                if(direction == "down"){

                    if(["{", "[", "(", "\"", "'"].includes(event.key)){ // Blocks.
                        event.preventDefault();

                        item.value = t.start_fragment + event.key + t.fragment + {
                            "{" : "}",
                            "[" : "]",
                            "(" : ")",
                            "\"" : "\"",
                            "'" : "'"
                        }[event.key] + t.end_fragment;
                        item.setSelectionRange(t.start, t.start + t.fragment.length);

                    }else if(["}", "]", ")"].includes(event.key)){ // Blocks closers.
                        if(/^\s+$/.test((t.lines = t.start_fragment.split(/\r\n|[\r\n]/))[t.lines.length - 1])){
                            event.preventDefault();

                            t.start --;
                            item.value = t.start_fragment.substring(0, t.start - 1) + event.key + t.end_fragment;
                            item.setSelectionRange(t.start, t.start);

                        };
                    };

                };

            break;
        };

    };

    const selection_change = event => {

        const item = event.target.activeElement;

        if(
            item && 
            item.hasAttribute && 
            item.hasAttribute("data-component-name") && 
            item.getAttribute("data-component-name") == "SyntaxHighlighter"
        ){
            const box = get(item),
                  lines = item.value.substring(0, item.selectionStart).split(/\r\n|[\r\n]/);

            box.querySelector(".line>.value").innerText = lines[lines.length - 1].length;
            box.querySelector(".column>.value").innerText = lines.length;
            box.querySelector(".selected>.value").innerText = item.value.substring(item.selectionStart, item.selectionEnd).length;
        };

    };

    this.test_wmarkdown = (item, event) => {};

    const thread_function = () => {};

    construct();

};