AnP.Components.Tables = function(anp, input){

    const self = this,
          order_index = ["order_no", "order_ascendent", "order_descendent"],
          cache = {},
          bool_check = ["unknown", "checked", "unchecked"];
    let started = false,
        thread = null;

    const construct = () => {};

    const thread_function = () => {

        const hashes = [];

        anp.item_self.querySelectorAll("form.table").forEach(table => {

            const header = table.querySelector("thead"),
                  hash = table.getAttribute("data-hash");

            hashes.push(hash);

            !cache[hash] && (cache[hash] = {
                sizes : {},
                order : []
            });

            ["menus", "header", "search", "pagination", "actions", "options", "columns"].forEach(key => {

                const visible = table.getAttribute("data-" + key + "-visible") == "true",
                      toggle = table.querySelector(".menus fieldset.options button[data-i18n=show_" + key + "]");

                toggle.getAttribute("data-toggle") != (visible ? "true" : "false") && toggle.setAttribute("data-toggle", visible);

            });
            table.querySelectorAll(".table>table>tfoot>:first-child>th").forEach((cell, i) => {

                const name = cell.getAttribute("data-field"),
                      item = header.querySelector(".order>[data-field='" + name + "']"),
                      size = item.getBoundingClientRect().width;

                if(cache[hash].sizes[name] == size)
                    return;

                const computed_styles = getComputedStyle(item)

                cell.style.width = cell.style.minWidth = cell.style.maxWidth = (cache[hash].sizes[name] = size - (parseFloat(computed_styles.paddingLeft) + parseFloat(computed_styles.paddingRight))) + "px";

            });

        });

        for(const hash in cache)
            if(!hashes.includes(hash))
                delete cache[hash];

    };

    this.start = callback => {

        if(started)
            return;
        started = true;

        anp.comp.table = self.create;
        thread = anp.threads.add(thread_function);

        anp.is_function(callback) && callback();

    };

    const get = this.get = table => {

        if(table)
            while((!table.hasAttribute || !table.hasAttribute("data-module") || !["table"].includes(table.getAttribute("data-module"))) && (table = table.parentNode));

        return table || null;
    };

    this.show_menus = (item, event) => (item = get(item)) && item.setAttribute("data-menus-visible", true);

    this.hide_menus = (item, event) => (item = get(item)) && item.setAttribute("data-menus-visible", false);

    this.show = (item, event, key) => (item = get(item)) && item.setAttribute("data-" + key + "-visible", !item.hasAttribute("data-" + key + "-visible") || item.getAttribute("data-" + key + "-visible") != "true");

    this.refresh = (item, event) => {

        const table = get(item);

        table && table.onsubmit(table, event);

    };

    this.clean = (item, event) => (item = get(item)) && item.reset();

    this.download = (item, event) => {};

    this.show_options = (item, event) => (item = get(item)) && item.setAttribute("data-options-visible", true);

    this.order_set = (item, event) => {

        const table = get(item);

        if(!table)
            return;

        const field = item.getAttribute("data-field"),
              hash = table.getAttribute("data-hash"),
              i = (order_index.indexOf((item = item.querySelector("[data-icon]")).getAttribute("data-icon")) + 1) % order_index.length;

        item.setAttribute("data-icon", order_index[i]);

        !cache[hash] && (cache[hash] = {
            sizes : {},
            order : []
        });

        const m = cache[hash].order.length;

        if(i){

            let exists = false;

            for(let j = 0; j < m; j ++)
                if(exists = cache[hash].order[j][0] == field){
                    cache[hash].order[j][1] = i - 1;
                    break;
                };
            !exists && cache[hash].order.push([field, i - 1]);

        }else
            for(let j = m - 1; j >= 0 ; j --)
                cache[hash].order[j][0] == field && cache[hash].order.splice(j, 1);

        self.refresh(item, event);

    };

    this.bool_where_change = (item, event) => {

        item.setAttribute("data-icon", bool_check[(bool_check.indexOf(item.getAttribute("data-icon")) + 1) % bool_check.length]);
        self.refresh(item);

    };

    const remove_events = variables => Object.assign({}, ...Object.entries(variables).filter(([key, value]) => key.indexOf("on")).map(([key, value]) => ({[key] : value})));

    const type = (input, _default) => {

        const type = input.type || _default, 
              data = [];

        input = Object.assign({}, input);

        ["type", "visible", "data_visible", "hidden", "data_hidden"].forEach(attribute => {
            if(input[attribute] !== undefined)
                delete input[attribute];
        });

        switch(type){
            case "integer":
            case "number":
            case "float":
            case "session":
            case "size":
                ["from", "to"].forEach(key => data.push(remove_events(Object.assign({}, input, {name : input.name + "[]"}, {i18n : key}))));
                return (`
                    <span` + anp.attr.set({class : "group", data_direction : "vertical"}) + `>
                        ` + anp.comp.number(data[0]) + `
                        ` + anp.comp.number(data[1]) + `
                    </span>
                `);
            case "datetime":
                ["from", "to"].forEach(key => data.push(remove_events(Object.assign({}, input, {name : input.name + "[]"}, {i18n : key}))));
                return (`
                    <span` + anp.attr.set({class : "group", data_direction : "vertical"}) + `>
                        ` + anp.comp.datetime(data[0]) + `
                        ` + anp.comp.datetime(data[1]) + `
                    </span>
                `);
            case "bool":
            case "boolean":
                return `<span` + anp.attr.set({class : "bool-where", data_icon : input.state && bool_check.includes(input.state) ? input.state : bool_check[0], onclick : anp.object_name + ".comp.tables.bool_where_change(this, event);"}) + `></span>`;
        };
        return anp.comp.text(remove_events(input));
    };

    this.show_column = (item, event) => {

        const table = get(item);

        if(!table)
            return;

        const visible = item.getAttribute("data-toggle") == "true";

        table.querySelectorAll("tr>[data-field=" + item.getAttribute("data-field") + "]").forEach(cell => cell.setAttribute("data-visible", visible));

    };

    this.header_scroll = (item, event) => {

        let table_box = item;

        if(table_box)
            while((!table_box.classList || !table_box.classList.contains("table")) && (table_box = table.parentNode));

        if(!table_box)
            return;

        const header = table_box.querySelector("thead~tfoot");

        header.scrollLeft = table_box.scrollLeft;
        header.style.top = table_box.scrollTop + "px";
        header.style.width = table_box.querySelector("table").offsetWidth + "px";

    };

    const page_attribute = name => `<span` + anp.attr.set({class : name.replace(/_/g, "-"), data_i18n : name, data_i18n_without : true, title : anp.i18n_get(name)}) + `>0</span>`;

    const cell_custom_attributes = field => anp.attr.set(field, ["i18n", "name", "type", "no_order", "no_where", "key", "execute", "text", "hidden"]);

    const set_columns = (fields, input) => {

        const i18n = {
                  actions_pre : anp.i18n_get("actions_pre"),
                  actions_post : anp.i18n_get("actions_post")
              },
              header = {
                  order : (`
                      <th` + anp.attr.set({data_field : "actions_pre", data_i18n : "actions_pre", data_i18n_without : true, title : i18n.actions_pre}) + `>
                          ` + anp.comp.i18n("actions_pre") + `
                      </th>
                  `),
                  searcher : `<th` + anp.attr.set({data_field : "actions_pre", data_i18n : "actions_pre", data_i18n_without : true, title : i18n.actions_pre}) + `></th>`
              },
              columns = [],
              default_order = anp.settings_get(["tables_default_order", "tables_order", "order", "default_order"], input),
              default_type = anp.settings_get(["tables_default_type", "tables_type", "default_type"], input);

        fields.forEach(field => {
            if(anp.is_dictionary(field)){

                const name = field.name || "unknown",
                      i18n = field.i18n || name,
                      text = anp.i18n_get(i18n),
                      custom = remove_events(cell_custom_attributes(field));

                header.order += (`
                    <th` + anp.attr.set([{data_field : name, data_i18n : i18n, data_i18n_without : true, title : text, data_visible : !field.hidden}, field.no_order ? null : {onclick : anp.object_name + ".comp.tables.order_set(this, event);"}]) + custom + `>
                        ` + (field.no_order ? `` : anp.comp.icon("order_" + (field.order || default_order))) + `
                        ` + anp.comp.i18n(i18n, "span") + `
                    </th>
                `);
                header.searcher += (`
                    <th` + anp.attr.set({data_field : name, data_i18n : i18n, data_i18n_without : true, title : text, data_visible : !field.hidden, data_type : field.type || default_type}) + custom + `>
                        ` + (field.no_search ? `` : type(field, default_type)) + `
                    </th>
                `);
                columns.push({i18n : i18n, icon : field.icon || name, onclick : anp.object_name + ".comp.tables.show_column(this, event);", toggle : !field.hidden, data_field : name, data_type : field.type || "text"});

            };
        });

        header.order += (`
            <th` + anp.attr.set({data_field : "actions_post", data_i18n : "actions_post", data_i18n_without : true, title : i18n.actions_post}) + `>
                ` + anp.comp.i18n("actions_post") + `
            </th>
        `);
        header.searcher += `<th` + anp.attr.set({data_field : "actions_post", data_i18n : "actions_post", data_i18n_without : true, title : i18n.actions_post}) + `></th>`;

        return [header, columns];
    };

    this.create = (input, callback) => {

        !anp.is_dictionary(input) && (input = {});

        const hashes = {
                  search : anp.hash(),
                  page : anp.hash(),
                  preload : anp.hash()
              },
              i18n = {
                  search : anp.i18n_get("search"),
                  page : anp.i18n_get("page"),
                  actions_pre : anp.i18n_get("actions_pre"),
                  actions_post : anp.i18n_get("actions_post")
              },
              visible_attributes = {},
              visible_buttons = [],
              actions = [{i18n : "refresh", icon : "refresh", type : "submit"}],
              ellipsis = (`
                  <span` + anp.attr.set({class : "ellipsis", data_visible : false, data_i18n : "more", data_i18n_without : true, title : anp.i18n_get("more")}) + `>
                      ` + anp.comp.icon("ellipsis") + `
                      ` + anp.comp.i18n("more") + `
                  </span>
              `),
              table_pages = anp.settings_get(["tables_pages", "pages"], input);
        let [header, columns] = set_columns(input.structure ? anp.is_string(input.structure) ? input.structure = anp.views.get_raw(input.structure) : anp.is_array(input.structure) ? input.structure : input.structure = [input.structure] : input.structure = [], input),
            buttons_pre = ``,
            buttons_post = ``;

        ["menus", "header", "search", "pagination", "actions", "options", "columns", "columns_names", "advanced_search", "advanced_options", "actions_pre", "actions_post"].forEach(key => {

            const visible = anp.settings_get(["tables_" + key + "_visibles", key + "_visibles", key + "_visible"], input);

            visible_attributes["data_" + key + "_visible"] = visible;
            visible_buttons.push({i18n : "show_" + key, icon : key, onclick : anp.object_name + ".comp.tables.show(this, event, '" + key.replace(/_/g, "-") + "');", toggle : visible});

        });

        ["actions", "actions_pre", "actions_post"].forEach(key => input[key] && anp.is_string(input[key]) && (input[key] = anp.views.get_raw(input[key])));
        input.actions && (anp.is_array(input.actions) ? input.actions : [input.actions]).forEach(action => anp.is_dictionary(action) && actions.push(action));

        actions.push({i18n : "clean", icon : "clean", onclick : anp.object_name + ".comp.tables.clean(this, event);"});
        actions.push({i18n : "download", icon : "download", onclick : anp.object_name + ".comp.tables.download(this, event);", disabled : true});


        header = (`
            <tr` + anp.attr.set({class : "order"}) + `>` + header.order + `</tr>
            <tr` + anp.attr.set({class : "searcher"}) + `>` + header.searcher + `</tr>
        `);

        for(let i = 0; i < table_pages; i ++){

            const pre = table_pages - i,
                  post = i + 1;

            buttons_pre += `<button` + anp.attr.set({type : "button", title : pre, onclick : anp.object_name + ".comp.tables.go(this, event);", data_i : pre, data_visible : false}) + `>` + pre + `</button>`;
            buttons_post += `<button` + anp.attr.set({type : "button", title : post, onclick : anp.object_name + ".comp.tables.go(this, event);", data_i : post, data_visible : false}) + `>` + post + `</button>`;

        };

        new AnP.Preload(anp, "[data-preloader=" + hashes.preload + "]", preloader => {
            // console.log(hashes.preload);
            if(!preloader || !preloader.parentNode){
                anp.print("error", "tables_preloader_error");
                return;
            };

            const form = preloader.parentNode;

            preloader.remove();
            anp.settings_get(["tables_autoupdate", "autoupdate"], input) && form.onsubmit();

            [callback, input.callback, input.onload].forEach(method => {
                if(!method)
                    return;
                if(anp.function(method))
                    method(form);
                else try{
                    eval(method)(form);
                }catch(exception){};
            });

        });

        return (`
            <form` + anp.attr.set([{
                class : "table" + (input.class ? " " + input.class : ""),
                method : "post",
                action : "#",
                onsubmit : (input.refresh ? "try{" + anp.set_globals(input.refresh) + "(this, event);}catch(exception){console.error(exception);};" : "") + "return false;",
                onreset : "{object_name}.comp.tables.reset(this, event);", 
                data_module : "table",
                data_hash : anp.hash(),
                data_actions_direction : anp.settings_get(["tables_default_actions_direction", "tables_actions_direction", "default_actions_direction", "actions_direction", "direction"], input),
                data_table_pages : table_pages
            }, visible_attributes]) + `>
                <fieldset>
                    ` + anp.comp.i18n(input.i18n, "legend") + `
                    ` + anp.comp.i18n(input.i18n + "_text", "p") + `
                    <div` + anp.attr.set({class : "search group"}) + `>
                        <label` + anp.attr.set({data_i18n : "search", for : hashes.search, title : i18n.search}) + `>` + i18n.search + `</label>
                        ` + anp.comp.text({i18n : "search", name : "search", id : hashes.search}) + `
                        ` + anp.comp.button({i18n : "with_hides", icon : "with_hides", onclick : anp.object_name + ".comp.tables.with_hides(this, event);", toggle : anp.settings_get(["table_with_hides", "with_hides"], input)}) + `
                        ` + anp.comp.button({i18n : "case_sensitive", icon : "case_sensitive", onclick : anp.object_name + ".comp.tables.case_sensitive(this, event);", toggle : anp.settings_get(["table_case_sensitive", "case_sensitive"], input)}) + `
                        ` + anp.comp.button({i18n : "search", icon : "search", onclick : anp.object_name + ".comp.tables.refresh(this, event);"}) + `
                    </div>
                    <div` + anp.attr.set({class : "table", onscroll : anp.object_name + ".comp.tables.header_scroll(this, event);"}) + `>
                        <table>
                            <thead>` + header.replace(/ (for|name|id|data\-(hash|for|module))\="[^"]+"/g, "") + `</thead>
                            <tbody></tbody>
                            <tfoot>` + header + `</tfoot>
                        </table>
                    </div>
                    <div` + anp.attr.set({class : "menus"}) + `>
                        ` + anp.comp.button({i18n : "show", icon : "show", onclick : anp.object_name + ".comp.tables.show_menus(this, event);"}) + `
                        ` + anp.comp.button({i18n : "hide", icon : "hide", onclick : anp.object_name + ".comp.tables.hide_menus(this, event);"}) + `
                        ` + anp.comp.button({i18n : "show_options", icon : "options", onclick : anp.object_name + ".comp.tables.show_options(this, event);"}) + `
                        <fieldset` + anp.attr.set({class : "actions"}) + `>
                            ` + anp.comp.i18n("actions", "legend") + `
                            ` + anp.comp.menu({options : actions}) + `
                        </fieldset>
                        <fieldset` + anp.attr.set({class : "options"}) + `>
                            ` + anp.comp.i18n("options", "legend") + `
                            ` + anp.comp.menu({options : visible_buttons}) + `
                        </fieldset>
                        <fieldset` + anp.attr.set({class : "columns"}) + `>
                            ` + anp.comp.i18n("columns", "legend") + `
                            ` + anp.comp.menu({options : columns}) + `
                        </fieldset>
                    </div>
                    <div` + anp.attr.set({class : "group pagination"}) + `>
                        <label` + anp.attr.set({data_i18n : "page", for : hashes.page, title : i18n.page}) + `>` + i18n.page + `</label>
                        ` + anp.comp.button({i18n : "first", icon : "first", onclick : anp.object_name + ".comp.tables.first(this, event);", disabled : true}) + `
                        <span` + anp.attr.set({class : "group buttons-pre"}) + `>
                            ` + ellipsis + `
                            ` + buttons_pre + `
                        </span>
                        ` + anp.comp.button({i18n : "previous", icon : "previous", onclick : anp.object_name + ".comp.tables.previous(this, event);", disabled : true}) + `
                        ` + anp.comp.number({i18n : "page", name : "page", min : 1, step : 1, id : hashes.page, value : anp.settings_get(["tables_page", "page"], input)}) + `
                        ` + anp.comp.button({i18n : "go", icon : "go", onclick : anp.object_name + ".comp.tables.refresh(this, event);"}) + `
                        ` + anp.comp.button({i18n : "next", icon : "next", onclick : anp.object_name + ".comp.tables.next(this, event);", disabled : true}) + `
                        <span` + anp.attr.set({class : "group buttons-post"}) + `>
                            ` + buttons_post + `
                            ` + ellipsis + `
                        </span>
                        ` + anp.comp.button({i18n : "last", icon : "last", onclick : anp.object_name + ".comp.tables.last(this, event);", disabled : true}) + `
                        ` + anp.comp.number({i18n : "items_per_page", name : "items_per_page", min : 1, step : 1, value : anp.settings_get(["tables_items_per_page", "items_per_page"], input)}) + `
                        <span` + anp.attr.set({class : "page-info"}) + `>
                            ` + page_attribute("item_from") + `
                            ` + page_attribute("item_to") + `
                            ` + page_attribute("results") + `
                            ` + page_attribute("total") + `
                            ` + page_attribute("pages") + `
                        </span>
                    </div>
                </fieldset>
                <div` + anp.attr.set({data_preloader : hashes.preload}) + `></div>
            </form>
        `);
    };

    const set_globals = (action, row) => {

        action = Object.assign({}, action);

        ["go", "onclick"].forEach(method => action[method] && (action[method] = anp.set_globals(action[method], row)));

        return action;
    };

    const get_fill_commons = table => {

        const hash = table.getAttribute("data-hash"),
              search = [table.querySelector("[name=search]").value || null],
              search_fields = table.querySelectorAll("tfoot>.searcher>th"),
              search_fields_l = search_fields.length - 1,
              with_hides = !!table.querySelector("button[data-i18n=with_hides][data-toggle=true]"), 
              options = {
                  case_sensitive : !!table.querySelector("button[data-i18n=case_sensitive][data-toggle=true]"),
                //   with_hides : with_hides
              },
              fields_names = [],
              order_by = [];

        cache[hash] && 
        cache[hash].order.forEach(([key, direction]) => {
            table.querySelector("tfoot>.searcher>th[data-field=" + key + "]" + (with_hides ? "" : "[data-visible=true]")) &&
            order_by.push([key, direction]);
        });

        for(let i = 1; i < search_fields_l; i ++){
            if(!with_hides && search_fields[i].getAttribute("data-visible") != "true")
                continue;

            const type = search_fields[i].getAttribute("data-type");
            let value = null,
                has = false,
                field;

            switch(type){
                case "string":
                case "text":
                case "varchar":
                    has = !!(value = (field = search_fields[i].querySelector("input")) ? field.value : null);
                    fields_names.push([search_fields[i].getAttribute("data-field"), "string"]);
                break;
                case "number":
                case "integer":
                case "session":
                case "float":
                case "size":
                    value = [];
                    search_fields[i].querySelectorAll("input").forEach(input => {
                        if(input.value){
                            value.push(Number(input.value));
                            has = true;
                        }else
                            value.push(null);
                    });
                    fields_names.push([search_fields[i].getAttribute("data-field"), "number"]);
                break;
                case "datetime":
                case "calendar":
                    value = [];
                    search_fields[i].querySelectorAll("[type=text]").forEach(input => {
                        if(input.value){
                            value.push(anp.comp.calendars.get(input).getTime());
                            has = true;
                        }else
                            value.push(null);
                    });
                break;
                case "bool":
                case "boolean":
                    if(field = search_fields[i].querySelector("[data-icon]")){
                        value = field.getAttribute("data-icon");
                        (has = value != bool_check[0]) && (value = value == bool_check[1]);
                    };
                break;
            };

            has && search.push([search_fields[i].getAttribute("data-field"), value, type]);

        };

        search[0] && (search[0] = [search[0], fields_names]);

        return [hash, search, search_fields, search_fields_l, options, order_by];
    };

    const get_fills_data = table => {

        const columns = {},
              view = table.parentNode.hasAttribute("data-view") ? anp.views.get_data(table.parentNode.getAttribute("data-view")) : null,
              keys = [],
              cells = table.querySelectorAll("thead .order>th"),
              l = cells.length - 1,
              actions_pre = view && view.actions_pre ? anp.is_array(view.actions_pre) ? view.actions_pre : [view.actions_pre] : [],
              actions_post = view && view.actions_post ? anp.is_array(view.actions_post) ? view.actions_post : [view.actions_post] : [],
              direction = table.getAttribute("data-actions-direction"),
              actions_starter = `<span` + anp.attr.set({class : "group", data_direction : direction}) + `>`,
              default_date_format = (table.hasAttribute("data-date-format") ? table.getAttribute("data-date-format") : null) || anp.settings_get(["default_date_format", "date_format"]);

        table.querySelectorAll(".menus .columns button").forEach(column => {

            const name = column.getAttribute("data-field");
            let data = null;

            view && view.structure.forEach(row => {
                if(row.name == name){
                    data = row;
                    return;
                };
            });

            columns[name] = {
                type : column.getAttribute("data-type"),
                visible : column.getAttribute("data-toggle") == "true",
                data : data
            };

        });

        for(let i = 1; i < l; i ++)
            keys.push(cells[i].getAttribute("data-field"));

        return [columns, keys, cells, l, actions_pre, actions_post, direction, actions_starter, default_date_format, view];
    };

    const set_variables = this.set_variables = (variables, placeholding) => {

        variables = Object.assign({}, ...Object.entries(variables).filter(([key, value]) => ![
            "type", "hidden"
        ].includes(key)).map(([key, value]) => ({[key] : value})));

        if(placeholding){

            const i18n_key = variables.i18n || variables.name;

            if(i18n_key)
                variables.placeholder = anp.i18n_get(i18n_key) + "...";

        };

        return variables;
    };

    const set_fill = (table, store) => {

        let html = ``;
        const [columns, keys, cells, l, actions_pre, actions_post, direction, actions_starter, default_date_format, view] = get_fills_data(table);

        store && store.forEach((row, i) => {

            const actions = {
                pre : `` + actions_starter,
                post : `` + actions_starter
            };

            if(view){
                actions_pre.forEach(action => anp.is_dictionary(action) && (actions.pre += anp.comp.button(set_globals(action, row))));
                actions_post.forEach(action => anp.is_dictionary(action) && (actions.post += anp.comp.button(set_globals(action, row))));
            };

            html += (`
                <tr` + anp.attr.set({data_i : i, data_data : anp.encode_set(row)}) + `>
                    <td` + anp.attr.set({data_field : "actions_pre"}) + `>` + actions.pre + `</span></td>
            `);
            keys.forEach(key => {

                let value = row[key];
                const data = {...columns[key]};

                if(data.data.execute){
                    try{
                        anp.is_function(value = eval(anp.set_globals(data.data.execute, [row, data.data, {value : value}]))) && (value = value(row, key, data));
                    }catch(exception){console.error(exception);};
                }else if(value === null)
                    value = anp.comp.null();
                else{

                    let done = false;

                    if(data.data.editable)
                        switch(data.type){
                            case "bool":
                            case "boolean":
                                value = anp.comp.checkbox({
                                    i18n : key,
                                    checked : !!Number(value),
                                    onchange : data.data.onchange || null
                                });
                                done = true;
                            break;
                            case "string":
                            case "varchar":
                            case "text":

                                const text_variables = {
                                    ...set_variables(data.data, true), 
                                    value : value
                                };

                                if(text_variables.onchange){
                                    text_variables.on_change = text_variables.onchange;
                                    delete text_variables.onchange;
                                };

                                value = anp.comp.text(text_variables);
                                done = true;

                            break;
                            case "integer":
                            case "float":
                            case "double":
                            case "decimal":
                                value = anp.comp.number({
                                    ...set_variables(data.data, true), 
                                    value : value
                                });
                                done = true;
                            break;
                        };

                    if(!done)
                        switch(data.type){
                            case "bool":
                            case "boolean":
                                value = anp.comp.icon(!!Number(value) ? "checked" : "unchecked");
                            break;
                            case "i18n":
                                value = anp.comp.i18n(anp.set_globals(value !== undefined ? value : data.data.key || data.data.text, row));
                            break;
                            case "icon":
                                value = anp.comp.icon(anp.set_globals(value || data.data.icon, row));
                            break;
                            case "full_data":
                                value = anp.comp.full_item(anp.set_globals(row[data.data.field] || value, row), "span");
                            break;
                            case "html":
                                value = string_variables(anp.set_globals(anp.is_array(value) ? value.join("") : value, row), [row[key], data.data, {value : value}]);
                            break;
                            case "execute":
                            case "js":
                            case "javascript":
                            case "script":
                                try{value = eval(anp.set_globals(row[key].execute, [row[key], data.data, {value : value}]))(row, key, data);}catch(exception){};
                            break;
                            case "date":
                            case "datetime":
                            case "time":
                                value = anp.date_format(value, data.data.format || default_date_format) || "-";
                            break;
                            case "color":
                                value = anp.comp.colors.create_field(value);
                            break;
                            case "map":
                            case "place":
                                value = anp.comp.place_sample(key, row);
                            break;
                            case "size":
                                value = anp.comp.files.size_build(value, data.data.mode);
                            break;
                            case "file":
                                value = anp.comp.files.table_build(key, {
                                    ...row, 
                                    mode : data.data.mode
                                });
                            break;
                            case "url":
                                value = anp.comp.url({...row[key], url : value});
                            break;
                            case "urlfile":
                                value = anp.comp.files.url_sample({...row[key], url : value});
                            break;
                            case "json":
                                value = anp.is_string(value) ? value : JSON.stringify(value);
                            break;
                            case "integer":
                            case "float":
                            case "double":
                            case "decimal":
                            case "string":
                            case "varchar":
                            case "text":
                            default:
                                value = anp.to_html(value);
                            break;
                        };

                };

                data.data_type = data.type;
                delete data.type;

                html += `<td` + anp.attr.set(remove_events({data_field : key, ...data}), "data") + cell_custom_attributes(remove_events(data.data)) + `>` + value + `</td>`;

            });
            html += (`
                    <td` + anp.attr.set({data_field : "actions_post"}) + `>` + actions.post + `</span></td>
                </tr>
            `);

        });

        table.querySelector(".table>table>tbody").innerHTML = html;

        return html;
    };

    const set_pagination = (table, page, pages, items_per_page, results, items) => {

        if(page > pages)
            table.querySelector("[name=page]").value = (page = pages) || 1;
        else if(page < 1)
            table.querySelector("[name=page]").value = page = 1;

        const from = page ? ((page - 1) * items_per_page) + 1 : 0,
              table_pages = Number(table.getAttribute("data-table-pages"));

        with(table.querySelector(".pagination>.page-info>.item-from"))setAttribute("title", innerText = from);
        with(table.querySelector(".pagination>.page-info>.item-to"))setAttribute("title", innerText = from ? from + results - 1 : 0);
        with(table.querySelector(".pagination>.page-info>.results"))setAttribute("title", innerText = results);
        with(table.querySelector(".pagination>.page-info>.total"))setAttribute("title", innerText = items);
        with(table.querySelector(".pagination>.page-info>.pages"))setAttribute("title", innerText = pages);

        table.querySelector(".pagination>.buttons-pre .ellipsis").setAttribute("data-visible", page > table_pages + 3);
        table.querySelector(".pagination>.buttons-post .ellipsis").setAttribute("data-visible", page < pages - table_pages - 3);

        table.querySelectorAll(".pagination>[data-i18n=first],.pagination>[data-i18n=previous]").forEach(button => {
            if(page <= 1){
                !button.hasAttribute("disabled") && button.setAttribute("disabled", true);
            }else
                button.hasAttribute("disabled") && button.removeAttribute("disabled");
        });

        table.querySelectorAll(".pagination>[data-i18n=last],.pagination>[data-i18n=next]").forEach(button => {
            if(page >= pages){
                !button.hasAttribute("disabled") && button.setAttribute("disabled", true);
            }else
                button.hasAttribute("disabled") && button.removeAttribute("disabled");
        });

        table.querySelectorAll(".pagination>.buttons-pre>button").forEach(button => {

            const i = Number(button.getAttribute("data-i")),
                  page_i = page - i - 1,
                  visible = page_i > 1;

            button.setAttribute("data-visible", visible);

            if(visible){
                button.setAttribute("title", page_i);
                button.innerText = page_i;
            };

        });

        table.querySelectorAll(".pagination>.buttons-post>button").forEach(button => {

            const i = Number(button.getAttribute("data-i")),
                  page_i = page + i + 1,
                  visible = page_i < pages;

            button.setAttribute("data-visible", visible);

            if(visible){
                button.setAttribute("title", page_i);
                button.innerText = page_i;
            };

        });

    };

    const set_texts_variables = (table, data) => {
        if(!data)
            return;

        const data_encoded = anp.encode_set(data);

        [
            table.querySelector("legend"), 
            table.querySelector("legend+p")
        ].forEach(item => {
            item.setAttribute("data-i18n-variables", data_encoded);
            item.innerHTML = anp.i18n_get(item.getAttribute("data-i18n"), data);
        });

    };

    this.fill = (item, url, variables, callback, error_message, errors) => {

        const table = get(item);

        if(!table)
            return;

        const [hash, search, search_fields, search_fields_l, search_options, order_by] = get_fill_commons(table),
              items_per_page = Number(table.querySelector(".pagination [name=items_per_page]").value);
        let page = Number(table.querySelector(".pagination [name=page]").value);

        // console.log([search, cache[hash] ? cache[hash].order : []]);

        anp.ajax.send("post", url, {
            session : anp.sessions ? anp.sessions.get() : null,
            page : page,
            items_per_page : items_per_page,
            where : search,
            search_options : search_options, 
            order_by : order_by,
            ...variables
        }, (response, ok) => {
            // console.log(response);
            response.variables && (response = {
                ...response.variables,
                results : response.tables[0] || [], 
                ...(response.entity_data ? {entity_data : response.entity_data} : {}) 
            });
            // console.log(response);
            if(ok && anp.errors.validate(
                table, response.error, 
                errors || ["sql_exception"], 
                {
                    ...variables, 
                    ...(response.entity_data || {})
                }, 
                error_message || "table_error_message"
            )){
                set_fill(table, response.results);
                set_pagination(table, page, Number(response.pages), items_per_page, response.results.length, Number(response.items));
                set_texts_variables(table, response.entity_data);
                anp.is_function(callback) && setTimeout(() => callback(item, response), 50);
            }else 
                anp.is_function(callback) && callback(item, response);
        });

    };

    this.first = (item, event) => {

        const table = get(item);

        if(table){
            table.querySelector("[name=page]").value = 1;
            self.refresh(item, event);
        };

    };

    this.previous = (item, event) => {

        const table = get(item);

        if(table){
            with(table.querySelector("[name=page]"))value = Number(value) - 1;
            self.refresh(item, event);
        };

    };

    this.next = (item, event) => {

        const table = get(item);

        if(table){
            with(table.querySelector("[name=page]"))value = Number(value) + 1;
            self.refresh(item, event);
        };

    };

    this.last = (item, event) => {

        const table = get(item);

        table.querySelector("[name=page]").value = Number(table.querySelector(".page-info>.pages").innerText);
        self.refresh(item, event);

    };

    this.go = (item, event) => {

        const table = get(item);

        if(table){
            table.querySelector("[name=page]").value = Number(item.innerText);
            self.refresh(item, event);
        };

    };

    this.get_row = row => {

        if(row)
            while(row.tagName && row.tagName.toLowerCase() != "tr" && (row = row.parentNode));

        return row || null;
    };

    this.get_id = item => Number(self.get_row(item).querySelector("[data-field=id]").innerText);

    this.get_field = (item, key) => self.get_row(item).querySelector("[data-field=" + key + "]");

    this.reset_columns = (table, fields) => {

        if(anp.is_string(table))
            table = anp.item_self.querySelector(table);
        if(!anp.is_html_object(table))
            return;

        const [headers, columns] = set_columns(fields);

        (table = get(table)).querySelectorAll(".menus .columns nav>button").forEach(column => {
            table.querySelectorAll("th[data-field=" + column.getAttribute("data-field") + "]").forEach(cell => cell.remove());
            column.remove();
        });

        table.querySelector(".columns>nav").innerHTML = anp.comp.menu({options : columns}).replace(/<nav([^>]+)?>|<\/nav>/g, "");
        table.querySelectorAll("tr.order").forEach(row => row.innerHTML = headers.order);
        table.querySelectorAll("tr.searcher").forEach(row => row.innerHTML = headers.searcher);

        table.parentNode.hasAttribute("data-view") && (anp.views.get_data(table.parentNode.getAttribute("data-view")).structure = fields);

        table.querySelector("tbody").innerHTML = ``;

        // self.refresh(table);

    };

    this.fill_store = (table, store, callback) => {

        table = typeof table == "string" ? anp.item_self.querySelector(table) : get(table);

        if(!table || !store)
            return;

        const [hash, search, search_fields, search_fields_l, search_options, order_by] = get_fill_commons(table),
              items_per_page = Number(table.querySelector(".pagination [name=items_per_page]").value),
              page_field = table.querySelector(".pagination [name=page]");
        let page = Number(page_field.value);

        store.set_parameters(search[0] || "", search.slice(1), order_by);

        const pages = store.get_results_pages();

        page > pages && (page = pages);
        pages < 1 && (page = 1);

        const response = store.get(page - 1, items_per_page);

        store.set_items_per_page(items_per_page);

        set_fill(table, response);
        set_pagination(table, page, pages, items_per_page, response.length, store.get_results_count());
        page_field.value = page;

        typeof callback == "function" && callback();

    };

    this.draw_columns = (table, new_columns) => {

        const [header, buttons] = set_columns(new_columns);

        (table = get(table)).querySelectorAll("thead,tfoot").forEach(block => {
            block.querySelector(".order").innerHTML = header.order;
            block.querySelector(".searcher").innerHTML = header.searcher;
        });
        table.querySelector("fieldset.columns .menu").innerHTML = anp.comp.menu({buttons : buttons}).replace(/<nav [^>]+>|<\/nav>/g, "");

    };

    this.get_row_values = row => {

        const values = {};

        row = self.get_row(row);

        row.querySelectorAll("td[data-type").forEach(field => {

            const name = field.getAttribute("data-field");

            if(field.innerText == "-")
                values[name] = null;
            else 
                switch(field.getAttribute("data-type")){
                    case "integer":
                    case "float":
                    case "number":
                    case "session":
                        values[name] = Number(field.innerText);
                    break;
                    default:
                        values[name] = field.innerText;
                    break;
                };

        });

        return [values, row];
    };

    this.get_row_field = (item, key) => {

        const field = self.get_row(item).querySelector("[data-field=" + key + "]");
        let value = null;

        switch(field.getAttribute("data-type")){
            case "integer":
            case "int":
            case "bigint":
            case "tinyint":
            case "smallint":
            case "mediumint":
            case "number":
            case "float":
            case "decimal":
            case "double":
                isNaN(value = Number(field.innerText)) && (value = null);
            break;
            default:
                value = field.innerText;
            break;
        };

        return value;
    };

    this.case_sensitive = (item, event) => {};

    this.with_hides = (item, event) => {};

    this.reset = (table, event) => {
        
        table.querySelectorAll("tfoot>.order [data-icon]").forEach(icon => icon.setAttribute("data-icon", "order_no"));
        setTimeout(() => self.refresh(table, event), 100);
        
    };

    this.get_row_data = (item, event) => {

        const row = self.get_row(item),
              data = {};

        // row.querySelectorAll("[data-field][data-data]").forEach(cell => data[cell.getAttribute("data-field")] = key.getAttribute("data-data"));

        return anp.decode_set(self.get_row(item).getAttribute("data-data"));
    };

    construct();

};
