AnP.FilesHashes = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/files/hashes/list", 
        {}, 
        null, // callback
        "files_hashes_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => anp.confirm("file_hash_delete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/files/hashes/delete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "file_hash_delete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "file_hash_null",
                    "file_hash_not_id",
                    "file_hash_not_exists",
                    "file_hash_deleted"
                ]
            ), "file_hash_delete_ok") && 
            self.refresh(item, event);
        });
    });

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => anp.ajax.send("post", "/app/sessions/files/hashes/basics", {}, data => {
        if(anp.errors.validate(data.variables.error, [].concat(
            anp.sessions.get_errors_list()
        ), "files_hashes_basics_error")){

            const id = Number(anp.get_globals("file_hash")); 

            anp.comp.selectors.set("[data-view=files_hashes_edit] [name=file]", anp.format_selector_values(data.tables[0]));
            anp.comp.selectors.set("[data-view=files_hashes_edit] [name=algorithm]", anp.format_selector_values(data.tables[1]));

            if(isNaN(id)){
                cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                return () => set_wait_values(item);
            }else 
                anp.ajax.send("post", "/app/sessions/files/hashes/load", {id : id}, data => {
                    if(anp.errors.validate(data.variables.error, [].concat(
                        anp.sessions.get_errors_list(),
                        [
                            "file_hash_null",
                            "file_hash_not_id",
                            "file_hash_not_exists",
                            "file_hash_deleted"
                        ]
                    ), "file_hash_load_error")){
                        anp.comp.forms.set(item, data.data);
                        cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                        return () => set_wait_values(item);
                    };
                });

        };
    });

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.file === undefined ? 1 << 0 : 
            data.file === null ? 1 << 1 : 
            !anp.is_number(data.file) ? 1 << 2 : 
            !anp.is_integer(data.file) ? 1 << 3 : 
            data.file < 1 ? 1 << 4 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.algorithm === undefined ? 1 << 0 : 
            data.algorithm === null ? 1 << 1 : 
            !anp.is_number(data.algorithm) ? 1 << 2 : 
            !anp.is_integer(data.algorithm) ? 1 << 3 : 
            data.algorithm < 1 ? 1 << 4 : 
        0), 10, 0);

        error = anp.errors.set(error, (
            data.hash === undefined ? 1 << 0 : 
            data.hash === null ? 1 << 1 : 
            !anp.is_string(data.hash) ? 1 << 2 : 
            data.hash.length > 256 ? 1 << 5 : 
        0), 15, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const id = Number(anp.get_globals("file_hash")),
              key = id ? "edit" : "add";

        id && (data.id = id);  

        if(!anp.errors.validate(form, self.validate(data), "file_hash_" + key + "_local_error", [
            "file_hash_undefined", 
            "file_hash_null", 
            "file_hash_not_number", 
            "file_hash_not_integer",
            "file_hash_not_id" ,
            "file_undefined",
            "file_null",
            "file_not_number",
            "file_not_integer",
            "file_not_id",
            "algorithm_undefined",
            "algorithm_null",
            "algorithm_not_number",
            "algorithm_not_integer",
            "algorithm_not_id",
            "hash_undefined",
            "hash_null",
            "hash_not_string",
            "hash_empty",
            "hash_too_few_length",
            "hash_too_length",
            "hash_not_pattern" 
        ]))
            return;

        anp.confirm("file_hash_confirm_changes", () => {
            anp.ajax.send("post", "/app/sessions/files/hashes/" + key, data, (response, ok) => {
                if(anp.errors.validate(response.variables.error, "file_hash_" + key + "_error", [].concat(
                    anp.sessions.get_errors_list(), 
                    id ? [
                        "file_hash_null",
                        "file_hash_not_id",
                        "file_hash_not_exists",
                        "file_hash_deleted"
                    ] : [],
                    [,
                        "file_null",
                        "file_not_id",
                        "file_not_exist",
                        "file_deleted",
                        "algorithm_null",
                        "algorithm_not_id",
                        "algorithm_not_exist",
                        "algorithm_deleted",
                        "hash_null",
                        "hash_empty",
                        "hash_too_few_length",
                        "hash_not_pattern"
                    ]
                ), "file_hash_" + key + "_ok")){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("file_hash_exit_edit", anp.back_edit);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("file_hash_has_changes", anp.back_edit, null);
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/files/hashes/deleted", 
        {}, 
        null, // callback
        "files_hashes_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => anp.confirm("file_hash_undelete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/files/hashes/undelete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "file_hash_undelete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "file_hash_null",
                    "file_hash_not_id",
                    "file_hash_not_exists",
                    "file_hash_not_deleted"
                ]
            ), "file_hash_undelete_ok") && 
            self.refresh_deleted(item, event);
        });
    });

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/files/hashes/history", 
        {id : Number(anp.get_globals("file_hash"))}, 
        null, // callback
        "{error_message}", 
        anp.sessions.get_errors_list().concat([
            anp.sessions.get_errors_list(), [
                "file_hash_null",
                "file_hash_not_id",
                "file_hash_not_exists",
                "file_hash_deleted"
            ]
        ])
    );

    this.restore = (item, event) => anp.confirm("file_hash_restore_confirm", () => {
        anp.ajax.send("post", "/app/sessions/files/hashes/restore", {
            id : anp.comp.tables.get_row_field(item, "plain_id")
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "file_hash_restore_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "file_hash_null",
                    "file_hash_not_id",
                    "file_hash_not_exists",
                    "file_hash_not_deleted"
                ]
            ), "file_hash_restore_ok") && 
            self.refresh_history(item, event);
        });
    });

    construct();

};