AnP.Groups = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        anp.anp_route() + "/administration/groups/list", 
        {}, 
        null, // callback
        "groups_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => {
        
        const variables = anp.comp.tables.get_row_data(item);

        anp.confirm("group_delete_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/groups/delete", {
                id : Number(variables.id)
            }, (response, ok) => {
                anp.errors.validate(
                    response.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(), [
                            "group_null",
                            "group_not_id",
                            "group_not_exists",
                            "group_deleted"
                        ]
                    ), 
                    variables, 
                    "group_delete_error", 
                    "group_delete_ok"
                ) && 
                self.refresh(item, event);
            });
        }, null, variables);

    };

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => {
            
        const id = Number(anp.get_globals("group"));

        if(isNaN(id)){
            cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
            anp.comp.forms.set_texts(item, "groups_add");
            return () => set_wait_values(item);
        }else{

            const texts_button = item.querySelector("button[data-i18n=i18n_texts]");

            texts_button && texts_button.removeAttribute("disabled");
            anp.ajax.send("post", anp.anp_route() + "/administration/groups/load", {id : id}, data => {
                if(anp.errors.validate(
                    data.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(),
                        [
                            "group_null",
                            "group_not_id",
                            "group_not_exists",
                            "group_deleted"
                        ]
                    ), 
                    {
                        ...data.data, 
                        id : id
                    }, 
                    "group_load_error"
                )){
                    anp.comp.forms.set_texts(item, "groups_edit");
                    anp.comp.forms.set(item, data.data);
                    cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                    return () => set_wait_values(item);
                };
            });

        };

    };

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.name === undefined ? 1 << 0 : 
            data.name === null ? 1 << 1 : 
            !anp.is_string(data.name) ? 1 << 2 : 
            data.name.length > 64 ? 1 << 5 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.default === undefined ? 1 << 0 : 
            data.default === null ? 1 << 1 : 
        0), 12, 0);

        error = anp.errors.set(error, (
            data.is_entity === undefined ? 1 << 0 : 
            data.is_entity === null ? 1 << 1 : 
        0), 14, 0);

        error = anp.errors.set(error, (
            data.description === undefined ? 1 << 0 : 
            data.description === null ? 0 : 
            !anp.is_string(data.description) ? 1 << 2 : 
            data.description.length > 512 ? 1 << 5 : 
        0), 16, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const key = id ? "edit" : "add", 
              id = Number(variables.id),
              variables = id ? anp.comp.forms.get_variables(item) : data;

        if(!anp.errors.validate(
            form, 
            (
                JSON.stringify(data) == cache ? 1 << 0 : 
            0), 
            [
                "form_without_changes"
            ], 
            variables, 
            "group_" + key + "_local_error"
        ))
            return;

        id && (data.id = id);  

        if(!anp.errors.validate(
            form, 
            self.validate(data), 
            [
                "group_undefined", 
                "group_null", 
                "group_not_number", 
                "group_not_integer",
                "group_not_id" ,
                "name_undefined",
                "name_null",
                "name_not_string",
                "name_empty",
                "name_too_few_length",
                "name_too_length",
                "name_not_pattern",
                "default_undefined",
                "default_null",
                "is_entity_undefined",
                "is_entity_null",
                "description_undefined",
                "description_null",
                "description_not_string",
                "description_empty",
                "description_too_few_length",
                "description_too_length",
                "description_not_pattern" 
            ], 
            variables, 
            "group_" + key + "_local_error"
        ))
            return;

        anp.confirm("group_confirm_changes", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/groups/" + key, data, (response, ok) => {
                if(anp.errors.validate(
                    response.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(), 
                        id ? [
                            "group_null",
                            "group_not_id",
                            "group_not_exists",
                            "group_deleted"
                        ] : [],
                        [
                            "name_null",
                            "name_empty",
                            "name_too_few_length",
                            "name_not_pattern",
                            "default_null",
                            "is_entity_null",
                            "description_null",
                            "description_empty",
                            "description_too_few_length",
                            "description_not_pattern"
                        ]
                    ), 
                    variables, 
                    "group_" + key + "_error", 
                    "group_" + key + "_ok"
                )){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("group_exit_edit", anp.back_edit, null, variables);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("group_has_changes", anp.back_edit, null, anp.comp.forms.get_variables(item));
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        anp.anp_route() + "/administration/groups/deleted", 
        {}, 
        null, // callback
        "groups_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => {

        const variables = anp.comp.tables.get_row_data(item);
        
        anp.confirm("group_undelete_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/groups/undelete", {
                id : anp.comp.tables.get_id(item)
            }, (response, ok) => {
                anp.errors.validate(
                    response.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(), [
                            "group_null",
                            "group_not_id",
                            "group_not_exists",
                            "group_not_deleted"
                        ]
                    ), 
                    variables, 
                    "group_undelete_error", 
                    "group_undelete_ok"
                ) && 
                self.refresh_deleted(item, event);
            });
        }, null, variables);

    };

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        anp.anp_route() + "/administration/groups/history", 
        {id : Number(anp.get_globals("group"))}, 
        null, // callback
        "groups_history_error", 
        [].concat([
            anp.sessions.get_errors_list(), [
                "group_null",
                "group_not_id",
                "group_not_exists",
                "group_deleted"
            ]
        ])
    );

    this.restore = (item, event) => {
        
        const variables = {
            ...anp.comp.forms.get_variables(item), 
            ...anp.comp.tables.get_row_data(item)
        };
        
        anp.confirm("group_restore_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/groups/restore", {
                id : anp.comp.tables.get_row_field(item, "plain_id")
            }, (response, ok) => {
                anp.errors.validate(
                    response.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(), [
                            "group_null",
                            "group_not_id",
                            "group_not_exists",
                            "group_date_out"
                        ]
                    ), 
                    variables, 
                    "group_restore_error", 
                    "group_restore_ok"
                ) && 
                self.refresh_history(item, event);
            });
        }, null, variables);

    };

    construct();

};