AnP.Languages = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/languages/list", 
        {}, 
        null, // callback
        "languages_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => anp.confirm("language_delete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/languages/delete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "language_delete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "language_null",
                    "language_not_id",
                    "language_not_exists",
                    "language_deleted"
                ]
            ), "language_delete_ok") && 
            self.refresh(item, event);
        });
    });

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => anp.ajax.send("post", "/app/sessions/languages/basics", {}, data => {
        if(anp.errors.validate(data.variables.error, [].concat(
            anp.sessions.get_errors_list()
        ), "languages_basics_error")){

            const id = Number(anp.get_globals("language")); 

            anp.comp.selectors.set("[data-view=languages_edit] [name=country]", anp.format_selector_values(data.tables[0]));

            if(isNaN(id)){
                cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                return () => set_wait_values(item);
            }else 
                anp.ajax.send("post", "/app/sessions/languages/load", {id : id}, data => {
                    if(anp.errors.validate(data.variables.error, [].concat(
                        anp.sessions.get_errors_list(),
                        [
                            "language_null",
                            "language_not_id",
                            "language_not_exists",
                            "language_deleted"
                        ]
                    ), "language_load_error")){
                        anp.comp.forms.set(item, data.data);
                        cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                        return () => set_wait_values(item);
                    };
                });

        };
    });

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.country === undefined ? 1 << 0 : 
            data.country === null ? 1 << 1 : 
            !anp.is_number(data.country) ? 1 << 2 : 
            !anp.is_integer(data.country) ? 1 << 3 : 
            data.country < 1 ? 1 << 4 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.name === undefined ? 1 << 0 : 
            data.name === null ? 1 << 1 : 
            !anp.is_string(data.name) ? 1 << 2 : 
            data.name.length > 64 ? 1 << 5 : 
        0), 10, 0);

        error = anp.errors.set(error, (
            data.key === undefined ? 1 << 0 : 
            data.key === null ? 1 << 1 : 
            !anp.is_string(data.key) ? 1 << 2 : 
            data.key.length > 64 ? 1 << 5 : 
            !/[object Object]/.test(data.key) ? 1 << 6 : 
        0), 17, 0);

        error = anp.errors.set(error, (
            data.code === undefined ? 1 << 0 : 
            data.code === null ? 1 << 1 : 
            !anp.is_string(data.code) ? 1 << 2 : 
            data.code.length > 4 ? 1 << 5 : 
            !/[object Object]/.test(data.code) ? 1 << 6 : 
        0), 24, 0);

        error = anp.errors.set(error, (
            data.flag_id === undefined ? ((
                ((
                    data.flag_name === undefined ? 0 : 
                    data.flag_name === null ? 1 << 1 : 
                    !anp.is_string(data.flag_name) ? 1 << 2 : 
                    !data.flag_name ? 1 << 3 : 
                0) << 4) | 
                ((
                    data.flag_mime === undefined ? 0 : 
                    data.flag_mime === null ? 1 << 1 : 
                    !anp.is_string(data.flag_mime) ? 1 << 2 : 
                    !data.flag_mime ? 1 << 3 : 
                0) << 8) | 
                ((
                    data.flag_size === undefined ? 0 : 
                    data.flag_size === null ? 1 << 1 : 
                    !anp.is_number(data.flag_size) ? 1 << 2 : 
                    !anp.is_integer(data.flag_size) ? 1 << 3 : 
                    data.flag_size < 0 ? 1 << 4 : 
                    !data.flag_size ? 1 << 5 : 
                0) << 12) | 
            0) << 5) : 
            data.flag_id === null ? 1 << 1 : 
            !anp.is_number(data.flag_id) ? 1 << 2 : 
            !anp.is_integer(data.flag_id) ? 1 << 3 : 
            data.flag_id < 1 ? 1 << 4 : 
        0), 31, 0);

        error = anp.errors.set(error, (
            data.description === undefined ? 1 << 0 : 
            data.description === null ? 0 : 
            !anp.is_string(data.description) ? 1 << 2 : 
            data.description.length > 512 ? 1 << 5 : 
        0), 49, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const id = Number(anp.get_globals("language")),
              key = id ? "edit" : "add";

        id && (data.id = id);  

        ["flag"].forEach(subkey => {
            if(data[subkey + "_id"] === null)
                delete data[subkey + "_id"];
            else if(delete data[subkey + "_name"] !== undefined) 
                ["name", "mime", "size"].forEach(attribute => {
                    delete data[subkey + "_" + attribute];
                });
        });

        if(!anp.errors.validate(form, self.validate(data), "language_" + key + "_local_error", [
            "language_undefined", 
            "language_null", 
            "language_not_number", 
            "language_not_integer",
            "language_not_id" ,
            "country_undefined",
            "country_null",
            "country_not_number",
            "country_not_integer",
            "country_not_id",
            "name_undefined",
            "name_null",
            "name_not_string",
            "name_empty",
            "name_too_few_length",
            "name_too_length",
            "name_not_pattern",
            "key_undefined",
            "key_null",
            "key_not_string",
            "key_empty",
            "key_too_few_length",
            "key_too_length",
            "key_not_pattern",
            "code_undefined",
            "code_null",
            "code_not_string",
            "code_empty",
            "code_too_few_length",
            "code_too_length",
            "code_not_pattern",
            "flag_id_undefined",
            "flag_id_null",
            "flag_id_not_number",
            "flag_id_not_integer",
            "flag_id_not_id",
            "flag_name_undefined",
            "flag_name_null",
            "flag_name_not_string",
            "flag_name_empty",
            "flag_mime_undefined",
            "flag_mime_null",
            "flag_mime_not_string",
            "flag_mime_empty",
            "flag_size_undefined",
            "flag_size_null",
            "flag_size_not_number",
            "flag_size_not_integer",
            "flag_size_negative",
            "flag_size_empty",
            "description_undefined",
            "description_null",
            "description_not_string",
            "description_empty",
            "description_too_few_length",
            "description_too_length",
            "description_not_pattern" 
        ]))
            return;

        anp.confirm("language_confirm_changes", () => {
            anp.ajax.send("post", "/app/sessions/languages/" + key, data, (response, ok) => {
                if(anp.errors.validate(response.variables.error, "language_" + key + "_error", [].concat(
                    anp.sessions.get_errors_list(), 
                    id ? [
                        "language_null",
                        "language_not_id",
                        "language_not_exists",
                        "language_deleted"
                    ] : [],
                    [,
                        "country_null",
                        "country_not_id",
                        "country_not_exist",
                        "country_deleted",
                        "name_null",
                        "name_empty",
                        "name_too_few_length",
                        "name_not_pattern",
                        "key_null",
                        "key_empty",
                        "key_too_few_length",
                        "key_not_pattern",
                        "code_null",
                        "code_empty",
                        "code_too_few_length",
                        "code_not_pattern",
                        "flag_id_null",
                        "flag_id_not_id",
                        "flag_not_exists",
                        "flag_deleted",
                        "flag_name_null",
                        "flag_name_empty",
                        "flag_mime_null",
                        "flag_mime_empty",
                        "flag_size_null",
                        "flag_size_negative",
                        "flag_size_empty",
                        "description_null",
                        "description_empty",
                        "description_too_few_length",
                        "description_not_pattern"
                    ]
                ), "language_" + key + "_ok")){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("language_exit_edit", anp.back_edit);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("language_has_changes", anp.back_edit, null);
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/languages/deleted", 
        {}, 
        null, // callback
        "languages_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => anp.confirm("language_undelete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/languages/undelete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "language_undelete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "language_null",
                    "language_not_id",
                    "language_not_exists",
                    "language_not_deleted"
                ]
            ), "language_undelete_ok") && 
            self.refresh_deleted(item, event);
        });
    });

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/languages/history", 
        {id : Number(anp.get_globals("language"))}, 
        null, // callback
        "{error_message}", 
        anp.sessions.get_errors_list().concat([
            anp.sessions.get_errors_list(), [
                "language_null",
                "language_not_id",
                "language_not_exists",
                "language_deleted"
            ]
        ])
    );

    this.restore = (item, event) => anp.confirm("language_restore_confirm", () => {
        anp.ajax.send("post", "/app/sessions/languages/restore", {
            id : anp.comp.tables.get_row_field(item, "plain_id")
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "language_restore_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "language_null",
                    "language_not_id",
                    "language_not_exists",
                    "language_not_deleted"
                ]
            ), "language_restore_ok") && 
            self.refresh_history(item, event);
        });
    });

    construct();

};