AnP.Lines = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/lines/list", 
        {}, 
        null, // callback
        "lines_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => {
            
        const id = Number(anp.get_globals("line"));

        if(isNaN(id)){
            cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
            return () => set_wait_values(item);
        }else{

            const texts_button = item.querySelector("button[data-i18n=i18n_texts]");

            texts_button && texts_button.removeAttribute("disabled");
            anp.ajax.send("post", "/app/sessions/lines/load", {id : id}, data => {
                if(anp.errors.validate(data.variables.error, "line_load_error", [].concat(
                    anp.sessions.get_errors_list(),
                    [
                        "line_null",
                        "line_not_id",
                        "line_not_exists",
                        "line_deleted"
                    ]
                ))){
                    anp.comp.forms.set(item, data.data);
                    cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                    return () => set_wait_values(item);
                };
            });

        };

    };

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.trace === undefined ? 1 << 0 : 
            data.trace === null ? 1 << 1 : 
            !anp.is_string(data.trace) ? 1 << 2 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.description === undefined ? 1 << 0 : 
            data.description === null ? 0 : 
            !anp.is_string(data.description) ? 1 << 2 : 
            data.description.length > 512 ? 1 << 5 : 
        0), 12, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const id = Number(anp.get_globals("line")),
              key = id ? "edit" : "add";

        id && (data.id = id);  

        if(!anp.errors.validate(form, self.validate(data), "line_" + key + "_local_error", [
            "line_undefined", 
            "line_null", 
            "line_not_number", 
            "line_not_integer",
            "line_not_id" ,
            "trace_undefined",
            "trace_null",
            "trace_not_string",
            "trace_empty",
            "trace_too_few_length",
            "trace_too_length",
            "trace_not_pattern",
            "description_undefined",
            "description_null",
            "description_not_string",
            "description_empty",
            "description_too_few_length",
            "description_too_length",
            "description_not_pattern" 
        ]))
            return;

        anp.confirm("line_confirm_changes", () => {
            anp.ajax.send("post", "/app/sessions/lines/" + key, data, (response, ok) => {
                if(anp.errors.validate(response.variables.error, "line_" + key + "_error", [].concat(
                    anp.sessions.get_errors_list(), 
                    id ? [
                        "line_null",
                        "line_not_id",
                        "line_not_exists",
                        "line_deleted"
                    ] : [],
                    [,
                        "trace_null",
                        "trace_empty",
                        "trace_too_few_length",
                        "trace_not_pattern",
                        "description_null",
                        "description_empty",
                        "description_too_few_length",
                        "description_not_pattern"
                    ]
                ), "line_" + key + "_ok")){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("line_exit_edit", anp.back_edit);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("line_has_changes", anp.back_edit, null);
        else 
            anp.back_edit();
    };

    construct();

};