AnP.Settings = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/list", 
        {}, 
        null, // callback
        "settings_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => anp.confirm("settings_delete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/delete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_delete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_null",
                    "settings_not_id",
                    "settings_not_exists",
                    "settings_deleted"
                ]
            ), "settings_delete_ok") && 
            self.refresh(item, event);
        });
    });

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => anp.ajax.send("post", "/app/sessions/settings/basics", {}, data => {
        if(anp.errors.validate(data.variables.error, [].concat(
            anp.sessions.get_errors_list()
        ), "settings_basics_error")){

            const id = Number(anp.get_globals("settings")); 

            anp.comp.selectors.set("[data-view=settings_edit] [name=type]", anp.format_selector_values(data.tables[0]));

            if(isNaN(id)){
                cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                return () => set_wait_values(item);
            }else 
                anp.ajax.send("post", "/app/sessions/settings/load", {id : id}, data => {
                    if(anp.errors.validate(data.variables.error, [].concat(
                        anp.sessions.get_errors_list(),
                        [
                            "settings_null",
                            "settings_not_id",
                            "settings_not_exists",
                            "settings_deleted"
                        ]
                    ), "settings_load_error")){
                        anp.comp.forms.set(item, data.data);
                        cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                        return () => set_wait_values(item);
                    };
                });

        };
    });

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.type === undefined ? 1 << 0 : 
            data.type === null ? 1 << 1 : 
            !anp.is_number(data.type) ? 1 << 2 : 
            !anp.is_integer(data.type) ? 1 << 3 : 
            data.type < 1 ? 1 << 4 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.name === undefined ? 1 << 0 : 
            data.name === null ? 1 << 1 : 
            !anp.is_string(data.name) ? 1 << 2 : 
            data.name.length > 64 ? 1 << 5 : 
        0), 10, 0);

        error = anp.errors.set(error, (
            data.range === undefined ? 1 << 0 : 
            data.range === null ? 1 << 1 : 
            !anp.is_number(data.range) ? 1 << 2 : 
            !anp.is_integer(data.range) ? 1 << 3 : 
        0), 17, 0);

        error = anp.errors.set(error, (
            data.value === undefined ? 1 << 0 : 
            data.value === null ? 0 : 
            !anp.is_string(data.value) ? 1 << 2 : 
        0), 23, 0);

        error = anp.errors.set(error, (
            data.description === undefined ? 1 << 0 : 
            data.description === null ? 0 : 
            !anp.is_string(data.description) ? 1 << 2 : 
        0), 30, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const id = Number(anp.get_globals("settings")),
              key = id ? "edit" : "add";

        id && (data.id = id);  

        if(!anp.errors.validate(form, self.validate(data), "settings_" + key + "_local_error", [
            "settings_undefined", 
            "settings_null", 
            "settings_not_number", 
            "settings_not_integer",
            "settings_not_id" ,
            "type_undefined",
            "type_null",
            "type_not_number",
            "type_not_integer",
            "type_not_id",
            "name_undefined",
            "name_null",
            "name_not_string",
            "name_empty",
            "name_too_few_length",
            "name_too_length",
            "name_not_pattern",
            "range_undefined",
            "range_null",
            "range_not_number",
            "range_not_integer",
            "range_too_low",
            "range_too_high",
            "value_undefined",
            "value_null",
            "value_not_string",
            "value_empty",
            "value_too_few_length",
            "value_too_length",
            "value_not_pattern",
            "description_undefined",
            "description_null",
            "description_not_string",
            "description_empty",
            "description_too_few_length",
            "description_too_length",
            "description_not_pattern" 
        ]))
            return;

        anp.confirm("settings_confirm_changes", () => {
            anp.ajax.send("post", "/app/sessions/settings/" + key, data, (response, ok) => {
                if(anp.errors.validate(response.variables.error, "settings_" + key + "_error", [].concat(
                    anp.sessions.get_errors_list(), 
                    id ? [
                        "settings_null",
                        "settings_not_id",
                        "settings_not_exists",
                        "settings_deleted"
                    ] : [],
                    [,
                        "type_null",
                        "type_not_id",
                        "type_not_exist",
                        "type_deleted",
                        "name_null",
                        "name_empty",
                        "name_too_few_length",
                        "name_not_pattern",
                        "range_null",
                        "range_too_low",
                        "range_too_high",
                        "value_null",
                        "value_empty",
                        "value_too_few_length",
                        "value_not_pattern",
                        "description_null",
                        "description_empty",
                        "description_too_few_length",
                        "description_not_pattern"
                    ]
                ), "settings_" + key + "_ok")){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("settings_exit_edit", anp.back_edit);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("settings_has_changes", anp.back_edit, null);
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/deleted", 
        {}, 
        null, // callback
        "settings_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => anp.confirm("settings_undelete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/undelete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_undelete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_null",
                    "settings_not_id",
                    "settings_not_exists",
                    "settings_not_deleted"
                ]
            ), "settings_undelete_ok") && 
            self.refresh_deleted(item, event);
        });
    });

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/history", 
        {id : Number(anp.get_globals("settings"))}, 
        null, // callback
        "{error_message}", 
        anp.sessions.get_errors_list().concat([
            anp.sessions.get_errors_list(), [
                "settings_null",
                "settings_not_id",
                "settings_not_exists",
                "settings_deleted"
            ]
        ])
    );

    this.restore = (item, event) => anp.confirm("settings_restore_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/restore", {
            id : anp.comp.tables.get_row_field(item, "plain_id")
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_restore_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_null",
                    "settings_not_id",
                    "settings_not_exists",
                    "settings_not_deleted"
                ]
            ), "settings_restore_ok") && 
            self.refresh_history(item, event);
        });
    });

    construct();

};