AnP.SettingsGroups = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/groups/list", 
        {}, 
        null, // callback
        "settings_groups_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => anp.confirm("settings_group_delete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/groups/delete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_group_delete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_group_null",
                    "settings_group_not_id",
                    "settings_group_not_exists",
                    "settings_group_deleted"
                ]
            ), "settings_group_delete_ok") && 
            self.refresh(item, event);
        });
    });

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => anp.ajax.send("post", "/app/sessions/settings/groups/basics", {}, data => {
        if(anp.errors.validate(data.variables.error, [].concat(
            anp.sessions.get_errors_list()
        ), "settings_groups_basics_error")){

            const id = Number(anp.get_globals("settings_group")); 

            anp.comp.selectors.set("[data-view=settings_groups_edit] [name=group]", anp.format_selector_values(data.tables[0]));
            anp.comp.selectors.set("[data-view=settings_groups_edit] [name=settings]", anp.format_selector_values(data.tables[1]));

            if(isNaN(id)){
                cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                return () => set_wait_values(item);
            }else 
                anp.ajax.send("post", "/app/sessions/settings/groups/load", {id : id}, data => {
                    if(anp.errors.validate(data.variables.error, [].concat(
                        anp.sessions.get_errors_list(),
                        [
                            "settings_group_null",
                            "settings_group_not_id",
                            "settings_group_not_exists",
                            "settings_group_deleted"
                        ]
                    ), "settings_group_load_error")){
                        anp.comp.forms.set(item, data.data);
                        cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                        return () => set_wait_values(item);
                    };
                });

        };
    });

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.group === undefined ? 1 << 0 : 
            data.group === null ? 1 << 1 : 
            !anp.is_number(data.group) ? 1 << 2 : 
            !anp.is_integer(data.group) ? 1 << 3 : 
            data.group < 1 ? 1 << 4 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.settings === undefined ? 1 << 0 : 
            data.settings === null ? 1 << 1 : 
            !anp.is_number(data.settings) ? 1 << 2 : 
            !anp.is_integer(data.settings) ? 1 << 3 : 
            data.settings < 1 ? 1 << 4 : 
        0), 10, 0);

        error = anp.errors.set(error, (
            data.value === undefined ? 1 << 0 : 
            data.value === null ? 0 : 
            !anp.is_string(data.value) ? 1 << 2 : 
        0), 15, 0);

        error = anp.errors.set(error, (
            data.belongs === undefined ? 1 << 0 : 
            data.belongs === null ? 1 << 1 : 
        0), 22, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const id = Number(anp.get_globals("settings_group")),
              key = id ? "edit" : "add";

        id && (data.id = id);  

        if(!anp.errors.validate(form, self.validate(data), "settings_group_" + key + "_local_error", [
            "settings_group_undefined", 
            "settings_group_null", 
            "settings_group_not_number", 
            "settings_group_not_integer",
            "settings_group_not_id" ,
            "group_undefined",
            "group_null",
            "group_not_number",
            "group_not_integer",
            "group_not_id",
            "settings_undefined",
            "settings_null",
            "settings_not_number",
            "settings_not_integer",
            "settings_not_id",
            "value_undefined",
            "value_null",
            "value_not_string",
            "value_empty",
            "value_too_few_length",
            "value_too_length",
            "value_not_pattern",
            "belongs_undefined",
            "belongs_null" 
        ]))
            return;

        anp.confirm("settings_group_confirm_changes", () => {
            anp.ajax.send("post", "/app/sessions/settings/groups/" + key, data, (response, ok) => {
                if(anp.errors.validate(response.variables.error, "settings_group_" + key + "_error", [].concat(
                    anp.sessions.get_errors_list(), 
                    id ? [
                        "settings_group_null",
                        "settings_group_not_id",
                        "settings_group_not_exists",
                        "settings_group_deleted"
                    ] : [],
                    [,
                        "group_null",
                        "group_not_id",
                        "group_not_exist",
                        "group_deleted",
                        "settings_null",
                        "settings_not_id",
                        "settings_not_exist",
                        "settings_deleted",
                        "value_null",
                        "value_empty",
                        "value_too_few_length",
                        "value_not_pattern",
                        "belongs_null"
                    ]
                ), "settings_group_" + key + "_ok")){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("settings_group_exit_edit", anp.back_edit);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("settings_group_has_changes", anp.back_edit, null);
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/groups/deleted", 
        {}, 
        null, // callback
        "settings_groups_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => anp.confirm("settings_group_undelete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/groups/undelete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_group_undelete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_group_null",
                    "settings_group_not_id",
                    "settings_group_not_exists",
                    "settings_group_not_deleted"
                ]
            ), "settings_group_undelete_ok") && 
            self.refresh_deleted(item, event);
        });
    });

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/groups/history", 
        {id : Number(anp.get_globals("settings_group"))}, 
        null, // callback
        "{error_message}", 
        anp.sessions.get_errors_list().concat([
            anp.sessions.get_errors_list(), [
                "settings_group_null",
                "settings_group_not_id",
                "settings_group_not_exists",
                "settings_group_deleted"
            ]
        ])
    );

    this.restore = (item, event) => anp.confirm("settings_group_restore_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/groups/restore", {
            id : anp.comp.tables.get_row_field(item, "plain_id")
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_group_restore_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_group_null",
                    "settings_group_not_id",
                    "settings_group_not_exists",
                    "settings_group_not_deleted"
                ]
            ), "settings_group_restore_ok") && 
            self.refresh_history(item, event);
        });
    });

    construct();

};