AnP.SettingsTypes = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]).replace(/"language":[0-9]+/, "") != cache.replace(/"language":[0-9]+/, "");

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/types/list", 
        {}, 
        null, // callback
        "settings_types_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => anp.confirm("settings_type_delete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/types/delete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_type_delete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_type_null",
                    "settings_type_not_id",
                    "settings_type_not_exists",
                    "settings_type_deleted"
                ]
            ), "settings_type_delete_ok") && 
            self.refresh(item, event);
        });
    });

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => {
            
        const id = Number(anp.get_globals("settings_type"));

        if(isNaN(id)){
            cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
            return () => set_wait_values(item);
        }else{

            const texts_button = item.querySelector("button[data-i18n=i18n_texts]");

            texts_button && texts_button.removeAttribute("disabled");
            anp.ajax.send("post", "/app/sessions/settings/types/load", {id : id}, data => {
                if(anp.errors.validate(data.variables.error, "settings_type_load_error", [].concat(
                    anp.sessions.get_errors_list(),
                    [
                        "settings_type_null",
                        "settings_type_not_id",
                        "settings_type_not_exists",
                        "settings_type_deleted"
                    ]
                ))){
                    anp.comp.forms.set(item, data.data);
                    cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                    return () => set_wait_values(item);
                };
            });

        };

    };

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.name === undefined ? 1 << 0 : 
            data.name === null ? 1 << 1 : 
            !anp.is_string(data.name) ? 1 << 2 : 
            data.name.length > 64 ? 1 << 5 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.description === undefined ? 1 << 0 : 
            data.description === null ? 0 : 
            !anp.is_string(data.description) ? 1 << 2 : 
            data.description.length > 512 ? 1 << 5 : 
        0), 12, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item);

        if(JSON.stringify(data) == cache)
            return;

        const id = Number(anp.get_globals("settings_type")),
              key = id ? "edit" : "add";

        id && (data.id = id);  

        if(!anp.errors.validate(form, self.validate(data), "settings_type_" + key + "_local_error", [
            "settings_type_undefined", 
            "settings_type_null", 
            "settings_type_not_number", 
            "settings_type_not_integer",
            "settings_type_not_id" ,
            "name_undefined",
            "name_null",
            "name_not_string",
            "name_empty",
            "name_too_few_length",
            "name_too_length",
            "name_not_pattern",
            "description_undefined",
            "description_null",
            "description_not_string",
            "description_empty",
            "description_too_few_length",
            "description_too_length",
            "description_not_pattern" 
        ]))
            return;

        anp.confirm("settings_type_confirm_changes", () => {
            anp.ajax.send("post", "/app/sessions/settings/types/" + key, data, (response, ok) => {
                if(anp.errors.validate(response.variables.error, "settings_type_" + key + "_error", [].concat(
                    anp.sessions.get_errors_list(), 
                    id ? [
                        "settings_type_null",
                        "settings_type_not_id",
                        "settings_type_not_exists",
                        "settings_type_deleted"
                    ] : [],
                    [,
                        "name_null",
                        "name_empty",
                        "name_too_few_length",
                        "name_not_pattern",
                        "description_null",
                        "description_empty",
                        "description_too_few_length",
                        "description_not_pattern"
                    ]
                ), "settings_type_" + key + "_ok")){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("settings_type_exit_edit", anp.back_edit);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("settings_type_has_changes", anp.back_edit, null);
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/types/deleted", 
        {}, 
        null, // callback
        "settings_types_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => anp.confirm("settings_type_undelete_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/types/undelete", {
            id : anp.comp.tables.get_id(item)
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_type_undelete_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_type_null",
                    "settings_type_not_id",
                    "settings_type_not_exists",
                    "settings_type_not_deleted"
                ]
            ), "settings_type_undelete_ok") && 
            self.refresh_deleted(item, event);
        });
    });

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        "/app/sessions/settings/types/history", 
        {id : Number(anp.get_globals("settings_type"))}, 
        null, // callback
        "{error_message}", 
        anp.sessions.get_errors_list().concat([
            anp.sessions.get_errors_list(), [
                "settings_type_null",
                "settings_type_not_id",
                "settings_type_not_exists",
                "settings_type_deleted"
            ]
        ])
    );

    this.restore = (item, event) => anp.confirm("settings_type_restore_confirm", () => {
        anp.ajax.send("post", "/app/sessions/settings/types/restore", {
            id : anp.comp.tables.get_row_field(item, "plain_id")
        }, (response, ok) => {
            anp.errors.validate(response.variables.error, "settings_type_restore_error", [].concat(
                anp.sessions.get_errors_list(), [
                    "settings_type_null",
                    "settings_type_not_id",
                    "settings_type_not_exists",
                    "settings_type_not_deleted"
                ]
            ), "settings_type_restore_ok") && 
            self.refresh_history(item, event);
        });
    });

    construct();

};