AnP.Users = function(anp, input){

    const self = this,
          for_wait = {};
    let started = false,
        cache = null;

    const construct = () => {};

    this.start = callback => {

        const end = status => anp.is_function(callback) && callback(status);

        if(started){
            end(false);
            return false;
        };
        started = true;

        end(true);

        return true;
    };

    const has_changes = (item) => JSON.stringify(anp.comp.forms.get_values(item)[0]) != cache;

    this.refresh = (item, event) => anp.comp.tables.fill(
        item, 
        anp.anp_route() + "/administration/users/list", 
        {}, 
        null, // callback
        "users_refresh_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.delete = (item, event) => {
        
        const variables = anp.comp.tables.get_row_data(item);
        
        anp.confirm("user_delete_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/users/delete", {
                id : Number(variables.id)
            }, (response, ok) => {
                anp.errors.validate(
                    response.variables.error, [].concat(
                        anp.sessions.get_errors_list(), [
                            "user_null",
                            "user_not_id",
                            "user_not_exists",
                            "user_deleted"
                        ]
                    ), 
                    variables, 
                    "user_delete_error", 
                    "user_delete_ok"
                ) && 
                self.refresh(item, event);
            });
        }, null, variables);

    };

    const set_wait_values = item => {
        
        for(const key in for_wait)
            if((wait_data = anp.get_data_for_set(for_wait[key])) && wait_data.id){

                const form = anp.comp.forms.get(item);

                anp.comp.selects_lists.set(form.querySelector("[name=" + key + "]"), wait_data.id, wait_data);
                anp.remove_data_for_set(for_wait[key]);

                break;
            };

    };

    this.load = (item, event) => {
            
        const id = Number(anp.get_globals("user"));

        if(isNaN(id)){
            cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
            anp.comp.forms.set_texts(item, "users_add");
            return () => set_wait_values(item);
        }else{

            const texts_button = item.querySelector("button[data-i18n=i18n_texts]");

            texts_button && texts_button.removeAttribute("disabled");
            anp.ajax.send("post", anp.anp_route() + "/administration/users/load", {id : id}, data => {
                if(anp.errors.validate(
                    data.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(),
                        [
                            "user_null",
                            "user_not_id",
                            "user_not_exists",
                            "user_deleted"
                        ]
                    ), 
                    {
                        ...data.data, 
                        id : id
                    }, 
                    "user_load_error"
                )){
                    anp.comp.forms.set_texts(item, "users_edit");
                    anp.comp.forms.set(item, data.data);
                    cache = JSON.stringify(anp.comp.forms.get_values(item)[0]);
                    return () => set_wait_values(item);
                };
            });

        };

    };

    this.validate = data => {

        let error = anp.errors.set("", (
            data.id === undefined ? 0 : 
            data.id === null ? 1 << 1 : 
            !anp.is_number(data.id) ? 1 << 2 : 
            !anp.is_integer(data.id) ? 1 << 3 : 
            data.id < 1 ? 1 << 4 : 
        0), 0, 0); 

        error = anp.errors.set(error, (
            data.nick === undefined ? 1 << 0 : 
            data.nick === null ? 1 << 1 : 
            !anp.is_string(data.nick) ? 1 << 2 : 
            data.nick.length > 32 ? 1 << 5 : 
        0), 5, 0);

        error = anp.errors.set(error, (
            data.password === undefined ? 1 << 0 : 
            data.password === null ? 1 << 1 : 
            !anp.is_string(data.password) ? 1 << 2 : 
            data.password.length > 256 ? 1 << 5 : 
        0), 12, 0);

        error = anp.errors.set(error, (
            data.accessible === undefined ? 1 << 0 : 
            data.accessible === null ? 1 << 1 : 
        0), 19, 0);

        return error;
    };

    this.save = (item, event) => {

        const [data, form] = anp.comp.forms.get_values(item), 
              id = Number(anp.get_globals("user")),
              key = id ? "edit" : "add",
              variables = id ? anp.comp.forms.get_variables(item) : data;

        if(!anp.errors.validate(
            form, 
            (
                JSON.stringify(data) == cache ? 1 << 0 : 
            0), 
            [
                "form_without_changes"
            ], 
            variables, 
            "user_" + key + "_local_error"
        ))
            return;

        id && (data.id = id);

        if(!anp.errors.validate(
            form, 
            self.validate(data), 
            [
                "user_undefined", 
                "user_null", 
                "user_not_number", 
                "user_not_integer",
                "user_not_id" ,
                "nick_undefined",
                "nick_null",
                "nick_not_string",
                "nick_empty",
                "nick_too_few_length",
                "nick_too_length",
                "nick_not_pattern",
                "password_undefined",
                "password_null",
                "password_not_string",
                "password_empty",
                "password_too_few_length",
                "password_too_length",
                "password_not_pattern",
                "accessible_undefined",
                "accessible_null" 
            ], 
            variables, 
            "user_" + key + "_local_error"
        ))
            return;

        anp.confirm("user_confirm_changes", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/users/" + key, data, (response, ok) => {
                if(anp.errors.validate(
                    form, 
                    response.variables.error, 
                    [].concat(
                        anp.sessions.get_errors_list(), 
                        id ? [
                            "user_null",
                            "user_not_id",
                            "user_not_exists",
                            "user_deleted"
                        ] : [],
                        id ? [
                            "nick_null",
                            "nick_empty",
                            "password_null",
                            "password_empty",
                            "password_too_few_length",
                            "password_needs_lower",
                            "password_needs_upper",
                            "password_needs_digit",
                            "password_needs_special",
                            "accessible_null", 
                            "nick_exists", 
                            "nick_group_exists", 
                            "data_without_changes"
                        ] : [
                            "nick_null",
                            "nick_empty",
                            "nick_exists", 
                            "nick_group_exists", 
                            "password_null",
                            "password_empty",
                            "password_too_few_length",
                            "password_needs_lower",
                            "password_needs_upper",
                            "password_needs_digit",
                            "password_needs_special",
                            "accessible_null"
                        ]
                    ), 
                    variables, 
                    "user_" + key + "_error", 
                    "user_" + key + "_ok"
                )){
    
                    const i18n_button = form.querySelector("button[data-i18n=i18n_texts]");
    
                    if(i18n_button && i18n_button.hasAttribute("disabled")){
                        response.variables.id && 
                        (window.location = window.location.hash.replace(/\/add$/, "/" + response.variables.id + "/edit"));
                    }else 
                        anp.confirm("user_exit_edit", anp.back_edit, null, variables);
                    
                };
            });
        });

    };

    this.cancel = (item, event) => {
        if(has_changes(item))
            anp.confirm("user_has_changes", anp.back_edit, null, anp.comp.forms.get_variables(item));
        else 
            anp.back_edit();
    };

    this.refresh_deleted = (item, event) => anp.comp.tables.fill(
        item, 
        anp.anp_route() + "/administration/users/deleted", 
        {}, 
        null, // callback
        "users_refresh_deleted_error", 
        anp.sessions.get_errors_list().concat([])
    );

    this.undelete = (item, event) => {

        const variables = anp.comp.tables.get_row_data(item);
        
        anp.confirm("user_undelete_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/users/undelete", {
                id : anp.comp.tables.get_id(item)
            }, (response, ok) => {
                anp.errors.validate(response.variables.error, "user_undelete_error", [].concat(
                    anp.sessions.get_errors_list(), [
                        "user_null",
                        "user_not_id",
                        "user_not_exists",
                        "user_not_deleted",
                        "nick_group_exists"
                    ]
                ), "user_undelete_ok", variables) && 
                self.refresh_deleted(item, event);
            });
        }, null, variables);

    };

    this.refresh_history = (item, event) => anp.comp.tables.fill(
        item, 
        anp.anp_route() + "/administration/users/history", 
        {id : Number(anp.get_globals("user"))}, 
        null, // callback
        "users_history_refresh_error", 
        [].concat([
            anp.sessions.get_errors_list(), [
                "user_null",
                "user_not_id",
                "user_not_exists",
                "user_deleted"
            ]
        ])
    );

    this.restore = (item, event) => {
        
        const variables = {
            ...anp.comp.forms.get_variables(item), 
            ...anp.comp.tables.get_row_data(item)
        };
        
        anp.confirm("user_restore_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/users/restore", {
                id : anp.comp.tables.get_row_field(item, "plain_id")
            }, (response, ok) => {
                anp.errors.validate(response.variables.error, "user_restore_error", [].concat(
                    anp.sessions.get_errors_list(), [
                        "user_null",
                        "user_not_id", 
                        "user_current_data", 
                        "user_data_deleted",
                        "nick_group_exists"
                    ]
                ), "user_restore_ok", variables) && 
                self.refresh_history(item, event);
            });
        }, null, variables);

    };

    this.reset_password = (item, event) => {
        
        const variables = {
            ...anp.comp.forms.get_variables(item), 
            ...anp.comp.tables.get_row_data(item)
        };
        
        anp.confirm("user_reset_password_confirm", () => {
            anp.ajax.send("post", anp.anp_route() + "/administration/users/reset_password", {
                id : anp.comp.tables.get_id(item)
            }, (response, ok) => {
                anp.errors.validate(response.variables.error, "user_reset_password_error", [].concat(
                    anp.sessions.get_errors_list(), [
                        "user_null",
                        "user_not_id",
                        "user_not_exists",
                        "user_deleted",
                        "user_password_resetted"
                    ]
                ), "user_reset_password_ok", variables) && 
                self.refresh(item, event);
            });
        }, null, variables);

    };

    construct();

};