#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import datetime
import subprocess
from os.path import dirname as directory_name
from os.path import abspath as path_absolute
from os.path import isfile as is_file
from os.path import isdir as is_directory
from os.path import exists as path_exists
from os import makedirs as make_directories
from re import compile as re_compile
from json import dumps as json_encode
from json import loads as json_decode
from traceback import print_exc as print_trace_exception
from sys import exc_info as exception_information
from time import mktime as timestamp
from time import time as timestamp_now
from base64 import b64encode as base64_encode
from base64 import b64decode as base64_decode
from inspect import stack as inspect_stack
from random import randrange as random_integer
if sys.version_info[0] == 2:
    import types
    from urllib import unquote as url_decode
else:
    from urllib.parse import unquote as url_decode

class Anonymous: # AnP.Common

    anp = None
    __built_on_settings = False
    __built_with_settings = False
    __platform = None
    __application_root = None
    __roots = None
    _print_sql_save = False
    _print_types = True # Gestión de impresión por tipados. True es todos.
    _connections = {}
    _procedures = {}
    _logs_in_stream = False
    _closing = False
    __hash_alphabet = None
    __hash_length = None
    __hashes = []
    __print_format = None
    __allow_print_logs = None

    __re_string_variables = re_compile(r'\{([^\{\}]+)\}')
    __re_directory_path = re_compile(r'^(.+)([\/\\\\])([^\/\\\\]+)?$')
    __re_unix_subsystem = re_compile(r'^\/mnt\/([a-z])\/')
    __re_camel_to_snake = re_compile(r'[A-Z]')

    def platform(self):return self.__platform
    def is_dos(self):return self.__platform == "dos"
    def is_unix(self):return self.__platform == "unix"

    @staticmethod
    def python2():return sys.version_info[0] == 2
    @staticmethod
    def python3():return sys.version_info[0] == 3

    @staticmethod
    def is_bool(value):return type(value) == (types.BooleanType if AnP.python2() else bool)
    @staticmethod
    def is_array(value):return isinstance(value, (list, tuple))
    @staticmethod
    def is_list(value):return isinstance(value, list)
    @staticmethod
    def is_tuple(value):return isinstance(value, tuple)
    @staticmethod
    def is_dictionary(value):return isinstance(value, dict)
    @staticmethod
    def is_string(value):return isinstance(value, basestring if AnP.python2() else str)
    @staticmethod
    def is_function(value):return callable(value)
    @staticmethod
    def is_number(value):return isinstance(value, (int, float))
    @staticmethod
    def is_integer(value):return isinstance(value, int)
    @staticmethod
    def is_datetime(value):return type(value) in (datetime, datetime.date, datetime.datetime, datetime.time)

    @staticmethod
    def if_none(value, callback):
        if value == None:
            if AnP.is_function(callback):
                return callback()
            return callback
        return value

    @classmethod
    def string_variables(self, string, inputs = None, default = None):

        if not AnP.is_array(inputs):
            inputs = [inputs]

        def callback(matches):
            key = matches.group(1)
            if key:
                for variables in inputs:
                    if AnP.is_dictionary(variables) and key in variables:
                        return str(variables[key])
            if default != None:
                return str(default)
            return matches.group(0)

        return self.__re_string_variables.sub(callback, str(string))

    @classmethod
    def get_directory_path(self, path):
        return self.__re_directory_path.sub(r'\1', path)

    @staticmethod
    def make_directories(path):
        if AnP.is_string(path) and not path_exists(path):
            make_directories(path)

    def paths_get(self, path = None):

        relative = directory_name(path_absolute('__file__'))
        absolutes = ["", self.__application_root, relative + "/..", relative] + self.__roots

        if AnP.is_string(path):
            if path[0] not in ["/", "\\"] and path[1] != ":":
                path = "/" + path
            for i in range(len(absolutes)):
                if absolutes[i] == None:
                    continue
                if absolutes[i][:-1] in ["/", "\\"]:
                    absolutes[i] = absolutes[i][0:-1]
                absolutes[i] += path
                absolutes[i] = absolutes[i].replace("/", "\\") if self.__platform == "dos" else absolutes[i].replace("\\", "/")

        return absolutes

    def file_load(self, path):
        for absolute in self.paths_get(path):
            if absolute != None and path_exists(absolute) and is_file(absolute):
                with open(absolute, "rb") as file:
                    return file.read()
        return None

    def json_decode(self, string, show_errors = None):
        try:
            try:
                string = string.decode("utf-8")
            except:
                pass
            if AnP.is_string(string):
                return json_decode(string)
        except:
            self.anp.exception("AnP.Common.json_decode", exception_information(), show_errors if AnP.is_bool(show_errors) else self.anp.settings.get(["json_show_errors", "show_errors"]))
        return None

    def json_encode(self, object, show_errors = None):
        try:
            return json_encode(object, separators = (",", ":"))
        except:
            self.anp.exception("AnP.Common.json_encode", exception_information(), show_errors if AnP.is_bool(show_errors) else self.anp.settings.get(["json_show_errors", "show_errors"]))
        return None

    @staticmethod
    def base64_decode(data):
        try:
            data = base64_decode(data)
            try:
                data = data.decode("utf-8")
            except:
                pass
            return data
        except:
            pass
        return None

    @staticmethod
    def base64_encode(data):
        try:
            return base64_encode(data)
        except:
            try:
                return base64_encode(data.encode("utf-8")).decode("utf-8")
            except:
                self.anp.exception("AnP.Common.base64_encode", exception_information())
        return None

    @staticmethod
    def shell(command):
        try:
            with subprocess.Popen(command, stdin = None, stdout = subprocess.PIPE, stderr = subprocess.PIPE, shell = True) as stream:
                response = stream.communicate()
                try:
                    return (response[0].decode("utf-8"), response[1])
                except:
                    pass
                return response
        except:
            self.anp.exception("AnP.Common.shell", exception_information())
        return None

    @staticmethod
    def load_url(self, url, data = None, cookies = None):

        response = AnP.shell('curl -k ' + ('-X POST -d "' + url_encode(data, quote_via = quote_plus) + '" ' if isinstance(data, (dict, list, tuple)) else '') + '"' + url + '"')

        if response == None:
            return None
        return response[0]

    @staticmethod
    def are_string(data):
        if AnP.is_array(data):
            for item in data:
                if item != None and not AnP.is_string(item):
                    return False
        elif not AnP.is_string(data):
            return False
        return True

    @staticmethod
    def ternary(condition, true, false = None):
        return true if condition else false

    @staticmethod
    def timestamp():
        return round(timestamp_now())
    @staticmethod
    def timestamp_milliseconds():
        return round(timestamp_now() * 1000)
    @staticmethod
    def now():
        return datetime.datetime.now()
    @staticmethod
    def datetime(date = None):
        return datetime.datetime.fromtimestamp(int(date)) if AnP.is_number(date) else AnP.now()

    def format_path(self, path):
        if self.is_dos():
            letter = self.__re_unix_subsystem.findall(path)
            path = path.replace("/", "\\")
            if path[1] != ":" and path[0] == "\\":
                return (letter[0] if letter else "c").upper() + ":\\" + path[7 if letter else 1:]
            return path
        return ("/mnt/" + path[0].lower() + path[2:] if path[1] == ":" else path).replace("\\", "/")

    def procedure_exists(self, name):
        return name and name in self._procedures

    def procedure_get(self, name):
        return self._procedures[name] if self.procedure_exists(name) else None

    def exception(self, method, exception, _print = True, input = None, error = 0):
        if _print:
            print_trace_exception()
        if "AnPSQLite" in self._connections and self.procedure_exists("logs"):
            self.procedure_get("logs").exception(method, exception, input, error)

    def log(self, procedure, input, error):
        if "AnPSQLite" in self._connections and self.procedure_exists("logs"):
            self.procedure_get("logs").log(procedure, input, error)

    def procedure(self, procedure):
        if "AnPSQLite" in self._connections and self.procedure_exists("logs"):
            return self.procedure_get("logs").procedure(procedure)
        return None

    def _print(self, _type, text, variables = None):

        date = AnP.now()
        settings_built = self.is_built("settings")
        traces = inspect_stack()
        input = {
            "i18n" : text,
            "format" : self.settings.get("print_format") if settings_built else "[ {type} ] {yyyy}{mm}{dd} {hh}{ii}{ss}.{nnn}: {message}",
            "full_type" : _type,
            "yyyy" : str(date.year),
            "y" : date.year % 100,
            "m" : date.month,
            "d" : date.day,
            "h" : date.hour,
            "i" : date.minute,
            "s" : date.second,
            "n" : str(int(int(date.strftime("%f")) / 1000)),
            "timestamp" : str(timestamp(date.timetuple())),
            "file" : traces[1].filename,
            "line" : traces[1].lineno,
            "method" : traces[1].function
        }

        if not AnP.is_array(variables):
            variables = [variables]
        elif AnP.is_tuple(variables):
            variables = list(variables)
        variables += [input]

        input["nnn"] = "000"[len(input["n"]):] + input["n"]
        for character in ["y", "m", "d", "h", "i", "s"]:
            input[character] = str(input[character])
            input[character + character] = "00"[len(input[character]):] + input[character]

        if settings_built:

            types = self.settings.get("types")

            if not types or _type in types:
                input["type"] = _type
            else:
                for key, options in types:
                    if _type in options:
                        input["type"] = key
                        break
        else:
            input["type"] = _type

        input["type"] = input["type"].upper()
        length = len(input["type"])
        if length != 4:
            input["type"] = input["type"][0:4] if length > 4 else (" " if length < 3 else "") + input["type"] + "  "[0:int(length / 2)]

        if input["type"].lower() == "igno" or (self._print_types not in [None, True] and input["type"].lower().strip() not in self._print_types):
            return

        input["message"] = self.i18n.get(input["i18n"], variables) if self.is_built("i18n") else AnP.string_variables(input["i18n"], variables)

        settings_exists = hasattr(self.anp, "settings")
        if self.__allow_print_logs == None and settings_exists:
            self.__allow_print_logs = self.anp.settings.get(["allow_print_logs", "allow_print", "print_logs"])
        (self.__allow_print_logs or (self.__allow_print_logs == None and settings_exists)) and print(AnP.string_variables(input["format"], variables))

        if self._print_sql_save and self.anp.is_built("logs"):
            self.anp.logs.add(input["type"], date, input["i18n"], variables, traces)
        # else:
        #     self.__logs.append({
        #         "type" : input["type"],
        #         "date" : date,
        #         "text" : input["i18n"],
        #         "variables" : variables
        #     })

    def __init__(self, anp):
        self.anp = anp

        self.__platform = "dos" if path_exists("C:\\") else "unix"

    def __update_roots(self):

        roots = self.anp.settings.get("roots")

        self.__application_root = self.anp.settings.get("application_root")
        self.__roots = self.anp.settings.get("root")

        if not AnP.is_array(self.__roots):
            self.__roots = [self.__roots] if self.__roots else []
        if roots:
            for root in roots if AnP.is_array(roots) else [roots]:
                if root not in self.__roots:
                    self.__roots.append(root)

    def build_on_settings(self):

        if self.__built_on_settings:
            return
        self.__built_on_settings = True

        self.__update_roots()

    def build_with_settings(self):

        if self.__built_with_settings:
            return
        self.__built_with_settings = True

        self.__update_roots()

        platform = self.settings.get("platform")
        self.__hash_alphabet = self.settings.get(["hash_alphabet", "alphabet"])
        self.__hash_length = self.settings.get(["hash_length", "length"])

        if platform:
            self.__platform = platform

    @classmethod
    def camel_to_snake(self, name):
        def snake(matches):
            return "_" + matches.group(0).lower()
        return name[0].lower() + self.__re_camel_to_snake.sub(snake, name[1:])

    def logs_in_stream(self):
        return self._logs_in_stream

    def is_closing(self):
        return self._closing

    @staticmethod
    def url_decode(value):
        return url_decode(value)

    def hash(self):

        hash = None
        l = len(self.__hash_alphabet) - 1

        while True:
            hash = ""
            while len(hash) < self.__hash_length:
                hash += self.__hash_alphabet[random_integer(0, l)]
            if hash not in self.__hashes and hash[0] not in "1234567890":
                self.__hashes.append(hash)
                return hash

    @classmethod
    def check_data(self, *dictionaries):

        set = []

        for dictionary in dictionaries:
            if self.is_dictionary(dictionary):
                set.append(dictionary)

        if len(set) > 1:
            for key in set[0].keys():
                for dictionary in set[1:]:
                    if set[0][key] != dictionary[key]:
                        return False
        return True

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Common"):
    AnP.Common = Anonymous
del globals()["Anonymous"]
