#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Abstract.
class Anonymous: # AnP.Controller

    anp = None
    __input = None
    __connections = None
    __request_order = None
    __request_variables = None

    def __init__(self, anp, input = None, connections = None):
        self.anp = anp
        self.__input = input
        self.__connections = connections
        self.__request_order = self.anp.settings.get("request_order", input)

    def json(self, content, code = None):

        if not AnP.is_integer(code) or code < 0 or code > 999:
            code = self.anp.settings.get(["http_default_code", "http_code", "code"], self.__input)

        i18n = "http_code_" + str(code)
        data = {
            "ok" : code != None and ((code >= 200 and code < 300) or code in [301, 304]),
            "code" : code,
            "content" : content
        }

        if self.anp.settings.get("json_response_i18n", self.__input):
            data["i18n"] = i18n
        if self.anp.settings.get("json_response_message", self.__input):
            data["message"] = self.anp.i18n.get(i18n)
        if self.anp.settings.get("json_response_description", self.__input):
            data["description"] = self.anp.i18n.get(i18n + "_description")

        return (self.anp.json_encode(data).encode("utf-8"), "application/json", None)

    def set_request_variables(self, variables):
        self.__request_variables = variables

    def __request(self, names):

        if names and AnP.is_dictionary(self.__request_variables):

            if not AnP.is_array(names):
                names = [names]

            for key in self.__request_order:
                if self.__request_variables[key]:
                    for name in names:
                        if AnP.is_string(name) and name in self.__request_variables[key]:
                            return self.__request_variables[key][name]
            for name in names:
                if name in self.__request_variables:
                    return self.__request_variables[name]
        return self.anp.settings.get("default_value")

    def request(self, names):

        value = self.__request(names)

        if AnP.is_dictionary(value) and "__type__" in value and "__value__" in value:
            if value["__type__"] in ["date", "datetime", "time"]:
                return AnP.datetime(value["__value__"])
        return value

if "AnP" not in globals():
    class AnP:
        pass
AnP.Controller = Anonymous
del globals()["Anonymous"]
