#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Manager
class Anonymous: # AnP.Controllers

    anp = None
    __controllers = {}
    __connections = {}

    def set_connections(self, connections):
        if not self.__connections:
            self.__connections = connections

    def __get_controller_class(self, name):

        names = name.split(".")

        if names[0] not in globals():
            return None

        controller = globals()[names[0]]

        for level in names[1:]:
            if hasattr(controller, level):
                controller = getattr(controller, level)
            else:
                return None
        return controller

    def __add(self, controllers, overwrite):
        for key, controller in controllers.items():
            if overwrite or key not in self.__controllers:

                input = None

                if AnP.is_dictionary(controller):
                    if "controller" not in controller:
                        continue
                    if "input" in controller:
                        input = controller["input"]
                    controller = controller["controller"]

                if AnP.is_string(controller):
                    controller = self.__get_controller_class(controller)
                if controller:
                    try:
                        self.__controllers[key] = controller(self.anp, input, self.__connections)
                    except:
                        pass

    def add(self, items, overwrite = None):
        if not items:
            return

        if not AnP.is_bool(overwrite):
            overwrite = self.anp.settings.get(["controllers_overwrite", "overwrite"])

        for item in items if AnP.is_array(items) else [items]:
            if not item:
                continue
            if AnP.is_string(item):
                try:
                    json = self.anp.json_decode(item)
                    if json:
                        self.add(json, overwrite)
                        continue
                except:
                    pass
                item = self.anp.load_file(item)
                if not item:
                    continue
                try:
                    json = self.anp.json_decode(item)
                    if json:
                        self.add(json, overwrite)
                except:
                    pass
            elif AnP.is_array(item):
                self.add(item, overwrite)
            elif AnP.is_dictionary(item):
                self.__add(item, overwrite)

    def execute(self, controller, method, *arguments):
        if controller in self.__controllers and hasattr(self.__controllers[controller], method):
            method = getattr(self.__controllers[controller], method)
            if callable(method):
                self.__controllers[controller].set_request_variables(*arguments)
                return method(self.__connections, *arguments)
        return self.anp.json_encode({
            "ok" : True,
            "code" : 404
        }).encode("utf-8"), "application/json", None

    def __init__(self, anp, input = None):
        self.anp = anp

        for key in ["default_controllers", "controllers"]:
            self.add(self.anp.settings.get(key))

        self.anp._print("ok", "controllers_module_built")

if "AnP" not in globals():
    class AnP:
        pass
AnP.Controllers = Anonymous
del globals()["Anonymous"]
