#!/usr/bin/env python
# -*- coding: utf-8 -*-

from re import compile as re_compile
from math import ceil as ceil
from math import floor as floor
from math import log2 as log2

class Anonymous: # AnP.Errors

    __re_email = re_compile(r'^[^\@]+\@[^\.]+(\.[^\.]+)*?$')

    def __init__(self, anp, input = []):
        self.anp = anp

        self.__alphabet = None

        self.anp and self.anp._print("ok", "errors_module_built")

    @staticmethod
    def null_or_empty(value):
        return 1 << 0 if value == None else 1 << 1 if value == "" else 0
    @classmethod
    def empty(self, value):
        return self.null_or_empty(value)
    @classmethod
    def email(self, address):

        error = self.null_or_empty(address)

        if error:
            return error
        return 0 if self.__re_email.match(address) else 1 << 2

    @classmethod
    def integer(self, value):

        error = self.null_or_empty(value)

        return error if error else 0 if AnP.is_integer(value) else 1 << 2

    @classmethod
    def number(self, value):

        error = self.null_or_empty(value)

        return error if error else 0 if AnP.is_number(value) else 1 << 2

    @classmethod
    def id(self, value):

        error = self.integer(value)

        return error if error else 0 if value > 0 else 1 << 3

    def process(self, error, messages, variables = None):

        i = 0
        ok = True

        while error > 1 << i:
            if error & (1 << i):
                ok = False
                self.anp and self.anp._print("warn", messages[i], variables)

        return ok

    def get_alphabet(self):

        if self.__alphabet == None:
            self.__alphabet = self.anp.settings.get("base64_alphabet") if self.anp else "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="

        return self.__alphabet

    def to_array(self, code, length = 0):

        array = []

        if isinstance(code, str):

            alphabet = self.get_alphabet()
            l = c = len(code)

            length = ceil(length / 6)
            if length and length < l:
                l = length

            for i in range(l):
                array.append(alphabet.index(code[i]) if i < c else 0)

        elif isinstance(code, int):

            l = ceil(log2(code)) if code != 0 else 0

            if l < length:
                l = length

            l = ceil(l / 6)
            for i in range(l):
                array.append((code >> (i * 6)) & 0x3F)

        elif isinstance(code, (list, tuple)):

            array = code
            length /= 6

            while len(code) < length:
                code.append(alphabet[0])

        return array

    def to_array_binary(self, code):

        array = []
        alphabet = self.get_alphabet()

        for hexa in code:
            array.append(format(alphabet.index(hexa), "b").zfill(6))

        return array

    def to_integer(self, code, length = 0):

        integer = 0

        if isinstance(code, str):

            alphabet = self.get_alphabet()

            for i, hexa in enumerate(code):
                integer |= alphabet.index(hexa) << i * 6

        elif isinstance(code, int):
            integer = code
        elif isinstance(code, (list, tuple)):
            for i, hexa in enumerate(code):
                integer |= hexa << i * 6

        return integer

    def to_string(self, code, length = 0):

        string = ""
        alphabet = self.get_alphabet()

        if isinstance(code, str):
            string = code
        elif isinstance(code, int):

            i = 0

            while code >> i:
                string += alphabet[(code >> i) & 0x3F]
                i += 6

        elif isinstance(code, (tuple, list)):
            for i, hexa in enumerate(code):
                string += alphabet[hexa]

        length /= 6
        while len(string) < length:
            string += alphabet[0]

        return string if string else alphabet[0]

    def has(self, code):
        if isinstance(code, str):
            return bool(re_compile(r'[^' + self.get_alphabet()[0] + ']').match(code))
        if isinstance(code, int):
            return code != 0
        if isinstance(code, (list, tuple)):
            for hexa in code:
                if hexa != 0:
                    return True
            return False
        return None

    def type(self, code):
        for method, types in {
            "string" : (str,), 
            "integer" : (int,), 
            "array" : (list, tuple)
        }:
            if isinstance(code, types):
                return method
        return "unknown"

    def to_unknown(self, code):
        return code

    def compact(self, code):
        if isinstance(code, str):

            alphabet = self.get_alphabet()

            while code[-1:] == alphabet[0]:
                code = code[0:-1]

            return code
        if isinstance(code, (list, tuple)):
            while not code[-1:]:
                code = code[0:-1]
            return code
        return code if code else 0

    def bitwise(self, code, bits):

        if not bits or not self.has(code):
            return code

        reverse = bits < 0

        if reverse:
            bits *= -1

        start = floor(bits / 6)
        rest = bits % 6
        type = "to_" + self.type(code)

        if reverse: # right

            code = self.to_array(code)[start:]

            if rest:
                if code:

                    results = []
                    l = len(code) - 1
                    r = 6 - rest
                    block = ~-(1 << rest)

                    for i in range(l):
                        code[i] = (code[i] >> rest) | ((code[i + 1] & block) << r)
                    code[l] >>= rest

                else:
                    code = [0]

        else: # left

            code = self.to_array(code)

            if rest:

                r = 6 - rest
                block = [~-(1 << r), ~-(1 << rest) << r]
                l = len(code) - 1
                last = (code[l] & block[1]) >> r

                if last:
                    code += [last]
                for i in range(l, 0, -1):
                    code[i] = ((code[i] & block[0]) << rest) | (code[i - 1] >> r)
                code[0] = (code[0] & block[0]) << rest

            for i in range(start):
                code = [0] + code

        return getattr(self, type)(code)

    def set(self, code, error, bit = 0, length = 0):

        code = self.to_array(code)
        error = self.to_array(error)

        if bit:
            error = self.bitwise(error, bit)

        i = int(bit / 6)

        if length:

            ampersand = []
            start = bit % 6
            m = floor((length + start) / 6)
            end = (start + length) % 6

            for j in range(m):
                ampersand += [0]
            ampersand[0] |= ~-(1 << start)
            if end:
                ampersand += [~-(1 << (6 - end)) << end]

            for j, hexa in enumerate(ampersand):
                if len(code) <= j + i:
                    code += [0]
                else:
                    code[j + i] &= hexa

        if self.has(error):

            l = len(error)

            while len(code) < l:
                code += [0]

            for j in range(i, l, 1):
                code[j] = (code[j] if len(code) > j else 0) | error[j]

        return self.compact(self.to_string(code))
        # return self.to_string(code)

if "AnP" not in globals():
    class AnP:
        pass
AnP.Errors = Anonymous
del globals()["Anonymous"]
