#!/usr/bin/env python
# -*- coding: utf-8 -*-

from re import compile as re_compile
from os.path import getmtime as last_modified

class Anonymous: # AnP.Files

    anp = None
    __extension_mime = {}
    __mime_extension = {}
    __default_mime = None
    __default_extension = None

    __re_extension = re_compile(r'^.+\.([^\.]+)$')

    def load(self, path):
        self.anp.file_load(path)

    def __add_extension_mime(self, data):
        for extension, mimes in data.items():
            if not mimes:
                continue
            if extension not in self.__extension_mime:
                self.__extension_mime[extension] = []
            for mime in mimes if AnP.is_array(mimes) else [mimes]:
                if mime not in self.__extension_mime[extension]:
                    self.__extension_mime[extension].append(mime)

    def add_extension_mime(self, data):
        if data:
            for item in data if AnP.is_array(data) else [data]:
                if not item:
                    continue
                if AnP.is_string(item):
                    try:
                        json = self.anp.json_decode(item)
                        if json:
                            self.__add_extension_mime(json)
                            continue
                    except:
                        pass
                    try:
                        json = self.anp.json_decode(self.anp.file_load(item))
                        if json:
                            self.__add_extension_mime(json)
                    except:
                        pass
                elif AnP.is_array(item):
                    self.add_extension_mime(item)
                elif AnP.is_dictionary(item):
                    self.__add_extension_mime(item)

    def __add_mime_extension(self, data):
        for mime, extensions in data.items():
            if not extensions:
                continue
            if mime not in self.__mime_extension:
                self.__mime_extension[mime] = []
            for extension in extensions if AnP.is_array(extensions) else [extensions]:
                if mime not in self.__mime_extension[mime]:
                    self.__mime_extension[mime].append(extension)

    def add_mime_extension(self, data):
        if data:
            for item in data if AnP.is_array(data) else [data]:
                if not item:
                    continue
                if AnP.is_string(item):
                    try:
                        json = self.anp.json_decode(item)
                        if json:
                            self.__add_mime_extension(json)
                            continue
                    except:
                        pass
                    try:
                        json = self.anp.json_decode(self.anp.file_load(item))
                        if json:
                            self.__mime_extension(json)
                    except:
                        pass
                elif AnP.is_array(item):
                    self.add_mime_extensione(item)
                elif AnP.is_dictionary(item):
                    self.__add_mime_extension(item)

    def __init__(self, anp, input = None):
        self.anp = anp

        self.__default_mime = self.anp.settings.get("default_mime")
        self.__default_extension = self.anp.settings.get("default_extension")
        for key in ["default_mime_extension", "mime_extension"]:
            self.add_mime_extension(self.anp.settings.get(key))
        for key in ["default_extension_mime", "extension_mime"]:
            self.add_extension_mime(self.anp.settings.get(key))

        self.anp._print("ok", "files_module_built")

    def get_default_mime(self):return self.__default_mime
    def get_default_extension(self):return self.__default_extension

    def mime_to_extension(self, mime):
        if AnP.is_string(mime) and mime in self.__mime_extension and len(self.__mime_extension[mime]):
            return self.__mime_extension[mime][0]
        return self.__default_extension

    def mime_to_extensions(self, path):
        if AnP.is_string(mime) and mime in self.__mime_extension:
            return self.__mime_extension[mime]
        return [self.__default_extension]

    def extension_to_mime(self, path):

        if AnP.is_string(path):

            extension = self.__re_extension.match(path)

            extension = extension.group(1) if extension else path

            if extension and extension in self.__extension_mime and len(self.__extension_mime[extension]):
                return self.__extension_mime[extension][0]
        return self.__default_mime

    def extension_to_mimes(self, path):

        if AnP.is_string(path):

            extension = self.__re_extension.match(path)

            extension = extension.group(1) if extension else path

            if extension and extension in self.__extension_mime:
                return self.__extension_mime[extension]
        return [self.__default_mime]

    def last_modified(self, path):
        for absolute in self.anp.paths_get(path):
            if absolute != None and path_exists(absolute) and is_file(absolute):
                return last_modified(absolute)

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Files"):
    AnP.Files = Anonymous
del globals()["Anonymous"]
