#!/usr/bin/env python
# -*- coding: utf-8 -*-

class Anonymous: # AnP.I18N

    anp = None
    __i18n = {
        "english" : {
            "hello_anp" : "Welcome to AnP application!",
            "starting_anp" : "The AnP application is starting. Please, wait...",
            "settings_loading" : "The settings are been loading...",
            "settings_loading_block" : "The settings block '{block}' is been loading...",
            "settings_loaded" : "All settings was loaded successfully.",
            "i18n_loading" : "The I18N is been loading...",
            "i18n_loading_block" : "The I18N block '{block}' is been loading...",
            "i18n_loaded" : "All I18N was loaded successfully.",
            "settings_starting" : "The settings module is starting...",
            "settings_started" : "The settings module is started.",
            "i18n_starting" : "The I18N module is starting...",
            "i18n_started" : "The I18N module is started."
        }
    }
    __language = None
    __started = False

    def default(self, default = None, nulls = None):
        return str(
            default if default != None or (
                nulls if AnP.is_bool(nulls) else
                self.anp.settings.get(["i18n_null", "nulls"])
            ) else self.anp.settings.get("default_text")
        )

    def __get(self, names, default, nulls):

        has_settings = self.anp.is_built("settings")

        if not AnP.is_bool(nulls):
            nulls = self.anp.settings.get(["i18n_null", "nulls"]) if has_settings else False

        if not names:
            return self.default(default, nulls)

        if not AnP.is_array(names):
            names = [names]

        for language in [self.__language, self.anp.settings.get("default_language") if has_settings else None] + list(self.__i18n.keys()):
            if language and language in self.__i18n:
                for name in names:
                    if name in self.__i18n[language] and (nulls or self.__i18n[language][name] != None):
                        return " ".join(self.__i18n[language][name]) if AnP.is_array(self.__i18n[language][name]) else str(self.__i18n[language][name])
        for name in names:
            if AnP.is_string(name):
                return name
        return self.default(default, nulls)

    def get(self, names, variables = None, default = None, nulls = None):
        return AnP.string_variables(self.__get(names, default, nulls), variables)

    def __add(self, input, overwrite):
        for language, sentences in input.items():
            if language not in self.__i18n:
                self.__i18n[language] = {}
            for key, text in sentences.items():
                if overwrite or key not in self.__i18n[language]:
                    self.__i18n[language][key] = text

    def add(self, inputs, overwrite = None):
        if not inputs:
            return

        if not AnP.is_bool(overwrite):
            overwrite = self.anp.settings.get(["i18n_overwrite", "overwrite"])

        for input in inputs if AnP.is_array(inputs) else [inputs]:
            if not input:
                continue
            if AnP.is_dictionary(input):
                self.__add(input, overwrite)
            elif AnP.is_array(input):
                self.add(input, overwrite)
            elif AnP.is_string(input):
                json = self.anp.json_decode(input)
                if json == None:
                    data = self.anp.file_load(input)
                    if data:
                        json = self.anp.json_decode(data)
                if json:
                    if AnP.is_array(json):
                        self.add(json)
                    else:
                        self.__add(json, overwrite)

    def change(self, language):
        if language and self.__language != language and language in self.__i18n:
            self.__language = language

    def __init__(self, anp, input = None):
        self.anp = anp

        self.anp._print("ok", "The I18N module was built.")

    def start(self):

        if self.__started:
            return
        self.__started = True

        self.anp._print("info", "i18n_starting")

        self.anp._print("info", "i18n_loading")
        for key in ["default_i18n_files", "i18n_files"]:
            self.anp._print("info", "i18n_loading_block", {"block" : key})
            self.add(self.anp.settings.get(key), True)
        self.anp._print("info", "i18n_set_default")
        self.change(self.anp.settings.get(["language", "default_language"]))
        self.anp._print("ok", "i18n_loaded")

        self.anp._print("ok", "i18n_started")

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "I18N"):
    AnP.I18N = Anonymous
del globals()["Anonymous"]
