#!/usr/bin/env python
# -*- coding: utf-8 -*-

from re import compile as re_compile
from traceback import print_exc as print_trace_exception

# Manager
class Anonymous: # AnP.Routes

    anp = None
    __routes = {}
    __default_method = None
    __indexes = [""]
    __roots = []

    __re_route = re_compile(r'^(([^\:]+)\:)?(\/([^ ]+)?) ([\/\\\\]([^ ]+)?|([^\@]+)\@([^ ]+))( (.+))?$')
    __re_permissions = re_compile(r'[,; \:\/\|]')
    __re_variables = re_compile(r'\\\\\{(([^\{\}\\\\]+|\\\\.)+)\\\\\}')
    __to_regular_expression = re_compile(r'([\\\\\/\(\)\{\}\:\.\+\-\*\^\$\|\!\?])')

    def __add(self, items, overwrite):
        for item in items:

            matches = self.__re_route.findall(item)

            if not matches or len(matches) == 0:
                continue
            matches = matches[0]
            variables = []

            def callback(matches):
                name = matches.group(1)
                if name:
                    variables.append(name)
                    return r'([^\/]+)'
                return matches.group(0)

            method = (matches[1] if matches[1] else self.__default_method).lower()
            route = self.__re_variables.sub(callback, self.__to_regular_expression.sub(r'\\\1', matches[2]))
            # route = self.__re_variables.sub(callback, matches[2])
            path = matches[4] if matches[4][0] in ["/", "\\"] else None

            if path != None:
                variables.append("path")

            new_route = {
                "route" : re_compile(r'^' + (route if route[-1] != "/" else route[:-2]) + (r'(.*)' if path != None else r'\/?') + r'$'),
                "variables" : variables,
                "path" : path,
                "method" : matches[6],
                "controller" : matches[7],
                "permissions" : self.__re_permissions.split(matches[9]) if matches[9] != None else []
            }

            if method not in self.__routes:
                self.__routes[method] = []
            for current in self.__routes[method]:
                if current["route"].pattern == new_route["route"].pattern:
                    for key, value in new_route.items():
                        if key != "route":
                            current[key] = value
                    return
            self.__routes[method].append(new_route)

    def add(self, items, overwrite = None):
        if not items:
            return

        if not AnP.is_bool(overwrite):
            overwrite = self.anp.settings.get(["routes_overwrite", "overwrite"])

        for item in items if AnP.is_array(items) else [items]:
            if not item:
                continue

            if AnP.is_string(item):
                try:
                    json = self.anp.json_decode(item)
                    if json:
                        self.add(json, overwrite)
                        continue
                except:
                    pass
                try:
                    json = self.anp.json_decode(self.anp.file_load(item))
                    if json:
                        self.add(json, overwrite)
                        continue
                except:
                    pass
                self.__add([item], overwrite)
            elif AnP.are_string(item):
                self.__add(item, overwrite)

    def __init__(self, anp, input = None):
        self.anp = anp

        if input and (AnP.is_array(input) or AnP.is_string(input)):
            input = {"routes" : input}

        self.__default_method = self.anp.settings.get(["default_method", "default_http_medhod", "http_method", "method"], input)

        roots = []
        # roots = self.anp.settings.get(["routes_roots", "roots", "routes_root", "root"], input)
        indexes = self.anp.settings.get(["routes_indexes", "indexes"], input)

        for key in ["routes_roots", "roots", "routes_root", "root"]:
            new_roots = self.anp.settings.get(key, input)
            if new_roots:
                for new_root in new_roots if AnP.is_array(new_roots) else [new_roots]:
                    roots.append(new_root)

        self.__indexes = (indexes if AnP.is_array(indexes) else [indexes]) if indexes else []
        self.__indexes.insert(0, "")
        self.__roots = (roots if AnP.is_array(roots) else [roots]) if roots else []
        self.__routes = {}

        for key in ["routes", "default_routes"]:
            self.add(self.anp.settings.get(key, input))

        self.anp._print("ok", "routes_module_built")

    def date_format(self, datetime):
        return AnP.datetime(datetime).strftime("%a, %d %b %Y %H:%M:%S GTM")

    def go(self, method, route, variables):
        if method and AnP.is_string(route) and method in self.__routes:
            for current in self.__routes[method]:
                matches = current["route"].match(route)
                if matches:

                    matches = matches.groups()

                    for i in range(len(matches)):
                        variables[current["variables"][i]] = matches[i]

                    if current["path"] != None:
                        if not current["controller"] or not current["method"] or self.anp.controllers.execute(current["controller"], current["method"], variables):
                            path = None
                            ok = False
                            for root in self.__roots:
                                for index in self.__indexes:
                                    path = root + current["path"] + variables["path"]
                                    if index:
                                        path += ("/" if path[-1] != "/" else "") + index
                                    data = None
                                    if data == None:
                                        data = self.anp.file_load(path)
                                    if data:
                                        data = data, self.anp.files.extension_to_mime(path), self.date_format(self.anp.files.last_modified(path))
                                        ok = True
                                        break
                                if ok:
                                    break
                            if AnP.python3():
                                if data == None:
                                    return (self.anp.json_encode({
                                        "ok" : True,
                                        "code" : 520
                                    }).encode("utf-8"), self.anp.files.get_default_mime(), None)
                                return data
                            if data == None:
                                return (self.anp.json_encode({
                                    "ok" : True,
                                    "code" : 520
                                }), self.anp.files.get_default_mime(), None)
                            return data
                        if AnP.python3():
                            return (self.anp.json_encode({
                                "ok" : True,
                                "code" : 501
                            }).encode("utf-8"), self.anp.files.get_default_mime(), None)
                        return (self.anp.json_encode({
                            "ok" : True,
                            "code" : 501
                        }), self.anp.files.get_default_mime(), None)
                    return self.anp.controllers.execute(current["controller"], current["method"], variables)
        return self.anp.json_encode({
            "ok" : True,
            "code" : 404
        }).encode("utf-8"), "application/json", None

if "AnP" not in globals():
    class AnP:
        pass
AnP.Routes = Anonymous
del globals()["Anonymous"]
