AnP#!/usr/bin/env python
# -*- coding: utf-8 -*-

class Anonymous: # AnP.Settings

    anp = None
    __default_settings = {
        "settings_nulls" : False,
        "default_value" : None,
        "autostart" : True,
        "print_format" : "[ {type} ] {yyyy}{mm}{dd} {hh}{ii}{ss}.{nnn}: {message}",
        "default_settings_files" : "/JSON/AnP.py.settings.json",
        "settings_overwrite" : False,
        "json_show_errors" : False
    }
    __settings = {}
    __input = None
    __started = False

    def __get_settings_block(self):
        return [self.__input, self.__settings, self.__default_settings]

    def __default(self, default, nulls):
        if nulls or default != None:
            return default
        for input in self.__get_settings_block():
            for key in ["default", "default_value"]:
                if input and key in input and (nulls or input[key] != None):
                    return input[key]
        return None

    def get(self, names = None, inputs = None, default = None, nulls = None):

        if not AnP.is_bool(nulls):
            for input in self.__get_settings_block():
                if input:
                    for key in ["settings_nulls", "nulls"]:
                        if key in input and AnP.is_bool(input[key]):
                            nulls = input[key]
            if not AnP.is_bool(nulls):
                nulls = False

        if not names:
            return self.__default(default, nulls)

        if not AnP.is_array(names):
            names = [names]

        for entries in [inputs if AnP.is_array(inputs) else [inputs]] + [self.__get_settings_block()]:
            if AnP.is_array(entries):
                for input in entries:
                    if AnP.is_dictionary(input):
                        for name in names:
                            if name in input and (nulls or input[name] != None):
                                return input[name]
        return self.__default(default, nulls)

    def __add(self, input, overwrite):
        for key, value in input.items():
            if overwrite or key not in _self.__settings:
                self.__settings[key] = value

    def add(self, inputs, overwrite = None):
        if not inputs:
            return

        if not AnP.is_bool(overwrite):
            overwrite = self.get(["settings_overwrite", "overwrite"])

        for input in inputs if AnP.is_array(inputs) else [inputs]:
            if not input:
                continue
            if AnP.is_dictionary(input):
                self.__add(input, overwrite)
            elif AnP.is_array(input):
                self.add(input, overwrite)
            elif AnP.is_string(input):
                json = self.anp.json_decode(input)
                if json == None:
                    data = self.anp.file_load(input)
                    if data:
                        json = self.anp.json_decode(data)
                if json:
                    if AnP.is_array(json):
                        self.add(json)
                    else:
                        self.__add(json, overwrite)

    def __init__(self, anp, input = None):
        self.anp = anp
        self.__input = input

        self.anp._print("ok", "The Settings module was built.")

    def start(self):

        if self.__started:
            return
        self.__started = True

        self.anp._print("info", "settings_starting")

        self.anp._print("info", "settings_loading")
        for key in ["default_settings_files", "settings_files", "secrets_files"]:
            self.anp._print("info", "settings_loading_block", {"block" : key})
            self.add(self.anp.settings.get(key), True)
        self.anp._print("ok", "settings_loaded")

        self.anp._print("ok", "settings_started")

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Settings"):
    AnP.Settings = Anonymous
del globals()["Anonymous"]
