#!/usr/bin/env python
# -*- coding: utf-8 -*-

from re import compile as re_compile

class Anonymous: # AnP.Terminals

    __started = False
    __thread = None
    __raw_input = None
    __orders = {}

    __re_commands = re_compile(r'^([^ ]+)( (.+)?)?$')
    __re_parameters = re_compile(r'([^ \=\'"]+|\'(([^\'\\\\]+|\\\\(.|[\r\n]))*)\'|"(([^\\\\"]+|\\\\(.|[\r\n]))*)")(\=([^ \'"]+|\'(([^\'\\\\]+|\\\\(.|[\r\n]))*)\'|"(([^\\\\"]+|\\\\(.|[\r\n]))*)"))?')

    def add(self, names, action, help = None):

        if not AnP.is_array(names):
            names = [names]

        while len(names) > 0:
            name = names[0]
            names = names[1:]
            if AnP.is_string(name):
                break
            name = None

        if not name:
            return

        if name in self.__orders:
            if self.__orders[name]["action"] != action:
                self.__orders[name]["action"] = action
            for synonymous in names:
                if synonymous not in self.__orders[name]:
                    self.__orders[name]["synonymous"].append(synonymous)
        else:
            self.__orders[name] = {
                "action" : action,
                "synonymous" : names,
                "help" : help
            }

    def __init__(self, anp, _input = None):
        self.anp = anp

        self.anp._print("info", "setting_terminal_input")
        self.__raw_input = input if AnP.python3() else raw_input

        self.add(["help", "h", "?"], self.help, {
            "text" : "help_help_text",
            "attributes" : {
                "1 .. N" : "help_commands_text"
            }
        })

        if self.anp.settings.get(["terminals_autostart", "autostart"]):
            self.start()

        self.anp._print("ok", "terminals_module_build")

    def __execute_command(self, order, values):

        parameters = {}

        if values:
            i = 0
            for parameter in self.__re_parameters.findall(values):
                if parameter[7]:
                    parameters[parameter[0][1:-1] if parameter[0][0] in "'\"" else parameter[0]] = parameter[8][1:-1] if parameter[8][0] in "'\"" else parameter[8]
                else:
                    parameters[str(i)] = parameter[0][1:-1] if parameter[0][0] in "'\"" else parameter[0]
                    i += 1

        for key, values in self.__orders.items():
            if order == key or (values["synonymous"] and order in values["synonymous"]):
                try:
                    values["action"](parameters, values)
                except:
                    self.anp.exception("AnP.Terminals.__execute_command")
                return
        self.anp._print("warn", "unknown_command", {"order" : order})

    def __thread_method(self, thread = None, i = None):
        self.__re_commands.sub(lambda matches: self.__execute_command(matches.group(1), matches.group(3)), self.__raw_input())

    def start(self):
        self.anp._print("info", "terminals_starting")

        if self.__started:
            self.anp._print("warn", "terminals_already_started")
            return
        self.__started = True

        self.__thread = self.anp.threads.add(self.__thread_method)
        self.anp._print("ok", "terminals_started")

    def stop(self):
        self.anp._print("info", "terminals_stopping")

        if not self.__started:
            self.anp._print("warn", "terminals_already_stopped")
            return
        self.__started = False

        self.__thread = self.anp.threads.remove(self.__thread)
        self.anp._print("ok", "terminals_stopped")

    def __get_help(self, command, full = False):

        original = command

        if "help" not in self.__orders[command] or not self.__orders[command]["help"]:

            ok = False

            for key, values in self.__orders.items():
                if command in values["synonymous"]:
                    command = key
                    ok = True
                    break

            if not ok:
                return self.anp.i18n.get("command_without_help", {
                    "order" : original,
                    "synonymous" : ", ".join(self.__orders[command]["synonymous"])
                })

        if AnP.is_string(self.__orders[command]["help"]):
            return self.anp.i18n.get(self.__orders[command]["help"], {
                "order" : original,
                "synonymous" : ", ".join(self.__orders[command]["synonymous"])
            })

        attributes = ""

        if full:
            for key, description in self.__orders[command]["help"]["attributes"].items():
                attributes += " - " + key + ": " + self.anp.i18n.get(description) + "\n"
        return self.anp.i18n.get(self.__orders[command]["help"]["text"], {
            "order" : original,
            "synonymous" : ", ".join(self.__orders[command]["synonymous"])
        }) + ("\n" + attributes if attributes else "")

    def help(self, parameters = None, values = None):

        i = 0
        commands = ""

        if AnP.is_dictionary(parameters):
            while True:
                _i = str(i)
                if _i in parameters:
                    commands += self.anp.i18n.get(self.__get_help(parameters[_i], True) if parameters[_i] in self.__orders else "command_unknown", {
                        "order" : parameters[_i],
                        "synonymous" : ", ".join(self.__orders[parameters[_i]]["synonymous"]) if parameters[_i] in self.__orders else ""
                    }) + "\n"
                    i += 1
                else:
                    break

        if i != 0:
            self.anp._print("infor", "anp_help", {"orders" : commands})
            return

        for command, data in self.__orders.items():
            commands += self.__get_help(command) + "\n"

        self.anp._print("info", "anp_help_full", {"orders" : commands})

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Terminals"):
    AnP.Terminals = Anonymous
del globals()["Anonymous"]
