#!/usr/bin/env python
# -*- coding: utf-8 -*-

import threading
from time import sleep

class Anonymous: # AnP.Threads

    __started = False
    __threads = []
    __working = False

    def __init__(self, anp, input = None):
        self.anp = anp

        if self.anp.settings.get(["threads_autostart", "autostart"]):
            self.start()

        self.anp._print("ok", "threads_module_built")

    def __thread(self, i):
        while self.__threads[i] and self.__threads[i]["working"]:

            date = AnP.timestamp_milliseconds() / 1000.0

            if date - self.__threads[i]["last"] > self.__threads[i]["execute"]:
                self.__threads[i]["last"] = date
                self.__threads[i]["callback"](self.__threads[i], i)
            if self.__working and self.__threads[i] and self.__threads[i]["working"]:
                sleep(self.__threads[i]["sleep"])
                continue
            break

    def start(self):
        self.anp._print("info", "threads_starting")

        if self.__working:
            self.anp._print("warn", "threads_already_working")
            return

        self.__working = True
        self.anp._print("ok", "threads_working")

    def stop(self):
        self.anp._print("info", "threads_stopping")

        if not self.__working:
            self.anp._print("warn", "threads_already_stopped")
            return

        self.__working = False
        self.anp._print("ok", "threads_stopped")

    def add(self, callback, frames_per_second = None):
        if not callable(callback):
            return None

        i = 0
        l = len(self.__threads)
        ok = False
        data = {
            "callback" : callback,
            "execute" : frames_per_second if AnP.is_number(frames_per_second) else self.anp.settings.get("frames_per_second"),
            "working" : True,
            "last" : 0
        }
        sleep_time = 1.0 / self.anp.settings.get("threads_minimum_sleep")

        if AnP.is_number(data["execute"]):
            data["execute"] = 1.0 / data["execute"]
        data["sleep"] = sleep_time if data["execute"] > sleep_time else data["execute"]

        while not ok:
            if i >= l:
                break
            ok = not self.__threads[i]
            if not ok:
                i += 1

        if ok:
            self.__threads[i] = data
        else:
            self.__threads.append(data)

        data["thread"] = threading.Thread(target = lambda: self.__thread(i))
        data["thread"].start()

        return i

    def remove(self, i):
        if AnP.is_number(i) and self.__threads[i]:
            self.__threads[i] = None

    def close(self):
        for i in range(len(self.__threads)):
            self.remove(i)
        self.stop();

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Threads"):
    AnP.Threads = Anonymous
del globals()["Anonymous"]
