#!/usr/bin/env python
# -*- coding: utf-8 -*-

from re import compile as re_compile
from socket import socket
from socket import AF_INET as net_ipv4
from socket import SOCK_STREAM as secure_tcp
from socket import SOL_SOCKET as socket_level
from socket import SO_REUSEADDR as socket_reuse_address
from socket import SHUT_RDWR as socket_shutdown
from time import sleep
import select

class Anonymous: # AnP.WebServers

    __started = False
    __server = None
    __name = None
    __in_thread = None
    __thread = None
    __working = False
    __clients = []
    __starting = False
    __stopping = False
    __input = None
    __port = None
    __hosts = None
    __simultaneous_connections = None
    __response_header = None
    __full_prints = None
    __routes = None
    __variable_name = None
    __variables_method = None

    __re_post_data = re_compile(r'^(([^\r\n]+(\r\n|[\r\n]))+)(\r\n|[\r\n])((.+|[\r\n]+))$')
    __re_get = re_compile(r'^([^ ]+) (([^ \?]+)(\?([^ ]+)?)?) .+$')
    __re_properties = re_compile(r'^([^\:]+)(\: ?([^\r\n]+)?)?')
    __re_uri_variables = re_compile(r'([^\&\=]+)=([^\&]*)')

    def __get_variables(self, uri):

        variables = {}

        for (key, value) in self.__re_uri_variables.findall(uri):
            variables[key] = value

        return variables

    def __thread_method(self, parameters = None, values = None):

        print_data = None
        client = None

        try:

            try:
                client, (ip, port) = self.__server.accept()
            except:
                return

            print_data = {
                "name" : self.__name,
                "ip" : ip,
                "port" : port
            }
            cache = None

            if self.__full_prints:
                self.anp._print("info", "web_server_request", print_data)

            method = None
            cache_size = self.anp.settings.get("web_server_cache_size", self.__input)
            variables = {}
            attributes = []
            receibed = []
            cache = ""

            # print(cache_size)

            while True:
                try:
                    _buffer = client.recv(cache_size)
                    length = len(_buffer)
                    cache += _buffer.decode("utf-8")
                    if length < cache_size:
                        break
                except:
                    break

            receibed = self.__re_post_data.findall(cache)

            if not len(receibed):
                return
            receibed = receibed[0]

            attributes = receibed[0].split("\n")
            get = self.__re_get.findall(attributes[0])[0]
            method = get[0].lower()
            for key, value in {
                "ip" : ip,
                "ip_route" : [ip],
                "port" : port,
                "length" : len(cache),
                "route" : get[2],
                "get" : self.__get_variables(get[4]),
                "post" : self.__get_variables(receibed[4]),
                "method" : method,
                "custom" : {}
            }.items():
                variables[key] = value

            for property in attributes[1:]:
                matches = self.__re_properties.findall(property)
                if not len(matches):
                    continue
                matches = matches[0]
                variables[matches[0].replace("-", "_").lower()] = matches[2]

            if not receibed[4].strip() and "content_length" in variables:
                body = ''
                length = int(variables["content_length"])
                while len(body) < length:
                    try:
                        body += client.recv(cache_size).decode("utf-8")
                    except:
                        pass
                variables["post"] = self.__get_variables(body)

            for web_method in ["get", "post"]:
                if self.__variable_name in variables[web_method]:
                    if self.__variables_method == "base64":
                        data = self.anp.base64_decode(AnP.url_decode(variables[web_method][self.__variable_name]))
                        for coding in ["latin-1", "utf-8"]:
                            try:
                                data = data.decode(coding)
                                break
                            except:
                                pass
                        if data:
                            variables["custom"].update(self.anp.json_decode(data))

            for key, value in {
                "cache" : len(cache),
                "method" : method,
                "route" : variables["route"]
            }.items():
                print_data[key] = value
            if self.__full_prints:
                self.anp._print("info", "web_server_request_method", print_data)

            response, variables["mime"], variables["last_modified"] = self.__routes.go(variables["method"], variables["route"], variables)
            variables["date"] = AnP.now().strftime("%a, %d %b %Y %H:%M:%S GTM")
            if variables["last_modified"] == None:
                variables["last_modified"] = variables["date"]
            variables["response_length"] = len(response)

            print_data["response"] = len(response)
            response = AnP.string_variables(self.__response_header + "\n\n", variables).encode("utf-8") + response

            if self.__full_prints:
                self.anp._print("info", "web_server_request_responding", print_data)

            client.sendall(response)

        except:
            print_trace_exception()
            self.anp.exception("AnP.WebServers.__thread_method", exception_information(), self.anp.settings.get("web_server_exception", self.__input))
        self.__close(client, "client")
        if self.__full_prints:
            self.anp._print("info", "web_server_waiting", {"name" : self.__name})
        else:
            self.anp._print("info", "web_server_full_print", print_data)

    def start(self):

        self.anp._print("info", "web_server_starting", {"name" : self.__name})

        if self.__started:
            self.anp._print("warn", "web_server_already_started", {"name" : self.__name})
            return
        if self.__starting:
            self.anp._print("warn", "web_server_already_starting", {"name" : self.__name})
            return
        if self.__stopping:
            self.anp._print("warn", "web_server_already_stopping", {"name" : self.__name})
            return
        self.__started = True
        self.__starting = True

        self.__port = self.anp.settings.get(["web_server_port", "port"], self.__input)
        self.__hosts = self.anp.settings.get("hosts", self.__input)
        self.__simultaneous_connections = self.anp.settings.get("simultaneous_connections", self.__input)
        self.__variable_name = self.anp.settings.get(["web_server_variables_name", "variables_name", "variable_name"], self.__input)
        self.__variables_method = self.anp.settings.get(["web_server_variables_method", "variables_method", "variable_method"], self.__input)

        print_data = {
            "name" : self.__name,
            "port" : self.__port,
            "hosts" : ", ".join(self.__hosts) if AnP.is_array(self.__hosts) else self.__hosts,
            "simultaneous_connections" : self.__simultaneous_connections
        }

        self.anp._print("info", "web_server_building", print_data)

        self.__server = socket(net_ipv4, secure_tcp)

        self.__server.setsockopt(socket_level, socket_reuse_address, 1)
        self.__server.bind((self.__hosts, self.__port))
        self.__server.listen(self.__simultaneous_connections)

        self.anp._print("ok", "web_server_built", print_data)

        if self.__in_thread:
            self.anp._print("info", "web_server_starting_thread", {"name" : self.__name})
            self.__thread = self.anp.threads.add(self.__thread_method)
        else:
            self.anp._print("info", "web_server_starting_node", {"name" : self.__name})
            self.__working = True
            while self.__working:
                self.__thread_method()

        self.anp._print("ok", "web_server_started", {"name" : self.__name})
        self.__starting = False

    def __close(self, socket, name):

        print_data = {
            "name" : self.__name,
            "socket_type" : name
        }
        _print = socket == "client" and self.__full_prints

        if _print:
            self.anp._print("info", "web_server_closing_socket", print_data)

        try:
            socket.shutdown(socket_shutdown)
        except:
            pass
        try:
            socket.close()
        except:
            pass

        if _print:
            self.anp._print("ok", "web_server_closed_socket", print_data)

        return None

    def close(self):

        self.anp._print("info", "web_server_closing", {"name" : self.__name})

        if not self.__started:
            self.anp._print("warn", "web_server_already_closed", {"name" : self.__name})
            return
        if self.__starting:
            self.anp._print("warn", "web_server_already_starting", {"name" : self.__name})
            return
        if self.__stopping:
            self.anp._print("warn", "web_server_already_stopping", {"name" : self.__name})
            return
        self.__started = False
        self.__stopping = True

        self.__close(self.__server, "server")
        self.anp.threads.remove(self.__thread)
        self.__thread = None

        self.anp._print("ok", "web_server_closed", {"name" : self.__name})
        self.__stopping = False

    def __init__(self, anp, input = None):
        self.anp = anp
        self.__input = input

        routes = self.anp.settings.get(["web_server_routes", "routes"], input)

        self.__name = self.anp.settings.get(["web_server_name", "name"], input)
        self.__in_thread = self.anp.settings.get("web_server_thread", input)
        self.__response_header = self.anp.settings.get(["http_response_header", "http_header", "response_header", "header"], input)
        self.__full_prints = self.anp.settings.get("web_server_full_prints", input)

        if AnP.is_array(self.__response_header):
            self.__response_header = "\n".join(self.__response_header)
        elif not AnP.is_string(self.__response_header):
            self.__response_header = str(self.__response_header)

        self.__routes = AnP.Routes(self.anp, routes)

        if self.anp.settings.get(["web_server_autostart", "autostart"], input):
            self.start()

        self.anp._print("ok", "web_server_module_built", {"name" : self.__name})

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "WebServers"):
    AnP.WebServers = Anonymous
del globals()["Anonymous"]
