#!/usr/bin/env python
# -*- coding: utf-8 -*-

import inspect
from os.path import exists as path_exists
from os import kill
from os import getpid as get_pid
from signal import SIGTERM as termination_signal
from traceback import format_tb as format_traceback

class AnP(AnP.Common):

    __built = {}
    __started = False
    __web_servers = {}
    __session = None

    def is_built(self, key):
        return self.__built[key] if key in self.__built else None

    def __build(self, key, _class, input = None):
        if hasattr(AnP, _class) and not hasattr(self, key):
            setattr(self, key, getattr(AnP, _class)(self, input))
            self.__built[key] = True

    def __start(self, key):
        if hasattr(self, key):
            item = getattr(self, key)
            hasattr(item, "start") and item.start()

    def __init__(self, input = None):
        super().__init__(self)

        # Construir los módulos de configuración e I18N.
        self.__build("settings", "Settings", input)
        super().build_on_settings()
        self.__build("i18n", "I18N", input)
        # Construímos el módulo de gestión de Logs.
        if hasattr(AnP, "Logs") and not hasattr(self, "logs"):
            setattr(self, "logs", AnP.Logs(self, input))
            self.__built["logs"] = True
        # Iniciamos los módulos iniciales.
        self.__start("settings")
        super().build_with_settings()
        self.__start("i18n")

        self._print_sql_save = self.settings.get("print_sql_save")
        self._print_types = self.settings.get("print_types")

        for key, _class in {
            "threads" : "Threads",
            "terminals" : "Terminals",
            "files" : "Files",
            "controllers" : "Controllers",
            "errors" : "Errors"
        }.items():
            self.__build(key, _class)
            self.__start(key)

        hasattr(self, "terminals") and self.terminals and self.terminals.add(["close", "bye"], self.close, "close_help_text")

        self.settings.get("autostart") and self.start()

    __web_servers_error = [
        "exception",
        "input_not_dictionary",
        "name_not_input",
        "connection_exists"
    ]

    def web_server_add(self, input, overwrite = None):

        error = 0

        try:

            error |= (0 if "name" in input else 1 << 2) if AnP.is_dictionary(input) else 1 << 1

            if error:
                return error

            if not AnP.is_bool(overwrite):
                overwrite = self.settings.get(["web_servers_overwrite", "overwrite"])

            if input["name"] in self.__web_servers:
                if overwrite:
                    self.__web_servers[input["name"]].close()
                else:
                    error |= 1 << 3

            self.__web_servers[input["name"]] = AnP.WebServers(input)
        except:
            error |= 1 << 0
            self.exception("AnP.connection_add", exception_information())

        return error

    __connections_error = [
        "exception",
        "input_not_dictionary",
        "name_not_input",
        "type_not_input",
        "type_no_driver",
        "connection_exists"
    ]

    def connection_add(self, input, overwrite = None):
        if not hasattr(AnP, "Drivers"):
            return 1 << 6

        error = 0

        try:

            error |= (
                (0 if "name" in input else 1 << 2) |
                ((0 if hasattr(AnP.Drivers, input["type"]) else 1 << 4) if "type" in input else 1 << 3)
            ) if AnP.is_dictionary(input) else 1 << 1

            if error:
                return error

            if not AnP.is_bool(overwrite):
                overwrite = self.settings.get(["connections_overwrite", "overwrite"])

            if input["name"] in self._connections:
                if overwrite:
                    self._connections[input["name"]].close()
                else:
                    error |= 1 << 5

            self._connections[input["name"]] = getattr(AnP.Drivers, input["type"])(self, input)

        except:
            error |= 1 << 0
            self.exception("AnP.connection_add", exception_information())

        return error

    def close(self, thread = None, i = None):
        for web_server in self.__web_servers.values():
            web_server.close()
        # for connection in self._connections.values():
        #     connection.close()
        "users" in self._procedures and self._procedures["users"].close()
        self._closing = True
        not self._logs_in_stream and self.logs.stop()
        self.is_built("threads") and self.threads.close()
        self._print("info", "anp_bye")
        # The Web Servers has problems with Webkit for finishing fully.
        # This is the reason for the next line.
        kill(get_pid(), termination_signal)

    def start(self):

        if self.__started:
            return
        self.__started = True

        mode = self.settings.get("base_mode")
        connections = self.settings.get("connections")
        web_servers = self.settings.get("web_servers")

        if connections:
            for connection in connections if AnP.is_array(connections) else [connections]:
                error = self.connection_add(connection)

        if hasattr(AnP, "Procedures"):
            for item in AnP.Procedures.__dict__.keys():
                if item[0] != "_" and hasattr(AnP.Procedures, item):
                    name = AnP.camel_to_snake(item)
                    self._procedures[name] = getattr(AnP.Procedures, item)(self, self._connections)
            for procedures in self._procedures.values():
                hasattr(procedures, "build") and procedures.build()

        self._logs_in_stream = self.settings.get("logs_in_stream")

        hasattr(self, "logs") and self.logs and not self._logs_in_stream and self.logs.start()

        if web_servers:
            for web_server in web_servers if AnP.is_array(web_servers) else [web_servers]:
                if "name" in web_server and web_server["name"] not in self.__web_servers:
                    self.__web_servers[web_server["name"]] = AnP.WebServers(self, web_server)

        if mode == "sqlite":
            pass

    def session_set(self, id):
        self.__session = id

    def session_get(self):
        return self.__session
