#!/usr/bin/env python
# -*- coding: utf-8 -*-

from traceback import format_tb as format_traceback

class Anonymous: # AnP.Procedures.Groups

    anp = None
    __connections = {}
    __built = False
    __key = "AnPSQLite"
    __registered = None

    def __init__(self, anp, connections):
        self.anp = anp
        self.__connections = connections

    def __update_groups(self):
        self.__connections[self.__key].query(
            "delete from _Groups;" +
            "delete from sqlite_sequence where name = '_Groups';" +
            "insert into _Groups(\"session\", \"data\", \"group\", name, description, owner, deleted, date_in, changes) " +
                "select \"session\", id, \"group\", name, description, owner, deleted, date_in, changes from GroupsDataView order by id asc;"
        )

    def __update_groups_users(self):
        self.__connections[self.__key].query(
            "delete from _GroupsUsers;" +
            "delete from sqlite_sequence where name = '_GroupsUsers';" +
            "insert into _GroupsUsers(\"session\", \"data\", \"group\", \"user\", date_in, changes, name) " +
                "select \"session\", id, \"group\", \"user\", date_in, changes, name from GroupsUsersView order by id asc;"
        )

    def __update_groups_ips(self):
        self.__connections[self.__key].query(
            "delete from _GroupsIps;" +
            "delete from sqlite_sequence where name = '_GroupsIps';" +
            "insert into _GroupsIps(\"session\", \"data\", \"group\", ip, date_in, changes, name) " +
                "select \"session\", id, \"group\", ip, date_in, changes, name from GroupsIpsView order by id asc;"
        )

    def __update_plains(self):
        self.__update_groups()
        self.__update_groups_users()
        self.__update_groups_ips()

    def __insert_group(self, name, owner):
        return self.__connections[self.__key].query((
            "insert into Groups default values;" +
            "select last_insert_rowid() as id;" +
            "insert into GroupsData(\"session\", \"group\", name, description, owner) values" +
                "({session}, last_insert_rowid(), {name}, '', {owner});"
        ), {
            "session" : self.anp.procedure_get("users").get_machine(),
            "name" : name,
            "owner" : owner
        })["data"][1]["table"][0]["id"]

    def __insert_group_by_entity(self, id, name, table, column):
        self.__connections[self.__key].query((
            "insert into Groups" + table + "(\"session\", \"group\", " + column + ", belong) values" +
                "({session}, {group}, {entity}, true)"
        ), {
            "session" : self.anp.procedure_get("users").get_machine(),
            "group" : self.__insert_group(name, True),
            "entity" : id
        })

    def get_id(self, name):
        results = self.__connections[self.__key].query("select \"group\" from _Groups where name = {name};", {"name" : name})["data"][0]["table"]
        return results[0]["group"] if len(results) else None

    def build(self):

        if self.__built:
            return
        self.__built = True

        self.__connections[self.__key].execute_sql("/SQLite/AnP.12.groups.lite.sql")
        updated = False
        ids = {}

        for name in ["guest", "bot", "registered", "administrator"]:
            results = self.__connections[self.__key].query("select id from _Groups where name = {name}", {"name" : name})["data"][0]["table"]
            if len(results):
                ids[name] = results[0]["id"]
            else:
                updated = True
                ids[name] = self.__insert_group(name, False)

        self.__registered = ids["registered"]

        for user in self.__connections[self.__key].query("select id, nick from _Users where nick not in (select name from _Groups);")["data"][0]["table"]:
            if user["nick"] != "guest":
                self.__insert_group_by_entity(user["id"], user["nick"], "Users", "\"user\"")
            if user["nick"] not in ["guest", "machine"]:
                self.__connections[self.__key].query((
                    "insert into GroupsUsers(\"session\", \"group\", \"user\", belong) values" +
                        "({session}, {group}, {user}, true);"
                ), {
                    "session" : self.anp.procedure_get("users").get_machine(),
                    "group" : self.__registered,
                    "entity" : user["id"]
                })
            updated = True
        for ip in self.__connections[self.__key].query("select id, address from _Ips where address not in (select name from _Groups);")["data"][0]["table"]:
            self.__insert_group_by_entity(ip["id"], ip["address"], "Ips", "ip")
            updated = True

        if updated:
            self.__update_groups();

        for user, group in {
            "machine" : "bot",
            "guest" : "guest"
        }.items():

            user_id = self.anp.procedure_get("users").get_user_id(user)
            group_id = self.get_id(group)

            if not len(self.__connections[self.__key].query("select 1 from GroupsUsers where \"group\" = {group} and \"user\" = {user} limit 1;", {
                "user" : user_id,
                "group" : group_id
            })["data"][0]["table"]):
                updated = True
                self.__connections[self.__key].query((
                    "insert into GroupsUsers(\"session\", \"group\", \"user\", belong) values" +
                        "({session}, {group}, {user}, true);"
                ), {
                    "session" : self.anp.procedure_get("users").get_machine(),
                    "user" : user_id,
                    "group" : group_id
                })

        if updated:
            self.__update_groups_users()
            self.__update_groups_ips()

    def exists(self, name):
        return bool(len(self.__connections[self.__key].query("select 1 from _Groups where name = {name} limit 1;", {"name" : name})["data"][0]["table"]))

    def register_ip(self, ip, id):

        results = self.__connections[self.__key].query("select \"group\" from _Groups where name = {ip} limit 1;", {"ip" : ip})["data"][0]["table"]
        group = None

        if len(results):
            group = results[0]["group"]
        else:
            group = self.__insert_group(ip, True)
            self.__update_groups()

        if not len(self.__connections[self.__key].query("select 1 from _GroupsIps where \"group\" = {group} and ip = {ip};", {
            "group" : group,
            "ip" : id
        })["data"][0]):
            self.__connections[self.__key].query((
                "insert into GroupsIps(\"session\", \"group\", ip, belong) values" +
                    "({session}, {group}, {ip}, true);"
            ), {
                "session" : self.anp.procedure_get("users").get_machine(),
                "group" : group,
                "ip" : id
            })
            self.__update_groups_ips()

    def register_user(self, user, id):
        results = self.__connections[self.__key].query("select \"group\" from _Groups where name = {user} limit 1;", {"user" : user})["data"][0]["table"]
        group = None

        if len(results):
            group = results[0]["group"]
        else:
            group = self.__insert_group(user, True)
            self.__update_groups()

        if not len(self.__connections[self.__key].query("select 1 from _GroupsUsers where \"group\" = {group} and user = {user};", {
            "group" : group,
            "user" : id
        })["data"][0]["table"]):
            self.__connections[self.__key].query((
                "insert into GroupsUsers(\"session\", \"group\", \"user\", belong) values" +
                    "({session}, {group}, {user}, true)," +
                    "({session}, {registered}, {user}, true);"
            ), {
                "session" : self.anp.procedure_get("users").get_machine(),
                "group" : group,
                "user" : id,
                "registered" : self.__registered
            })
            self.__update_groups_users()

    def register_first_administrator(self, id):
        if not len(self.__connections[self.__key].query("select 1 from _GroupsUsers where name = 'administrator' limit 1;")["data"][0]["table"]):
            self.__connections[self.__key].query((
                "insert into GroupsUsers(\"session\", \"group\", \"user\", belong) values" +
                    "({session}, (select \"group\" from _Groups where name = 'administrator' limit 1), {user}, true);"
            ), {
                "session" : self.anp.procedure_get("users").get_machine(),
                "user" : id
            })
            self.__update_groups_users()

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Procedures"):
    class Procedures:
        pass
    AnP.Procedures = Procedures
    del globals()["Procedures"]
AnP.Procedures.Groups = Anonymous
del globals()["Anonymous"]
