#!/usr/bin/env python
# -*- coding: utf-8 -*-

from traceback import format_tb as format_traceback

class Anonymous: # AnP.Procedures.Logs

    anp = None
    __connections = {}
    __built = False
    __key = "AnPSQLite"

    def __init__(self, anp, connections):
        self.anp = anp
        self.__connections = connections
        self.build()

    def build(self):

        if self.__built:
            return
        self.__built = True

        self.__connections[self.__key].execute_sql("/SQLite/AnP.11.logs.lite.sql")

    def __get_id(self, table, column, value, connection = None):

        if connection == None:
            connection = self.__connections[self.__key].connect()

        id = self.__connections[self.__key].query("select id from " + table + " where date_out is null and ifnull(" + column + ", '') = ifnull({value}, '') limit 1", {"value" : value}, connection)["data"][0]["table"]
        if len(id) == 0:
            id = self.__connections[self.__key].query("insert into " + table + "(" + column + ") values({value})", {"value" : value}, connection)["data"]

        return id[0]["id"]

    def exception(self, method, exception, input = None, error = 0):

        type, message, trace = exception
        connection = self.__connections[self.__key].connect()
        id = self.__connections[self.__key].query((
            "insert into Traces('session', method, 'trace', 'type', message, 'input', error) values" +
                "({session}, {method}, {trace}, {type}, {message}, {input}, {error})"
        ), {
            "session" : self.anp.session_get(),
            "method" : self.__get_id("Methods", "name", method, connection),
            "type" : self.__get_id("Types", "name", type.__name__, connection),
            "message" : self.__get_id("Messages", "message", str(message), connection),
            "trace" : self.__get_id("TracesData", "'data'", [line.strip() for line in format_traceback(trace)], connection),
            "error" : error,
            "input" : input
        }, connection)["data"][0]["id"]

        self.__connections[self.__key].close(connection)

        return id

    def to_print(self, printers):

        ids = []
        connection = self.__connections[self.__key].connect()

        for printer in [] if not printers else printers if AnP.is_list(printers) else [printers]:
            ids.append(self.__connections[self.__key].query((
                "insert into Prints('session', method, 'function', 'type', message, 'input', line, 'date') values" +
                    "({session}, {method}, {function}, {type}, {message}, {data}, {line}, {date})"
            ), {
                "session" : self.anp.session_get(),
                "method" : self.__get_id("Methods", "name", "_print", connection),
                "type" : self.__get_id("Types", "name", printer["type"], connection),
                "message" : self.__get_id("Messages", "message", printer["text"], connection),
                "data" : self.__get_id("JSONData", "'data'", printer["variables"], connection) if "variables" in printer else None,
                "date" : printer["date"],
                "line" : printer["trace"][1].lineno,
                "function" : self.__get_id("Methods", "name", printer["trace"][1].filename + "." + printer["trace"][1].function, connection)
            }, connection)["data"][0]["id"])

        self.__connections[self.__key].close(connection)

        return ids

    def log(self, procedure, input, error):

        connection = self.__connections[self.__key].connect()
        id = self.__connections[self.__key].query("insert into Logs('session', 'method', 'input', error) values({session}, {method}, {input}, {error})", {
            "session" : self.anp.session_get(),
            "method" : self.__get_id("Methods", "name", procedure, connection),
            "input" : self.__get_id("JSONData", "'data'", input, connection) if input else None,
            "error" : error
        }, connection)["data"][0]["id"]

        self.__connections[self.__key].close(connection)

        return id

    def procedure(self, procedure, connection = None):
        return self.__get_id("Methods", "name", procedure, connection)

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Procedures"):
    class Procedures:
        pass
    AnP.Procedures = Procedures
    del globals()["Procedures"]
AnP.Procedures.Logs = Anonymous
del globals()["Anonymous"]
