#!/usr/bin/env python
# -*- coding: utf-8 -*-

from traceback import format_tb as format_traceback

class Anonymous: # AnP.Procedures.Permissions

    anp = None
    __connections = {}
    __built = False
    __key = "AnPSQLite"
    __loaded = False

    def __init__(self, anp, connections):
        self.anp = anp
        self.__connections = connections

    def __update_data(self):
        self.__connections[self.__key].query(
            "delete from _Permissions;" +
            "delete from sqlite_sequence where name = '_Permissions';" +
            "insert into _Permissions(\"session\", \"data\", permission, name, description, deleted, date_in, date_created, changes) " +
                "select \"session\", id, permission, name, description, deleted, date_in, date_created, changes from PermissionsDataView"
        )

    def __update_groups(self):
        self.__connections[self.__key].query(
            "delete from _PermissionsGroups;" +
            "delete from sqlite_sequence where name = '_PermissionsGroups';" +
            "insert into _PermissionsGroups(\"session\", permission_group, permission, \"group\", name, date_in, changes, name) " +
                "select \"session\", id, permission, \"group\", name, date_in, changes, name from PermissionsGroupsView"
        )

    def __update_plains(self):
        self.__update_data()
        self.__update_groups()

    def build(self):

        if self.__built:
            return
        self.__built = True

        self.__connections[self.__key].execute_sql("/SQLite/AnP.14.permissions.lite.sql")
        updated_data = False
        updated_groups = False
        machine = self.anp.procedure_get("users").get_machine()

        for permission in ["guest", "bot", "registered", "administrator"]:

            id = self.__connections[self.__key].query("select permission from _Permissions where name = {permission} limit 1;", {"permission" : permission})["data"][0]["table"]
            group = self.anp.procedure_get("groups").get_id(permission)

            if len(id):
                id = id[0]["permission"]
            else:
                updated_data = True
                id = self.__connections[self.__key].query((
                    "insert into Permissions default values;" +
                    "select last_insert_rowid() as id;" +
                    "insert into PermissionsData(\"session\", permission, name, description) values" +
                        "({session}, last_insert_rowid(), {name}, '');"
                ), {
                    "session" : machine,
                    "name" : permission
                })["data"][1]["table"][0]["id"]
            if not len(self.__connections[self.__key].query("select 1 from PermissionsGroups where date_out is null and permission = {permission} and \"group\" = {group} limit 1;", {
                "permission" : id,
                "group" : group
            })["data"][0]["table"]):
                updated_groups = True
                self.__connections[self.__key].query((
                    "insert into PermissionsGroups(\"session\", permission, \"group\", \"granted\") values" +
                        "({session}, {permission}, {group}, true);"
                ), {
                    "session" : machine,
                    "permission" : id,
                    "group" : group
                })

        if updated_data:
            self.__update_data()
        if updated_groups:
            self.__update_groups()

        self.__loaded = True

    def get_by_session(self, session = None):
        session = self.anp.procedure_get("users").session(session)

        permissions = []

        for permission in self.__connections[self.__key].query("select name from PermissionsBySessionsView where \"session\" = {session};", {"session" : session if session != None else self.anp.procedure_get("users").get_session()})["data"][0]["table"]:
            permissions.append(permission["name"])

        return permissions

    def is_loaded(self):
        return self.__loaded

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Procedures"):
    class Procedures:
        pass
    AnP.Procedures = Procedures
    del globals()["Procedures"]
AnP.Procedures.Permissions = Anonymous
del globals()["Anonymous"]
