#!/usr/bin/env python
# -*- coding: utf-8 -*-

from traceback import format_tb as format_traceback
from re import compile as re_compile
from hashlib import sha256 as sha256

class Anonymous: # AnP.Procedures.Users

    anp = None
    __connections = {}
    __built = False
    __key = "AnPSQLite"
    __machine = None
    __guest = None
    __session = None
    __timeout = 0
    __timeout_cache = 0
    __sessions = {}
    __thread = None

    __re_email = re_compile(r'^[^\@]+\@[^\.]+(\.[^\.]+)*?')

    def __init__(self, anp, connections):
        self.anp = anp
        self.__connections = connections

    def session(self, hash):
        if AnP.is_integer(hash):
            return hash
        if hash and hash in self.__sessions:
            self.__sessions[hash]["time"] = AnP.timestamp_milliseconds()
            return self.__sessions[hash]["id"]
        return None

    def __session_create(self, id, permissions):

        hash = self.anp.hash()

        self.__sessions[hash] = {
            "time" : AnP.timestamp_milliseconds(),
            "id" : id,
            "permissions" : permissions if AnP.is_array(permissions) else [permissions] if permissions else None
        }

        return hash

    def session_permissions(self, id):
        if AnP.is_integer(id):
            for hash, data in self.__sessions.items():
                if hash == id:
                    return data["permissions"]
        elif AnP.is_string(id) and id in self.__sessions:
            return self.__sessions[id]["permissions"]
        return None

    def __sessions_cleaner(self, i, thread):

        time = AnP.timestamp_milliseconds()
        keys = []

        for key, data in self.__sessions.items():
            if time - data["time"] > self.__timeout_cache:
                keys.append(key)

        for key in keys:
            self.anp._print("info", "session_cleaned", {"key" : key})
            del self.__sessions[key]

    def __hash_password(self, password):
        return sha256(password.encode("utf-8")).hexdigest()

    def __update_plains(self, users = True, ips = True):
        if users:
            self.__connections[self.__key].query(
                "delete from _Users;" +
                "delete from sqlite_sequence where name = '_Users';" +
                "insert into _Users('session', 'data', 'user', nick, 'password', accessible, deleted, changes, date_in, date_created) " +
                    "select \"session\", id, \"user\", nick, \"password\", accessible, deleted, changes, date_in, date_created from UsersView;"
            )
        if ips:
            self.__connections[self.__key].query(
                "delete from _Ips;" +
                "delete from sqlite_sequence where name = '_Ips';" +
                "insert into _Ips('session', 'data', ip, address, accessible, deleted, changes, date_in, date_created) " +
                    "select \"session\", id, ip, address, accessible, deleted, changes, date_in, date_created from IpsView;"
            )

    def build(self):

        if self.__built:
            return
        self.__built = True

        self.__connections[self.__key].execute_sql("/SQLite/AnP.10.users.lite.sql")
        self.__timeout = self.anp.settings.get("session_timeout")
        self.__timeout_cache = self.anp.settings.get("session_cache_timeout")

        users = {"machine" : None, "guest" : None}
        ips = {"::1" : None}
        users_create = []
        ips_create = []

        for nick in ["machine", "guest"]:
            users[nick] = self.__connections[self.__key].query("select \"user\" from UsersView where nick = {nick} limit 1;", {
                "nick" : nick
            })["data"][0]["table"]
            if len(users[nick]):
                users[nick] = users[nick][0]["user"]
            else:
                users[nick] = self.__connections[self.__key].query(
                    "insert into Users default values;" +
                    "select last_insert_rowid() as 'id';"
                )["data"][1]["id"]
                users_create.append(nick)
        for address in ["::1"]:
            ips[address] = self.__connections[self.__key].query("select ip from IpsView where address = {address} limit 1;", {
                "address" : address
            })["data"][0]["table"]
            if len(ips[address]):
                ips[address] = ips[address][0]["ip"]
            else:
                ips[address] = self.__connections[self.__key].query(
                    "insert into Ips default values;" +
                    "select last_insert_rowid() as 'id';"
                )["data"][1]["id"]
                ips_create.append(address)

        data_session = {
            "user" : users["machine"],
            "ip" : ips["::1"]
        }

        self.__guest = users["guest"]
        self.__machine = self.__connections[self.__key].query("select id from Sessions where \"user\" = {user} and ip = {ip} limit 1;", data_session)["data"][0]["table"]
        self.__machine = self.__machine[0]["id"] if len(self.__machine) else self.__connections[self.__key].query((
            "insert into Sessions('user', ip) values({user}, {ip});" +
            "select last_insert_rowid() as 'id';"
        ), data_session)["data"][1]["id"]

        for nick, id in users.items():
            if nick in users_create:
                self.__connections[self.__key].query("insert into UsersData('session', 'user', nick, password) values({session}, {user}, {nick}, '');", {
                    "session" : self.__machine,
                    "user" : id,
                    "nick" : nick
                })
        for address, id in ips.items():
            if address in ips_create:
                self.__connections[self.__key].query("insert into IpsData('session', ip, address) values({session}, {ip}, {address});", {
                    "session" : self.__machine,
                    "ip" : id,
                    "address" : address
                })

        self.__update_plains(len(users_create) != 0, len(ips_create) != 0)

        self.__session = self.__machine
        self.anp.session_set(self.__session)
        self.__session = self.create_guest("::1", None, True)["id"]
        self.anp.session_set(self.__session)

        self.__thread = self.anp.threads.add(self.__sessions_cleaner, 1)

    def __get_ip_id(self, ip, session = None, starting = False):

        error = 0
        id = 0

        try:

            error |= AnP.Errors.null_or_empty(ip) << 1

            if not error:

                ip_id = self.__connections[self.__key].query("select ip, accessible from _Ips where address = {ip};", {"ip" : ip})["data"][0]["table"]

                if len(ip_id):
                    error |= 0 if ip_id[0]["accessible"] else 1 << 3
                    if not error:
                        id = ip_id[0]["ip"]
                else:
                    id = self.__connections[self.__key].query((
                        "insert into Ips default values;" +
                        "select last_insert_rowid() as 'id';" +
                        "insert into IpsData('session', ip, address, accessible) values({session}, last_insert_rowid(), {address}, {accessible});"
                    ), {
                        "session" : session if session != None else self.__session if self.__session != None else self.__machine,
                        "address" : ip,
                        "accessible" : self.anp.settings.get("ips_accessible") if len(self.__connections[self.__key].query(
                            "select 1 from _Ips where accessible limit 1;"
                        )["data"][0]["table"]) else True
                    })["data"][1]["id"]
                    if not starting:
                        self.anp.procedure_get("groups").register_ip(ip, id)
                    self.__update_plains(False, True)

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.__get_ip_id", exception_information())

        return (error, id)

    def create_guest(self, ip, session = None, starting = False):
        session = self.session(session)

        error = 0
        id = None
        permissions = None

        if session == None:
            session = self.__session

        try:

            ip_error, ip_id = self.__get_ip_id(ip, session, starting)
            error |= (
                # (self.session_validate(session) << 1) |
                (ip_error << 8) |
                0
            )

            if not error:

                response = self.__connections[self.__key].query((
                    "insert into Sessions(\"session\", \"user\", ip) values({session}, {user}, {ip});" +
                    "select last_insert_rowid() as 'id';"
                ), {
                    "session" : session,
                    "user" : self.__guest,
                    "ip" : ip_id
                })

                error = response["data"][0]["error"] << 10
                id = response["data"][1]["id"]
                if self.anp.procedure_get("permissions").is_loaded():
                    permissions = self.anp.procedure_get("permissions").get_by_session(id)
                else:
                    permissions = ["guest"]
                id = self.__session_create(id, permissions)

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.create_guest", exception_information())

        self.anp.log("AnP.Procedures.Users.create_guest", {
            "session" : session,
            "ip" : ip,
            "id" : id
        }, error)

        return {
            "error" : error,
            "id" : id,
            "permissions" : permissions
        }

    def session_validate(self, session, permissions = None):
        session = self.session(session)

        error = 1 << 0 if session == None else (0 if session > 0 else 1 << 2) if AnP.is_integer(session) else 1 << 1

        if not error:

            response = self.__connections[self.__key].query("select (case \
                when date_out is not null then 1 << 4 \
                when julianday(datetime('now')) - julianday(date_last) > {timeout} then 1 << 5 \
            else 0 end) as error from Sessions where id = {session} limit 1;", {
                "session" : session,
                "timeout" : self.__timeout
            })["data"][0]["table"]

            if len(response):
                error |= response[0]["error"]
            else:
                error |= 1 << 3

            if not error and permissions and len(permissions):

                session_permissions = self.__session_permissions(session)
                has = False

                for permission in permissions if AnP.is_array(permissions) else [permissions]:
                    if permission in session_permissions:
                        has = True
                        break
                if not has:
                    error |= 1 << 6

        return error

    def __user_data_validate(self, name, password):

        id = 0
        error = (
            AnP.Errors.null_or_empty(name) |
            (AnP.Errors.null_or_empty(password) << 2) |
            0
        )

        if not error:

            response = self.__connections[self.__key].query("select \"user\", (case when \"password\" != {password} then 1 << 5 when not accessible then 1 << 6 else 0 end) as 'error' from UsersView where nick = {name} limit 1;", {
                "name" : name,
                "password" : self.__hash_password(password)
            })["data"][0]

            if len(response["table"]):
                error |= response["table"][0]["error"]
                if not error:
                    id = response["table"][0]["user"]
            else:
                error |= 1 << 4

        return (error, id)

    def __new_user_validate(self, name, password):

        error = (
            AnP.Errors.null_or_empty(name) |
            (AnP.Errors.null_or_empty(password) << 2) |
            0
        )

        if not error:

            response = self.__connections[self.__key].query("select 1 from UsersView where nick = {name} limit 1;", {
                "name" : name
            })["data"][0]

            error |= (
                (1 << 4 if len(response["table"]) else 0) |
                (1 << 5 if self.anp.procedure_get("groups").exists(name) else 0) |
                0
            )

        return error

    def __email_validate(self, address):
        return AnP.Errors.email(address)

    def close(self, session = None):
        session = self.session(session)

        error = 0
        _input = {"session" : session}

        if session == None:
            session = self.__session

        try:

            error |= (
                (self.session_validate(session) << 1) |
                0
            )

            if error:
                return error

            response = self.__connections[self.__key].query("select ifnull((select 0 from Sessions where id = {session}), 1) as error;", {"session" : session})["data"][0]

            if len(response):
                error |= response["table"][0]["error"] << 4
                if not error:
                    self.__connections[self.__key].query("update Sessions set date_out = datetime('now') where id = {session};", {"session" : session})
            else:
                error |= 1 << 5

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.close", exception_information(), True, _input, error)

        self.anp.log("AnP.Procedures.Users.close", _input, error)

        return error

    def email(self, session, email):
        session = self.session(session)

        error = 0
        id = 0
        _input = {
            "session" : session,
            "email" : email
        }

        if session == None:
            session = self.__session

        try:

            error |= (
                (self.__email_validate(email) << 1) |
                0
            )

            if not error:

                response = self.__connections[self.__key].query("select email from EmailsView where address = {address} limit 1", {"address" : email})["data"][0]

                if len(response["table"]):
                    id = response["table"][0]["email"]
                else:
                    id = self.__connections[self.__key].query((
                        "insert into Emails default values;" +
                        "select last_insert_rowid();" +
                        "insert into EmailsData(session, email, address) values({session}, last_insert_rowid(), {address});"
                    ), {
                        "session" : session,
                        "address" : email
                    })["data"][1]["id"]

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.email", exception_information(), True, _input, error)

        self.anp.log("AnP.Procedures.Users.email", _input, error)

        return (error, id)

    def create(self, session, name, password, email):
        session = self.session(session)

        error = 0
        id = 0
        _input = {
            "session" : session,
            "name" : name,
            "password" : password,
            "email" : email
        }

        if session == None:
            session = self.__session

        try:

            email_error, email_id = self.email(session, email)
            error |= (
                # (self.session_validate(session) << 1) |
                (self.__new_user_validate(name, password) << 8) |
                (email_error << 15) |
                0
            )

            if not error:

                id = self.__connections[self.__key].query(
                    "insert into Users default values;" +
                    "select last_insert_rowid() as 'id';"
                )["data"][1]["id"]

                self.__connections[self.__key].query((
                    "insert into UsersData(\"session\", \"user\", nick, \"password\", accessible) values({session}, {user}, {name}, {password}, {accessible});" +
                    "insert into UsersEmails(\"session\", \"user\", email, belong) values({session}, {user}, {email}, true);"
                ), {
                    "session" : session,
                    "name" : name,
                    "password" : self.__hash_password(password),
                    "user" : id,
                    "email" : email_id,
                    "accessible" : self.anp.settings.get("users_accessible") if len(self.__connections[self.__key].query(
                        "select 1 from _Users where accessible limit 1;"
                    )["data"][0]["table"]) else True
                })
                self.anp.procedure_get("groups").register_user(name, id)
                self.anp.procedure_get("groups").register_first_administrator(id)

            if not error:
                self.__update_plains(True, False)

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.create", exception_information(), True, _input, error)

        self.anp.log("AnP.Procedures.Users.create", _input, error)

        return {
            "error" : error,
            "id" : id
        }

    def login(self, session, name, password, ip):
        session = self.session(session)

        error = 0
        id = None
        _input = {
            "session" : session,
            "name" : name,
            # "password" : password
        }
        permissions = None

        if not session:
            session = self.__session

        try:

            user_error, user_id = self.__user_data_validate(name, password)
            ip_error, ip_id = self.__get_ip_id(ip, session)
            error |= (
                # (self.session_validate(session) << 1) |
                (user_error << 8) |
                (ip_error << 15) |
                0
            )

            if not error:
                id = self.__connections[self.__key].query((
                    "insert into Sessions(\"session\", \"user\", ip) values({session}, {user}, {ip});" +
                    "select last_insert_rowid() as 'id';"
                ), {
                    "session" : session,
                    "user" : user_id,
                    "ip" : ip_id
                })["data"][1]["id"]
                permissions = self.anp.procedure_get("permissions").get_by_session(id)
                id = self.__session_create(id, permissions)

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.login", exception_information(), True, _input, error)

        self.anp.log("AnP.Procedures.Users.login", _input, error)

        return {
            "error" : error,
            "id" : id,
            "permissions" : permissions
        }

    def logout(self, session):
        session = self.session(session)

        error = 0
        _input = {"session" : session}

        try:

            error = self.session_validate(session) << 1

            if not error:
                self.__connections[self.__key].query("update Sessions set date_out = datetime('now') where id = {session}", _input)

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.logout", exception_information(), True, _input, error)

        self.anp.log("AnP.Procedures.Users.logout", _input, error)

        return error

    def get_machine(self):
        return self.__machine

    def get_user_id(self, nick):
        results = self.__connections[self.__key].query("select \"user\" as id from _Users where nick = {nick};", {"nick" : nick})["data"][0]["table"]
        return results[0]["id"] if len(results) else None

    def get_ip_id(self, address):
        results = self.__connections[self.__key].query("select ip as id from _Ips where address = {address};", {"address" : address})["data"][0]["table"]
        return results[0]["id"] if len(results) else None

    def get_session(self):
        return self.__session

    def refresh(self, session):
        session = self.session(session)

        error = 0
        permissions = None
        name = None
        _input = {"session" : session}

        try:

            error = self.session_validate(session) << 1

            if not error:
                self.__connections[self.__key].query("update Sessions set date_last = datetime('now') where id = {session};", {"session" : session})
                permissions = self.anp.procedure_get("permissions").get_by_session(session)
                name = self.__connections[self.__key].query((
                    "select nick from _Users where \"user\" = (select \"user\" from Sessions where id = {session} limit 1) limit 1;"
                ), {"session" : session})["data"][0]["table"][0]["nick"]

                for hash, data in self.__sessions.items():
                    if data["id"] == session:
                        self.__sessions[hash]["permissions"] = permissions
                        break

        except:
            error |= 1 << 0
            self.anp.exception("AnP.Procedures.Users.refresh", exception_information(), True, _input, error)

        # self.anp.log("AnP.Procedures.Users.refresh", _input, error)

        return {
            "error" : error,
            "permissions" : permissions,
            "name" : name
        }

if "AnP" not in globals():
    class AnP:
        pass
if not hasattr(AnP, "Procedures"):
    class Procedures:
        pass
    AnP.Procedures = Procedures
    del globals()["Procedures"]
AnP.Procedures.Users = Anonymous
del globals()["Anonymous"]
