-- Level 0.
create table if not exists Users(
	id integer not null primary key autoincrement,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

create table if not exists Ips(
	id integer not null primary key autoincrement,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

create table if not exists Emails(
	id integer not null primary key autoincrement,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

-- Level 1.
create table if not exists Sessions(
	id integer not null primary key autoincrement,
	'session' integer,
	'user' integer not null,
	ip integer not null,
	date_in datetime not null default (datetime('now')),
	date_last datetime not null default (datetime('now')),
	date_out datetime,
	constraint sessions_session foreign key('session') references Sessions(id),
	constraint sessions_user foreign key('user') references Users(id),
	constraint sessions_ip foreign key(ip) references Ips(id)
);

-- Level 2.
create table if not exists UsersData(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'user' integer not null,
	nick varchar(32) not null,
	'password' varchar(256) not null,
	accessible boolean not null default (false),
	deleted datetime,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint users_data_session foreign key('session') references Sessions(id),
	constraint users_data_user foreign key('user') references Users(id)
);

create table if not exists IpsData(
	id integer not null primary key autoincrement,
	'session' integer not null,
	ip integer not null,
	address varchar(39) not null, -- IPv4 & IPv6.
	accessible boolean not null default (false),
	deleted datetime,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint ips_data_session foreign key('session') references Sessions(id),
	constraint ips_data_ip foreign key(ip) references Ips(id)
);

create table if not exists EmailsData(
	id integer not null primary key autoincrement,
	'session' integer not null,
	email integer not null,
	address varchar(256) not null,
	deleted datetime,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint emails_data_session foreign key('session') references Sessions(id),
	constraint emails_data_email foreign key(email) references Emails(id)
);

-- Level 3.
create table if not exists UsersEmails(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'user' integer not null,
	email integer not null,
	belong boolean not null,
	constraint users_emails_session foreign key('session') references Sessions(id),
	constraint users_emails_user foreign key('user') references Users(id),
	constraint users_emails_email foreign key(email) references Emails(id)
);

-- Plain tables.
create table if not exists _Users(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'data' integer not null,
	'user' integer not null,
	nick varchar(32) not null,
	'password' varchar(256) not null,
	accessible boolean not null,
	deleted datetime,
	changes integer not null,
	date_in datetime not null,
	date_created datetime not null,
	constraint _users_session foreign key('session') references Sessions(id),
	constraint _users_data foreign key('data') references UsersData(id),
	constraint _users_user foreign key('user') references Users(id)
);

create table if not exists _Ips(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'data' integer not null,
	ip integer not null,
	address varchar(39) not null, -- IPv4 & IPv6.
	accessible boolean not null,
	deleted datetime,
	changes integer not null,
	date_in datetime not null,
	date_created datetime not null,
	constraint _ips_session foreign key('session') references Sessions(id),
	constraint _ips_data foreign key('data') references IpsData(id),
	constraint _ips_user foreign key(ip) references Ips(id)
);

-- Views.
drop view if exists UsersDataView;
create view UsersDataView as
	select
		users_data.id,
		users_data.'session',
		users_data.'user',
		users_data.nick,
		users_data.'password',
		users_data.accessible,
		users_data.date_in,
		users_data.deleted,
		entities.changes
	from UsersData users_data
	join (select
		max(id) as id,
		count(1) as changes
	from UsersData where date_out is null group by "user") entities on users_data.id = entities.id
	where "user" in (select id from Users where date_out is null);

drop view if exists UsersView;
create view UsersView as
	select
		users_data.*,
		users.date_in as date_created
	from Users users
	join UsersDataView users_data on users.id = users_data.'user';

drop view if exists IpsDataView;
create view IpsDataView as
	select
		ips_data.id,
		ips_data.'session',
		ips_data.ip,
		ips_data.address,
		ips_data.accessible,
		ips_data.date_in,
		ips_data.deleted,
		entities.changes
	from IpsData ips_data
	join (select
		max(id) as id,
		count(1) as changes
	from IpsData where date_out is null group by ip) entities on ips_data.id = entities.id
	where ips_data.ip in (select id from Ips where date_out is null);

drop view if exists IpsView;
create view IpsView as
	select
		ips_data.*,
		ips.date_in as date_created
	from Ips ips
	join IpsDataView ips_data on ips_data.ip = ips.id;

drop view if exists EmailsDataView;
create view EmailsDataView as
	select
		emails_data.id,
		emails_data.'session',
		emails_data.email,
		emails_data.address,
		emails_data.date_in,
		emails_data.deleted,
		entities.changes
	from EmailsData emails_data
	join (select
		max(id) as id,
		count(1) as changes
	from EmailsData where date_out is null group by email) entities on emails_data.id = entities.id
	where emails_data.email in (select id from Emails where date_out is null);

drop view if exists EmailsView;
create view EmailsView as
	select
		emails_data.*,
		emails.date_in as date_created
	from Emails emails
	join EmailsDataView emails_data on emails_data.email = emails.id;

drop view if exists SessionsView;
create view SessionsView as
	select
		sessions.id as id,
		users.'user' as 'user',
		users.nick as nick,
		ips.ip as ip,
		ips.address as address,
		sessions.date_in as date_in,
		sessions.date_last as date_last,
		sessions.date_out as date_out,
		cast((julianday(ifnull(sessions.date_out, sessions.date_last)) - julianday(sessions.date_in)) * 86400 as integer) as active
	from Sessions sessions
	join _Users users on sessions.'user' = users.'user'
	join _Ips ips on sessions.ip = ips.ip;
