-- Level 0.
create table if not exists Methods(
	id integer not null primary key autoincrement,
	name varchar(64) not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

create table if not exists JSONData(
	id integer not null primary key autoincrement,
	'data' text not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

create table if not exists TracesData(
	id integer not null primary key autoincrement,
	'data' text not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

create table if not exists Types(
	id integer not null primary key autoincrement,
	name varchar(256) not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

create table if not exists Messages(
	id integer not null primary key autoincrement,
	message text not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

-- Level 2.
create table if not exists Logs(
	id integer not null primary key autoincrement,
	'session' integer,
	'method' integer not null,
	'input' integer,
	error bigint not null default (0),
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint logs_session foreign key('session') references Sessions(id),
	constraint logs_method foreign key('method') references Methods(id),
	constraint logs_input foreign key('input') references JSONData(id)
);

create table if not exists Traces(
	id integer not null primary key autoincrement,
	'session' integer,
	'method' integer not null,
	trace integer not null,
	'type' integer not null,
	message integer not null,
	'input' integer,
	error bigint not null default (0),
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint traces_session foreign key('session') references Sessions(id),
	constraint traces_method foreign key('method') references Methods(id),
	constraint traces_trace foreign key(trace) references TracesData(id),
	constraint traces_type foreign key('type') references Types(id),
	constraint traces_message foreign key(message) references Messages(id),
	constraint traces_input foreign key('input') references JSONData(id)
);

create table if not exists Prints(
	id integer not null primary key autoincrement,
	'session' integer,
	'method' integer not null,
	'function' integer not null,
	'type' integer not null,
	message integer not null,
	'input' integer,
	line integer not null,
	'date' datetime not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint prints_session foreign key('session') references Sessions(id),
	constraint prints_method foreign key('method') references Methods(id),
	constraint prints_function foreign key('function') references Methods(id),
	constraint prints_type foreign key('type') references Types(id),
	constraint prints_message foreign key(message) references Messages(id),
	constraint prints_input foreign key('input') references JSONData(id)
);

-- Views.
drop view if exists LogsView;
create view LogsView as 
	select 
		logs.id as id,
		sessions.id as session_id,
		sessions.'user' as 'user',
		sessions.nick as nick,
		sessions.ip as ip,
		sessions.address as address,
		sessions.date_in as date_in,
		sessions.date_last as date_last,
		sessions.date_out as date_out,
		sessions.active as active,
		methods.id as method_id,
		methods.name as 'method',
		inputs.id as input_id,
		inputs.'data' as 'input',
		logs.error as error,
		logs.date_in as date_in
	from Logs logs
	left join SessionsView sessions on logs.'session' = sessions.id
	join Methods methods on logs.'method' = methods.id
	left join JSONdata inputs on logs.'input' = inputs.id;

drop view if exists TracesView;
create view TracesView as 
	select 
		traces.id as id,
		sessions.id as session_id,
		sessions.'user' as 'user',
		sessions.nick as nick,
		sessions.ip as ip,
		sessions.address as address,
		sessions.date_in as date_in,
		sessions.date_last as date_last,
		sessions.date_out as date_out,
		sessions.active as active,
		methods.id as method_id,
		methods.name as 'method',
		traces_data.id as trace_id,
		traces_data.'data' as trace,
		types.id as type_id,
		types.name as 'type',
		messages.id as message_id,
		messages.message as message,
		inputs.id as input_id,
		inputs.'data' as 'input',
		traces.error as error,
		traces.date_in as date_in
	from Traces traces
	left join SessionsView sessions on traces.'session' = sessions.id
	join Methods methods on traces.'method' = methods.id
	join TracesData traces_data on traces.trace = traces_data.id
	join Types types on traces.'type' = types.id
	join Messages messages on traces.message = messages.id
	left join JSONdata inputs on traces.'input' = inputs.id;

drop view if exists PrintsView;
create view PrintsView as 
	select 
		prints.id as id,
		sessions.id as session_id,
		sessions.'user' as 'user',
		sessions.nick as nick,
		sessions.ip as ip,
		sessions.address as address,
		sessions.date_in as date_in,
		sessions.date_last as date_last,
		sessions.date_out as date_out,
		sessions.active as active,
		methods.id as method_id,
		methods.name as 'method',
		functions.id as function_id,
		functions.name as 'function',
		types.id as type_id,
		types.name as 'type',
		messages.id as message_id,
		messages.message as message,
		inputs.id as input_id,
		inputs.'data' as 'input',
		prints.line as line,
		prints.'date' as 'date',
		prints.date_in as date_in
	from Prints prints
	left join SessionsView sessions on prints.'session' = sessions.id
	join Methods methods on prints.'method' = methods.id
	join Methods functions on prints.'function' = functions.id
	join Types types on prints.'type' = types.id
	join Messages messages on prints.message = messages.id
	left join JSONdata inputs on prints.'input' = inputs.id;