-- Level 0.
create table if not exists Groups(
	id integer not null primary key autoincrement,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

-- Level 2.
create table if not exists GroupsData(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'group' integer not null,
	name varchar(32) not null,
	description varchar(512),
	owner boolean not null default (false),
	deleted datetime,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint groups_data_session foreign key('session') references Sessions(id),
	constraint groups_data_group foreign key('group') references Groups(id)
);

create table if not exists GroupsUsers(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'group' integer not null,
	'user' integer not null,
	belong boolean not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint groups_users_session foreign key('session') references Sessions(id),
	constraint groups_users_user foreign key('user') references Users(id)
);

create table if not exists GroupsIps(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'group' integer not null,
	ip integer not null,
	belong boolean not null,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint groups_ips_session foreign key('session') references Sessions(id),
	constraint groups_ips_ip foreign key(ip) references Ips(id)
);

-- Plains.
create table if not exists _Groups(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'data' integer not null,
	'group' integer not null,
	name varchar(32) not null,
	description varchar(512),
	owner boolean not null,
	deleted datetime,
	date_in datetime not null,
	changes integer not null,
	constraint _groups_session foreign key('session') references Sessions(id),
	constraint _groups_data foreign key('data') references GroupsData(id),
	constraint _groups_group foreign key('group') references Groups(id)
);

create table if not exists _GroupsUsers(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'data' integer not null,
	'group' integer not null,
	'user' integer not null,
	name varchar(32) not null,
	date_in datetime not null,
	changes integer not null,
	constraint _groups_users_session foreign key('session') references Sessions(id),
	constraint _groups_users_data foreign key('data') references GroupsUsers(id),
	constraint _groups_users_group foreign key('group') references Groups(id),
	constraint _groups_users_user foreign key('user') references Users(id)
);

create table if not exists _GroupsIps(
	id integer not null primary key autoincrement,
	'session' integer not null,
	'data' integer not null,
	'group' integer not null,
	ip integer not null,
	name varchar(32) not null,
	date_in datetime not null,
	changes integer not null,
	constraint _groups_ips_session foreign key('session') references Sessions(id),
	constraint _groups_ips_data foreign key('data') references GroupsIps(id),
	constraint _groups_ips_group foreign key('group') references Groups(id),
	constraint _groups_ips_ip foreign key(ip) references Ips(id)
);

-- Views.
drop view if exists GroupsDataView;
create view GroupsDataView as select
	groups.id as id,
	groups."session" as 'session',
	groups."group" as 'group',
	groups.name as name,
	groups.description as description,
	groups.owner as owner,
	groups.deleted as deleted,
	groups.date_in as date_in,
	changes.changes as changes
from GroupsData groups
join (select 
	max(id) as id,
	count(1) as changes
from GroupsData where date_out is null group by "group") changes on groups.id = changes.id;

drop view if exists GroupsUsersView;
create view GroupsUsersView as select 
	groups_users.id as id,
	groups_users."session" as "session",
	groups_users."group" as "group",
	groups_users."user" as "user",
	groups_users.date_in as date_in,
	changes.changes as changes,
	groups_data.name as name
from GroupsUsers groups_users 
join (select 
	max(id) as id,
	count(1) as changes
from GroupsUsers where date_out is null group by "group", "user" having belong) changes on changes.id = groups_users.id
join GroupsDataView groups_data on 
	groups_data.deleted is null and 
	groups_data."group" = groups_users."group";

drop view if exists GroupsIpsView;
create view GroupsIpsView as select 
	groups_ips.id as id,
	groups_ips."session" as "session",
	groups_ips."group" as "group",
	groups_ips.ip as ip,
	groups_ips.date_in as date_in,
	changes.changes as changes,
	groups_data.name as name
from GroupsIps groups_ips 
join (select 
	max(id) as id,
	count(1) as changes
from GroupsIps where date_out is null group by "group", "user" having belong) changes on changes.id = groups_ips.id
join GroupsDataView groups_data on 
	groups_data.deleted is null and 
	groups_data."group" = groups_ips."group";
	
drop view if exists GroupsEntitiesView;
create view GroupsEntitiesView as 
	select "group", "user" as 'user', null as ip, name from _GroupsUsers union all
	select "group", null as 'user', ip as ip, name from _GroupsIps;
	
drop view if exists GroupsBySessionsView;
create view GroupsBySessionsView as select distinct 
	sessions.id as 'session',
	groups_entities."group" as "group",
	groups_entities.name as name
from (select * from Sessions where date_out is null) sessions 
join GroupsEntitiesView groups_entities on (
	(groups_entities."user" not null and groups_entities."user" = sessions."user") or 
	(groups_entities.ip not null and groups_entities.ip = sessions.ip)
);