-- Level 0.
create table if not exists Permissions(
	id integer not null primary key autoincrement,
	date_in datetime not null default (datetime('now')),
	date_out datetime
);

-- Level 2.
create table if not exists PermissionsData(
	id integer not null primary key autoincrement,
	'session' integer not null,
	permission integer not null,
	name varchar(32) not null,
	description varchar(512),
	deleted datetime,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint permissions_data_session foreign key('session') references Sessions(id),
	constraint permissions_data_permission foreign key(permission) references Permissions(id)
);

create table if not exists PermissionsGroups(
	id integer not null primary key autoincrement,
	'session' integer not null,
	permission integer not null,
	'group' integer not null,
	'granted' boolean not null default (true),
	deleted datetime,
	date_in datetime not null default (datetime('now')),
	date_out datetime,
	constraint permissions_groups_session foreign key('session') references Sessions(id),
	constraint permissions_groups_permission foreign key(permission) references Permissions(id),
	constraint permissions_groups_group foreign key('group') references Groups(id)
);

-- Plains.
create table if not exists _Permissions(
	id integer not null primary key autoincrement,
	'data' integer not null,
	'session' integer not null,
	permission integer not null,
	name varchar(32) not null,
	description varchar(512),
	deleted datetime,
	date_in datetime not null,
	date_created datetime not null,
	changes integer not null,
	constraint _permissions_data foreign key('data') references PermissionsData(id),
	constraint _permissions_session foreign key('session') references Sessions(id),
	constraint _permissions_permission foreign key(permission) references Permissions(id)
);

create table if not exists _PermissionsGroups(
	id integer not null primary key autoincrement,
	permission_group integer not null,
	'session' integer not null,
	permission integer not null,
	'group' integer not null,
	name varchar(32) not null,
	date_in datetime not null,
	changes integer not null,
	constraint _permissions_groups_permission_group foreign key(permission_group) references PermissionsGroups(id),
	constraint _permissions_groups_session foreign key('session') references Sessions(id),
	constraint _permissions_groups_permission foreign key(permission) references Permissions(id),
	constraint _permissions_groups_group foreign key('group') references Groups(id)
);

-- Views.
drop view if exists PermissionsDataView;
create view PermissionsDataView as select 
	changes.id as id,
	permissions_data."session" as "session",
	permissions.id as permission,
	permissions_data.name as name,
	permissions_data.description as description,
	permissions_data.deleted as deleted,
	permissions_data.date_in as date_in,
	permissions.date_in as date_created,
	changes.changes as changes
from PermissionsData permissions_data 
join (select 
	max(id) as id,
	count(1) as changes
from PermissionsData where date_out is null group by permission) changes on permissions_data.id = changes.id
join Permissions permissions on 
	permissions.date_out is null and
	permissions.id = permissions_data.permission;
	
drop view if exists PermissionsGroupsView;
create view PermissionsGroupsView as select 
	changes.id as id,
	permissions_groups.id as permission_group,
	permissions_groups."session" as "session",
	permissions_groups.permission as permission,
	permissions_groups."group" as "group",
	permissions.name as name,
	permissions_groups.date_in as date_in,
	changes.changes as changes
from PermissionsGroups permissions_groups
join (select 
	max(id) as id,
	count(1) as changes
from PermissionsGroups where date_out is null group by permission, 'group' 
having deleted is null and "granted") changes on changes.id = permissions_groups.id 
join PermissionsDataView permissions on permissions_groups.permission = permissions.permission;

drop view if exists PermissionsBySessionsView;
create view PermissionsBySessionsView as select 
	groups."session" as "session",
	permissions.permission as permission,
	permissions.name as name
from GroupsBySessionsView groups
join _PermissionsGroups permissions on groups."group" = permissions."group";