if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP
	
if object_id(N'dbo.procedures_get', N'P') is not null drop procedure dbo.procedures_get
go
create procedure dbo.procedures_get
	@database varchar(64),
	@procedure varchar(64),
	@id integer output
as begin 
		
	declare @database_id integer = (select top 1 id from dbo.[Databases] where date_out is null and [name] = @database)

	set nocount on
	
	if @database_id is null begin 
		insert into dbo.[Databases]([name]) values(@database)
		set @database_id = scope_identity() 
	end
		
	set @id = (select top 1 id from dbo.[Procedures] where date_out is null and [database] = @database_id and [name] = @procedure)
	
	if @id is null begin 
		insert into dbo.[Procedures]([database], [name]) values(@database_id, @procedure)
		set @id = scope_identity()  
	end
		
end
go
	
if object_id(N'dbo.errors_get', N'P') is not null drop procedure dbo.errors_get
go
create procedure dbo.errors_get
	@error varchar(512),
	@id integer output
as begin 

	set nocount on

	set @id = (select top 1 id from dbo.Errors where date_out is null and code = @error)
	
	if @id is null begin 
		insert into dbo.Errors(code) values(@error)
		set @id = scope_identity() 
	end

end
go
	
if object_id(N'dbo.parameters_get', N'P') is not null drop procedure dbo.parameters_get
go
create procedure dbo.parameters_get
	@parameters varchar(512),
	@id integer output
as begin 

	set nocount on
		
	set @id = (select top 1 id from dbo.[Parameters] where date_out is null and [json] = @parameters)
	
	if @id is null begin 
		insert into dbo.[Parameters]([json]) values(@parameters)
		set @id = scope_identity() 
	end
		
end
go
	
if object_id(N'dbo.messages_get', N'P') is not null drop procedure dbo.messages_get
go
create procedure dbo.messages_get
	@message varchar(512),
	@id integer output
as begin 

	set nocount on
		
	set @id = (select top 1 id from dbo.[Messages] where date_out is null and [text] = @message)
	
	if @id is null begin 
		insert into dbo.[Messages]([text]) values(@message)
		set @id = scope_identity() 
	end
		
end
go
	
if object_id(N'dbo.projects_get', N'P') is not null drop procedure dbo.projects_get
go
create procedure dbo.projects_get
	@name varchar(64),
	@id integer output
as begin 

	set nocount on
		
	set @id = (select top 1 id from dbo.Projects where date_out is null and [name] = @name)
	
	if @id is null begin 
		insert into dbo.Projects([name]) values(@name)
		set @id = scope_identity() 
	end
		
end
go
	
if object_id(N'dbo.traces_files_get', N'P') is not null drop procedure dbo.traces_files_get
go
create procedure dbo.traces_files_get
	@path varchar(512),
	@id integer output
as begin 

	set nocount on
		
	set @id = (select top 1 id from dbo.TracesFiles where date_out is null and [path] = @path)
	
	if @id is null begin 
		insert into dbo.TracesFiles([path]) values(@path)
		set @id = scope_identity() 
	end
		
end
go
	
if object_id(N'dbo.lines_get', N'P') is not null drop procedure dbo.lines_get
go
create procedure dbo.lines_get
	@trace varchar(max),
	@id integer output
as begin 

	set nocount on
		
	set @id = (select top 1 id from dbo.Lines where date_out is null and trace = @trace)
	
	if @id is null begin 
		insert into dbo.Lines(trace) values(@trace)
		set @id = scope_identity() 
	end
		
end
go

if object_id(N'dbo.bw', N'FN') is not null drop function dbo.bw
go
create function dbo.bw(
	@number bigint,
	@bits smallint
) returns bigint begin 
	return (case 
		when @bits > 0 then @number * power(cast(2 as bigint), @bits)
		when @bits < 0 then @number / power(cast(2 as bigint), -@bits)
	else @number end)
end
go
	
if object_id(N'dbo.settings_get', N'FN') is not null drop function dbo.settings_get
go
create function dbo.settings_get(
	@name varchar(64),
	@nulls bit
) returns varchar(max) begin 
		
	declare @value varchar(max) = (select top 1 [value] from dbo._Settings where deleted is null and [name] = @name)
	
	if @value is null and @nulls = 0 
		set @value = (select top 1 [value] from dbo._Settings where deleted is null and [name] = 'default_value')
	
	return @value
end
go
	
if object_id(N'dbo.string_to_integer', N'FN') is not null drop function dbo.string_to_integer
go
create function dbo.string_to_integer(
	@string varchar(max)
) returns integer begin 
	return (case when @string like '[^0-9]' then null else cast(@string as integer) end)
end
go
	
if object_id(N'dbo.settings_get_int', N'FN') is not null drop function dbo.settings_get_int
go
create function dbo.settings_get_int(
	@name varchar(64),
	@nulls bit
) returns integer begin 
	return dbo.string_to_integer(dbo.settings_get(@name, @nulls))
end
go
	
if object_id(N'dbo.string_to_bool', N'FN') is not null drop function dbo.string_to_bool
go
create function dbo.string_to_bool(
	@string varchar(max)
) returns bit begin 

	set @string = lower(@string)

	return (case 
		when @string in ('1', 'true', 'yes') then 1
		when @string in ('0', 'false', 'no') then 0
	else null end)
end
go
	
if object_id(N'dbo.settings_get_bool', N'FN') is not null drop function dbo.settings_get_bool
go
create function dbo.settings_get_bool(
	@name varchar(64),
	@nulls bit
) returns bit begin 
	return dbo.string_to_bool(dbo.settings_get(@name, @nulls))
end
go
	
if object_id(N'dbo.sessions_get_machine', N'FN') is not null drop function dbo.sessions_get_machine
go
create function dbo.sessions_get_machine() returns integer begin 
	return isnull(dbo.settings_get_int('machine_session', 0), (select top 1 id from dbo.[Sessions] where [user] = (
		select top 1 entity from dbo._Users where deleted is null and nick = dbo.settings_get('machine_user', 0)
	)))
end
go
	
if object_id(N'dbo.sessions_get_unknown', N'FN') is not null drop function dbo.sessions_get_unknown
go
create function dbo.sessions_get_unknown() returns integer begin 
	return isnull(dbo.settings_get_int('unknown_session', 0), (select top 1 id from dbo.[Sessions] where [user] = (
		select top 1 entity from dbo._Users where deleted is null and nick = dbo.settings_get('unknown_user', 0)
	)))
end
go
	
if object_id(N'dbo.logs_set', N'P') is not null drop procedure dbo.logs_set
go
create procedure dbo.logs_set
	@session integer,
	@database varchar(64),
	@procedure varchar(64),
	@ip integer,
	@user_agent integer, 
	@error varchar(512),
	@parameters varchar(max),
	@level tinyint
as begin 
	if @level is null or isnull(dbo.settings_get_int('logs_level', 0), ~-dbo.bw(1, 3)) & dbo.bw(1, @level) != 0 begin 
		
		declare @procedure_id integer
		declare @error_id integer
		declare @parameters_id integer
			
		execute dbo.procedures_get @database, @procedure, @procedure_id output
		execute dbo.errors_get @error, @error_id output
		execute dbo.parameters_get @parameters, @parameters_id output
		
		insert into dbo.Logs([procedure], [session], [ip], user_agent, own_error, own_parameters) values
			(@procedure_id, isnull((
				select top 1 id from dbo.[Sessions] where id = @session
			), dbo.sessions_get_unknown()), @ip, @user_agent, @error_id, @parameters_id)
			
	end
end
go
	
if object_id(N'dbo.exceptions_set', N'P') is not null drop procedure dbo.exceptions_set
go
create procedure dbo.exceptions_set
	@session integer,
	@database varchar(64),
	@procedure varchar(64),
	@ip integer,
	@user_agent integer, 
	@error varchar(512),
	@parameters varchar(max)
as begin 
	
	declare @message varchar(max) = error_message()
	declare @status varchar(16) = error_state()
	declare @code integer = error_number()
	declare @procedure_id integer
	declare @error_id integer
	declare @parameters_id integer
	declare @message_id integer
		
	execute dbo.procedures_get @database, @procedure, @procedure_id output
	execute dbo.errors_get @error, @error_id output
	execute dbo.parameters_get @parameters, @parameters_id output
	execute dbo.messages_get @message, @message_id output
	
	insert into dbo.Exceptions([procedure], [session], [ip], user_agent, own_error, own_parameters, [message], [status], code) values
		(@procedure_id, isnull((
			select top 1 id from dbo.[Sessions] where id = @session
		), dbo.sessions_get_unknown()), @ip, @user_agent, @error_id, @parameters_id, @message_id, @status, @code)
		
end
go

if object_id(N'dbo.common_tables_fill', N'P') is not null drop procedure dbo.common_tables_fill
go
create procedure dbo.common_tables_fill
	@session varchar(max)
as begin 
		
	declare @database varchar(64) = isnull(dbo.settings_get('database_anp', 0), 'AnP')
	declare @procedure varchar(64) = 'common_tables_fill'
	declare @procedure_id integer
	declare @machine varchar(32) = isnull(dbo.settings_get('machine_user', 0), 'machine')
	declare @unknown varchar(32) = isnull(dbo.settings_get('unknown_user', 0), 'unknown')
	declare @local_ip varchar(39) = isnull(dbo.settings_get('local_ip', 0), '::1')
	declare @guest varchar(32) = isnull(dbo.settings_get('guest_user', 0), 'guest')
	declare @user_agent varchar(max) = isnull(dbo.settings_get('user_agent', 0), 'AnP Machine DB Server')
	declare @machine_id integer = (select top 1 entity from dbo._Users where deleted is null and nick = @machine)
	declare @unknown_id integer = (select top 1 entity from dbo._Users where deleted is null and nick = @unknown)
	declare @guest_id integer = (select top 1 entity from dbo._Users where deleted is null and nick = @guest)
	declare @local_ip_id integer = (select top 1 entity from dbo._Ips where deleted is null and [address] = @local_ip)
	declare @machine_session integer = (select top 1 id from dbo.[Sessions] where [user] = @machine_id order by id asc)
	declare @unknown_session integer = (select top 1 id from dbo.[Sessions] where [user] = @unknown_id order by id asc)
	declare @user_agent_id integer = (select top 1 id from dbo.UsersAgents where [data] = @user_agent order by id asc)
	declare @machine_add bit = (case when @machine_id is null then 1 else 0 end)
	declare @local_ip_add bit = (case when @local_ip_id is null then 1 else 0 end)
	declare @done bit = 0
	declare @type varchar(32)
	declare @name varchar(64)
	declare @range integer
	declare @value varchar(max)
	declare @description varchar(max)
	declare @type_id integer
	
	if object_id(N'tempdb..#default_settings_temporary_anp', N'U') is not null drop table #default_settings_temporary_anp
	create table #default_settings_temporary_anp(
		id integer not null identity(1, 1),
		[type] varchar(32) not null,
		[name] varchar(64) not null,
		[range] integer not null default 1,
		[value] varchar(max),
		[description] varchar(max),
		date_in datetime not null default getdate(),
		date_out datetime,
		constraint default_settings_temporary_anp_id primary key(id)
	)
	
	execute dbo.procedures_get @database, @procedure, @procedure_id output
	
	insert into #default_settings_temporary_anp([type], [name], [range], [value], [description]) values
		('varchar', 'database_anp', dbo.bw(1, 0), @database, 'Default AnP database name.'),
		('varchar', 'machine_user', dbo.bw(1, 0), @machine, 'Default AnP machine Bot user.'),
		('varchar', 'unknown_user', dbo.bw(1, 0), @unknown, 'Default AnP unknown user.'),
		('varchar', 'guest_user', dbo.bw(1, 0), @guest, 'Default AnP guest name user.'),
		('varchar', 'local_ip', dbo.bw(1, 0), @local_ip, 'Default local IP address.'),
		('varchar', 'base64_alphabet', dbo.bw(1, 0), 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/', 'Base64 alphabet without "=" fill character.'),
		('varchar', 'default_value', dbo.bw(1, 0), null, 'Default value for unknown settings items.'),
		('integer', 'logs_level', dbo.bw(1, 0), concat('', ~-dbo.bw(1, 3)), 'Default Logs level.'),
		('integer', 'default_integer_value', dbo.bw(1, 0), null, 'Default integer value for unkown settings items.'),
		('json', 'users_excluded', dbo.bw(1, 0), '["' + @machine + '","' + @unknown + '","' + @guest + '"]', 'Default excluded users for sessions validations.'),
		('json', 'ips_excluded', dbo.bw(1, 0), '[]', 'Default excluded IPs for sessions validations.'),
		('integer', 'session_timeout', dbo.bw(1, 1), '300', 'The session timeout for inactivity time in seconds.'),
		('boolean', 'allow_any_new_ip', dbo.bw(1, 0), 'true', 'Verifies if any new IP is allowed to access to application.'),
		('boolean', 'allow_any_new_user', dbo.bw(1, 0), 'true', 'Verifies if any new user is allowed to access to application.'),
		('integer', 'password_minimum_length', dbo.bw(1, 0), '4', 'The minimum password length.'),
		('boolean', 'password_needs_lower', dbo.bw(1, 0), 'true', 'Verifies if password needs at least 1 lower case.'),
		('boolean', 'password_needs_upper', dbo.bw(1, 0), 'true', 'Verifies if password needs at least 1 upper case.'),
		('boolean', 'password_needs_digit', dbo.bw(1, 0), 'true', 'Verifies if password needs at least 1 digit.'),
		('boolean', 'password_needs_special', dbo.bw(1, 0), 'true', 'Verifies if password needs at least 1 special character.'),
		('boolean', 'mail_allow_reuse', dbo.bw(1, 0), 'false', 'Verifies if allow reuse the E-Mails.'),
		('varchar', 'password_hash_function', dbo.bw(1, 0), 'sha2', 'Hash method for hashing passwords.'),
		('integer', 'hash_sha2_length', dbo.bw(1, 0), '512', 'Length for SHA2 function Hash.'),
		('integer', 'ips_maximum_sessions', dbo.bw(1, 1), '10', 'The maximum number of simultaneous actived sessions per IP.'),
		('integer', 'users_maximum_sessions', dbo.bw(1, 1), '1', 'The maximum number of simultaneous actived sessions per user.'),
		('boolean', 'session_check_ip', dbo.bw(1, 1), 'true', 'Verifies if session checks its IP address.'),
		('boolean', 'session_check_user_agent', dbo.bw(1, 1), 'true', 'Verifies if session checks its client User Agent.'),
		('varchar', 'default_user_agent', dbo.bw(1, 0), @user_agent, 'Default User Agent for internal Bot sessions.'),
		('integer', 'user_login_bad_times', dbo.bw(1, 1), '5', 'Default user Login tries before lock account temporary for lock brute force attack.'),
		('integer', 'user_lock_time', dbo.bw(1, 1), '900', 'Default user Login lock time for lock brute force attack before tries.'),
		('integer', 'ip_login_bad_times', dbo.bw(1, 1), '20', 'Default IP Login tries before lock account temporary for lock brute force attack.'),
		('integer', 'ip_lock_time', dbo.bw(1, 1), '3600', 'Default IP Login lock time for lock brute force attack before tries.'),
		('integer', 'users_passwords_differents', dbo.bw(1, 0), '3', 'Last differents passwords for user password changing.')
	
	if @machine_add = 1 begin
		insert into dbo.Users default values
		set @machine_id = scope_identity()
	end
	
	if @user_agent_id is null begin 
		insert into dbo.UsersAgents([data]) values(@user_agent)
		set @user_agent_id = scope_identity()
	end
	
	if @local_ip_add = 1 begin 
		insert into dbo.Ips default values
		set @local_ip_id = scope_identity()
	end
	
	if @machine_session is null begin 
		insert into dbo.[Sessions]([user]) values(@machine_id)
		set @machine_session = scope_identity()
		insert into dbo.SessionsIps default values
		insert into dbo.SessionsIpsData(entity, [session], ip, ok) values 
			(scope_identity(), @machine_session, @local_ip_id, 1)
		insert into dbo.SessionsUsersAgents default values
		insert into dbo.SessionsUsersAgentsData(entity, [session], user_agent, ok) values 
			(scope_identity(), @machine_session, @user_agent_id, 1)
	end
	
	if @machine_add = 1  
		insert into dbo.UsersData([session], [procedure], entity, nick, [password], accessible) values
			(@machine_session, @procedure_id, @machine_id, @machine, '', 0)
	
	if @local_ip_add = 1  
		insert into dbo.IpsData([session], [procedure], entity, [address]) values
			(@machine_session, @procedure_id, @local_ip_id, @local_ip)
	
	if @guest_id is null begin 
		insert into dbo.Users default values
		insert into dbo.UsersData([session], [procedure], entity, nick, [password], accessible) values
			(@machine_session, @procedure_id, scope_identity(), @guest, '', 0)
	end
	
	if @unknown_id is null begin 
		insert into dbo.Users default values
		set @unknown_id = scope_identity()
		insert into dbo.UsersData([session], [procedure], entity, nick, [password], accessible) values
			(@machine_session, @procedure_id, @unknown_id, @unknown, '', 0)
	end
	if @unknown_session is null begin 
		insert into dbo.[Sessions]([user]) values(@unknown_id)
		set @unknown_session = scope_identity()
		insert into dbo.SessionsIps default values
		insert into dbo.SessionsIpsData(entity, [session], ip, ok) values 
			(scope_identity(), @unknown_session, @local_ip_id, 1)
		insert into dbo.SessionsUsersAgents default values
		insert into dbo.SessionsUsersAgentsData(entity, [session], user_agent, ok) values 
			(scope_identity(), @unknown_session, @user_agent_id, 1)
	end
	
	insert into #default_settings_temporary_anp([type], [name], [range], [value], [description]) values 
		('integer', 'machine_session', dbo.bw(1, 0), concat('', @machine_session), 'Default machine session ID.'),
		('integer', 'unknown_session', dbo.bw(1, 0), concat('', @unknown_session), 'Default unknown user session ID.')
			
	declare default_settings_cursor cursor local for select [type], [name], [range], [value], [description] from #default_settings_temporary_anp
		
	open default_settings_cursor
	while 1 = 1 begin 
				
		fetch next from default_settings_cursor into @type, @name, @range, @value, @description
		if @@fetch_status != 0
			break
			
		set @type_id = (select top 1 id from dbo._SettingsTypes where deleted is null and [name] = @type)
			
		if @type_id is null begin 
			insert into dbo.SettingsTypes default values
			set @type_id = scope_identity()
			insert into dbo.SettingsTypesData([session], [procedure], entity, [name]) values 
				(@machine_session, @procedure_id, @type_id, @type)
		end
			
		if (select top 1 1 from dbo._Settings where deleted is null and [name] = @name) is null begin 
			insert into dbo.Settings default values
			insert into dbo.SettingsData([procedure], [session], entity, [type], [name], [range], [value], [description]) values 
				(@procedure_id, @machine_session, scope_identity(), @type_id, @name, @range, @value, @description)
		end
			
	end
	close default_settings_cursor
	deallocate default_settings_cursor
	
	drop table #default_settings_temporary_anp
		
end
go

execute dbo.common_tables_fill null
	
if object_id(N'dbo.SessionsFullView', N'V') is not null drop view dbo.SessionsFullView
go
create view dbo.SessionsFullView as select 
	[sessions].id as id,
	users.entity as [user_id],
	users.nick as nick,
	users.accessible as user_accessible,
	users.deleted as user_deleted, 
	ips.entity as ip_id,
	ips.[address] as [ip],
	ips.accessible as ip_accessible, 
	ips.deleted as ip_deleted, 
	users_agents.id as user_agent_id,
	users_agents.[data] as user_agent,
	[sessions].date_in as date_in, 
	[sessions].date_out as date_out,
	[sessions].date_last as date_last
from dbo.[Sessions] [sessions] 
left join dbo._Users users on [sessions].[user] = users.entity 
left join dbo._SessionsIps sessions_ips on sessions_ips.[session] = [sessions].id 
left join dbo._Ips ips on sessions_ips.[ip] = ips.entity
left join dbo._SessionsUsersAgents sessions_users_agents on sessions_users_agents.[session] = [sessions].id 
left join dbo.UsersAgents users_agents on sessions_users_agents.user_agent = users_agents.id
go
	
if object_id(N'dbo.ProceduresFullView', N'V') is not null drop view dbo.ProceduresFullView
go
create view dbo.ProceduresFullView as select 
	[procedures].id as id,
	[databases].id as database_id,
	[databases].[name] as [database],
	[procedures].[name] as [procedure],
	[databases].date_in as database_date_in,
	[procedures].date_in as procedure_date_in,
	[databases].date_out as database_date_out,
	[procedures].date_out as procedure_date_out
from dbo.[Procedures] [procedures] 
join dbo.[Databases] [databases] on [procedures].[database] = [databases].id
go

if object_id(N'dbo.where_set', N'FN') is not null drop function dbo.where_set
go
create function dbo.where_set(
	@new_where varchar(max),
	@old_where varchar(max)
) returns varchar(max) begin 
	return (case when isnull(@new_where, '') = '' then isnull(@old_where, '') else (case when isnull(@old_where, '') = '' then @new_where else '(' + @new_where + ') and (' + @old_where + ')' end) end)
end
go
	
if object_id(N'dbo.tables_list', N'P') is not null drop procedure dbo.tables_list
go
create procedure tables_list(
	@view varchar(max),
	@page integer,
	@items_per_page integer,
	@where varchar(max),
	@order_by varchar(max),
	@has_errors bit,
	@pages integer output,
	@items integer output,
	@results integer output
) as begin 
		
	declare @item_from integer
	declare @query nvarchar(max)
	declare @where_string varchar(max)

	set nocount on

	if @has_errors = 1
		set @where = dbo.where_set('0 = 1', @where)
	set @where_string = (case when isnull(@where, '') = '' then '' else ' where ' + @where end)

	set @query = N'select top 1 @subitems = count(1) from ' + @view + @where_string

	execute sp_executesql @query, N'@subitems integer output', @subitems = @items output

	set @pages = ceiling(@items / cast(@items_per_page as float))
			
	if @page < 1 
		set @page = 1
	else if @pages > 0 and @page > @pages
		set @page = @pages

	set @query = (
		N'select * from ' + @view + @where_string + 
		' order by ' + (case when isnull(@order_by, '') = '' then 'id asc' else @order_by end) + 
		' offset ' + convert(varchar(14), (@page - 1) * @items_per_page) + ' rows fetch next ' + convert(varchar(14), @items_per_page) + ' rows only'
	)

	execute sp_executesql @query
	set @results = @@rowcount
	
end
go

if object_id(N'dbo.machine_get_session_id', N'FN') is not null drop function dbo.machine_get_session_id
go
create function dbo.machine_get_session_id() returns integer begin 
	return dbo.settings_get_int('machine_session', 0)
end
go
	
-- Las dos siguientes vistas est�n puestas por culpa de las banderas de la I18N.
if object_id(N'dbo.FilesSingleView', N'V') is not null drop view dbo.FilesSingleView
go
create view dbo.FilesSingleView as select 
	null as id,
	null as [type],
	null as [name],
	null as [weight]
go
		
if object_id(N'dbo.FilesMultiView', N'V') is not null drop view dbo.FilesMultiView
go
create view dbo.FilesMultiView as select 
	null as id,
	null as [type_id],
	null as [type],
	null as [name],
	null as [weight],
	null as date_in
go
		
if object_id(N'dbo.FilesFullView', N'V') is not null drop view dbo.FilesFullView
go
create view dbo.FilesFullView as select 
	null as id,
	null as database_id,
	null as [database],
	null as procedure_id,
	null as [procedure],
	null as [session],
	null as [user_id],
	null as nick,
	null as ip_id,
	null as [ip],
	null as [type_id],
	null as [type],
	null as [name],
	null as [weight],
	null as date_in
go
		
if object_id(N'dbo.LanguagesMultiView', N'V') is not null drop view dbo.LanguagesMultiView
go
create view dbo.LanguagesMultiView as select 
	null as id,
	null as country_id,
	null as country,
	null as country_key,
	null as country_code, 
	null as flag_id,
	null as flag_type,
	null as flag_name,
	null as flag_weight, 
	null as [name],
	null as [key],
	null as code,
	null as date_in
go

if object_id(N'dbo.TerminalView', N'V') is not null drop view dbo.TerminalView
go
create view dbo.TerminalView as select 
	terminal.id as id,
	sessions.id as session_id,
	sessions.[user_id] as user_id,
	sessions.nick as nick,
	sessions.user_accessible as user_accessible,
	sessions.user_deleted as user_deleted, 
	sessions.ip_id as ip_id,
	sessions.[ip] as ip,
	sessions.ip_accessible as ip_accessible, 
	sessions.ip_deleted as ip_deleted, 
	sessions.user_agent_id as user_agent_id,
	sessions.user_agent as user_agent, 
	sessions.date_in as session_date_in, 
	sessions.date_out as session_date_out,
	sessions.date_last as session_date_last,
	[procedures].id as procedure_id,
	[procedures].id as database_id,
	[procedures].[database] as [database],
	[procedures].[procedure] as [procedure],
	[procedures].database_date_in as database_date_in,
	[procedures].procedure_date_in as procedure_date_in,
	[data].id as data_id,
	[data].[data] as [data],
	[data].date_in as data_date_in,
	terminal.date_in as date_in
from dbo.Terminal terminal 
left join dbo.SessionsFullView sessions on terminal.[session] = sessions.id 
left join dbo.ProceduresFullView [procedures] on terminal.[procedure] = procedures.id 
join dbo.TerminalData [data] on terminal.[data] = [data].id 
where terminal.date_out is null and [data].date_out is null
go

if object_id(N'dbo.terminal_set', N'P') is not null drop procedure dbo.terminal_set
go
create procedure dbo.terminal_set 
	@session integer,
	@database varchar(64),
	@procedure varchar(64), 
	@data varchar(max)
as begin 
	
	declare @data_id integer = (select top 1 id from dbo.TerminalData where [data] = @data)
	declare @procedure_id integer = null

	if isnull(@database, '') != '' and isnull(@procedure, '') != ''  
		execute dbo.procedures_get @database, @procedure, @procedure_id output

	if isnull(@session, 0) < 1 or (select top 1 1 from dbo.Sessions where id = @session) is null 
		set @session = null

	if @data_id is null begin 
		insert into dbo.TerminalData([data]) values(@data)
		set @data_id = scope_identity()
	end

	insert into dbo.Terminal([session], [procedure], [data]) values(@session, @procedure_id, @data_id)
	
end
go

if object_id(N'dbo.terminal_clean', N'P') is not null drop procedure dbo.terminal_clean
go
create procedure dbo.terminal_clean as begin 
	
	delete from dbo.Terminal where 1 = 1
	delete from dbo.TerminalData where 1 = 1

end
go

-- https://en.wikipedia.org/wiki/List_of_URI_schemes

if object_id(N'dbo.is_url', N'FN') is not null drop function dbo.is_url
go
create function dbo.is_url(
	@url varchar(256)
) returns bit begin 
	if @url is null 
		return 0
	if substring(@url, 1, 2) = '//'
		return 1

	declare @i integer = charindex(':', @url)

	if @i = 0 
		return 0

	declare @protocol varchar(16) = substring(@url, 1, @i - 1)


	if @protocol in (
		'aaas', 'coaps', 'ircs', 'ldaps', 'caps', 'ftps', 'https', 'nfss', 'sips', 'stuns', 
		'svns', 'turns', 'wss'
	) 
		set @protocol = substring(@protocol, 1, len(@protocol) - 1)

	if @protocol in (
		'aaa', 'acap', 'adiumxtra', 'aw', 'beshare', 'bolo', 'chrome', 'chrome-extension',
		'coap', 'content', 'crid', 'cvs', 'dict', 'ed2k', 'facetime', 'file', 'finger', 
		'fish', 'git', 'gizmoproject', 'go', 'gopher', 'hcp', 'imap', 'irc', 'irc6',
		'keyparc', 'lastfm', 'ldap', 'market', 'mms', 'ms-help', 'ni', 'nntp', 'notes', 
		'payto', 'platform', 'cap', 'ftp', 'http', 'nfs', 'pop', 'psyc', 'reload', 'res',
		'resource', 'rmi', 'rsync', 'rtmfp', 'rtmp', 's3', 'secondlife', 'sftp', 'sgn', 
		'smb', 'snmp', 'soldat', 'ssh', 'svn', 'udp', 'unreal', 'ut2004', 'ventrilo', 
		'vnc', 'wais', 'webcal', 'wtai', 'xri', 'z39.50r', 'z39.50s', 'app', 'rdar',
		'slack', 'stratum+tcp', 'viber', 'zoommtg', 'snv+ssh', 'teamspeak'
	) 
		return (case when substring(@url, @i + 1, 2) = '//' then 1 else 0 end)
	return (case when @protocol in (
		'afp', 'aim', 'attachment', 'amss', 'barion', 'bitcoin', 'callto', 'cid', 'dab', 
		'data', 'dav', 'dns', 'doi'', drm', 'example', 'fax', 'feed', 'fm', 'geo', 'gg', 
		'gtalk', 'h323', 'iax', 'im', 'itms', 'jar', 'magnet', 'mailto', 'maps', 'message',
		'mid', 'msnim', 'mumble', 'mvn', 'news', 'openpgp4fpr', 'palm', 'paparazzi', 
		'press', 'proxy', 'query', 'session', 'shc', 'sip', 'sms', 'spotify', 'steam', 
		'stun', 'tag', 'tel', 'things', 'turn', 'urn', 'view-source', 'ws', 'wyciwyg',
		'xfire', 'xmpp', 'ymsgr', 'admin', 'freeplane', 'jdbc', 'msteams', 'trueconf',
		'javascript'
	) then 1 else 0 end)
end
go
