if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP
	
if object_id(N'dbo.errors_to_integer', N'FN') is not null drop function dbo.errors_to_integer
go
create function dbo.errors_to_integer(
	@code varchar(2048)
) returns bigint begin
		
	declare @alphabet char(64) = dbo.settings_get('base64_alphabet', 0)
	declare @results bigint = 0
	
	if substring(@code, 1, 1) = '['  
		set @results = (select top 1 sum(dbo.bw([value], [key] * 6)) from openjson(@code, '$') where [key] < 10 group by [type])
	else begin
	
		declare @i tinyint = 0
		declare @l smallint = len(@code)

		while @i < @l and @i < 10 begin 
			set @i = @i + 1
			set @results = @results + dbo.bw(charindex(substring(@code, @i, 1), @alphabet collate latin1_general_cs_as) - 1, (@i - 1) * 6)
		end

	end
	
	return @results
end
go
	
if object_id(N'dbo.errors_to_string', N'FN') is not null drop function dbo.errors_to_string
go
create function dbo.errors_to_string(
	@code varchar(2048)
) returns varchar(512) begin
		
	declare @alphabet char(64) = dbo.settings_get('base64_alphabet', 0)
	declare @results varchar(512) = ''
	
	if substring(@code, 1, 1) = '[' 
		set @results = (select top 1 substring(@alphabet, [value] + 1, 1) from openjson(@code, '$') for xml path(''))
	else begin 
		
		declare @integer bigint = convert(bigint, @code)

		while @integer != 0 begin
			set @results = @results + substring(@alphabet, (@integer & 0x3F) + 1, 1)
			set @integer = dbo.bw(@integer, -6)
		end

	end
	
	return (case when @results = '' then substring(@alphabet, 1, 1) else @results end)
end
go

if object_id(N'dbo.errors_to_array', N'FN') is not null drop function dbo.errors_to_array
go 
create function dbo.errors_to_array(
	@code varchar(512)
) returns varchar(2048) begin 

	declare @alphabet varchar(64) = dbo.settings_get('base64_alphabet', null)
	declare @l integer = len(@code) + 1
	declare @i integer = 1
	declare @results varchar(2048) = ''

	while @i < @l begin 
		set @results = @results + ',' + convert(varchar(4), charindex(substring(@code, @i, 1), @alphabet collate latin1_general_cs_as) - 1)
		set @i = @i + 1
	end

	return '[' + substring(@results, 2, 2048) + ']'
end
go

if object_id(N'dbo.errors_to_array_back', N'FN') is not null drop function dbo.errors_to_array_back
go 
create function dbo.errors_to_array_back(
	@code varchar(2048)
) returns varchar(512) begin 

	declare @alphabet varchar(64) = dbo.settings_get('base64_alphabet', null)

	return (select top 1 substring(@alphabet, [value] + 1, 1) from openjson(@code, '$') for xml path(''))
end
go

if object_id(N'dbo.convert_to_base', N'FN') is not null drop function dbo.convert_to_base
go
create function dbo.convert_to_base(
	@integer bigint,
	@base tinyint
) returns varchar(2048) begin 

	declare @results varchar(2048) = ''
	declare @alphabet varchar(64) = '0123456789' + substring(dbo.settings_get('base64_alphabet', null), 1, 54)

	while @integer != 0 begin 
		set @results = substring(@alphabet, (@integer % @base) + 1, 1) + @results
		set @integer = floor(@integer / @base)
	end

	return @results
end
go

if object_id(N'dbo.errors_to_array_binary', N'FN') is not null drop function dbo.errors_to_array_binary
go
create function dbo.errors_to_array_binary(
	@code varchar(2048)
) returns varchar(3584) begin
	return '[' + substring((select top 1 ',"' + right('000000' + dbo.convert_to_base([value], 2), 6) + '"' from openjson(case when substring(@code, 1, 1) = '[' then @code else dbo.errors_to_array(@code) end) for xml path('')), 2, 3584) + ']'
end
go
	
if object_id(N'dbo.errors_has', N'FN') is not null drop function dbo.errors_has
go
create function dbo.errors_has(
	@code varchar(512)
) returns bit begin 
	return (case when isnull(@code, '') = '' then 0 else (case when charindex(@code, substring(dbo.settings_get('base64_alphabet', null), 1, 1) collate latin1_general_cs_as) = 0 then 1 else 0 end) end)
end
go
	
if object_id(N'dbo.errors_get_hexa', N'FN') is not null drop function dbo.errors_get_hexa
go
create function dbo.errors_get_hexa(
	@alphabet char(64),
	@code varchar(512),
	@position smallint
) returns smallint begin 
		
	declare @i smallint = charindex(substring(@code, @position + 1, 1), @alphabet collate latin1_general_cs_as) - 1
	
	return (case when @i < 0 then null else @i end)
end
go
	
if object_id(N'dbo.errors_get_bit', N'FN') is not null drop function dbo.errors_get_bit
go
create function dbo.errors_get_bit(
	@code varchar(512),
	@position smallint
) returns bit begin 
	return dbo.errors_get_hexa(dbo.settings_get('base64_alphabet', 0), @code, @position / 6) & dbo.bw(1, @position % 6)
end
go
	
if object_id(N'dbo.errors_set_in', N'FN') is not null drop function dbo.errors_set_in
go
create function dbo.errors_set_in(
	@alphabet char(64),
	@code varchar(512),
	@hexa smallint,
	@position smallint
) returns varchar(512) begin 
		
	declare @l smallint
	declare @zero char(1) = substring(@alphabet, 1, 1)

	while len(@code) <= @position
		set @code = @code + @zero
	set @l = len(@code)
	
	return (case 
		when @position >= 0 and @hexa >= 0 and @hexa < 0x40 
		then concat(left(@code, @position), substring(@alphabet, @hexa + 1, 1), right(@code, @l - @position - 1)) 
		else @code 
	end)
end
go
	
if object_id(N'dbo.errors_bitwise', N'FN') is not null drop function dbo.errors_bitwise
go
create function dbo.errors_bitwise(
	@code varchar(512),
	@bits smallint
) returns varchar(512) begin
		
	declare @alphabet char(64) = dbo.settings_get('base64_alphabet', 0)
	declare @reverse bit = (case when @bits < 0 then 1 else 0 end)
		
	if isnull(@bits, 0) = 0 or dbo.errors_has(@code) = 0
		return (case when isnull(@code, '') = '' then substring(@alphabet, 1, 1) else @code end)
	
	if @reverse = 1
		set @bits = -@bits
	
	begin
			
		declare @start tinyint = floor(@bits / 6)
		declare @rest tinyint = @bits % 6
		declare @r tinyint = 6 - @rest
		declare @i smallint
		declare @l smallint
		declare @first char(1) = substring(@alphabet, 1, 1)
		
		if @reverse = 1 begin -- Bitwise Right.

			set @code = substring(@code, @start + 1, 512)
			set @l = len(@code) - 1

			if @rest != 0 begin 

				set @i = 0

				while @i < @l begin  

					set @code = dbo.errors_set_in(@alphabet, @code, (
						dbo.bw(dbo.errors_get_hexa(@alphabet, @code, @i), -@rest) |
						(dbo.bw(dbo.errors_get_hexa(@alphabet, @code, @i + 1), @r) & 0x3F)
					), @i)
					set @i = @i + 1
				end

				set @code = dbo.errors_set_in(@alphabet, @code, dbo.bw(dbo.errors_get_hexa(@alphabet, @code, @l), -@rest), @l)

			end
		end else begin -- Bitwise Left.
				
			declare @last_hexa smallint

			set @l = len(@code) - 1
			set @last_hexa = dbo.bw(dbo.errors_get_hexa(@alphabet, @code, @l), -@r)
			
			if @rest != 0 begin
				
				set @i = @l

				while @i > 0 begin
					set @code = dbo.errors_set_in(@alphabet, @code, (
						(dbo.bw(dbo.errors_get_hexa(@alphabet, @code, @i), @rest) & 0x3F) |
						dbo.bw(dbo.errors_get_hexa(@alphabet, @code, @i - 1), -@r)
					), @i)
					set @i = @i - 1
				end

				set @code = dbo.errors_set_in(@alphabet, @code, dbo.bw(dbo.errors_get_hexa(@alphabet, @code, 0), @rest) & 0x3F, 0)
				if @last_hexa != 0  
					set @code = @code + substring(@alphabet, @last_hexa + 1, 1)
						
			end
			
			set @i = 0
			while @i < @start begin
				set @i = @i + 1
				set @code = @first + @code
			end
				
		end
		
	end
	
	return (case when @code = '' then @first else @code end)
end
go
	
if object_id(N'dbo.errors_compact', N'FN') is not null drop function dbo.errors_compact
go
create function dbo.errors_compact(
	@code varchar(512),
	@alphabet char(64)
) returns varchar(512) begin
	
	declare @l smallint = len(@code)
	declare @zero char(1) = substring(isnull(@alphabet, dbo.settings_get('base64_alphabet', null)), 1, 1)
		
	while @l > 1 and substring(@code, @l, 1) = @zero begin
		set @l = @l - 1
	end
	
	if @l != 0
		set @code = substring(@code, 1, @l)
		
	-- return (case when isnull(@code, '') = '' then @zero else @code end)
	return @code
end
go
	
if object_id(N'dbo.errors_length_code', N'FN') is not null drop function dbo.errors_length_code
go
create function dbo.errors_length_code(
	@alphabet char(64),
	@bit smallint,
	@length smallint
) returns varchar(512) begin
		
	declare @length_code varchar(512) = substring(@alphabet, 1, 1)
	declare @l smallint = floor(@length / 6)
		
	if @length is not null and @length != 0 begin 
			
		declare @k tinyint = @length % 6
		
		set @length_code = ''
		while len(@length_code) < @l begin
			set @length_code = substring(@alphabet, 64, 1) + @length_code
		end
		if @k != 0
			set @length_code = substring(@alphabet, ~-dbo.bw(1, @k) + 1, 1) + @length_code
		if @bit != 0
			set @length_code = dbo.errors_bitwise(@length_code, @bit)
		
	end
	
	return @length_code;	
end
go

if object_id(N'dbo.errors_reset', N'FN') is not null drop function dbo.errors_reset
go
create function dbo.errors_reset(
	@code varchar(512),
	@bit smallint,
	@length smallint,
	@alphabet char(64)
) returns varchar(512) begin 

	declare @zero char(1)
	declare @l smallint

	if isnull(@alphabet, '') in ('', '0') 
		set @alphabet = dbo.settings_get('base64_alphabet', 0)
	set @zero = substring(@alphabet, 1, 1)
	
	if @length > 0 begin
		
		declare @i smallint = floor(@bit / 6)
		declare @rest tinyint = @bit % 6
		declare @subtotal smallint = @length - (6 - @rest)
		declare @last smallint = @subtotal % 6
		declare @start_fragment tinyint = (case when @length + @rest < 6 then (@length + @rest) % 6 else 0 end)
		declare @m smallint = len(@code)
		declare @total smallint = floor((@bit + @length) / 6)
		
		set @l = @i + 1 + floor(@subtotal / 6)

		if @length != 0 or @rest != 0 begin 
			set @code = dbo.errors_set_in(@alphabet, @code, dbo.errors_get_hexa(@alphabet, @code, @i) & (~-dbo.bw(1, @rest) | (case when @start_fragment = 0 then 0 else dbo.bw(~-dbo.bw(1, 6 - @start_fragment), @start_fragment) end)), @i)
			set @i = @i + 1
		end
		if @m < @total begin 
			set @code = substring(@code, 1, @i)
			while len(@code) < @total 
				set @code = @code + @zero
		end else begin 
			set @l = (case when @l > @m then @m else @l - 1 end)
			while @i <= @l begin 
				set @code = dbo.errors_set_in(@alphabet, @code, 0, @i)
				set @i = @i + 1
			end
			if @last > 0 and @i < @m
				set @code = dbo.errors_set_in(@alphabet, @code, dbo.errors_get_hexa(@alphabet, @code, @i) & dbo.bw(~-dbo.bw(1, 6 - @last), @last), @i)
		end

	end else begin 
		
		set @l = floor(@bit / 6)

		while len(@code) < @l 
			set @code = @code + @zero

	end

	return @code
end
go

if object_id(N'dbo.errors_join', N'FN') is not null drop function dbo.errors_join
go
create function dbo.errors_join(
	@code varchar(512),
	@error varchar(512),
	@bit smallint,
	@length smallint
) returns varchar(512) begin

	declare @alphabet char(64) = dbo.settings_get('base64_alphabet', 0)
	declare @l smallint
	declare @i smallint = 0
	declare @zero char(1) = substring(@alphabet, 1, 1)

	set @code = dbo.errors_reset(@code, @bit, @length, @alphabet)

	if isnull(@bit, 0) != 0 begin 
		set @error = dbo.errors_bitwise(@error, @bit)
		set @i = floor(@bit / 6)
	end
	set @l = len(@error)

	while len(@code) <= @i
		set @code = @code + @zero

	while @i < @l begin
		set @code = dbo.errors_set_in(@alphabet, @code, isnull(dbo.errors_get_hexa(@alphabet, @code, @i), 0) | dbo.errors_get_hexa(@alphabet, @error, @i), @i)
		set @i = @i + 1
	end

	return dbo.errors_compact(@code, @alphabet)
end
go
	
if object_id(N'dbo.errors_set', N'FN') is not null drop function dbo.errors_set
go
create function dbo.errors_set(
	@code varchar(512),
	@error bigint,
	@bit integer,
	@length integer
) returns varchar(512) begin
	return dbo.errors_join(@code, dbo.errors_to_string(@error), @bit, @length)
end
go
	
if object_id(N'dbo.errors_messages_get_group', N'FN') is not null drop function dbo.errors_messages_get_group
go
create function dbo.errors_messages_get_group(
	@name varchar(64)
) returns integer begin 
	return (select top 1 id from dbo.ErrorsMessagesGroups where date_out is null and [name] = @name)
end
go

if object_id(N'dbo.errors_messages_clean_all', N'P') is not null drop procedure dbo.errors_messages_clean_all
go
create procedure dbo.errors_messages_clean_all as begin 

	set nocount on
		
	delete from dbo.ErrorsMessages where 1 = 1
	delete from dbo.ErrorsMessagesGroups where 1 = 1
		
end
go

if object_id(N'dbo.errors_messages_clean', N'P') is not null drop procedure dbo.errors_messages_clean
go
create procedure dbo.errors_messages_clean 
	@name varchar(64)
as begin 
		
	declare @group_id integer = dbo.errors_messages_get_group(@name)

	set nocount on
		
	delete from dbo.ErrorsMessages where [group] = @group_id
	delete from dbo.ErrorsMessagesGroups where id = @group_id
		
end
go
	
if object_id(N'dbo.errors_messages_add', N'P') is not null drop procedure dbo.errors_messages_add
go
create procedure dbo.errors_messages_add 
	@name varchar(64),
	@i smallint,
	@message varchar(64)
as begin 
		
	declare @group_id integer = dbo.errors_messages_get_group(@name)

	set nocount on
	
	if @group_id is null begin 
		insert into ErrorsMessagesGroups([name]) values(@name)
		set @group_id = scope_identity()
	end
	
	if (select top 1 1 from dbo.ErrorsMessages where date_out is null and [group] = @group_id and i = @i and [name] = @message) is null begin 
		if (select top 1 1 from dbo.ErrorsMessages where date_out is null and [group] = @group_id and i = @i) is not null 
			update dbo.ErrorsMessages set [name] = @message where date_out is null and [group] = @group_id and i = @i
		else 
			insert into dbo.ErrorsMessages([group], i, [name]) values(@group_id, @i, @message)
	end
		
end
go
	
if object_id(N'dbo.errors_get_data', N'P') is not null drop procedure dbo.errors_get_data
go
create procedure dbo.errors_get_data 
	@name varchar(64),
	@code varchar(512),
	@session bit
as begin 
		
	declare @length smallint = len(@code)
	declare @group_id integer = dbo.errors_messages_get_group(@name)
	declare @session_id integer = (case when @session = 1 then dbo.errors_messages_get_group('sessions_validate_full') else null end)

	set nocount on
	
	if object_id(N'tempsb..#default_errors_messages_temporary_anp', N'U') is not null drop table #default_errors_messages_temporary_anp

	create table #default_errors_messages_temporary_anp(
		id integer not null identity(1, 1),
		i smallint not null,
		[message] varchar(64) not null,
		primary key(id)
	)

	if @group_id is not null 
		insert into #default_errors_messages_temporary_anp(i, [message]) 
			select i, [name] from dbo.ErrorsMessages where 
				date_out is null and 
				[group] = @group_id and 
				i >= 0 and 
				[name] is not null
	
	if @session_id is not null 
		insert into #default_errors_messages_temporary_anp(i, [message]) 
			select i, [name] from dbo.ErrorsMessages where 
				date_out is null and 
				[group] = @session_id and 
				i >= 0 and 
				[name] is not null and 
				i not in (select i from #default_errors_messages_temporary_anp where i >= 0)

	insert into #default_errors_messages_temporary_anp(i, [message]) 
		select unknown_messages.i, concat('message_', unknown_messages.i) from (
			select top (@length * 6) 
				row_number() over(order by table_name asc) - 1 as i
			from information_schema.columns
		) unknown_messages where unknown_messages.i not in (select i from #default_errors_messages_temporary_anp)

	select 
		errors_messages.i as i, 
		errors_messages.[message] as [message] 
	from (
		select top (@length) 
			row_number() over(order by a.table_name asc) - 1 as i, 
			dbo.errors_to_integer(substring(@code, row_number() over(order by a.table_name asc), 1)) as hexa 
		from information_schema.columns a
		join information_schema.columns b on 1 = 1
	) characters 
	join #default_errors_messages_temporary_anp errors_messages on 
		characters.hexa != 0 and 
		floor(errors_messages.i / 6) = characters.i and 
		dbo.bw(1, errors_messages.i % 6) & characters.hexa != 0
	
	drop table #default_errors_messages_temporary_anp
		
end
go

if object_id(N'dbo.errors_messages_set', N'P') is not null drop procedure dbo.errors_messages_set
go
create procedure dbo.errors_messages_set 
	@session varchar(max)
as begin 

	declare @group_i integer
	declare @group_name varchar(64)
	declare @counter integer
	declare group_cursor cursor local for select i, [name] from dbo.ErrorsTemporaryGroups where date_out  is null
	
	set nocount on

	-- execute dbo.errors_messages_clean_all

	open group_cursor
	while 1 = 1 begin 
				
		fetch next from group_cursor into @group_i, @group_name
		if @@fetch_status != 0
			break
		
		declare @i smallint
		declare @message varchar(64)
		declare message_cursor cursor local for select i, [message] from dbo.ErrorsTemporaryMessages where date_out is null and [group] = @group_i

		execute dbo.errors_messages_clean @group_name
		set @counter = 30

		open message_cursor
		while 1 = 1 begin 
				
			fetch next from message_cursor into @i, @message
			if @@fetch_status != 0
				break

			if @i is null 
				set @i = @counter

			execute dbo.errors_messages_add @group_name, @i, @message
			set @counter = @counter + 1
		
		end
		close message_cursor
		deallocate message_cursor
		
	end
	close group_cursor
	deallocate group_cursor
	
	delete from dbo.ErrorsTemporaryGroups where 1 = 1
	delete from dbo.ErrorsTemporaryMessages where 1 = 1

end
go