if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.SessionsIpsValidateView', N'V') is not null drop view dbo.SessionsIpsValidateView
go
create view dbo.SessionsIpsValidateView as select 
	ips.entity as entity, 
	ips.[address] as [address], 
	ips.accessible as accessible, 
	ips.deleted as deleted, 
	sessions_ips.[session] as [session] 
from _SessionsIps sessions_ips 
join _Ips ips on sessions_ips.[ip] = ips.entity
go

if object_id(N'dbo.SessionsUsersAgentsValidateView', N'V') is not null drop view dbo.SessionsUsersAgentsValidateView
go
create view dbo.SessionsUsersAgentsValidateView as select 
	users_agents.id as id, 
	users_agents.[data] as [data], 
	sessions_users_agents.[session] as [session] 
from _SessionsUsersAgents sessions_users_agents 
join UsersAgents users_agents on sessions_users_agents.user_agent = users_agents.id
go

if object_id(N'dbo.SessionsValidateView', N'V') is not null drop view dbo.SessionsValidateView
go
create view dbo.SessionsValidateView as select 
	[sessions].id as id,
	users.entity as [user_id],
	users.nick as nick,
	users.accessible as user_accessible,
	users.deleted as user_deleted, 
	ips.entity as ip_id,
	ips.[address] as [ip],
	ips.accessible as ip_accessible, 
	ips.deleted as ip_deleted, 
	users_agents.id as user_agent_id,
	users_agents.[data] as user_agent,
	[sessions].date_out as date_out,
	[sessions].date_last as date_last
from dbo.[Sessions] [sessions] 
left join dbo._Users users on [sessions].[user] = users.entity 
left join dbo._SessionsIps sessions_ips on sessions_ips.[session] = [sessions].id 
left join dbo._Ips ips on sessions_ips.[ip] = ips.entity
left join dbo._SessionsUsersAgents sessions_users_agents on sessions_users_agents.[session] = [sessions].id 
left join dbo.UsersAgents users_agents on sessions_users_agents.user_agent = users_agents.id
go
	
if object_id(N'dbo.sessions_has_permissions', N'FN') is not null drop function dbo.sessions_has_permissions
go
create function dbo.sessions_has_permissions(
	@session integer,
	@permissions varchar(max)
) returns bit begin
	return 1
end
go

if object_id(N'dbo.sessions_is_banned', N'FN') is not null drop function dbo.sessions_is_banned
go
create function dbo.sessions_is_banned(
	@session integer
) returns bit begin 
	return 0
end
go

if object_id(N'dbo.sessions_timeout', N'FN') is not null drop function dbo.sessions_timeout
go
create function dbo.sessions_timeout(
	@session integer
) returns bit begin
	return 0
end
go

if object_id(N'dbo.sessions_is_guest', N'FN') is not null drop function dbo.sessions_is_guest
go
create function dbo.sessions_is_guest(
	@session integer
) returns bit begin 
	return 1
end
go

if object_id(N'dbo.users_settings_get', N'FN') is not null drop function dbo.users_settings_get
go
create function dbo.users_settings_get(
	@user integer,
	@name varchar(64)
) returns varchar(max) begin 
	return (select top 1 [value] from dbo._Settings where deleted is null and [name] = @name)
end
go

if object_id(N'dbo.users_settings_get_int', N'FN') is not null drop function dbo.users_settings_get_int
go
create function dbo.users_settings_get_int(
	@user integer,
	@name varchar(64)
) returns integer begin 
	return dbo.string_to_integer(dbo.users_settings_get(@user, @name))
end
go

if object_id(N'dbo.users_settings_get_bool', N'FN') is not null drop function dbo.users_settings_get_bool
go
create function dbo.users_settings_get_bool(
	@user integer,
	@name varchar(64)
) returns bit begin 
	return dbo.string_to_bool(dbo.users_settings_get(@user, @name))
end
go
	
if object_id(N'dbo.sessions_validate_full', N'P') is not null drop procedure dbo.sessions_validate_full
go
create procedure dbo.sessions_validate_full 
	@session varchar(max), 
	@permissions varchar(max),
	@error varchar(512) output,
	@session_id integer output,
	@ip_id integer output,
	@user_agent_id integer output
as begin

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_validate_full'
	declare @procedure_id integer
	declare @error_integer integer

	set nocount on

	execute dbo.procedures_get @database, @procedure, @procedure_id output

	set @error_integer = (case
		when @session is null then dbo.bw(1, 0)
		when @session = '' then dbo.bw(1, 1)
		when @session not like '%,%,%' then dbo.bw(1, 2)
	else 0 end)

	if @error_integer = 0 begin 

		declare @separator1 integer = charindex(',', @session)
		declare @separator2 integer = charindex(',', @session, @separator1 + 1)
		declare @session_text varchar(15) = (case when @separator1 = 0 then null else substring(@session, 1, @separator1 - 1) end)
		declare @ip varchar(39) = (case when @separator2 = 0 then null else substring(@session, @separator1 + 1, @separator2 - @separator1 - 1) end)
		declare @user_agent varchar(max) = (case when @separator2 = 0 then null else substring(@session, @separator2 + 1, 9999999) end)

		if @ip = 'null' 
			set @ip = null

		if @user_agent = 'null' 
			set @user_agent = null

		set @session_id = (case when @session_text like '%[^0-9]%' then null else convert(integer, @session_text) end)

		set @error_integer = dbo.bw((
			dbo.bw((case 
				when @session_id is null then dbo.bw(1, 0)
				when @session_id < 1 then dbo.bw(1, 1)
			else 0 end), 0) |
			dbo.bw((case 
				when @ip is null then dbo.bw(1, 0)
				when @ip = '' then dbo.bw(1, 1)
				-- when @ip not like '' then dbo.bw(1, 2)
			else 0 end), 2) |
			dbo.bw((case 
				when @user_agent is null then dbo.bw(1, 0)
				when @user_agent = '' then dbo.bw(1, 1)
			else 0 end), 5) |
		0), 3)

		set @ip_id = (select top 1 entity from dbo._Ips where deleted is null and (
			(@ip is null and [address] is null) or 
			(@ip is not null and [address] = @ip)
		))
		set @user_agent_id = (select top 1 id from dbo.UsersAgents where date_out is null and (
			(@user_agent is null and [data] is null) or 
			(@user_agent is not null and [data] = @user_agent)
		))

		if @ip_id is null begin 
			insert into Ips default values
			set @ip_id = scope_identity()
			insert into IpsData([procedure], [session], entity, [address], accessible) values 
				(@procedure_id, isnull((
					select top 1 id from dbo.[Sessions] where id = @session_id
				), dbo.settings_get_int('unknown_session', 0)), @ip_id, @ip, dbo.settings_get_bool('allow_any_new_ip', 0))
		end

		if @user_agent_id is null begin 
			insert into dbo.UsersAgents([data]) values(@user_agent)
			set @user_agent_id = scope_identity()
		end

		if @error_integer = 0 begin 
			
			declare @user_id integer = (select top 1 [user] from dbo.[Sessions] where id = @session_id)
			declare @check_ip bit = dbo.users_settings_get_bool(@user_id, 'session_check_ip')
			declare @check_user_agent bit = dbo.users_settings_get_bool(@user_id, 'session_check_user_agent')

			set @error_integer = dbo.bw(isnull((
				select top 1 (

					dbo.bw((case when [user_id] is null then dbo.bw(1, 0) else (
						(case when user_accessible = 1 or dbo.sessions_is_guest(@session_id) = 1 then 0 else dbo.bw(1, 1) end) |
						(case when user_deleted is null then 0 else dbo.bw(1, 2) end) |
						isnull((select top 1 dbo.bw(1, 3) from dbo._Users where entity = [user_id] and '%"' + nick + '"%' like dbo.settings_get('users_excluded', 0)), 0) |
					0) end), 1) | 

					dbo.bw((case when @check_ip = 1 then (case when @ip_id = ip_id then (case when ip_id is null then dbo.bw(1, 1) else (
						(case when ip_accessible = 1 then 0 else dbo.bw(1, 3) end) |
						(case when ip_deleted is null then 0 else dbo.bw(1, 4) end) |
						isnull((select top 1 dbo.bw(1, 5) from dbo._Ips where entity = ip_id and '%"' + @ip + '"%' like dbo.settings_get('ips_excluded', 0)), 0) |
					0) end) else dbo.bw(1, 0) end) else (case when @ip_id is null then dbo.bw(1, 1) else isnull((
						select top 1 (
							(case when accessible = 1 then 0 else dbo.bw(1, 3) end) |
							(case when deleted is null then 0 else dbo.bw(1, 4) end) |
							(case when '%"' + [address] + '"%' like dbo.settings_get('ips_excluded', 0) then dbo.bw(1, 5) else 0 end) |
						0) from dbo._Ips where entity = @ip_id
					), dbo.bw(1, 2)) end) end), 5) |

					dbo.bw((case when @check_user_agent = 1 then (case 
						when @user_agent_id = user_agent_id then (case when user_agent_id is null then dbo.bw(1, 1) else 0 end) else dbo.bw(1, 0) end) 
						else (case when @user_agent_id is null then dbo.bw(1, 1) else isnull((select top 1 0 from dbo.UsersAgents where date_out is null and id = @user_agent_id), dbo.bw(1, 2)) end) 
					end), 11) | 

					dbo.bw((case 
						when date_out is not null then dbo.bw(1, 0) 
						when dbo.sessions_timeout(id) = 1 then dbo.bw(1, 1)
					else 0 end), 14) |

					(case when dbo.sessions_is_banned(id) = 1 then dbo.bw(1, 16) else 0 end) | 
				0) from dbo.SessionsValidateView where id = @session_id
			), dbo.bw(1, 0)), 10)

			if @check_ip = 1 and (select top 1 1 from dbo._SessionsIps where 
				[session] = @session_id and 
				[ip] = @ip_id 
			) is null begin 
				
				declare @session_ip_id integer = (select top 1 entity from dbo._SessionsIps where [session] = @session_id)
				
				if @session_ip_id is null begin 
					insert into dbo.SessionsIps default values
					insert into dbo.SessionsIpsData(entity, [session], [ip], ok) values 
						(scope_identity(), @session_id, @ip_id, 1 - @check_ip)
				end else 
					insert into dbo.SessionsIpsData(entity, [session], [ip], ok) values 
						(@session_ip_id, @session_id, @ip_id, 1 - @check_user_agent)

			end

			if @check_user_agent = 1 and (select top 1 1 from dbo._SessionsUsersAgents where 
				[session] = @session_id and 
				user_agent = @user_agent_id 
			) is null begin 
				
				declare @session_user_agent_id integer = (select top 1 entity from dbo._SessionsUsersAgents where [session] = @session_id)
				
				if @session_user_agent_id is null begin 
					insert into dbo.SessionsUsersAgents default values
					insert into dbo.SessionsUsersAgentsData(entity, [session], user_agent, ok) values 
						(scope_identity(), @session_id, @user_agent_id, 1 - @check_ip)
				end else 
					insert into dbo.SessionsUsersAgentsData(entity, [session], user_agent, ok) values 
						(@session_user_agent_id, @session_id, @user_agent_id, 1 - @check_user_agent)

			end

		end

		set @session_id = (select top 1 id from dbo.[Sessions] where id = @session_id)

	end

	if @error_integer = 0 and dbo.sessions_has_permissions(@session_id, @permissions) = 0
		set @error_integer = dbo.bw(1, 27)

	set @error = dbo.errors_set('', @error_integer, 1, 0)
	
end
go
	
if object_id(N'dbo.sessions_get_id', N'FN') is not null drop function dbo.sessions_get_id
go
create function dbo.sessions_get_id(
	@session varchar(max)
) returns integer begin 
		
	declare @session_text varchar(max) = (case when isnull(@session, '') = '' then null else substring(@session, 1, charindex(',', @session) - 1) end)
	
	return (case when @session_text like '%[^0-9]$' then null else convert(integer, @session_text) end)
end
go
	
if object_id(N'dbo.anp_initial_sessions_set_errors_messages', N'P') is not null drop procedure dbo.anp_initial_sessions_set_errors_messages
go
create procedure dbo.anp_initial_sessions_set_errors_messages 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'anp_initial_sessions_set_errors_messages'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer
	declare @error varchar(512) = ''

	set nocount on

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	insert into dbo.ErrorsTemporaryGroups(i, [name]) values 
		(1, 'sessions_validate_full')

	insert into dbo.ErrorsTemporaryMessages([group], i, [message]) values 
		(1, 0, 'exception'),
		(1, 1, 'session_null'),
		(1, 2, 'session_empty'),
		(1, 3, 'session_bad_format'),
		(1, 4, 'session_id_null'),
		(1, 5, 'session_id_bad'),
		(1, 6, 'ip_null'),
		(1, 7, 'ip_empty'),
		(1, 8, 'ip_bad_format'),
		(1, 9, 'user_agent_null'),
		(1, 10, 'user_agent_empty'),
		(1, 11, 'session_not_exists'),
		(1, 12, 'user_id_null'),
		(1, 13, 'user_unaccessible'),
		(1, 14, 'user_deleted'),
		(1, 15, 'user_excluded'),
		(1, 16, 'ip_not_same'),
		(1, 17, 'ip_id_null'),
		(1, 18, 'ip_not_exists'),
		(1, 19, 'ip_unaccessible'),
		(1, 20, 'ip_deleted'),
		(1, 21, 'ip_excluded'),
		(1, 22, 'user_agent_not_same'),
		(1, 23, 'user_agent_id_null'),
		(1, 24, 'user_agent_not_exists'),
		(1, 25, 'session_date_out'),
		(1, 26, 'session_timeout'),
		(1, 27, 'session_banned'),
		(1, 28, 'session_not_perimssions')

	execute dbo.errors_messages_set @session

	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, '{}', 2
		
end
go
	
if object_id(N'dbo.traces_set', N'P') is not null drop procedure dbo.traces_set
go
create procedure dbo.traces_set 
	@session varchar(max), 
	@project varchar(64), 
	@path varchar(512), 
	@own_error varchar(512),
	@own_parameters varchar(max), 
	@trace varchar(max), 
	@message varchar(max), 
	@line integer, 
	@error varchar(512) output, 
	@id integer output 
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'traces_set'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	declare @parameters varchar(max) = '{"project":null,"path":null,"own_error":null,"own_parameters":null,"trace":null,"message":null,"line":null}'
	
	set nocount on
	set @id = 0
	set @parameters = json_modify(@parameters, 'strict $.project', @project)
	set @parameters = json_modify(@parameters, 'strict $.path', @path)
	set @parameters = json_modify(@parameters, 'strict $.own_error', @own_error)
	set @parameters = json_modify(@parameters, 'strict $.own_parameters', @own_parameters)
	set @parameters = json_modify(@parameters, 'strict $.trace', @trace)
	set @parameters = json_modify(@parameters, 'strict $.message', @message)
	set @parameters = json_modify(@parameters, 'strict $.line', @line)

	begin try
			
		declare @project_id integer
		declare @path_id integer
		declare @error_id integer
		declare @parameters_id integer
		declare @trace_id integer
		declare @message_id integer

		execute dbo.sessions_validate_full @session, null, @error output, @session_id output, @ip_id output, @user_agent_id output
		
		execute dbo.projects_get @project, @project_id output
		execute dbo.traces_files_get @path, @path_id output
		execute dbo.errors_get @error, @error_id output
		execute dbo.parameters_get @parameters, @parameters_id output
		execute dbo.lines_get @trace, @trace_id output
		execute dbo.messages_get @message, @message_id output
		
		insert into dbo.[Traces]([session], [ip], user_agent, project, [file], own_error, own_parameters, trace, [message], line) values 
			(@session_id, @ip_id, @user_agent_id, @project_id, @path_id, @error_id, @parameters_id, @trace_id, @message_id, @line)
		set @id = scope_identity()

	end try
	begin catch
		set @error = @error | dbo.bw(1, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 4
		
end
go
	
execute dbo.anp_initial_sessions_set_errors_messages null

if object_id(N'dbo.password_validate', N'FN') is not null drop function dbo.password_validate
go
create function dbo.password_validate(
	@password varchar(256)
) returns integer begin 
	return (case 
		when @password is null then dbo.bw(1, 0)
		when @password = '' then dbo.bw(1, 1)
	else (
		(case when len(@password) < isnull(dbo.settings_get_int('password_minimum_length', 0), 1) then dbo.bw(1, 2) else 0 end) |
		(case when isnull(dbo.settings_get_bool('password_needs_lower', 0), 0) = 1 and @password collate latin1_general_cs_as not like '%[qwertyuiopasdfghjklzxcvbnm]%' then dbo.bw(1, 3) else 0 end) |
		(case when isnull(dbo.settings_get_bool('password_needs_upper', 0), 0) = 1 and @password collate latin1_general_cs_as not like '%[QWERTYUIOPASDFGHJKLZXCVBNM]%' then dbo.bw(1, 4) else 0 end) |
		(case when isnull(dbo.settings_get_bool('password_needs_digit', 0), 0) = 1 and @password not like '%[0-9]%' then dbo.bw(1, 5) else 0 end) |
		(case when isnull(dbo.settings_get_bool('password_needs_special', 0), 0) = 1 and @password not like '%[^a-zA-Z0-9]%' then dbo.bw(1, 6) else 0 end) |
	0) end)
end
go

if object_id(N'dbo.sessions_password_hash', N'FN') is not null drop function dbo.sessions_password_hash
go
create function dbo.sessions_password_hash(
	@password varchar(256)
) returns varchar(256) begin 

	declare @method varchar(32) = dbo.settings_get('password_hash_function', 0)

	if @method in ('md2', 'md4', 'md5', 'sha', 'sha1')
		return substring(convert(nvarchar(max), hashbytes(@method, convert(nvarchar(256), @password)), 2), 1, 256)

	if @method in ('sha2') begin 

		set @method = 'sha2_' + dbo.settings_get('hash_sha2_length', 0)

		return substring(convert(nvarchar(max), hashbytes(@method, convert(nvarchar(256), @password)), 2), 1, 256)
	end

	if @method = 'pwdencrypt'
		return substring(pwdencrypt(@password), 1, 256)
	return @password
end
go

if object_id(N'dbo.users_groups_create', N'P') is not null drop procedure dbo.users_groups_create
go
create procedure dbo.users_groups_create 
	@procedure_id integer, 
	@session_id integer, 
	@nick varchar(32), 
	@id integer 
as begin 

	declare @group_id integer
	declare @group integer

	insert into dbo.Groups default values
	set @group_id = scope_identity()
	insert into dbo.GroupsData([procedure], [session], entity, [name], [default], is_entity, [description]) values 
		(@procedure_id, @session_id, @group_id, @nick, 0, 1, 'Group for entity user "' + @nick + '".')
		
	insert into dbo.UsersGroups default values
	insert into dbo.UsersGroupsData([procedure], [session], entity, [user], [group], belongs) values 
		(@procedure_id, @session_id, scope_identity(), @id, @group_id, 1)

	declare default_groups_cursor cursor local for select entity from _Groups where deleted is null and [default] = 1

	open default_groups_cursor
	while 1 = 1 begin 
		
		fetch next from default_groups_cursor into @group
		if @@fetch_status != 0
			break
		
		insert into dbo.UsersGroups default values
		insert into dbo.UsersGroupsData([procedure], [session], entity, [user], [group], belongs) values 
			(@procedure_id, @session_id, scope_identity(), @id, @group, 1)
	
	end
	close default_groups_cursor
	deallocate default_groups_cursor

end
go

if object_id(N'dbo.users_get_entity_group', N'FN') is not null drop function dbo.users_get_entity_group
go
create function dbo.users_get_entity_group(
	@entity integer 
) returns integer begin 
	return (
		select top 1 groups.entity 
		from dbo._UsersGroups users_groups 
		join dbo._Groups groups on users_groups.[group] = groups.entity 
		join dbo._Users users on users_groups.[user] = users.entity 
		where 
			users.entity = @entity and 
			users_groups.belongs = 1 and 
			groups.is_entity = 1 and 
			users.nick = groups.name 
		order by users_groups.entity asc 
	)
end
go

if object_id(N'dbo.users_update_group_data', N'P') is not null drop procedure dbo.users_update_group_data
go
create procedure dbo.users_update_group_data 
	@procedure_id integer, 
	@session_id integer, 
	@group_id integer, 
	@nick varchar(32)
as begin 
	
	declare @group_name varchar(64)
	declare @group_default bit
	declare @group_is_entity bit
	declare @group_description varchar(512)

	select top 1 
		@group_name = [name], 
		@group_default = [default], 
		@group_is_entity = is_entity, 
		@group_description = [description] 
	from dbo._Groups where entity = @group_id;

	if @group_name != @nick 
		insert into dbo.GroupsData([procedure], [session], entity, [name], [default], is_entity, [description]) values 
			(@procedure_id, @session_id, @group_id, @nick, @group_default, @group_is_entity, @group_description)

end
go