if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.users_tables_delete', N'P') is not null drop procedure dbo.users_tables_delete
go
create procedure dbo.users_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._Users', N'U') is not null drop table dbo._Users
	if object_id(N'dbo.UsersData', N'U') is not null drop table dbo.UsersData
	if object_id(N'dbo.Users', N'U') is not null drop table dbo.Users
		
end
go

if object_id(N'dbo.users_tables_create', N'P') is not null drop procedure dbo.users_tables_create
go
create procedure dbo.users_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.Users', N'U') is null create table dbo.Users(
		id integer not null identity(1, 1),
		date_in datetime not null constraint users_date_in default getdate(),
		date_out datetime,
		constraint users_id primary key(id)
	)
	
	if object_id(N'dbo.UsersData', N'U') is null create table dbo.UsersData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		nick varchar(32) not null,
		[password] varchar(256) not null,
		accessible bit not null constraint users_accessible_data default 0,
		deleted datetime,
		date_in datetime not null constraint users_data_date_in default getdate(),
		date_out datetime,
		constraint users_data_id primary key(id),
		constraint users_data_entity foreign key(entity) references dbo.Users(id), 
		constraint users_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint users_data_session foreign key([session]) references dbo.[Sessions](id)
	)
	
	if object_id(N'dbo._Users', N'U') is null create table dbo._Users(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		nick varchar(32) not null,
		[password] varchar(256) not null,
		accessible bit not null constraint _users_accessible default 0,
		deleted datetime,
		date_in datetime not null,
		date_created datetime not null,
		constraint _users_id primary key(id),
		constraint _users_entity foreign key(entity) references dbo.Users(id), 
		constraint _users_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _users_session foreign key([session]) references dbo.[Sessions](id)
	)
		
end
go

if object_id(N'dbo.users_tables_update', N'P') is not null drop procedure dbo.users_tables_update
go
create procedure dbo.users_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'users_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'UsersData' and column_name = 'nick') is null begin 
		alter table dbo.UsersData add nick varchar(32) not null
		alter table dbo._Users add nick varchar(32) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'UsersData' and column_name = 'password') is null begin 
		alter table dbo.UsersData add [password] varchar(256) not null
		alter table dbo._Users add [password] varchar(256) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'UsersData' and column_name = 'accessible') is null begin 
		alter table dbo.UsersData add accessible bit not null constraint _users_accessible default 0
		alter table dbo._Users add accessible bit not null constraint _users_accessible default 0
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.users_tables_delete null
execute dbo.users_tables_create null
execute dbo.users_tables_update null
	
