if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP
	
drop view if exists UsersDataView
go
create view UsersDataView as select 
	[data].*,
	entities.date_in as date_created
from (select * from dbo.UsersData where id in (
	select max(id) from dbo.UsersData where date_out is null group by entity
)) [data]
join Users entities on [data].entity = entities.id 
where entities.date_out is null
go
	
if object_id(N'dbo.users_plain_reset', N'P') is not null drop procedure dbo.users_plain_reset
go
create procedure dbo.users_plain_reset 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'users_plain_reset'
	declare @session_id integer = dbo.sessions_get_id(@session)
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on

	if @session is null 
		set @session = dbo.sessions_get_machine()
		
	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	begin transaction

	delete from dbo._Users where 1 = 1
	dbcc checkident('_Users', reseed, 0)
	insert into dbo._Users([procedure], [session], entity, [data], nick, [password], accessible, deleted, date_in, date_created) 
		select [procedure], [session], entity, id, nick, [password], accessible, deleted, date_in, date_created from dbo.UsersDataView

	commit
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2

end
go
	
if object_id(N'dbo.users_plain_update', N'P') is not null drop procedure dbo.users_plain_update
go
create procedure dbo.users_plain_update 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'users_plain_update'
	declare @session_id integer = dbo.sessions_get_id(@session)
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on

	if @session is null 
		set @session = dbo.sessions_get_machine()
		
	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	begin transaction

	delete from dbo._Users where entity not in (select entity from dbo.UsersDataView)
	update plain set 
		plain.[procedure] = [data].[procedure],
		plain.[session] = [data].[session],
		plain.[data] = [data].id,
		plain.nick = [data].nick,
		plain.[password] = [data].[password],
		plain.accessible = [data].accessible,
		plain.deleted = [data].deleted,
		plain.date_in = [data].date_in, 
		plain.date_created = [data].date_created
	from dbo._Users plain 
	join dbo.UsersDataView [data] on plain.entity = [data].entity
	insert into dbo._Users([procedure], [session], entity, [data], nick, [password], accessible, deleted, date_in, date_created) 
		select [procedure], [session], entity, id, nick, [password], accessible, deleted, date_in, date_created from dbo.UsersDataView where entity not in (
			select entity from dbo._Users
		)

	commit
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2

end
go
	
