if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.users_insert_validate', N'FN') is not null drop function dbo.users_insert_validate
go
create function dbo.users_insert_validate(
	@nick varchar(32),
	@password varchar(256),
	@accessible bit
) returns varchar(512) begin 
		
	declare @error varchar(512) = ''

	set @error = dbo.errors_set(@error, (case 
		when @nick is null then dbo.bw(1, 1)
	else 1 end), 0, 0)
	set @error = dbo.errors_set(@error, (case 
		when @password is null then dbo.bw(1, 0)
	else 0 end), 2, 0)
	set @error = dbo.errors_set(@error, (case 
		when @accessible is null then dbo.bw(1, 0)
	else 0 end), 3, 0)
	
	return @error
end
go
	
if object_id(N'dbo.users_add', N'P') is not null drop procedure dbo.users_add
go
create procedure dbo.users_add
	@session varchar(max),
	@nick varchar(32),
	@password varchar(256),
	@accessible bit,
	@error varchar(512) output,
	@id integer output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'users_add'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"entity":null,"id":null,"nick":null,"password":null,"accessible":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on
	set @error = ''
	set @id = 0 
	
	begin transaction

	set @parameters = json_modify(@parameters, 'strict $.nick', @nick)
	set @parameters = json_modify(@parameters, 'strict $.password', @password)
	set @parameters = json_modify(@parameters, 'strict $.accessible', @accessible)
	set @parameters = json_modify(@parameters, 'strict $.entity', 0)
	set @parameters = json_modify(@parameters, 'strict $.id', 0)

	begin try

		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_join(@error, dbo.users_insert_validate(@nick, @password, @accessible), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin
			
			execute dbo.procedures_get @database, @procedure, @procedure_id output
		
			insert into dbo.Users default values
			set @id = scope_identity()
			insert into dbo.UsersData([procedure], [session], entity, nick, [password], accessible) values
				(@procedure_id, @session_id, @id, @nick, @password, @accessible)
			
			set @parameters = json_modify(@parameters, 'strict $.entity', @id)
			set @parameters = json_modify(@parameters, 'strict $.id', scope_identity())
			
		end
	
		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go
	
