if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.groups_tables_delete', N'P') is not null drop procedure dbo.groups_tables_delete
go
create procedure dbo.groups_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._Groups', N'U') is not null drop table dbo._Groups
	if object_id(N'dbo.GroupsData', N'U') is not null drop table dbo.GroupsData
	if object_id(N'dbo.Groups', N'U') is not null drop table dbo.Groups
		
end
go

if object_id(N'dbo.groups_tables_create', N'P') is not null drop procedure dbo.groups_tables_create
go
create procedure dbo.groups_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.Groups', N'U') is null create table dbo.Groups(
		id integer not null identity(1, 1),
		date_in datetime not null constraint groups_date_in default getdate(),
		date_out datetime,
		constraint groups_id primary key(id)
	)
	
	if object_id(N'dbo.GroupsData', N'U') is null create table dbo.GroupsData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		[name] varchar(64) not null,
		[default] bit not null constraint groups_default_data default 0,
		is_entity bit not null constraint groups_is_entity_data default 0,
		[description] varchar(512),
		deleted datetime,
		date_in datetime not null constraint groups_data_date_in default getdate(),
		date_out datetime,
		constraint groups_data_id primary key(id),
		constraint groups_data_entity foreign key(entity) references dbo.Groups(id), 
		constraint groups_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint groups_data_session foreign key([session]) references dbo.[Sessions](id)
	)
	
	if object_id(N'dbo._Groups', N'U') is null create table dbo._Groups(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		[name] varchar(64) not null,
		[default] bit not null constraint _groups_default default 0,
		is_entity bit not null constraint _groups_is_entity default 0,
		[description] varchar(512),
		deleted datetime,
		date_in datetime not null,
		date_created datetime not null,
		constraint _groups_id primary key(id),
		constraint _groups_entity foreign key(entity) references dbo.Groups(id), 
		constraint _groups_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _groups_session foreign key([session]) references dbo.[Sessions](id)
	)
		
end
go

if object_id(N'dbo.groups_tables_update', N'P') is not null drop procedure dbo.groups_tables_update
go
create procedure dbo.groups_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'groups_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'GroupsData' and column_name = 'name') is null begin 
		alter table dbo.GroupsData add [name] varchar(64) not null
		alter table dbo._Groups add [name] varchar(64) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'GroupsData' and column_name = 'default') is null begin 
		alter table dbo.GroupsData add [default] bit not null constraint _groups_default default 0
		alter table dbo._Groups add [default] bit not null constraint _groups_default default 0
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'GroupsData' and column_name = 'is_entity') is null begin 
		alter table dbo.GroupsData add is_entity bit not null constraint _groups_is_entity default 0
		alter table dbo._Groups add is_entity bit not null constraint _groups_is_entity default 0
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'GroupsData' and column_name = 'description') is null begin 
		alter table dbo.GroupsData add [description] varchar(512)
		alter table dbo._Groups add [description] varchar(512)
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.groups_tables_delete null
execute dbo.groups_tables_create null
execute dbo.groups_tables_update null
	
