if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.groups_insert_validate', N'FN') is not null drop function dbo.groups_insert_validate
go
create function dbo.groups_insert_validate(
	@name varchar(64),
	@default bit,
	@is_entity bit,
	@description varchar(512)
) returns varchar(512) begin 
		
	declare @error varchar(512) = ''

	set @error = dbo.errors_set(@error, (case 
		when @name is null then dbo.bw(1, 0)
	else 0 end), 0, 0)
	set @error = dbo.errors_set(@error, (case 
		when @default is null then dbo.bw(1, 0)
	else 0 end), 1, 0)
	set @error = dbo.errors_set(@error, (case 
		when @is_entity is null then dbo.bw(1, 0)
	else 0 end), 2, 0)
	
	return @error
end
go
	
if object_id(N'dbo.groups_add', N'P') is not null drop procedure dbo.groups_add
go
create procedure dbo.groups_add
	@session varchar(max),
	@name varchar(64),
	@default bit,
	@is_entity bit,
	@description varchar(512),
	@error varchar(512) output,
	@id integer output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'groups_add'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"entity":null,"id":null,"name":null,"default":null,"is_entity":null,"description":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on
	set @error = ''
	set @id = 0 
	
	begin transaction

	set @parameters = json_modify(@parameters, 'strict $.name', @name)
	set @parameters = json_modify(@parameters, 'strict $.default', @default)
	set @parameters = json_modify(@parameters, 'strict $.is_entity', @is_entity)
	set @parameters = json_modify(@parameters, 'strict $.description', @description)
	set @parameters = json_modify(@parameters, 'strict $.entity', 0)
	set @parameters = json_modify(@parameters, 'strict $.id', 0)

	begin try

		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_join(@error, dbo.groups_insert_validate(@name, @default, @is_entity, @description), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin
			
			execute dbo.procedures_get @database, @procedure, @procedure_id output
		
			insert into dbo.Groups default values
			set @id = scope_identity()
			insert into dbo.GroupsData([procedure], [session], entity, [name], [default], is_entity, [description]) values
				(@procedure_id, @session_id, @id, @name, @default, @is_entity, @description)
			
			set @parameters = json_modify(@parameters, 'strict $.entity', @id)
			set @parameters = json_modify(@parameters, 'strict $.id', scope_identity())
			
		end
	
		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go
	
