if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.ips_tables_delete', N'P') is not null drop procedure dbo.ips_tables_delete
go
create procedure dbo.ips_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._Ips', N'U') is not null drop table dbo._Ips
	if object_id(N'dbo.IpsData', N'U') is not null drop table dbo.IpsData
	if object_id(N'dbo.Ips', N'U') is not null drop table dbo.Ips
		
end
go

if object_id(N'dbo.ips_tables_create', N'P') is not null drop procedure dbo.ips_tables_create
go
create procedure dbo.ips_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.Ips', N'U') is null create table dbo.Ips(
		id integer not null identity(1, 1),
		date_in datetime not null constraint ips_date_in default getdate(),
		date_out datetime,
		constraint ips_id primary key(id)
	)
	
	if object_id(N'dbo.IpsData', N'U') is null create table dbo.IpsData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		[address] varchar(39) not null,
		accessible bit not null constraint ips_accessible_data default 1,
		deleted datetime,
		date_in datetime not null constraint ips_data_date_in default getdate(),
		date_out datetime,
		constraint ips_data_id primary key(id),
		constraint ips_data_entity foreign key(entity) references dbo.Ips(id), 
		constraint ips_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint ips_data_session foreign key([session]) references dbo.[Sessions](id)
	)
	
	if object_id(N'dbo._Ips', N'U') is null create table dbo._Ips(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		[address] varchar(39) not null,
		accessible bit not null constraint _ips_accessible default 1,
		deleted datetime,
		date_in datetime not null,
		date_created datetime not null,
		constraint _ips_id primary key(id),
		constraint _ips_entity foreign key(entity) references dbo.Ips(id), 
		constraint _ips_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _ips_session foreign key([session]) references dbo.[Sessions](id)
	)
		
end
go

if object_id(N'dbo.ips_tables_update', N'P') is not null drop procedure dbo.ips_tables_update
go
create procedure dbo.ips_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'ips_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'IpsData' and column_name = 'address') is null begin 
		alter table dbo.IpsData add [address] varchar(39) not null
		alter table dbo._Ips add [address] varchar(39) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'IpsData' and column_name = 'accessible') is null begin 
		alter table dbo.IpsData add accessible bit not null constraint _ips_accessible default 1
		alter table dbo._Ips add accessible bit not null constraint _ips_accessible default 1
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.ips_tables_delete null
execute dbo.ips_tables_create null
execute dbo.ips_tables_update null
	
