if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.permissions_tables_delete', N'P') is not null drop procedure dbo.permissions_tables_delete
go
create procedure dbo.permissions_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._Permissions', N'U') is not null drop table dbo._Permissions
	if object_id(N'dbo.PermissionsData', N'U') is not null drop table dbo.PermissionsData
	if object_id(N'dbo.Permissions', N'U') is not null drop table dbo.Permissions
		
end
go

if object_id(N'dbo.permissions_tables_create', N'P') is not null drop procedure dbo.permissions_tables_create
go
create procedure dbo.permissions_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.Permissions', N'U') is null create table dbo.Permissions(
		id integer not null identity(1, 1),
		date_in datetime not null constraint permissions_date_in default getdate(),
		date_out datetime,
		constraint permissions_id primary key(id)
	)
	
	if object_id(N'dbo.PermissionsData', N'U') is null create table dbo.PermissionsData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		[name] varchar(64) not null,
		[description] varchar(512),
		deleted datetime,
		date_in datetime not null constraint permissions_data_date_in default getdate(),
		date_out datetime,
		constraint permissions_data_id primary key(id),
		constraint permissions_data_entity foreign key(entity) references dbo.Permissions(id), 
		constraint permissions_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint permissions_data_session foreign key([session]) references dbo.[Sessions](id)
	)
	
	if object_id(N'dbo._Permissions', N'U') is null create table dbo._Permissions(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		[name] varchar(64) not null,
		[description] varchar(512),
		deleted datetime,
		date_in datetime not null,
		date_created datetime not null,
		constraint _permissions_id primary key(id),
		constraint _permissions_entity foreign key(entity) references dbo.Permissions(id), 
		constraint _permissions_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _permissions_session foreign key([session]) references dbo.[Sessions](id)
	)
		
end
go

if object_id(N'dbo.permissions_tables_update', N'P') is not null drop procedure dbo.permissions_tables_update
go
create procedure dbo.permissions_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'permissions_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'PermissionsData' and column_name = 'name') is null begin 
		alter table dbo.PermissionsData add [name] varchar(64) not null
		alter table dbo._Permissions add [name] varchar(64) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'PermissionsData' and column_name = 'description') is null begin 
		alter table dbo.PermissionsData add [description] varchar(512)
		alter table dbo._Permissions add [description] varchar(512)
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.permissions_tables_delete null
execute dbo.permissions_tables_create null
execute dbo.permissions_tables_update null
	
