if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.settings_types_edit_validate', N'FN') is not null drop function dbo.settings_types_edit_validate
go
create function dbo.settings_types_edit_validate(
	@entity integer,
	@name varchar(64),
	@description varchar(512)
) returns varchar(512) begin 
		
	declare @error varchar(512) = dbo.errors_set('', dbo.settings_types_validate(@entity, 1), 0, 0)

	set @error = dbo.errors_set(@error, (case 
		when @name is null then dbo.bw(1, 0)
	else 0 end), 4, 0)

	if dbo.errors_has(@error) = 0 
		set @error = dbo.errors_set(@error, (
			isnull((
				select top 1 dbo.bw(1, 0) from dbo._SettingsTypes where deleted is null and entity = @entity and [name] = @name and [description] = @description
			), 0) | 
		0), 5, 0)

	
	return @error
end
go
	
if object_id(N'dbo.settings_types_edit', N'P') is not null drop procedure dbo.settings_types_edit
go
create procedure dbo.settings_types_edit
	@session varchar(max),
	@entity integer,
	@name varchar(64),
	@description varchar(512),
	@error varchar(512) output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_types_edit'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"entity":null,"id":null,"name":null,"description":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on
	set @error = ''
	
	begin transaction

	set @parameters = json_modify(@parameters, 'strict $.name', @name)
	set @parameters = json_modify(@parameters, 'strict $.description', @description)
	set @parameters = json_modify(@parameters, 'strict $.entity', @entity)
	set @parameters = json_modify(@parameters, 'strict $.id', 0)

	begin try
	
		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_join(@error, dbo.settings_types_edit_validate(@entity, @name, @description), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin 
		
			execute dbo.procedures_get @database, @procedure, @procedure_id output
		
			insert into dbo.SettingsTypesData([procedure], [session], entity, [name], [description]) values
				(@procedure_id, @session_id, @entity, @name, @description)
			
			set @parameters = json_modify(@parameters, 'strict $.id', scope_identity())
		
		end
	
		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go