if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.settings_types_state_validate', N'FN') is not null drop function dbo.settings_types_state_validate
go
create function dbo.settings_types_state_validate(
	@id integer
) returns integer begin 
	return (case 
		when @id is null then dbo.bw(1, 0)
		when @id < 1 then dbo.bw(1, 1)
		when @id in (select [data] from dbo._SettingsTypes) then dbo.bw(1, 2)
	else isnull((
		select top 1 0 from dbo.SettingsTypesData where date_out is null and id = @id
	), dbo.bw(1, 3)) end)
end
go
	
if object_id(N'dbo.settings_types_restore', N'P') is not null drop procedure dbo.settings_types_restore
go
create procedure dbo.settings_types_restore
	@session varchar(max),
	@id integer,
	@error varchar(512) output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_types_restore'
	declare @procedure_id integer
	declare @old_id integer
	declare @parameters varchar(max) = '{"old":null,"new":null,"id":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set @error = ''
	set @parameters = json_modify(@parameters, 'strict $.old', 0)
	set @parameters = json_modify(@parameters, 'strict $.new', @id)
	set @parameters = json_modify(@parameters, 'strict $.id', 0)

	begin transaction

	begin try
	
		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_set(@error, dbo.settings_types_state_validate(@id), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin 
			
			execute dbo.procedures_get @database, @procedure, @procedure_id output
			set @old_id = (select top 1 [data] from dbo._SettingsTypes where entity = (
				select top 1 entity from dbo.SettingsTypesData where id = @id
			))
		
			insert into dbo.SettingsTypesData([procedure], [session], entity, [name], [description], deleted) 
				select top 1 @procedure_id, @session_id, entity, [name], [description], deleted from dbo.SettingsTypesData where id = @id
			
			set @parameters = json_modify(@parameters, 'strict $.old', @old_id)
			set @parameters = json_modify(@parameters, 'strict $.id', scope_identity() )
		
		end

		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go

if object_id(N'dbo.settings_types_state_i18n_validate', N'FN') is not null drop function dbo.settings_types_state_i18n_validate
go
create function dbo.settings_types_state_i18n_validate(
	@id integer
) returns integer begin 
	return (case 
		when @id is null then dbo.bw(1, 0)
		when @id < 1 then dbo.bw(1, 1)
		when @id in (select i18n from dbo._SettingsTypesI18N) then dbo.bw(1, 2)
	else isnull((
		select top 1 0 from dbo.SettingsTypesI18N where date_out is null and id = @id
	), dbo.bw(1, 3)) end)
end
go

if object_id(N'dbo.settings_types_restore_i18n', N'P') is not null drop procedure dbo.settings_types_restore_i18n
go
create procedure dbo.settings_types_restore_i18n
	@session varchar(max),
	@id integer,
	@error varchar(512) output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_types_restore_i18n'
	declare @procedure_id integer
	declare @old_id integer
	declare @parameters varchar(max) = '{"old":null,"new":null,"id":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set @error = ''
	set @parameters = json_modify(@parameters, 'strict $.old', 0)
	set @parameters = json_modify(@parameters, 'strict $.new', @id)
	set @parameters = json_modify(@parameters, 'strict $.id', 0)

	begin transaction

	begin try
	
		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_set(@error, dbo.settings_types_state_i18n_validate(@id), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin 
			
			execute dbo.procedures_get @database, @procedure, @procedure_id output
			set @old_id = (select top 1 i18n from dbo._SettingsTypesI18N where entity = (
				select top 1 entity from dbo.SettingsTypesI18N where id = @id
			))
		
			insert into dbo.SettingsTypesI18N([procedure], [session], entity) 
				select top 1 @procedure_id, @session_id, entity from dbo.SettingsTypesI18N where id = @id
			
			set @parameters = json_modify(@parameters, 'strict $.old', @old_id)
			set @parameters = json_modify(@parameters, 'strict $.id', scope_identity() )
		
		end

		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go

