if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.ips_groups_tables_delete', N'P') is not null drop procedure dbo.ips_groups_tables_delete
go
create procedure dbo.ips_groups_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._IpsGroups', N'U') is not null drop table dbo._IpsGroups
	if object_id(N'dbo.IpsGroupsData', N'U') is not null drop table dbo.IpsGroupsData
	if object_id(N'dbo.IpsGroups', N'U') is not null drop table dbo.IpsGroups
		
end
go

if object_id(N'dbo.ips_groups_tables_create', N'P') is not null drop procedure dbo.ips_groups_tables_create
go
create procedure dbo.ips_groups_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.IpsGroups', N'U') is null create table dbo.IpsGroups(
		id integer not null identity(1, 1),
		date_in datetime not null constraint ips_groups_date_in default getdate(),
		date_out datetime,
		constraint ips_groups_id primary key(id)
	)
	
	if object_id(N'dbo.IpsGroupsData', N'U') is null create table dbo.IpsGroupsData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		[group] integer not null,
		[ip] integer not null,
		belongs bit not null,
		date_in datetime not null constraint ips_groups_data_date_in default getdate(),
		date_out datetime,
		constraint ips_groups_data_id primary key(id),
		constraint ips_groups_data_entity foreign key(entity) references dbo.IpsGroups(id), 
		constraint ips_groups_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint ips_groups_data_session foreign key([session]) references dbo.[Sessions](id),
		constraint ips_groups_data_group foreign key([group]) references dbo.Groups(id),
		constraint ips_groups_data_ip foreign key([ip]) references dbo.Ips(id)
	)
	
	if object_id(N'dbo._IpsGroups', N'U') is null create table dbo._IpsGroups(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		[group] integer not null,
		[ip] integer not null,
		belongs bit not null,
		date_in datetime not null,
		date_created datetime not null,
		constraint _ips_groups_id primary key(id),
		constraint _ips_groups_entity foreign key(entity) references dbo.IpsGroups(id), 
		constraint _ips_groups_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _ips_groups_session foreign key([session]) references dbo.[Sessions](id),
		constraint _ips_groups_group foreign key([group]) references dbo.Groups(id),
		constraint _ips_groups_ip foreign key([ip]) references dbo.Ips(id)
	)
		
end
go

if object_id(N'dbo.ips_groups_tables_update', N'P') is not null drop procedure dbo.ips_groups_tables_update
go
create procedure dbo.ips_groups_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'ips_groups_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'IpsGroupsData' and column_name = 'group') is null begin 
		alter table dbo.IpsGroupsData add [group] integer not null
		alter table dbo.IpsGroupsData add constraint ips_groups_data_group foreign key([group]) references dbo.Groups(id)
		alter table dbo._IpsGroups add [group] integer not null
		alter table dbo._IpsGroups add constraint _ips_groups_group foreign key([group]) references dbo.Groups(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'IpsGroupsData' and column_name = 'ip') is null begin 
		alter table dbo.IpsGroupsData add [ip] integer not null
		alter table dbo.IpsGroupsData add constraint ips_groups_data_ip foreign key([ip]) references dbo.Ips(id)
		alter table dbo._IpsGroups add [ip] integer not null
		alter table dbo._IpsGroups add constraint _ips_groups_ip foreign key([ip]) references dbo.Ips(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'IpsGroupsData' and column_name = 'belongs') is null begin 
		alter table dbo.IpsGroupsData add belongs bit not null
		alter table dbo._IpsGroups add belongs bit not null
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.ips_groups_tables_delete null
execute dbo.ips_groups_tables_create null
execute dbo.ips_groups_tables_update null
	
