if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.settings_tables_delete', N'P') is not null drop procedure dbo.settings_tables_delete
go
create procedure dbo.settings_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._Settings', N'U') is not null drop table dbo._Settings
	if object_id(N'dbo.SettingsData', N'U') is not null drop table dbo.SettingsData
	if object_id(N'dbo.Settings', N'U') is not null drop table dbo.Settings
		
end
go

if object_id(N'dbo.settings_tables_create', N'P') is not null drop procedure dbo.settings_tables_create
go
create procedure dbo.settings_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.Settings', N'U') is null create table dbo.Settings(
		id integer not null identity(1, 1),
		date_in datetime not null constraint settings_date_in default getdate(),
		date_out datetime,
		constraint settings_id primary key(id)
	)
	
	if object_id(N'dbo.SettingsData', N'U') is null create table dbo.SettingsData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		[type] integer not null,
		[name] varchar(64) not null,
		[range] integer not null constraint settings_range_data default 1,
		[value] varchar(max),
		[description] varchar(max),
		deleted datetime,
		date_in datetime not null constraint settings_data_date_in default getdate(),
		date_out datetime,
		constraint settings_data_id primary key(id),
		constraint settings_data_entity foreign key(entity) references dbo.Settings(id), 
		constraint settings_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint settings_data_session foreign key([session]) references dbo.[Sessions](id),
		constraint settings_data_type foreign key([type]) references dbo.SettingsTypes(id)
	)
	
	if object_id(N'dbo._Settings', N'U') is null create table dbo._Settings(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		[type] integer not null,
		[name] varchar(64) not null,
		[range] integer not null constraint _settings_range default 1,
		[value] varchar(max),
		[description] varchar(max),
		deleted datetime,
		date_in datetime not null,
		date_created datetime not null,
		constraint _settings_id primary key(id),
		constraint _settings_entity foreign key(entity) references dbo.Settings(id), 
		constraint _settings_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _settings_session foreign key([session]) references dbo.[Sessions](id),
		constraint _settings_type foreign key([type]) references dbo.SettingsTypes(id)
	)
		
end
go

if object_id(N'dbo.settings_tables_update', N'P') is not null drop procedure dbo.settings_tables_update
go
create procedure dbo.settings_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsData' and column_name = 'type') is null begin 
		alter table dbo.SettingsData add [type] integer not null
		alter table dbo.SettingsData add constraint settings_data_type foreign key([type]) references dbo.SettingsTypes(id)
		alter table dbo._Settings add [type] integer not null
		alter table dbo._Settings add constraint _settings_type foreign key([type]) references dbo.SettingsTypes(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsData' and column_name = 'name') is null begin 
		alter table dbo.SettingsData add [name] varchar(64) not null
		alter table dbo._Settings add [name] varchar(64) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsData' and column_name = 'range') is null begin 
		alter table dbo.SettingsData add [range] integer not null constraint _settings_range default 1
		alter table dbo._Settings add [range] integer not null constraint _settings_range default 1
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsData' and column_name = 'value') is null begin 
		alter table dbo.SettingsData add [value] varchar(max)
		alter table dbo._Settings add [value] varchar(max)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsData' and column_name = 'description') is null begin 
		alter table dbo.SettingsData add [description] varchar(max)
		alter table dbo._Settings add [description] varchar(max)
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.settings_tables_delete null
execute dbo.settings_tables_create null
execute dbo.settings_tables_update null
	
