if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.settings_data_update', N'P') is not null drop procedure dbo.settings_data_update
go
create procedure dbo.settings_data_update 
	@entity integer
as begin 
	
	set nocount on

	if (
		(select top 1 1 from dbo.Settings where id = @entity and date_out is null) is null or 
		(select top 1 1 from dbo.SettingsData where entity = @entity and date_out is null) is null
	) 
		delete from dbo._Settings where entity = @entity
	else begin 
		if (select top 1 1 from dbo._Settings where entity = @entity) is null 
			insert into dbo._Settings([procedure], [session], entity, [data], [type], [name], [range], [value], [description], deleted, date_in, date_created) 
				select [procedure], [session], entity, id, [type], [name], [range], [value], [description], deleted, date_in, (
					select top 1 date_in from dbo.Settings where id = @entity
				) from dbo.SettingsData where id = (
					select top 1 max(id) from dbo.SettingsData where date_out is null group by entity having entity = @entity
				)
		else 
			update plain set 
				plain.[procedure] = [data].[procedure],
				plain.[session] = [data].[session],
				plain.[data] = [data].id,
				plain.[type] = [data].[type],
				plain.[name] = [data].[name],
				plain.[range] = [data].[range],
				plain.[value] = [data].[value],
				plain.[description] = [data].[description],
				plain.deleted = [data].deleted,
				plain.date_in = [data].date_in
			from dbo._Settings plain 
			join (select [procedure], [session], entity, id, [type], [name], [range], [value], [description], deleted, date_in from dbo.SettingsData where id = (
				select top 1 max(id) from dbo.SettingsData where date_out is null group by entity having entity = @entity
			)) [data] on plain.entity = [data].entity
			where plain.entity = @entity
	end

end
go

if object_id(N'dbo.settings_full_update', N'P') is not null drop procedure dbo.settings_full_update
go
create procedure dbo.settings_full_update 
	@procedure integer,
	@session integer,
	@entity integer,
	@data integer,
	@type integer,
	@name varchar(64),
	@range integer,
	@value varchar(max),
	@description varchar(max),
	@deleted datetime,
	@date_in datetime,
	@date_out datetime
as begin 
	
	set nocount on

	if (select top 1 1 from dbo.Settings where id = @entity and date_out is null) is null 
		delete from dbo._Settings where entity = @entity
	else begin 
		if @date_out is not null 
			execute dbo.settings_data_update @entity
		else if (select top 1 1 from dbo._Settings where entity = @entity) is null 
			insert into _Settings([procedure], [session], entity, [data], [type], [name], [range], [value], [description], deleted, date_in, date_created) values
				(@procedure, @session, @entity, @data, @type, @name, @range, @value, @description, @deleted, @date_in, (
					select top 1 date_in from dbo.Settings where id = @entity
				))
		else 
			update dbo._Settings set 
				[procedure] = @procedure,
				[session] = @session,
				[data] = @data,
				[type] = @type,
				[name] = @name,
				[range] = @range,
				[value] = @value,
				[description] = @description,
				deleted = @deleted,
				date_in = @date_in
			where entity = @entity
	end

end
go

if object_id(N'dbo.settings_trigger_update_entity', N'TR') is not null drop trigger dbo.settings_trigger_update_entity
go
create trigger dbo.settings_trigger_update_entity on dbo.Settings after update as begin 

	declare @id integer
	declare update_cursor cursor local for select id from inserted

	set nocount on

	open update_cursor
	while 1 = 1 begin 
		fetch next from update_cursor into @id
		if @@fetch_status != 0
			break
		execute dbo.settings_data_update @id
	end
	close update_cursor
	deallocate update_cursor

end 
go

if object_id(N'dbo.settings_trigger_update_data', N'TR') is not null drop trigger dbo.settings_trigger_update_data
go
create trigger dbo.settings_trigger_update_data on dbo.SettingsData after insert, update, delete as begin 

	declare @procedure integer
	declare @session integer
	declare @entity integer
	declare @data integer
	declare @type integer
	declare @name varchar(64)
	declare @range integer
	declare @value varchar(max)
	declare @description varchar(max)
	declare @deleted datetime
	declare @date_in datetime
	declare @date_out datetime
	declare @id integer
	declare inserted_cursor cursor local for select [procedure], [session], entity, id, [type], [name], [range], [value], [description], deleted, date_in, date_out from inserted
	declare deleted_cursor cursor local for select entity from deleted where id not in (select id from inserted)

	set nocount on
	
	open deleted_cursor
	while 1 = 1 begin 
		fetch next from deleted_cursor into @id
		if @@fetch_status != 0
			break
		execute dbo.settings_data_update @id
	end
	close deleted_cursor
	deallocate deleted_cursor
	
	open inserted_cursor
	while 1 = 1 begin 
		fetch next from inserted_cursor into @procedure, @session, @entity, @id, @type, @name, @range, @value, @description, @deleted, @date_in, @date_out
		if @@fetch_status != 0
			break
		execute dbo.settings_full_update @procedure, @session, @entity, @id, @type, @name, @range, @value, @description, @deleted, @date_in, @date_out
	end
	close inserted_cursor
	deallocate inserted_cursor

end 
go