if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.users_emails_data_update', N'P') is not null drop procedure dbo.users_emails_data_update
go
create procedure dbo.users_emails_data_update 
	@entity integer
as begin 
	
	set nocount on

	if (
		(select top 1 1 from dbo.UsersEmails where id = @entity and date_out is null) is null or 
		(select top 1 1 from dbo.UsersEmailsData where entity = @entity and date_out is null) is null
	) 
		delete from dbo._UsersEmails where entity = @entity
	else begin 
		if (select top 1 1 from dbo._UsersEmails where entity = @entity) is null 
			insert into dbo._UsersEmails([procedure], [session], entity, [data], email, [user], belongs, date_in, date_created) 
				select [procedure], [session], entity, id, email, [user], belongs, date_in, (
					select top 1 date_in from dbo.UsersEmails where id = @entity
				) from dbo.UsersEmailsData where id = (
					select top 1 max(id) from dbo.UsersEmailsData where date_out is null group by entity having entity = @entity
				)
		else 
			update plain set 
				plain.[procedure] = [data].[procedure],
				plain.[session] = [data].[session],
				plain.[data] = [data].id,
				plain.email = [data].email,
				plain.[user] = [data].[user],
				plain.belongs = [data].belongs,
				plain.date_in = [data].date_in
			from dbo._UsersEmails plain 
			join (select [procedure], [session], entity, id, email, [user], belongs, date_in from dbo.UsersEmailsData where id = (
				select top 1 max(id) from dbo.UsersEmailsData where date_out is null group by entity having entity = @entity
			)) [data] on plain.entity = [data].entity
			where plain.entity = @entity
	end

end
go

if object_id(N'dbo.users_emails_full_update', N'P') is not null drop procedure dbo.users_emails_full_update
go
create procedure dbo.users_emails_full_update 
	@procedure integer,
	@session integer,
	@entity integer,
	@data integer,
	@email integer,
	@user integer,
	@belongs bit,
	@date_in datetime,
	@date_out datetime
as begin 
	
	set nocount on

	if (select top 1 1 from dbo.UsersEmails where id = @entity and date_out is null) is null 
		delete from dbo._UsersEmails where entity = @entity
	else begin 
		if @date_out is not null 
			execute dbo.users_emails_data_update @entity
		else if (select top 1 1 from dbo._UsersEmails where entity = @entity) is null 
			insert into _UsersEmails([procedure], [session], entity, [data], email, [user], belongs, date_in, date_created) values
				(@procedure, @session, @entity, @data, @email, @user, @belongs, @date_in, (
					select top 1 date_in from dbo.UsersEmails where id = @entity
				))
		else 
			update dbo._UsersEmails set 
				[procedure] = @procedure,
				[session] = @session,
				[data] = @data,
				email = @email,
				[user] = @user,
				belongs = @belongs,
				date_in = @date_in
			where entity = @entity
	end

end
go

if object_id(N'dbo.users_emails_trigger_update_entity', N'TR') is not null drop trigger dbo.users_emails_trigger_update_entity
go
create trigger dbo.users_emails_trigger_update_entity on dbo.UsersEmails after update as begin 

	declare @id integer
	declare update_cursor cursor local for select id from inserted

	set nocount on

	open update_cursor
	while 1 = 1 begin 
		fetch next from update_cursor into @id
		if @@fetch_status != 0
			break
		execute dbo.users_emails_data_update @id
	end
	close update_cursor
	deallocate update_cursor

end 
go

if object_id(N'dbo.users_emails_trigger_update_data', N'TR') is not null drop trigger dbo.users_emails_trigger_update_data
go
create trigger dbo.users_emails_trigger_update_data on dbo.UsersEmailsData after insert, update, delete as begin 

	declare @procedure integer
	declare @session integer
	declare @entity integer
	declare @data integer
	declare @email integer
	declare @user integer
	declare @belongs bit
	declare @date_in datetime
	declare @date_out datetime
	declare @id integer
	declare inserted_cursor cursor local for select [procedure], [session], entity, id, email, [user], belongs, date_in, date_out from inserted
	declare deleted_cursor cursor local for select entity from deleted where id not in (select id from inserted)

	set nocount on
	
	open deleted_cursor
	while 1 = 1 begin 
		fetch next from deleted_cursor into @id
		if @@fetch_status != 0
			break
		execute dbo.users_emails_data_update @id
	end
	close deleted_cursor
	deallocate deleted_cursor
	
	open inserted_cursor
	while 1 = 1 begin 
		fetch next from inserted_cursor into @procedure, @session, @entity, @id, @email, @user, @belongs, @date_in, @date_out
		if @@fetch_status != 0
			break
		execute dbo.users_emails_full_update @procedure, @session, @entity, @id, @email, @user, @belongs, @date_in, @date_out
	end
	close inserted_cursor
	deallocate inserted_cursor

end 
go