if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.countries_data_update', N'P') is not null drop procedure dbo.countries_data_update
go
create procedure dbo.countries_data_update 
	@entity integer
as begin 
	
	set nocount on

	if (
		(select top 1 1 from dbo.Countries where id = @entity and date_out is null) is null or 
		(select top 1 1 from dbo.CountriesData where entity = @entity and date_out is null) is null
	) 
		delete from dbo._Countries where entity = @entity
	else begin 
		if (select top 1 1 from dbo._Countries where entity = @entity) is null 
			insert into dbo._Countries([procedure], [session], entity, [data], [name], [key], code, flag, [description], deleted, date_in, date_created) 
				select [procedure], [session], entity, id, [name], [key], code, flag, [description], deleted, date_in, (
					select top 1 date_in from dbo.Countries where id = @entity
				) from dbo.CountriesData where id = (
					select top 1 max(id) from dbo.CountriesData where date_out is null group by entity having entity = @entity
				)
		else 
			update plain set 
				plain.[procedure] = [data].[procedure],
				plain.[session] = [data].[session],
				plain.[data] = [data].id,
				plain.[name] = [data].[name],
				plain.[key] = [data].[key],
				plain.code = [data].code,
				plain.flag = [data].flag,
				plain.[description] = [data].[description],
				plain.deleted = [data].deleted,
				plain.date_in = [data].date_in
			from dbo._Countries plain 
			join (select [procedure], [session], entity, id, [name], [key], code, flag, [description], deleted, date_in from dbo.CountriesData where id = (
				select top 1 max(id) from dbo.CountriesData where date_out is null group by entity having entity = @entity
			)) [data] on plain.entity = [data].entity
			where plain.entity = @entity
	end

end
go

if object_id(N'dbo.countries_full_update', N'P') is not null drop procedure dbo.countries_full_update
go
create procedure dbo.countries_full_update 
	@procedure integer,
	@session integer,
	@entity integer,
	@data integer,
	@name varchar(64),
	@key varchar(64),
	@code varchar(4),
	@flag integer,
	@description varchar(512),
	@deleted datetime,
	@date_in datetime,
	@date_out datetime
as begin 
	
	set nocount on

	if (select top 1 1 from dbo.Countries where id = @entity and date_out is null) is null 
		delete from dbo._Countries where entity = @entity
	else begin 
		if @date_out is not null 
			execute dbo.countries_data_update @entity
		else if (select top 1 1 from dbo._Countries where entity = @entity) is null 
			insert into _Countries([procedure], [session], entity, [data], [name], [key], code, flag, [description], deleted, date_in, date_created) values
				(@procedure, @session, @entity, @data, @name, @key, @code, @flag, @description, @deleted, @date_in, (
					select top 1 date_in from dbo.Countries where id = @entity
				))
		else 
			update dbo._Countries set 
				[procedure] = @procedure,
				[session] = @session,
				[data] = @data,
				[name] = @name,
				[key] = @key,
				code = @code,
				flag = @flag,
				[description] = @description,
				deleted = @deleted,
				date_in = @date_in
			where entity = @entity
	end

end
go

if object_id(N'dbo.countries_trigger_update_entity', N'TR') is not null drop trigger dbo.countries_trigger_update_entity
go
create trigger dbo.countries_trigger_update_entity on dbo.Countries after update as begin 

	declare @id integer
	declare update_cursor cursor local for select id from inserted

	set nocount on

	open update_cursor
	while 1 = 1 begin 
		fetch next from update_cursor into @id
		if @@fetch_status != 0
			break
		execute dbo.countries_data_update @id
	end
	close update_cursor
	deallocate update_cursor

end 
go

if object_id(N'dbo.countries_trigger_update_data', N'TR') is not null drop trigger dbo.countries_trigger_update_data
go
create trigger dbo.countries_trigger_update_data on dbo.CountriesData after insert, update, delete as begin 

	declare @procedure integer
	declare @session integer
	declare @entity integer
	declare @data integer
	declare @name varchar(64)
	declare @key varchar(64)
	declare @code varchar(4)
	declare @flag integer
	declare @description varchar(512)
	declare @deleted datetime
	declare @date_in datetime
	declare @date_out datetime
	declare @id integer
	declare inserted_cursor cursor local for select [procedure], [session], entity, id, [name], [key], code, flag, [description], deleted, date_in, date_out from inserted
	declare deleted_cursor cursor local for select entity from deleted where id not in (select id from inserted)

	set nocount on
	
	open deleted_cursor
	while 1 = 1 begin 
		fetch next from deleted_cursor into @id
		if @@fetch_status != 0
			break
		execute dbo.countries_data_update @id
	end
	close deleted_cursor
	deallocate deleted_cursor
	
	open inserted_cursor
	while 1 = 1 begin 
		fetch next from inserted_cursor into @procedure, @session, @entity, @id, @name, @key, @code, @flag, @description, @deleted, @date_in, @date_out
		if @@fetch_status != 0
			break
		execute dbo.countries_full_update @procedure, @session, @entity, @id, @name, @key, @code, @flag, @description, @deleted, @date_in, @date_out
	end
	close inserted_cursor
	deallocate inserted_cursor

end 
go