if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.settings_groups_tables_delete', N'P') is not null drop procedure dbo.settings_groups_tables_delete
go
create procedure dbo.settings_groups_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._SettingsGroups', N'U') is not null drop table dbo._SettingsGroups
	if object_id(N'dbo.SettingsGroupsData', N'U') is not null drop table dbo.SettingsGroupsData
	if object_id(N'dbo.SettingsGroups', N'U') is not null drop table dbo.SettingsGroups
		
end
go

if object_id(N'dbo.settings_groups_tables_create', N'P') is not null drop procedure dbo.settings_groups_tables_create
go
create procedure dbo.settings_groups_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.SettingsGroups', N'U') is null create table dbo.SettingsGroups(
		id integer not null identity(1, 1),
		date_in datetime not null constraint settings_groups_date_in default getdate(),
		date_out datetime,
		constraint settings_groups_id primary key(id)
	)
	
	if object_id(N'dbo.SettingsGroupsData', N'U') is null create table dbo.SettingsGroupsData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		[group] integer not null,
		settings integer not null,
		[value] varchar(max),
		belongs bit not null,
		date_in datetime not null constraint settings_groups_data_date_in default getdate(),
		date_out datetime,
		constraint settings_groups_data_id primary key(id),
		constraint settings_groups_data_entity foreign key(entity) references dbo.SettingsGroups(id), 
		constraint settings_groups_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint settings_groups_data_session foreign key([session]) references dbo.[Sessions](id),
		constraint settings_groups_data_group foreign key([group]) references dbo.Groups(id),
		constraint settings_groups_data_settings foreign key(settings) references dbo.Settings(id)
	)
	
	if object_id(N'dbo._SettingsGroups', N'U') is null create table dbo._SettingsGroups(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		[group] integer not null,
		settings integer not null,
		[value] varchar(max),
		belongs bit not null,
		date_in datetime not null,
		date_created datetime not null,
		constraint _settings_groups_id primary key(id),
		constraint _settings_groups_entity foreign key(entity) references dbo.SettingsGroups(id), 
		constraint _settings_groups_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _settings_groups_session foreign key([session]) references dbo.[Sessions](id),
		constraint _settings_groups_group foreign key([group]) references dbo.Groups(id),
		constraint _settings_groups_settings foreign key(settings) references dbo.Settings(id)
	)
		
end
go

if object_id(N'dbo.settings_groups_tables_update', N'P') is not null drop procedure dbo.settings_groups_tables_update
go
create procedure dbo.settings_groups_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_groups_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsGroupsData' and column_name = 'group') is null begin 
		alter table dbo.SettingsGroupsData add [group] integer not null
		alter table dbo.SettingsGroupsData add constraint settings_groups_data_group foreign key([group]) references dbo.Groups(id)
		alter table dbo._SettingsGroups add [group] integer not null
		alter table dbo._SettingsGroups add constraint _settings_groups_group foreign key([group]) references dbo.Groups(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsGroupsData' and column_name = 'settings') is null begin 
		alter table dbo.SettingsGroupsData add settings integer not null
		alter table dbo.SettingsGroupsData add constraint settings_groups_data_settings foreign key(settings) references dbo.Settings(id)
		alter table dbo._SettingsGroups add settings integer not null
		alter table dbo._SettingsGroups add constraint _settings_groups_settings foreign key(settings) references dbo.Settings(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsGroupsData' and column_name = 'value') is null begin 
		alter table dbo.SettingsGroupsData add [value] varchar(max)
		alter table dbo._SettingsGroups add [value] varchar(max)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'SettingsGroupsData' and column_name = 'belongs') is null begin 
		alter table dbo.SettingsGroupsData add belongs bit not null
		alter table dbo._SettingsGroups add belongs bit not null
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.settings_groups_tables_delete null
execute dbo.settings_groups_tables_create null
execute dbo.settings_groups_tables_update null
	
