if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.settings_groups_history', N'P') is not null drop procedure dbo.settings_groups_history
go
create procedure dbo.settings_groups_history
	@session varchar(max),
	@entity integer,
	@page integer,
	@items_per_page integer,
	@where varchar(max),
	@order_by varchar(max),
	@pages integer output,
	@items integer output,
	@results integer output,
	@error varchar(512) output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_groups_history'
	declare @has_errors bit
	declare @view varchar(max) = @database + '.dbo.SettingsGroupsHistoryView'
	declare @parameters varchar(max) = '{"entity":null,"page":null,"items_per_page":null,"where":null,"order_by":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set @error = ''
	set @parameters = json_modify(@parameters, 'strict $.entity', @entity)
	set @parameters = json_modify(@parameters, 'strict $.page', @page)
	set @parameters = json_modify(@parameters, 'strict $.items_per_page', @items_per_page)
	set @parameters = json_modify(@parameters, 'strict $.where', @where)
	set @parameters = json_modify(@parameters, 'strict $.order_by', @order_by)

	begin try
		
		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_set(@error, dbo.settings_groups_validate(@entity, 1), 30, 0)
		set @has_errors = dbo.errors_has(@error)
	
		set @where = dbo.where_set(concat('id=', @entity), @where)
		execute dbo.tables_list @view, @page, @items_per_page, @where, @order_by, @has_errors, @pages output, @items output, @results output

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 2
		
end
go

if object_id(N'dbo.settings_groups_history_i18n', N'P') is not null drop procedure dbo.settings_groups_history_i18n
go
create procedure dbo.settings_groups_history_i18n
	@session varchar(max),
	@entity integer,
	@language integer, 
	@page integer,
	@items_per_page integer,
	@where varchar(max),
	@order_by varchar(max),
	@pages integer output,
	@items integer output,
	@results integer output,
	@error varchar(512) output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'settings_groups_history_i18n'
	declare @has_errors bit
	declare @view varchar(max) = @database + '.dbo.SettingsGroupsHistoryI18NView'
	declare @parameters varchar(max) = '{"entity":null,"language":null,"page":null,"items_per_page":null,"where":null,"order_by":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set @error = ''
	set @parameters = json_modify(@parameters, 'strict $.entity', @entity)
	set @parameters = json_modify(@parameters, 'strict $.language', @language)
	set @parameters = json_modify(@parameters, 'strict $.page', @page)
	set @parameters = json_modify(@parameters, 'strict $.items_per_page', @items_per_page)
	set @parameters = json_modify(@parameters, 'strict $.where', @where)
	set @parameters = json_modify(@parameters, 'strict $.order_by', @order_by)

	begin try
		
		execute dbo.sessions_validate_full @session, '["administrator"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_set(@error, (
			dbo.bw(dbo.settings_groups_validate(@entity, 1), 0) |
			dbo.bw(dbo.languages_validate(@language, 1), 4) |
		0), 30, 0)
		set @has_errors = dbo.errors_has(@error)
	
		set @where = dbo.where_set(concat('id=', @entity, ' and language_id=', @language), @where)
		execute dbo.tables_list @view, @page, @items_per_page, @where, @order_by, @has_errors, @pages output, @items output, @results output

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 2
		
end
go

