if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.languages_tables_delete', N'P') is not null drop procedure dbo.languages_tables_delete
go
create procedure dbo.languages_tables_delete
	@session varchar(max)
as begin
	
	if object_id(N'dbo._Languages', N'U') is not null drop table dbo._Languages
	if object_id(N'dbo.LanguagesData', N'U') is not null drop table dbo.LanguagesData
	if object_id(N'dbo.Languages', N'U') is not null drop table dbo.Languages
		
end
go

if object_id(N'dbo.languages_tables_create', N'P') is not null drop procedure dbo.languages_tables_create
go
create procedure dbo.languages_tables_create
	@session varchar(max)
as begin 
	
	if object_id(N'dbo.Languages', N'U') is null create table dbo.Languages(
		id integer not null identity(1, 1),
		date_in datetime not null constraint languages_date_in default getdate(),
		date_out datetime,
		constraint languages_id primary key(id)
	)
	
	if object_id(N'dbo.LanguagesData', N'U') is null create table dbo.LanguagesData(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null,
		country integer not null,
		[name] varchar(64) not null,
		[key] varchar(64) not null,
		code varchar(4) not null,
		flag integer,
		[description] varchar(512),
		deleted datetime,
		date_in datetime not null constraint languages_data_date_in default getdate(),
		date_out datetime,
		constraint languages_data_id primary key(id),
		constraint languages_data_entity foreign key(entity) references dbo.Languages(id), 
		constraint languages_data_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint languages_data_session foreign key([session]) references dbo.[Sessions](id),
		constraint languages_data_country foreign key(country) references dbo.Countries(id),
		constraint languages_data_flag foreign key(flag) references dbo.Files(id)
	)
	
	if object_id(N'dbo._Languages', N'U') is null create table dbo._Languages(
		id integer not null identity(1, 1),
		[procedure] integer not null,
		[session] integer not null,
		entity integer not null, 
		[data] integer not null,
		country integer not null,
		[name] varchar(64) not null,
		[key] varchar(64) not null,
		code varchar(4) not null,
		flag integer,
		[description] varchar(512),
		deleted datetime,
		date_in datetime not null,
		date_created datetime not null,
		constraint _languages_id primary key(id),
		constraint _languages_entity foreign key(entity) references dbo.Languages(id), 
		constraint _languages_procedure foreign key([procedure]) references dbo.[Procedures](id), 
		constraint _languages_session foreign key([session]) references dbo.[Sessions](id),
		constraint _languages_country foreign key(country) references dbo.Countries(id),
		constraint _languages_flag foreign key(flag) references dbo.Files(id)
	)
		
end
go

if object_id(N'dbo.languages_tables_update', N'P') is not null drop procedure dbo.languages_tables_update
go
create procedure dbo.languages_tables_update
	@session varchar(max)
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'languages_tables_update'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'LanguagesData' and column_name = 'country') is null begin 
		alter table dbo.LanguagesData add country integer not null
		alter table dbo.LanguagesData add constraint languages_data_country foreign key(country) references dbo.Countries(id)
		alter table dbo._Languages add country integer not null
		alter table dbo._Languages add constraint _languages_country foreign key(country) references dbo.Countries(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'LanguagesData' and column_name = 'name') is null begin 
		alter table dbo.LanguagesData add [name] varchar(64) not null
		alter table dbo._Languages add [name] varchar(64) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'LanguagesData' and column_name = 'key') is null begin 
		alter table dbo.LanguagesData add [key] varchar(64) not null
		alter table dbo._Languages add [key] varchar(64) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'LanguagesData' and column_name = 'code') is null begin 
		alter table dbo.LanguagesData add code varchar(4) not null
		alter table dbo._Languages add code varchar(4) not null
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'LanguagesData' and column_name = 'flag') is null begin 
		alter table dbo.LanguagesData add flag integer
		alter table dbo.LanguagesData add constraint languages_data_flag foreign key(flag) references dbo.Files(id)
		alter table dbo._Languages add flag integer
		alter table dbo._Languages add constraint _languages_flag foreign key(flag) references dbo.Files(id)
	end

	if (select top 1 1 from information_schema.columns where table_catalog = @database and table_name = 'LanguagesData' and column_name = 'description') is null begin 
		alter table dbo.LanguagesData add [description] varchar(512)
		alter table dbo._Languages add [description] varchar(512)
	end
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2
	
end
go
	
-- execute dbo.languages_tables_delete null
execute dbo.languages_tables_create null
execute dbo.languages_tables_update null
	
