if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP
	
drop view if exists LanguagesDataView
go
create view LanguagesDataView as select 
	[data].*,
	entities.date_in as date_created
from (select * from dbo.LanguagesData where id in (
	select max(id) from dbo.LanguagesData where date_out is null group by entity
)) [data]
join Languages entities on [data].entity = entities.id 
where entities.date_out is null
go
	
if object_id(N'dbo.languages_plain_reset', N'P') is not null drop procedure dbo.languages_plain_reset
go
create procedure dbo.languages_plain_reset 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'languages_plain_reset'
	declare @session_id integer = dbo.sessions_get_id(@session)
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on

	if @session is null 
		set @session = dbo.sessions_get_machine()
		
	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	begin transaction

	delete from dbo._Languages where 1 = 1
	dbcc checkident('_Languages', reseed, 0)
	insert into dbo._Languages([procedure], [session], entity, [data], country, [name], [key], code, flag, [description], deleted, date_in, date_created) 
		select [procedure], [session], entity, id, country, [name], [key], code, flag, [description], deleted, date_in, date_created from dbo.LanguagesDataView

	commit
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2

end
go
	
if object_id(N'dbo.languages_plain_update', N'P') is not null drop procedure dbo.languages_plain_update
go
create procedure dbo.languages_plain_update 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'languages_plain_update'
	declare @session_id integer = dbo.sessions_get_id(@session)
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on

	if @session is null 
		set @session = dbo.sessions_get_machine()
		
	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	begin transaction

	delete from dbo._Languages where entity not in (select entity from dbo.LanguagesDataView)
	update plain set 
		plain.[procedure] = [data].[procedure],
		plain.[session] = [data].[session],
		plain.[data] = [data].id,
		plain.country = [data].country,
		plain.[name] = [data].[name],
		plain.[key] = [data].[key],
		plain.code = [data].code,
		plain.flag = [data].flag,
		plain.[description] = [data].[description],
		plain.deleted = [data].deleted,
		plain.date_in = [data].date_in, 
		plain.date_created = [data].date_created
	from dbo._Languages plain 
	join dbo.LanguagesDataView [data] on plain.entity = [data].entity
	insert into dbo._Languages([procedure], [session], entity, [data], country, [name], [key], code, flag, [description], deleted, date_in, date_created) 
		select [procedure], [session], entity, id, country, [name], [key], code, flag, [description], deleted, date_in, date_created from dbo.LanguagesDataView where entity not in (
			select entity from dbo._Languages
		)

	commit
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, 0, '{}', 2

end
go
	
