if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.languages_insert_validate', N'FN') is not null drop function dbo.languages_insert_validate
go
create function dbo.languages_insert_validate(
	@country integer,
	@name varchar(64),
	@key varchar(64),
	@code varchar(4),
	@flag_id integer,
	@description varchar(512)
) returns varchar(512) begin 
		
	declare @error varchar(512) = ''

	set @error = dbo.errors_set(@error, dbo.countries_validate(@country, 1), 0, 0)
	set @error = dbo.errors_set(@error, (case 
		when @name is null then dbo.bw(1, 0)
	else 0 end), 4, 0)
	set @error = dbo.errors_set(@error, (case 
		when @key is null then dbo.bw(1, 0)
	else 0 end), 5, 0)
	set @error = dbo.errors_set(@error, (case 
		when @code is null then dbo.bw(1, 0)
	else 0 end), 6, 0)
	
	return @error
end
go
	
if object_id(N'dbo.languages_add', N'P') is not null drop procedure dbo.languages_add
go
create procedure dbo.languages_add
	@session varchar(max),
	@country integer,
	@name varchar(64),
	@key varchar(64),
	@code varchar(4),
	@flag_id integer,
	@flag_name varchar(256),
	@flag_mime varchar(64),
	@flag_size integer,
	@flag_date_modified bigint,
	@flag_path varchar(512),
	@flag_hashes varchar(max),
	@description varchar(512),
	@error varchar(512) output,
	@id integer output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp2', 0)
	declare @procedure varchar(64) = 'languages_add'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"entity":null,"id":null,"country":null,"name":null,"key":null,"code":null,"flag_id":null,"flag_name":null,"flag_mime":null,"flag_size":null,"flag_date_modified":null,"flag_path":null,"flag_hashes":null,"description":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	declare @suberror integer

	set nocount on
	set @error = ''
	set @id = 0 
	
	begin transaction

	set @parameters = json_modify(@parameters, 'strict $.country', @country)
	set @parameters = json_modify(@parameters, 'strict $.name', @name)
	set @parameters = json_modify(@parameters, 'strict $.key', @key)
	set @parameters = json_modify(@parameters, 'strict $.code', @code)
	set @parameters = json_modify(@parameters, 'strict $.flag_id', @flag_id)
	set @parameters = json_modify(@parameters, 'strict $.flag_name', @flag_name)
	set @parameters = json_modify(@parameters, 'strict $.flag_mime', @flag_mime)
	set @parameters = json_modify(@parameters, 'strict $.flag_size', @flag_size)
	set @parameters = json_modify(@parameters, 'strict $.flag_date_modified', @flag_date_modified)
	set @parameters = json_modify(@parameters, 'strict $.flag_path', @flag_path)
	set @parameters = json_modify(@parameters, 'strict $.flag_hashes', @flag_hashes)
	set @parameters = json_modify(@parameters, 'strict $.description', @description)
	set @parameters = json_modify(@parameters, 'strict $.entity', 0)
	set @parameters = json_modify(@parameters, 'strict $.id', 0)

	begin try

		execute dbo.sessions_validate_full @session, '["administrador"]', @error output, @session_id output, @ip_id output, @user_agent_id output
		if @flag_id is null and isnull(@flag_name, '') != '' begin 
			execute dbo.files_set @session_id, @ip_id, @user_agent_id, 'AnP.Languages.flag', @flag_name, @flag_mime, @flag_size, @flag_date_modified, @flag_path, @flag_hashes, 1, 1, @suberror output, @flag_id output
			set @error = dbo.errors_set(@error, dbo.bw(@suberror, -13), 38, 0)
		end
		set @parameters = json_modify(@parameters, 'strict $.flag_id', @flag_id)

		set @error = dbo.errors_join(@error, dbo.languages_insert_validate(@country, @name, @key, @code, @flag_id, @description), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin
			
			execute dbo.procedures_get @database, @procedure, @procedure_id output
		
			insert into dbo.Languages default values
			set @id = scope_identity()
			insert into dbo.LanguagesData([procedure], [session], entity, country, [name], [key], code, flag, [description]) values
				(@procedure_id, @session_id, @id, @country, @name, @key, @code, @flag_id, @description)
			
			set @parameters = json_modify(@parameters, 'strict $.entity', @id)
			set @parameters = json_modify(@parameters, 'strict $.id', scope_identity())
			
		end
	
		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go
	
