if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP
	
if object_id(N'dbo.FilesSingleView', N'V') is not null drop view dbo.FilesSingleView
go
create view dbo.FilesSingleView as select 
	files.id as id,
	[types].mime as [type],
	files.[name] as [name],
	files.[weight] as [weight]
from Files files 
join FilesTypes [types] on files.[type] = [types].id 
where 
	[types].date_out is null and 
	files.date_out is null
go
		
if object_id(N'dbo.FilesMultiView', N'V') is not null drop view dbo.FilesMultiView
go
create view dbo.FilesMultiView as select 
	files.id as id,
	[types].id as [type_id],
	[types].mime as [type],
	files.[name] as [name],
	files.[weight] as [weight],
	files.date_in as date_in
from Files files 
join FilesTypes [types] on files.[type] = [types].id 
where 
	files.date_out is null and 
	[types].date_out is null
go
		
if object_id(N'dbo.FilesFullView', N'V') is not null drop view dbo.FilesFullView
go
create view dbo.FilesFullView as select 
	files.id as id,
	[procedures].database_id as database_id,
	[procedures].[database] as [database],
	[procedures].id as procedure_id,
	[procedures].[procedure] as [procedure],
	[sessions].id as [session],
	[sessions].[user_id] as [user_id],
	[sessions].nick as nick,
	[sessions].ip_id as ip_id,
	[sessions].[ip] as [ip],
	[types].id as [type_id],
	[types].mime as [type],
	files.[name] as [name],
	files.[weight] as [weight],
	files.date_in as date_in
from Files files 
join FilesTypes [types] on files.[type] = [types].id 
join SessionsFullView [sessions] on files.[session] = [sessions].id 
join ProceduresFullView [procedures] on files.[procedure] = [procedures].id 
where 
	files.date_out is null and 
	[types].date_out is null
go
		
if object_id(N'dbo.LanguagesMultiView', N'V') is not null drop view dbo.LanguagesMultiView
go
create view dbo.LanguagesMultiView as select 
	languages.entity as id,
	countries.entity as country_id,
	countries.[name] as country,
	countries.[key] as country_key,
	countries.code as country_code, 
	flags.id as flag_id,
	flags.[type] as flag_type,
	flags.[name] as flag_name,
	flags.[weight] as flag_weight, 
	languages.[name] as [name],
	languages.[key] as [key],
	languages.code as code,
	languages.date_in as date_in
from _Languages languages 
join _Countries countries on languages.country = countries.entity 
join FilesSingleView flags on countries.flag = flags.id 
where 
	languages.deleted is null and 
	countries.deleted is null
go

if object_id(N'dbo.SessionsGroupsView', N'V') is not null drop view dbo.SessionsGroupsView
go
create view dbo.SessionsGroupsView as select distinct * from (
	select 
		[sessions].id as [session], 
		groups.entity as group_id, 
		groups.[name] as [group], 
		groups.is_entity as is_entity, 
		'ip' as [type] 
	from dbo.[Sessions] [sessions] 
	join dbo._SessionsIps sessions_ips on [sessions].id = sessions_ips.[session]
	join dbo._IpsGroups ips_groups on sessions_ips.[ip] = ips_groups.[ip] 
	join dbo._Groups groups on groups.entity = ips_groups.[group] 
	where 
		ips_groups.belongs = 1 and 
		groups.deleted is null
	union all 
	select 
		[sessions].id as [session], 
		groups.entity as group_id, 
		groups.[name] as [group], 
		groups.is_entity as is_entity, 
		'user' as [type] 
	from dbo.[Sessions] [sessions] 
	join dbo._UsersGroups users_groups on [sessions].[user] = users_groups.[user] 
	join dbo._Groups groups on groups.entity = users_groups.[group] 
	where 
		users_groups.belongs = 1 and 
		groups.deleted is null
) groups
go
	
if object_id(N'dbo.SessionsPermissionsView', N'V') is not null drop view dbo.SessionsPermissionsView
go
create view dbo.SessionsPermissionsView as select distinct 
	groups.[session] as [session],
	[permissions].id as permission_id,
	[permissions].[name] as permission
from dbo.SessionsGroupsView groups 
join dbo._PermissionsGroups permissions_groups on groups.group_id = permissions_groups.[group] 
join dbo._Permissions [permissions] on [permissions].entity = permissions_groups.permission 
where 
	[permissions].deleted is null and 
	permissions_groups.belongs = 1
go
	
if object_id(N'dbo.sessions_has_permissions', N'FN') is not null drop function dbo.sessions_has_permissions
go
create function dbo.sessions_has_permissions(
	@session integer,
	@permissions varchar(max)
) returns bit begin 

	set @permissions = ltrim(rtrim(@permissions))

	return (case 
		when isnull(@permissions, '') in ('', '[]') then 1 
		when substring(@permissions, 1, 1) = '[' then isnull((
			select top 1 1 from dbo.SessionsPermissionsView where [session] = @session and charindex('"' + permission + '"', @permissions) != 0
		), 0)
	else isnull((
		select 1 from dbo.SessionsPermissionsView where 
			[session] = @session and 
			permission = @permissions 
	), 0) end)
end
go
	
if object_id(N'dbo.sessions_is_banned', N'FN') is not null drop function dbo.sessions_is_banned
go
create function dbo.sessions_is_banned(
	@session integer
) returns bit begin 
	return isnull((
		select top 1 1 from dbo.SessionsPermissionsView where 
			[session] = @session and 
			permission = 'banned' 
	), 0)
end
go
	
-- Mirar la posibilidad de mejorar el rendimiento substituyendo el "where in" por un 
-- "join on min(group_id)" con selecci�n de "groups.*".
if object_id(N'dbo.SessionsSettingsView', N'V') is not null drop view dbo.SessionsSettingsView
go
create view dbo.SessionsSettingsView as select distinct 
	settings.entity as entity,
	[sessions].id as [session], 
	settings.[name] as [name], 
	settings_types.[name] as [type],
	(case when groups.entity is null then settings.[value] else groups.[value] end) as [value],
	isnull(groups.date_in, settings.date_in) as date_in
from dbo._Settings settings 
join dbo.[Sessions] [sessions] on 1 = 1 
join dbo._SettingsTypes settings_types on settings.[type] = settings_types.entity 
left join (
	select 
		settings_groups.entity as entity,
		settings_groups.settings as settings,
		settings_groups.[value] as [value],
		settings_groups.date_in as date_in,
		sessions_groups.[session] as [session]
	from dbo._SettingsGroups settings_groups
	join (
		select * from dbo.SessionsGroupsView where group_id in (
			select min(group_id) from dbo.SessionsGroupsView where [type] = 'user' and is_entity = 1 group by [session]
		)
	) sessions_groups on settings_groups.[group] = sessions_groups.group_id
	where settings_groups.belongs = 1
) groups on 
	[sessions].id = groups.[session] and 
	groups.settings = settings.entity
where 
	[sessions].date_out is null and 
	settings.deleted is null and 
	settings.[range] & dbo.bw(1, 1) != 0
go
	
if object_id(N'dbo.sessions_settings_get', N'FN') is not null drop function if exists dbo.sessions_settings_get
go
create function dbo.sessions_settings_get(
	@session integer,
	@name varchar(64)
) returns varchar(max) begin 
	return (select top 1 [value] from dbo.SessionsSettingsView where [session] = @session and [name] = @name order by date_in desc)
end
go
	
if object_id(N'dbo.sessions_settings_get_int', N'FN') is not null drop function dbo.sessions_settings_get_int
go
create function dbo.sessions_settings_get_int(
	@session integer,
	@name varchar(64)
) returns integer begin 
	return dbo.string_to_integer(dbo.sessions_settings_get(@session, @name))
end
go
	
if object_id(N'dbo.sessions_settings_get_bool', N'FN') is not null drop function dbo.sessions_settings_get_bool
go
create function dbo.sessions_settings_get_bool(
	@session integer,
	@name varchar(64)
) returns bit begin 
	return dbo.string_to_bool(dbo.sessions_settings_get(@session, @name))
end
go

if object_id('dbo.IpsSettingsView', N'V') is not null drop view dbo.IpsSettingsView
go
create view dbo.IpsSettingsView as select distinct 
	settings.entity as entity,
	ips.entity as [ip], 
	settings.[name] as [name], 
	settings_types.[name] as [type],
	(case when settings_ips_groups.[group] is null then settings.[value] else settings_ips_groups.[value] end) as [value]
from dbo._Settings settings 
join dbo._SettingsTypes settings_types on settings.[type] = settings_types.entity 
join dbo._Ips ips on 1 = 1 
left join (
	select 
		settings_groups.[group] as [group],
		settings_groups.settings as settings,
		ips_groups.[ip] as [ip],
		settings_groups.[value] as [value]
	from dbo._SettingsGroups settings_groups 
	join dbo._Groups groups on groups.is_entity = 1
	join dbo._IpsGroups ips_groups on 
		settings_groups.[group] = ips_groups.[group] and 
		ips_groups.[group] = groups.entity
	where 
		settings_groups.belongs = 1 and 
		ips_groups.belongs = 1
) settings_ips_groups on 
	settings_ips_groups.[ip] = ips.entity and 
	settings_ips_groups.settings = settings.entity 
where settings.[range] & dbo.bw(1, 1) != 0
go

if object_id('dbo.ips_settings_get', N'FN') is not null drop function dbo.ips_settings_get
go
create function dbo.ips_settings_get(
	@ip integer,
	@name varchar(64)
) returns varchar(max) begin 
	return (select top 1 [value] from dbo.IpsSettingsView where [ip] = @ip and [name] = @name)
end
go

if object_id('dbo.ips_settings_get_int', N'FN') is not null drop function dbo.ips_settings_get_int
go
create function dbo.ips_settings_get_int(
	@ip integer,
	@name varchar(64)
) returns integer begin 
	
	declare @value varchar(max) = dbo.ips_settings_get(@ip, @name)
	
	return dbo.string_to_integer(@value)
end
go

if object_id('dbo.ips_settings_get_bool', N'FN') is not null drop function dbo.ips_settings_get_bool
go
create function dbo.ips_settings_get_bool(
	@ip integer,
	@name varchar(64)
) returns bit begin 
	
	declare @value varchar(max) = dbo.ips_settings_get(@ip, @name)
	
	return dbo.string_to_bool(@value)
end
go

if object_id('dbo.UsersSettingsView', N'V') is not null drop view dbo.UsersSettingsView
go
create view dbo.UsersSettingsView as select distinct 
	settings.entity as entity,
	users.entity as [user], 
	settings.[name] as [name], 
	settings_types.[name] as [type],
	(case when settings_users_groups.[group] is null then settings.[value] else settings_users_groups.[value] end) as [value]
from dbo._Settings settings 
join dbo._SettingsTypes settings_types on settings.[type] = settings_types.entity
join dbo._Users users on 1 = 1
left join (
	select 
		settings_groups.[group] as [group],
		settings_groups.settings as settings,
		users_groups.[user] as [user],
		settings_groups.[value] as [value]
	from dbo._SettingsGroups settings_groups 
	join dbo._Groups groups on groups.is_entity = 1
	join dbo._UsersGroups users_groups on 
		settings_groups.[group] = users_groups.[group] and 
		users_groups.[group] = groups.entity
	where 
		settings_groups.belongs = 1 and 
		users_groups.belongs = 1
) settings_users_groups on 
	settings_users_groups.[user] = users.entity and 
	settings_users_groups.settings = settings.entity 
where settings.[range] & dbo.bw(1, 1) != 0
go

if object_id('dbo.users_settings_get', N'FN') is not null drop function dbo.users_settings_get
go
create function dbo.users_settings_get(
	@user integer,
	@name varchar(64)
) returns varchar(max) begin 
	return (select top 1 [value] from dbo.UsersSettingsView where [user] = @user and [name] = @name)
end
go

if object_id('dbo.users_settings_get_int', N'FN') is not null drop function dbo.users_settings_get_int
go
create function dbo.users_settings_get_int(
	@user integer,
	@name varchar(64)
) returns integer begin 
	
	declare @value varchar(max) = dbo.users_settings_get(@user, @name)
	
	return dbo.string_to_integer(@value)
end
go

if object_id('dbo.users_settings_get_bool', N'FN') is not null drop function dbo.users_settings_get_bool
go
create function dbo.users_settings_get_bool(
	@user integer,
	@name varchar(64)
) returns integer begin 
	
	declare @value varchar(max) = dbo.users_settings_get(@user, @name)
	
	return dbo.string_to_bool(@value)
end
go
	
if object_id(N'dbo.sessions_timeout', N'FN') is not null drop function dbo.sessions_timeout
go
create function dbo.sessions_timeout(
	@session integer
) returns bit begin 
	return isnull((
		select top 1 (case when 
			date_out is not null or 
			datediff(second, date_last, getdate()) > dbo.users_settings_get_int([user], 'session_timeout') 
		then 1 else 0 end) from dbo.[Sessions] where id = @session
	), 0)
end
go

if object_id(N'dbo.full_tables_fill', N'P') is not null drop procedure dbo.full_tables_fill
go
create procedure dbo.full_tables_fill 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'full_tables_fill'
	declare @procedure_id integer
	declare @name varchar(64)
	declare @default bit
	declare @description varchar(512)
	declare @group varchar(64)
	declare @permission varchar(64)
	declare @belongs bit
	declare @user varchar(32)
	declare @ip varchar(39)
	declare @is_entity bit
	declare @nick varchar(32)
	declare @value varchar(max)
	declare @user_id integer
	declare @machine varchar(32) = dbo.settings_get('machine_user', 0)
	declare @unknown varchar(32) = dbo.settings_get('unknown_user', 0)
	declare @guest varchar(32) = dbo.settings_get('guest_user', 0)

	set nocount on

	if object_id(N'tempdb..#default_groups_temporary_anp', N'U') is not null drop table #default_groups_temporary_anp
	if object_id(N'tempdb..#default_permissions_temporary_anp', N'U') is not null drop table #default_permissions_temporary_anp
	if object_id(N'tempdb..#default_permissions_groups_temporary_anp', N'U') is not null drop table #default_permissions_groups_temporary_anp
	if object_id(N'tempdb..#default_users_groups_temporary_anp', N'U') is not null drop table #default_users_groups_temporary_anp
	if object_id(N'tempdb..#default_ips_groups_temporary_anp', N'U') is not null drop table #default_ips_groups_temporary_anp
	if object_id(N'tempdb..#default_users_settings_temporary_anp', N'U') is not null drop table #default_users_settings_temporary_anp

	if @session is null 
		set @session = dbo.machine_get_session_id()
	
	create table #default_groups_temporary_anp(
		id integer not null identity(1, 1),
		[name] varchar(64) not null,
		[default] bit not null,
		is_entity bit not null,
		[description] varchar(max),
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)
	
	create table #default_permissions_temporary_anp(
		id integer not null identity(1, 1),
		[name] varchar(64) not null,
		[description] varchar(max),
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)
	
	create table #default_permissions_groups_temporary_anp(
		id integer not null identity(1, 1),
		permission varchar(64) not null,
		[group] varchar(64) not null,
		belongs bit not null default 1,
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)
	
	create table #default_users_groups_temporary_anp(
		id integer not null identity(1, 1),
		[user] varchar(32) not null,
		[group] varchar(64) not null,
		belongs bit not null default 1,
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)
	
	create table #default_ips_groups_temporary_anp(
		id integer not null identity(1, 1),
		[ip] varchar(39) not null,
		[group] varchar(64) not null,
		belongs bit not null default 1,
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)
	
	create table #default_users_settings_temporary_anp(
		id integer not null identity(1, 1),
		nick varchar(32) not null,
		[name] varchar(64) not null,
		[value] varchar(max),
		date_in datetime not null default getdate(),
		date_out datetime,
		primary key(id)
	)

	execute dbo.procedures_get @database, @procedure, @procedure_id output
	
	insert into #default_groups_temporary_anp([name], [default], is_entity, [description]) 
	select nick, 0, 1, concat('The entity group for ', nick, '.') from dbo._Users union all
	select [address], 0, 1, concat('The entity group for ', [address], '.') from dbo._Ips
		
	insert into #default_users_groups_temporary_anp([user], [group]) 
	select nick, nick from dbo._Users
		
	insert into #default_ips_groups_temporary_anp([ip], [group]) 
	select [address], [address] from dbo._Ips
	
	insert into #default_groups_temporary_anp([name], [default], is_entity, [description]) values 
		('bots', 0, 0, 'The item type is a bot.'),
		('users', 1, 0, 'The item type is a simple registered user.'),
		('guests', 0, 0, 'The item type is a Guest user.'),
		('banned', 0, 0, 'The item is banned.'),
		('administrators', 0, 0, 'The item type is a global administrator.')
	
	insert into #default_permissions_temporary_anp([name], [description]) values 
		('user', 'The item has only the simple registered user permissions.'),
		('guest', 'The item has only the Guest user perimssions.'),
		('banned', 'The item is banned.'),
		('administrator', 'The item has the global administrator.')
	
	insert into #default_permissions_groups_temporary_anp(permission, [group]) values 
		('user', 'users'),
		('guest', 'guests'),
		('banned', 'banned'),
		('administrator', 'administrators')
	
	insert into #default_users_groups_temporary_anp([user], [group]) values 
		('guest', 'guests'),
		('machine', 'bots'),
		('machine', 'administrators')
	
	insert into #default_users_settings_temporary_anp(nick, [name], [value]) values 
		(@machine, 'session_timeout', '0'),
		(@machine, 'users_maximum_sessions', '0'),
		(@unknown, 'session_timeout', '0'),
		(@unknown, 'users_maximum_sessions', '0'),
		(@guest, 'session_timeout', '3600'),
		(@guest, 'users_maximum_sessions', '2147483647')
			
	declare default_groups_cursor cursor for select [name], [default], is_entity, [description] from #default_groups_temporary_anp
	declare default_permissions_cursor cursor for select [name], [description] from #default_permissions_temporary_anp
	declare default_permissions_groups_cursor cursor for select permission, [group], belongs from #default_permissions_groups_temporary_anp
	declare default_users_groups_cursor cursor for select [user], [group], belongs from #default_users_groups_temporary_anp
	declare default_ips_groups_cursor cursor for select [ip], [group], belongs from #default_ips_groups_temporary_anp
	declare default_users_settings_cursor cursor for select nick, [name], [value] from #default_users_settings_temporary_anp
		
	open default_groups_cursor
	while 1 = 1 begin 
				
		fetch next from default_groups_cursor into @name, @default, @is_entity, @description
		if @@fetch_status != 0
			break

		if (select top 1 1 from dbo._Groups where [name] = @name) is null begin 
			insert into dbo.Groups default values
			insert into dbo.GroupsData([procedure], [session], entity, [name], [default], is_entity, [description]) values 
				(@procedure_id, @session, scope_identity(), @name, @default, @is_entity, @description)
		end
		
	end
	close default_groups_cursor
		
	open default_permissions_cursor
	while 1 = 1 begin 
				
		fetch next from default_permissions_cursor into @name, @description
		if @@fetch_status != 0
			break

		if (select top 1 1 from dbo._Permissions where [name] = @name) is null begin 
			insert into dbo.[Permissions] default values
			insert into dbo.PermissionsData([procedure], [session], entity, [name], [description]) values 
				(@procedure_id, @session, scope_identity(), @name, @description)
		end
		
	end
	close default_permissions_cursor
		
	open default_permissions_groups_cursor
	while 1 = 1 begin 
				
		fetch next from default_permissions_groups_cursor into @permission, @group, @belongs
		if @@fetch_status != 0
			break

		if (
			select top 1 1 
			from dbo._PermissionsGroups permissions_groups 
			join dbo._Permissions [permissions] on permissions_groups.permission = [permissions].entity 
			join dbo._Groups groups on permissions_groups.[group] = groups.entity 
			where 
				[permissions].[name] = @permission and 
				groups.[name] = @group 
		) is null begin 
			insert into dbo.PermissionsGroups default values
			insert into dbo.PermissionsGroupsData([procedure], [session], entity, permission, [group], belongs) values 
				(@procedure_id, @session, scope_identity(), (
					select top 1 entity from dbo._Permissions where [name] = @permission
				), (
					select top 1 entity from dbo._Groups where [name] = @group
				), @belongs)
		end
		
	end
	close default_permissions_groups_cursor
		
	open default_users_groups_cursor
	while 1 = 1 begin 
				
		fetch next from default_users_groups_cursor into @user, @group, @belongs
		if @@fetch_status != 0
			break

		if (
			select top 1 1 
			from dbo._UsersGroups users_groups 
			join dbo._Users users on users_groups.[user] = users.entity 
			join dbo._Groups groups on users_groups.[group] = groups.entity 
			where 
				users.nick = @user and 
				groups.[name] = @group 
		) is null begin 
			insert into dbo.UsersGroups default values
			insert into dbo.UsersGroupsData([procedure], [session], entity, [user], [group], belongs) values 
				(@procedure_id, @session, scope_identity(), (
					select top 1 entity from dbo._Users where nick = @user
				), (
					select top 1 entity from dbo._Groups where [name] = @group
				), @belongs)
		end
		
	end
	close default_users_groups_cursor
		
	open default_ips_groups_cursor
	while 1 = 1 begin 
				
		fetch next from default_ips_groups_cursor into @ip, @group, @belongs
		if @@fetch_status != 0
			break

		if (
			select top 1 1 
			from dbo._IpsGroups ips_groups 
			join dbo._Ips ips on ips_groups.[ip] = ips.entity 
			join dbo._Groups groups on ips_groups.[group] = groups.entity 
			where 
				ips.[address] = @ip and 
				groups.[name] = @group 
		) is null begin 
			insert into dbo.IpsGroups default values
			insert into dbo.IpsGroupsData([procedure], [session], entity, [ip], [group], belongs) values 
				(@procedure_id, @session, scope_identity(), (
					select top 1 entity from dbo._Ips where [address] = @ip
				), (
					select top 1 entity from dbo._Groups where [name] = @group
				), @belongs)
		end
		
	end
	close default_ips_groups_cursor
		
	open default_users_settings_cursor
	while 1 = 1 begin 
				
		fetch next from default_users_settings_cursor into @nick, @name, @value
		if @@fetch_status != 0
			break

		set @user_id = (select top 1 entity from dbo._Users where deleted is null and nick = @nick)

		if dbo.users_settings_get(@user_id, @name) != @value begin 
			insert into dbo.SettingsGroups default values
			insert into dbo.SettingsGroupsData([procedure], [session], entity, [group], settings, [value], belongs) values 
				(@procedure_id, @session, scope_identity(), (
					select top 1 entity from dbo._Groups where deleted is null and [name] = @nick
				), (
					select top 1 entity from dbo._Settings where deleted is null and [name] = @name
				), @value, 1)
		end
		
	end
	close default_users_settings_cursor
	
	drop table #default_groups_temporary_anp
	drop table #default_permissions_temporary_anp
	drop table #default_permissions_groups_temporary_anp
	drop table #default_users_groups_temporary_anp
	drop table #default_ips_groups_temporary_anp
	drop table #default_users_settings_temporary_anp

end
go

execute dbo.full_tables_fill null

if object_id(N'dbo.LogsFullView', N'V') is not null drop view dbo.LogsFullView
go
create view dbo.LogsFullView as select 
	logs.id as id,
	[procedures].database_id as database_id,
	[procedures].id as procedure_id,
	[procedures].[database] as [database],
	[procedures].[procedure] as [procedure],
	[sessions].id as session_id,
	[sessions].[user_id] as [user_id],
	[sessions].nick as nick,
	[sessions].ip_id as ip_id,
	[sessions].[ip] as [ip],
	errors.id as error_id,
	errors.code as error,
	[parameters].id as parameters_id,
	[parameters].[json] as [parameters],
	logs.date_in as date_in
from dbo.Logs logs
join dbo.ProceduresFullView [procedures] on logs.[procedure] = [procedures].id
join dbo.SessionsFullView [sessions] on logs.[session] = [sessions].id
join dbo.Errors errors on logs.own_error = errors.id 
join dbo.[Parameters] [parameters] on logs.own_parameters = [parameters].id
go

if object_id(N'dbo.ExceptionsFullView', N'V') is not null drop view dbo.ExceptionsFullView
go
create view dbo.ExceptionsFullView as select 
	exceptions.id as id,
	[procedures].database_id as database_id,
	[procedures].id as procedure_id,
	[procedures].[database] as [database],
	[procedures].[procedure] as [procedure],
	[sessions].id as session_id,
	[sessions].[user_id] as [user_id],
	[sessions].nick as nick,
	[sessions].ip_id as ip_id,
	[sessions].[ip] as [ip],
	errors.id as error_id,
	errors.code as error,
	[parameters].id as parameters_id,
	[parameters].[json] as [parameters],
	[messages].id as message_id,
	[messages].[text] as [message],
	exceptions.[status] as [status],
	exceptions.code as code,
	exceptions.date_in as date_in
from dbo.Exceptions exceptions
join dbo.ProceduresFullView [procedures] on exceptions.[procedure] = [procedures].id
join dbo.SessionsFullView [sessions] on exceptions.[session] = [sessions].id
join dbo.Errors errors on exceptions.own_error = errors.id 
join dbo.[Parameters] [parameters] on exceptions.own_parameters = [parameters].id
join dbo.[Messages] [messages] on exceptions.[message] = [messages].id
go
	
if object_id(N'dbo.TracesFullView', N'V') is not null drop view dbo.TracesFullView
go
create view dbo.TracesFullView as select 
	[traces].id as id, 
	[sessions].id as session_id, 
	[sessions].[user_id] as [user_id],
	[sessions].nick as nick,
	[sessions].ip_id as ip_id,
	[sessions].[ip] as [ip],
	[sessions].user_agent_id as user_agent_id,
	[sessions].user_agent as user_agent, 
	[sessions].date_in as session_date_in, 
	[sessions].date_out as session_date_out,
	[sessions].date_last as session_date_last, 
	projects.id as project_id, 
	projects.[name] as project, 
	files.id as [file_id], 
	files.[path] as [path],
	errors.id as error_id, 
	errors.code as error, 
	[parameters].id as parameters_id, 
	[parameters].[json] as [parameters], 
	lines.id as traces_lines_id, 
	lines.trace as trace, 
	[messages].id as message_id, 
	[messages].[text] as [message], 
	[traces].line as line, 
	[traces].date_in as date_in 
from dbo.[Traces] [traces] 
join dbo.SessionsFullView [sessions] on [traces].[session] = [sessions].id 
join dbo.Projects projects on [traces].project = projects.id 
join dbo.TracesFiles files on [traces].[file] = files.id 
join dbo.Errors errors on [traces].own_error = errors.id 
join dbo.[Parameters] [parameters] on [traces].own_parameters = [parameters].id
join dbo.Lines lines on [traces].trace = lines.id 
join dbo.[Messages] [messages] on [traces].[message] = [messages].id
go
	
if object_id(N'dbo.LanguagesIdSelectView', N'V') is not null drop view dbo.LanguagesIdSelectView
go
create view dbo.LanguagesIdSelectView as select entity as id, [key] from dbo._Languages where deleted is null
go
	
if object_id(N'dbo.languages_get_id_select', N'P') is not null drop procedure dbo.languages_get_id_select
go
create procedure dbo.languages_get_id_select as begin 

	set nocount on
		
	select * from dbo.LanguagesIdSelectView
		
end
go

if object_id(N'dbo.languages_validate', N'FN') is not null drop function dbo.languages_validate
go
create function dbo.languages_validate(
	@language_id integer,
	@undeleted bit
) returns integer begin 
	return (case 
		when @language_id is null then dbo.bw(1, 0)
		when @language_id < 1 then dbo.bw(1, 1)
	else isnull((
		select top 1 (case 
			when @undeleted = 1 then (case when deleted is null then 0 else dbo.bw(1, 3) end)
			else (case when deleted is null then dbo.bw(1, 3) else 0 end) 
		end) from dbo._Languages where entity = @language_id
	), dbo.bw(1, 2)) end)
end
go

if object_id(N'dbo.locates_set_raw', N'P') is not null drop procedure dbo.locates_set_raw
go
create procedure dbo.locates_set_raw 
	@session_id integer,
	@ip_id integer, 
	@user_agent_id integer, 
	@group varchar(64), 
	@coordenates varchar(32),
	@owner bit,
	@error integer output,
	@id integer output
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'locates_set_raw'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"group":null,"coordenates":null,"owner":null,"id":0}'

	set nocount on
	set @error = 0
	set @id = 0
	set @parameters = json_modify(@parameters, 'strict $.group', @group)
	set @parameters = json_modify(@parameters, 'strict $.coordenates', @coordenates)
	set @parameters = json_modify(@parameters, 'strict $.owner', @owner)

	execute dbo.procedures_get @database, @procedure, @procedure_id output

	begin try

		set @error = dbo.bw((
			/*dbo.bw((case 
				when @session_id is null then dbo.bw(1, 0)
				when @session_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case when date_out is null then 0 else dbo.bw(1, 3) end) from dbo.[Sessions] where id = @session_id
			), dbo.bw(1, 2)) end), 0) |
			dbo.bw((case 
				when @ip_id is null then dbo.bw(1, 0)
				when @ip_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case 
					when deleted is null then dbo.bw(1, 3) 
					when [accessible] = 0 then dbo.bw(1, 4) 
				else 0 end) from dbo._Ips where entity = @ip_id
			), dbo.bw(1, 2)) end), 4) |
			dbo.bw((case 
				when @user_agent_id is null then dbo.bw(1, 0)
				when @user_agent_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case when date_out is null then dbo.bw(1, 3) else 0 end) from dbo.UsersAgents where id = @user_agent_id
			), dbo.bw(1, 2)) end), 9) |*/
			dbo.bw((case 
				when @coordenates is null then dbo.bw(1, 0)
				when @coordenates = '' then dbo.bw(1, 1)
				when @coordenates not like '%,%,%' then dbo.bw(1, 2)
			else 0 end), 13) |
			dbo.bw((case when @owner is null then dbo.bw(1, 0) else 0 end), 16) |
			dbo.bw((case 
				when @group is null then dbo.bw(1, 0)
				when @group = '' then dbo.bw(1, 1)
			else 0 end), 17) |
		0), 1)

		if @error = 0 begin 
			
			declare @longitude_i integer = charindex(',', @coordenates)
			declare @longitude float = convert(float, substring(@coordenates, 1, @longitude_i))
			declare @latitude_i integer = charindex(',', @coordenates, @longitude_i + 1)
			declare @latitude float = convert(float, substring(@coordenates, @longitude_i + 1, @latitude_i))
			declare @altitude float = convert(float, substring(@coordenates, @latitude_i + 1, 32))

			set @error = dbo.bw((
				dbo.bw((case 
					when @longitude is null then dbo.bw(1, 0)
					when @longitude < -90 then dbo.bw(1, 1)
					when @longitude > 90 then dbo.bw(1, 2)
				else 0 end), 0) |
				dbo.bw((case 
					when @latitude is null then dbo.bw(1, 0)
					when @latitude < -180 then dbo.bw(1, 1)
					when @latitude > 180 then dbo.bw(1, 2)
				else 0 end), 3) |
				dbo.bw((case 
					when @altitude is null then dbo.bw(1, 0)
					when @altitude < -50000 then dbo.bw(1, 1)
					when @altitude > 100000 then dbo.bw(1, 2)
				else 0 end), 6) |
			0), 19)

			if @error = 0 begin 

				declare @group_id integer = (select top 1 id from dbo.LocatesGroups where deleted is null and [name] = @group)

				if @group_id is null begin 
					insert into dbo.LocatesGroups([procedure], [session], [name]) values
						(@procedure_id, @session_id, @group)
					set @group_id = scope_identity()
				end
				
				if @owner = 1 
					set @id = isnull((
						select top 1 id from dbo.Locates where 
							deleted is null and 
							[group] = @group_id and 
							longitude = @longitude and 
							latitude = @latitude and 
							altitude = @altitude
					), 0)

				if @id = 0 begin 
					insert into dbo.Locates([procedure], [session], [group], longitude, latitude, altitude) values 
						(@procedure_id, @session_id, @group_id, @longitude, @latitude, @altitude)
					set @id = scope_identity()
				end

				set @parameters = json_modify(@parameters, 'strict $.id', @id)

			end

		end

	end try
	begin catch
		set @error = @error | dbo.bw(1, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0

end
go

if object_id(N'dbo.locates_set', N'P') is not null drop procedure dbo.locates_set
go
create procedure dbo.locates_set 
	@session_id integer,
	@ip_id integer, 
	@user_agent_id integer, 
	@group varchar(64), 
	@longitude float, 
	@latitude float, 
	@altitude float, 
	@owner bit,
	@error integer output,
	@id integer output
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'locates_set'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"group":null,"longitude":null,"latitude":null,"altitude":null,"owner":null,"id":0}'

	set nocount on
	set @error = 0
	set @id = 0
	set @parameters = json_modify(@parameters, 'strict $.group', @group)
	set @parameters = json_modify(@parameters, 'strict $.longitude', @longitude)
	set @parameters = json_modify(@parameters, 'strict $.latitude', @latitude)
	set @parameters = json_modify(@parameters, 'strict $.altitude', @altitude)
	set @parameters = json_modify(@parameters, 'strict $.owner', @owner)

	execute dbo.procedures_get @database, @procedure, @procedure_id output

	begin try

		set @error = dbo.bw((
			/*dbo.bw((case 
				when @session_id is null then dbo.bw(1, 0)
				when @session_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case when date_out is null then 0 else dbo.bw(1, 3) end) from dbo.[Sessions] where id = @session_id
			), dbo.bw(1, 2)) end), 0) |
			dbo.bw((case 
				when @ip_id is null then dbo.bw(1, 0)
				when @ip_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case 
					when deleted is null then dbo.bw(1, 3) 
					when [accessible] = 0 then dbo.bw(1, 4) 
				else 0 end) from dbo._Ips where entity = @ip_id
			), dbo.bw(1, 2)) end), 4) |
			dbo.bw((case 
				when @user_agent_id is null then dbo.bw(1, 0)
				when @user_agent_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case when date_out is null then dbo.bw(1, 3) else 0 end) from dbo.UsersAgents where id = @user_agent_id
			), dbo.bw(1, 2)) end), 9) |*/
			dbo.bw((case 
				when @group is null then dbo.bw(1, 0) 
				when @group = '' then dbo.bw(1, 1) 
			else 0 end), 13) |
			dbo.bw((case 
				when @longitude is null then dbo.bw(1, 0)
				when @longitude < -90 then dbo.bw(1, 1)
				when @longitude > 90 then dbo.bw(1, 2)
			else 0 end), 15) |
			dbo.bw((case 
				when @latitude is null then dbo.bw(1, 0)
				when @latitude < -180 then dbo.bw(1, 1)
				when @latitude > 180 then dbo.bw(1, 2)
			else 0 end), 18) |
			dbo.bw((case 
				when @altitude is null then dbo.bw(1, 0)
				-- when @altitude < -50000 then dbo.bw(1, 1)
				-- when @altitude > 100000 then dbo.bw(1, 2)
			else 0 end), 21) |
			dbo.bw((case when @owner is null then dbo.bw(1, 0) else 0 end), 24) |
		0), 1)

		if @error = 0 begin 

			declare @group_id integer = (select top 1 id from dbo.LocatesGroups where deleted is null and [name] = @group)

			if @group_id is null begin 
				insert into dbo.LocatesGroups([procedure], [session], [name]) values
					(@procedure_id, @session_id, @group)
				set @group_id = scope_identity()
			end
				
			if @owner = 1 
				set @id = isnull((
					select top 1 id from dbo.Locates where 
						deleted is null and 
						[group] = @group_id and 
						longitude = @longitude and 
						latitude = @latitude and 
						altitude = @altitude
				), 0)

			if @id = 0 begin 
				insert into dbo.Locates([procedure], [session], [group], longitude, latitude, altitude) values 
					(@procedure_id, @session_id, @group_id, @longitude, @latitude, @altitude)
				set @id = scope_identity()
			end

			set @parameters = json_modify(@parameters, 'strict $.id', @id)

		end

	end try
	begin catch
		set @error = @error | dbo.bw(1, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0

end
go

if object_id(N'dbo.files_set', N'P') is not null drop procedure dbo.files_set
go
create procedure dbo.files_set 
	@session_id integer,
	@ip_id integer,
	@user_agent_id integer,
	@group varchar(256),
	@name varchar(256),
	@type varchar(64),
	@weight integer, 
	@date_modified bigint, 
	@path varchar(512),
	@hashes varchar(max), 
	@owner bit,
	@allow_null bit,
	@error integer output,
	@id integer output 
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'files_set'
	declare @procedure_id integer
	declare @parameters varchar(max) = '{"group":null,"name":null,"type":null,"weight":null,"date_modified":null,"path":null,"owner":null}'
	
	set nocount on
	set @error = 0
	set @id = 0
	set @parameters = json_modify(@parameters, 'strict $.group', @group)
	set @parameters = json_modify(@parameters, 'strict $.name', @name)
	set @parameters = json_modify(@parameters, 'strict $.type', @type)
	set @parameters = json_modify(@parameters, 'strict $.weight', @weight)
	set @parameters = json_modify(@parameters, 'strict $.date_modified', @date_modified)
	set @parameters = json_modify(@parameters, 'strict $.path', @path)
	set @parameters = json_modify(@parameters, 'strict $.owner', @owner)

	begin try 
	
		execute dbo.procedures_get @database, @procedure, @procedure_id output
	
		set @error = dbo.bw((
			/*dbo.bw((case 
				when @session_id is null then dbo.bw(1, 0)
				when @session_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case when date_out is null then 0 else dbo.bw(1, 3) end) from dbo.[Sessions] where id = @session_id
			), dbo.bw(1, 2)) end), 0) |
			dbo.bw((case 
				when @ip_id is null then dbo.bw(1, 0)
				when @ip_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case 
					when deleted is null then dbo.bw(1, 3) 
					when accessible = 0 then dbo.bw(1, 4) 
				else 0 end) from dbo._Ips where entity = @ip_id
			), dbo.bw(1, 2)) end), 4) |
			dbo.bw((case 
				when @user_agent_id is null then dbo.bw(1, 0)
				when @user_agent_id < 1 then dbo.bw(1, 1)
			else isnull((
				select top 1 (case when date_out is null then dbo.bw(1, 3) else 0 end) from dbo.UsersAgents where id = @user_agent_id
			), dbo.bw(1, 2)) end), 9) |*/
			dbo.bw((case 
				when @group is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end)
				when @group = '' then dbo.bw(1, 1)
			else 0 end), 13) |
			dbo.bw((case 
				when @name is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end)
				when @name = '' then dbo.bw(1, 1)
			else 0 end), 15) |
			dbo.bw((case 
				when @type is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end)
				when @type = '' then dbo.bw(1, 1)
			else 0 end), 17) |
			dbo.bw((case 
				when @weight is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end)
				when @weight < 0 then dbo.bw(1, 1)
				when @weight < 1 then dbo.bw(1, 2)
			else 0 end), 19) |
			dbo.bw((case when @date_modified is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end) else 0 end), 22) |
			dbo.bw((case 
				when @path is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end)
				when @path = '' then dbo.bw(1, 1)
			else 0 end), 23) |
			dbo.bw((case 
				when @hashes is null then (case when @allow_null = 1 then dbo.bw(1, 0) else 0 end)
				when @hashes = '' then dbo.bw(1, 1)
			else 0 end), 25) |
			dbo.bw((case when @owner is null and @allow_null = 1 then dbo.bw(1, 0) else 0 end), 27) |
		0), 1)

		if @error = 0 begin 

			declare @group_id integer = (select top 1 id from dbo.FilesGroups where date_out is null and deleted is null and [name] = @group)
			declare @type_id integer = (select top 1 id from dbo.FilesTypes where date_out is null and deleted is null and [mime] = @type)

			if @group_id is null begin 
				insert into dbo.FilesGroups([procedure], [session], [name]) values
					(@procedure_id, @session_id, @group)
				set @group_id = scope_identity()
			end

			if @type_id is null begin 
				insert into FilesTypes([procedure], [session], mime) values
					(@procedure_id, @session_id, @type)
				set @type_id = scope_identity()
			end
			
			if @owner = 1 
				set @id = isnull((
					select top 1 id from dbo.Files where 
						date_out is null and 
						deleted is null and 
						[group] = @group_id and 
						[type] = @type_id and 
						[name] = @name and 
						[path] = @path and 
						[weight] = @weight 
				), 0)

			if isnull(@id, 0) = 0 begin 
				
				declare @hash_name varchar(16)
				declare @hash_value varchar(256)
				declare @hash_colon integer
				declare @algorithm_id integer
				
				insert into dbo.Files([procedure], [session], [group], [type], [name], [path], [weight], date_modified) values 
					(@procedure_id, @session_id, @group_id, @type_id, @name, @path, @weight, @date_modified)
				set @id = scope_identity()
			
				while len(@hashes) != 0 begin 
					
					set @hash_colon = charindex(':', @hashes)
					
					if @hash_colon = 0 
						set @hashes = ''
					else begin 
						
						set @hash_name = substring(@hashes, 1, @hash_colon - 1)
						set @hashes = substring(@hashes, @hash_colon + 1, 999999999)
						set @hash_colon = charindex(',', @hashes)
						
						if @hash_colon = 0 
							set @hash_colon = 99999999
							
						set @hash_value = substring(@hashes, 1, @hash_colon - 1)
						set @hashes = substring(@hashes, @hash_colon + 1, 999999999)
						set @algorithm_id = (select top 1 id from Hashes where date_out is null and deleted is null and [name] = @hash_name)
					
						if @algorithm_id is null begin 
							insert into dbo.Hashes([procedure], [session], [name]) values 
								(@procedure_id, @session_id, @hash_name)
							set @algorithm_id = scope_identity()
						end
					
						insert into dbo.FilesHashes([procedure], [session], [file], [algorithm], [hash]) values 
							(@procedure_id, @session_id, @id, @algorithm_id, @hash_value)
							
					end
				
				end
				
			end

		end

	end try
	begin catch
		set @error = @error | dbo.bw(1, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 0
		
end
go

if object_id(N'dbo.FilesLoadItemView', N'V') is not null drop view dbo.FilesLoadItemView
go
create view dbo.FilesLoadItemView as select 
	files.id as id, 
	groups.id as group_id, 
	groups.name as [group], 
	types.id as type_id, 
	types.mime as mime, 
	files.name as name,
	files.[path] as [path], 
	files.weight as [size], 
	files.date_modified as last_modified, 
	files.description as description 
from dbo.Files files 
join dbo.FilesGroups groups on files.[group] = groups.id 
join dbo.FilesTypes types on files.[type] = types.id 
where 
	files.date_out is null and files.deleted is null and 
	groups.date_out is null and groups.deleted is null and 
	types.date_out is null and types.deleted is null
go
		
if object_id(N'dbo.files_get_path', N'P') is not null drop procedure dbo.files_get_path
go
create procedure files_get_path 
	@session varchar(max),
	@file_id integer, 
	@group varchar(256), 
	@file_name varchar(256), 
	@extra_permissions varchar(max),
	@error varchar(512) output,
	@path varchar(512) output, 
	@file_type varchar(64) output 
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'files_get_path'
	declare @parameters varchar(max) = '{}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	declare @has_errors bit

	set nocount on
	set @error = ''
	set @path = null
	set @parameters = json_modify(@parameters, '$.file_name', @file_name)
	set @parameters = json_modify(@parameters, '$.extra_permissions', @extra_permissions)

	begin try
	
		execute dbo.sessions_validate_full @session, @extra_permissions, @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_set(@error, (
			dbo.bw(dbo.files_validate(@file_id, 1), 0) | 
			dbo.bw((case 
				when @group is null then dbo.bw(1, 0) 
				when @group = '' then dbo.bw(1, 1) 
			else 0 end), 4) |
			dbo.bw((case 
				when @file_name is null then 0 
				when @file_name = '' then 0 
			else isnull((
				select top 1 (
					(case when [name] != @file_name then dbo.bw(1, 3) else 0 end) | 
					(case when deleted is null then 0 else dbo.bw(1, 4) end) | 
					(case when date_out is null then 0 else dbo.bw(1, 5) end) | 
				0) from dbo.Files where id = @file_id
			), dbo.bw(1, 2)) end), 6) | 
		0), 30, 0)
		set @has_errors = dbo.errors_has(@error)
		
		if @has_errors = 0 begin 
			
			declare @group_id integer = (select top 1 id from dbo.FilesGroups where date_out is null and deleted is null and [name] = @group)
		
			set @error = dbo.errors_set(@error, (case 
				when @group_id is null then dbo.bw(1, 0) 
				when (select top 1 1 from dbo.Files where id = @file_id and [group] = @group_id) is null then dbo.bw(1, 1) 
			else 0 end), 42, 0)
			set @has_errors = dbo.errors_has(@error)
			
		end
	
		select top 1 
			@path = [path], 
			@file_type = [mime] 
		from dbo.FilesLoadItemView where @has_errors = 0 and id = @file_id

	end try
	begin catch
		set @error = @error | dbo.bw(1, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 4
		
end
go

if object_id(N'dbo.locates_errors', N'FN') is not null drop function dbo.locates_errors
go
create function dbo.locates_errors(
	@error integer
) returns integer begin 
	return dbo.bw(dbo.bw(@error, -10), 1) | (@error & 1)
end
go

if object_id(N'dbo.sessions_get_user', N'FN') is not null drop function dbo.sessions_get_user 
go
create function dbo.sessions_get_user(
	@session integer
) returns integer begin 
	return (select top 1 [user] from dbo.Sessions where id = @session)
end
go
