if (select top 1 1 from sys.databases where name = 'AnP') is null create database AnP
go
use AnP

if object_id(N'dbo.ips_validate_get', N'FN') is not null drop function dbo.ips_validate_get
go
create function dbo.ips_validate_get(
	@ip varchar(39)
) returns varchar(512) begin 
	return dbo.errors_set('', (case 
		when @ip is null then dbo.bw(1, 0)
		when @ip = '' then dbo.bw(1, 1)
		-- when @ip like '[^0-9a-fA-F.:]' then dbo.bw(1, 2)
	else isnull((
		select top 1 (
			(case when ips.accessible = 1 then 0 else dbo.bw(1, 3) end) |
			isnull((
				select top 1 dbo.bw(1, 4) 
				from dbo._IpsGroups ips_groups
				join dbo._Groups groups on ips_groups.[group] = groups.entity 
				where 
					ips_groups.belongs = 1 and 
					groups.deleted is null and 
					ips_groups.[ip] = ips.entity and 
					groups.[name] = 'banned' 
			), 0) |
			(case when (
				select top 1 count(1) from dbo.SessionsFullView where 
					ip_id = ips.entity and 
					datediff(second, date_last, getdate()) < dbo.ips_settings_get_int(ips.entity, 'session_timeout')
			) < dbo.ips_settings_get_int(ips.entity, 'ips_maximum_sessions') then 0 else dbo.bw(1, 5) end) | 
		0) from dbo._Ips ips where ips.deleted is null and ips.[address] = @ip
	), 0) end), 0, 0)
end
go
	
if object_id(N'dbo.sessions_get_guest', N'P') is not null drop procedure dbo.sessions_get_guest
go
create procedure dbo.sessions_get_guest 
	@session varchar(max),
	@error varchar(512) output,
	@id integer output, 
	@user varchar(32) output, 
	@timeout integer output 
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 1)
	declare @procedure varchar(64) = 'sessions_get_guest'
	declare @parameters varchar(max) = '{}'
	declare @has_error bit
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	
	set @parameters = json_modify(@parameters, '$.id', 0)
	set @error = ''
	set @id = 0

	begin try

		
		execute dbo.sessions_validate_full @session, null, @error output, @session_id output, @ip_id output, @user_agent_id output

		if dbo.errors_get_bit(@error, 1) != 0 or dbo.errors_get_bit(@error, 4) != 0 begin 
			set @session_id = dbo.sessions_get_machine()
			set @error = ''
			if @ip_id is null 
				set @ip_id = (select top 1 entity from dbo._Ips where deleted is null and [address] = dbo.settings_get('local_ip', 0))
			if @user_agent_id is null 
				set @user_agent_id = (select top 1 id from dbo.UsersAgents where date_out is null and [data] = dbo.settings_get('default_user_agent', 0))
		end
		
		set @has_error = (case when dbo.errors_has(@error) = 0 then 0 else 1 end)
		set @id = 0
		
		if @has_error = 0 begin 
			
			declare @user_id integer

			set @user = dbo.settings_get('guest_user', 0)
			set @user_id = (select top 1 entity from dbo._Users where nick = @user)

			insert into dbo.[Sessions]([user]) values(@user_id)
			set @id = scope_identity()
			insert into dbo.SessionsIps default values
			insert into dbo.SessionsIpsData(entity, [session], [ip], ok) values 
				(scope_identity(), @id, @ip_id, 1)
			insert into dbo.SessionsUsersAgents default values
			insert into dbo.SessionsUsersAgentsData(entity, [session], user_agent, ok) values 
				(scope_identity(), @id, @user_agent_id, 1)

			set @timeout = dbo.users_settings_get_int(@user_id, 'session_timeout')
			
			set @parameters = json_modify(@parameters, '$.id', @id)

		end

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	select permission from dbo.SessionsPermissionsView where @has_error = 0 and [session] = @id
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 1
		
end
go

if object_id(N'dbo.sessions_update', N'P') is not null drop procedure dbo.sessions_update
go
create procedure dbo.sessions_update 
	@session varchar(max),
	@error varchar(512) output,
	@user varchar(32) output,
	@timeout integer output, 
	@reset_password bit output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_update'
	declare @parameters varchar(max) = '{}'
	declare @has_error bit
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set @error = ''

	begin try
	
		execute dbo.sessions_validate_full @session, null, @error output, @session_id output, @ip_id output, @user_agent_id output
		set @has_error = (case when dbo.errors_has(@error) = 0 then 0 else 1 end)
	
		if @has_error = 0 begin 

			declare @user_id integer = (select top 1 [user] from dbo.[Sessions] where id = @session_id)

			update dbo.[Sessions] set date_last = getdate() where id = @session_id
			
			select top 1 
				@user = users.nick, 
				@reset_password = (case when isnull(users.password, '') = '' then 1 else 0 end) 
			from dbo._Users users
			join dbo.[Sessions] [sessions] on [sessions].[user] = users.entity
			where [sessions].id = @session_id
			
			set @timeout = dbo.users_settings_get_int(@user_id, 'session_timeout')

		end else if dbo.errors_get_bit(@error, 13) = 1 
			update dbo.[Sessions] set date_out = getdate() where id = @session_id

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	select permission from dbo.SessionsPermissionsView where @has_error = 0 and [session] = @session_id
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 4
	
end
go

if object_id(N'dbo.sessions_close', N'P') is not null drop procedure dbo.sessions_close
go
create procedure dbo.sessions_close 
	@session varchar(max),
	@error varchar(512) output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_close'
	declare @parameters varchar(max) = '{}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer

	set @error = ''

	begin try
	
		execute dbo.sessions_validate_full @session, null, @error output, @session_id output, @ip_id output, @user_agent_id output
	
		if dbo.errors_has(@error) = 0
			update dbo.[Sessions] set date_out = getdate() where id = @session_id

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 1
	
end
go

if object_id(N'dbo.sessions_register_validate', N'FN') is not null drop function dbo.sessions_register_validate
go
create function dbo.sessions_register_validate(
	@nick varchar(32),
	@email varchar(256),
	@password varchar(256)
) returns varchar(512) begin 
		
	declare @error varchar(512) = ''
	
	set @error = dbo.errors_set(@error, (case 
		when @nick is null then dbo.bw(1, 0)
		when @nick = '' then dbo.bw(1, 1)
	else isnull((
		select top 1 dbo.bw(1, 2) from dbo._Users where nick = @nick
	), isnull((
		select top 1 dbo.bw(1, 3) from dbo._Groups where [name] = @nick
	), 0)) end), 0, 0)
	
	set @error = dbo.errors_set(@error, (case 
		when @email is null then dbo.bw(1, 0)
		when @email = '' then dbo.bw(1, 1)
		when @email not like '%@%.%' then dbo.bw(1, 2)
	else (case when isnull(dbo.settings_get_bool('mail_allow_reuse', 0), 0) = 1 then 0 else isnull((
		select top 1 dbo.bw(1, 3) from dbo.Emails where deleted is null and [address] = @email
	), 0) end) end), 4, 0)
	
	set @error = dbo.errors_set(@error, dbo.password_validate(@password), 8, 0)
	
	return @error
end
go

if object_id(N'dbo.sessions_register', N'P') is not null drop procedure dbo.sessions_register
go
create procedure dbo.sessions_register 
	@session varchar(max),
	@nick varchar(32),
	@email varchar(256),
	@password varchar(256),
	@error varchar(512) output,
	@id integer output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_register'
	declare @parameters varchar(max) = '{"nick":null,"email":null,"password":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	
	set @parameters = json_modify(@parameters, 'strict $.nick', @nick)
	set @parameters = json_modify(@parameters, 'strict $.email', @email)
	-- set @parameters = json_modify(@parameters, 'strict $.password', @password)
	set @parameters = json_modify(@parameters, '$.id', 0)
	set @error = ''
	set @id = 0
	
	begin transaction

	begin try
	
		execute dbo.sessions_validate_full @session, null, @error output, @session_id output, @ip_id output, @user_agent_id output
		set @error = dbo.errors_join(@error, dbo.sessions_register_validate(@nick, @email, @password), 30, 0)
	
		if dbo.errors_has(@error) = 0 begin 
			
			declare @email_id integer = (select top 1 id from dbo.Emails where deleted is null and [address] = @email)
			declare @procedure_id integer
		
			execute dbo.procedures_get @procedure, @database, @procedure_id output
		
			insert into dbo.Users default values
			set @id = scope_identity()
			insert into dbo.UsersData([procedure], [session], entity, nick, [password], accessible) values
				(@procedure_id, @session_id, @id, @nick, dbo.sessions_password_hash(@password), dbo.settings_get_bool('allow_any_new_user', 0))
			
			if @email_id is null begin 
				insert into dbo.Emails([procedure], [session], [address]) values
					(@procedure_id, @session_id, @email)
				set @email_id = scope_identity()
			end
		
			if (select top 1 1 from dbo._UsersEmails where belongs = 1 and [user] = @id and email = @email_id) is null begin 
				insert into dbo.UsersEmails default values
				insert into dbo.UsersEmailsData([procedure], [session], entity, email, [user], belongs) values
					(@procedure_id, @session_id, scope_identity(), @email_id, @id, 1)
			end
		
			execute dbo.users_groups_create @procedure_id, @session_id, @nick, @id
		
			if (
				select top 1 1 
				from dbo._UsersGroups users_groups
				join dbo._Groups groups on groups.entity = users_groups.[group] 
				join dbo._Users users on users.entity = users_groups.[user]
				where 
					users_groups.belongs = 1 and 
					users.accessible = 1 and 
					groups.deleted is null and 
					users.deleted is null and 
					groups.[name] = 'administrators'
			) is null begin 
				insert into dbo.UsersGroups default values
				insert into dbo.UsersGroupsData([procedure], [session], entity, [user], [group], belongs) values 
					(@procedure_id, @session_id, scope_identity(), @id, (
						select top 1 entity from dbo._Groups where deleted is null and [name] = 'administrators'
					), 1)
			end

			set @parameters = json_modify(@parameters, '$.id', @id)
		
		end
	
		commit

	end try
	begin catch
		rollback
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 1
		
end
go
	
if object_id(N'dbo.sessions_login_validate', N'P') is not null drop procedure dbo.sessions_login_validate
go
create procedure dbo.sessions_login_validate 
	@user varchar(256),
	@password varchar(256),
	@ip_id integer,
	@error varchar(512) output
as begin 
	
	set @error = dbo.errors_set('', (
		dbo.bw((case 
			when @user is null then dbo.bw(1, 0)
			when @user = '' then dbo.bw(1, 1)
		else 0 end), 0) |
		dbo.bw((case 
			when @password is null then dbo.bw(1, 0)
			when @password = '' then dbo.bw(1, 1)
		else 0 end), 2) |
	0), 0, 0)
	
	if dbo.errors_has(@error) = 0 begin 
			
		declare @user_id integer
		declare @ips_times integer = dbo.ips_settings_get_int(@ip_id, 'ip_login_bad_times')
		declare @users_times integer
		
		if object_id(N'tempdb..#ips_tries_temporary_anp', N'U') is not null drop table #ips_tries_temporary_anp
		if object_id(N'tempdb..#users_tries_temporary_anp', N'U') is not null drop table #users_tries_temporary_anp
		
		create table #ips_tries_temporary_anp(
			id integer not null identity(1, 1),
			i integer not null,
			date_in datetime not null,
			primary key(id)
		);
		
		create table #users_tries_temporary_anp(
			id integer not null identity(1, 1),
			i integer not null,
			date_in datetime not null,
			primary key(id)
		);
	
		if @user like '%@%.%'  
			set @user_id = (
				select top 1 users_emails.[user]
				from dbo.Emails emails 
				join dbo._UsersEmails users_emails on emails.id = users_emails.email 
				where 
					users_emails.belongs = 1 and 
					emails.deleted is null and 
					emails.[address] = @user 
			)
		else 
			set @user_id = (select top 1 entity from dbo._Users where deleted is null and nick = @user)
		
		set @users_times = isnull(dbo.users_settings_get_int(@user_id, 'user_login_bad_times'), 0)
		
		insert into #ips_tries_temporary_anp(i, date_in) 
			select top (@ips_times) i, date_in from dbo.LoginsTries where date_out is null and [ip] = @ip_id
		insert into #users_tries_temporary_anp(i, date_in) 
			select top (@users_times) i, date_in from dbo.LoginsTries where date_out is null and [user] = @user_id
		
		set @error = dbo.errors_set(@error, (
			dbo.bw((case 
				when @user_id is null then dbo.bw(1, 0)
				when @user_id < 1 then dbo.bw(1, 1) 
			else isnull((
				select top 1 (
					(case when deleted is null then 0 else dbo.bw(1, 3) end) |
					(case when (
						0 not in (select top 1 i from #users_tries_temporary_anp) and 
						(select top 1 count(1) from #users_tries_temporary_anp) = @users_times and 
						datediff(second, (select top 1 date_in from #users_tries_temporary_anp order by i desc), getdate()) <= dbo.users_settings_get_int(@user_id, 'user_lock_time')
					) then dbo.bw(1, 4) else dbo.bw((
						(case when accessible = 1 then 0 else dbo.bw(1, 0) end) | 
						isnull((
							select top 1 dbo.bw(1, 1)
							from dbo._UsersGroups users_groups
							join dbo._Groups groups on users_groups.[group] = groups.entity 
							where 
								users_groups.belongs = 1 and 
								groups.deleted is null and 
								users_groups.[user] = @user_id and 
								groups.[name] = 'banned' 
						), 0) |
						(case when (
							select top 1 count(1) from dbo.[Sessions] where 
								[user] = @user_id and 
								date_out is null and 
								datediff(second, date_last, getdate()) < dbo.users_settings_get_int(@user_id, 'session_timeout')
						) < dbo.users_settings_get_int(@user_id, 'users_maximum_sessions') then 0 else dbo.bw(1, 2) end) | 
					0), 5) end) |
				0) from dbo._Users where entity = @user_id
			), dbo.bw(1, 2)) end), 0) |
			dbo.bw((case 
				when @ip_id is null then dbo.bw(1, 0) 
				when @ip_id < 1 then dbo.bw(1, 1)
			else isnull((
				select (
					(case when deleted is null then 0 else dbo.bw(1, 3) end) |
					(case when (
						0 not in (select i from #users_tries_temporary_anp) and 
						(select top 1 count(1) from #users_tries_temporary_anp) = @users_times and 
						datediff(second, (select top 1 date_in from #users_tries_temporary_anp order by i desc), getdate()) <= dbo.users_settings_get_int(@user_id, 'user_lock_time')
					) then dbo.bw(1, 4) else dbo.bw((
						(case when accessible = 1 then 0 else dbo.bw(1, 0) end) | 
						isnull((
							select top 1 dbo.bw(1, 1) 
							from dbo._IpsGroups ips_groups
							join dbo._Groups groups on ips_groups.[group] = groups.entity 
							where 
								ips_groups.belongs = 1 and 
								groups.deleted is null and 
								ips_groups.[ip] = @ip_id and 
								groups.[name] = 'banned' 
						), 0) |
						(case when (
							select top 1 count(1) from dbo.SessionsFullView where 
								ip_id = @ip_id and 
								date_out is null and 
								datediff(second, date_last, getdate()) < dbo.ips_settings_get_int(@ip_id, 'session_timeout')
						) < dbo.ips_settings_get_int(@ip_id, 'ips_maximum_sessions') then 0 else dbo.bw(1, 2) end) | 
					0), 5) end) |
				0) from dbo._Ips where entity = @ip_id 
			), dbo.bw(1, 2)) end), 8) |
		0), 4, 0)
		
		if dbo.errors_has(@error) = 0 
			set @error = dbo.errors_set(@error, (
				select top 1 (case 
					when [accessible] = 1 and (
						isnull([password], '') = '' or 
						[password] = dbo.sessions_password_hash(@password) collate latin1_general_cs_as
					) then 0 
					else dbo.bw(1, 0) 
				end) from dbo._Users where entity = @user_id
			), 20, 0)
		
		drop table #ips_tries_temporary_anp
		drop table #users_tries_temporary_anp
		
	end

end
go
	
if object_id(N'dbo.sessions_login', N'P') is not null drop procedure dbo.sessions_login
go
create procedure dbo.sessions_login 
	@session varchar(max), 
	@user varchar(256),
	@password varchar(256),
	@error varchar(512) output,
	@id integer output, 
	@user_nick varchar(32) output,
	@timeout integer output,
	@reset_password bit output
as begin 
		
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_login'
	declare @parameters varchar(max) = '{"user":null,"password":null,"ip":null}'
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	declare @own_errors varchar(512)
	declare @has_errors bit

	set nocount on
		
	set @parameters = json_modify(@parameters, 'strict $.user', @user)
	set @parameters = json_modify(@parameters, 'strict $.password', @password)
	set @parameters = json_modify(@parameters, '$.id', 0)
	set @id = 0
	set @error = ''

	begin try
	
		execute dbo.sessions_validate_full @session, 'guest', @error output, @session_id output, @ip_id output, @user_agent_id output
		set @has_errors = dbo.errors_has(@error)
	
		if @has_errors = 0 begin 

			execute dbo.sessions_login_validate @user, @password, @ip_id, @own_errors output
			set @error = dbo.errors_join(@error, @own_errors, 30, 0)
			set @has_errors = dbo.errors_has(@error)

			if @has_errors = 0 begin 

				declare @user_id integer

				if @user like '%@%.%' 
					select top 1 
						@user_id = users.id,
						@user_nick = users.nick
					from dbo.Emails emails
					join dbo._UsersEmails users_emails on emails.id = users_emails.email 
					join dbo._Users users on users.entity = users_emails.[user] 
					where 
						users_emails.belongs = 1 and 
						emails.deleted is null and 
						emails.[address] = @user 
				else 
					select top 1 
						@user_id = entity, 
						@user_nick = nick 
					from dbo._Users where deleted is null and nick = @user

				insert into dbo.[Sessions]([user]) values (@user_id)
				set @id = scope_identity()
				insert into dbo.SessionsIps default values
				insert into dbo.SessionsIpsData(entity, [session], [ip], ok) values 
					(scope_identity(), @id, @ip_id, 1)
				insert into dbo.SessionsUsersAgents default values
				insert into dbo.SessionsUsersAgentsData(entity, [session], user_agent, ok) values 
					(scope_identity(), @id, @user_agent_id, 1)

				set @reset_password = (select top 1 (case when [accessible] = 1 and isnull([password], '') = '' then 1 else 0 end) from dbo._Users where entity = @user_id)
				update dbo.[Sessions] set date_out = getdate() where id = @session_id
				
				set @timeout = dbo.users_settings_get_int(@user_id, 'session_timeout')

			end

		end else 
			set @error = dbo.errors_join('', @error, 1, 0)

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
	
	select permission from dbo.SessionsPermissionsView where @has_errors = 0 and [session] = @id
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 1
		
end
go

if object_id(N'dbo.sessions_autocloser', N'P') is not null drop procedure dbo.sessions_autocloser
go
create procedure dbo.sessions_autocloser 
	@session varchar(max),
	@error varchar(512) output
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_autocloser'
	declare @machine_session integer = dbo.settings_get_int('machine_session', 0)
	declare @unknown_session integer = dbo.settings_get_int('unknown_session', 0)
	declare @session_id integer = dbo.sessions_get_id(@session)
	declare @ip_id integer
	declare @user_agent_id integer

	set nocount on

	set @error = ''

	begin try

		if @session != @machine_session
			execute dbo.sessions_validate_full @session, null, @error output, @session_id output, @ip_id output, @user_agent_id output
		else 
			select top 1 
				@ip_id = ip_id,
				@user_agent_id = user_agent_id
			from dbo.SessionsFullView where id = @session_id 

		if dbo.errors_has(@error) = 0 
			update dbo.[Sessions] set date_out = getdate() where 
				date_out is null and 
				id not in (@machine_session, @unknown_session) and 
				datediff(second, date_last, getdate()) > dbo.users_settings_get_int([user], 'session_timeout')

	end try
	begin catch
		set @error = dbo.errors_set(@error, dbo.bw(1, 0), 0, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, '{}'
	end catch

	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, '{}', 1

end
go

if object_id(N'dbo.sessions_errors_messages_set', N'P') is not null drop procedure dbo.sessions_errors_messages_set
go
create procedure dbo.sessions_errors_messages_set 
	@session varchar(max)
as begin 

	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_errors_messages_set'
	declare @session_id integer = isnull(dbo.sessions_get_id(@session), dbo.sessions_get_machine())
	declare @ip_id integer
	declare @user_agent_id integer
	declare @error varchar(512) = ''

	set nocount on

	select top 1 
		@ip_id = ip_id,
		@user_agent_id = user_agent_id
	from dbo.SessionsFullView where id = @session_id

	insert into dbo.ErrorsTemporaryGroups(i, [name]) values 
			(6, 'sessions_get_guest'),
			(5, 'sessions_update'),
			(4, 'sessions_close'),
			(3, 'sessions_register'),
			(2, 'sessions_login'), 
			(1, 'sessions_autocloser');
		insert into dbo.ErrorsTemporaryMessages([group], [message]) values 
			(2, 'user_null'),
			(2, 'user_empty'),
			(2, 'password_null'),
			(2, 'password_empty'),
			(2, 'user_id_null'),
			(2, 'user_id_bad'),
			(2, 'user_not_exists'),
			(2, 'user_deleted'),
			(2, 'user_antinuke'),
			(2, 'user_unaccessible'),
			(2, 'user_banned'),
			(2, 'user_limit_sessions'),
			(2, 'ip_id_null'),
			(2, 'ip_id_bad'),
			(2, 'ip_not_exists'),
			(2, 'ip_deleted'),
			(2, 'ip_antinuke'),
			(2, 'ip_unaccessible'),
			(2, 'ip_banned'),
			(2, 'ip_limit_sessions'),
			(2, 'password_wrong'),
			(3, 'nick_null'),
			(3, 'nick_empty'),
			(3, 'nick_exists'),
			(3, 'nick_group_exists'),
			(3, 'email_null'),
			(3, 'email_empty'),
			(3, 'email_bad_format'),
			(3, 'email_exists'),
			(3, 'password_null'),
			(3, 'password_empty'),
			(3, 'password_minimum_length'),
			(3, 'password_needs_lower'),
			(3, 'password_needs_upper'),
			(3, 'password_needs_digit'),
			(3, 'password_needs_special')

	execute dbo.errors_messages_set @session

	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, '{}', 2

end
go

execute dbo.sessions_errors_messages_set null

if object_id(N'dbo.sessions_reset_password_validate', N'FN') is not null drop function dbo.sessions_reset_password_validate
go
create function dbo.sessions_reset_password_validate(
	@session_id integer, 
	@password varchar(512)
) returns integer begin 

	declare @error integer = dbo.password_validate(@password)

	if @error = 0 begin 

		declare @user_id integer = (select [user] from dbo.[Sessions] where id = @session_id)
		declare @differents integer = isnull(dbo.settings_get_int('users_passwords_differents', 0), 3)

		set @error = (case when (
			select top 1 1 from (
				select top (@differents) * from (
					select distinct * from (
						select top 10000 [password] from dbo.UsersData where entity = @user_id order by id desc
					) query_historical_passwords
				) query_distinct
			) passwords where passwords.[password] = dbo.sessions_password_hash(@password)
		) is null then 0 else dbo.bw(1, 7) end)

	end

	return @error
end
go

if object_id(N'dbo.sessions_reset_password', N'P') is not null drop procedure dbo.sessions_reset_password
go
create procedure dbo.sessions_reset_password 
	@session varchar(max),
	@password varchar(256),
	@error varchar(512) output
as begin 
	
	declare @database varchar(64) = dbo.settings_get('database_anp', 0)
	declare @procedure varchar(64) = 'sessions_reset_password'
	declare @procedure_id integer
	declare @session_id integer
	declare @ip_id integer
	declare @user_agent_id integer
	declare @parameters varchar(max) = '{}'

	set @parameters = json_modify(@parameters, '$.password', @password)

	begin try

		execute dbo.sessions_validate_full @session, 'user', @error output, @session_id output, @ip_id output, @user_agent_id output

		if dbo.errors_has(@error) = 0 begin 

			set @error = dbo.errors_set(@error, dbo.sessions_reset_password_validate(@session_id, @password), 30, 0)

			if dbo.errors_has(@error) = 0 begin 

				declare @data_id integer = (select top 1 [data] from dbo._Users where entity = (select top 1 [user] from dbo.[Sessions] where id = @session_id))

				execute dbo.procedures_get @database, @procedure, @procedure_id output

				insert into dbo.UsersData([procedure], [session], entity, nick, [password], accessible) 
				select @procedure_id, @session_id, entity, nick, dbo.sessions_password_hash(@password), accessible from dbo.UsersData where id = @data_id

			end

		end

	end try
	begin catch
		set @error = @error | dbo.bw(1, 0)
		execute dbo.exceptions_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters
	end catch
		
	execute dbo.logs_set @session_id, @database, @procedure, @ip_id, @user_agent_id, @error, @parameters, 1

end
go